from __future__ import print_function

"""
Computational accompaniment for 'Tonality: An Owner's Manual'


This program serves as a user interface, or maybe introduction, to the code in the directory.  
It allows easy reconstruction of all the computational examples in the book 'Tonality: An Owner's Manual'.

	> import TAOM
	> TAOM.help()

TODO:

	Test remake_data on Liz's computer

"""

introText = "Type TAOM.help() for help."

print(introText)

def help(s = ''):
	print(helpText)

def sections():
	print("Information is available for the following sections; type TAOM.section('P7') to run.") 
	sections = sorted([x.replace('__section', '').replace('_', '.') for x in globals() if x.startswith('__section')], key = sort_example_names)
	for s in sections:
		print('\t', s)

def examples():
	examples = sorted([x.replace('__example', '').replace('_', '.') for x in globals() if x.startswith('__example')], key = sort_example_names)
	print("Code is available for the following " + str(len(examples)) + " examples; type TAOM.example('1.4.1') to run.") 
	for s in examples:
		print('\t', s)

def topics():
	print("Information is available for the following topics; type TAOM.topic('Raw Progressions') to run.") 
	tempTopics = sorted([x.replace('__topic', '').replace('_', ' ') for x in globals() if x.startswith('__topic')])
	for s in tempTopics:
		print('\t', s)

def sort_example_names(s):
	sSplit = s.split('.')
	if sSplit[0].count('P'):
		sSplit[0] = float(sSplit[0].replace('P', '')) - .01
	return [float(x) for x in sSplit]
	
def section(s = ''):
	if not s:
		sections()
		return
	sectionVar = '__section' + s.replace('.', '_')
	if sectionVar in globals():
		print (globals()[sectionVar])
	else:
		print('Sorry, no info for that section.  ', end = '')
		sections()

def topic(s = ''):
	if not s:
		topics()
		return
	s = ' '.join([x.upper() for x in s.split() if x])
	tempTopics = sorted([[x, x.replace('__topic', '').replace('_', ' ').upper()] for x in globals() if x.startswith('__topic')])
	for t, tText in tempTopics:
		if tText == s:
			print (globals()[t])
			return
	print('Sorry, no info for that topic.  ', end = '')
	topics()

def figure(*args, **kwargs):
	example(*args, **kwargs)

def example(*args, **kwargs):
	"""args should be the example number as a string followed by example-dependent parameters"""
	args = list(args)
	if not args:
		examples()
		return
	elif len(args) < 2:
		args.append(0)
	exampleNumber = args[0]
	parameters = args[1:]
		
	exampleString = '__example' + exampleNumber.replace('.', '_')
	if exampleString in globals():
		f = globals()[exampleString]
		f(*parameters)
	else:
		print('Sorry, no code for that example.  ', end = '')
		examples()
	
def load_module(s):
	if s not in globals():
		globals()[s] = __import__(s)
	
def automatic_bach_analysis():
	load_module('analyzer')
	load_module('rn')
	load_module('eighthnote_harmonies')
	analyzer.go('Bach chorales rawmachine', raw = True, overwrite = True)
	rn.gather_stats("Bach chorales rawmachine")
	eighthnote_harmonies.gather_stats('Bach Chorales RAWMACHINE')
	analyzer.go('Bach chorales cleanmachine', raw = False, overwrite = True)

helpText = """This program is meant to accompany the book Tonality; An Owner's Manual.  You can use it to recreate specific examples 
or to dive deeper into ideas discussed in the book, or as an introduction to using my badly written software.  The three main routines 
are example, section, and topic.

	> TAOM.example('1.4.1', 1) # passing an example number as a string and any extra parameters afterwards.
	> TAOM.section('5.1') # for additional information about calculations not associated with specific examples. 
	> TAOM.topic('raw progressions')	# for information about some general topic
	
To list examples, sections, or topics for which code is available, type:
	> TAOM.examples()
	> TAOM.sections()
	> TAOM.topics()

This program is does not try to be user friendly and is meant for programmers who want to explore the data further.  
The goal is just to show what I have done.

There is much more material available online at http://www.madmusicalscience.com/TAOM"""

"""================ CODE FOR EXAMPLES ================ """


def __example1_4_1(*args):
	p1 = args[0]
	print("Use TAOM.example('1.4.1', x), where x is a parameter (0 = real Palestrina, 1 = fake Palestrina, 2 = real Bach, 3 = fake Bach)")
	load_module('reversals')
	if p1 in [0, 1]:
		reversals.load("Palraw")
	elif p1 in [2, 3]:
		reversals.load("wtcf")
	if p1 in [1, 3]:
		reversals.make_piece(5000)
	reversals.graph()
	return
	
def __example1_4_2(*args):
	load_module('reversals')
	temp = {0: "josquincomplete", 1: "beethoven quartets", 2: "monteverdi", 3: "bach", 4: "sacred harp", 5: "rock"}
	reversals.load(temp.get(args[0], ''))
	if args[0] in [4, 5]:
		reversals.graph(thresh = 3)
	else:
		reversals.graph()

def __example1_5_4(*args):
	print("""TAOM.example('1.5.4', x), where x is a composer name or list of names""")
	load_module('rn')
	p1 = args[0]
	if type(p1) is str:
		compList = [p1]
	elif type(p1) is list:
		compList = p1
	else: 
		compList = ['Morley', 'Monteverdi', 'Lully', 'Corelli', 'Bach', 'Mozart', 'Palestrina Ionian']
	print("   iii vs. I6")
	for c in compList:
		rn.load(c)
		s = float(rn.counts[0]['iii'] + rn.counts[0]['I6'])/100.
		print(c, round(rn.counts[0]['iii']/s, 1), round(rn.counts[0]['I6']/s, 1))
	
def __example2_2_4(*args):
	load_module('rn')
	print("\n\nTemperley-deCLerq rock corpus")
	rn.load("rock") 
	rn.rock_roots()
	print("\n\nNot in the book, but also relevant, the McGill Billboard corpus")
	rn.load("bill") 
	rn.rock_roots()
	
def __exampleP3_8(*args):
	load_module('cpt2')
	motionTypes = {'.*  ((1 .*)|(.* 1))$': 'OBLIQUE', '.*  (.*) \\1': 'PARALLEL', '.*  (([0-9]* [0-9]*)|(-[0-9]* -[0-9]*))$': 'SIMILAR', '.*  (([0-9]* -[0-9]*)|(-[0-9]* [0-9]*))$': 'CONTRARY'}
	labelFunc = lambda s: [motionTypes.get(s[0], s[0])]
	comps = ['OCKEGHEM', 'JOSQUINCOMPLETE', 'PALRAW']
	matchStrings = ['.*  ((1 .*)|(.* 1))$', '.*  (.*) \\1', '.*  (([0-9]* [0-9]*)|(-[0-9]* -[0-9]*))$', '.*  (([0-9]* -[0-9]*)|(-[0-9]* [0-9]*))$']
	print('\nMOTION TO PERFECT CONSONANCES')
	cpt2.getall(4, '.* (1|5|8|12)  ', matchStrings, returnFunc = labelFunc, compList = comps)
	print('\nMOTION TO IMPERFECT CONSONANCES')
	cpt2.getall(4, '.* (3|6|10|13)  ', matchStrings, returnFunc = labelFunc, compList = comps)

def __example3_5_7(*args):
	load_module('cpt3')
	comps = ['OCKEGHEM', 'JOSQUINCOMPLETE', 'PALRAW']
	for c in comps:
		print(c)
		cpt3.fig357(c)

def __example3_6_3(*args):
	load_module('ouch')
	ouch.graph(comps = ['PALRAW', 'BACH'])

def __example3_6_4(*args):
	load_module('ouch')
	ouch.load("Bach")
	ouch.get_data()
	d = ouch.progs
	progs = sorted(d.items(), key = lambda x: -x[1])
	progs = [x for x in progs if not x[0].count('False')]
	totals = sum([x[1] for x in progs])
	for prog in progs:
		print(prog[0], round(prog[1]*100./totals, 1))

def __example3_6_8(*args):
	load_module('doublings')
	print("\nBACH CHORALES\n")
	doublings.load("Bach")
	doublings.count2()
	print("\nSELECTED PALESTRINA MASSES\n")
	doublings.load("Palestrina")
	doublings.count2()

def __example5_1_5(*args):
	load_module('progressions')
	progressions.load("PALRAW")
	progressions.chord_progressions(2)
	print("Repeated calls to progressions.show() will cycle through all appearances of the progression.")
	print("Type\n\t> import progressions\n\t> progressions.show('GFM ATd')")
	print("It is also fun to look at the pattern on other degrees, e.g. CFM DTd, EFm FTm")
	progressions.show("GFM ATd")

def __example5_1_13(*args):
	load_module('renaissance_analyzer')
	renaissance_analyzer.load('PALRAW')
	renaissance_analyzer.nht_graph(['P', 'S', 'N'])
	
def __example5_1_14(*args):
	load_module('renaissance_analyzer')
	renaissance_analyzer.load('PALRAW')
	renaissance_analyzer.nht_graph([((0, 5, 7), (0, 4, 7)), ((0, 6, 9), (0, 4, 7)), ((0, 2, 4, 7), (0, 4, 7)), ((0, 3, 10), (0, 3, 8))])
	
def __example5_2_2(*args):
	load_module('renaissance_analyzer')
	renaissance_analyzer.load('PALRAW')
	renaissance_analyzer.catalog_suspensions()
	
def __example5_2_3(*args):
	load_module('history')
	history.trim(before = 1700)
	history.trim(['Dalza', 'Schutz', 'Castello', 'Merula', 'Cavalli', 'Lully'])
	p1a = [['IV -> I', 'IV -> I6', 'IV6 -> I', 'IV6 -> I6'], ['IV -> I', 'IV -> I6', 'IV6 -> I', 'IV6 -> I6']]
	p1b = ['[9, 3', '[9, 3P']
	p1labels = ['with or without p.t.', 'with p.t.']
	pairs = [['Willaert', 'Willaert2']]
	history.voice_leading(p1a, p1b, labels = p1labels, combinePairs = pairs)

def __example5_4_7(*args):
	load_module('classical_analyzer')
	classical_analyzer.gather_stats(source = 'MOZART', fileName='K279-1')

def __example6_1_1(*args):
	load_module('rn')
	groupChords = [['I', 'Imaj7', 'I6', 'Imaj6/5', 'I6/4', 'Imaj4/3', 'Imaj2'], ['ii', 'ii7', 'ii6', 'ii6/5', 'ii6/4', 'ii4/3', 'ii2'], 
				['iii', 'iii7', 'iii6', 'iii6/5', 'iii6/4', 'iii4/3', 'iii2'], ['IV', 'IVmaj7', 'IV6', 'IVmaj6/5', 'IV6/4', 'IVmaj4/3', 'IVmaj2'],
				['V', 'V7', 'V6', 'V6/5', 'V6/4', 'V4/3', 'V2'], ['vi', 'vi7', 'vi6', 'vi6/5', 'vi6/4', 'vi6/4', 'vi2'], 
				['viio', 'vii/o7', 'viio6', 'vii/o6/5', 'viio6/4', 'vii/o4/3', 'vii/o2', 'viio7', 'viio6/5', 'viio4/3', 'viio2']]
	rn.diatonic_percentages(['Josquin', 'Palestrina', 'Bach', 'Beethoven'], groupChords)

textFor623 = """\nTo generate this table you need four commands, each of which makes a graphical representation of chordal tendencies.  The terminal output contains the values in the text.

Start with:
	import rn
Then type either: 
	rn.load("Dalza)	
or:
	rn.load("Frottola)

Then type either:
	rn.tendency_histo("IV -> ")
or:
	rn.tendency_histo("I -> ")
	
You can also type TAOM.example('6.2.3', <arg>)	# <arg> can be 1, 2, 3, 4
"""

def __example6_2_3(*args):
	load_module('rn')
	p1 = args[0]
	if p1 == 0:
		print(textFor623)
	elif p1 == 1:
		rn.load("Dalza")
		rn.tendency_histo("IV -> ")
	elif p1 == 2:
		rn.load("Frottola")
		rn.tendency_histo("IV -> ")
	elif p1 == 3:
		rn.load("Dalza")
		rn.tendency_histo("I -> ")
	elif p1 == 4:
		rn.load("Frottola")
		rn.tendency_histo("I -> ")

def __example6_2_4(*args):
	load_module('rn')
	if type(args[0]) is str:
		comps = [args[0]]
	elif type(args[0]) is list:
		comps = args[0]
	else:
		comps = ['Dalza', 'Frottola']
	for c in comps:
		print(c)
		rn.load(c)
		rn.inversions_on_degrees()

def __example6_2_7(*args):
	load_module('history')
	history.comps = ['Encina', 'Frottola', 'Dalza', 'Willaert', 'Sermisy', 'Gombert', 'Tallis', 'Clemens', 'Goudimel', 'Rore', 
	'Palestrina', 'Lassus', 'Victoria', 'Cavalieri', 'Gastoldi', 'Morley', 'Monteverdi', 'Schutz', 'Castello', 'Merula', 'Cavalli', 
	'Lully', 'Corelli', 'Bach', 'Haydn', 'Mozart', 'Beethoven', 'Chopin', 'Brahms', 'Rock']
	history.show('IV // V')

def __example6_3_6(*args):
	load_module('cadences')
	if type(args[0]) is str:
		comps = [args[0]]
	elif type(args[0]) is list:
		comps = args[0]
	else:
		comps = ['Ockeghem', 'JosquinComplete', 'Palraw', 'Victoriaraw']
	print('BASS NOTE, PERCENTAGE OF CADENCES')
	for c in comps:
		print (c)
		cadences.load(c)
		cadences.fig636()
		
def __example6_4_1(*args):
	load_module('progressions')
	if type(args[0]) is str:
		comps = [args[0]]
	elif type(args[0]) is list:
		comps = args[0]
	else:
		comps = ['Ockeghem', 'JosquinComplete', 'Palraw']
	for c in comps:
		print (c)
		progressions.load(c)
		progressions.fig641()

def __example6_4_2(*args):
	load_module('progressions')
	print("The table from which the example is constructed:")
	progressions.doall(['[RTF][RTF].*'], useLongProgs = False, printIt = False)
	print("\nComparing the tendency of triads to descend by fifth:\n\tRRMM3 and RRMm3 vs. RRmM3 and RRmm3")
	print("\nHow major triads move:")
	progressions.load("PALRAW")
	progressions.regex('R[RTF]M[AMmd]', qualities=True, roots = True)
	print("\nHow minor triads move:")
	progressions.regex('R[RTF]m[AMmd]', qualities=True, roots = True)

def __example6_4_3(*args):
	load_module('progressions')
	progressions.load("Ockeghem")
	progressions.regex(['[RTF][RTF].*'], useLongProgs = False, printIt = True)

def __example6_4_4(*args):
	load_module('progressions')	
	if type(args[0]) is str:
		comps = [args[0]]
	elif type(args[0]) is list:
		comps = args[0]
	else:
		comps = ['VICTORIARAW', 'Josquincomplete']
	progressions.chord_histo(comps, averageIonian = True)
	
def __example6_4_5(*args):
	load_module('progressions')
	if len(args) > 1 and type(args[1]) is str:
		progressions.load(args[1])
	else:
		progressions.load('PALRAW')
	print('\nChords that are most often approached in specific ways\n')
	progressions.tendency_summary(0, chordPosition = 1)
	print('\nChords that most often progress in specific ways\n')
	progressions.tendency_summary(0, chordPosition = 0)

def __example6_4_6(*args):
	load_module('progressions')
	progressions.load("PALRAW")
	if args[0] == 0:
		progressions.tendency_histo("viio6 -> ")
	else: 
		progressions.tendency_histo(" -> viio6")

def __example6_4_7(*args):
	load_module('vl')
	if type(args[0]) is str:
		comps = [args[0]]
	elif type(args[0]) is list:
		comps = args[0]
	else:
		comps = ['Bach']
	for c in comps:
		print (c)
		vl.load(c)
		vl.report("viio6 -> I", percent = True, details = False)

def __example6_4_8(*args):
	load_module('rn')
	if type(args[0]) is str:
		comps = [args[0]]
	elif type(args[0]) is list:
		comps = args[0]
	else:
		comps = ['Corelli', 'Bach', 'Haydn', 'Mozart', 'Beethoven', 'Chopin']
	for c in comps:
		print(c)
		rn.load(c)
		rn.regex("(viio6|V4/3)$")

def __example6_6_1(*args):
	load_module('history')
	history.untrim()
	history.size()

def __example6_6_2(*args):
	load_module('sd')
	print('This table outputs the most common harmonizations of a given melodic scale-degree pattern.  It outputs a progression if at least one')
	print('composer uses that progression at least 20% of the time.  In the book I list only those values greater than or equal to 20%.')
	sd.SD_comparison(secondaryDominants = True, comps = ['Josquin', 'Goudimel', 'Monteverdi'])
	
def __example6_6_3(*args):
	load_module('history')
	history.comps = ['Dufay', 'Ockeghem', 'Josquin', 'Frottola', 'Dalza', 'Willaert', 'Sermisy', 'Gombert', 'Tallis', 'Clemens',
					'Palestrina', 'Lassus', 'Victoria', 'Cavalieri', 'Gastoldi', 'Morley', 'Monteverdi', 'Schutz', 
					'Merula', 'Cavalli', 'Lully', 'Corelli', 'Bach', 'Haydn', 'Mozart', 'Beethoven']
	history.inversions()

def __example6_6_4(*args):
	load_module('history')
	history.comps = ['Dufay', 'Ockeghem', 'Josquin', 'Willaert', 'Sermisy', 'Gombert', 'Tallis', 'Clemens', 'Goudimel', 
					'Rore', 'Palestrina', 'Lassus', 'Victoria', 'Cavalieri', 'Gastoldi', 'Morley', 'Monteverdi', 'Schutz', 
					'Castello', 'Merula', 'Cavalli', 'Lully', 'Corelli', 'Bach', 'Haydn', 'Mozart', 'Beethoven']
	history.show()

def __example6_6_5(*args):
	load_module('history')
	history.comps = ['Dufay', 'Ockeghem', 'Josquin', 'Encina', 'Frottola', 'Dalza',  'Willaert', 'Sermisy', 'Gombert', 'Tallis', 'Clemens', 'Goudimel', 
					'Rore', 'Palestrina', 'Lassus', 'Victoria', 'Cavalieri', 'Gastoldi', 'Morley', 'Monteverdi', 'Schutz', 
					'Castello', 'Merula', 'Cavalli', 'Lully', 'Corelli', 'Bach']
	history.tendency(["ii -> V", "V -> I"])

def __example6_6_6(*args):
	load_module('history')
	history.comps = ['Dufay', 'Ockeghem', 'Josquin', 'Encina', 'Frottola', 'Dalza', 'Willaert', 'Sermisy', 'Gombert', 'Tallis', 'Clemens', 'Goudimel', 
					'Rore', 'Palestrina', 'Lassus', 'Victoria', 'Cavalieri', 'Gastoldi', 'Morley', 'Monteverdi', 'Schutz', 
					'Castello', 'Merula', 'Cavalli', 'Lully', 'Corelli', 'Bach', 'Haydn', 'Mozart', 'Beethoven']
	history.show(['ii -> V / ii -> V + vi -> V + IV -> V', 'vi -> V / ii -> V + vi -> V + IV -> V', 'IV -> V / ii -> V + vi -> V + IV -> V'])

def __example6_6_7(*args):
	load_module('rn')
	if type(args[0]) is str:
		comps = [args[0]]
	elif type(args[0]) is list:
		comps = args[0]
	else:
		comps = ['Palestrina', 'Monteverdi', 'Corelli', 'Bach', 'Haydn', 'Mozart', 'Beethoven', 'Brahms']
	for c in comps:
		rn.load(c)
		a = rn.regex("vi7* -> V7*$", printIt = False)
		b = rn.regex("vi7* -> CHORD$", printIt = False)
		print(c, int((a*100./b) + .5))

def __example6_6_8(*args):
	load_module('history')
	history.untrim()
	"""trimm out homophonic rep from Willaert, and other rep either not analyzed by me or with small chord counts"""
	history.trim(['Willaert2', 'Tallis2', 'Rore', 'Mendelssohn', 'Brahms', 'Beethoven Quartets', 'Rock', 'bill'])
	history.grammar()

def __exampleP7_6(*args):
	load_module('rn')
	if type(args[0]) is str:
		rn.load(args[0])	# try 'Bach HMM'
	else:
		rn.load('Bach Chorales RAWMACHINE')
	theProgs = [["IV -> I6"], ["IV -> ii6", "ii6 -> I6", "IV -> ii6 -> I6"], ['ii -> I'], ['ii -> viio6', 'viio6 -> I', 'ii -> viio6 -> I'],
				['V -> I6'], ['V -> ii6', 'ii6 -> I6', 'V -> ii6 -> I6'], ['V -> V2', 'V2 -> I6', 'V -> V2 -> I6'], ['viio6 -> I6'], 
				['viio6 -> ii', 'ii -> I6', 'viio6 -> ii -> I6']]
	print('first table calculations')
	for l in theProgs:
		results = [x + ' ' + str(rn.counts[0].get(x, 0)) for x in l]
		print(' '.join(results))
	print('\nsecond table calculations')
	minorProgs = ['VII6(/5)*( -> V6/5/III| -> iio)* -> III$', 'VII6(/5)*( -> V6/5/III| -> iio)* -> i6$']
	for m in minorProgs:
		print('looking for', m)
		result = rn.regex(m, 'minor', pct = False)
		print('found', result)
	print('\n')
	majorProgs = ['viio6( -> ii)* -> iii$', 'viio6( -> ii)* -> I6$']
	for m in majorProgs:
		print('looking for', m)
		result = rn.regex(m, pct = False)
		print('found', result)

def __example7_1_1(*args):
	p1 = args[0]
	load_module('rn')
	if p1 == 0:
		rn.load('Bach')
	elif p1 == 1:
		rn.load('Mozart')
	elif type(p1) is str:
		rn.load(p1)
	groupChords = [['I', 'Imaj7'], ['I6', 'Imaj6/5'], ['I6/4', 'Imaj4/3', 'Imaj2'], ['ii', 'ii7'], ['ii6', 'ii6/5'], ['ii6/4', 'ii4/3', 'ii2'], 
					['iii', 'iii7'], ['iii6', 'iii6/5'], ['iii6/4', 'iii4/3', 'iii2'], ['IV', 'IVmaj7'], ['IV6', 'IVmaj6/5'], ['IV6/4', 'IVmaj4/3', 'IVmaj2'],
					['V', 'V7'], ['V6', 'V6/5'], ['V6/4', 'V4/3', 'V2'], ['vi', 'vi7'], ['vi6', 'vi6/5'], ['vi6/4', 'vi6/4', 'vi2'], 
					['viio', 'vii/o7'], ['viio6', 'vii/o6/5'], ['viio6/4', 'viio4/3', 'vii/o2']]
	rn.diatonic_percentages2(groupChords)

def __example7_1_3(*args):
	load_module('rn')
	print("Here I average the root progressions found in both modes of the Bach chorales,\nthe Mozart piano sonatas, and a selection of Beethoven piano sonatas.\n")
	print("I consider only progressions A->B where chords A and B each account for \nat least 4% of the chords and where A->B accounts for at least 4% of \nthe two-chord progressions starting with A.\n")
	print("I compare these common progressions to those allowed by my model.\n")
	rn.load(["Bach", "Mozart", "Beethoven"])
	rn.grammar_model('both', 4, 4, fauxbourdon = False)
	
def __example7_4_6(*args):
	print("The Mozart data use the variant sd2.py program, which tracks the interval from")
	print("the highest note of chord1 to the highest note of chord2; for Bach, I use")
	print("sd.py, which captures the interval from the last harmonic tone in chord1 to the")
	print("first harmonic tone in chord2; see the code for details.\n")
	load_module('sd')
	sd.SD_comparison(secondaryDominants = False, comps = ['Bach', '2Mozart'], sixFour = True)

def __example7_5_7(*args):
	load_module('rn')
	if type(args[0]) is str:
		comps = [args[0]]
	elif type(args[0]) is list:
		comps = args[0]
	else:
		comps = ['Bach', 'Mozart', 'Beethoven']
	for c in comps:
		print (c)
		rn.load(c)
		rn.passing_six_four()

def __example7_6_1(*args):
	load_module('raw_sequences')
	comps = ['Corelli', 'WTC', 'Mozart']
	for c in comps:
		print('\n')
		print(c)
		raw_sequences.load(c)
		raw_sequences.basic_report()

def sequence_report(i = -1):
	load_module('raw_sequences')
	comps = ['Corelli', 'WTC', 'Mozart']
	for c in comps:
		print('\n')
		print(c)
		raw_sequences.load(c)
		raw_sequences.specific_report(i)

def __example7_6_3(*args):
	sequence_report(-1)

def __example7_6_4(*args):
	sequence_report(1)

def __example7_6_6(*args):
	sequence_report(-2)

def __example7_6_9(*args):
	sequence_report(2)

def __example8_6_3(*args):
	load_module('progressions')
	progressions.fig863()

def __example8_6_4(*args):
	load_module('cadences')
	cadences.load('palraw')
	cadences.compare_modes({'lastChord': 2, 'theSig': 0, 'bestChord': (2, 5, 9)}, {'lastChord': 9, 'theSig': 0, 'bestChord': (9, 0, 4)}, 
		{'lastChord': 0, 'theSig': 0, 'bestChord': (0, 4, 7)}, {'lastChord': 7, 'theSig': 0, 'bestChord': (7, 11, 2)}, labels = ('dorian', 'aeolian', 'ionian', 'mixolydian'), manualKeys = [5, 0, 7, 2, 9, 4])

def __example8_6_13(*args):
	load_module('history')
	history.chord_histo(['Bach', 'palmin'], 'minor', customLabels = ['Bach', 'Palestrina'])
	
def __example8_6_14(*args):
	load_module('history')
	history.comps = ['Encina', 'Willaert', 'Sermisy', 'Gombert', 'Tallis', 'Clemens', 'Rore', 
	'Palestrina', 'Lassus', 'Victoria', 'Cavalieri', 'Gastoldi', 'Morley', 'Monteverdi', 'Schutz', 'Castello', 'Merula', 'Cavalli', 
	'Lully', 'Corelli', 'Bach', 'Haydn', 'Mozart', 'Beethoven']
	history.show('i6 // III', mode = 'minor', removeInversions = False, labels = ['i6 vs. III'])

def __example9_2_1(*args):
	load_module('phrase_and_melody')
	phrase_and_melody.load("Bach")
	phrase_and_melody.strength_histo()

def __example9_2_2(*args):
	load_module('phrase_and_melody')
	phrase_and_melody.load("Bach")
	phrase_and_melody.rhythm_histo(['ii6/5', 'V7']) 

def __example9_2_3(*args):
	load_module('phrase_and_melody')
	phrase_and_melody.load("Bach")
	phrase_and_melody.rhythm_histo(phrase_and_melody.Iequiv + phrase_and_melody.IVequiv + phrase_and_melody.Vequiv, specialLabels = ["I (all inversions)", "ii or IV (all inversions)", "vii° or V (all inversions)"]) 

def __example9_6_3(*args):
	load_module('vlgen')
	p1 = args[0]
	if p1 == 0:
		print("The harmonization must be done phrase by phrase, and will output in C major.  Type TAOM.example('9.6.3', 1) for phrase 1 and Type TAOM.example('9.6.3', 2) for phrase 2.")
		print("The program has been set to produce very standard harmonizations; by manipulating the calculation of probability in find_melody() you can get less obvious options.")
		print("Keep in mind that the harmonizer knows nothing about meter.")
	elif p1 == 1:
		vlgen.harmonize([60, 64, 62, 64, 65, 67, 67, 64])
	elif p1 == 2:
		vlgen.harmonize([69, 67, 65, 64, 62, 64])
		
def __exampleP10_1(*args):
	load_module('attack_density')
	attack_density.graph_rhythm()  # [DT._PATHS['Beethoven'][1]+'op053-1.krn', DT._PATHS['Mozart'][1]+'K279-1.xml']

def __exampleP10_2(*args):
	load_module('attack_density')
	attack_density.register()  	# [DT._PATHS['Beethoven'][1]+'op053-1.krn', DT._PATHS['Mozart'][1]+'K279-1.xml']
		
"""================ SECTION TEXT DEFINITIONS ================ """

__section1_5 = """To search for unresolved sevenths in the Bach chorales, type:

	> import sevenths
	> sevenths.unresolved_sevenths()
	
For unprepared sevenths, try:

	> import sevenths
	> sevenths.unprepared_sevenths()
	
These lists will contain false alarms, but they are small enough to search by hand.  
"""

__section5_1 = """1. To try out the automatic Renaissance analyzer, type

	> import renaissance_analyzer
	> renaissance_analyzer.random_piece("PALRAW")
	
You can also use the nht module as described in the info for section 5.4, loading any of the hand-analyzed Renaissance corpora.

2. To look for parallel fifths disguised by suspensions, you can use overlap.py on the 'reduced' scores that have removed all nonharmonic tones:

	> import overlap
	> overlap.load("BACH CHORALES REDUCED")	# overlap.load("BACH") for the version with nonharmonic tones
	> overlap.show_parallels(0, 5, 4)		# show the first five instances of parallels with 4 voices.
	
Or use PALESTRINA REDUCED and PALESTRINA for those composers.  You can analyze more of the REDUCED directories in the Music folder (Morley, Monteverdi, etc.) 
and make more reduced files using the nht program.

"""

__section5_4 = """To explore nonharmonic tones in hand-analyzed corpora, use the nht module:

	> import nht
	> nht.load("BACH")	# load the analyses
	> nht.counts	# a dict counting the different nonharmonic tones
	> nht.show('?') # shows that specific nonharmonic tone, use rangeList = [0, 10] to show just the first 10
	> nht.rewrite('riemenschneider001', 'bach') # annotate nonharmonic tones in the first Bach chorale

For a list of the pedals in the Mozart Sonatas, type:
	> import pedals
	> pedals.gather_stats()

"""

__section6_4 = """For the proportion of major- and minor-mode pieces, mentioned in a footnote:

	> import chord_distributions
	> chord_distributions.major_modes()

And for the proportion of major and minor chords within these different piece groups:

	> import progressions
	> progressions.summarize_chord_types(includeIncomplete = True)

"""

__sectionP7 = """I have included three computer analyses of the Bach chorales.

	Bach Chorales RAWMACHINE - produced with analyzer.py, counting every sonority as a harmony
	Bach Chorales CLEANMACHINE - produced with analyzer.py, considering some sonorities as passing tones (using the data of the RAWMACHINE analyses)
	Bach Chorales HMM - produced by a hidden Markov model, written by Mark Newman, that learns almost all the musical structure automatically

You can recreate the first two sets of analyses by typing:

	> TAOM.automatic_bach_analysis()
"""

__section7_6 = """Sequences were found by machine using the 'raw_sequences' program:

	> import raw_sequences
	> raw_sequences.gather_stats('wtc')
	
I then analyzed each sequence by hand and assigned it to a harmonic category.  Sometimes this required some approximation and judgment.
"""


"""================ TOPIC DEFINITIONS ================ """

__topicRaw_Progressions = """To find an arbitrary progression in Palestrina's masses, type:

	> import progressions
	> progressions.load("palraw")
	> progressions.chord_progressions(4)	# length of prog
	> progressions.regex( ... )
	> progressions.show( ... )
	
These routines are used for unanalyzed scores; they know nothing about keys, nonharmonic tones, etc.  They just look for triads of the specified form.
Chords are specified in the format <BASS><BASS CHORDDEGREE><CHORD QUALITY> so CRM means that bass note C is the root (R) of a major chord (M).
CTm means the bass is C and it is the third of a minor chord.  The possible degrees are RTF and the qualities are Mmd+.

"""

__topicRoman_Numerals = """To find an arbitrary progression in some composer's work, you can use the rn module.  The simplest method is the regex method,
which uses 'regular expressions' augmented with two abbreviations: CHORD [for any chord] and INV [for any inversion].  The symbols ^ and $ refer to the
start and end of a progression.  Type:

	> import rn
	> rn.load("Bach")
	> rn.regex("viio6 -> CHORD$", 'major')			# 'major' is understood if you don't specify.
	> rn.regex("viio6 -> CHORD$", 'minor')
	> rn.show("viio6 -> vi", rangeList = [0, 3])	# show the fist 3 instances of the progression

"""

__topicVoice_Leading = """The VL module will provide information on how composers voice-lead a particular Roman Numeral progression.  It woks best
with chorale textures.  Type:

	> import vl
	> rn.load("Bach")
	> vl.report("IV -> I", 'major')

The report shows how the voices move in [PITCHCLASS, INTERVAL] format.  The abbreviation S stands for soprano, and P stands for passing tone.
The report is broken into categories based on the upper voices.

"""

__topicNonharmonic_Tones = """The nht module labels nonharmonic tones for a hand-analyzed piece.  The counts dict contains a list of all the nonharmonic
tones:

	> import nht
	> nht.load("Bach")
	> nht.counts
	> nht.show("?", rangeList = [0, 3])
	
Here the most interesting method is the 'rewrite' method, which labels nonharmonic tones in a score.

	> nht.rewrite("riemenschneider072", "Bach")
	
NHT.py can also be used to remove nonharmonic tones, producing 'reduced' files that contain only the harmonic skeleton.
"""