from __future__ import print_function
import os
import time
import copy
import pickle

from music21 import *

import DT
import prune_progressions

chorale = stream.Score()
chords = stream.Stream()
counts = [{}, {}]
progs = [{}, {}]
locations = [{}, {}]
durationCounts = {}
RNoutput = []
output = stream.Score()
lastChorale = 0
chorale = stream.Score()
analyzed_file = stream.Score()
totalCount = 0
falseCount = 0
minDur = 0
lastOutputMeasure = -1
storedString = ''
lastTimeSignature = ''
analysisPath = ''

"""	
A ROMAN-NUMERAL ANALYZER for tonal chorale textures, with some specific tuning for Bach.

To do:
	- document
	- incorporate PRUNE
	- give comparator a 1-beat flexibility for key changes

The original goal was to do fairly objective RN analysis of the Bach chorales, in which every triad and seventh chord is labeled, and
in which keys were determined by scale membership.  The first stage of the program (dochorale) works by finding regions in particular 
scales and assigning Roman numerals to any verticality it can.  It is intended to produce first-stage output to be postprocessed later,
and also for answering methodological questions about the justification of RN analysis.

The later "postprocess" routine (as well as the separate "prune" program) use data from this first stage to try to improve overall performance, while
adding in as little music-theoretical prejudice as possible.  Some information still had to be added however, particularly common cadence types
that involve secondary dominants.

Main data structures:
	chords[chordNumber] -- all the chords in the chorale
	completeScales[chordNumber][scaleIndex] -- a list of the [[major], [melodic minor], [harmonic minor]] scales containing the chord in question
	relativeMinors[chordNumber] -- the natural minor scales containing the notes
	bestKeys[keyIndex] -- the working list of keys in the chorale; a list containing [keyName, startChord, endChord, pivotStart, completeFlag]
	chordList[chordNumber] -- the preliminary analysis in the form "A: IV ||"  Up to three symbols (key, Roman Numeral, and phrase marker)
	
I've included a "raw" option for skipping the post-process and prune stage and a "superRaw" option for skipping virtually all interpretation.

TODO: 
	
	COMPARATOR: allow for 1 beat flexibility where keys start
	
	FINAL CHECK-FORWARD: 
		don't switch keys on a standard diatonic chord (I, ii, iv/IV, V) (see 88 for an example where the switch to G happens too fast)
	After a I or V cadence, a following I should be in the old key
	
	teach decorated suspensions to "COULD BE SUSPENSION!" in this program
	
	allow PRUNE to deal with bass leaps coupled with PTs 328 m2
	
			DONE: viio->I across key changes (184; should be fixed now)
	
	START PHRASES IN THE TONIC IF YOU CAN?  IS THIS USEFUL?
	
	KEY PROBLEM: 184 look at nonchord-tones that don't fit the key -- there's a B natural in the "F" part and that is wrong.  Where does that shift to F happen???
	
	Look at 19.
	
		#19 has a i7 that is unexplained (measure 1)
		19 has a pruning problem as well (why is that d not rewritten????)
	
	308 is a problem:
		- why is the opening vi not a ii? [tricky]
		113: tonicizations of iv followed by V should be caught?
		
	59m5 
		put in 11, 11, 1, 5 into tonic cadences?
		heck, add 3-4-5 to dominant cadences, and 3-4-5-1 to tonic cadences?
		after i, VII6->III defines a new key if it's leaving the old key?
	
	*** -> phrases that end on the dominant of the key they start in; should they be in one key? (113)
	
	Have the analysis file write the percentage correct?  (Hard to do, feasible but a hassle)
	
	Key finding ideas:
	
		count cadences
		
		go through at the end and widen TONIC key regions (check_forward, check_backward)
		
		fix check_forward, check_backward

		dominant cadence formulae (do these)?
			i - V with the half-diminished in between
			4-#4-5 (but check to make sure the last chord is a dominant)
	
		ORPHAN FIXING has problems
			problem with extending keys forward
			
		maybe teach IV6(/5) -> V6(/5) -> i as an idiom?
			
		CHECK_FORWARD and BACKWARD may need to be fixed -- only extend a key area backward if it is SIGNIFICANTLY more likely than the old key (1.5?)  We're switching too quickly.
			this simple solution doesn't work, unfortunately, need a more sophisticated approach
			maybe just a list of newKeyTriggers?  or, a chord has to be really unusual in the old key?
			check forward after check backward?  (one final round of check forward?)
		6/4 as key finder?  more general "key identifying progressions?"  (this led to the cadence idea; other use for it?)
		
		GLOBAL TONALITY BASED KEY FINDING.  MAYBE USE GLOBAL TONIC FINDING SO THAT TONIC REGIONS ARE PRIVILEGED?

	Chord ID ideas:
	
		
		CHEAT: goose the stats to favor certain progressions like (i - v6 - iv6)
		Pruning 3/4 chorales -- quarter note b1 and b2 should be treated as eighth notes; complicated
		
		A more general approach to 16th notes (see 202)
		
	NB: per-chorale accuracy stats get a .16 gain from eliminating two bad chorales, one modal, one without cadences
	NB: Prune is currently not eliminating V and viio; this is good for .1 gain in accuracy, but it is a BRUTE INPUT to the model

Tiny things:

	132 suspension resolution at ambiguous Picardy tonic
	138m11 is interestingly problematic -- harmonic major diminished sound
	201 m7 in Eb going to Ab, with a little Bb in the middle; get rid of that key?  (Generalize this?)
	202 m22 is tricky
	205 has a dorian passage (15, 16)
	chorale 36 m12, beat 1; suspension resolution is being done without thinking about the G#.   pretty subtle
	Chorale 32.  If you resolve an NH tone, check to make sure you're not pushing back a key change!!!!

		
	{'DN': 4, 'ANd': 39, 'Na': 96, 'Nd': 588, 'DA': 86, 'Pa': 2047, 'Pd': 1822, 'PED': 67, 'CPT': 16, 'WBsus': 14, 'LA': 15, 'EQEp': 36, 
	'DS2': 88, 'EQEn': 14, '?': 595, 'A': 248, 'E': 76, 'ANa': 21, 'I': 120, 'S': 1358, 'R': 2, 'APa': 158, 'APd': 385, 'Nnh': 118, 'APP': 20}

for palestrina: 
	RESOLVE SUSPENSIONS RIGHT
	STAY IN ONE KEY THE WHOLE TIME
	NOTE: FIX SINGLE CHORD VIIo AND SINGLE PC PART OF PREV CHORD
	eliminate chords only an quarter beat long
	allow ii7 chords?
	viio6/4 -> ii6/5
	replace V/III and V6/III with VII and VII6

"""

def gather_stats(source='Bach Chorales Cleanmachine', fileName = '', overwrite = False, **kwargs):
	global globalSource, analysisPath, scorePath
	globalSource = source.upper()
	
	if globalSource == 'BACH' or globalSource == 'BACH CHORALES':
		if 'raw' in kwargs and kwargs['raw']:
			globalSource = 'BACH CHORALES RAWMACHINE'
		else:
			globalSource = 'BACH CHORALES CLEANMACHINE'
			
	if globalSource == '.' or globalSource in DT._PATHS:
		if globalSource != '.':
			analysisPath = DT._PATHS[globalSource][0]
			scorePath = DT._PATHS[globalSource][1]
		else:
			scorePath = '.'
			analysisPath = '.'
		fileStr = globalSource
		files = sorted(os.listdir(scorePath))
		analysisDirectory = os.listdir(analysisPath)
		recordResults = True
		counter = 0
		for myFile in files:
			if fileName and myFile != fileName:
				continue
			s = source + '/' + myFile
			if myFile[-3:] in ['xml', 'krn', 'mxl']:
				newName = myFile[:-3]+'txt'
				if (not overwrite) and newName in analysisDirectory:
					continue
				print(myFile)
				do_analysis(fileName = myFile, **kwargs)

def do_analysis(fileName = '', raw = False, superRaw = False, strict = False, major = True, unisons = True, rewrite = [], figuredBass = False):
	global chorale, counts, progs, locations, minDur, nhChords, durationCounts, myFileName, globalRaw, globalSuperRaw, globalStrict, globalMajor
	global totalVerticalities, identified, unknownKey, unknownHarmonies, knownKey, knownHarmonies, vertsInCompleteScale, harmoniesInCompleteScale, outputAnalysis
	global storedString, lastOutputMeasure, storedString, outputAnalysis, lastTimeSignature, newTimeSignature, lastKey, globalJosquin, globalRewrite
	global sus, dP, aP, lN, uN, AdP, AaP, AlN, AuN
	global chords, chordList, measureBeats, rnStream, doUnisons, globalFiguredBass, analysisPath, scorePath
	#print fileName, raw, superRaw, strict, major, unisons, rewrite, figuredBass
	globalFiguredBass = figuredBass
	globalMajor = major
	globalRewrite = rewrite
	doUnisons = unisons
	startT = time.time()
	composerName = globalSource.split()[0]
	composerName = composerName[0] + composerName[1:].lower()
	scoreLocation = scorePath + fileName
	analysisLocation = analysisPath + fileName
	globalStrict = strict
	globalRaw = raw
	globalSuperRaw = superRaw
	if superRaw: raw = True
	totalVerticalities = 0
	identified = 0
	unknownKey = 0
	unknownHarmonies = 0
	nhChords = 0
	knownKey = 0
	sus = dP = aP = lN = uN = AdP = AaP = AlN = AuN = 0
	knownHarmonies = 0
	vertsInCompleteScale = 0
	harmoniesInCompleteScale = 0
	counts = [{}, {}]
	progs = [{}, {}]
	locations = [{}, {}]
	durationCounts = {}
	if scoreLocation.upper().count('JOSQUIN') > 0:
		globalJosquin = True
		globalStrict = True
		strict = True
	else:
		globalJosquin = False
	"""NEED TO WRITE SOMETHING HERE TO CLEAR OUT CHORALE!"""
	chorale = converter.parse(scoreLocation)
	if composerName.count("Sacred") > 0:
		RSH_process_repeats(chorale)
	counter = 0
	for p in chorale.parts:
		p.id = str(counter) + 'A'
		counter += 1
	if chorale: 
		if 'BACH' in globalSource:
			DT.transpose_bass(chorale, '-P8')			# not good for early music, where there is lots of voice crossing
		chordList = {}
		measureBeats = {}
		outFile = analysisLocation[:-4]
		if raw:
			myFileName = outFile + '' + '.txt'
		else:
			myFileName = outFile + '.txt'
		print (' writing to', myFileName)
		outputAnalysis = open(myFileName, 'w+')
		outputAnalysis.write('Composer: ' + composerName + '\n')
		outputAnalysis.write('Analyst: Dmitri Tymoczko and his computer' + '\n')
		beatAndMeasure = chorale.beatAndMeasureFromOffset(0)
		lastTimeSignature = beatAndMeasure[1][-1].getContextByClass('TimeSignature').ratioString
		newTimeSignature = beatAndMeasure[1][-1].getContextByClass('TimeSignature').ratioString
		lastKey = ''
		outputAnalysis.write('Time Signature: ' + newTimeSignature + '\n')
		dochorale(-1)
		#print chordList
		if not raw:
			post_process()
			rnStream = make_stream()
		"""for item in bestKeys:
			print (item)
		for item in chordList:
			print (item, chordList[item])"""
		outputAnalysis.write('\n')
		toDelete = []
		if not raw:
			fakeChords, makeSevenths = prune_progressions.dochorale(chorale, rnStream, strict)
			fakeChords = list(set(fakeChords))
			for chordNo in chordList:
				if offsets[chordNo] in fakeChords:
					toDelete.append(chordNo)
			toDelete = sorted(toDelete)[::-1]
			#print(' deleting keys', toDelete)
			#print(' chordList keys', list(chordList.keys()))
			for chordNo in toDelete:												# untested code here!
				if (offsets[chordNo] - int(offsets[chordNo])) < .01 and (chordNo + 1 in chordList):
					chordList[chordNo] = chordList[chordNo + 1]
					del chordList[chordNo + 1]
				else:	
					if chordNo in chordList:
						del chordList[chordNo]
		final_write()
		outputAnalysis.write(storedString + '\n')
		outputAnalysis.close()
		#subprocess.call(('open', myFileName))
	minutes = (time.time() - startT)/60.
	print ('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	print ("Verticalities, harmonies (complete triads and sevenths)", totalVerticalities, identified)
	print ('Verticalities, harmonies in ambiguous or unknown scale', unknownKey, unknownHarmonies)
	print ('Verticalities, harmonies in unambiguous scale', knownKey, knownHarmonies)
	print ('Verticalities, harmonies in complete scale', vertsInCompleteScale, harmoniesInCompleteScale)
	return

ignoreFigs = ['11.10.']
figDict = {'#': [3, 1], '11.#': [3, 1], '#3': [3, 1], 'b': [3, -1], '#6': [6, +1], 'b6': [6, -1], '6': [6, 0]}

def is_alpha(s):
	for c in s:
		if c.isalpha(): return True
	return False

def split_figure(s):
	myList = ['', '']
	counter = 2
	for myChar in reversed(s):
		if not myChar.isdigit():
			myList[counter] = myChar + myList[counter]
		else:
			if counter > 0: counter = counter - 1
			myList[counter] = myChar + myList[counter]
	return myList

def fix_figured_bass():
	global chorale
	lastPart = chorale.parts[-1]
	cSignature = key.Key('C')
	counter = 0
	tempDict = {}
	textExps = lastPart.flat.getElementsByClass('TextExpression')
	for t in textExps:
		s = ''
		if t.offset in tempDict: 
			s = tempDict[t.offset]
		tempDict[t.offset] = s + t.content
	tempChords = lastPart.flat.getElementsByClass('ChordSymbol')
	for f in tempChords:
		s = ''
		if f.offset in tempDict: 
			s = tempDict[f.offset]
		tempFig = f.figure
		if tempFig.count('6') > 0: tempFig = '6'
		elif tempFig.count('7') > 0: tempFig = '7'
		elif tempFig.count('power') > 0: tempFig = '5'
		elif tempFig.count('add 4') > 0: tempFig = ''
		tempDict[f.offset] = s + tempFig
	delList = []
	for item in tempDict:
		if is_alpha(tempDict[item]): delList.append(item)
		tempDict[item] = tempDict[item].replace("3", "")
		tempDict[item] = tempDict[item].replace("##", "#")
		tempDict[item] = tempDict[item].replace("#56", "5#6")
		if tempDict[item] == '': delList.append(item)
	#for item in sorted(list(tempDict)):
	#	print item, tempDict[item]
	for item in delList:
		del tempDict[item]
	bassNotes = lastPart.flat.getElementsByClass('Note')
	for n in reversed(bassNotes):
		if n.offset in tempDict:
			myText = tempDict[n.offset]
			filteredText = ''.join([y for y in myText if y.isdigit()])
			if filteredText not in ['56', '65']:
				n.lyrics.append(note.Lyric(text=myText))
			else:
				tempDur = n.duration.quarterLength / 2.
				lyricSplit = split_figure(myText)
				for i in [0, 1]:
					if lyricSplit[i] == '5': lyricSplit[i] = ''
				n.duration.quarterLength = tempDur
				newNote = note.Note(n.nameWithOctave, quarterLength = tempDur)
				n.lyrics.append(note.Lyric(text=lyricSplit[0]))
				newNote.lyrics.append(note.Lyric(text=lyricSplit[1]))
				bigOffset = n.offset
				m = lastPart.beatAndMeasureFromOffset(bigOffset)[1]
				m.insert(bigOffset + tempDur - m.offset, newNote)
	for myPart in chorale.parts:
		for myStream in myPart.recurse(streamsOnly=True):
			myStream.removeByClass('ChordSymbol')
			myStream.removeByClass('TextExpression')
	temp = int(lastPart.id[:-1])
	newPart = copy.deepcopy(lastPart)
	newPart.id = str(temp + 1) + 'A'
	newPart2 = copy.deepcopy(lastPart)
	newPart2.id =  str(temp + 2) + 'A'
	bassNotes = lastPart.flat.getElementsByClass('Note')
	for myNote in bassNotes:
		keySig = myNote.getContextByClass('KeySig')
		counter += 1
		if not keySig:
			keySig = cSignature
		figures = {}
		if myNote.lyrics:
			foundIdiom1 = False
			for s in myNote.lyrics:
				myText = s.text.rstrip('.')
				if '7.6' in myText:
					foundIdiom1 = True
				myText = myText.replace('7.6', '6')
				myText = myText.replace('13', '6')
				myText = myText.replace('10', '3')
				myText = myText.replace('17', '3')
				myText = myText.replace(' ', '')
				if (s.text in ignoreFigs) or (len(myText) == 0): continue
				if myText in figDict:
					figures[figDict[myText][0]] = figDict[myText][1]
				else:
					print ("UNKNOWN FIGURE: ", myText)
					continue
				if foundIdiom1 and myText == '#':
					figures = {3: 0, 6:1}
		if (3 not in figures) and (4 not in figures):
			figures[3] = 0
		if (5 not in figures) and (6 not in figures):
			figures[5] = 0
		myThird = newPart.flat.getElementsByOffset(myNote.offset - .01, myNote.offset + .01, includeEndBoundary = False, mustBeginInSpan = True, classList = ['Note'])
		if not myThird:
			print ("ERROR FINDING THIRD!!!!", myNote.offset)
		myThird = myThird[0]
		myThird.transpose('M3', inPlace = True)
		myDegree = keySig.getScaleDegreeAndAccidentalFromPitch(myThird)
		if myDegree[1]:
			if int(myDegree[1].alter) == 1 and figures[3] <= 0:
				myThird.transpose('-A1', inPlace = True)
			if int(myDegree[1].alter) == 0 and figures[3] == -1:
				myThird.transpose('-A1', inPlace = True)
		elif (not myDegree[1]) and figures[3] == -1:
			myThird.transpose('-A1', inPlace = True)
		elif figures[3] == 1:
			myThird.transpose('A1', inPlace = True)
		myFifth = newPart2.flat.getElementsByOffset(myNote.offset - .01, myNote.offset + .01, includeEndBoundary = False, mustBeginInSpan = True, classList = ['Note'])
		if not myFifth:
			print ("ERROR FINDING FIFTH!!!!", myNote.offset)
		myFifth = myFifth[0]
		if 5 in figures:
			myFifth.transpose('P5', inPlace = True)
			alteration = figures[5]
		else:
			myFifth.transpose('M6', inPlace = True)
			alteration = figures[6]
		myDegree = keySig.getScaleDegreeAndAccidentalFromPitch(myFifth)
		if myDegree[1]:
			if int(myDegree[1].alter) == 1 and alteration <= 0:
				myFifth.transpose('-A1', inPlace = True)
			if int(myDegree[1].alter) == 0 and alteration == -1:
				myFifth.transpose('-A1', inPlace = True)
		else:
			if alteration == 1:
				myFifth.transpose('A1', inPlace = True)
			elif alteration == -1:
				myFifth.transpose('-A1', inPlace = True)
	chorale.insert(0, newPart)
	chorale.insert(0, newPart2)
	#chorale.show()
	#print myNote, myThird, myFifth, figures

def dochorale(n, superStrict = False):
	global chorale, chords, progs, minDur, durationCounts, fermatas, completeScales, offsets, choraleNumber, chordsPCs, lastAnalyzedKey, lastStrippedFig
	global totalVerticalities, identified, unknownKey, unknownHarmonies, knownKey, knownHarmonies, vertsInCompleteScale, harmoniesInCompleteScale
	global lastOutputMeasure, storedString, lastTimeSignature, newTimeSignature, bestKeys, incompleteTriads, relativeMinors, globalRaw, globalSuperRaw, globalMajor, globalFiguredBass
	
	choraleNumber = n
	lastOutputMeasure = -1
	storedString = ''
	
	if globalFiguredBass: fix_figured_bass()
	
	preChords = chorale.chordify(removeRedundantPitches = False, addPartIdAsGroup = True, toSoundingPitch = False)
	chords = preChords.flat.getElementsByClass('Chord').stream()
	
	for c in chords:
		counter = -1
		for n in c:
			counter += 1
			if not n.pitch.groups:
				n.pitch.groups = [str(counter) + 'A']
				print ("PROBLEM", c, c.offset, n)
	fermatas = []
	
	for i in range(len(chords)):
		if chords[i].expressions:
			fermatas.append(i)
	
	offsets = [x.offset for x in chords]
	chordsPCs = [x.pitchClasses for x in chords]
	
	make_key_list()											# create the list of keys
	
	if not globalMajor:										# for minor-key Palestrina, get rid of the major keys
		for item in bestKeys:
			if item[0].isupper():
				item[0] = transpose_letter(item[0], '-m3').lower()
	
	incompleteTriads = []
	lastAnalyzedKey = False
	lastStrippedFig = ''
	
	for i in range(len(bestKeys)):							# create the analysis
		analyze_key_area(i)

def make_key_list():
	global chorale, chords, progs, minDur, durationCounts, fermatas, completeScales, offsets, choraleNumber, chordsPCs, lastAnalyzedKey, lastStrippedFig
	global totalVerticalities, identified, unknownKey, unknownHarmonies, knownKey, knownHarmonies, vertsInCompleteScale, harmoniesInCompleteScale
	global lastOutputMeasure, storedString, lastTimeSignature, newTimeSignature, bestKeys, incompleteTriads, relativeMinors, globalRaw, globalSuperRaw, bestScales
	
	"""FIRST PART: make several passes through the chorale to find consecutive regions belonging to the same scales.
	This should not be eliminated.
	
	Step 1A. First we go through the chords to find what scales they belong to, and create regions of consecutive unambiguous scalar membership."""
	
	completeScales = []															# first we find all scales containing each chord
	relativeMinors = []															# for later convenience
	for i in range(len(chords)):												
		completeScales.append(DT.scale_from_letternames(chords[i].pitches))
		#print(i, [x.name for x in chords[i].pitches], completeScales[-1])
		relativeMinors.append([])
		for j in completeScales[i][0]:
			relativeMinors[i].append(pitch.Pitch(j).transpose('-m3').name)		# better to calculate just once
	bestScales = copy.deepcopy(completeScales)
	
	for i in range(1, len(chords)):												# go forward to find shared scales
		jointScales = shared_scales(bestScales[i-1], bestScales[i])
		if not is_empty(jointScales):
			bestScales[i-1]	= jointScales
			bestScales[i] = jointScales
	
	for i in reversed(range(1, len(chords))):									# now do it backward to eliminate ambiguities at the start
		jointScales = shared_scales(bestScales[i-1], bestScales[i])				# of a scalar block
		if not is_empty(jointScales):
			bestScales[i-1]	= jointScales
			bestScales[i] = jointScales
		
	"""Step 1B. Now we kill off all the ambiguities
	
	First we iterate backward from our ambiguous chord, until we get to the beginning of the piece or until we
	get out of the region sharing scales with the ambiguous chord.  
	
	For each of these preceding chords we then iterate the chords AFTER the ambiguous chord to try to make a complete region."""
		
	for i in range(len(chords)):		
		
		if not ambiguous(bestScales[i]): continue
		backwardJointScales = bestScales[i] = completeScales[i]
		foundMatch = False
		for j in reversed(range(0, i + 1)):						
			backwardJointScales = shared_scales(backwardJointScales, completeScales[j])
			if is_empty(backwardJointScales): break
			if not ambiguous(backwardJointScales):
				foundMatch = True
				bestScales[i] = backwardJointScales
				break
			forwardJointScales = copy.deepcopy(backwardJointScales)
			foundMatch = False
			for k in range(i+1, len(chords)):
				forwardJointScales = shared_scales(forwardJointScales, completeScales[k])
				if is_empty(forwardJointScales): break
				if not ambiguous(forwardJointScales):
					foundMatch = True
					bestScales[i] = forwardJointScales
					break
			if foundMatch: break
		if foundMatch:														# copy forward this solution to the next chords
			for j in range(i+1, len(chords)):
				if not ambiguous(bestScales[j]): break						# comment this out to extend our region forward as far as possible
				if not is_empty(shared_scales(bestScales[i], completeScales[j])):
					bestScales[j] = bestScales[i]
				else:
					break
	
	
	"""SECOND PART: Now having compiled a list of bestScales, we can identify the key more definitively
	building up our key list bestKeys.  This is still provisional, and needs to be fixed with chord-based modeling.
	In fact, a lot of the fussing with natural minor/major can probably be replaced.
	
	bestKeys is a list of lists, where each one records a key region:
			[keyLabel, labelStart, end, pivotRegionStart, isScaleComplete, [sharedScales {for ambiguous regions only}]]
	
	keyLabel is a key string like 'a', 'C', or '?'
	labelStart is the first chord at which we are definitively in that key
	end is the last chord we could be in that key
	pivotRegionStart is <= labelStart; it represents the earliest moment we could be in the key
	isScaleComplete is a boolean variable that tells us whether all the pitches of the scale appear
	
	There's some code in here that tries to guess between major and relative minor; it works OK, but is a little seat-of-the-pants.
	
	"""
	
	"""step 2A: assemble the list of keys"""
	
	bestKeys = []											
	lastKeyStr = ''
	lastAmbiguousScales = []
	
	for i in range(len(chords)):						
		if not ambiguous(bestScales[i]):
			lastAmbiguousScales = []
			if len(bestScales[i][0]) == 1: keyStr = list(bestScales[i][0])[0]
			elif len(bestScales[i][1]) == 1: keyStr = list(bestScales[i][1])[0].lower()
			elif len(bestScales[i][2]) == 1: keyStr = list(bestScales[i][2])[0].lower()
			elif len(bestScales[i][2]) == 0: keyStr = 'C'	# if we can't do it, just make it C major!!!
			if not keyStr == lastKeyStr:
				bestKeys.append([keyStr, i, i, i, False])
				lastKeyStr = keyStr
			else:
				bestKeys[len(bestKeys) - 1][2] = i
		else:
			jointScales = shared_scales(lastAmbiguousScales, completeScales[i])
			lastKeyStr = '?'
			if not is_empty(jointScales):
				lastAmbiguousScales = jointScales
				bestKeys[len(bestKeys) - 1][2] = i
				bestKeys[len(bestKeys) - 1][5] = jointScales
			else:
				lastAmbiguousScales = completeScales[i]
				bestKeys.append(['?', i, i, i, False, lastAmbiguousScales])
	
	"""step 2B: assemble pivot regions"""
	for i in range(len(bestKeys)):				
		for j in reversed(range(0, bestKeys[i][1])):
			if bestKeys[i][0] != '?' and not is_empty(shared_scales(completeScales[j], bestScales[bestKeys[i][1]])):
				bestKeys[i][3]= j
			else:
				break
				
	"""step 2C: Check major keys for completeness and remove natural minor sandwiched between regular minor"""
	for i in range(len(bestKeys)):
		if bestKeys[i][0].isupper():												# major keys only
			totalPCs = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
			for j in range(bestKeys[i][3], bestKeys[i][2] + 1):
				for pc in chordsPCs[j]: totalPCs[pc] += 1
			if sum([x > 0 for x in totalPCs]) == 7: bestKeys[i][4] = True				# check major keys for completeness
			basePitch = pitch.Pitch(bestKeys[i][0])
			basePC = basePitch.pitchClass
			relativeName = basePitch.transpose('-m3').name.lower()
			prevRelative = i == 0 or (i > 0 and bestKeys[i-1][0] == relativeName)
			nextRelative = ((i < len(bestKeys) - 1) and bestKeys[i+1][0] == relativeName)		# i = end of array could cause errors depending on the interpreter!
			if (not prevRelative) or (not nextRelative): continue
			totalChords = bestKeys[i][2] + 1 - bestKeys[i][1]
			if totalChords <= 3: bestKeys[i][0] = relativeName
			elif totalChords <= 5:
				majorTD = totalPCs[basePC] + totalPCs[(basePC + 7) % 12]
				minorTD = totalPCs[(basePC + 4) % 12] + totalPCs[(basePC + 9) % 12]
				if minorTD * 3 > majorTD * 2: bestKeys[i][0] = relativeName	
	
	"""step 2D."""															
	consolidate_keys()
	
	"""step 2E"""		
	for i in range(len(bestKeys)):								# check minor keys for completeness
		if not bestKeys[i][0].islower(): continue
		totalPCs = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
		basePC = pitch.Pitch(bestKeys[i][0]).pitchClass
		for j in range(bestKeys[i][3], bestKeys[i][2] + 1):
			for pc in chordsPCs[j]: 
				totalPCs[pc] += 1
			if (totalPCs[basePC] > 0 and
			totalPCs[(basePC + 2)%12] > 0 and
			totalPCs[(basePC + 3)%12] > 0 and
			totalPCs[(basePC + 5)%12] > 0 and
			totalPCs[(basePC + 7)%12] > 0 and
			(totalPCs[(basePC + 8)%12] + totalPCs[(basePC + 9)%12]) > 0 and 
			(totalPCs[(basePC + 11)%12]) > 0):
				bestKeys[i][4] = True
	
	"""THIRD PART: now we try to resolve ambiguous regions, all of which depends on interpretation to some degree
	
	First, iterate through the ambiguous regions looking for the following situation: the chords could belong to
	natural minor and have just followed a minor key.  Then fix the reverse situation, where we have an ambiguous
	but potentially natural-minor passage preceding a minor key"""
	
	if not globalSuperRaw:
	
		removeList = []
		for i in range(1, len(bestKeys)):									
			if bestKeys[i][0] != '?': continue
			fixedRegion = False
			prevKey = bestKeys[i-1][0]
			if prevKey == '?': continue
			if prevKey == prevKey.lower():									# previous key was minor
				targetKey = pitch.Pitch(prevKey).transpose('m3').name
				targetKey2 = prevKey.upper()
				for j in range(bestKeys[i][1], bestKeys[i][2] + 1):
					if targetKey in completeScales[j][0] or targetKey2 in completeScales[j][1] or targetKey2 in completeScales[j][2]:
						bestKeys[i-1][2] = j
						bestKeys[i][1] = j + 1
						fixedRegion = True
					else:
						break
				if bestKeys[i][1] > bestKeys[i][2]: removeList.append(bestKeys[i])		# in case we've wiped out an ambiguity
		for item in removeList: bestKeys.remove(item)			
	
		removeList = []
		for i in range(0, len(bestKeys) - 1):			
			if bestKeys[i][0] != '?': continue
			nextKey = bestKeys[i+1][0]
			if nextKey == '?': continue
			if nextKey == nextKey.lower():									# next key is minor
				targetKey = pitch.Pitch(nextKey).transpose('m3').name
				targetKey2 = nextKey.upper()
				for j in reversed(range(bestKeys[i][1], bestKeys[i][2] + 1)):
					if targetKey in completeScales[j][0] or targetKey2 in completeScales[j][1] or targetKey2 in completeScales[j][2]:
						bestKeys[i+1][1] = j
						bestKeys[i][2] = j - 1
					else:
						break
				if bestKeys[i][1] > bestKeys[i][2]: removeList.append(bestKeys[i])		# in case we've wiped out an ambiguity
		for item in removeList: bestKeys.remove(item)
	
		"""Now a second correction.  If the chorale opens with ambiguous regions that COULD be the same as the final labeled region of the piece, we 
		assume that they are in the final key.  This section can be commented out for a rawer analysis"""
	
		"""CHECK OUT #45 to figure out how to do this right!"""
	
		if bestKeys[0][0] == '?':
			for i in reversed(range(len(bestKeys))):
				if bestKeys[i][0] != '?':					# last labeled region (to deal with picardy endings in minor, or half-cadences)
					finalKeyStr = bestKeys[i][0]
					break
			for i in range(len(bestKeys)):
				if bestKeys[i][0] != '?': break
				totalScales = bestKeys[i][5]
				if finalKeyStr[0].isupper():			
					if finalKeyStr in totalScales[0]:
						bestKeys[i][0] = finalKeyStr
						bestKeys[i][4] = False
					elif finalKeyStr in totalScales[1] or finalKeyStr in totalScales[2] or pitch.Pitch(finalKeyStr).transpose('m3').name in totalScales[0]:
						bestKeys[i][0] = finalKeyStr.lower()
						bestKeys[i][4] = False
				elif ord(finalKeyStr[0]) >= 97 and ord(finalKeyStr[0]) <= 103:
					if finalKeyStr.upper() in totalScales[1] or finalKeyStr.upper() in totalScales[2] or pitch.Pitch(finalKeyStr).transpose('m3').name in totalScales[0]:
						bestKeys[i][0] = finalKeyStr
						bestKeys[i][4] = False
	
		"""THIRD CORRECTION: if we find a region with a lurking V7 or with an isolated major chord, we label accordingly
		This could be fixed -- there's a neapolitan that creates a very strange modulation.  Or fix at a later stage"""
	
		for i in range(0, len(bestKeys) - 1):			
			if bestKeys[i][0] != '?': continue
			keyStr = bestKeys[i]
			foundFirstChord = False
			for j in range(keyStr[1], keyStr[2] + 1):
				myChord = chords[j]	
				if myChord.isTriad() or myChord.isSeventh() or incomplete_seventh(myChord): 
					if myChord.isMajorTriad() and not foundFirstChord:
						possibleTonic = myChord.root().transpose('-P5').name
						if possibleTonic in keyStr[5][0]:
							beatAndMeasure = chorale.beatAndMeasureFromOffset(myChord.offset)
							measureBeat = [beatAndMeasure[1].number, beatAndMeasure[0]]
							#print "     FOUND ISOLATED MAJ TRIAD, CHORALE ", n, 'measure', measureBeat[0], 'beat', measureBeat[1]
							keyStr[0] = possibleTonic
							break
						elif possibleTonic in keyStr[5][1] or possibleTonic in keyStr[5][2]:
							beatAndMeasure = chorale.beatAndMeasureFromOffset(myChord.offset)
							measureBeat = [beatAndMeasure[1].number, beatAndMeasure[0]]
							#print "     FOUND ISOLATED MAJ TRIAD, CHORALE ", n, 'measure', measureBeat[0], 'beat', measureBeat[1]
							keyStr[0] = possibleTonic.lower()
							break
					foundFirstChord = True
					if myChord.isDominantSeventh() or incomplete_seventh(myChord):
						possibleTonic = myChord.root().transpose('-P5')
						minorThird = (possibleTonic.pitchClass + 3) % 12
						majorThird = (possibleTonic.pitchClass + 4) % 12
						possibleTonic = possibleTonic.name
						for k in range(j+1, len(chords)):
							foundIt = False
							if minorThird in chordsPCs[k]:
								keyStr[0] = possibleTonic.lower()
								foundIt = True
							if majorThird in chordsPCs[k]:
								keyStr[0] = possibleTonic
								foundIt = True
							if foundIt:
								beatAndMeasure = chorale.beatAndMeasureFromOffset(myChord.offset)
								measureBeat = [beatAndMeasure[1].number, beatAndMeasure[0]]
								break
	
	"""FOR DEBUGGING PURPOSES, TRY UNCOMMENTING THE FOLLOWING LINES, WHICH SHOW THE CHORDS AND THE BESTKEYS"""
	
	
	"""for i in range(len(chords)): 						# use this to see the complete list of chords and the best keys list
		pitchList = [x.name for x in chords[i]]
		print i, pitchList, chords[i].offset
		
	for item in bestKeys: print item
	print "*****STILL IN MAIN ROUTINE*****"
	"""

def analyze_key_area(i):
	
	"""TO DO: remove colons from key names!!!!!!"""
	
	global chorale, chords, progs, minDur, durationCounts, fermatas, completeScales, offsets, choraleNumber, chordsPCs, lastAnalyzedKey, globalStrict
	global totalVerticalities, identified, unknownKey, unknownHarmonies, knownKey, knownHarmonies, vertsInCompleteScale, harmoniesInCompleteScale
	global lastOutputMeasure, storedString, lastTimeSignature, newTimeSignature, bestKeys, incompleteTriads, lastStrippedFig, globalRaw, globalSuperRaw, doUnisons
	myKey = lastAnalyzedKey
	keyStr = bestKeys[i]
	ambiguousRegionStart = -1
	
	if keyStr[0] == '?':													# first we deal with ambiguous regions
		
		ambiguousRegionStart = keyStr[1]									# need these because we'll be messing around with keyStr
		ambiguousRegionEnd = keyStr[2]										# in the Picardy section.
		
		"""The following passage deals with Picardy-Third type situations: a major chord where a minor would be expected.
		If the previous key was minor and ended with a dominant chord, and if the ambiguous region starts with chords that fit
		in the parallel major, we analyze them in that key.
		
		This involves a certain amount of music-theoretical interpretation.  COMMENT IT OUT for a purer analysis."""
		
		if myKey: potentialKey = myKey.tonic.name
		
		if not globalSuperRaw:
		
			if myKey and myKey.mode == 'minor' and (potentialKey in completeScales[keyStr[1]][0]) and (lastStrippedFig == 'V' or lastStrippedFig[0:3] == 'vii'):
				keyStr[0] = myKey.tonic.name.upper()				# PICARDY SITUATION
				ambiguousRegionStart = keyStr[1] + 1
				keyStr[2] = keyStr[1]
				for j in range(keyStr[1] + 1, ambiguousRegionEnd + 1):
					if potentialKey in completeScales[j][0]:
						keyStr[2] = j
						ambiguousRegionStart = j + 1
						break
			
	"""We record ambiguous regions last; first we do the unambiguous stuff."""
			
	"""NOTE: the following if cannot be an ELSE or ELIF because keyStr[0] can be changed above"""
	
	if keyStr[0] != '?':										# now the case where we know the key;
			
		completeScale = keyStr[4]
		totalVerticalities += keyStr[2] + 1 - keyStr[1]
		displayedKey = False
		knownKey += keyStr[2] + 1 - keyStr[1]
		if completeScale: vertsInCompleteScale += keyStr[2] + 1 - keyStr[1]
		myKey = key.Key(keyStr[0])
		lastAnalyzedKey = myKey
		lastFig = ''
		lastStrippedFig = ''
		lastDur = None
		lastIndex = -1
		
		"""In the following, there is some legacy code from when I used to record roman numerals in this program 
			
		This could be reinstated or cleaned up someday
		"""
		
		for j in range(keyStr[1], keyStr[2] + 1):
			myChord = chords[j]
			myDur = chords[j].duration.quarterLength
			myOffset = offsets[j]
			#print myOffset
			measureBeat = DT.get_measure_number_and_beat_from_offset(chorale, myOffset)
			beatAndMeasure = chorale.beatAndMeasureFromOffset(myOffset)
			newTimeSignature = beatAndMeasure[1][-1].getContextByClass('TimeSignature').ratioString
			measureBeat = [beatAndMeasure[1].number, beatAndMeasure[0]]
			beatFrac = measureBeat[1] - int(measureBeat[1])
			if globalStrict:
				recordIt = myChord.isTriad()
			else:
				recordIt = myChord.isTriad() or myChord.isSeventh() or incomplete_seventh(myChord)
			
			"""Now try to resolve suspensions and incomplete triads.  Comment out for really raw analysis"""
			
			#SEMI-SUPER RAW USES THIS CODE:
			
			if not globalSuperRaw:
			
				if (not recordIt) and ((myDur >= 1.0) or (myDur >= .5 and beatFrac < .25) or (myDur >= .5 and newTimeSignature == '6/8')):			# on the quarter note; fix for 12/8 chorale
					if len(set(chordsPCs[j])) <= 2:								# bare thirds are recorded on the downbeat
						if myChord.third or myChord.fifth or doUnisons: 
							#incompleteTriads.append(j)	# only make triads complete
							recordIt = True
							if lastIndex != -1:
								# try the next line on Jos303b m11
								if myChord.third and chord.Chord(chords[j].pitches + chords[lastIndex].pitches).isTriad():
								#if myChord.third and set(chordsPCs[j]) < set(chordsPCs[lastIndex]):		# wrong!
									newChord = chord.Chord(myChord.transpose('-P8').pitches + chords[lastIndex].transpose('P15').pitches)
									myChord = newChord
					else:
						
						for k in range(len(myChord)):
							if myChord[k].pitch.groups[0]:
								voiceNumber = int(myChord[k].pitch.groups[0][:-1])
							else:
								voiceNumber = k
							if could_be_suspension(myOffset, voiceNumber):	
								#newChord = copy.deepcopy(myChord.pitches)													# voice number not k 
								#newChord = chord.Chord(newChord[:k] + [could_be_suspension.resolution] + newChord[k+1:])	# k not voice number
								#print k, [x.name for x in myChord.pitches], could_be_suspension.resolution
								#print myChord[:k]
								newChord = chord.Chord(list(myChord.pitches[:k]) + [could_be_suspension.resolution] + list(myChord.pitches[k+1:]))	# k not voice number
								if newChord.isTriad() or ((not globalStrict) and (newChord.isSeventh() or incomplete_seventh(newChord))):
									#print "FOUND SUSPENSION: Chorale", n, measureBeat
									recordIt = True
									myChord = chord.Chord(newChord)
									break
							
						"""NOW TRY ACCENTED PASSING AND NEIGHBORING TONES; this doesn't work for chorale 5, m16 b4, because of the incomplete 7th"""
						if not recordIt:			
							for k in range(len(myChord)):
								if myChord[k].pitch.groups[0]:
									voiceNumber = int(myChord[k].pitch.groups[0][:-1])
								else:
									voiceNumber = k
								if could_be_passing_or_neighboring(myOffset, voiceNumber):
									newChord = chord.Chord(list(myChord.pitches[:k]) + [could_be_passing_or_neighboring.resolution] + list(myChord.pitches[k+1:]))
									if newChord.isTriad() or newChord.isSeventh() or incomplete_seventh(newChord):
										#print "FOUND ACCENTED PASSING OR NEIGHBORING: Chorale", n, measureBeat
										recordIt = True
										myChord = chord.Chord(newChord)
										break
										
															
				if (not recordIt) and myOffset - int(myOffset) > .49 and myDur == .5 and j >= 1:			# weak eighth incomplete triad
					newChord = chord.Chord(myChord.transpose('-P8').pitches + chords[j-1].pitches)
					if newChord.isTriad() or newChord.isSeventh():
						recordIt = True
						myChord = newChord
						"""WHAT FOLLOWS IS KLUDGEY CODE FOR FINDING SEVENTH CHORDS WITH PASSING TONES.  NOT PROUD OF THIS"""
					elif chords[j-1].isMajorTriad() and chords[j-1].root().transpose('-M2').pitchClass in myChord.pitchClasses: 
						seventhPC = chords[j-1].root().transpose('-M2').pitchClass
						chordTones = chords[j-1].pitchClasses + [seventhPC]
						tempSeventhPitch = False
						problem = False
						for k in range(len(myChord)):
							if myChord[k].pitch.groups[0]:
								voiceNumber = int(myChord[k].pitch.groups[0][:-1])
							else:
								voiceNumber = k
							if myChord[k].pitch.pitchClass == seventhPC:
								tempSeventhPitch = myChord[k]
								continue
							if myChord[k].pitch.pitchClass in chordTones or could_be_passing_or_neighboring(myOffset, voiceNumber):
								continue
							problem = True
							break
						if not problem:
							recordIt = True
							myChord = chord.Chord(list(chords[j-1].pitches) + [tempSeventhPitch])

				if (not recordIt) and measureBeat[1] == 2.0 and j >=1 and myDur == 1.0 and newTimeSignature == '3/4': # sloppy float programming
					newChord = chord.Chord(myChord.transpose('-P8').pitches + chords[j-1].pitches)
					if newChord.isTriad() or newChord.isSeventh():
						recordIt = True
						myChord = newChord
			
			if (globalStrict) and DT.normal_form([x.midi for x in myChord.pitches]) == [0, 3, 6, 8]:
				recordIt = True
			
			if recordIt:
				identified += 1
				knownHarmonies += 1
				if completeScale: harmoniesInCompleteScale += 1
				myFig = DT.figure_from_chord(myChord, myKey)
				if myFig == False: 
					#print keyStr[0], j, chordList[j]
					continue
				if myFig == 'viio' and len(set(chordsPCs[j])) <= 2 and myChord.third:
					myFig = 'V6'
				#myStrippedFig = strip_roman(myFig).replace('maj','')
				parsedFig = DT.parse_figure(myFig)
				myStrippedFig = parsedFig[0].replace('maj','')	# get rid of major seventh
				
				if globalStrict:
					if parsedFig[1] and parsedFig[1] != '6':
						continue
								
				"""Filter out weak eighth chords that are merely the sevenths of preceding chords.  This can be
				commented out for a more literal analysis.
				
				"""
				
				if (not chords[j-1].isSeventh()) and myChord.isSeventh() and myStrippedFig == DT.parse_figure(lastFig)[0] and myStrippedFig.upper() not in ['V', 'VII']:
					if myDur == .5: 
						myFig = myFig.replace('maj', '')
						myFig = myFig.replace('7','')
						myFig = myFig.replace('6/5', '6')
						myFig = myFig.replace('4/3', '6/4')
						myFig = myFig.replace('2', '')
						if myFig == lastFig: continue
					if myDur == 1.0 and j >= 1 and measureBeat[1] == 2.0:			# sloppy float programming
						if newTimeSignature == '3/4' and chords[j-1].duration.quarterLength == 1.0:
							myFig = myFig.replace('maj', '')
							myFig = myFig.replace('7','')
							myFig = myFig.replace('6/5', '6')
							myFig = myFig.replace('4/3', '6/4')
							myFig = myFig.replace('2', '')
							if myFig == lastFig: continue
				
				if myChord.expressions: 
					myFig = myFig + ' ||'
				theString = keyStr[0] + ': ' + myFig
				outputFile(j, measureBeat, theString)
				lastFig = myFig
				lastStrippedFig = myStrippedFig
				lastDur = myDur
				lastOffset = myOffset
				lastIndex = j
				
	"""Now finally we record the ambiguous regions."""
	
	if ambiguousRegionStart != -1 and (ambiguousRegionStart <= ambiguousRegionEnd):
		recordedRegion = False
		unknownKey += ambiguousRegionEnd + 1 - ambiguousRegionStart		
		for j in range(ambiguousRegionStart, ambiguousRegionEnd + 1):
			myChord = chords[j]
			myOffset = myChord.offset
			beatAndMeasure = chorale.beatAndMeasureFromOffset(myOffset)
			measureBeat = [beatAndMeasure[1].number, beatAndMeasure[0]]
			print ('AMBIGUOUS CHORD, chorale ', choraleNumber, 'measure', measureBeat[0], 'beat', measureBeat[1])
			if myChord.isTriad() or myChord.isSeventh() or incomplete_seventh(myChord):
				identified += 1
				unknownHarmonies += 1
				if not recordedRegion:
					theString = '?:'
					outputFile(j, measureBeat, theString)
					recordedRegion = True	

"""
POST-PROCESS contains all the code that depends on prior knowledge of harmonic idioms; if you want a raw analysis, leave it out.
"""

key_labels = [['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio'],['i', 'iio', 'III', 'iv', 'V', 'VI', 'viio']]
dominants = ['V', 'V7', 'V6/5', 'V4/3', 'V2']

tonicCadences_2 = [[2, 5], [1, 5], [7, 5]]				# these are pc intervals in the bass
tonicCadences_3 = [[9, 5, 5]]
tonicCadences_4 = [[11, 11, 1, 5]]
dominantCadences_2 = [[1, 1]]

def post_process():
	global bestKeys, chords, chordList, outputAnalysis, fermatas, incompleteTriads, bestKeysRealEndings, offsets, relativeMinors, globalJosquin, globalRewrite
	
	#print "INPUT TO POSTPROCESS"
	#for item in bestKeys: print item
	#for i in chordList: print i, chordList[i]
	#for i in range(len(chords)): print i, chords[i]
	
	"""first check for common bass line cadence formulae"""
	
	for i in fermatas:
		bassIntervals = []
		startPoints = []
		lastNote = chords[i].bass()
		cadentialPatternStart = -1
		for j in reversed(range(i)):
			if j in fermatas: break
			newNote = chords[j].bass()
			if newNote.pitchClass == lastNote.pitchClass: continue
			myInterval = interval.notesToChromatic(newNote, lastNote).simpleDirected % 12					# should they be mod 12?
			bassIntervals.append(myInterval)
			startPoints.append(j)
			cadentialPatternStart = j
			if len(bassIntervals) >= 3: break
			lastNote = newNote
		bassIntervals.reverse()
		startPoints.reverse()
		if (bassIntervals[-2:] in tonicCadences_2) or (bassIntervals in tonicCadences_3):
			"""if i not in chordList: 
				print "PROBLEM HERE", i, "no analysis at cadence"
				continue"""
			if cadentialPatternStart == -1:
				print ("PROBLEM HERE", measureBeats[i], "can't find cadential pattern start")
				continue
			target_key = choose_major_or_minor(chords[i].bass().name, i)
			changed_anything = add_key(startPoints[-2], i, target_key)
			if changed_anything:
				if i in measureBeats:
					print ("CHANGED KEY AT THIS CADENCE:", i, measureBeats[i])
				else:
					print ("CHANGED KEY AT THIS CADENCE:", i)
				check_forward(changed_anything, changed_anything + 1)
				consolidate_keys()
	
	
	"""NOW WE TAKE ANOTHER LOOK AT INCOMPLETE TRIADS.  IN PRACTICE, THIS JUST SUBSTITUTES I6 FOR iii
	This could be done even better if we combined with Prune to look at the likeliness of progressions; see chorale 243
	"""
	
	for i in incompleteTriads:
		alertFlag = False
		oldRaw = chordList[i]
		oldData = chordList[i].split()
		myKey_label = oldData[0].rstrip(':')
		if myKey_label == '?:': continue
		myKey = key.Key(myKey_label)
		SDs = []
		for myNote in chords[i]:
			SDs.append(myKey.getScaleDegreeAndAccidentalFromPitch(myNote))
		for item in SDs:
			if item[1] != None: alertFlag = True
		if alertFlag: continue
		stripped_degrees = [(x[0] - 1) % 7 for x in SDs]
		possible_roots = set([0, 1, 2, 3, 4, 5, 6])
		for degree in stripped_degrees:
			new_roots = set([degree, (degree - 2) % 7, (degree - 4) % 7])
			possible_roots = possible_roots & new_roots
		if myKey_label[0].islower(): 
			modeIndex = 1
		else:
			modeIndex = 0
		bestFigure = ''
		bestCount = 0
		for x in possible_roots:
			newRawLabel = key_labels[modeIndex][x]
			newPitches = chords[i].transpose('-P15').pitches + roman.RomanNumeral(newRawLabel, myKey_label).pitches
			newFigure = DT.figure_from_chord(newPitches, myKey)
			newCount = allcounts[modeIndex][newFigure]
			if newCount > bestCount:
				bestCount = newCount
				bestFigure = newFigure
		oldData[1] = bestFigure
		chordList[i] = ' '.join(oldData)
		
		
	"""NOW EXTEND key boundaries backward if the chords and progressions make more sense in the new key"""
	for i in range(1, len(bestKeys)):
		check_boundaries(i, i-1)
		
	"""make sure V-I fermata progressions are registered accordingly
	
	search for major keys with no viio7->I, and either no V or I, and see if we can make them minor, either relative or dorian"""
	
	removeList = []
	chordsToRemove = []
	for i in range(0, len(bestKeys)):
		standardRelative = False
		dorianRelative = False
		if bestKeys[i][2] < bestKeys[i][1]: 
			removeList.append(bestKeys[i])
			continue
		if not bestKeys[i][0].isupper(): 
			continue
		if i == 0 and len(bestKeys) > 1:
			if relative(bestKeys[i+1][0], bestKeys[i][0]): standardRelative = True			# should replace relative and dorian_relative with 'transpose_letter'
			if dorian_relative(bestKeys[i+1][0], bestKeys[i][0]): dorianRelative = True
			if not (standardRelative or dorianRelative): continue
		elif i < len(bestKeys) - 1:
			if (relative(bestKeys[i-1][0], bestKeys[i][0]) or relative(bestKeys[i+1][0], bestKeys[i][0])): standardRelative = True
			if (dorian_relative(bestKeys[i-1][0], bestKeys[i][0]) or dorian_relative(bestKeys[i+1][0], bestKeys[i][0])): dorianRelative = True
			if not (standardRelative or dorianRelative): continue
		foundDominant = False
		foundTonic = False
		for j in range(bestKeys[i][1], bestKeys[i][2] + 1):
			if j not in chordList: 
				continue
			if chordList[j] == '?:':
				break
			oldData = chordList[j].split()
			if len(oldData) == 1: 
				del chordList[j]
				continue
			if DT.parse_figure(oldData[1])[0] == 'V': 	
				foundDominant = True
				if foundTonic: break
			if DT.parse_figure(oldData[1])[0] in ['I']: 
				if oldData[1] == 'I6/4' and chords[j].duration.quarterLength < 1: continue
				foundTonic = True
				if foundDominant: break
		if standardRelative and not foundDominant:						
			chordProgs = get_chord_progs(i, inversions = False)
			if chordProgs.count('viio -> I') == 0 and chordProgs.count('vii/o -> I') == 0:
				print ('REMOVING MAJOR SECTION', bestKeys[i])
				rewrite_key(i, get_relative(bestKeys[i][0]))
				if i != 0: check_boundaries(i, i-1)
		elif dorianRelative and not foundTonic:
			chordProgs = get_chord_progs(i, inversions = False)
			if chordProgs.count('viio -> I') == 0 and chordProgs.count('vii/o -> I') == 0 and chordProgs.count(' ii ') > 0:
				print ('DORIAN RELATIVE HERE?', bestKeys[i])
				rewrite_key(i, get_dorian_relative(bestKeys[i][0]))
				if i != 0: check_boundaries(i, i-1)
			
	for item in removeList:
		bestKeys.remove(item)
	
	consolidate_keys()
	
	"""Now we try to find the global tonic of the piece; this allows us to set first and last key areas"""
	
	keyDict = {}			
	for item in bestKeys: 
		keyDict[item[0]] = keyDict.setdefault(item[0], 0) + (item[2] + 1 - item[1])
	
	
	majorTransps = [['P1', 100, False],['P4', 75, False], ['P5', 75, False]]
	minorTransps = [['P1', 100, True],['m3', 75, False], ['P5', 75, True]]
	
	keysPlusSecondaries = {}
	for item in keyDict:
		if item == '?': continue
		basePitch = pitch.Pitch(item)
		if item.islower(): 
			totalCount = 0
			for myList in minorTransps:
				newKey = basePitch.transpose(myList[0]).name
				if myList[2]: newKey = newKey.lower()
				if newKey in keyDict: totalCount += keyDict[newKey] * myList[1]
			keysPlusSecondaries[item] = totalCount
		else:
			totalCount = 0
			for myList in majorTransps:
				newKey = basePitch.transpose(myList[0]).name
				if newKey in keyDict: totalCount += keyDict[newKey] * myList[1]
			keysPlusSecondaries[item] = totalCount
	
	maxKey = [0, 0]
	for item in keysPlusSecondaries:
		if keysPlusSecondaries[item] > maxKey[1]:
			maxKey = [item, keysPlusSecondaries[item]]
	
	"""CHECK HERE FOR THE ROOTS OF THE FIRST AND LAST CHORD
	"""
	
	firstRoot = chords[0].root().name
	lastRoot = chords[-1].root().name
	print ('First root', firstRoot, 'last root', lastRoot, 'largest key area', maxKey[0])
	finalTonic = maxKey[0]
	if firstRoot != lastRoot or firstRoot != maxKey[0].upper():
		if firstRoot == maxKey[0].upper():
			if chords[0].root().transpose('P5') == lastRoot:
				print ('ENDS ON THE DOMINANT')
		elif firstRoot == lastRoot:
			if chords[0].root().transpose('-P5').name == maxKey[0].upper() and (chords[0].isMajorTriad() or chords[0].isDominantSeventh()) and chords[-1].isMajorTriad():
				print ('STARTS AND ENDS ON DOMINANT')
			elif chords[0].isDominantSeventh() and chords[-1].isMajorTriad():
				finalTonic = chords[0].root().transpose('-P5').name.lower()
				print ('STARTS AND ENDS ON DOMINANT')
			else:
				print ('SPENDS MOST OF ITS TIME IN A NONTONIC KEY??')
				finalTonic = firstRoot
				if chords[0].third and (chords[0].third.pitchClass - chords[0].root().pitchClass) % 12 == 3: finalTonic = finalTonic.lower()			# CHECK THIS, IT COULD CAUSE PROBLEMS (incomplete triads)
		elif lastRoot == maxKey[0].upper():
			if chords[0].root().transpose('-P5').name == lastRoot:
				print ('STARTS ON THE DOMINANT')
		else:
			print ('CONFLICTING INFO HERE, first root', firstRoot, 'last root', lastRoot, 'largest key area', maxKey)
			finalTonic = '?'
	
	"""NOW ADJUST THE START AND END KEYS OF THE PIECE"""
	
	if (finalTonic not in [True, False]) and finalTonic != '?':
		if bestKeys[0][0].upper() != finalTonic.upper():
			if bestKeys[0][2] <= 3:
				rewrite_key(0, finalTonic)
				bestKeys[0][0] = finalTonic
				check_forward(0, 1)
			else:
				bestKeys.insert(0, [bestKeys[0][0], 0, 0, 0, True])
				bestKeys[1][1] = 1
				rewrite_key(0, finalTonic)
				bestKeys[0][0] = finalTonic
				check_forward(0, 1)
		"""SHOULD REALLY FIX THIS HERE.  VERY TRICKY TO DO RIGHT"""
		if bestKeys[-1][0] != finalTonic and bestKeys[-1][0] != transpose_letter(finalTonic.upper(), 'm3'): #and not (bestKeys[-1][0].islower() and bestKeys[-1][0].upper() == transpose_letter(finalTonic.upper(), '-P5')):
			print ('CHANGING FINAL KEY HERE', bestKeys[-1][0], 'to', finalTonic	)				
			bestKeys[-1][0] = finalTonic
			rewrite_key(-1, finalTonic)
			check_boundaries(-2, -1)
	
	outputAnalysis.write('Global Tonic: ' + finalTonic + '\n')
	
	"""OLD OPRHAN CLEANUP.  Seems to work better than the new version.  Eliminate small key areas.
	
	"""
	
	for i in range(1, len(bestKeys) - 1):
		if bestKeys[i][2] - bestKeys[i][1] == 0:
			rewrite_key(i, bestKeys[i+1][0])
	
	for i in range(1, len(bestKeys) - 1):	  
		if (bestKeys[i][2] - bestKeys[i][1] <= 2 or bestKeys[i][0].lower() == bestKeys[i-1][0]) and (bestKeys[i-1][0] == bestKeys[i+1][0]):
			rewrite_key(i, bestKeys[i+1][0])	
	
	if finalTonic.islower():														# get rid of V key areas in minor
		badKey = transpose_letter(finalTonic.upper(), 'P5')
		for i in range(len(bestKeys)):
			if bestKeys[i][0] == badKey and bestKeys[i][2] - bestKeys[i][1] < 3:
				if bestKeys[i-1][0] == finalTonic:
					print ("REPLACING MAJOR DOMINANT", bestKeys[i][0], 'WITH', finalTonic)
					rewrite_key(i, finalTonic)
				elif bestKeys[i-1][0] == badKey.lower():
					print ("REPLACING MAJOR DOMINANT", bestKeys[i][0], 'WITH', bestKeys[i][0].lower())
					rewrite_key(i, badKey.lower())
				elif i < len(bestKeys) - 1:
					if bestKeys[i+1][0] == finalTonic:
						print ("REPLACING MAJOR DOMINANT", bestKeys[i][0], 'WITH', finalTonic)
						rewrite_key(i, finalTonic)
					elif bestKeys[i+1][0] == badKey.lower():
						print ("REPLACING MAJOR DOMINANT", bestKeys[i][0], 'WITH', bestKeys[i][0].lower())
						rewrite_key(i, badKey.lower())
	
	if globalRewrite:
		for i in range(len(bestKeys)):
			if bestKeys[i][0] == globalRewrite[0]:
				rewrite_key(i, globalRewrite[1])
	
	consolidate_keys()
	fix_bestKeys()
	
	"""ORPHAN FIXING; somewhat experimental code that needs to be checked!!!
	doesn't work b/c of a bug in bestKeysRealEndings
	
	for i in range(1, len(bestKeys) - 1):	
		if (bestKeys[i-1][0] == finalTonic and finalTonic == bestKeys[i+1][0]) and (bestKeys[i+1][3] - bestKeysRealEndings[i-1] <= 3):
			rewrite_key(i, bestKeys[i+1][0])
	
	consolidate_keys()"""
	
	"""FINAL BITS OF CLEANUP: 
		don't change keys on a weak eighth unless we have to; and
		rewrite ambiguous regions in the following key (kind of a hail Mary)"""
	
	for i in reversed(range(1, len(bestKeys))):			
		if bestKeys[i][0] != '?':
			j = bestKeys[i][1]
			if offsets[j] - int(offsets[j]) > .3 and in_key(j, bestKeys[i-1][0]):
				if j not in chordList: continue
				myKey = key.Key(bestKeys[i-1][0])
				oldData = chordList[j].split()
				figureInNewKey = DT.figure_from_chord(roman.RomanNumeral(oldData[1], oldData[0].rstrip(':')).pitches, myKey)
				bestKeys[i-1][2] += 1
				bestKeys[i][1] += 1
				oldData[0] = bestKeys[i-1][0] + ':'
				oldData[1] = figureInNewKey
				chordList[j] = ' '.join(oldData)
		else:
			if i < len(bestKeys) - 1:
				rewrite_key(i, bestKeys[i+1][0])
			else:
				rewrite_key(i, bestKeys[i-1][0])
			check_forward(i-1, i)
			
	consolidate_keys()
	
	"""EXPERIMENTAL PHRASE-BASED KEY SETTING
	
	CHECK OUT CHORALE 10, which gets one phrase wrong.  Q: should we limit the size of the keys we overwrite?
	
	OR ADD A SPECIFIC IDIOM FOR RELATIVE MAJOR PASSAGES > THAN A CERTAIN LENGTH == uncomment the code below
	"""
	
	lastFermata = -1
	
	for i in fermatas:
		phraseStart = lastFermata + 1
		lastFermata = i
		firstKey = get_key_number(phraseStart)
		lastKey = get_key_number(i)
		if firstKey == lastKey: continue
		if bestKeys[firstKey][0] != bestKeys[lastKey][0]: continue
		doIt = True
		#if firstKey + 2 == lastKey and relative(bestKeys[firstKey][0], bestKeys[firstKey + 1][0]) and (bestKeys[firstKey + 1][2] - bestKeys[firstKey + 1][1] >= 4): 
		#	continue
		for j in range(firstKey + 1, lastKey):
			if bestKeys[j][2] - bestKeys[j][1] >= 4:
				doIt = False
				break
		if not doIt: continue
		print ("SECONDARY DOMINANT PHRASE HERE!", phraseStart, i)
		for j in range(firstKey + 1, lastKey):
			rewrite_key(j, bestKeys[firstKey][0])
	
	
	"""ANOTHER ATTEMPT AT PHRASE-BASED KEY SETTING, THIS TIME EMPHASIZING THE TONIC
	This is experimental code; we should run a test; also expand to minor"""
	
	lastFermata = -1
	
	majorMode = finalTonic.isupper()
	
	for phraseEnd in fermatas:
		phraseStart = lastFermata + 1				# i is the phrase end
		if lastFermata == -1:
			lastKey = 'x'
		else:
			keyNo = get_key_number(lastFermata)
			if get_key_number.errorFlag: 
				lastFermata = phraseEnd
				continue
			lastKey = bestKeys[keyNo][0]
		lastFermata = phraseEnd
		keyNo = get_key_number(phraseEnd)
		if not keyNo: continue
		if bestKeys[keyNo][0] != finalTonic: continue
		badQuarters = 0
		for j in range(phraseStart, phraseEnd + 1):
			if ((majorMode) and finalTonic not in completeScales[j][0]) or ((not majorMode) and (finalTonic not in completeScales[j][2])):
				badQuarters += chords[j].duration.quarterLength
				"""If the bad chord is in the previous key, don't rewrite!  This isn't foolproof, but it will help"""
				if lastKey.isupper() and lastKey in completeScales[j][0]: 
					badQuarters = 10000											# cheap flag to quit
					break
				elif lastKey.islower() and lastKey.upper() in completeScales[j][2]:
					badQuarters = 10000											# cheap flag to quit
					break
		if badQuarters <= 1:
			if majorMode:
				dummyVar = add_key(phraseStart, phraseEnd, finalTonic)
				consolidate_keys()
				if dummyVar:
					print( "*** GOING INTO THE TONIC HERE!", phraseStart, phraseEnd)
			else:
				print ("???? Go into minor here?", phraseStart, phraseEnd)
				
	#IDIOM CLEANUP iio -> III in minor, or more generally diminished triad to major/minor triad
	
	tempNextChord = -1
	nextChord = {}
	for i in reversed(range(len(chords))):
		if i in chordList:
			nextChord[i] = tempNextChord
			tempNextChord = i
	
	"""THIS IS MESSED UP!!!! NEED TO TEST IT
	experimental stuff in here"""
	
	for i in range(len(chords)):
		if i in chordList:
			myData = chordList[i].split()
			if (not myData) or myData[0] == '?' or len(myData) == 1: continue
			if myData[0].islower():
				parsedFigure = DT.parse_figure(myData[1])
				if (parsedFigure[0] == 'iio' or parsedFigure[0] == 'ii/o') and (not parsedFigure[2]) and nextChord[i] != -1:
					nextData = chordList[nextChord[i]].split()
					if (not nextData) or len(nextData) == 1: continue
					nextParsedFigure = DT.parse_figure(nextData[1])
					if nextData[0] == myData[0]:
						if nextParsedFigure[0] == 'III':
							print ("FOUND iio -> III", i)
							myData[1] = 'v' + myData[1] + '/III'
							chordList[i] = ' '.join(myData)
							"THIS NEXT THING IS KLUDGEY.  SHOULD REANALYZE THE CHORD IN THE FOLLOWING KEY"
					elif (chords[i+1].root().pitchClass - 1) % 12 == chords[i].root().pitchClass and (chords[i+1].isMajorTriad() or chords[i+1].isMinorTriad()):
						add_key(i, i, nextData[0].rstrip(':'))
	
	consolidate_keys()
	
	for i in range(len(bestKeys)): 
		print (bestKeys[i])
	

"""UTILITY ROUTINES FOR POST PROCESS"""

def choose_major_or_minor(keyName, cadencePoint):
	global bestKeys
	minorKeyName = keyName.lower()
	majorKeyName = keyName.upper()
	minorCount = 0
	majorCount = 0
	"""Could add something here that looks for the nearest third before the cadence"""
	for i in range(len(bestKeys)):
		if bestKeys[i][0] == minorKeyName:
			minorCount += bestKeys[i][2] + 1 - bestKeys[i][1]
		elif bestKeys[i][0] == majorKeyName:
			majorCount += bestKeys[i][2] + 1 - bestKeys[i][1]
	if majorCount > minorCount: return majorKeyName
	if minorCount > majorCount: return minorKeyName
	if chords[cadencePoint].isMinorTriad(): return minorKeyName
	return majorKeyName

def get_key_number(chordNo):
	global bestKeys
	get_key_number.errorFlag = False
	for i in range(len(bestKeys)):
		if bestKeys[i][1] <= chordNo and bestKeys[i][2] >= chordNo:
			return i
	print ("ERROR!!!! can't find a key containing chord number", chordNo)
	get_key_number.errorFlag = True													# can't return False, because that is 0!!!

def add_key(startChord, endChord, targetKey):		
	startIndex = -1
	endIndex = -1
	for i in range(len(bestKeys)):
		#print bestKeys[i]
		if bestKeys[i][1] <= startChord and bestKeys[i][2] >= startChord:
			startIndex = i
		if bestKeys[i][1] <= endChord and bestKeys[i][2] >= endChord:
			endIndex = i
			break
	if startIndex == -1 or endIndex == -1:
		print ("ERROR HERE!!! COULDN'T FIND START AND END KEY AREAS!")
		return False
	if startIndex == endIndex and bestKeys[startIndex][0] == targetKey: 
		return False
	if startIndex == endIndex:
		newKeyArea = copy.deepcopy(bestKeys[startIndex])
		bestKeys.insert(endIndex, newKeyArea)
		bestKeys.insert(endIndex + 1, [targetKey, startChord, endChord, startChord, False])			# should fix the pivot regions
		endIndex += 2
		bestKeys[startIndex][2] = startChord - 1
		bestKeys[endIndex][1] = endChord + 1
	else:
		bestKeys[startIndex][2] = startChord - 1
		bestKeys[endIndex][1] = endChord + 1
		bestKeys.insert(startIndex + 1, [targetKey, startChord, endChord, startChord, False])			# should fix the pivot regions
		#print "INSIDE_BESTKEYS", startIndex + 2, endIndex + 1
		#for j in range(len(bestKeys)): print j, bestKeys[j]
		for j in reversed(range(startIndex + 2, endIndex + 1)):
			bestKeys.pop(j)
	rewrite_key(startIndex + 1, targetKey)
	return startIndex + 1

def rewrite_key(target, myKey_name):
	global bestKeys
	ambiguityFlag = False
	if myKey_name[0] == '?': return					# ignore an incoherent request!
	if bestKeys[target][0] != '?':
		bestKeys[target][0] = myKey_name
		myKey = key.Key(myKey_name)
		#print myKey_name, myKey, type(myKey)
		for j in range(bestKeys[target][1], bestKeys[target][2] + 1):
			if j not in chordList: continue
			if chordList[j][0] == '?': 
				ambiguityFlag = True
				break
			oldData = chordList[j].split()
			if len(oldData) == 1:
				ambiguityFlag = True
				break
			print (oldData[1], oldData[0].rstrip(':'), myKey)
			#print roman.RomanNumeral(oldData[1], oldData[0].rstrip(':')).pitches
			figureInNewKey = DT.figure_from_chord(chord.Chord(roman.RomanNumeral(oldData[1], oldData[0].rstrip(':')).pitches), myKey)
			oldData[0] = myKey_name + ':'
			oldData[1] = figureInNewKey
			chordList[j] = ' '.join(oldData)
	if ambiguityFlag or bestKeys[target][0] == '?':			# need to reanalyze
		bestKeys[target][0] = myKey_name
		lastAnalyzedKey = False
		lastStrippedFig = ''
		analyze_key_area(target)

def fix_bestKeys():									# problem with realendings!!!
	global bestKeys, bestKeysRealEndings, chords
	bestKeys[0][3] = 0
	bestKeysRealEndings = []
	for i in range(1, len(bestKeys)):
		for j in reversed(range(0, bestKeys[i][1])):
			if in_key(j, bestKeys[i][0]):
				bestKeys[i][3] = j
			else:
				break
	for i in range(0, len(bestKeys)):
		bestKeysRealEndings.append(bestKeys[i][2])
		for j in range(bestKeys[i][2], len(chords)):
			if in_key(j, bestKeys[i][0]):
				bestKeysRealEndings[i] = j
			else:
				break

def check_boundaries(i, otherkey):				# stop at fermatas!
	global allcounts
	firstChord = True
	newFollowingChord = ''
	oldFollowingChord = ''
	if bestKeys[i][0] == '?' or bestKeys[otherkey][0] == '?': return
	for j in reversed(range(bestKeys[otherkey][1], bestKeys[i][1])):
		myKey = key.Key(bestKeys[i][0])
		if j in fermatas: break
		if not in_key(j, bestKeys[i][0]): break
		if j in chordList:
			oldData = chordList[j].split()
			#print j, oldData
			oldKey = oldData[0]
			if oldKey == '?:': return
			figureInOldKey = oldData[1]
			if firstChord:
				if oldKey.isupper(): 
					oldModeInd = 0
				else:
					oldModeInd = 1
					
				"""FIX THIS.  WHEN WE CHANGE A CHORD, RECORD THAT SOMEWHERE AND USE THOSE CHORDS"""
				figureInNewKey = DT.figure_from_chord(chord.Chord(roman.RomanNumeral(figureInOldKey, oldKey.rstrip(':')).pitches), myKey)
				newModeInd = int(myKey.mode == 'minor')
				if figureInOldKey in allcounts[oldModeInd]:
					oldKeyCount = to_int(allcounts[oldModeInd][figureInOldKey] / float(single_chords[oldModeInd]))
				else:
					oldKeyCount = -1
				if figureInNewKey in allcounts[newModeInd]:
					newKeyCount = to_int(allcounts[newModeInd][figureInNewKey] / float(single_chords[newModeInd]))
				else:
					newKeyCount = 0	
				
				#if newKeyCount < .0067 and oldKeyCount < .0067: continue				# new, experimental code!!!
				
				if newKeyCount > oldKeyCount: 
					
					firstChord = False
					newFollowingChord = figureInNewKey
					oldFollowingChord = figureInOldKey
					oldData[0] = bestKeys[i][0] + ':'
					oldData[1] = figureInNewKey
					chordList[j] = ' '.join(oldData)
					bestKeys[i][1] = j
					bestKeys[otherkey][2] = j - 1
				else: 
					
					break
			else:
				figureInNewKey = DT.figure_from_chord(chord.Chord(roman.RomanNumeral(figureInOldKey, oldKey.rstrip(':')).pitches), myKey)
				oldKeyProg = figureInOldKey + ' -> ' + oldFollowingChord
				newKeyProg = figureInNewKey + ' -> ' + newFollowingChord
				try:
					oldKeyCount = to_int(allcounts[oldModeInd][oldKeyProg] / float(chord_pairs[oldModeInd]))
				except KeyError:
					oldKeyCount = -1
				try:
					newKeyCount = to_int(allcounts[newModeInd][newKeyProg] / float(chord_pairs[newModeInd]))
				except KeyError:
					newKeyCount = 0
				#print bestKeys[i-1][0], oldKeyProg, oldKeyCount, bestKeys[i][0], newKeyProg, newKeyCount
				if newKeyCount > oldKeyCount: 
					#print "NEW KEY WINS:", bestKeys[i][0], newKeyProg
					newFollowingChord = figureInNewKey
					oldFollowingChord = figureInOldKey
					oldData[0] = bestKeys[i][0] + ':'
					oldData[1] = figureInNewKey
					chordList[j] = ' '.join(oldData)
					bestKeys[i][1] = j
					bestKeys[otherkey][2] = j - 1
				else: 
					#print "OLD KEY STAYS:", bestKeys[i-1][0], oldKeyProg
					break

def check_forward(i, otherkey):				# stop at fermatas
	global fermatas, chordList
	firstChord = True
	newPrecedingChord = ''
	oldPrecedingChord = ''
	if bestKeys[i][0] == '?' or bestKeys[otherkey][0] == '?': return
	for j in range(bestKeys[i][2] + 1, bestKeys[otherkey][2]):
		myKey = key.Key(bestKeys[i][0])
		if j in fermatas: break
		if not in_key(j, bestKeys[i][0]): break
		if j in chordList:
			oldData = chordList[j].split()
			oldKey = oldData[0]
			if oldKey == '?:': return
			figureInOldKey = oldData[1]
			
			if firstChord:
				if oldKey.isupper(): 
					oldModeInd = 0
				else:
					oldModeInd = 1
					
				"""FIX THIS.  WHEN WE CHANGE A CHORD, RECORD THAT SOMEWHERE AND USE THOSE CHORDS"""
				
				figureInNewKey = DT.figure_from_chord(chord.Chord(roman.RomanNumeral(figureInOldKey, oldKey.rstrip(':')).pitches), myKey)
				newModeInd = int(myKey.mode == 'minor')
				if figureInOldKey in allcounts[oldModeInd]:
					oldKeyCount = to_int(allcounts[oldModeInd][figureInOldKey] / float(single_chords[oldModeInd]))
				else:
					oldKeyCount = -1
				if figureInNewKey in allcounts[newModeInd]:
					newKeyCount = to_int(allcounts[newModeInd][figureInNewKey] / float(single_chords[newModeInd]))
				else:
					newKeyCount = 0	
				
				parsedFig = DT.parse_figure(figureInNewKey)
				
				"""EXPERIMENT: ADDING IV AND iv to the following.  See if it gets us above 79.81"""
				
				if (newModeInd == 0 and parsedFig[0] in ['I', 'IV', 'V'] and (not parsedFig[3])) or (newModeInd == 1 and parsedFig[0] in ['i', 'iv', 'V'] and (not parsedFig[3])):				# all experimental PUTTING I and i here is weak; should be keyed to mode
					oldData[0] = bestKeys[i][0] + ':'
					oldData[1] = figureInNewKey
					chordList[j] = ' '.join(oldData)
					bestKeys[i][2] = j
					bestKeys[otherkey][1] = j + 1
				elif newKeyCount > oldKeyCount:
					#print "NEW KEY WINS:", bestKeys[i][0], figureInNewKey
					firstChord = False
					newPrecedingChord = figureInNewKey
					oldPrecedingChord = figureInOldKey
					oldData[0] = bestKeys[i][0] + ':'
					oldData[1] = figureInNewKey
					chordList[j] = ' '.join(oldData)
					bestKeys[i][2] = j
					bestKeys[otherkey][1] = j + 1
				else: 
					#print "OLD KEY STAYS:", bestKeys[i-1][0], figureInOldKey
					break
			else:
				figureInNewKey = DT.figure_from_chord(roman.RomanNumeral(figureInOldKey, oldKey.rstrip(':')).pitches, myKey)
				oldKeyProg = oldPrecedingChord + ' -> ' + figureInOldKey
				newKeyProg = newPrecedingChord + ' -> ' + figureInNewKey
				try:
					oldKeyCount = to_int(allcounts[oldModeInd][oldKeyProg] / float(chord_pairs[oldModeInd]))
				except KeyError:
					oldKeyCount = -1
				try:
					newKeyCount = to_int(allcounts[newModeInd][newKeyProg] / float(chord_pairs[newModeInd]))
				except KeyError:
					newKeyCount = 0
				#print bestKeys[i-1][0], oldKeyProg, oldKeyCount, bestKeys[i][0], newKeyProg, newKeyCount
				if newKeyCount > oldKeyCount: 
					#print "NEW KEY WINS:", bestKeys[i][0], newKeyProg
					newPrecedingChord = figureInNewKey
					oldPrecedingChord = figureInOldKey
					oldData[0] = bestKeys[i][0] + ':'
					oldData[1] = figureInNewKey
					chordList[j] = ' '.join(oldData)
					bestKeys[i][2] = j
					bestKeys[otherkey][1] = j + 1
				else: 
					#print "OLD KEY STAYS:", bestKeys[i-1][0], oldKeyProg
					break
						
def to_int(myFloat):
	return int((100000. * myFloat) + .5)
	
def get_chord_progs(i, inversions = True):
	global bestKeys
	progs = ''
	if bestKeys[i] == '?': return ''
	for j in range(bestKeys[i][1], bestKeys[i][2] + 1):
		if j in chordList:
			oldData = chordList[j].split()
			if chordList[j] == '?:': return progs
			myChord = oldData[1]
			if not inversions:
				parsedFig = DT.parse_figure(oldData[1])
				myChord = parsedFig[0]
				if parsedFig[2]: myChord += '/' + parsedFig[2]
			if progs:
				progs = progs + ' -> ' + myChord
			else:
				progs = ' ' + myChord
	return progs

def in_key(i, myKeyName):
	global completeScales, relativeMinors
	if myKeyName == '?': return False
	if myKeyName[0].islower():
		if myKeyName.upper() in completeScales[i][1] or myKeyName.upper() in completeScales[i][2] or myKeyName.upper() in relativeMinors[i]:
			return True
	elif myKeyName in completeScales[i][0]:
		return True
	else:
		return False

def init():
	global allcounts, single_chords, chord_pairs
	try:
		with open (DT._DATAPATH + 'RNBACH CHORALES RAWMACHINE.pkl', 'rb') as storedFile:
			allcounts = pickle.load(storedFile)
			storedFile.close()
			single_chords = [0, 0]
			chord_pairs = [0, 0]
			for i in [0, 1]:
				for item in allcounts[i]:
					length = item.count(' -> ')
					if length == 0: single_chords[i] += allcounts[i][item]
					elif length == 1: chord_pairs[i] += allcounts[i][item]
		return True
	except:
		print("Can't load RNBACH CHORALES RAWMACHINE.pkl file, please run rn.gather_stats('Bach chorales rawmachine')")
		return False

init()

def consolidate_keys():
	global bestKeys
	for i in reversed(range(1, len(bestKeys))):
		if bestKeys[i][2] < bestKeys[i][1]:											# delete regions that end before they start!
			bestKeys.remove(bestKeys[i])
			continue
		if bestKeys[i][0] == bestKeys[i-1][0] and bestKeys[i][0] != '?':
			bestKeys[i-1][2] = bestKeys[i][2]
			bestKeys.remove(bestKeys[i])

def key_duration(i):					# note: call consolidate_keys before running this routine!
	global bestKeys, chords
	return ((chords(bestKeys[i][2]).offset + chords(bestKeys[i][2]).duration.quarterLength) - chords(bestKeys[i][1]).offset)

"""The following method writes the output file.  It expects the chord number, a [measure #, beat] list, followed by a string like

	D: IV ||
	
And performs various consolidations -- for instance, grouping together all chords belonging to a single measure, eliminating "b1", and getting
rid of redundant key areas.

Basically I tried to encapsulate things as much as possible so I could call the method from various points in the main dochorale routine.

"""

chordList = {}
measureBeats = {}

def outputFile(chordNo, measureBeat, theString):
	global chordList, measureBeats
	chordList[chordNo] = theString
	measureBeats[chordNo] = measureBeat
	return

def final_write():
	global lastOutputMeasure, storedString, lastKey, lastMeasure, lastString, outputAnalysis, lastTimeSignature, newTimeSignature, chorale, chords
	global chordList, measureBeats
	"""BUILD A FILTER HERE TO REMOVE LITERAL REPETITIONS?
	STORE THE INFORMATION FOR A LATER ROUND OF PROCESSING"""
	jValues = sorted(list(chordList))
	lastString = ''
	lastMeasure = -1
	for chordNo in jValues:
		theString = chordList[chordNo]
		measureBeat = measureBeats[chordNo]
		
		# lots of subtleties here, not 100% certain it will work
		
		mNum = measureBeat[0]
		bNum = measureBeat[1]
		if mNum < lastMeasure:
			bNum = chorale.measure(lastMeasure).highestTime + bNum
			mNum = lastMeasure	
		
		ts = chorale.beatAndMeasureFromOffset(chords[chordNo].offset)[1].getContextByClass('TimeSignature')
		if ts:
			newTimeSignature = ts.ratioString
		else:
			newTimeSignature = lastTimeSignature
		#newTimeSignature = chorale.beatAndMeasureFromOffset(chords[chordNo].offset)[1][-1].getContextByClass('TimeSignature').ratioString
		#print(chords[chordNo], chords[chordNo].offset, chorale.beatAndMeasureFromOffset(chords[chordNo].offset)[1].getContextByClass('TimeSignature'))
		if theString == lastString and mNum == lastMeasure: continue
		else: 
			lastString = theString
		stringSplit = theString.split()
		lastMeasure = mNum
		for i in range(len(stringSplit)):
			if stringSplit[i].count(':') > 0:
				stringSplit[i] = stringSplit[i].replace('-','b')
				if stringSplit[i] == lastKey or stringSplit[i].count('?'):
					stringSplit = stringSplit[:i] + stringSplit[i+1:]
					break
				else:
					lastKey = stringSplit[i]
		if bNum == 1: 
			beatStr = ''
		else:
			beatStr = 'b' + str(round(float(bNum), 2))
			if beatStr[-2:] == '.0': beatStr = beatStr[:-2]
			beatStr = beatStr.replace('.33333333333', '.33')
			beatStr = beatStr.replace('.66666666667', '.66')
		if mNum == lastOutputMeasure:
			newStr = ' '
			newStr = newStr.join(stringSplit)
			newStr = beatStr + ' ' + newStr
			storedString = storedString + ' ' + newStr
		else:
			if storedString:
				if newTimeSignature != lastTimeSignature:
					outputAnalysis.write('Time Signature: ' + newTimeSignature + '\n')
					lastTimeSignature = newTimeSignature
				if storedString.count(' b5 ') == 0:
					outputAnalysis.write(storedString + '\n')
				else:
					outputAnalysis.write('Time Signature: 5/4\n')			# kludge for two unnotated 5/4 bars in the chorales (197, 280)
					outputAnalysis.write(storedString + '\n')
					lastTimeSignature = '5/4'
			lastOutputMeasure = mNum
			newStr = ' '
			newStr = newStr.join(stringSplit)
			if beatStr == '':
				storedString = 'm' + str(mNum) + ' ' + newStr
			else:
				storedString = 'm' + str(mNum) + ' ' + beatStr + ' ' + newStr

"""MORE UTILITY FUNCTIONS"""

def make_stream():
	global chordList
	tempStream = stream.Stream()
	lastKey = ''
	for i in chordList:
		item = chordList[i]
		if item[0] == '?': continue
		item = item.split()
		k = item[0].rstrip(':')
		if len(k) > 1:
			k = k[0] + k[1:].replace('b', '-')
		if len(item) > 1:
			rn = roman.RomanNumeral(item[1], k)
			tempStream.insert(offsets[i], rn)
	for i in range(len(tempStream) - 1):
		tempStream[i].duration.quarterLength = tempStream[i+1].offset - tempStream[i].offset
	return tempStream

def transpose_letter(letterName, transp):
	if letterName == '?': return False
	return pitch.Pitch(letterName).transpose(transp).name

def relative(firstKey, secondKey):									# firstKey is he potential relative minor
	if firstKey == '?' or secondKey == '?': return False
	if secondKey == pitch.Pitch(firstKey).transpose('m3').name: return True
	return False

def dorian_relative(firstKey, secondKey):									# firstKey is the potential dorian minor
	if firstKey == '?' or secondKey == '?': return False
	if secondKey == pitch.Pitch(firstKey).transpose('-M2').name: return True
	return False

def get_relative(myKeyName):									
	if myKeyName == '?': return False
	if myKeyName[0].islower(): return pitch.Pitch(myKeyName).transpose('m3').name
	return pitch.Pitch(myKeyName).transpose('-m3').name.lower()

def get_dorian_relative(myKeyName):									
	if myKeyName == '?': return False
	if myKeyName[0].islower(): return pitch.Pitch(myKeyName).transpose('-M2').name
	return pitch.Pitch(myKeyName).transpose('M2').name.lower()

"""SHOULD BE ABLE TO KILL THIS NOW"""

def strip_roman(myString):
	newString = copy.deepcopy(myString)
	for i in range(0, 10):
		newString = newString.replace(str(i), '')
	newString = newString.rstrip('/')
	return newString

def could_be_suspension(myOffset, voice):
	global chorale
	beatDuration = False			# fix this!!!
	if voice >= len(chorale.parts): return False
	myNote = chorale.parts[voice].flat.getElementsByOffset(myOffset, myOffset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Note'])
	if not myNote: return False
	myNote = myNote[0]
	could_be_suspension.note = myNote
	nextNote = chorale.parts[voice].flat.getElementAfterElement(myNote, classList = ['Note'])
	if not nextNote: return False
	#print myNote, nextNote
	if myNote.offset != myOffset:					# note starts before beat
		effectiveDuration = nextNote.offset - myOffset
		prevDuration = myNote.duration.quarterLength - effectiveDuration
		#print prevDuration, effectiveDuration
		if prevDuration < effectiveDuration: return False
	else: 
		prevNote = chorale.parts[voice].flat.getElementBeforeOffset(myOffset, classList = ['Note'])
		if not prevNote: return False
		if prevNote.pitch.midi != myNote.pitch.midi: return False
		#check harmonicity here
	if myNote.pitch.midi - nextNote.pitch.midi not in [1, 2]: return False
	could_be_suspension.resolution = nextNote
	return True
	
def could_be_passing_or_neighboring(myOffset, voice):
	#print myOffset, voice
	if voice >= len(chorale.parts): return False
	myNote = chorale.parts[voice].flat.getElementsByOffset(myOffset, myOffset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Note'])
	if not myNote: return False
	myNote = myNote[0]
	if myNote.offset != myOffset: 
		return False
	could_be_passing_or_neighboring.note = myNote
	if myNote.duration.quarterLength >= 1.0: return False							# someday check for longer PT and NTs
	nextNote = chorale.parts[voice].flat.getElementAfterElement(myNote, classList = ['Note'])
	if not nextNote: return False
	prevNote = chorale.parts[voice].flat.getElementBeforeOffset(myOffset, classList = ['Note'])
	if not prevNote: return False
	if abs(prevNote.pitch.midi - myNote.pitch.midi) > 2: return False
	if abs(myNote.pitch.midi - nextNote.pitch.midi) > 2: return False
	could_be_passing_or_neighboring.resolution = nextNote
	return True

def shared_scales(scaleList1, scaleList2):
	outputList = []
	for i in range(len(scaleList1)):
		outputList.append(scaleList1[i] & scaleList2[i])
	return outputList

def ambiguous(scaleList):
	if len(scaleList[0]) > 1: return True												# multiple diatonics
	elif len(scaleList[1] | scaleList[2]) > 1: return True								# multiple tonics for harm min and mel min
	elif len(list(scaleList[0]) + list(scaleList[1] | scaleList[2])) > 1: return True	# ambiguous between maj and min
	else: return False

def is_empty(scaleList):
	totalScales = set([])
	for i in range(len(scaleList)):
		totalScales = totalScales | scaleList[i]
	if totalScales:
		return False
	else:
		return True

"""Note: do not casually mess with the following routine -- it's needed for key-finding, etc."""
		
def incomplete_seventh(myChord):					# currently only records incomplete DOMINANT sevenths (by design)
	nf = DT.normal_form(set(myChord.pitchClasses))
	if nf == [0, 2, 6] or nf == [0, 3, 6]:
		return True
	else:
		return False
		
"""routines to deal with sacred harp corpus"""
def RSH_process_repeats(myStream):
	todo = {}
	myPart = myStream.parts[0]
	myMeasures = myPart.getElementsByClass("Measure")
	for m in myMeasures:
		t = m.getContextByClass('TimeSignature')
		if m.highestTime != t.beatDuration.quarterLength * t.beatCount:
			todo[m.number] = [m.numberSuffix, m.highestTime, t.beatDuration.quarterLength * t.beatCount]
	for i in reversed(sorted(todo.keys())):
		if i - 1 not in todo:
			RSH_remove_number(i+1, myStream)
			print ("REMOVING", i+1)
		
def RSH_remove_number(targetMeasure, myStream):
	for myPart in myStream.parts:
		m = myPart.measure(targetMeasure-1)
		m.numberSuffix = ''
		m = myPart.measure(targetMeasure)
		m.number = targetMeasure - 1
		m.numberSuffix = 'X'
		for m in myPart.getElementsByClass("Measure"):
			if m.number > targetMeasure:
				m.number = m.number - 1