from __future__ import print_function
import matplotlib.pyplot as plt
from music21 import *
import DT



"""

A very simple program to calculate the density of attacks and plot them; used to compare the Waldstein Sonata Exposition and the first Mozart sonata

It also plots registral positio of the hands, used for mapping the Tempest.


"""

def graph_rhythm(fileList = [DT._PATHS['Beethoven'][1]+'op053-1.krn', DT._PATHS['Mozart'][1]+'K279-1.xml'], windowLength = 4, graph = True, useOffsets = False, smoothing = 8, offsetMax = [387, 0]):
	global attacks, chorale, offsets
	if type(fileList) is str:
		fileList = [fileList]
	fig = plt.figure(figsize = (15, 6))
	ax = fig.gca()
	plt.title('Attack density in the Waldstein first-movement exposition and K279-1', fontsize = 24)
	for fileNumber, fileName in enumerate(fileList):
		chorale = converter.parse(fileName)
		flatParts = [p.flat for p in chorale.parts]
		attacks = []
		offsets = []
		if offsetMax[fileNumber]:
			ht = offsetMax[fileNumber]
		else:
			ht = chorale.flat.highestTime - windowLength
		finalOffset = int(ht)
		for i in range(finalOffset):
			tempAttacks = 0
			tempOffsets = []
			for p in flatParts:
				theNotes = p.getElementsByOffset(i, i + windowLength - .01, mustBeginInSpan = True, classList = ['Note', 'Chord'])
				if not theNotes: continue
				noteCount = 0
				for n in theNotes:
					if 'Chord' in n.classes:
						noteCount += len(n.pitches)
					else:
						noteCount += 1
				tempAttacks += noteCount
				offsetList = [x.offset for x in theNotes]
				tempOffsets += offsetList
			attacks.append(tempAttacks/(1.0 * windowLength))
			tempOffsets = list(set(list(tempOffsets)))
			offsets.append(len(tempOffsets)/(1.0 * windowLength))
	
		if smoothing:
			attacks = smooth(attacks, smoothing)
			offsets = smooth(offsets, smoothing)
		
	
		if useOffsets:
			myList = offsets
		else:
			myList = attacks
		
		ax.plot(range(len(myList)), myList, label = fileName)
	myLeg = plt.legend(tuple([fix_names(s) for s in fileList]), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()
	
def smooth(l, s = 4):
	n = []
	for i in range(s, len(l)):
		n.append(sum(l[i-s:i])/float(s))
	return n
			
def fix_names(s):
	s = s.split('/')[-1]
	s = s.split('.')[0]
	s = s[0].upper() + s[1:]
	return s

def register(fileList = [DT._PATHS['Beethoven'][1]+'op031no2-1.krn'], windowLength = 4, graph = True, colors = ['black', 'r', 'b', 'g', 'y'], smoothing = 8, yLim = [30, 90], xLim = False, offsetMax = [359, 0], labels = ['RH', 'LH']):
	global attacks, chorale, offsets
	if type(fileList) is str:
		fileList = [fileList]
	fig = plt.figure(figsize = (15, 6))
	ax = fig.gca()
	plt.title('Register in the Exposition of the Tempest, I', fontsize = 24)
	if labels: 
		theLabels = labels
	else:
		theLabels = []
	for fileNumber, fileName in enumerate(fileList):
		chorale = converter.parse(fileName)
		flatParts = [p.flat for p in chorale.parts]
		theKeys = []
		theLines = []
		if offsetMax[fileNumber]:
			ht = offsetMax[fileNumber]
		else:
			ht = chorale.flat.highestTime - windowLength
		finalOffset = int(ht)
		for partNo, p in enumerate(flatParts):
			thePitches = {}
			offsets = []
			for i in range(finalOffset):
				tempAttacks = 0
				tempOffsets = []
				theNotes = p.getElementsByOffset(i, i + windowLength - .01, mustBeginInSpan = True, classList = ['Note', 'Chord'])
				tempPitches = []
				if not theNotes: 
					continue
				for n in theNotes:
					if 'Chord' in n.classes:
						tempPitches += [x.midi for x in n.pitches]
					else:
						tempPitches.append(n.pitch.midi)
				thePitches[i/4.] = sum(tempPitches)/float(len(tempPitches))
			theKeys = sorted(thePitches.keys())
			theValues = [thePitches[x] for x in theKeys]
			print("min and max values", min(theValues), max(theValues))
			if not labels:
				theLabel = str(fileNumber) + str(partNo)
				theLabels.append(theLabel)
			else:
				theLabel = labels[fileNumber + partNo]
			ax.plot(theKeys, theValues, label = theLabel, color = colors[fileNumber])
	if yLim:
		ax.set_ylim(*yLim)
	if xLim:
		ax.set_xlim(*xLim)
	myLeg = plt.legend(tuple(theLabels), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()
	