from __future__ import print_function
import os
import sys
import time
import pickle
import copy
import re

import matplotlib.pyplot as plt
from music21 import *

import DT
import chord_distributions

palestrinaData = DT.palestrina_data()

_THISFILE = 'CADENCES'

useReduced = True
capture = True
directFifthsOctavesLocations = []
overlapLocations = []
crossingsLocations = []
parallelsLocations = []
overlapVoices = {}
overlapCount = 0
crossingsCount = 0
parallelsCount = 0
antiParallelsCount = 0
directFifthsOctavesCount = 0
offsetHisto = {}
lastChorale = 1
chorale = stream.Score()
output = stream.Score()
locations = {}
counts = {}

fifths = ['A-', 'E-', 'B-', 'F', 'C', 'G', 'D', 'A', 'E', 'B', 'F#', 'C#', 'G#', 'D#']

"""
A program to find Renaissance cadential patterns in musical scores, defined operationally as a suspension that converges on a unison or octave.

Linked with progressions and chord_distributions as a trio of analysis programs designed for uninterpreted Renaissance scores.

TODO: 

	make an all-purpose suspension-detection algorithm that anyone can use; I've written this like 5 times

	add OFFSETS here to identify cadence offsets?

ERRORS: 

	>>> cadences.show("[2, 11, 2, 5, 9] [4, 0, 0, 9]")
	[[504, 62]]
	getting Credo_92 ['Octavi toni', 'Masses, Book 11'] [58, 62]
	<music21.stream.Score 0x10fbb8810>

"""

def gather_stats(source = 'PALRAW', testFile = '', minorSixths = True):							# write something new for raw searching of XML, etc.
	global chorale, locations, globalSource, locNames, doppiaLocations, doppias, sortedLocations, preparations, globalSixths, keyCadenceDict, accidentalCensus
	source = source.upper()
	if source not in DT._PATHS:
		print("SOURCE NOT FOUND!")
		return
	analysisPath = DT._PATHS[source][0]
	globalSixths = minorSixths
	scorePath = DT._PATHS[source][1]
	files = sorted(os.listdir(scorePath))
	recordResults = False
	startT = time.time()
	globalSource = source
	output = stream.Score()
	locations = {}
	preparations = {}
	sortedLocations = {}
	doppiaLocations = {}
	doppias = {}
	locNames = {}
	keyCadenceDict = {}
	accidentalCensus = {}
	counter = 1
	for myFile in files:
		if myFile[-3:] in ['xml', 'krn'] and myFile[:-4] not in locNames:
			if testFile and myFile != testFile: continue
			tempName = scorePath + myFile		#.replace('.txt', '.xml')
			print('File', myFile, end=' ')
			#try:
			chorale = converter.parse(tempName)
			if 'Opus' in chorale.classes:
				bigOpus = chorale
				for opusSection, chorale in enumerate(bigOpus):
					if 'Score' not in chorale.classes: continue
					print("  multi-score file, section:", opusSection + 1, end=' ')
					try:
						dochorale(counter)
						locNames[counter] = str(opusSection) + '***}' + myFile
						counter += 1
					except:
						print("ERROR: ", tempName, sys.exc_info()[0])
			else:
				dochorale(counter)
				locNames[counter] = myFile
				counter += 1
			if counter % 50 == 0:
				save()
	save()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))

def pr(myStr):
	tempList = sorted([[x, reducedPreparationsCounts[myStr][x]] for x in list(reducedPreparationsCounts[myStr])], key = lambda x: -1 * x[1])
	for item in tempList:
		print(item[0], item[1])

def save():
	make_counts()
	outputFile = open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'wb')
	pickle.dump(locations,outputFile)
	pickle.dump(locNames, outputFile)
	pickle.dump(doppiaLocations, outputFile)
	pickle.dump(doppias, outputFile)
	pickle.dump(sortedLocations, outputFile)
	pickle.dump(preparations, outputFile)
	pickle.dump(keyCadenceDict, outputFile)
	pickle.dump(accidentalCensus, outputFile)
	outputFile.close()

def consolidate():
	global locations, reduced, reducedLocations, preparations, reducedPreparations, reducedPreparationsCounts
	reduced = {}
	reducedLocations = {}
	reducedPreparations = {}
	reducedPreparationsCounts = {}
	for item in locations:
		if item.count("*") > 0: continue
		fromString = [to_list(x) for x in item.split('] [')]
		items = [remove_duplicates(x) for x in fromString]
		#print item, items
		newStr = str(items[0]) + ' ' + str(items[1])
		reduced[newStr] = reduced.setdefault(newStr, 0) + len(locations[item])
		if newStr not in reducedLocations:
			reducedLocations[newStr] = []
			reducedPreparations[newStr] = {}
		for loc in locations[item]:
			reducedLocations[newStr].append(loc)
		if item not in preparations: continue
		for prep in preparations[item]:
			if prep == '?': 
				newPrep = '?'
			else:
				newPrep = str(remove_duplicates(to_list(prep)))
			if newPrep in reducedPreparations[newStr]:
				for loc in preparations[item][prep]:
					reducedPreparations[newStr][newPrep].append(loc)
			else:
				reducedPreparations[newStr][newPrep] = preparations[item][prep]
	for i in reducedPreparations:
		reducedPreparationsCounts[i] = {}
		for j in reducedPreparations[i]:
			reducedPreparationsCounts[i][j] = len(reducedPreparations[i][j])

def categorize():
	global locations, results, unknown, reducedLocations, preparations, reducedPreparations, reducedPreparationsCounts
	results = {}
	unknown = {}
	for item in ['AUTHENTIC', 'AUTHENTIC MINOR', 'DECEPTIVE', 'DECEPTIVE MINOR', 'viio6 -> I', 'viio6 -> i', 'PLAGAL', 'PLAGAL MINOR', "PHRYGIAN"]:
		results[item] = []
	for item in reduced:
		fromString = [to_list(x) for x in item.split('] [')]
		items = [remove_duplicates(x) for x in fromString]
		newStr = str(items[0]) + ' ' + str(items[1])
		itemSets = [set(x) for x in items]
		if itemSets[0] <= set([7, 11, 2]) and itemSets[1] <= set([0, 4, 7]):
			results['AUTHENTIC'].append([newStr, reduced[item]])
		elif itemSets[0] <= set([7, 11, 2]) and itemSets[1] <= set([0, 3, 7]):
			results['AUTHENTIC MINOR'].append([newStr, reduced[item]])
		elif itemSets[0] <= set([7, 11, 2]) and itemSets[1] <= set([9, 0, 4]):
			results['DECEPTIVE'].append([newStr, reduced[item]])
		elif itemSets[0] <= set([7, 11, 2]) and itemSets[1] <= set([8, 0, 3]):
			results['DECEPTIVE MINOR'].append([newStr, reduced[item]])
		elif itemSets[0] <= set([2, 5, 11]) and itemSets[1] <= set([0, 4, 7]):
			results['viio6 -> I'].append([newStr, reduced[item]])
		elif itemSets[0] <= set([2, 5, 11]) and itemSets[1] <= set([0, 3, 7]):
			results['viio6 -> i'].append([newStr, reduced[item]])
		elif itemSets[0] <= set([10, 1, 5]) and (itemSets[1] <= set([0, 3, 7]) or itemSets[1] <= set([0, 4, 7])):
			results['PHRYGIAN'].append([newStr, reduced[item]])
		elif itemSets[0] <= set([10, 1, 5]) and (itemSets[1] <= set([5, 8, 0]) or itemSets[1] <= set([5, 9, 0])):
			results['PLAGAL MINOR'].append([newStr, reduced[item]])
		elif itemSets[0] <= set([10, 2, 5]) and itemSets[1] <= set([5, 9, 0]):
			results['PLAGAL'].append([newStr, reduced[item]])
		elif items[0][0] == 7 and itemSets[0] <= set([7, 11, 2, 5]) and itemSets[1] <= set([0, 4, 7]):
			results['AUTHENTIC'].append([newStr, reduced[item]])
		elif items[0][0] == 7 and itemSets[0] <= set([7, 11, 2, 5]) and itemSets[1] <= set([0, 3, 7]):
			results['AUTHENTIC MINOR'].append([newStr, reduced[item]])
		elif items[0][0] == 7 and itemSets[0] <= set([7, 11, 2, 5]) and itemSets[1] <= set([9, 0, 4]):
			results['DECEPTIVE'].append([newStr, reduced[item]])
		elif items[0][0] == 7 and itemSets[0] <= set([7, 11, 2, 5]) and itemSets[1] <= set([8, 0, 3]):
			results['DECEPTIVE MINOR'].append([newStr, reduced[item]])
		else:
			unknown[newStr] = reduced[item]
	for item in sorted(unknown.keys(), key = lambda x: -unknown[x]):
		print("UNKNOWN", item, unknown[item])
	tempList = []
	for i in results:
		tempList.append([i, sum([x[1] for x in results[i]])])
	for i in sorted(tempList, key = lambda x: x[1]):
		print(i)

def to_list(s):
	s = s.replace(']', '').replace('[', '').replace(' ', '')
	l = s.split(',')
	return [int(x) for x in l]

def remove_duplicates(l):
	outList = []
	for item in l:
		if item not in outList:
			outList.append(item)
	return outList

def remove_consecutive_duplicates(l):
	outList = []
	for item in l:
		if not outList or item != outList[-1]:
			outList.append(item)
	return outList

def make_counts():
	global locations, counts, sortedLocations, sortedCounts, preparationsCounts, preparations
	preparationsCounts = {}
	counts = {}
	sortedCounts = {}
	preparationCounts = {}
	for item in locations:
		counts[item] = len(locations[item])
	for item in sortedLocations:
		sortedCounts[item] = len(sortedLocations[item])
	for i in preparations:
		if i not in preparationsCounts:
			preparationsCounts[i] = {}
		for j in preparations[i]:
			if j not in preparationsCounts[i]:
				preparationsCounts[j] = {}
			preparationsCounts[i][j] = len(preparations[i][j])
		
def load(source='PALRAW'):
	global locations, locNames, globalSource, counts, doppiaLocations, doppias, sortedLocations, sortedCounts, preparations, keyCadenceDict, accidentalCensus
	try:
		counts = {}
		sortedCounts = {}
		globalSource = source.upper()
		with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'rb') as myFile:
			locations = pickle.load(myFile)
			locNames = pickle.load(myFile)
			doppiaLocations = pickle.load(myFile)
			doppias = pickle.load(myFile)
			sortedLocations = pickle.load(myFile)
			preparations = pickle.load(myFile)
			keyCadenceDict = pickle.load(myFile)
			accidentalCensus = pickle.load(myFile)
		make_counts()
		consolidate()
		return True
	except:
		return False

def filtermode(mode = 'Ionian'):
	global counts
	counts = {}
	mode = mode.upper()
	for m in keyCadenceDict:
		if m.upper().startswith(mode):
			mode = m
			break
	for thePiece in keyCadenceDict[mode]:
		for theCadence in thePiece[1]:
			address = str(theCadence[0]) + ' ' + str(theCadence[1])
			counts[address] = counts.setdefault(address, 0) + 1
	consolidate()

def dochorale(n):
	global chorale, totalNotes, offsets, locations, doppiaLocations, doppias, sortedLocations, preparations, globalSixths, keyCadenceDict, accidentalCensus
	
	foundMode = True
	chorale_notes = []
	transp = 0
	offsetList = []
	totalNotes = []
	accidentalCensus[n] = {}
	
	measures = chorale.parts[0].getElementsByClass('Measure')
	flatParts = []
	for i in range(0, len(chorale.parts)):
		
		flatParts.append(chorale.parts[i].flat.getElementsByClass('Note').stream())
		for myNote in flatParts[i]:
			offsetList.append(myNote.offset)
			accidentalCensus[n].setdefault(myNote.name, []).append(myNote.duration.quarterLength)
	
	offsets = sorted(list(set(offsetList)))
	
	for myOffset in offsets:
		totalNotes.append([])
		for j in range(0, len(chorale.parts)):
			testNote = flatParts[j].getElementsByOffset(myOffset + .01, myOffset + .02, includeEndBoundary = False, includeElementsThatEndAtStart=False, mustBeginInSpan = False)
			if testNote:
				totalNotes[-1].append(testNote[0])
			else:
				totalNotes[-1].append(note.Note('C0'))
	
	
	""" GET FINAL CHORD """
	
	lastChord = [x for x in totalNotes[-1] if x.pitch.midi != 12]
	"""for p in totalNotes[-1]:
		if p.midi != 12:
			lastChord.append(p)"""
	c = chord.Chord(lastChord)
	r = c.root()
	
	if not r:
		print("CAN'T FIND LAST ROOT", "\a")
	
	tonicPitchClass = r.pitchClass
	
	""" GET KEY SIGNATURES AND FIND MODE """ 
	
	keySigs = []
	
	for p in chorale.parts:
		k = p.flat.getElementsByClass("KeySignature")
		if not k:
			print("CAN'T FIND KEY SIGNATURE FOR AT LEAST ONE PART", "\a")
			continue
		keySigs.append(k[-1].sharps)
	
	if len(list(set(keySigs))) > 1:
		print("MULTIPLE KEY SIGNATURES", "\a")
		foundMode = False
	elif len(list(set(keySigs))) == 0:
		print("NO KEY SIGNATURES", "\a")
		foundMode = False
		
	k = key.Key(['C', 'G', 'D', 'A', 'E', 'B', 'F#', 'C#', 'G#', 'A-', 'E-', 'B-', 'F'][keySigs[0]])
	
	degreeNames = [x.name for x in k.pitches[:-1]]
	
	if r.name in degreeNames:
		modeIndex = degreeNames.index(r.name)
		modeName = ['Ionian', 'Dorian', 'Phrygian', 'Lydian', 'Mixolydian', 'Aeolian', 'Locrian'][modeIndex]
	else:
		modeName = '?'
	
	if modeName not in keyCadenceDict:
		keyCadenceDict[modeName] = []
	
	print("MODE IS", modeName, "tonic PC", tonicPitchClass)
	
	myCadenceList = []
	addendum = ''
	cadencesSoFar = []
	foundTrueCadence = False
	
	for i in range(1, len(totalNotes) - 1):				# i == offset
		
		if foundTrueCadence:							# clear duplicate cadences out
			for l in cadencesSoFar:
				for m in l:
					d, elem = m
					if elem in d:
						d.remove(elem)
		
		elif len(cadencesSoFar) > 1:
			for l in cadencesSoFar[:-1]:
				for m in l:
					d, elem = m
					if elem in d:
						d.remove(elem)
					
		cadencesSoFar = []
		foundTrueCadence = False
		for j in range(len(chorale.parts)):
			upInt = totalNotes[i+1][j].pitch.midi - totalNotes[i][j].pitch.midi
			if 0 < upInt < 3:					# ascends by step
				for k in range(len(chorale.parts)):
					if k == j or foundTrueCadence or totalNotes[i][k].pitch.midi == 12: continue
					downInt = totalNotes[i][k].pitch.midi - totalNotes[i+1][k].pitch.midi
					if ((upInt != 2 or downInt != 2) or globalSixths) and found_suspension(i, j, k) and (totalNotes[i][j].pitch.pitchClass -
						totalNotes[i][k].pitch.pitchClass) % 12 == 9: 								# 
						if (totalNotes[i+1][j].pitch.pitchClass == totalNotes[i+1][k].pitch.pitchClass) and 0 < downInt < 3:			# descends by step to the same PC
							addendum = ''
							foundTrueCadence = True
						else:
							addendum = '*'
						
						if upInt == 1: 
							transp = (11 - totalNotes[i][j].pitch.pitchClass) % 12
						else:
							transp = (10 - totalNotes[i][j].pitch.pitchClass) % 12
							
						finalChord1, sortedChord1 = fix_chords(i, j, k, transp)
						finalChord2, sortedChord2 = fix_chords(i+1, j, k, transp)
						
						if remove_duplicates(finalChord2) in [[0, 5, 7], [0, 5], [0, 5, 9], [0, 5, 8], [5, 0, 10]]:
							finalChord3 = find_cadence_chord(i, j, k, transp)
							if finalChord3:
								finalChord3 = finalChord3[0]
								finalChord3 = [finalChord3[0]] + sorted(finalChord3[1:])
							else:
								finalChord3 = []
						else:
							finalChord3 = []
						
						preChord = find_preparation(i, j, k, transp)
						if preChord:
							preChordStr = str(preChord[0])
						else:
							preChordStr = '?'
						
						address = str(finalChord1) + ' ' + addendum + str(finalChord2)
						finalChordTuple1 = [(x - tonicPitchClass - transp) % 12 for x in finalChord1]
						finalChordTuple2 = [(x - tonicPitchClass - transp) % 12 for x in finalChord2]
						tempList = [finalChordTuple1, finalChordTuple2]
						
						if finalChord3:
							address += ' ' + str(finalChord3)
							tempList.append([(x - tonicPitchClass - transp) % 12 for x in finalChord3])
						
						myCadenceList.append(tempList)
						
						sortedAddress = str(sortedChord1) + ' ' + str(sortedChord2)
						m = DT.get_measure_number_and_beat_from_offset(chorale, offsets[i+1])  #chorale.parts[j].beatAndMeasureFromOffset(offsets[i+1])
						mList = [n, m[0], (-transp) % 12, offsets[i+1]]
						
						#print i, j, k, offsets[i+1], mList, address
						
						if address in locations:
							locations[address].append(mList)
							preparations[address].setdefault(preChordStr, []).append(mList)
						else:
							locations[address] = [mList]
							preparations[address] = {}
							preparations[address][preChordStr] = [mList]
						
						sortedLocations.setdefault(sortedAddress, []).append(mList)
						
						if addendum: 
							cadencesSoFar.append([])
							cadencesSoFar[-1].append([locations[address], mList])
							cadencesSoFar[-1].append([preparations[address][preChordStr], mList])
							cadencesSoFar[-1].append([sortedLocations[sortedAddress], mList])
						
						if is_doppia(i, j):
							otherVoice = []
							for l in range(doppiaStart, i + 2):
								otherVoice.append((totalNotes[l][k].pitch.midi + transp) % 12)
							otherVoice = str(remove_consecutive_duplicates(otherVoice))
							doppias[otherVoice] = doppias.setdefault(otherVoice, 0) + 1
							"""need to add code to remove doppia overcounting, in if not addendum below"""
							doppiaLocations.setdefault(otherVoice, []).append(mList)
							if addendum:
								cadencesSoFar[-1].append([doppiaLocations[otherVoice], mList])					
										
	keyCadenceDict[modeName].append([n, myCadenceList])

def display_chord(myChord):
	return [x.nameWithOctave for x in myChord if x.pitch.midi != 12]

def remove_suspensions():
	global locations
	newCounts = {}
	for c in locations:
		value = locations[c]
		theChords = c.split('] ')
		if len(theChords) == 3:
			c = theChords[0] + '] ' + theChords[2]
		if c not in newCounts:
			newCounts[c] = []
		newCounts[c] += value
	locations = newCounts
	
def fix_chords(i, j, k, transp):
	global totalNotes
	tempChord = [x.pitch.midi for x in totalNotes[i]]
	chordTones = [x for x in tempChord if x != 12]
	bassNote = min(chordTones)
	orderedChord = []
	sortedChord = [(x + transp) % 12 for x in sorted(chordTones)]
	if bassNote != tempChord[j]:
		orderedChord.append(bassNote % 12)
	orderedChord.append(tempChord[j] % 12)
	if (bassNote != tempChord[k]) and tempChord[k] != 12:
		orderedChord.append(tempChord[k] % 12)
	holdingPen = []
	for x in chordTones:
		if (x % 12) not in orderedChord and (x % 12) not in holdingPen:
			holdingPen.append(x % 12)
	orderedChord = [(x + transp) % 12 for x in orderedChord]
	holdingPen = sorted([(x + transp) % 12 for x in holdingPen])
	return [orderedChord + holdingPen, sortedChord]

def find_preparation(i, j, k, transp):
	global totalNotes, suspensionLocation
	for l in reversed(range(max(0, suspensionLocation-2), min(len(totalNotes) - 1,  suspensionLocation + 1))):
		tempChord = [x.pitch.midi for x in totalNotes[l]]
		tempChord = [x for x in tempChord if x != 12]
		tempChord = sorted(list(set([x % 12 for x in tempChord])))
		if DT.normal_form(tempChord) in [[0, 4], [0, 3], [0, 7], [0, 4, 7], [0, 3, 6], [0, 3, 7], [0, 3, 5, 8]]: 
			return fix_chords(l, j, k, transp)
	return False

def find_cadence_chord(i, j, k, transp):
	global totalNotes, suspensionLocation
	for l in range(i+2, min(i + 5, len(totalNotes) - 1)):
		tempChord = [x.pitch.midi for x in totalNotes[l]]
		tempChord = [x for x in tempChord if x != 12]
		tempChord = sorted(list(set([x % 12 for x in tempChord])))
		if DT.normal_form(tempChord) in [[0, 4], [0, 3], [0, 7], [0, 4, 7], [0, 3, 7]]: 
			return fix_chords(l, j, k, transp)
	return False

def is_doppia(penultimateChord, ascendingVoice):				# need to add decorations here, but this is quick and dirty
	global totalNotes, offsets, chorale, doppiaStart
	leadingTone = totalNotes[penultimateChord][ascendingVoice].pitch.midi
	leadingToneDuration = 0
	preLeadingToneDuration = 0
	preLeadingTone = 0
	cadenceOffset = offsets[penultimateChord+1]
	for i in reversed(range(penultimateChord)):
		if totalNotes[i][ascendingVoice].pitch.midi != leadingTone:
			leadingToneDuration = cadenceOffset - offsets[i+1]
			preLeadingTone = i + 1
			if totalNotes[i][ascendingVoice].pitch.midi - leadingTone == 1:							# add decorations here
				break
			else:
				return False
	if preLeadingTone == 0: return False
	for i in reversed(range(preLeadingTone)):
		if totalNotes[i][ascendingVoice].pitch.midi != leadingTone + 1:
			preLeadingToneDuration = offsets[preLeadingTone] - offsets[i+1]
			if totalNotes[i][ascendingVoice].pitch.midi == leadingTone and preLeadingToneDuration == 2 * leadingToneDuration:
				doppiaStart = i
				return True
			else:
				return False
	return False

"""LOOK AT THE MORLEY APRIL.  THIS ALGORITHM DOESN'T WORK THERE.  NEED TO FIND PREPARATION."""

def prev_note(chordNo, voiceNo):
	global totalNotes
	curNote = totalNotes[chordNo][voiceNo].pitch.midi
	for i in reversed(range(chordNo)):
		if totalNotes[i][voiceNo].pitch.midi != curNote: 
			break
	return totalNotes[i][voiceNo].pitch.midi
		
def found_suspension(penultimateChord, ascendingVoice, descendingVoice):				# need to add decorations here, but this is quick and dirty
	global totalNotes, offsets, chorale, suspensionLocation
	foundSus = False
	foundDecoration = 0
	leadingTone = totalNotes[penultimateChord][ascendingVoice].pitch.midi
	preLeadingTone = 0
	foundPrep = False
	descendingTone = totalNotes[penultimateChord][descendingVoice].pitch.midi
	ltArrival = penultimateChord
	suspensionEnd = 0
	
	for i in reversed(range(penultimateChord)):											# first go back to find the immediate tone prior to the suspension
		curMidi = totalNotes[i][ascendingVoice].pitch.midi
		otherVoice = totalNotes[i][descendingVoice].pitch.midi
		if otherVoice != descendingTone:												# try without this?  
			return False
		if curMidi != leadingTone:
			ltArrival = i + 1
			break
	""" Now check for decorations """
	
	suspensionEnd = ltArrival
	
	if 0 < leadingTone - curMidi < 3 and totalNotes[i][descendingVoice].pitch.midi == descendingTone:
		if i < 2 or totalNotes[i-1][descendingVoice].pitch.midi != descendingTone:
			return False
		"""Need duration check"""
		if 0 < (totalNotes[i-1][ascendingVoice].pitch.midi - leadingTone) < 3:
			foundDecoration = 1
			suspensionEnd = i
		"""Replace TRUE with a duration check"""
		if totalNotes[i-1][ascendingVoice].pitch.midi == leadingTone and True:
			foundDecoration = 2
			suspensionEnd = i-1
	
	for i in reversed(range(suspensionEnd)):
		curMidi = totalNotes[i][ascendingVoice].pitch.midi
		if 0 < curMidi - leadingTone < 3 and totalNotes[i][descendingVoice].pitch.midi == descendingTone:
			preLeadingTone = curMidi
			if ((offsets[ltArrival] - offsets[i]) >= 2 * (offsets[penultimateChord + 1] - offsets[ltArrival])):
				#print "  possibly no preparation in the following"
				foundSus = True
				foundPrep = True
			#print offsets[i], offsets[ltArrival]
			if beat_strength(chorale, offsets[i], offsets[ltArrival]):	 			#DT.beat_strength(chorale.parts[ascendingVoice], offsets[i]) > DT.beat_strength(chorale.parts[descendingVoice], offsets[ltArrival]):
				foundSus = True
		if totalNotes[i][ascendingVoice].pitch.midi != preLeadingTone:
			break
		if totalNotes[i][descendingVoice].pitch.midi != descendingTone:
			suspensionLocation = i
			if (preLeadingTone - totalNotes[i][descendingVoice].pitch.midi) % 12 in [0, 3, 4, 7, 8, 9]:
				foundPrep = True
			else:
				pass 
				#print "unprepared suspension?", (preLeadingTone - totalNotes[i][descendingVoice].pitch.midi) % 12 
			break
	if foundSus and foundPrep: 
		return True
	return False

def beat_strength(myStream, firstOffset, secondOffset):
	s1 = DT.beat_strength(myStream, firstOffset)
	s2 = DT.beat_strength(myStream, secondOffset)
	if s1 > s2 or (s2 == 1.0 and s1 == .5):
		return True
	return False
	
def is_triad(noteList):
	pcs = [x.pitch.midi for x in noteList]
	pcs = list(set([x % 12 for x in pcs if x != 12]))
	if DT.normal_form(pcs) in [[0, 4, 7], [0, 3, 6], [0, 3, 7]]: return True
	return False

def show(rnProg, prepStr = '', myDict = locations, mode = 'MAJ', rangeList = [], useReduced = False, useSorted = False, useRNs = False, RNs = False):
	global output, locNames, globalSource, rnLocations
	output = stream.Score()
	tempOutput = stream.Score()
	lastChoraleNo = False
	if rangeList:
		rangeList = range(rangeList[0], rangeList[1])
	if type(rnProg) is str:
		rnProg = [rnProg]
	myDict = locations			# remove this
	locList = []
	for r in rnProg:
		locList += myDict[r]
	print(locList)
	for i in range(0, 4):
		tempOutput.append(stream.Part())
	for counter, instance in enumerate(locList):
		if rangeList and counter not in rangeList:
			continue
		#print instance
		measureList = instance[1:-1]
		if len(measureList) == 1:
			measureList = [max(1, measureList[0] - 4), measureList[0]]
		else:
			measureList[-1] = measureList[-1] + 1
		fileName = locNames[instance[0]]
		sectionStr = ''
		if fileName.count('***}') > 0:
			sectionStr, fileName = fileName.split('***}')
			sectionNum = int(sectionStr)
			sectionStr = '-SEC' + sectionStr
		if globalSource == 'PALRAW':
			print('getting', fileName, palestrinaData[fileName][:2], measureList)
			header = str(fileName)+ sectionStr + ', m'+str(measureList[0]) +' (' + '-'.join(palestrinaData[fileName][:2]) + ')'
		else:
			print('getting', locNames[instance[0]], measureList)
			header = str(locNames[instance[0]])+'m'+str(measureList[0])
		if locNames[instance[0]] != lastChoraleNo:
			choraleNo = fileName
			lastChoraleNo = locNames[instance[0]]
			#if choraleNo.count('***') > 0: 
			chorale = converter.parse(DT._PATHS[globalSource][1] + choraleNo)
			if sectionStr:
				chorale = chorale[sectionNum]
			if RNs: 
				analyzed_file = converter.parse(DT._PATHS[globalSource][0] + choraleNo + '.txt', format='romantext')
			if globalSource == 'MOZART':
				DT.remove_repeats(chorale)
				DT.remove_repeats(analyzed_file)
		if RNs:
			DT.add_roman(chorale, analyzed_file, measureList[0], measureList[1])
		tempChorale = copy.deepcopy(DT.get_measures(chorale, measureList[0], measureList[1]))
		DT.insert_text(tempChorale.parts[0][0], header, 55, 0)
		output = DT.append(tempChorale, output)
	output = DT.fix_roman_placement(output)
	for p in output.parts:
		for item in p.flat:
			if 'Note' in item.classes or 'Chord' in item.classes: item.lyrics = []
	output.show()
	return output

def limit_pieces(lastChord = (0,), theSig = (0,), bestChord = False, **kwargs):
	global pieceList, pieceNumbers
	if type(lastChord) is int:
		lastChord = tuple([lastChord])
	if type(theSig) is int:
		theSig = tuple([theSig])
	pieceNumbers = []
	chord_distributions.load(globalSource)
	chord_distributions.census(printOut = False)
	for k, l in chord_distributions.keyCensus.items():
		if (not lastChord or k[0] == lastChord) and k[1] == theSig and ((not bestChord) or (k[2] == bestChord)):
			pieceNumbers += l

def tonal_types(tonalTypes = [((2,), (-1,), 'G2')],**kwargs):
	global pieceList, pieceNumbers
	
	pieceTitles = []
	chord_distributions.load(globalSource)
	chord_distributions.tonal_types(printOut = False, **kwargs)
	
	for t in tonalTypes:
		l = chord_distributions.tonalTypes.get(t, 0)
		tempList = [chord_distributions.locNames[x] for x in l]
		pieceTitles += tempList
	pieceNumbers = [x for x in locNames if locNames[x] in pieceTitles]

keyCounts = {}

def average_cadences(limitFunction = limit_pieces, printOut = True, sortFunc = lambda x: -keyCounts.count(x), **kwargs):
	global keyCounts
	if 'comp' in kwargs:
		load(kwargs['comp'])
	global pieceList, pieceNumbers
	limitFunction(**kwargs)
	if printOut:
		print(len(pieceNumbers), "pieces")
	totalCadences = []
	for p in pieceNumbers:
		totalCadences += get_cadences(p)
	keyCounts = [x[-2] for x in totalCadences]
	sortedCounts = sorted(set(keyCounts), key = sortFunc)
	if printOut:
		print("CADENCE DESTINATIONS", end=' ')
		for i in sortedCounts: print(str(i)+':',keyCounts.count(i), end=' ')
		print()
		print("PERCENTAGES", end=' ')
		for i in sortedCounts: 
			print(str(i) + ': %.1f, ' % (keyCounts.count(i)*100./len(keyCounts)), end=' ')
	else:
		return [[i, keyCounts.count(i)*100./len(keyCounts)] for i in sortedCounts]

def cadence_matrix(tonicList = [0, 2, 4, 7, 9], theSig = 0):
	def f(i):
		l = [5, 0, 7, 2, 9, 4, 11]
		if i in l: return l.index(i)
		return 99999
	for tonic in tonicList:
		cadencePct = average_cadences(printOut = False, lastChord = tonic, theSig = theSig, sortFunc = f)
		print("TONIC", tonic, [[x[0], round(x[1], 1)] for x in cadencePct])


def average_graph(comps = ['JOSQUINCOMPLETE', 'PALRAW'], **kwargs):
	values = []
	for c in comps:
		load(c)
		values.append({x[0]: x[1] for x in average_cadences(printOut = False, **kwargs)})
		print(c, len(pieceNumbers))
	totalKeys = []
	for d in values:
		totalKeys += d.keys()
	totalKeys = list(set(totalKeys))
	totalValues = {}
	for d in totalKeys:
		for i, c in enumerate(comps):
			totalValues.setdefault(d, []).append(values[i].get(d, 0))
	totalKeys = sorted(totalKeys, key = lambda x: -sum(totalValues[x]))
	finalValues = zip(*[totalValues[x] for x in totalKeys])
	plt.xlabel('Cadential Destinations')
	plt.ylabel('Percentage of all cadences')
	plt.title('Cadential destinations')
	for i, v in enumerate(finalValues):
		plt.plot(v, label = comps[i])
	ticks = range(len(totalKeys))
	myLeg = plt.legend(tuple(comps), loc=0)	
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.xticks(ticks, (str(i) for i in totalKeys))
	plt.show()

def compare_modes(*args, **kwargs):				# 
	"""pass a series of dictionaries containing keys for calls to "average" 
	for instance: 
		cadences.compare_modes({'lastChord': 5, 'theSig': -1}, {'lastChord': 0, 'theSig': 0}) # compares C ionian to F ionian
 cadences.compare_modes({'lastChord': 0, 'theSig': 0, 'bestChord': (0, 4, 7)}, {'lastChord': 7, 'theSig': 0, 'bestChord': (7, 11, 2)}, labels = ('ionian', 'mixolydian'))
 cadences.compare_modes({'lastChord': 2, 'theSig': 0, 'bestChord': (2, 5, 9)}, {'lastChord': 9, 'theSig': 0, 'bestChord': (9, 0, 4)}, labels = ('dorian', 'aeolian'))
	"""
	values = []
	theLabels = kwargs.get('labels', '')
	normalizeToTonic = kwargs.get('normalizeToTonic', False)
	transposeLabels = kwargs.get('transposeLabels', True)
	delete6 = kwargs.get('delete6', True)
	
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	
	for parameter in args:
		transpInterval = 0
		tempList = average_cadences(printOut = False, **parameter)
		theSig = parameter.get('theSig', 0)
		lastChord = parameter.get('lastChord', 0)
		
		if normalizeToTonic:
			if type(lastChord) in [list, tuple]:
				lastChord = lastChord[0]
			if lastChord:
				transpInterval = -lastChord
		elif transposeLabels and theSig != 0:
			if type(theSig) in [list, tuple]:
				theSig = theSig[0]
			transpInterval = 5 * theSig
			
		tempList = [[(x[0] + transpInterval) % 12, x[1]] for x in tempList]
		values.append({x[0]: x[1] for x in tempList})
		print(parameter, len(pieceNumbers))
	totalKeys = []
	for d in values:
		totalKeys += d.keys()
	totalKeys = list(set(totalKeys))
	
	if (delete6) and (6 in totalKeys):
		totalKeys.remove(6)
	
	if "manualKeys" in kwargs:
		totalKeys = kwargs['manualKeys']
	
	totalValues = {}
	for d in totalKeys:
		for i, c in enumerate(args):
			totalValues.setdefault(d, []).append(values[i].get(d, 0))
			
	if "manualKeys" not in kwargs:
		totalKeys = sorted(totalKeys, key = lambda x: -sum(totalValues[x]))
	finalValues = zip(*[totalValues[x] for x in totalKeys])
	plt.xlabel('Cadential Destinations')
	plt.ylabel('Percentage of all cadences')
	plt.title('Cadential destinations')
	pitchNames = {0: 'C', 2: 'D', 3: 'Eb', 4: 'E', 5: 'F', 7: 'G', 9: 'A', 11: 'B', 10: 'Bb'}
	if not theLabels:
		theLabels = [str(x) for x in args]
	#plt.text(10, 15, 'Last chord ' + ' '.join([str(x) for x in lastChords[pieceNumber]]))
	#plt.text(10, 14, 'Signature ' + ' '.join([str(x) for x in theSigs[pieceNumber]]))
	#plt.text(10, 13, 'Sonorities ' + str(bigSum))
	for i, v in enumerate(finalValues):
		plt.plot(v, label = theLabels[i])
	ticks = range(len(totalKeys))
	ax.set_xlim(0, len(totalKeys) -1)
	myLeg = plt.legend(tuple(theLabels), loc=0)	
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.xticks(ticks, (pitchNames.get(x, str(x)) for x in totalKeys))
	plt.show()
		

def get_piece(locNumber):
	subPieceNumber = False
	locName = locNames[locNumber]
	if locName.count('***}') > 0:
		subPieceNumber, locName = locName.split('***}')
	scorePath = DT._PATHS[globalSource][1]
	files = os.listdir(scorePath)
	targetFile = [f for f in files if f.startswith(locName) and f[-3:] in ['xml', 'krn', 'mxl', 'mid']]
	if targetFile:
		piece = converter.parse(scorePath + targetFile[0])
		if subPieceNumber:
			piece = piece.scores[int(subPieceNumber)]
		piece.show()

def mean_cadence_distances():
	averageDistances = 0
	counts = 0
	for i in locNames:
		cadenceList = get_cadences(i)
		if cadenceList:
			#print cadenceList
			dists = [cadenceList[i][-1] - cadenceList[i-1][-1] for i in range(1, len(cadenceList))]
			averageDistances += sum(dists)
			counts += len(dists)
	return averageDistances/float(counts)

def get_cadences(pieceNumber):
	outList = []
	for c, item in locations.items():
		cadences = [[c] + x for x in item if x[0] == pieceNumber]
		outList += cadences
	return sorted(outList, key = lambda x: x[2])

"""A useful routine for looking at specific pieces"""
def print_cadences(pieceNumber):
	theName = locNames[pieceNumber]
	if globalSource == 'PALRAW':
		if theName.count("***}"):
			theName = theName.split("***}")[-1]
		print(theName, palestrinaData[theName])
	targetName = [x[0] for x in chord_distributions.locNames.items() if x[1] == theName]
	print("LAST CHORD", [chord_distributions.lastChords[x] for x in targetName])
	l = get_cadences(pieceNumber)
	keyCounts = [x[-1] for x in l]
	x = accidental_report(pieceNumber)
	print("MOST POPULAR FIFTH-RELATED NOTES (THIS PLUS THE FIFTH ABOVE)", x[0])
	print("MOST POPULAR ACCIDENTAL", x[1])
	print("CADENCE DESTINATIONS", end=' ')
	for i in sorted(set(keyCounts), key = lambda x: -keyCounts.count(x)): print(str(i)+':',keyCounts.count(i), end=' ')
	print()
	for i in sorted(l, key = lambda x: x[2]): print(i)

def print_mode(finalChordTargets = [[2, 5, 9], [2, 9], [2]]):
	finalChordTargetSet = [set(x) for x in finalChordTargets]
	for i in locNames.keys():
		theName = locNames[i]
		targetName = [x[0] for x in chord_distributions.locNames.items() if x[1] == theName]
		lastChords = [chord_distributions.lastChords[x] for x in targetName]
		for s in lastChords:
			if set(s) in finalChordTargetSet:
				print("PIECE", i)
				print_cadences(i)

def accidental_report(pieceNumber):
	distributionTonic = fifths[max(range(len(fifths) - 1), key = lambda x: sum(accidentalCensus[pieceNumber].get(fifths[x], [])))]
	maxAccidental = sorted([x for x in fifths if (x.count('-') > 0 or x.count('#') > 0)], key = lambda x: -sum(accidentalCensus[pieceNumber].get(x, [])))
	return distributionTonic, maxAccidental[:3]
	
def convert_list(fromString):
	fromString = [x + 60 for x in fromString]
	fromString[0] += -12
	fromString = [pitch.Pitch(p) for p in fromString]
	for j in range(len(fromString)):
		if fromString[j].accidental.alter == 1.0:
			fromString[j] = fromString[j].transpose('D2')
	return fromString

def preps():
	global cleanPreps
	cleanPreps = {}
	for item in reducedPreparations:
		firstChord = item.split('] [')[0] + ']'
		if firstChord not in cleanPreps:
			cleanPreps[firstChord] = {}
		for tempPrep in reducedPreparations[item]:
			cleanPreps[firstChord][tempPrep] = cleanPreps[firstChord].setdefault(tempPrep, 0) + len(reducedPreparations[item][tempPrep])

def cheap_fix_fig(myFig):
	if myFig:
		return myFig.replace('bVI+', 'bVI').replace('IVmaj6/5/4', 'Isus').replace('i6/5/4', 'Vsus')
	else:
		return '?'

def regex(patternString, printIt = True, pcts = True):
	global tempList
	source = counts
	totalCounts = sum(counts.values())
	results = []
	for prog in source:
		if re.match(patternString, prog):
			#print prog
			results.append([prog, source[prog]])
	tempList = sorted(results, key=lambda x:x[1], reverse = True)
	mySum = sum([x[1] for x in tempList])
	if printIt:
		found = sum([x[1] for x in results])
		print("TOTAL:", found, "cadences out of ", totalCounts, "%.1f" % (100. * found/totalCounts))
		for pair in tempList: 
			if pcts:
				print(pair[0], "%.1f" % (100. * pair[1]/mySum))
			else:
				print(pair[0], pair[1])
	return mySum

def authentic_bass():
	global output, outputLocs
	output = {}
	outputLocs = {}
	bassSubs = {3: 4, 8:9}
	for c, l in locations.items():
		cSplit = c.split('] ')
		firstChord = cSplit[0]
		if firstChord.count('[7') == 0 or firstChord.count('11') == 0 or firstChord.count('2') == 0:
			continue
		bassNote = cSplit[1].split(',')[0]
		bassNote = int(''.join([s for s in bassNote if s.isdigit()]))
		bassNote = bassSubs.get(bassNote, bassNote)
		output[bassNote] = output.get(bassNote, 0) + len(l)
		outputLocs.setdefault(bassNote, {}).update({c: l})
	DT.print_dict(output, pct = True)

"""two programs for TAOM figures"""
def fig636():
	global result
	result = sorted(reduced.items(), key = lambda x: -x[1])
	result = [x for x in result if x[0].startswith('[7, 11, 2]') and x[0].count('8') == 0 and x[0].count('3') == 0]
	final = {}
	for prog, count in result:
		bass = int(prog.split('] [')[1][0])
		final[bass] = final.get(bass, 0) + count
	if 7 in final: del final[7]
	fSum = sum(final.values())
	for pair in final.items():
		print(pair[0], int(round(pair[1]*100./fSum, 0)))

def fig636_v2():
	rn(removeInversions = False, printOut = False)
	cadenceTypes = ['V -> I', 'V -> vi6', 'V -> vi', 'V -> IV6', 'V -> IV', 'V -> I6']
	equivs = [['V -> I', 'V -> vi6'], ['V -> vi', 'V -> IV6']]
	result = {x[0]:x[1] for x in RNs.items() if x[0] in cadenceTypes}
	for pList in equivs:
		target = pList[0]
		for p in pList[1:]:
			result[target] += result.get(p, 0)
			del result[p]
	DT.print_dict(result, pct = True)

"""assigns RNs to cadences"""
def rn(removeInversions = True, printOut = True):
	global locations, reduced, reducedLocations, RNs, rnLocations, finalDict, rnPreparations
	problemCases = ['IVmaj6/5/4', 'bVI+', 'i6/5/4']
	RNs = {}
	rnLocations = {}
	rnPreparations = {}
	k = key.Key("C")
	k2 = key.Key("f")
	for item in reduced:
		fromString = [to_list(x) for x in item.split('] [')]
		if 1 in fromString[0] and 10 in fromString[0]:
			myKey = k2
		else:
			myKey = k
		myFigs = [DT.figure_from_chord(chord.Chord(convert_list(x)), myKey) for x in fromString]
		if False in myFigs: 
			if printOut: print("COULDN'T GET RN", item)
			continue
		#print myFigs, fromString
		myString = cheap_fix_fig(' -> '.join(myFigs))
		myString = myString.replace('iio -> III', 'viio -> I')
		
		if removeInversions:
			tempStr = myString.split(" -> ")
			for i in range(len(tempStr)):
				parsedFig = DT.parse_figure(tempStr[i])
				if parsedFig:
					tempStr[i] = parsedFig[0]
			myString = ' -> '.join(tempStr)
			
		for s in problemCases:
			if myString.count(s) > 0:
				print(myString, item)
		RNs[myString] = RNs.setdefault(myString, 0) + reduced[item]
		
		if myString in rnLocations:
			for newLoc in reducedLocations[item]:
				rnLocations[myString].append(newLoc)
		else:
			rnLocations[myString] = reducedLocations[item]

		if myString not in rnPreparations:
			rnPreparations[myString] = {}
		for newLoc in reducedPreparationsCounts[item]:
			if newLoc != "?":
				fromString = to_list(newLoc)
				prepFig = cheap_fix_fig(DT.figure_from_chord(chord.Chord(convert_list(fromString)), myKey))
			else:
				prepFig = '?'
			rnPreparations[myString][prepFig] = rnPreparations[myString].setdefault(prepFig, 0) + reducedPreparationsCounts[item][newLoc]

	if printOut:
		DT.print_dict(RNs)
	
	if removeInversions:
		finalDict = {}
		for item in RNs:
			newItem = item.replace("maj", "").replace('b', '').replace('o', '').upper()
			if newItem[0:2] != 'V ': continue
			finalDict[newItem] = finalDict.setdefault(newItem, 0) + RNs[item]
		if printOut: DT.print_dict(finalDict, pct = True)
