from __future__ import print_function
import os
import pickle
import time
import re
import copy

import matplotlib.pyplot as plt
from music21 import *
import DT

import ranges

_THISFILE = 'CHORD_DISTRIBUTIONS'

"""
Identify raw consonances in various corpora.  

Linked with progressions and chord_distributions as a trio of analysis programs designed for uninterpreted Renaissance scores.  

Largely superseded by progressions.  The main methods still in use are limit_pieces and census, which are used to try to consider only pieces in a given mode.

"""

with open(DT._DATAPATH + 'PALESTRINACORPUSDATA.pkl', 'rb') as myFile:
	palestrinaData = pickle.load(myFile)

_THISFILE = "CHORD_DISTRIBUTIONS"
#_MASTERCOMPOSERS = ['Ockeghem', 'Obrecht', 'Josquincomplete', 'Palraw']
_MASTERCOMPOSERS = ['Ockeghem', 'Josquincomplete', 'Palraw']

globalTransposeToC = False

def gather_stats(source = 'PALRAW', fileName = '', printOut = False):
	global piece, globalSource, globalPrint, progs, theSigs, lastChords, locNames, weightedProgs, rawProgs
	startT = time.time()
	globalPrint = printOut
	globalSource = source.upper()
	progs = {}
	theSigs = {}
	lastChords = {}
	locNames = {}
	weightedProgs = {}
	rawProgs = {}
	if globalSource not in DT._PATHS:
		print("Source not found!")
		return
	scorePath = DT._PATHS[globalSource][1]
	files = sorted(os.listdir(scorePath))
	counter = 1
	for myFile in files:
		if myFile[-3:] in ['xml', 'krn', 'mxl']:
			if fileName and myFile != fileName: continue
			print('Analyzing', myFile)
			piece = converter.parse(scorePath + myFile)
			if 'Opus' in piece.classes:
				bigOpus = piece
				for opusSection, piece in enumerate(bigOpus):
					if 'Score' not in piece.classes: continue
					print("  multi-score file, section:", opusSection + 1, end=' ')
					try:
						dochorale(counter)
						locNames[counter] = str(opusSection) + '***}' + myFile
						counter += 1
					except:
						print("ERROR: ", tempName, sys.exc_info()[0])
			else:
				dopiece(counter)
				locNames[counter] = myFile
				counter += 1
			if counter % 50 == 0: save()
	if not fileName:
		save()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))

def save():
	with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'wb') as outputFile:
		pickle.dump(progs, outputFile)
		pickle.dump(theSigs, outputFile)
		pickle.dump(lastChords, outputFile)
		pickle.dump(locNames, outputFile)
		pickle.dump(weightedProgs, outputFile)

def load(source = 'PALRAW'):
	global globalSource, progs, theSigs, lastChords, locNames, weightedProgs, rawProgs, locNumbers
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'rb') as inputFile:
			progs = pickle.load(inputFile)
			theSigs = pickle.load(inputFile)
			lastChords = pickle.load(inputFile)
			locNames = pickle.load(inputFile)
			weightedProgs = pickle.load(inputFile)
		locNumbers = {v:k for k, v in locNames.items()}
		return True
	except:
		return False

def get_piece(locNumber):
	locName = locNames[locNumber]
	print(locName)
	if globalSource == 'PALRAW':
		print(palestrinaData.get(locName, ""))
	scorePath = DT._PATHS[globalSource][1]
	files = os.listdir(scorePath)
	targetFile = [f for f in files if f.startswith(locName) and f[-3:] in ['xml', 'krn', 'mxl', 'mid']]
	if targetFile:
		piece = converter.parse(scorePath + targetFile[0])
		piece.show()

def dopiece(pieceNumber):	
	global progs, piece, consonantO, globalPrint, theSigs, lastChords, weightedProgs
	score = [piece.parts[i].flat.getElementsByClass(['Note', 'Chord']).stream() for i in range(len(piece.parts))]
	tempSigs = list(set([x.sharps for x in piece.recurse().getElementsByClass('KeySignature')]))
	theSigs[pieceNumber] = tempSigs
	newO = sorted(list(set([n.offset for p in score for n in p])))
	consonantO = {}
	progs[pieceNumber] = {}
	weightedProgs[pieceNumber] = {}
	orderedO = []
	for o in newO:
		myMidi = []
		myNotes = []
		for p in score:
			tempNotes = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False)
			if tempNotes:
				for n in tempNotes:
					if 'Note' in n.classes:
						myNotes.append(n)
					elif 'Chord' in n.classes:
						for n1 in n.pitches:
							myNotes.append(note.Note(n1))
		theNotes = list(set([(x.pitch.midi) % 12 for x in myNotes]))
		if DT.normal_form(theNotes) in [[0, 4, 7], [0, 3, 7], [0, 3, 6], [0, 4, 8], [0, 3], [0, 4], [0, 5], [0]]:
			fbMidi = tuple(figured_bass_midi(myNotes))
			progs[pieceNumber][fbMidi] = progs[pieceNumber].setdefault(fbMidi, 0) + 1
			consonantO[o] = fbMidi
			orderedO.append(o)
			if globalPrint: print(DT.get_measure_number_and_beat_from_offset(piece, o), [fbMidi])
	lastChords[pieceNumber] = fbMidi
	for i, o in enumerate(orderedO[:-1]):
		duration = orderedO[i+1] - o
		fbMidi = consonantO[o]
		weightedProgs[pieceNumber][fbMidi] = weightedProgs[pieceNumber].setdefault(fbMidi, 0) + duration
	o = orderedO[-1]
	fbMidi = consonantO[o]
	duration = piece.highestTime - o
	weightedProgs[pieceNumber][fbMidi] = weightedProgs[pieceNumber].setdefault(fbMidi, 0) + duration
		
def figured_bass_midi(myNotes):
	sortedNotes = sorted([x.pitch.midi for x in myNotes])
	tempList = sorted(list(set([(x - sortedNotes[0]) % 12 for x in sortedNotes])))
	return [(x + sortedNotes[0]) % 12 for x in tempList]

def graph(pieceNumber = 0, chordLength = 3, norm = True, useDurations = False, equivalenceClasses = []):
	if equivalenceClasses and type(equivalenceClasses[0]) is int:
		newList = []
		for x in equivalenceClasses:
			newList.append([tuple(y) for y in make_equivalence_classes([x])])
		equivalenceClasses = newList
	#print equivalenceClasses
	if useDurations:
		tempDict = resolve_equivalence_classes(weightedProgs[pieceNumber], equivalenceClasses)
	else:
		tempDict = resolve_equivalence_classes(progs[pieceNumber], equivalenceClasses)
	#print tempDict
	graphItems = [x for x in tempDict.items() if (type(x[0]) is not int and len(x[0]) == chordLength)]	
	graphItems = sorted(graphItems, key = lambda x: -x[1])
	values = [x[1] for x in graphItems]
	labels = [' '.join([str(x) for x in y[0]]) for y in graphItems]
	bigSum = sum(values)
	if norm:
		values = [100. * x/bigSum for x in values]
	plt.xlabel('Harmonies (length ' + str(chordLength) + ') only')
	plt.ylabel('Percentage of all sonorities of length ' + str(chordLength))
	plt.title('Chord distributions in ' + locNames[pieceNumber])
	plt.text(10, 15, 'Last chord ' + ' '.join([str(x) for x in lastChords[pieceNumber]]))
	plt.text(10, 14, 'Signature ' + ' '.join([str(x) for x in theSigs[pieceNumber]]))
	plt.text(10, 13, 'Sonorities ' + str(bigSum))
	plt.plot(values)
	ticks = range(len(values))
	plt.xticks(ticks, addbreaks(labels))
	plt.show()

def resolve_equivalence_classes(theDict, equivalenceClasses = False):
	if not equivalenceClasses:
		return theDict
	tempDict = copy.deepcopy(theDict)
	for c in equivalenceClasses:
		basicChord = c[0]
		if basicChord not in tempDict:
			tempDict[basicChord] = 0
		for otherChord in c[1:]:
			if otherChord in tempDict:
				tempDict[basicChord] += tempDict[otherChord]
				del tempDict[otherChord]
	return tempDict

def make_equivalence_classes(intList, subsets = False):
	outList = []
	for i in intList:
		outList.append([i, (i+3) % 12, (i+7) % 12])
		outList.append([i, (i+4) % 12, (i+7) % 12])
		if subsets:
			outList.append([i, (i+3) % 12])
			outList.append([i, (i+4) % 12])
			outList.append([i, (i+7) % 12])
			outList.append([i])
	return outList

pieceNumbers = []

def limit_pieces(lastChord = (5,), targetSig = (-1,), bestChord = False, transposeToC = False, printOut = False):
	global pieceNumbers
	pieceNumbers = []
	
	census(printOut = False, transposeToC = transposeToC)
	
	for k, l in keyCensus.items():
		if (not lastChord or k[0] == lastChord) and k[1] == targetSig and ((not bestChord) or (k[2] == bestChord)):
			if printOut:
				print(k, "\t", len(l))
			pieceNumbers += l
	

def average(chordLength = 3, subsets = True, norm = True, filterDiatonic = False, transposeToC = False, equivalenceClasses = [], useWeights = False, filterRare = 0, printTotal = False):
	#lastChord = tuple(lastChord)
	tempDict = {}
	
	if useWeights:
		currentDict = weightedProgs
	else:
		currentDict = progs
	
	#print "HERE!", lastChord, targetSig, bestChord
	
	"""if type(lastChord[0]) is not tuple:
		lastChord = make_subsets(lastChord)
		print "Using for last chords", lastChord"""
	
	if equivalenceClasses and type(equivalenceClasses[0]) is int:
		newList = []
		for x in equivalenceClasses:
			newList.append([tuple(y) for y in make_equivalence_classes([x], subsets = subsets)])
		equivalenceClasses = newList
	
	for p in pieceNumbers:
		newDict = resolve_equivalence_classes(currentDict[p], equivalenceClasses)
		for k, v in newDict.items():
			if transposeToC:
				transp = (-7 * theSigs[p][0])
				k = tuple([(x + transp) % 12 for x in k])
			tempDict[k] = tempDict.setdefault(k, 0) + v
			
	graphItems = [x for x in tempDict.items() if (type(x[0]) is not int and len(x[0]) == chordLength)]
	if filterDiatonic and len(targetSig) == 1:
		graphItems = filter_diatonic(graphItems, targetSig[0])
	graphItems = sorted(graphItems, key = lambda x: -x[1])
	bigSum = sum([x[1] for x in graphItems])
	if printTotal:
		print(globalSource, "total sonorities", bigSum)
	if norm:
		graphItems = [[x[0],x[1]*100./bigSum] for x in graphItems if x[1]*100./bigSum > filterRare]
	return graphItems

def chord_and_bass(**kwargs):
	global chordDict
	chordDict = {x[0]:x[1] for x in average(**kwargs)}
	ratioList = []
	for c, percent in  chordDict.items():
		if (c[-1] - c[0]) % 12 not in [6, 7]: continue
		firstInvPct = 0
		for i in [8, 9]:
			firstInv = c[:-1] + ((c[0] + i) % 12,)
			firstInvPct += chordDict.get(firstInv, 0)
		ratioList.append([c, percent, firstInvPct, percent/(firstInvPct + .000000001)])
	ratioList = sorted(ratioList, key = lambda x: -x[-1])
	for i in ratioList: 
		print(i[0], [round(x, 1) for x in i[1:]])


def filter_diatonic(graphItems, targetSig):
	myScale = sorted([((i) * 7) % 12 for i in range(targetSig-1, targetSig+6)])
	newGraphItems = []
	for item in graphItems:
		inScale = [x in myScale for x in item[0]]
		if all(inScale):
			newGraphItems.append(item)
	return newGraphItems
			
def bass_notes(**kwargs):
	global chordDict
	chordDict = {x[0]:x[1] for x in average(**kwargs)}
	bassNotes = {}
	for c, percent in  chordDict.items():
		bass = c[0]
		bassNotes[c[0]] = bassNotes.get(c[0], 0) + percent
	bassData = sorted(bassNotes.items(), key = lambda x: -x[-1])
	for i in bassData: 
		print(i[0], [round(x, 1) for x in i[1:]])
		
def combine_dyads_and_triads(**kwargs):
	global graphItems
	graphItems = sorted(average(norm = False, **kwargs), key = lambda x: -x[-1])
	newItems = average(chordLength = 2, norm = False, **kwargs)
	for k, p in newItems:
		for j in range(len(graphItems)):
			k2 = graphItems[j]
			if k == k2[:-1] or k == k2[::2]:
				graphItems[j][-1] += p
				break
	graphItems = sorted(graphItems, key = lambda x: -x[-1])
	for i in graphItems: print(i)

def make_subsets(myList):
	return [tuple(x) for x in [[myList[0]], [myList[0], myList[-1]], myList[:-1], myList]]
	
def key_name(sigNumber):
	if sigNumber < 0:
		return ['', 'one flat', 'two flats', 'three flats'][abs(sigNumber)]
	else:
		return ['no flats', 'one sharp', 'two sharps'][abs(sigNumber)]

def chord_histo(composers = _MASTERCOMPOSERS, filterRare = .5, **kwargs):
	global totalDict
	
	replacementNames = {'JOSQUINCOMPLETE': 'Josquin', 'PALRAW': "Palestrina", "VICTORIARAW": "Victoria"}
	kwargs.setdefault('chordLength', 3)
	noteDict = ['C', '', 'D', 'E-flat', 'E', 'F', 'F#', 'G', 'G#', 'A', 'Bb', 'B']
		
	totalDict = []
	for c in composers:
		load(c)
		graphItems = average(**kwargs)
		totalDict.append({x[0]:x[1] for x in graphItems})
		print (c, graphItems)
	
	totalKeys = []
	for d in totalDict:
		totalKeys += d.keys()
	
	#print totalKeys
	totalKeys = list(set(totalKeys))
	totalValues = {}
	for k in totalKeys:
		totalValues[k] = [d.get(k, 0) for d in  totalDict]
	
	finalKeys = [k for k in totalKeys if max(totalValues[k]) > filterRare]
	
	finalKeys = sorted(finalKeys, key = lambda x: -totalValues[x][0])
	
	yMax = 0
	for i, c in enumerate(composers):
		newList = [totalValues[k][i] for k in finalKeys]
		yMax = max(max(newList), yMax)
		print(c, [str(finalKeys[i]) + ' ' + str(round(newList[i], 1)) for i in range(len(newList))])
		plt.plot(newList, label=c)
	
	labels = [' '.join([str(x) for x in y]) for y in finalKeys]
	plt.xlabel('Harmonies (length ' + str(kwargs['chordLength']) + ' only)')
	plt.ylabel('Percentage of all sonorities of length ' + str(kwargs['chordLength']))
	#plt.title('Chord distributions in pieces ending on ' + noteDict[kwargs['lastChord'][0]] + ' with a key signature of ' + key_name(kwargs['targetSig'][0]))
	#plt.plot(values)
	ticks = range(len(labels))
	plt.xticks(ticks, addbreaks(labels))
	xMax = len(ticks)
	#plt.text(xMax * .75, yMax * .5, 'Last chord ' + ' '.join([str(x) for x in kwargs['lastChord']]))
	#plt.text(xMax * .75, yMax * .45, 'Signature ' + ' '.join([str(x) for x in kwargs['targetSig']]))
	
	tempComps = [replacementNames[x.upper()] if x.upper() in replacementNames else x for x in composers]
	
	#plt.text(xMax * .8, yMax * .85, 'Sonorities ' + str(bigSum))
	myLeg = plt.legend(tuple(tempComps), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def piece_counts(lastChord = [0, 4, 7], targetSig = [0], equivalenceClasses = [], requirePopularTonic = True, printOut = True):
	global totalPieces
	
	if lastChord:
		if type(lastChord[0]) is list:
			newLastChord = []
			for c in lastChord:
				newLastChord += make_subsets(c)
			lastChord = newLastChord
		elif type(lastChord[0]) is not tuple:
			lastChord = make_subsets(lastChord)
	
	if equivalenceClasses and type(equivalenceClasses[0]) is int:
		equivalenceClasses = make_equivalence_classes(equivalenceClasses)
	
	totalPieces = len(progs)
	matches = []
	
	for p in range(1, totalPieces + 1):
		if theSigs[p] == targetSig:
			if (not lastChord) or lastChords[p] in lastChord:
				if (not requirePopularTonic) or (not lastChord):
					matches.append(p)
				else:
					newDict = resolve_equivalence_classes(progs[p], equivalenceClasses)
					bestChord = sorted(newDict.items(), key = lambda x: -x[1])[0][0]
					if bestChord in lastChord:
						matches.append(p)
	
	if printOut:
		print(str(round(100.*len(matches)/totalPieces, 1)) + "% of " + str(totalPieces) + " pieces")
	
	return matches
	
def major_modes(**kwargs):
	masterList = [[[0, 4, 7], [0]], [[0, 4, 7], [-1]], [[5, 9, 0], [0]], [[5, 9, 0], [-1]], [[7, 11, 2], [0]]]
	counts = [0] * len(_MASTERCOMPOSERS)
	totals = [0] * len(_MASTERCOMPOSERS)
	for i, c in enumerate(_MASTERCOMPOSERS):
		c = c.upper()
		load(c)
		for lastChord, targetSig in masterList:
			matches = piece_counts(lastChord = lastChord, targetSig = targetSig, printOut = False, **kwargs)
			totals[i] = totalPieces
			counts[i] += len(matches)
		print(c, str(round(100.*counts[i]/totals[i], 1)) + "% of " + str(totals[i]) + " pieces")

"""def set_progressions(**kwargs):
	for c in _MASTERCOMPOSERS:
		c = c.upper()
		load(c)
		progressions.pieceCounts[c] = piece_counts(printOut = False, **kwargs)"""

targetPieceDict = {}
targetMinorKeys = {	((2,), (-1,), (2, 5, 9)), ((2,), (-1,), (7, 10, 2)), ((2,), (0,), (2, 5, 9)), ((2,), (0,), (2, 5)),
					((2,), (0,), (5, 9)), ((2,), (0,), (9, 0, 4)), ((4,), (0,), (2, 5, 9)), ((4,), (0,), (4, 7, 11)),
					((4,), (0,), (9, 0, 4)), ((7,), (-1,), (2, 5, 9)), ((7,), (-1,), (7, 10, 2)), ((7,), (-1,), (7, 10)),
					((9,), (0,), (2, 5, 9)), ((9,), (0,), (9, 0, 4)), ((9,), (0,), (9, 0))}
targetMajorKeys = {	((5,), (-1,), (5, 9, 0)), ((7,), (0,), (7, 11, 2)), ((0,), (0,), (0, 4)), ((5,), (-1,), (0, 4, 7)), 
					((0,), (0,), (0, 4, 7)), ((0,), (0,), (7, 11, 2)), ((5,), (0,), (5, 9, 0)), ((5,), (-1,), (5, 9)), 
					((7,), (0,), (0, 4, 7)), ((7,), (0,), (7, 11, 2)), ((5,), (-1,), (5, 9, 0)), ((7,), (0,), (0, 4, 7)), 
					((0,), (0,), (0, 4, 7)), ((0,), (0,), (7, 11, 2)), ((5,), (-1,), (0, 4, 7)), ((0,), (-1,), (5, 9, 0)), 
					((0,), (-1,), (0, 4, 7)), ((5,), (-1,), (10, 2, 5)), ((7,), (0,), (7, 11, 2)), ((5,), (-1,), (10, 2, 5)), 
					((7,), (0,), (0, 4, 7)), ((5,), (0, -1), (5, 9)), ((5,), (0, -1), (0, 4)), ((5,), (0, -1), (5, 9, 0)), 
					((7,), (0,), (0, 4)), ((5,), (0, -2, -1), (5, 9)), ((5,), (-1,), (5, 9, 0)), ((5,), (-2, -1), (10, 2)) }

def combine_piece_fragments(pNum):
	baseName = locNames[pNum][:-4]
	if globalSource != 'PALRAW' or (not baseName.count('_')) or (not baseName[-1].islower()):
		return progs[pNum], lastChords[pNum], theSigs[pNum]
	baseName = baseName[:-1]
	compositeProgs = {}
	lastChord = False
	theSig = False
	for letter in 'abcdefghijklmnopqrstuvwxyz':
		newName = baseName + letter + '.krn'
		newNum = locNumbers.get(newName, -1)
		if newNum == -1:
			break
		for prog, count in progs[newNum].items():
			compositeProgs[prog] = compositeProgs.get(prog, 0) + count
		lastChord = lastChords[newNum]
		theSig = theSigs[newNum]
	return compositeProgs, lastChord, theSig

def census(equivalenceClasses = [], justRoot = True, printOut = True, transposeToC = False, ignoreBestChord = False):
	global keyCensus, targetMinorKeys, globalTransposeToC
	
	globalTransposeToC = transposeToC
	
	targetPieces = []
	
	keyCensus = {}
	if equivalenceClasses and type(equivalenceClasses[0]) is int:
		equivalenceClasses = make_equivalence_classes(equivalenceClasses)
	
	totalPieces = len(progs)
	
	for p in range(1, totalPieces + 1):
		theProg, theLastChord, theSig = combine_piece_fragments(p)
		newDict = resolve_equivalence_classes(theProg, equivalenceClasses)
		if not newDict: 
			print('skipping file', p)
			return newDict
			continue
		bestChord = sorted(newDict.items(), key = lambda x: -x[1])
		bestChord = bestChord[0][0]
		if justRoot:
			lastChord = tuple([theLastChord[0]])
		else:
			lastChord = theLastChord
		if transposeToC and len(theSigs[p]) == 1:
			transp = (-7 * theSig[0])
			lastChord = tuple([(x + transp) % 12 for x in lastChord])
			bestChord = tuple([(x + transp) % 12 for x in bestChord])
			myKey = (lastChord, tuple([0]), bestChord)
		else:
			myKey = (lastChord, tuple(theSig), bestChord)
		if ignoreBestChord:
			myKey = myKey[:-1] + (False,)
		if myKey in targetMajorKeys:
			targetPieces.append(p)
			
		keyCensus.setdefault(myKey, []).append(p)
	
	totalSum = sum([len(x) for x in keyCensus.values()])	
	
	if printOut:
		for i in sorted(keyCensus.items(), key = lambda x: -len(x[1])): print(i[0], len(i[1]), round(len(i[1]) * 100./totalSum, 0))
		
	targetPieceDict[globalSource] = targetPieces

def tonal_types(popularChord = True, separateHighPart = True, printOut = True):
	global tonalTypes
	ranges.load(globalSource)
	tonalTypes = {}
	census(printOut = False)
	for k, locList in keyCensus.items():
		if not popularChord:
			k = k[:-1]
		for l in locList:
			cleffing = ranges.overall_range(l)
			if not cleffing:
				cleffing = ranges.overall_range(l, [0])
				if cleffing:
					if separateHighPart:
						cleffing += '+'
				else:
					newCleffing, failPct = ranges.best_fit(l)
					#print "approximate here", l, failPct, k
					cleffing = newCleffing
			newKey = k + tuple([cleffing])
			tonalTypes.setdefault(newKey, []).append(l)
	
	totalSum = sum([len(x) for x in tonalTypes.values()])	
	
	if printOut:
		for i in sorted(tonalTypes.items(), key = lambda x: -len(x[1])): print(i[0], len(i[1]), round(len(i[1]) * 100./totalSum, 0))

def count_all(composers = _MASTERCOMPOSERS, **kwargs):
	for c in composers:
		print(c)
		load(c)
		piece_counts(**kwargs)
	

def addbreaks(s, extraLine = False):
	if extraLine:
		mySym = '\n'
	else:
		mySym = ''
	t = []
	for i in range(len(s)):
		if i % 2 == 1:
			t.append('\n' + mySym + s[i])
		else:
			t.append(s[i])
	return t

def regex(patternStrings, qualities = False, roots = False, printOut = True, pct = True, filterRare = 2, useSum = False):
	global results, progs, rootProgs, progs2, rootProgs2, allProgs, bigSum
	source = [[progs2, rootProgs2][int(roots)], [progs, rootProgs][int(roots)]][int(qualities)]
	results = {}
	theSums = {}
	if type(patternStrings) is str:
		patternStrings = [patternStrings] 
	bigSum = sum(source.values())
	for prog in source:
		for patternString in patternStrings:
			if patternString not in results:
				results[patternString] = []
			if re.match(patternString, prog):
				results[patternString].append([prog, source[prog]])
	for patternString in patternStrings:
		tempList = sorted(results[patternString], key=lambda x: x[1], reverse = True)
		mySum = sum([x[1] for x in tempList])
		print(patternString, mySum, "%.1f of all progressions" % (100. * mySum/bigSum))
		if useSum:
			tempList = [[patternString, mySum * mySum/bigSum]]
		if printOut:
			for pair in tempList: 
				if pct:
					p = (100. * pair[1]/mySum)
					#print pair[0], allProgs.setdefault(pair[0], [])
					allProgs[pair[0]] = allProgs.setdefault(pair[0], [])
					allProgs[pair[0]].append(p)
					if not filterRare or p > filterRare:
						print(pair[0], "%.1f" % p)
				else:
					print(pair[0], pair[1])

def doall(*args, **kwargs):
	global allProgs
	allProgs = {}
	for c in ['OCKEGHEM', 'JOSQUINCOMPLETE', 'PALRAW']:
		print('Composer ', c)
		load(c)
		regex(*args, **kwargs)
	print() 
	theKeys = [x for x in allProgs.keys() if len(allProgs[x]) > 2]
	k = sorted(theKeys, key = lambda x: -abs(allProgs[x][0] - allProgs[x][2]))
	anomalies = []
	for theKey in k:
		print(theKey, ' '.join(["%.1f" % x for x in allProgs[theKey]]))
		data = allProgs[theKey]
		if (data[1] > data[0] and data[1] > data[2]) or (data[1] < data[0] and data[1] < data[2]):
			anomalies.append(theKey)
	print()
	print()
	print()
	print("ANOMALIES")
	for theKey in anomalies:
		print(theKey, ' '.join(["%.1f" % x for x in allProgs[theKey]]))

def compare_inversions_on_bass(targetFinals = [0, 2, 4, 7, 9], sig = (0,), chords = [[4, 7, 11], [4, 7, 0]]):
	for f in targetFinals:
		print("final", f, "sig", sig)
		limit_pieces(tuple([f]), sig, printOut = False)
		chordDict = {x[0]:x[1] for x in average(subsets = False)}
		results = [chordDict.get(tuple(x), 0) for x in chords]
		totalSum = float(sum(results))
		if not totalSum:
			print("  NOT FOUND")
			continue
		results = [x*100./totalSum for x in results]
		results = [[chords[i], results[i]] for i in range(len(chords))]
		print("  ", results)
			 
				
			
