from __future__ import print_function
import pickle, copy, os, time, collections
from music21 import *
import DT

epsilon = .01

workingAnalysis = {}			# will hold the temporary analysis
chordFrequencies = []
progressionFrequencies = []

"""chord frequencies are updated at the bottom of the file."""


"""

Analyze a classical-style work; uses the basic code of analyzer but first turns a piano-style piece into a chorale texture.
The basic idea was developed by my student Jeffrey Hodes: it uses the rules of nonharmonic tone identification to 'score' passages of music relative to a particular chord.
	i.e. "if this is a C chord then I can account for all the notes"
It does this first for an entire measure, then a half measure, then each beat.

So the entire approach:

	1. First, go through the piece and get everything that is fairly obvious
		check whole measures, then half note durations, then quarter notes, then eighth notes
	2. Then use simple scale-based chorale-style key finding to identify key areas and basic RNs (inherited from the chorale analyzer)
	3. Then go back for a more detailed pass
		- check zeroth-order plausibility
		
	TO DO:
	
	4. An even more careful pass? look for first-order plausibility?
		- will need a bass-line finder eventually this stage

Mozart Sonata scores
May 29, 2014: Correct key 84.04 Correct chord 74.96 (piece average)

To do:

	Incorporate full-measure Alberti (important for Chopin!)

	Idea: define a structure indexedBassNotes which is indexed by the durations in possibleAnalyses, so you can easily access the previous bass note at a given eighth (say)

	New strategy idea? for each measure, look for (a) all verticalities; and (b) all lower-register notes; (could add high register notes for rare moments when chord is up above)

	Do probability-based key-boundary finding.
	
	try first order changes (in but not working great)
	
	Aug 6ths?  Why don't they work?
	
	bass line evaluation of chords
	
	Maybe penalize eighth note echappees relative to ordinary PTs

	CHECK EACH CHORD FOR ITS BASS NOTE -- DISCARD CHORD CHANGES OVER SINGLE NOTE MELODIES?
	DEAL WITH SINGLE-NOTE MEASURES, MEASURES WHERE YOU CAN'T FIND ANYTHING
	GET ALTERNATIVE READINGS EVEN IF YOU HAVE FOUND A CHORD
	HIERARCHY OF IMPROVEMENT CRITERIA -- REALLY RARE CHORDS SHOULD BE REPLACED EVEN IF THIS INVOLVES A WEAKER (BUT TOLERABLE) CPT SCORE
	REPLACE VII7 AND IV7 IN MINOR?
	Incorporate minor sevenths, incomplete 0, 7, 10 chords? (and maybe major sevenths?)
	Neapolitan sixths?
	LOOK FOR REALLY BAD MEASURES -- VERY POOR SCORES
	Write fourth-pass code with bass-line finding?
	
Question -- should I just collect the notes on every minimum tempo duration so I don't have to keep calling getElementsByOffset?
	
"""

"""gather_stats is a wrapper that allows you to analyze all the XML files in a directory.

	fileName will analyze just a single file
	source identifies the directory
	overwrite = False to add to previous analyses.  make sure fileName is blank
	raw = True to bypass the routines that check the plausibility of the analysis
	maxMeasure X stop analyzing after measure X.  Useful for debugging

"""

def gather_stats(source = 'MOZART', fileName='', overwrite = False, raw = False, maxMeasure = -1, printOut = True):
	global globalSource, myScore, analysis, outputAnalysis, globalRaw, globalMaxMeasure, globalPrintOut
	globalSource = source.upper()
	globalRaw = raw
	globalMaxMeasure = maxMeasure
	globalPrintOut = printOut
	startT = time.time()
	dummyString = ''
	outFiles = os.listdir(DT._PATHS['Classical Analyzer'][0])
	if maxMeasure != -1: dummyString = 'X'
	if globalSource in DT._PATHS:
		analysisPath = DT._PATHS[globalSource][0]
		scorePath = DT._PATHS[globalSource][1]
		fileStr = globalSource
		files = sorted(os.listdir(scorePath))
		recordResults = True
		counter = 0
		for myFile in files:
			if myFile[-3:] in ['xml', 'krn']:
				if fileName and fileName != myFile[:-4]:
					continue
				baseFile = myFile[:-4] + dummyString + 'comp.txt'
				#print(baseFile, baseFile in outFiles)
				if (not overwrite) and (baseFile in outFiles):
					continue
				myFileName = DT._PATHS['Classical Analyzer'][0] + baseFile
				print('Analyzing', myFile)
				myScore = converter.parse(scorePath + myFile)
				with open(myFileName, 'w+') as outputAnalysis:
					outputAnalysis.write('Composer: ' + globalSource[0] + globalSource[1:].lower() + '\n')
					outputAnalysis.write('Analyst: Dmitri Tymoczko and his computer' + '\n')
					outputAnalysis.write('\n')
					do_piece()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and', int((minutes - int(minutes)) * 60), 'seconds')
	return

"""do_piece handles the analysis of a single file
	
	lots of important global variables
	
	workingMeasureNumber = the current measure number
	workingMeasure = the current measure.  This is a list with a stream at each index.
	previousMeasure, nextMeasure = what they sound like, same format as workingMeasure
	highestNumber = maxMeasure (see gather_stats) or the last measure of the piece

"""

def do_piece():
	global workingMeasureNumber, workingMeasure, previousMeasure, nextMeasure, highestNumber, possibleAnalyses, finalAnalyses
	global chords, chordsPCs, offsets, measuresAndBeats, chordList, measureBeats, timeSignatures, lastSig, measurePossibilities
	global lastAnalyzedKey, imcompleteTriads, lastStrippedFig, totalVerticalities, outputAnalysis, totalPossibilities, totalKeys
	global chordsByMeasure, keyDict, newChordList, newMeasureBeats, m0leftPad, globalRaw, bassNotes, newBassNotes, albertiLocations, noChange, globalPrintOut
	if globalPrintOut: print("RAW CHORDS")
	keyDict = {}
	possibleAnalyses = {}
	bassNotes = {}
	totalPossibilities = {}
	
	newBassNotes = {}
	finalAnalyses = {}
	measuresAndBeats = {}
	chordList = {}
	measureBeats = {}
	timeSignatures = {}
	noChange = {}					# for beats with no bass, where no chordchange should be 
	totalKeys = {}
	newChordList = {}
	newMeasureBeats = {}
	beatLengths = {}
	albertiLocations = {2: [], 1: [], .5: [], 'WHOLE': []}
	m0leftPad = 0
	lastSig = ''
	chords = stream.Stream()
	tempMeas = myScore.parts[0].getElementsByClass("Measure")
	highestNumber = tempMeas[-1].number
	firstNumber = tempMeas[0].number
	if globalMaxMeasure != -1:
		highestNumber = globalMaxMeasure
	for i in range(firstNumber, highestNumber + 1):
		workingMeasureNumber = i
		get_measure(i, firstNumber, highestNumber)
		bassNotes[i] = get_lowest_notes()
		if i != highestNumber:
			bassNotes[i+1] = get_lowest_notes(nextMeasure)
		find_alberti()
		do_measure(i)
		possibleAnalyses[i] = workingAnalyses
		totalPossibilities[i] = measurePossibilities
		finalAnalyses[i] = consolidate_analyses(i)
		print_analysis(i)									# Do not delete!  Performs some important consolidation functions!
		#print "  ", len(chords)
	offsets = [x.offset for x in chords]
	chordsPCs = [[y.pitchClass for y in x.pitches] for x in chords]
	make_key_list()
	
	incompleteTriads = []
	lastAnalyzedKey = False
	lastStrippedFig = ''
	totalVerticalities = 0
	
	for i in range(len(bestKeys)):							# create the analysis
		analyze_key_area(i)
	
	"""now for post processing!"""
	
	"""first compile a list of chord changes, as [locationInBeats, 'KEY: CHORDSYMBOL']"""
	if not globalRaw:
		
		chordsByMeasure = {}
		for i in measureBeats:
			m = measureBeats[i][0]
			if m in chordsByMeasure:
				chordsByMeasure[m].append([measureBeats[i][1], i])
			else:
				chordsByMeasure[m] = [[measureBeats[i][1], i]]
				
		"""now check the analysis for zeroth-order plausibility"""
		
		for i in range(firstNumber, highestNumber + 1):
			check_measure(i)
	
		chordList = newChordList
		measureBeats = newMeasureBeats
		
		#filter_chordList()
	
		#check_bass_notes()				# gathers the bass notes for each chord
	
	if globalPrintOut: print("")
	if globalPrintOut: print("Roman numerals")
	if globalPrintOut: print("")
	final_write()
	return	

"""Unused routine to delete chord changes on noChange points; this has been superseded by a better approach """

"""def filter_chordList():							# remove chord changes with no bass line
	delList = []
	for i in chordList:
		m, b = measureBeats[i]
		if m not in noChange:
			continue
		if float(b) in noChange[m]:
			delList.append(i)
	for i in delList:
		del chordList[i]
		del measureBeats[i]"""

def clean_chordList():															# removes duplicates and makes index #s the same
	offset = 0
	lastChord = ''
	#for i in chordList: if globalPrintOut: print i, chordList[i]
	for i in range(min(chordList.keys()), max(chordList.keys()) + 1):
		if i not in chordList:
			offset += 1
		else:
			if chordList[i] != lastChord:
				lastChord = chordList[i]
				if offset != 0:
					chordList[i - offset] = chordList[i]
					measureBeats[i - offset] = measureBeats[i]	
					del chordList[i]
					del measureBeats[i]
			else:
				del chordList[i]
				del measureBeats[i]
				offset += 1
	#for i in chordList: if globalPrintOut: print i, chordList[i]

def check_bass_notes():
	global bassNotesByChord
	clean_chordList()
	bassNotesByChord = {}
	minBass = 10000
	for c in chordList: 
		startMeasure = measureBeats[c][0]
		startBeat = measureBeats[c][1]
		if c + 1 in chordList:
			endMeasure = measureBeats[c+1][0]
			endBeat = measureBeats[c+1][1]
			if endMeasure == startMeasure + 1 and endBeat == 1.0:
				endMeasure = startMeasure
				endBeat = 10000
		else:
			endMeasure = startMeasure
			endBeat = 10000
		tempNote = get_bass_notes_in_span(startMeasure, startBeat, endMeasure, endBeat)
		bassNotesByChord[c] = tempNote
			
def get_bass_notes_in_span(startMeasure, startBeat, endMeasure, endBeat):
	for i in reversed(range(0, startMeasure + 1)):
		if i in timeSignatures:
			update_time_signature(timeSignatures[i])
			break
	if endMeasure == startMeasure:
		myNotes = bassNotes[startMeasure].getElementsByOffset((startBeat - 1) * beatLength, (endBeat - 1) * beatLength, includeEndBoundary = False, includeElementsThatEndAtStart = False, mustBeginInSpan = False, classList = ['Note'])
		if myNotes:
			return min(myNotes.pitches)
		else:
			return False
	else:
		tempNote = pitch.Pitch('C12')
		myNotes = bassNotes[startMeasure].getElementsByOffset((startBeat - 1) * beatLength, 10000, includeEndBoundary = False, includeElementsThatEndAtStart = False, mustBeginInSpan = False, classList = ['Note'])	
		if myNotes:
			testNote = min(myNotes.pitches)
			if testNote.pitch.midi < tempNote.pitch.midi:
				tempNote = testNote
		for mNum in range(startMeasure + 1, endMeasure):
			if mNum in timeSignatures:
				update_time_signature(mNum)
			myNotes = bassNotes[mNum].getElementsByOffset(0, 10000, includeEndBoundary = False, mustBeginInSpan = False, includeElementsThatEndAtStart = False, classList = ['Note'])	
			if myNotes:
				testNote = min(myNotes.pitches)
				if testNote.pitch.midi < tempNote.pitch.midi:
					tempNote = testNote
		if endMeasure in timeSignatures:
			update_time_signature(endMeasure)
		myNotes = bassNotes[endMeasure].getElementsByOffset(0, (endBeat - 1) * beatLength, includeEndBoundary = False, mustBeginInSpan = False, includeElementsThatEndAtStart = False, classList = ['Note'])	
		if myNotes:
			testNote = min(myNotes.pitches)
			if testNote.pitch.midi < tempNote.pitch.midi:
				tempNote = testNote
		return testNote

def fix_bass_notes(mNum):
	myNotes = bassNotes[mNum]
	if mNum in timeSignatures:
		update_time_signature(timeSignatures[mNum])

def assemble_analysis(mNum):
	workingAnalysis = possibleAnalyses[mNum]
	durs = sorted(workingAnalysis.keys(), reverse=True)
	minDur = durs[-1]
	rnAnalysis = [False] * len(workingAnalysis[durs[-1]])
	
def get_previous_chord(mNum):
	for i in reversed(range(0, mNum - 1)):
		if i in chordsByMeasure:
			return chordsByMeasure[i][-1][1]

def get_next_chord(mNum):
	for i in range(mNum + 1, highestNumber):
		if i in chordsByMeasure:
			return chordsByMeasure[i][0][1]

"""This routine looks for Alberti bass patterns, and also for passages of high-note figuration that
should not be interpreted chordally.

Exactly how to do the latter correctly is a bit of an issue.  Really we should allow high chord changes if they are very explicit.

"""

def find_alberti():
	
	i = workingMeasureNumber
	noChange[i] = []
	
	"""if i != highestNumber:
		result = look_for_alberti_values(i, beatLength * beatCount)
		result2 = look_for_alberti_values(i + 1, beatLength * beatCount)
		
		if result and result2:
			withinBeatDistance, tempBass = result
			junk, tempBass2 = result2
			myDist = abs(tempBass2[0] - tempBass[0])
			if myDist < withinBeatDistance or myDist <= 2:
				print "   POSSIBLE MEASURE-LENGTH ALBERTI, MEASURE", i 
				albertiLocations['WHOLE'].append(i)
				return"""
			
			
	
	result = look_for_alberti_values(i, beatLength/2.)
	
	if result:
		
		withinBeatDistance, tempBass = result
		
		myDists = []
		for j in range(1, len(tempBass)):
			myDists.append(abs(tempBass[j] - tempBass[j-1]))
		stepCount = sum([x<=2 for x in myDists])
		if (len(tempBass) >= 4 and max(myDists) < withinBeatDistance) or stepCount + 1 >= len(myDists):
			#print("   POSSIBLE HALF-BEAT-LENGTH ALBERTI, MEASURE", i) 
			albertiLocations[.5].append(i)
		return
	
	result = look_for_alberti_values(i, beatLength)
	
	if result:
		
		withinBeatDistance, tempBass = result
		myDists = []
		for j in range(1, len(tempBass)):
			myDists.append(abs(tempBass[j] - tempBass[j-1]))
		stepCount = sum([x<=2 for x in myDists])
		if max(myDists) < withinBeatDistance or (len(tempBass) >= 4 and stepCount + 1 >= len(myDists)):
			#print("   POSSIBLE BEAT-LENGTH ALBERTI, MEASURE", i) 
			#print tempBass, myDists, withinBeatDistance
			albertiLocations[1].append(i)
		elif len(tempBass) == 4:
			wb = min(abs(tempBass[1] - tempBass[0]), abs(tempBass[3] - tempBass[2]))
			cb = abs(tempBass[2] - tempBass[0])
			if cb < wb or cb <= 2:
				#print("   POSSIBLE HALF-MEASURE ALBERTI, MEASURE", i) 
				albertiLocations[2].append(i)

def look_for_alberti_values(i, myDur):
	
	if myDur == beatLength:
		noChangeFlag = True
	else:
		noChangeFlag = False
	withinBeatDistance = 10000
	betweenBeatDistance = 0
	tempBass = []
	
	haltFlag = False
	
	measureDuration = beatCount * beatLength
	
	myUnits = int(measureDuration/myDur)
	
	for b in range(myUnits):
		
		myNotes = bassNotes[i].getElementsByOffset(b * myDur, (b+1) * myDur, includeEndBoundary = False, mustBeginInSpan = False, includeElementsThatEndAtStart = False, classList = ['Note'])
		
		if myNotes:
			tempBass.append(myNotes[0].pitch.midi)										# DANGEROUS!  REQUIRE BASS NOTE TO BE FIRST?
			if len(myNotes) >= 2:
				sNotes = sorted(myNotes)
				if noChangeFlag and sNotes[0].pitch.midi >= 72 and i > 2:					# need to do better on this, to allow for high-register chord changes
					noChange[i].append(b + 1)										# beat number, on which there is no bass line
				if myNotes[0].pitch.midi == sNotes[0].pitch.midi:
					tempDist = sNotes[1].pitch.midi - sNotes[0].pitch.midi
				else:
					tempDist = 10000
				if tempDist < withinBeatDistance:
					withinBeatDistance = tempDist
			else:
				if myDur == beatLength / 2.:
					haltFlag = True
		else:
			haltFlag = True
	
	if haltFlag: 
		return False
		
	return [withinBeatDistance, tempBass]

def check_measure(mNum):
	
	highestChordNumber = max(chordList)
	
	workingAnalysis = possibleAnalyses[mNum]
	
	if mNum in timeSignatures:
		ts = timeSignatures[mNum]
		update_time_signature(ts)
	durs = sorted(workingAnalysis.keys(), reverse=True)
	
	
	minDur = durs[-1]
	finalAnalysis = [None] * len(workingAnalysis[durs[-1]])
	changesToMake = [None] * len(workingAnalysis[durs[-1]])
	myKeys = [None] * len(workingAnalysis[durs[-1]])
	newFinalAnalysis = [None] * len(workingAnalysis[durs[-1]])
	
	if mNum in chordsByMeasure:
		for b, i in chordsByMeasure[mNum]:
			b = b - 1.0
			b = int(b * beatLength/minDur)
			if i == None:
				i = 0
			finalAnalysis[b] = i
	
	#print mNum
	#for i in finalAnalysis: print i
	
	prevMeasureChord = get_previous_chord(mNum)
	nextMeasureChord = get_next_chord(mNum)
	lastChord = prevMeasureChord
	#print prevMeasureChord
	
	for i in range(len(finalAnalysis)):
		if finalAnalysis[i] != None:
			lastChord = finalAnalysis[i]
		else:
			finalAnalysis[i] = lastChord
	
	for i in range(len(finalAnalysis)):
		
		if finalAnalysis[i] in [None, False] or finalAnalyses[mNum][i] in [None, False]: continue
		
		cptScore = finalAnalyses[mNum][i][0]
		symbol = chordList[finalAnalysis[i]]
		chordSymbol = symbol.split()
		if len(chordSymbol) > 1 and chordSymbol[0] != '?:':
			myKeyName = chordSymbol[0]
			if myKeyName not in keyDict:
				keyDict[myKeyName] = key.Key(myKeyName[:-1])
			myKey = keyDict[myKeyName]
			modeInd = chordSymbol[0][0].islower()
			chordSymbol = chordSymbol[1]
		else:
			modeInd = 0
			myKeyName = 'C:'
			if myKeyName not in keyDict:
				keyDict[myKeyName] = key.Key(myKeyName[:-1])
			myKey = keyDict[myKeyName]
			chordSymbol = 'I'
		
		myKeys[i] = myKey
			
		rnScore = chordFrequencies[modeInd][chordSymbol]
		
		if finalAnalysis[i] == 1:
			prevChordSymbol = False
		else:
			prevSymbol = chordList[finalAnalysis[i] - 1].split()					# this is wrong
			if len(prevSymbol) > 1 and prevSymbol[0] != '?:':
				prevKeyName = prevSymbol[0]
				if prevKeyName == myKeyName:
					prevChordSymbol = prevSymbol[1]
				else:
					prevChordSymbol = False
			else:
				prevChordSymbol = False
		
		if finalAnalysis[i] >= highestChordNumber:
			nextChordSymbol = False
		else:
			nextSymbol = chordList[finalAnalysis[i] + 1].split()
			if len(nextSymbol) > 1 and nextSymbol[0] != '?:':
				nextKeyName = nextSymbol[0]
				if nextKeyName == myKeyName:
					nextChordSymbol = nextSymbol[1]
				else:
					nextChordSymbol = False
			else:
				nextChordSymbol = False
		
		if prevChordSymbol and nextChordSymbol:
			firstOrderScore = progressionFrequencies[modeInd][prevChordSymbol + ' -> ' + chordSymbol] * progressionFrequencies[modeInd][chordSymbol + ' -> ' + nextChordSymbol]
		else: 
			firstOrderScore = 0
		
		"""if prevChordSymbol and nextChordSymbol:
			firstOrderScore = progressionFrequencies[modeInd][prevChordSymbol + ' -> ' + chordSymbol] * progressionFrequencies[modeInd][chordSymbol + ' -> ' + nextChordSymbol]
		else: 
			firstOrderScore = 0"""
		
		for j in range(len(durs)):								# more elegant to iterate over durs, I think
			myDur = durs[j]
			
			"""TRY THIS -- DOES IT WORK BETTER???  IDEA: set a switch to 'strict' that requires things to be really tight to change chords"""
			
			stopFlag = False
			
			for k in [.5, 1, 2]:
				if myDur != beatLength * k and mNum in albertiLocations[k]:
					stopFlag = True
					break
			
			if stopFlag: continue
			
			startAddress = int((i * minDur)/myDur)
			if not workingAnalysis[myDur][startAddress]: continue
			
			cptScore2 = workingAnalysis[myDur][startAddress][0]
			newChord = chord.Chord(workingAnalysis[myDur][startAddress][1])
			"""cptScore2 = myPair[0]
			newChord = chord.Chord(myPair[1])"""
			
			myNF = DT.normal_form(list(set([x.pitchClass for x in newChord.pitches])))
		
			#print mNum, DT.nice_name(newChord), myNF
		
			if myNF in [[0, 4], [0, 3]]:
				chord2 = chord.Chord(list(newChord.pitches) + [newChord.root().transpose('M6')])
				fig2 = DT.figure_from_chord(chord2, myKey)
				chord3 = chord.Chord(list(newChord.pitches) + [newChord.root().transpose('m6')])
				fig3 = DT.figure_from_chord(chord3, myKey)
	
				newSymbol = DT.figure_from_chord(newChord, myKey)
				
				if newSymbol == False: 
					continue
				
				checkProbs = True
				
				if prevChordSymbol and fig2 and fig3 and newSymbol:
					prevParse = DT.parse_figure(prevChordSymbol)
					if len(prevParse) == 4:
						for testSym in [fig2, fig3, newSymbol]:
							testParse = DT.parse_figure(testSym)
							if len(testParse) == 4 and prevParse[0] == testParse[0] and prevParse[2] == testParse[2]:
								checkProbs = False
								newSymbol = testSym
							
				if checkProbs and fig2:
					newSymbol = max([newSymbol, fig2, fig3], key = lambda x: chordFrequencies[modeInd][x])		# return the most likely reading of a third
				
			else:
				newSymbol = DT.figure_from_chord(newChord, myKey)
			
			rnScore2 = chordFrequencies[modeInd][newSymbol]
			oldParse = DT.parse_figure(chordSymbol)
			newParse = DT.parse_figure(newSymbol)
			
			if prevChordSymbol and nextChordSymbol:
				firstOrderScore2 = progressionFrequencies[modeInd][prevChordSymbol + ' -> ' + newSymbol] * progressionFrequencies[modeInd][newSymbol + ' -> ' + nextChordSymbol]
			else: 
				firstOrderScore2 = 0
			
			"""if prevChordSymbol and nextChordSymbol:
				firstOrderScore2 = progressionFrequencies[modeInd][prevChordSymbol + ' -> ' + newSymbol] * progressionFrequencies[modeInd][newSymbol + ' -> ' + nextChordSymbol]
			else: 
				firstOrderScore2 = 0"""
			
			"""if rnScore2 < rnScore and firstOrderScore2 > firstOrderScore:
				print " POSSIBLE FIX HERE", mNum, newSymbol, "for", chordSymbol"""
			
			"Don't just change inversions here -- we need special code for finding a bass line"
		
			if oldParse[0] == newParse[0] and oldParse[2] == newParse[2] and oldParse[3] == newParse[3] and oldParse[1] in ['', '6', '6/4'] and newParse[1] in ['', '6', '6/4']:
				continue
				
			"""need to fix the first order testing."""
		
			if rnScore2 >= rnScore and ((cptScore2 == 2 and (cptScore != 0 or chordSymbol == 'V2' or rnScore <= 1)) or (cptScore != 0 and cptScore2 <= cptScore)): 	# better harmony and better counterpoint
				changesToMake[i] = myKeyName + ' ' + newSymbol
			elif (rnScore2 - rnScore) > 5 and (cptScore2 - cptScore) <= .25 and (cptScore != 0 or symbol == 'V2' or rnScore <= 1):		# trade off worse counterpoint for better harmony		
				changesToMake[i] = myKeyName + ' ' + newSymbol
			elif firstOrderScore2 >= firstOrderScore and (cptScore2 - cptScore) <= .25 and (cptScore != 0 or symbol == 'V2' or rnScore <= 1):
				changesToMake[i] = myKeyName + ' ' + newSymbol
			elif newParse[0] == 'viio' and oldParse[0] not in ['viio', 'V']:
				changesToMake[i] = myKeyName + ' ' + newSymbol
	
	durationList = []
	
	try:
		lastSymbol = chordList[finalAnalysis[0]]
	except:
		print("ERROR", mNum)
		for i in finalAnalysis: print(i)
		return
	myDur = 1 
	
	if changesToMake[0] != None:
		lastSymbol =  changesToMake[0]
	
	for i in range(1, len(finalAnalysis)):
		
		if changesToMake[i] != None:
			newSymbol = changesToMake[i]
		else:
			newSymbol = chordList[finalAnalysis[i]]
		
		if newSymbol == lastSymbol:
			myDur += 1
		else:
			durationList.append(myDur * minDur)
			lastSymbol = newSymbol
			myDur = 1
	
	durationList.append(myDur * minDur)
	#print "measure", mNum, "durations", durationList
	
	"""Suspicious code in the following."""
	
	if len(durationList) > 1:
		startDur = 0
		for myDur in durationList:
			startAddress = float(startDur) / (myDur)	
			if not is_int(startAddress):
				startDur += myDur
				continue
			startAddress = int(startAddress)
			if (not myDur in workingAnalysis) or (not workingAnalysis[myDur][startAddress]) or (not myKeys[startAddress]):
				startDur += myDur
				continue
			
			newChord = chord.Chord(workingAnalysis[myDur][startAddress][1])				# startAddress is indexed in units of myDur
			
			startAddress2 = int(startDur / minDur)										# indexed in unts of minDur
			
			
			myNF = DT.normal_form(list(set([x.pitchClass for x in newChord.pitches])))
			myKey = myKeys[startAddress2]
		
			if myNF in [[0, 4], [0, 3]]:
				chord2 = chord.Chord(list(newChord.pitches) + [newChord.root().transpose('M6')])
				fig2 = DT.figure_from_chord(chord2, myKey)
				chord3 = chord.Chord(list(newChord.pitches) + [newChord.root().transpose('m6')])
				fig3 = DT.figure_from_chord(chord3, myKey)
	
				newSymbol = DT.figure_from_chord(newChord, myKey)
				if newSymbol == False: 
					continue
				
				checkProbs = True
				
				if prevChordSymbol and fig2 and fig3 and newSymbol:
					prevParse = DT.parse_figure(prevChordSymbol)
					if len(prevParse) == 4:
						for testSym in [fig2, fig3, newSymbol]:
							testParse = DT.parse_figure(testSym)
							if len(testParse) == 4 and prevParse[0] == testParse[0] and prevParse[2] == testParse[2]:
								checkProbs = False
								newSymbol = testSym
							
				if checkProbs and fig2:
					newSymbol = max([newSymbol, fig2, fig3], key = lambda x: chordFrequencies[modeInd][x])		# return the most likely reading of a third
			else:
				newSymbol = DT.figure_from_chord(newChord, myKey)
			
			
			if changesToMake[startAddress2] != None:
				oldSymbol = changesToMake[startAddress2]
				oldSymbol = oldSymbol.split()
			else:
				if finalAnalysis[startAddress2] in chordList:
					oldSymbol = chordList[finalAnalysis[startAddress2]]
					oldSymbol = oldSymbol.split()
				else:
					oldSymbol = False
				
			if ((not oldSymbol) or len(oldSymbol) < 2 or oldSymbol[1] != newSymbol):
				#print("  CHANGE HERE", mNum, startDur, startAddress2, oldSymbol, newSymbol, myDur)
				for i in range(int(myDur/minDur)):
					changesToMake[startAddress2 + i] = oldSymbol[0] + ' ' + newSymbol
					
			startDur += myDur
	
	lastAnalysis = ''
	
	for i in range(len(finalAnalysis)):
		newSymbol = ''
		if changesToMake[i] != None:
			newSymbol = changesToMake[i]
		else:
			if finalAnalysis[i] in chordList:
				newSymbol = chordList[finalAnalysis[i]]
		if newSymbol == lastAnalysis: 
			continue
		
		tempSplit = lastAnalysis.split()
		tempSplit2 = newSymbol.split()
				
		if len(tempSplit) > 1 and len(tempSplit2) > 1 and tempSplit[0] == tempSplit2[0]:
		
			oldParse = DT.parse_figure(tempSplit[1])
			newParse = DT.parse_figure(tempSplit2[1])
		
			"""prevent sevenths from just disappearing"""
		
			if oldParse[0] == 'V' and oldParse[1] in ['7', '6/5', '4/3', '2'] and newParse[0] == 'V' and newParse[1] in ['', '6', '6/4'] and oldParse[2] == newParse[2]:
				continue
			
		lastAnalysis = newSymbol
		newAddress = len(newChordList)
		newBeatNum = ((i * minDur) / beatLength) + 1
		newChordList[newAddress] = newSymbol
		newMeasureBeats[newAddress] = [mNum, newBeatNum]
	

def is_int(f, tolerance = .001):
	if abs(f - int(f)) <= tolerance:
		return True
	return False

"""First we gather a list of possible analyses, then we consolidate them"""

def consolidate_analyses(mNum):
	workingAnalysis = possibleAnalyses[mNum]
	durs = sorted(workingAnalysis.keys(), reverse=True)
	minDur = durs[-1]
	finalAnalysis = [False] * len(workingAnalysis[durs[-1]])
	for i in range(len(finalAnalysis)):
		pitchClasses = set([])									# PCs of current chord 
		for j in range(len(durs)):								
			myDur = durs[j]
			
			"""TRY THIS -- DOES IT WORK BETTER???"""
			
			stopFlag = False
			
			for k in [.5, 1, 2]:
				if myDur != beatLength * k and mNum in albertiLocations[k]:
					stopFlag = True
					break
			
			if stopFlag: continue
			
			startAddress = int((i * minDur)/myDur)
			beatNumber = int((i * minDur)/beatLength) + 1
			
			if beatNumber in noChange[mNum] and myDur < beatLength:				# only allow larger durations over a "nochange" passage
				continue
			
			if not workingAnalysis[myDur][startAddress]: continue
			grade = workingAnalysis[myDur][startAddress][0]
			
			
			if (not finalAnalysis[i]) or grade <= finalAnalysis[i][0]:								# big difference between < and <=
				#finalAnalysis[i] = [grade, workingAnalysis[myDur][startAddress][1]]					# reverting to this; commented-out stuff below gives an alternative
				tempPCs = set([x.pitchClass for x in workingAnalysis[myDur][startAddress][1]])
				if tempPCs == pitchClasses: continue
				if (not finalAnalysis[i]) or len(tempPCs) > 2 or (not tempPCs <= pitchClasses) or len(pitchClasses) > 3:	# don't choose a 1 or 2-note subset just because it fits better
					finalAnalysis[i] = [grade, workingAnalysis[myDur][startAddress][1]]					
					pitchClasses = tempPCs
	#print mNum, finalAnalysis
	return finalAnalysis

"""Formats the current analysis"""

def print_analysis(mNum):										# DO NOT DELETE!!!!!
	global globalPrintOut
	finalAnalysis = finalAnalyses[mNum]
	lastSym = ''
	beatStr = ''
	outStr = 'm' + str(mNum)
	minDur = sorted(possibleAnalyses[mNum].keys(), reverse=True)[-1]
	for i in range(len(finalAnalysis)):
		if not finalAnalysis[i]: continue
		tempStr = chord_name(finalAnalysis[i][1])
		if tempStr != lastSym:
			chords.append(chord.Chord(finalAnalysis[i][1]))					# this is needed
			measuresAndBeats[len(chords) - 1] = [workingMeasureNumber, ((i * minDur)/beatLength) + 1]
			if i != 0:
				beatStr = ' b' + '%.2f' % (((i * minDur)/beatLength) + 1)
			outStr = outStr + beatStr + ' ' + tempStr
			lastSym = tempStr
	outStr = outStr.replace('.00', '').replace('.50', '.5')
	if globalPrintOut: print(outStr)

def debug_analysis(mNum):
	workingAnalysis = possibleAnalyses[mNum]
	durs = sorted(workingAnalysis.keys(), reverse=True)
	for i in durs:
		print('DURATION', i)
		for j in workingAnalysis[i]:
			if j:
				print('   ', j[0], chord_name(j[1]))
			else:
				print('   False')   

def get_measure(i, firstNumber, highestNumber):
	global previousMeasure, workingMeasure, nextMeasure, beatCount, beatLength, barLength, atomicDuration, atomicUnits, m0leftPad
	global timeSignatures, lastSig, lastRawSignature
	if i == firstNumber:
		previousMeasure = False
		workingMeasure = []
		for myPart in myScore.parts:
			workingMeasure.append(myPart.measures(i, i, collect=[]))
		if i == 0:
			m0leftPad = workingMeasure[0][0].paddingLeft
		nextMeasure = []
		for myPart in myScore.parts:
			nextMeasure.append(myPart.measures(i + 1, i + 1, collect=[]))
	elif i > firstNumber and i < highestNumber:
		previousMeasure = workingMeasure
		workingMeasure = nextMeasure
		nextMeasure = []
		for myPart in myScore.parts:
			nextMeasure.append(myPart.measures(i + 1, i + 1, collect=[]))
	elif i == highestNumber:
		previousMeasure = workingMeasure
		workingMeasure = nextMeasure
		nextMeasure = []
	"""THE FOLLOWING IS BROKEN SOMEHOW, AND NEEDS MORE THOROUGH INVESTIGATION"""
	for p in workingMeasure:
		ts = p[0].getContextByClass('TimeSignature')							# figure out how to write out the ratioString
		if ts:
			break
	if not ts:
		for p in workingMeasure:
			newTS = p[0].getElementsByClass('TimeSignature')
			if newTS:
				ts = newTS[0]
				break
		if not ts:
			if lastRawSignature:
				ts = lastRawSignature
			else:
				print("PROBLEM: can't find time signature for measure", workingMeasureNumber)
	lastRawSignature = ts
	update_time_signature(ts)
	if ratioString != lastSig:
		timeSignatures[i] = ts
		lastSig = ratioString
	if i == firstNumber:
		m0leftPad = m0leftPad / beatLength

def get_lowest_notes(targetMeasure = False):
	if not targetMeasure:
		targetMeasure = workingMeasure
	outStream = stream.Measure()
	outDict = {}
	lastNote = ''
	totalOffsets = []
	for i in range(len(targetMeasure)):
		tempStream = targetMeasure[i].flat.getElementsByClass(['Note', 'Chord'])
		totalOffsets += [x.offset for x in tempStream]
		totalOffsets.append(targetMeasure[i].highestTime)
	totalOffsets = sorted(list(set(totalOffsets)))
	for index in range(len(totalOffsets) - 1):
		i = totalOffsets[index]
		tempMidi = 10000
		tempNote = False
		for j in range(len(targetMeasure)):
			testList = find_minimum(targetMeasure[j].flat.getElementsByOffset(i + epsilon, i + 2 * epsilon, mustBeginInSpan = False, includeEndBoundary = True, classList = ['Note', 'Chord']))
			if not testList:
				continue
			if testList[0] < tempMidi:
				tempMidi = testList[0]
				tempNote = testList[1]
		if tempNote:
			if tempNote == lastNote:
				outStream[-1].duration.quarterLength = outStream[-1].duration.quarterLength + totalOffsets[index + 1] - totalOffsets[index]
			else:
				outStream.insert(i, note.Note(tempNote, quarterLength = totalOffsets[index + 1] - totalOffsets[index]))
				lastNote = tempNote
	return outStream

def find_minimum(myStream):		# should contain only notes and chords
	if not myStream:
		return False
	tempMidi = 10000
	tempNote = False
	for item in myStream:
		if 'Note' in item.classes:
			testItem = item
		else:
			testItem = min(item)
		if testItem.pitch.midi < tempMidi:
			tempMidi = testItem.pitch.midi
			tempNote = testItem.nameWithOctave
	return [tempMidi, tempNote]

def update_time_signature(ts):
	global ratioString, beatLength, beatCount, barLength, atomicDuration
	ratioString = ts.ratioString
	beatLength = ts.beatDuration.quarterLength
	beatCount = ts.beatCount
	barLength = beatLength * beatCount
	atomicDuration = ts.beatDivisionDurations[0].quarterLength
	
def do_measure(n):
	global workingAnalyses, measurePossibilities
	workingAnalyses = {}
	measurePossibilities = {}
	iterate_durations(barLength)						# whole bar
	if beatCount == 4:
		iterate_durations(beatLength * 2)			# half bar for 4/4 (could add something for 3/4 here)
	iterate_durations(beatLength)					# each beat (2, 3, or 4)
	iterate_durations(atomicDuration)				# one-half or one-third of a beat
	if beatCount == 2 and beatLength == 2.0:		# go down to eighth notes in 2/2
		iterate_durations(.5)
	return

def iterate_durations(durLength):
	global workingAnalyses, measurePossibilities, possibleInterpretations
	outList = []
	outList2 = []									# records all possible analyses
	for i in range(int(.01 + barLength/durLength)):
		results = do_section(i * durLength, durLength)
		if results:
			outList.append(results)
			outList2.append(possibleInterpretations)
		else:
			outList.append(False)
			outList2.append(False)
	workingAnalyses[durLength] = outList
	measurePossibilities[durLength] = outList2
		
triadAndSeventh = ([0, 4, 7], [0, 3, 7], [0, 3, 6], [0, 3, 6, 8], [0, 3, 6, 9])
triadList = ([0, 4, 7], [0, 3, 7])
fullTriadList = ([0, 4, 7], [0, 3, 7], [0, 3, 6], [0, 4, 8])
triadDict = {}
subsetList = ([0, 4, 7], [0, 3, 7], [0, 3, 6], [8, 0, 3, 6], [0, 3, 6, 9])
subsetIntervals = (['P1', 'M3', 'P5'], ['P1', 'm3', 'P5'], ['P1', 'm3', 'd5'], ['P1', 'M3', 'P5', 'm7'], ['P1', 'm3', 'd5', 'd7'], ['P1', 'm3', 'd5', 'm7'], ['P1', 'd3', 'd5'], ['P1', 'd3', 'd5', 'd7'], ['P1', 'M3', 'd5', 'm7'])
bareThirdIntervals = (['P1', 'M3'], ['P1', 'm3'], ['P1', 'P5'], ['P1', 'M3', 'm7'], ['P1', 'm3', 'd7'], ['P1', 'd5', 'd7'])		# added a fifth here for cadences

def find_subsets(pitchList, triadList = subsetIntervals):
	interpretations = []
	letternameSet = set([])
	pitchSet = set([])
	for item in pitchList:
		n = item.name
		if n in letternameSet: continue
		letternameSet.add(n)
		pitchSet.add(item)
	for basePitch in pitchSet:
		for subset in triadList:
			tempChord = [basePitch.transpose(x) for x in subset]
			tempChordNames = set([x.name for x in tempChord])
			if tempChordNames < letternameSet:
				interpretations.append(tempChord)
	return interpretations

def get_notes_in_span(myMeas, startOffset, sectionDuration):
	tempNotes = []
	if not myMeas:
		return tempNotes
	for i in range(len(myMeas)):
		tryPitches = myMeas[i].flat.getElementsByOffset(startOffset, startOffset + sectionDuration, includeEndBoundary = False, includeElementsThatEndAtStart = False, mustBeginInSpan = False, classList = ['Note', 'Chord'])
		for item in tryPitches:
			if 'Chord' in item.classes:
				for tempPitch in item.pitches:
					myNote = note.Note(tempPitch, quarterLength = item.duration.quarterLength)
					myNote.offset = item.offset
					tempNotes.append(myNote)
			else:
				tempNotes.append(item)
	return tempNotes

def get_pitches_in_span(myMeas, startOffset, sectionDuration):
	tempPitches = []
	if not myMeas:
		return tempPitches
	tempNotes = get_notes_in_span(myMeas, startOffset, sectionDuration)
	for myNote in tempNotes:
		tempPitch = myNote.pitch
		tempPitch.offest = myNote.offset
		tempPitches.append(tempPitch)
	return tempPitches

def remove_octaves(pitchList):
	letterSet = []
	outputSet = []
	for p in pitchList:
		if p.name not in letterSet:
			letterSet.append(p.name)
			outputSet.append(p)
	return outputSet

def do_section(startOffset, sectionDuration, literalOnly = False):
	global previousMeasure, workingMeasure, nextMeasure, look_for_sets_foundThird
	global possibleInterpretations
	outList = False
	doubt = -1
	tempPitches = []
	possibleInterpretations = []
	tempPitches = get_pitches_in_span(workingMeasure, startOffset, sectionDuration)
	if not tempPitches:
		return False
	tempPitches = sorted(tempPitches)
	tempPitches = remove_octaves(tempPitches)
	if len(tempPitches) == 1:
		return [3, tempPitches]										# found single pitches
	if look_for_sets(tempPitches):
		outList = [0, tempPitches]														# 0 = found complete chord
		"""possibleInterpretations.append([0, tempPitches])"""
		return outList												# comment out for slower but more thorough catalog
	elif look_for_sets_foundThird:
		outList = [2, tempPitches]
		"""possibleInterpretations.append([2, tempPitches])	"""			# ditto
		return outList															# 2 = found incomplete chord
	for interpretations in [find_subsets(tempPitches), find_subsets(tempPitches, bareThirdIntervals)]:
		if len(interpretations) >= 1 and not literalOnly:
			evaluate_reading(startOffset, sectionDuration, [x.pitchClass for x in interpretations[0]])
			myChord = interpretations[0]
			myDict = copy.deepcopy(reportDict)
			if reportDict['ROGUE'] == 0: 
				possibleInterpretations.append([myDict['PCTNHT'], restore_octaves(tempPitches, myChord)])
			for newReading in interpretations[1:]:
				evaluate_reading(startOffset, sectionDuration, [x.pitchClass for x in newReading])
				if reportDict['ROGUE'] == 0: 
					possibleInterpretations.append([reportDict['PCTNHT'], restore_octaves(tempPitches, newReading)])
				if compare_scores(myDict, reportDict) == 2:
					myDict = copy.deepcopy(reportDict)
					myChord = sorted(newReading)
			if myDict['ROGUE'] == 0 and not outList:
				outList = [myDict['PCTNHT'], restore_octaves(tempPitches, myChord)] 						# 1 = found chord amid passing tones
	if not outList:																					# ugly duplication; could clean up logic
		if look_for_sets(tempPitches, triadList = weirdChords):		
			return [0, tempPitches]
		interpretations = find_subsets(tempPitches, weirdChords)
		if len(interpretations) >= 1 and not literalOnly:
			evaluate_reading(startOffset, sectionDuration, [x.pitchClass for x in interpretations[0]])
			myChord = interpretations[0]
			myDict = copy.deepcopy(reportDict)
			if reportDict['ROGUE'] == 0: 
				possibleInterpretations.append([myDict['PCTNHT'], restore_octaves(tempPitches, myChord)])
			for newReading in interpretations[1:]:
				evaluate_reading(startOffset, sectionDuration, [x.pitchClass for x in newReading])
				if reportDict['ROGUE'] == 0: 
					possibleInterpretations.append([reportDict['PCTNHT'], restore_octaves(tempPitches, newReading)])
				if compare_scores(myDict, reportDict) == 2:
					myDict = copy.deepcopy(reportDict)
					myChord = sorted(newReading)
			if myDict['ROGUE'] == 0 and not outList:
				outList = [myDict['PCTNHT'], restore_octaves(tempPitches, myChord)]
	return outList

def restore_octaves(pitchList, trueChord):
	myNames = [x.name for x in trueChord]
	outList = []
	for p in pitchList:
		if p.name in myNames:
			outList.append(p)
	return outList

look_for_sets_foundThird = False

myTriads = (['P1', 'M3', 'P5'], ['P1', 'm3', 'P5'], ['P1', 'm3', 'd5'], ['P1', 'M3', 'P5', 'm7'], ['P1', 'm3', 'd5', 'd7'], ['P1', 'm3', 'd7'], ['P1', 'd5', 'd7'], ['P1', 'M3', 'm7'], ['P1', 'd3', 'D5', 'd7'])
myThirds = (['P1', 'M3'], ['P1', 'm3'], ['P1', 'P5'])			# adding fifth for cadences
weirdChords = (['P1', 'P5', 'm7'], ['P1', 'M3', 'A5'], ['P1', 'm3', 'P5', 'm7'])

def look_for_sets(pitchList, triadList = myTriads):
	global look_for_sets_foundThird
	letternameSet = set([])
	pitchSet = set([])
	look_for_sets_foundThird = False
	for item in pitchList:
		n = item.name
		if n in letternameSet: continue
		letternameSet.add(n)
		pitchSet.add(item)
	for basePitch in pitchSet:
		for subset in triadList:
			tempChord = [basePitch.transpose(x) for x in subset]
			tempChordNames = set([x.name for x in tempChord])
			if tempChordNames == letternameSet:
				return True
	for basePitch in pitchSet:
		for subset in myThirds:
			tempChord = [basePitch.transpose(x) for x in subset]
			tempChordNames = set([x.name for x in tempChord])
			if tempChordNames == letternameSet:
				look_for_sets_foundThird = True
				break
	return False

def chord_name(listOfPitches):
	if not listOfPitches:
		return 'False'
	c = chord.Chord(listOfPitches)
	n = c.commonName
	inversion = ''
	if n.count('triad') > 0 or n.count('interval class') > 0:
		if c.bass() == c.third:
			inversion = '6'
		elif c.bass() == c.fifth:
			inversion = '6/4'
	else:
		inversion = '7'
		if c.bass() == c.third:
			inversion = '6/5'
		elif c.bass() == c.fifth:
			inversion = '4/3'
		elif c.bass() == c.seventh:
			inversion = '2'
	if n == 'major triad':
		return c.root().name + inversion
	elif n == 'minor triad':
		return c.root().name.lower() + inversion
	elif n == 'diminished triad':
		return c.root().name.lower() + 'o' + inversion
	elif n == 'dominant seventh chord':
		return c.root().name + inversion
	elif n == 'diminished seventh chord':
		return c.root().name.lower() + 'o' + inversion
	else:
		return DT.nice_name(listOfPitches)

"""This routine checks how well a particular passage fits a particular harmonic interpretation."""

def evaluate_reading(startOffset, sectionDuration, proposedPCs, weightByDuration = True):
	global previousMeasure, workingMeasure, nextMeasure, reportDict, prevNote, nextNote, reportDict, myMeas, prevNote, nextNote
	reportDict = collections.defaultdict(lambda: 0)
	score = 1
	for i in range(len(workingMeasure)):
		myMeas = [workingMeasure[i]]
		tempPitches = get_notes_in_span(myMeas, startOffset, sectionDuration)
		for myNote in tempPitches:
			myPC = myNote.pitch.pitchClass
			if weightByDuration:
				score = myNote.duration.quarterLength
			if myPC in proposedPCs:
				position = proposedPCs.index(myPC)
				reportDict[position] += score
				reportDict['CHORDTONE'] += score
			else:
				if previousMeasure:
					prevNote = get_notes_in_span([previousMeasure[i]], previousMeasure[i].flat.highestTime - .02, previousMeasure[i].flat.highestTime - .01)
				else:
					prevNote = []
				if nextMeasure:
					nextNote = get_notes_in_span([nextMeasure[i]], 0, .01)
				else:
					nextNote = []
				prevIntervals = get_previous_intervals(myNote, i, absoluteValue = True)		# needs to chase backwards if there is a common tone
				nextIntervals = get_next_intervals(myNote, i, absoluteValue = True)			# ditto for chasing forwards
				#print myNote, prevIntervals, nextIntervals
				if (2 in prevIntervals or 1 in prevIntervals) and (1 in nextIntervals or 2 in nextIntervals):
					reportDict['SNP'] += score
				elif 2 in prevIntervals or (2 in nextIntervals or 1 in nextIntervals):		# leapt-to anticipation?
					reportDict['INC'] += score
				else:
					reportDict['ROGUE'] += score
	if reportDict['CHORDTONE'] != 0:
		reportDict['PCTNHT'] = (reportDict['SNP'] + reportDict['INC'] * 1.5 + reportDict['ROGUE'])/reportDict['CHORDTONE']
	else:
		reportDict['PCTNHT'] = 10000

"""Two recursive routines to get the intervals preceding and following a particular note.  If there is a common tone,
we recurse to just beyond the edge of the measure.

These are used to judge whether a note can be a passing tone, incomplete neighbor, etc."""

def get_previous_intervals(myNote, i, absoluteValue = True):
	intervalList = []
	recurseFlag = True
	if myNote.offset != 0:
		prevNotes = get_notes_in_span(myMeas, myNote.offset - .02, .01)
	else:
		prevNotes = prevNote
		recurseFlag = False
	for tempNote in prevNotes:
		x = process_intervals(tempNote, myNote)
		if absoluteValue: 
			x = abs(x)
		if recurseFlag and tempNote.pitch.midi == myNote.pitch.midi:
			return get_previous_intervals(tempNote, i, absoluteValue)
		intervalList.append(x)
	return intervalList

def get_next_intervals(myNote, i, absoluteValue = True):
	intervalList = []
	recurseFlag = True
	if myNote.offset + myNote.duration.quarterLength < myMeas[0].flat.highestTime:
		nextNotes = get_notes_in_span(myMeas, myNote.offset + myNote.duration.quarterLength + .01, .02)
	else:
		nextNotes = nextNote
		recurseFlag = False
	for tempNote in nextNotes:
		x = process_intervals(myNote, tempNote)
		if absoluteValue: 
			x = abs(x)
		if recurseFlag and tempNote.pitch.midi == myNote.pitch.midi:
			return get_next_intervals(tempNote, i, absoluteValue)
		intervalList.append(x)
	return intervalList

def process_intervals(tempNote, myNote):
	x = interval.notesToInterval(tempNote, myNote)
	y = x.generic.value
	x = x.name
	out = 3
	if x == 'P1':
		out = 1												# only perfect unison
	elif y == 1:
		out = 2												# 1 = augmented unison or regular second
	elif x == 'm2' or x == 'M2':
		out = 2
	elif y == 2:
		out = 3
	else:
		out = y
	return out

"""Given two scores for two different harmonic interpretations, prefer the one with no rogue notes, or fewer NHtones, or the most roots.
This probably needs some fixing."""

def compare_scores(dict1, dict2):
	if dict1['ROGUE'] > dict2['ROGUE']:					# first choose the reading with no rogue notes
		return 2
	elif dict1['ROGUE'] < dict2['ROGUE']:
		return 1
	if dict2['PCTNHT'] > dict1['PCTNHT']:
		return 1
	elif dict1['PCTNHT'] > dict2['PCTNHT']:
		return 2
	elif dict1[0] > dict2[0]:							# then the reading with the most roots ???
		return 1
	elif dict1[0] < dict2[0]:
		return 2
	return 0

def update_chord_frequencies(comp = 'Mozart'):
	import rn
	success = rn.load(comp)
	if not success:
		print("Need to analyze", comp, "before updating chord frequencies; try reloading the program later.")
		return
	rn.get_size(1)
	with open(DT._DATAPATH + 'classical_analyzer_data.pkl', 'wb') as storedFile:
		cf = copy.deepcopy(rn.working)
		rn.get_size(2)
		pf = copy.deepcopy(rn.working)
		for myDict in [cf, pf]:
			for i in [0, 1]:
				mySum = sum(myDict[i].values())
				for elem in myDict[i]:
					myDict[i][elem] = 100. * myDict[i][elem]/mySum
		pickle.dump(cf, storedFile)
		pickle.dump(pf, storedFile)
		
"""The following routines are copied (and lightly adapted) from my chorale analyzer"""

def make_key_list():
	global chorale, chords, progs, minDur, durationCounts, completeScales, offsets, choraleNumber, chordsPCs, lastAnalyzedKey, lastStrippedFig
	global totalVerticalities, unknownKey, unknownHarmonies, knownKey, knownHarmonies, vertsInCompleteScale, harmoniesInCompleteScale, globalPrintOut
	global lastOutputMeasure, storedString, lastTimeSignature, newTimeSignature, bestKeys, incompleteTriads, relativeMinors, globalRaw, globalSuperRaw
	
	"""FIRST PART: make several passes through the chorale to find consecutive regions belonging to the same scales.
	This should not be eliminated.
	
	Step 1A. First we go through the chords to find what scales they belong to, and create regions of consecutive unambiguous scalar membership."""
	
	completeScales = []															# first we find all scales containing each chord
	relativeMinors = []															# for later convenience
	for i in range(len(chords)):												
		completeScales.append(DT.scale_from_letternames(chords[i].pitches))
		relativeMinors.append([])
		for j in completeScales[i][0]:
			relativeMinors[i].append(pitch.Pitch(j).transpose('-m3').name)		# better to calculate just once
	bestScales = copy.deepcopy(completeScales)
	
	for i in range(1, len(chords)):												# go forward to find shared scales
		jointScales = shared_scales(bestScales[i-1], bestScales[i])
		if not is_empty(jointScales):
			bestScales[i-1]	= jointScales
			bestScales[i] = jointScales
	
	for i in reversed(range(1, len(chords))):									# now do it backward to eliminate ambiguities at the start
		jointScales = shared_scales(bestScales[i-1], bestScales[i])				# of a scalar block
		if not is_empty(jointScales):
			bestScales[i-1]	= jointScales
			bestScales[i] = jointScales
		
	"""Step 1B. Now we kill off all the ambiguities
	
	First we iterate backward from our ambiguous chord, until we get to the beginning of the piece or until we
	get out of the region sharing scales with the ambiguous chord.  
	
	For each of these preceding chords we then iterate the chords AFTER the ambiguous chord to try to make a complete region."""
		
	for i in range(len(chords)):		
		
		if not ambiguous(bestScales[i]): continue
		backwardJointScales = bestScales[i] = completeScales[i]
		foundMatch = False
		for j in reversed(range(0, i + 1)):						
			backwardJointScales = shared_scales(backwardJointScales, completeScales[j])
			if is_empty(backwardJointScales): break
			if not ambiguous(backwardJointScales):
				foundMatch = True
				bestScales[i] = backwardJointScales
				break
			forwardJointScales = copy.deepcopy(backwardJointScales)
			foundMatch = False
			for k in range(i+1, len(chords)):
				forwardJointScales = shared_scales(forwardJointScales, completeScales[k])
				if is_empty(forwardJointScales): break
				if not ambiguous(forwardJointScales):
					foundMatch = True
					bestScales[i] = forwardJointScales
					break
			if foundMatch: break
		if foundMatch:														# copy forward this solution to the next chords
			for j in range(i+1, len(chords)):
				if not ambiguous(bestScales[j]): break						# comment this out to extend our region forward as far as possible
				if not is_empty(shared_scales(bestScales[i], completeScales[j])):
					bestScales[j] = bestScales[i]
				else:
					break
	
	
	"""SECOND PART: Now having compiled a list of bestScales, we can identify the key more definitively
	building up our key list bestKeys.  This is still provisional, and needs to be fixed with chord-based modeling.
	In fact, a lot of the fussing with natural minor/major can probably be replaced.
	
	bestKeys is a list of lists, where each one records a key region:
			[keyLabel, labelStart, end, pivotRegionStart, isScaleComplete, [sharedScales {for ambiguous regions only}]]
	
	keyLabel is a key string like 'a', 'C', or '?'
	labelStart is the first chord at which we are definitively in that key
	end is the last chord we could be in that key
	pivotRegionStart is <= labelStart; it represents the earliest moment we could be in the key
	isScaleComplete is a boolean variable that tells us whether all the pitches of the scale appear
	
	There's some code in here that tries to guess between major and relative minor; it works OK, but is a little seat-of-the-pants.
	
	"""
	
	"""step 2A: assemble the list of keys"""
	
	bestKeys = []											
	lastKeyStr = ''
	lastAmbiguousScales = []
	
	for i in range(len(chords)):						
		if not ambiguous(bestScales[i]):
			lastAmbiguousScales = []
			if len(bestScales[i][0]) == 1: keyStr = list(bestScales[i][0])[0]
			elif len(bestScales[i][1]) == 1: keyStr = list(bestScales[i][1])[0].lower()
			elif len(bestScales[i][2]) == 1: keyStr = list(bestScales[i][2])[0].lower()
			elif len(bestScales[i][2]) == 0: keyStr = '?'	# if we can't do it, just make it C major!!!
			if not keyStr == lastKeyStr:
				bestKeys.append([keyStr, i, i, i, False])
				lastKeyStr = keyStr
			else:
				bestKeys[len(bestKeys) - 1][2] = i
		else:
			jointScales = shared_scales(lastAmbiguousScales, completeScales[i])
			lastKeyStr = '?'
			if not is_empty(jointScales):
				lastAmbiguousScales = jointScales
				bestKeys[len(bestKeys) - 1][2] = i
				bestKeys[len(bestKeys) - 1][5] = jointScales
			else:
				lastAmbiguousScales = completeScales[i]
				bestKeys.append(['?', i, i, i, False, lastAmbiguousScales])
	
	"""step 2B: assemble pivot regions"""
	for i in range(len(bestKeys)):				
		for j in reversed(range(0, bestKeys[i][1])):
			if bestKeys[i][0] != '?' and not is_empty(shared_scales(completeScales[j], bestScales[bestKeys[i][1]])):
				bestKeys[i][3]= j
			else:
				break
				
	"""step 2C: Check major keys for completeness and remove natural minor sandwiched between regular minor"""
	for i in range(len(bestKeys)):
		if bestKeys[i][0].isupper():												# major keys only
			totalPCs = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
			for j in range(bestKeys[i][3], bestKeys[i][2] + 1):
				for pc in chordsPCs[j]: totalPCs[pc] += 1
			if sum([x > 0 for x in totalPCs]) == 7: bestKeys[i][4] = True				# check major keys for completeness
			basePitch = pitch.Pitch(bestKeys[i][0])
			basePC = basePitch.pitchClass
			relativeName = basePitch.transpose('-m3').name.lower()
			prevRelative = i == 0 or (i > 0 and bestKeys[i-1][0] == relativeName)
			nextRelative = ((i < len(bestKeys) - 1) and bestKeys[i+1][0] == relativeName)		# i = end of array could cause errors depending on the interpreter!
			if (not prevRelative) or (not nextRelative): continue
			totalChords = bestKeys[i][2] + 1 - bestKeys[i][1]
			if totalChords <= 3: bestKeys[i][0] = relativeName
			elif totalChords <= 5:
				majorTD = totalPCs[basePC] + totalPCs[(basePC + 7) % 12]
				minorTD = totalPCs[(basePC + 4) % 12] + totalPCs[(basePC + 9) % 12]
				if minorTD * 3 > majorTD * 2: bestKeys[i][0] = relativeName	
	
	"""step 2D."""															
	consolidate_keys()
	
	"""step 2E"""		
	for i in range(len(bestKeys)):								# check minor keys for completeness
		if not bestKeys[i][0].islower(): continue
		totalPCs = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
		basePC = pitch.Pitch(bestKeys[i][0]).pitchClass
		for j in range(bestKeys[i][3], bestKeys[i][2] + 1):
			for pc in chordsPCs[j]: 
				totalPCs[pc] += 1
			if (totalPCs[basePC] > 0 and
			totalPCs[(basePC + 2)%12] > 0 and
			totalPCs[(basePC + 3)%12] > 0 and
			totalPCs[(basePC + 5)%12] > 0 and
			totalPCs[(basePC + 7)%12] > 0 and
			(totalPCs[(basePC + 8)%12] + totalPCs[(basePC + 9)%12]) > 0 and 
			(totalPCs[(basePC + 11)%12]) > 0):
				bestKeys[i][4] = True
	
	
	"""THIRD PART: now we try to resolve ambiguous regions, all of which depends on interpretation to some degree
	
	First, iterate through the ambiguous regions looking for the following situation: the chords could belong to
	natural minor and have just followed a minor key.  Then fix the reverse situation, where we have an ambiguous
	but potentially natural-minor passage preceding a minor key"""
	
	
	removeList = []
	for i in range(1, len(bestKeys)):									
		if bestKeys[i][0] != '?': continue
		fixedRegion = False
		prevKey = bestKeys[i-1][0]
		if prevKey == '?': continue
		if prevKey == prevKey.lower():									# previous key was minor
			targetKey = pitch.Pitch(prevKey).transpose('m3').name
			targetKey2 = prevKey.upper()
			for j in range(bestKeys[i][1], bestKeys[i][2] + 1):
				if targetKey in completeScales[j][0] or targetKey2 in completeScales[j][1] or targetKey2 in completeScales[j][2]:
					bestKeys[i-1][2] = j
					bestKeys[i][1] = j + 1
					fixedRegion = True
				else:
					break
			if bestKeys[i][1] > bestKeys[i][2]: removeList.append(bestKeys[i])		# in case we've wiped out an ambiguity
	for item in removeList: bestKeys.remove(item)			

	removeList = []
	for i in range(0, len(bestKeys) - 1):			
		if bestKeys[i][0] != '?': continue
		nextKey = bestKeys[i+1][0]
		if nextKey == '?': continue
		if nextKey == nextKey.lower():									# next key is minor
			targetKey = pitch.Pitch(nextKey).transpose('m3').name
			targetKey2 = nextKey.upper()
			for j in reversed(range(bestKeys[i][1], bestKeys[i][2] + 1)):
				if targetKey in completeScales[j][0] or targetKey2 in completeScales[j][1] or targetKey2 in completeScales[j][2]:
					bestKeys[i+1][1] = j
					bestKeys[i][2] = j - 1
				else:
					break
			if bestKeys[i][1] > bestKeys[i][2]: removeList.append(bestKeys[i])		# in case we've wiped out an ambiguity
	for item in removeList: bestKeys.remove(item)

	"""Now a second correction.  If the chorale opens with ambiguous regions that COULD be the same as the final labeled region of the piece, we 
	assume that they are in the final key.  This section can be commented out for a rawer analysis"""

	"""CHECK OUT #45 to figure out how to do this right!"""

	if bestKeys[0][0] == '?':
		for i in reversed(range(len(bestKeys))):
			if bestKeys[i][0] != '?':					# last labeled region (to deal with picardy endings in minor, or half-cadences)
				finalKeyStr = bestKeys[i][0]
				break
		for i in range(len(bestKeys)):
			if bestKeys[i][0] != '?': break
			totalScales = bestKeys[i][5]
			if finalKeyStr[0].isupper():			
				if finalKeyStr in totalScales[0]:
					bestKeys[i][0] = finalKeyStr
					bestKeys[i][4] = False
				elif finalKeyStr in totalScales[1] or finalKeyStr in totalScales[2] or pitch.Pitch(finalKeyStr).transpose('m3').name in totalScales[0]:
					bestKeys[i][0] = finalKeyStr.lower()
					bestKeys[i][4] = False
				elif ord(finalKeyStr[0]) >= 97 and ord(finalKeyStr[0]) <= 103:
					if finalKeyStr.upper() in totalScales[1] or finalKeyStr.upper() in totalScales[2] or pitch.Pitch(finalKeyStr).transpose('m3').name in totalScales[0]:
						bestKeys[i][0] = finalKeyStr
						bestKeys[i][4] = False
	
	
	"""THIRD CORRECTION: if we find a region with a lurking V7 or with an isolated major chord, we label accordingly
	This could be fixed -- there's a neapolitan that creates a very strange modulation.  Or fix at a later stage"""

	for i in range(0, len(bestKeys) - 1):			
		if bestKeys[i][0] != '?': continue
		keyStr = bestKeys[i]
		foundFirstChord = False
		for j in range(keyStr[1], keyStr[2] + 1):
			myChord = chords[j]
			if myChord.isAugmentedSixth():
				#print "FOUND AUGMENTED SIXTH!!!!"
				bestKeys[i][0] = myChord.bass().transpose('M3').name.lower()
			if myChord.isTriad() or myChord.isSeventh() or incomplete_seventh(myChord): 
				if myChord.isMajorTriad() and not foundFirstChord:
					possibleTonic = myChord.root().transpose('-P5').name
					if possibleTonic in keyStr[5][0]:
						measureBeat = measuresAndBeats[j]
						#print "     FOUND ISOLATED MAJ TRIAD, CHORALE ", n, 'measure', measureBeat[0], 'beat', measureBeat[1]
						keyStr[0] = possibleTonic
						break
					elif possibleTonic in keyStr[5][1] or possibleTonic in keyStr[5][2]:
						measureBeat = measuresAndBeats[j]
						#print "     FOUND ISOLATED MAJ TRIAD, CHORALE ", n, 'measure', measureBeat[0], 'beat', measureBeat[1]
						keyStr[0] = possibleTonic.lower()
						break
				foundFirstChord = True
				if myChord.isDominantSeventh() or incomplete_seventh(myChord):
					possibleTonic = myChord.root().transpose('-P5')
					minorThird = (possibleTonic.pitchClass + 3) % 12
					majorThird = (possibleTonic.pitchClass + 4) % 12
					possibleTonic = possibleTonic.name
					for k in range(j+1, len(chords)):
						foundIt = False
						if minorThird in chordsPCs[k]:
							keyStr[0] = possibleTonic.lower()
							foundIt = True
						if majorThird in chordsPCs[k]:
							keyStr[0] = possibleTonic
							foundIt = True
						if foundIt:
							measureBeat = measuresAndBeats[j]
							break
	
	"""ORPHAN CLEANUP"""
	
	"""print "before orphan cleanup"
	for item in bestKeys: print item"""
	
	keyTotals = {}
	
	for item in bestKeys: 
		myName = item[0]
		keyTotals[myName] = keyTotals.setdefault(myName, 0) + item[2] + 1 - item[1]
	
	keysBySize = sorted(keyTotals.keys(), key = lambda x: keyTotals[x], reverse=True)
	
	globalTonic = keysBySize[0]
	
	if len(keysBySize) > 1:
		secondKey = keysBySize[1]
	else:
		secondKey = False
	
	if globalPrintOut: print("GLOBAL TONIC", globalTonic, "SECOND KEY", secondKey)
	
	if bestKeys[0][0] != globalTonic:
		print("doesn't start in tonic?")
		bestKeys[0][0] = globalTonic
	elif bestKeys[-1][0] != globalTonic:
		print("doesn't end in tonic?")
		bestKeys[-1][0] = globalTonic
	
	remove_keychanges(globalTonic)
	
	if secondKey:
		remove_keychanges(secondKey)

	for i in range(0, len(bestKeys) - 1):
		chordsSinceTonic = 0
		thirdKeyArea = bestKeys[i][0]
		if bestKeys[i][0] in [globalTonic, secondKey, '?:'] or (bestKeys[i][2] + 1 - bestKeys[i][1]) < 10: continue
		for j in range(i+1, len(bestKeys)):
			if bestKeys[j][0] == thirdKeyArea:
				for k in range(i+1, j):
					bestKeys[k][0] = thirdKeyArea
			chordsSinceTonic += bestKeys[j][2] + 1 - bestKeys[j][1]
			if chordsSinceTonic > 3: break
	
	for i in range(1, len(bestKeys)):
		if bestKeys[i][0] == '?':
			bestKeys[i][0] = bestKeys[i-1][0] 
	
	
	"""print "after orphan cleanup"
	for item in bestKeys: print item"""
	
	consolidate_keys()
	
	"""FOR DEBUGGING PURPOSES, TRY UNCOMMENTING THE FOLLOWING LINES, WHICH SHOW THE CHORDS AND THE BESTKEYS"""
	
	
	
	"""for i in range(len(chords)): 						# use this to see the complete list of chords and the best keys list
		pitchList = [x.name for x in chords[i]]
		print i, pitchList, chords[i].offset
		
	for item in bestKeys: print item
	print "*****STILL IN MAIN ROUTINE*****"
	"""

def remove_keychanges(targetKey, maxChords = 12):
	for i in range(0, len(bestKeys) - 1):
		chordsSinceTarget = 0
		if bestKeys[i][0] != targetKey: continue
		for j in range(i+1, len(bestKeys)):
			if bestKeys[j][0] == targetKey:
				for k in range(i+1, j):
					bestKeys[k][0] = targetKey
			chordsSinceTarget += bestKeys[j][2] + 1 - bestKeys[j][1]
			if chordsSinceTarget > maxChords: break

def zero_default(myDict, item):
	if item in myDict:
		return item
	else:
		return 0

def analyze_key_area(i):
	
	"""TO DO: remove colons from key names!!!!!!"""
	
	global chorale, chords, progs, minDur, durationCounts, completeScales, offsets, choraleNumber, chordsPCs, lastAnalyzedKey
	global totalVerticalities, unknownKey, unknownHarmonies, knownKey, knownHarmonies, vertsInCompleteScale, harmoniesInCompleteScale
	global lastOutputMeasure, storedString, lastTimeSignature, newTimeSignature, bestKeys, incompleteTriads, lastStrippedFig, globalRaw, globalSuperRaw, doUnisons
	global totalKeys
	myKey = lastAnalyzedKey
	keyStr = bestKeys[i]
	ambiguousRegionStart = -1
	
	if keyStr[0] == '?':													# first we deal with ambiguous regions
		
		ambiguousRegionStart = keyStr[1]									# need these because we'll be messing around with keyStr
		ambiguousRegionEnd = keyStr[2]										# in the Picardy section.
		
		"""The following passage deals with Picardy-Third type situations: a major chord where a minor would be expected.
		If the previous key was minor and ended with a dominant chord, and if the ambiguous region starts with chords that fit
		in the parallel major, we analyze them in that key.
		
		This involves a certain amount of music-theoretical interpretation.  COMMENT IT OUT for a purer analysis."""
		
		if myKey: potentialKey = myKey.tonic.name
		
		
		if myKey and myKey.mode == 'minor' and (potentialKey in completeScales[keyStr[1]][0]) and (lastStrippedFig == 'V' or lastStrippedFig[0:3] == 'vii'):
			keyStr[0] = myKey.tonic.name.upper()				# PICARDY SITUATION
			ambiguousRegionStart = keyStr[1] + 1
			keyStr[2] = keyStr[1]
			for j in range(keyStr[1] + 1, ambiguousRegionEnd + 1):
				if potentialKey in completeScales[j][0]:
					keyStr[2] = j
					ambiguousRegionStart = j + 1
					break
			
	"""We record ambiguous regions last; first we do the unambiguous stuff."""
			
	"""NOTE: the following if cannot be an ELSE or ELIF because keyStr[0] can be changed above"""
	
	
	if keyStr[0] != '?':										# now the case where we know the key;
			
		completeScale = keyStr[4]
		displayedKey = False
		myKey = key.Key(keyStr[0])
		modeInd = int(myKey.mode == 'minor')
		if measuresAndBeats[keyStr[1]][0] not in totalKeys:
			totalKeys[measuresAndBeats[keyStr[1]][0]] = []
		totalKeys[measuresAndBeats[keyStr[1]][0]].append([measuresAndBeats[keyStr[1]][1], myKey])
		lastAnalyzedKey = myKey
		lastFig = ''
		lastStrippedFig = ''
		lastDur = None
		lastIndex = -1
		
		"""In the following, there is some legacy code from when I used to record roman numerals in this program 
			
		This could be reinstated or cleaned up someday
		"""
		
		for j in range(keyStr[1], keyStr[2] + 1):
			myChord = chords[j]
			myDur = chords[j].duration.quarterLength
			myOffset = offsets[j]
			#print myOffset
			measureBeat = measuresAndBeats[j]
			# newTimeSignature = beatAndMeasure[1].getContextByClass('TimeSignature').ratioString		# NEED TO FIX THIS
			beatFrac = measureBeat[1] - int(measureBeat[1])
			recordIt = True
			
			myNF = DT.normal_form([x.midi for x in myChord.pitches])
			
			fig2 = ''
			
			
			if myNF in [[0, 4], [0, 3]]:
				chord2 = chord.Chord(list(myChord.pitches) + [myChord.root().transpose('M6')])
				fig2 = DT.figure_from_chord(chord2, myKey)
				chord3 = chord.Chord(list(myChord.pitches) + [myChord.root().transpose('m6')])
				fig3 = DT.figure_from_chord(chord3, myKey)
			
			if recordIt:
				myFig = DT.figure_from_chord(myChord, myKey)
				if myFig == False: 
					#print keyStr[0], j, chordList[j]
					continue
				
				if fig2:
					myFig = max([myFig, fig2, fig3], key = lambda x: chordFrequencies[modeInd][x])		# return the most likely reading of a third
				
				"""if myFig == 'viio' and len(set(chordsPCs[j])) <= 2 and myChord.third:
					myFig = 'V6'"""
				#myStrippedFig = strip_roman(myFig).replace('maj','')
				parsedFig = DT.parse_figure(myFig)
				myStrippedFig = parsedFig[0].replace('maj','')	# get rid of major seventh
				
				if myChord.expressions: 
					myFig = myFig + ' ||'
				theString = keyStr[0] + ': ' + myFig
				outputFile(j, measureBeat, theString)
				lastFig = myFig
				lastStrippedFig = myStrippedFig
				lastDur = myDur
				lastOffset = myOffset
				lastIndex = j
				
	"""Now finally we record the ambiguous regions."""
	
	if ambiguousRegionStart != -1 and (ambiguousRegionStart <= ambiguousRegionEnd):
		recordedRegion = False	
		for j in range(ambiguousRegionStart, ambiguousRegionEnd + 1):
			myChord = chords[j]
			myOffset = myChord.offset
			measureBeat = measuresAndBeats[j]
			if myChord.isTriad() or myChord.isSeventh() or incomplete_seventh(myChord):
				if not recordedRegion:
					theString = '?:'
					outputFile(j, measureBeat, theString)
					recordedRegion = True

def outputFile(chordNo, measureBeat, theString):
	global chordList, measureBeats
	chordList[chordNo] = theString
	measureBeats[chordNo] = measureBeat
	return

"""def rewrite_key(target, myKey_name):
	global bestKeys
	ambiguityFlag = False
	if myKey_name[0] == '?': return					# ignore an incoherent request!
	if bestKeys[target][0] != '?':
		bestKeys[target][0] = myKey_name
		myKey = key.Key(myKey_name)
		for j in range(bestKeys[target][1], bestKeys[target][2] + 1):
			if j not in chordList: continue
			if chordList[j][0] == '?': 
				ambiguityFlag = True
				break
			oldData = chordList[j].split()
			if len(oldData) == 1:
				ambiguityFlag = True
				break
			figureInNewKey = DT.figure_from_chord(roman.RomanNumeral(oldData[1], oldData[0].rstrip(':')).pitches, myKey)
			oldData[0] = myKey_name + ':'
			oldData[1] = figureInNewKey
			chordList[j] = ' '.join(oldData)
	if ambiguityFlag or bestKeys[target][0] == '?':			# need to reanalyze
		bestKeys[target][0] = myKey_name
		lastAnalyzedKey = False
		lastStrippedFig = ''
		analyze_key_area(target)"""

def final_write():
	global lastOutputMeasure, storedString, lastKey, lastMeasure, lastString, outputAnalysis, lastTimeSignature, newTimeSignature, chorale, chords
	global chordList, measureBeats, m0leftPad
	forbidden = ['?:', ' #7', " #4/3", " #6/5", " #", ' 7', ' 6', ' 6/4', ' 6/5', ' 4/3', ' 2']
	jValues = sorted(list(chordList))
	lastString = ''
	lastMeasure = -1
	lastTimeSignature = ''
	lastKey = ''
	storedString = ''
	lastOutputMeasure = -1
	
	if m0leftPad and measureBeats[0][0] == 1:					# there is a pickup, but we couldn't label it
		measureBeats[0][0] = 0
	
	for chordNo in jValues:
		theString = chordList[chordNo]
		bailOut = False
		for item in forbidden:
			if theString.count(item) > 0:
				bailOut = True
		if bailOut: continue
		measureBeat = measureBeats[chordNo]
		mNum = measureBeat[0]
		bNum = measureBeat[1]
		if mNum == 0:
			bNum += m0leftPad
		# FIX THIS!!!!
		newTimeSignature = '' # chorale.beatAndMeasureFromOffset(chords[chordNo].offset)[1].getContextByClass('TimeSignature').ratioString
		if theString == lastString and mNum == lastMeasure: continue
		else: 
			lastString = theString
		stringSplit = theString.split()
		if len(stringSplit) < 2:									# sometimes you just get a key with no chord
			continue
		lastMeasure = mNum
		for i in range(len(stringSplit)):
			if stringSplit[i].count(':') > 0:
				stringSplit[i] = stringSplit[i].replace('-','b')
				if stringSplit[i] == lastKey:
					stringSplit = stringSplit[:i] + stringSplit[i+1:]
					break
				else:
					lastKey = stringSplit[i]
		if bNum == 1: 
			beatStr = ''
		else:
			beatStr = 'b' + str(bNum)
			if beatStr[-2:] == '.0': beatStr = beatStr[:-2]
			if beatStr.endswith('6665') or beatStr.endswith('6667'):
				bSplit = beatStr.split('.')
				bSplit[-1] = '67'
				beatStr = '.'.join(bSplit)
			if beatStr.endswith('3333'):
				bSplit = beatStr.split('.')
				bSplit[-1] = '33'
				beatStr = '.'.join(bSplit)
		if mNum == lastOutputMeasure:
			newStr = ' '
			newStr = newStr.join(stringSplit)
			newStr = beatStr + ' ' + newStr
			storedString = storedString + ' ' + newStr
		else:
			if storedString:
				outputAnalysis.write(storedString + '\n')
				if globalPrintOut: print(storedString)
			lastOutputMeasure = mNum
			newStr = ' '
			newStr = newStr.join(stringSplit)
			if beatStr == '':
				storedString = 'm' + str(mNum) + ' ' + newStr
			else:
				storedString = 'm' + str(mNum) + ' ' + beatStr + ' ' + newStr
		if mNum in timeSignatures and timeSignatures[mNum].ratioString != lastTimeSignature:
			lastTimeSignature = timeSignatures[mNum].ratioString
			outputAnalysis.write('Time Signature: ' + lastTimeSignature + '\n')
			if globalPrintOut: print('Time Signature: ' + lastTimeSignature)
			
	if storedString:
		outputAnalysis.write(storedString + '\n')
		if globalPrintOut: print(storedString)
	


def shared_scales(scaleList1, scaleList2):
	outputList = []
	for i in range(len(scaleList1)):
		outputList.append(scaleList1[i] & scaleList2[i])
	return outputList

def ambiguous(scaleList):
	if len(scaleList[0]) > 1: return True												# multiple diatonics
	elif len(scaleList[1] | scaleList[2]) > 1: return True								# multiple tonics for harm min and mel min
	elif len(list(scaleList[0]) + list(scaleList[1] | scaleList[2])) > 1: return True	# ambiguous between maj and min
	else: return False

def is_empty(scaleList):
	totalScales = set([])
	for i in range(len(scaleList)):
		totalScales = totalScales | scaleList[i]
	if totalScales:
		return False
	else:
		return True

def consolidate_keys():
	global bestKeys
	for i in reversed(range(1, len(bestKeys))):
		if bestKeys[i][2] < bestKeys[i][1]:											# delete regions that end before they start!
			bestKeys.remove(bestKeys[i])
			continue
		if bestKeys[i][0] == bestKeys[i-1][0] and bestKeys[i][0] != '?':
			bestKeys[i-1][2] = bestKeys[i][2]
			bestKeys.remove(bestKeys[i])

"""Note: do not casually mess with the following routine -- it's needed for key-finding, etc."""

def incomplete_seventh(myChord):					# currently only records incomplete DOMINANT sevenths (by design)
	nf = DT.normal_form(set([x.pitchClass for x in myChord.pitches]))
	if nf == [0, 2, 6] or nf == [0, 3, 6]:
		return True
	else:
		return False
		
try:
	with open(DT._DATAPATH + 'classical_analyzer_data.pkl', 'rb') as storedFile:
		if not storedFile: 
			print("ERROR! can't find stored chord-frequency data!!!")
		cf = pickle.load(storedFile)
		for i in [0, 1]:
			chordFrequencies.append(collections.defaultdict(lambda: 0, cf[i]))
		pf = pickle.load(storedFile)
		for i in [0, 1]:
			progressionFrequencies.append(collections.defaultdict(lambda: 0, pf[i]))
except:
	print('Trying to remake stored chord frequencies ...')
	update_chord_frequencies()
