from __future__ import print_function
import pickle, os, time, re, collections, copy
from music21 import *
import DT

#twoPartComposers = ['Josquincomplete', 'LassusDuets', "BachInventions"]

_THISFILE = 'CPT2'

dates = {'OCKEGHEM': 1410, 'JOSQUINCOMPLETE': 1450, 'PALRAW': 1525, 'LASSUSDUETS': 1525, 'WTC': 1680, 'BACHINVENTIONS': 1680, 'BACH': 1680}
avoid = ['GOMBERT', 'MONTEVERDI', 'CLEMENS', 'BACH'] # very little data here

twoPartComposers = [x for x in os.listdir(DT._DATAPATH) if x.count(_THISFILE) > 0]
twoPartComposers = sorted(list(set([x.replace(_THISFILE,'').replace('no6', '').replace('.pkl', '') for x in twoPartComposers])), key = lambda x: dates.get(x, 0))

for c in avoid:
	if c in twoPartComposers:
		twoPartComposers.remove(c)

globalChromatic = False

"""look at the internal logic of two-part counterpoint

Using CENTER OF MASS coordinates 1, 3, 5 to record the size of the intervals (6 = 3)

this program is kind of a mess, sorry about that

totalMotions contains all the sonorities found; it has a variety of unrelated pieces of data in it, categorized by the number of spaces in the key

	single vertical intervals	no space
	pairs of vertical intervals	one space
	pairs of vertical intervals plus horizontal intervals (bass voice, top voice)	four space

You can use the getall command to query the data using regular expressions; this will work but it is tricky to get the formatting right.

TODO: 
	
	catch ancitipations
	check why there is one Palestrina cambiata treated as a QC?

"""

def gather_stats(source = 'BachInventions'):
	global thepiece, pieceNumber, theNotes, theInts, theMotions, totalMotions, globalSource, locations, globalSixths, bigString, allInts
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic, nhtDict, nhtLocations, filenames, timeSigs
	globalSixths = True
	source = source.upper()
	globalSource = source
	suffix = ''
	if source not in DT._PATHS:
		print("SOURCE NOT FOUND!")
		return
	analysisPath = DT._PATHS[source][0]
	scorePath = DT._PATHS[source][1]
	files = [x for x in os.listdir(scorePath) if (x.count('.xml') + x.count('.krn')) > 0]
	startT = time.time()
	totalMotions = {}
	chromaticMotions = {}
	locations = collections.defaultdict(lambda: [])
	chromaticLocations = collections.defaultdict(lambda: [])
	filenames = {}
	allInts = {}
	chromaticInts = {}
	nhtDict = {}
	nhtLocations = collections.defaultdict(lambda: [])
	pieceNumber = 0
	for i, myFile in enumerate(files):
		if myFile[-3:] in ['xml', 'krn']:
			tempName = scorePath + myFile
			try:
				thepiece = converter.parse(tempName)
			except Exception as e:
				print("ERROR ERROR ERROR", str(e))
				continue
			if globalSource == 'BACH':
				newpiece = stream.Score()
				newpiece.insert(thepiece.parts[0])
				newpiece.insert(thepiece.parts[-1])
				thepiece = newpiece
			if 'Opus' in thepiece.classes:
				continue
			pieceNumber += 1
			timeSigs = thepiece.flat.getTimeSignatures()
			if len(thepiece.parts) != 2:
				print('File',i, 'of ', len(files), myFile)
				check_for_two_part_passages()
				"""try:
					check_for_two_part_passages()
				except Exception, e:
					print "ERROR ERROR ERROR", str(e)"""
			else:
				print('File',i, 'of ', len(files), myFile, " two-part piece")
				dofile()
				"""try:
					dofile()
				except Exception, e:
					print "ERROR ERROR ERROR", str(e)"""
			filenames[pieceNumber] = myFile
	order_voices()
	with open(DT._DATAPATH + _THISFILE + globalSource + suffix +'.pkl', 'wb') as outputFile:
		pickle.dump(totalMotions, outputFile)
		pickle.dump(dict(locations), outputFile)
		pickle.dump(filenames, outputFile)
		pickle.dump(allInts, outputFile)
		pickle.dump(chromaticMotions, outputFile)
		pickle.dump(dict(chromaticLocations), outputFile)
		pickle.dump(chromaticInts, outputFile)
		pickle.dump(nhtDict, outputFile)
		pickle.dump(dict(nhtLocations), outputFile)
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	print("\a")
	return

def diatonic_to_steps(i):
	if i > 0:
		return i - 1
	if i < 0:
		return i + 1

def average_motions():
	get(4, '.*  ', printIt = False)
	totalVLmotions = [{}, {}]
	for item in working:
		vlMotions = [int(x) for x in item.split('  ')[1].split()]
		vlMotions = [abs(diatonic_to_steps(x)) for x in vlMotions]
		for i, vl in enumerate(vlMotions):
			totalVLmotions[i][vl] = totalVLmotions[i].setdefault(vl, 0) + working[item]
	for i, myDict in enumerate(totalVLmotions):
		motions = 0
		distance = 0
		for j in myDict:
			motions += myDict[j]
			distance += myDict[j] * j
		print("VOICE", i, 1.*distance/motions)

def get_beat_strength(myOffset):
	global thepiece, timeSigs
	measure = thepiece.parts[0].getElementsByOffset(myOffset + .01, myOffset + .01, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure'])
	if not measure:
		return 0
	mStart = measure[0].offset
	mOffset = myOffset - mStart
	timeSig = timeSigs.getElementAtOrBefore(myOffset)
	return timeSig.getAccentWeight(mOffset % timeSig.barDuration.quarterLength, forcePositionMatch = True)

def check_for_two_part_passages():
	global thepiece, flatparts, theNotes, bigString, allInts, chromaticString
	global chromaticMotions, chromaticLocations, chromaticInts
	offsets = []
	bigString = ''
	chromaticString = ''
	theNotes = {}
	flatparts = []
	totalPassages = 0
	totalNotes = 0
	for p in thepiece.parts:
		flatparts.append(p.flat.getElementsByClass(['Note', 'Rest']).stream())
		offsets += [x.offset for x in flatparts[-1]]
	offsets = sorted(list(set(offsets)))
	for o in offsets:
		temp = []
		for p in flatparts:
			tempNote = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False, classList = ['Note', 'Rest'])
			if (not tempNote) or ('Rest' in tempNote[0].classes):
				temp.append(False)
			else:
				temp.append(tempNote[0])
		theNotes[o] = temp
	i = 0
	while i < len(offsets):
		o = offsets[i]
		if note_count(theNotes[o]) == 2:
			myParts = parts_with_notes(theNotes[o])
			j = i + 1
			while j < len(theNotes):
				if (note_count(theNotes[offsets[j]]) != 2) or parts_with_notes(theNotes[offsets[j]]) != myParts:
					break
				j += 1
			if j - i > 5:
				totalPassages += 1
				totalNotes += j - i
				check_section(offsets[i:j], myParts)
				i = j
		i += 1
	if totalPassages != 0: print(" Found ", totalPassages, "passages with", totalNotes, "notes")
	allInts[pieceNumber] = bigString
	chromaticInts[pieceNumber] = chromaticString

def dofile():
	global thepiece, pieceNumber, flatparts, theNotes, bigString, chromaticString
	global chromaticMotions, chromaticLocations, chromaticInts
	offsets = []
	theNotes = {}
	flatparts = []
	bigString = ''
	chromaticString = ''
	for p in reversed(thepiece.parts):
		flatparts.append(p.flat.getElementsByClass(['Note', 'Rest']).stream())
		offsets += [x.offset for x in flatparts[-1]]
	offsets = sorted(list(set(offsets)))
	for o in offsets:
		temp = []
		for p in flatparts:
			tempNote = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False, classList = ['Note', 'Rest'])
			if (not tempNote) or ('Rest' in tempNote[0].classes):
				temp.append(False)
			else:
				temp.append(tempNote[0])
		theNotes[o] = temp
	check_section(offsets, [0, 1])
	allInts[pieceNumber] = bigString
	chromaticInts[pieceNumber] = chromaticString
		
def check_section(offsets, myParts):
	global thepiece, flatparts, totalMotions, locations, pieceNumber, theNotes, bigString
	global chromaticMotions, chromaticLocations, chromaticInts, chromaticString
	theInts = {}
	theMotions = {}
	cMotions = {}
	consonantOffsets = []
	chromaticConsonantOffsets = {}
	for o in offsets:
		theInts[o] = vertical_interval(theNotes[o][myParts[0]], theNotes[o][myParts[1]])
		if theInts[o]:
			midiSize = abs(theNotes[o][myParts[0]].pitch.midi - theNotes[o][myParts[1]].pitch.midi)
			reducedInt = normal_numbering(theInts[o])
		else:
			midiSize = False
			reducedInt = False
		if (reducedInt in [0, 2, 5, False]) or (reducedInt == 4 and ((midiSize % 12) == 7)):
			consonantOffsets.append(o)
			chromaticConsonantOffsets[o] = midiSize
	
	"""NOW RECORD NONHARMONIC TONES"""
	#dissonantOffsets = sorted(list(set(offsets) - set(consonantOffsets)))
	maxLen = len(offsets) - 1
	for i, o in enumerate(offsets[:-1]):
		if i == 0: continue
		if o in consonantOffsets: continue
		if offsets[i-1] not in consonantOffsets:
			
			print("Double dissonance", o)							# only record the second of a DD pair (to prevent double counting)
			nhtDict['DD'] = nhtDict.setdefault('DD', 0) + 1
			nhtLocations['DD'].append([pieceNumber, o])
			
			"""WRITE SOMETHING TO CHECK THESE"""
			
			continue
		if offsets[i+1] not in consonantOffsets: continue			# only categorize a dissonance surrounded by consonances
		h = []
		for j in myParts:
			"""try:
				print "OFFSET", o, "VOICE ", j, theNotes[offsets[i-1]][j].nameWithOctave, theNotes[offsets[i]][j].nameWithOctave,theNotes[offsets[i+1]][j].nameWithOctave
			except:
				pass"""
			h.append([horizontal_interval(theNotes[offsets[i-1]][j], theNotes[offsets[i]][j]), horizontal_interval(theNotes[offsets[i]][j], theNotes[offsets[i+1]][j])])
		foundIt = False
		
		for j in [0, 1]:
			if (h[j] == [2, 2] or h[j] == [-2, -2]) and 1 in h[1-j]:
				#print "PASSING TONE"
				theNotes[offsets[i]][myParts[j]].lyric = 'P'
				foundIt = True
				nhtDict['P'] = nhtDict.setdefault('P', 0) + 1
				break
			elif h[j] == [-2, 2] and 1 in h[1-j]:
				#print "NEIGHBOR TONE"
				theNotes[offsets[i]][myParts[j]].lyric = 'Nd'
				nhtDict['Nd'] = nhtDict.setdefault('Nd', 0) + 1
				nhtLocations['Nd'].append([pieceNumber, o])
				foundIt = True
				break
			elif h[j] == [2, -2] and 1 in h[1-j]:
				#print "NEIGHBOR TONE"
				theNotes[offsets[i]][myParts[j]].lyric = 'Na'
				nhtDict['Na'] = nhtDict.setdefault('Na', 0) + 1
				nhtLocations['Na'].append([pieceNumber, o])
				foundIt = True
				break
			elif (h[j] == [1, -2]) and (h[1-j][-1] == 1):								# CATCH ANTICIPATIONS
				if (get_beat_strength(offsets[i]) >= get_beat_strength(offsets[i+1])):      #and (offsets[i+1] - offsets[i] <= offsets[i] - offsets[i-1]):
					myStr = str(theInts[offsets[i]]) + '-' + str(theInts[offsets[i+1]]) 
					theNotes[offsets[i]][myParts[j]].lyric = 'S'
					nhtDict[myStr] = nhtDict.setdefault(myStr, 0) + 1
					nhtLocations[myStr].append([pieceNumber, o])
					foundIt = True
					break
			elif (h[j] == [-2, 1]) and (get_beat_strength(offsets[i]) < get_beat_strength(offsets[i+1])) and (theNotes[offsets[i]][myParts[j]].offset != theNotes[offsets[i+1]][myParts[j]].offset):	
				theNotes[offsets[i]][myParts[j]].lyric = 'A'
				nhtDict['A'] = nhtDict.setdefault('A', 0) + 1
				nhtLocations['A'].append([pieceNumber, o])
				foundIt = True
				break
			elif (h[j] == [1, -3]) and (h[1-j][-1] == 1) and i+2 <= maxLen:				# need to check rhythm here
				newInt = horizontal_interval(theNotes[offsets[i+1]][myParts[j]], theNotes[offsets[i+2]][myParts[j]])
				newInt2 = horizontal_interval(theNotes[offsets[i+1]][myParts[1-j]], theNotes[offsets[i+2]][myParts[1-j]])
				if newInt == 2 and newInt2 == 1 and (get_beat_strength(offsets[i]) >= get_beat_strength(offsets[i+2])): #and (offsets[i+2] - offsets[i] <= offsets[i] - offsets[i-1]):
					myStr = str(theInts[offsets[i]]) + '-D1-' + str(theInts[offsets[i+2]]) 
					theNotes[offsets[i]][myParts[j]].lyric = 'DS1'
					nhtDict[myStr] = nhtDict.setdefault(myStr, 0) + 1
					nhtLocations[myStr].append([pieceNumber, o])
					foundIt = True
					break
			elif (h[j] == [1, -2]) and (h[1-j][-1] != 1):								# need to check rhythm here
				newInt = vertical_interval(theNotes[offsets[i+1]][myParts[j]], theNotes[offsets[i]][myParts[1-j]])
				if newInt in [1, 3, 5, 6, 8, 10, 12, 13, 15] and (get_beat_strength(offsets[i]) >= get_beat_strength(offsets[i+1])): #and (offsets[i+1] - offsets[i] >= offsets[i] - offsets[i-1]):
					myStr = str(theInts[offsets[i]]) + '-L-' + str(newInt) 
					theNotes[offsets[i]][myParts[j]].lyric = 'S'
					nhtDict[myStr] = nhtDict.setdefault(myStr, 0) + 1
					nhtLocations[myStr].append([pieceNumber, o])
					foundIt = True
					break
			elif (h[j][0] == -2 and h[j][1] >= 3) and (h[1-j][0] == 1):					# step down, leap up
				theNotes[offsets[i]][myParts[j]].lyric = 'E'
				nhtDict['E'] = nhtDict.setdefault('E', 0) + 1
				nhtLocations['E'].append([pieceNumber, o])
				foundIt = True
				break
			elif (h[j][0] == 2 and h[j][1] <= -3) and (h[1-j][0] == 1):					# step up, leap down
				theNotes[offsets[i]][myParts[j]].lyric = 'E2'
				nhtDict['E2'] = nhtDict.setdefault('E2', 0) + 1
				nhtLocations['E2'].append([pieceNumber, o])
				foundIt = True
				break
			elif (h[j] == [-2, -3]):
				k = i + 2
				while k <= maxLen:
					if not theNotes[offsets[i+1]][myParts[j]]:
						break
					tempOffset = theNotes[offsets[i+1]][myParts[j]].offset
					if theNotes[offsets[i+2]][myParts[j]] and (theNotes[offsets[i+2]][myParts[j]].offset != tempOffset):
						newInt = horizontal_interval(theNotes[offsets[i+1]][myParts[j]], theNotes[offsets[k]][myParts[j]])
						if newInt == 2:
							theNotes[offsets[i]][myParts[j]].lyric = 'C'
							nhtDict['C'] = nhtDict.setdefault('C', 0) + 1
							nhtLocations['C'].append([pieceNumber, o])
						else:
							theNotes[offsets[i]][myParts[j]].lyric = 'QC'
							nhtDict['QC'] = nhtDict.setdefault('QC', 0) + 1		# quasi-cambiata
							nhtLocations['QC'].append([pieceNumber, o])
					foundIt = True
					break
					k += 1
				if foundIt: 
					break
				else:
					theNotes[offsets[i]][myParts[j]].lyric = 'QC'
					nhtDict['QC'] = nhtDict.setdefault('QC', 0) + 1
					nhtLocations['QC'].append([pieceNumber, o])
					foundIt = True
					break
		
		if not foundIt:
			uStr = '-'.join([str(x) for x in [theInts[offsets[i-1]], theInts[offsets[i]], theInts[offsets[i+1]]]]) + ' ' + ' '.join([str(h[i][j]) for j in range(2) for i in range(2)])
			#print "UNKNOWN!!! " + uStr
			nhtDict['U'] = nhtDict.setdefault('U', 0) + 1
			nhtLocations[uStr].append([pieceNumber, o])
	
	for i in range(len(consonantOffsets[:-1])):
		h = []
		hChromatic = []
		for j in myParts:
			h.append(horizontal_interval(theNotes[consonantOffsets[i]][j], theNotes[consonantOffsets[i+1]][j]))
			if theNotes[consonantOffsets[i]][j] and theNotes[consonantOffsets[i+1]][j]:
				hChromatic.append(abs(theNotes[consonantOffsets[i]][j].pitch.midi - theNotes[consonantOffsets[i+1]][j].pitch.midi))
			else:
				hChromatic.append(False)
		theMotions[consonantOffsets[i]] = h
		cMotions[consonantOffsets[i]] = hChromatic
		theMotions[consonantOffsets[-1]] = [False, False]
		cMotions[consonantOffsets[-1]] = [False, False]
	for i in range(len(consonantOffsets[:-1])):
		if False in [theNotes[consonantOffsets[i]][x] for x in myParts]:
			bigString += ' * '
			chromaticString += ' * '
			continue
		theSym = str(theInts[consonantOffsets[i]])
		cSym = str(chromaticConsonantOffsets[consonantOffsets[i]])
		totalMotions[theSym] = totalMotions.setdefault(theSym, 0) + 1
		chromaticMotions[cSym] = chromaticMotions.setdefault(cSym, 0) + 1
		twoSyms = theSym + ' ' + str(theInts[consonantOffsets[i+1]])
		twoCSyms = cSym + ' ' + str(chromaticConsonantOffsets[consonantOffsets[i+1]])
		if theMotions[consonantOffsets[i]][0] == 1 and theMotions[consonantOffsets[i]][1] == 1:
			continue
		if False in [theNotes[consonantOffsets[i + 1]][x] for x in myParts]:
			bigString += ' * '
			chromaticString += ' * '
			continue
		locations[theSym].append([pieceNumber, consonantOffsets[i]])
		chromaticLocations[cSym].append([pieceNumber, consonantOffsets[i]])
		totalMotions[twoSyms] = totalMotions.setdefault(twoSyms, 0) + 1
		chromaticMotions[twoCSyms] = chromaticMotions.setdefault(twoCSyms, 0) + 1
		locations[twoSyms].append([pieceNumber, consonantOffsets[i]])
		chromaticLocations[twoCSyms].append([pieceNumber, consonantOffsets[i]])
		theVL = str(interval.Interval(theNotes[consonantOffsets[i]][myParts[0]], theNotes[consonantOffsets[i + 1]][myParts[0]]).generic.semiSimpleDirected) + ' ' + str(interval.Interval(theNotes[consonantOffsets[i]][myParts[1]], theNotes[consonantOffsets[i + 1]][myParts[1]]).generic.semiSimpleDirected)
		cVL = str(interval.Interval(theNotes[consonantOffsets[i]][myParts[0]], theNotes[consonantOffsets[i + 1]][myParts[0]]).semitones) + ' ' + str(interval.Interval(theNotes[consonantOffsets[i]][myParts[1]], theNotes[consonantOffsets[i + 1]][myParts[1]]).semitones)
		vlLabel = twoSyms + '  ' + theVL
		cLabel = twoCSyms + '  ' + cVL
		bigString += ' | ' + vlLabel
		chromaticString += ' | ' + cLabel
		newVLLabel = reorder(vlLabel)
		if newVLLabel != vlLabel:
			cLabel = twoCSyms + '  ' + ' '.join(cVL.split()[-1::-1])
		totalMotions[newVLLabel] = totalMotions.setdefault(newVLLabel, 0) + 1
		chromaticMotions[cLabel] = chromaticMotions.setdefault(cLabel, 0) + 1
		locations[newVLLabel].append([pieceNumber, consonantOffsets[i]])
		chromaticLocations[cLabel].append([pieceNumber, consonantOffsets[i]])
	bigString += ' * '
	chromaticString += ' * '

def parts_with_notes(theList):
	return [i for i in range(len(theList)) if theList[i]]

def note_count(theList):
	return len([x for x in theList if x])
	
def newdata():
	for c in twoPartComposers:
		gather_stats(c)

def get_comp(inString):
	inString = inString.upper()
	if inString in twoPartComposers:
		return inString
	l = len(inString)
	myComps = [x for x in twoPartComposers if x[:l] == inString]
	if len(myComps) == 1: 
		return myComps[0]
	return ''

def interval_sequence(matchStr = '', matchStr2 = '', chromatic = -1):
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	if chromatic == -1:
		chromatic = globalChromatic
	else:
		globalChromatic = chromatic
	if chromatic:
		myInts = chromaticInts
	else:
		myInts = allInts
	for i, s in myInts.iteritems():
		if not s:
			continue
		phrases = s.split(' * ')
		for p in phrases:
			theInts = p.split(' | ')
			outStrs = ['', '']
			for vl in theInts:
				if (not vl): continue
				vl = vl.split(' ')
				if not outStrs[0]:
					outStrs[0] = ' '.join(vl[:2])
					outStrs[1] = ' '.join(vl[3:])
				else:
					outStrs[0] += ' ' + vl[1]
					outStrs[1] += ' ' + ' '.join(vl[3:])
			#print outStrs
			if matchStr and re.match(matchStr, outStrs[0]):
				print(i, outStrs)
			elif matchStr2 and re.match(matchStr2, outStrs[1]):
				print(i, outStrs)

def small_big():
	global output
	output = {}
	for i, s in allInts.iteritems():
		if not s:
			continue
		phrases = s.split(' * ')
		for p in phrases:
			theInts = p.split(' | ')
			outStrs = ['', '']
			for vl in theInts:
				if (not vl): continue
				vl = vl.split(' ')
				if not outStrs[0]:
					outStrs[0] = ' '.join(vl[:2])
					outStrs[1] = ' '.join(vl[3:])
				else:
					outStrs[0] += ' ' + vl[1]
					outStrs[1] += ' ' + ' '.join(vl[3:])
			#print outStrs
			v1 = [int(x) for x in outStrs[1].split()[0::2]]
			v2 = [int(x) for x in outStrs[1].split()[1::2]]
			for v in [v1, v2]:
				for i in range(len(v) - 1):
					i1 = v[i]
					i2 = v[i+1]
					if (i1 > 2 and i2 > 2) or (i1 < -2 and i2 < -2):
						seq = str(i1) + ' ' + str(i2)
						output[seq] = output.setdefault(seq, 0) + 1
						if seq == '-5 -5': print(i, p)
	DT.print_dict(output)
							
def load(source = 'BachInventions'):
	global globalSource, totalMotions, locations, filenames, allInts
	global chromaticMotions, chromaticLocations, chromaticInts, nhtDict, nhtLocations
	source = get_comp(source)
	suffix = ''
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource + suffix +'.pkl', 'rb') as myFile:
			totalMotions = pickle.load(myFile)
			locations = pickle.load(myFile)
			filenames = pickle.load(myFile)
			allInts = pickle.load(myFile)
			chromaticMotions = pickle.load(myFile)
			chromaticLocations = pickle.load(myFile)
			chromaticInts = pickle.load(myFile)
			nhtDict = pickle.load(myFile)
			nhtLocations = pickle.load(myFile)
		return True
	except:
		return False


def show(theStr, maxCount = 0, chromatic = -1):								# fix this up
	global output, locations, globalSource
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	if chromatic == -1:
		chromatic = globalChromatic
	else:
		globalChromatic = chromatic
	if chromatic:
		myLocs = chromaticLocations
	else:
		myLocs = locations
	if type(theStr) is int:
		chorale = converter.parse(DT._PATHS[globalSource][1] + '/' + filenames[theStr])
		chorale.show()
		return
	output = stream.Score()
	tempOutput = stream.Score()
	for i in range(0, 2):
		tempOutput.append(stream.Part())
	try:
		locList = myLocs[theStr]							# check for key error
	except KeyError:
		try:
			locList = nhtLocations[theStr]
		except:
			print('Not found')
			return
	count = 0
	for instance in locList:
		count += 1
		if maxCount and count > maxCount: 
			break
		header = str(filenames[instance[0]])
		print('getting', header)
		try:
			chorale = converter.parse(DT._PATHS[globalSource][1] + '/' + header)
		except:
			print("CAN'T READ FILE")
			return
		measureList = DT.get_measure_number_and_beat_from_offset(chorale, instance[1])
		tempChorale = copy.deepcopy(DT.get_measures(chorale, max(measureList[0] - 1, 0), measureList[0] + 1))
		DT.insert_text(tempChorale.parts[0][0], header + ' ' + str(max(measureList[0] - 1, 0)), 55, 0)
		output = DT.append(tempChorale, output)
	output.show()
	return output

def get(size = 0, matchStr = '', pct = True, printIt = True, chromatic = False):
	global working
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	if chromatic == -1:
		chromatic = globalChromatic
	else:
		globalChromatic = chromatic
	if chromatic:
		myDict = chromaticMotions
	else:
		myDict = totalMotions
	working = {}
	for item in myDict:
		if item.count(' ') == size:
			if matchStr and not re.match(matchStr, item):
				continue
			working[item] = myDict[item]
	if printIt:
		DT.print_dict(working, pct = pct)

def find_thirds_and_sixths(thresh = 10):
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	myInts = allInts
	for i, s in myInts.iteritems():
		if not s:
			continue
		phrases = s.split(' * ')
		for phraseNumber, p in enumerate(phrases):
			theInts = p.split(' | ')
			outStrs = ['', '']
			for vl in theInts:
				if (not vl): continue
				vl = vl.split(' ')
				if not outStrs[0]:
					outStrs[0] = ' '.join(vl[:2])
					outStrs[1] = ' '.join(vl[3:])
				else:
					outStrs[0] += ' ' + vl[1]
					outStrs[1] += ' ' + ' '.join(vl[3:])
			#print outStrs
			if not outStrs[0]: continue
			s = outStrs[0].replace('10', 'A').replace('12', 'B').replace(' ', '')
			s = ''.join([x if x in '36AB' else 'z' for x in s])
			longest = max([len(x) for x in s.split('z')])
			if longest >= thresh:
				print(i, filenames[i], s, "phrase", phraseNumber, "of", len(phrases))
			"""if matchStr and re.match(matchStr, outStrs[0]):
				print i, outStrs
			elif matchStr2 and re.match(matchStr2, outStrs[1]):
				print i, outStrs"""

def get_file(n = 0):
	global scorePath, output
	scorePath = DT._PATHS[globalSource][1]
	output = converter.parse(scorePath + filenames[n])
	output.show()
	
"""The main analysis routine, though actually using it is tricky.  See TAOM.py for an example."""	
def getall(size = 0, matchStr = '', categoryList = [], returnGroup = [], returnFunc = lambda x: x, reduceOctaves = False, filterRare = True, pct = True, printIt = True, chromatic = False, noBach = False, compList = ['OCKEGHEM', 'JOSQUINCOMPLETE', 'PALRAW']):
	global working
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	if type(returnGroup) is int:
		returnGroup = [returnGroup]
	if compList:
		tempComps = compList
	else:
		tempComps = twoPartComposers[:]
		if noBach:
			tempComps = tempComps[:-3]
	longestComp = max([len(x) for x in tempComps]) + 5
	longestLength = max([len(x) for x in categoryList] + [len(x) for x in tempComps]) + 5
	working = {}
	numberOfComps = len(tempComps)
	totals = [0] * numberOfComps
	bigTotals = totals[:]
	print()
	print(('{0:' + str(longestLength) + '}').format(' ') + ''.join([('{0:' + str(longestComp) + '}').format(x) for x in tempComps]))
	for i, c in enumerate(tempComps):
		load(c)
		if chromatic == -1:
			chromatic = globalChromatic
		else:
			globalChromatic = chromatic
		if chromatic:
			myDict = chromaticMotions
		else:
			myDict = totalMotions
		if reduceOctaves:
			myDict = reduce_octaves()
		for item in myDict:
			theCount = myDict[item]
			if item.count(' ') != size:
				continue
			bigTotals[i] += myDict[item]
			m = False
			if matchStr:
				m = re.match(matchStr, item)
				if not m: continue
			if categoryList:
				for thing in categoryList:
					if re.match(thing, item):
						"""if thing == '.*  (([0-9]* [0-9]*)|(-[0-9]* -[0-9]*))$':
							print(c, item, theCount)"""
						item = thing
						break
			itemList = [item]
			if m and returnGroup:
				itemList = [m.group(x) for x in returnGroup]
			itemList = returnFunc(itemList)
			for item in itemList:
				if item not in working:
					working[item] = [0] * numberOfComps
				working[item][i] += theCount
				totals[i] += theCount
	for item in working:
		if len(working[item]) != len(totals):
			#print item, len(working[item]), len(totals)
			working[item] += [0] * (len(totals) - len(working[item]))
	myKeys = sorted(working.keys(), key = lambda x: -sum(working[x]))
	truetotals = totals[:]
	for i in range(len(totals)):
		if totals[i] == 0:
			totals[i] = 1
	if pct:
		for item in myKeys:
			working[item] = [(100.*working[item][i]/totals[i]) for i in range(len(working[item]))]		
		if filterRare:
			myKeys = [x for x in myKeys if sum(working[x]) > len(working[x])]
		for item in myKeys:
			print(('{0:' + str(longestLength) + '}').format(item) + ''.join([('{0:' + str(longestComp) + '}').format(('%.2f' % working[item][i])) for i in range(len(working[item]))]))
	else:
		for item in myKeys:
			print(('{0:' + str(longestLength) + '}').format(item) + ''.join([('{0:' + str(longestComp) + '}').format((str(working[item][i]))) for i in range(len(working[item]))]))
	
	print()
	print(('{0:' + str(longestLength) + '}').format('TOTALS:') + ''.join([('{0:' + str(longestComp) + '}').format(str(i)) for i in totals]))
	print(('{0:' + str(longestLength) + '}').format('OUT OF') + ''.join([('{0:' + str(longestComp) + '}').format(str(i)) for i in bigTotals]))
	print(('{0:' + str(longestLength) + '}').format(' ') + ''.join([('{0:' + str(longestComp) + '}').format('%.1f' % (100.*totals[i]/bigTotals[i]) + '%') for i in range(len(bigTotals))]))


def allnhts(undirected = False, groupSuspensions = True, chromatic = -1, noBach = False, pct = True):
	global working
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	tempComps = twoPartComposers[:]
	if noBach:
		tempComps = tempComps[:-3]
	longestComp = max([len(x) for x in tempComps]) + 5
	longestLength = 9
	working = {}
	numberOfComps = len(tempComps)
	totals = [0] * numberOfComps
	bigTotals = totals[:]
	print()
	print(('{0:' + str(longestLength) + '}').format(' ') + ''.join([('{0:' + str(longestComp) + '}').format(x) for x in tempComps]))
	for i, c in enumerate(tempComps):
		load(c)
		for item in nhtDict:
			theCount = nhtDict[item]
			bigTotals[i] += theCount
			item = str(item)
			if groupSuspensions and item.count('-') >= 1:
				item = 'S'
			if item not in working:
				working[item] = [0] * numberOfComps
			working[item][i] += theCount
			totals[i] += theCount
	for item in working:
		if len(working[item]) != len(totals):
			#print item, len(working[item]), len(totals)
			working[item] += [0] * (len(totals) - len(working[item]))
	myKeys = sorted(working.keys(), key = lambda x: -sum(working[x]))
	truetotals = totals[:]
	for i in range(len(totals)):
		if totals[i] == 0:
			totals[i] = 1
	if pct:
		for item in myKeys:
			working[item] = [(100.*working[item][i]/totals[i]) for i in range(len(working[item]))]		
		for item in myKeys:
			print(('{0:' + str(longestLength) + '}').format(item) + ''.join([('{0:' + str(longestComp) + '}').format(('%.2f' % working[item][i])) for i in range(len(working[item]))]))
	else:
		for item in myKeys:
			print(('{0:' + str(longestLength) + '}').format(item) + ''.join([('{0:' + str(longestComp) + '}').format((str(working[item][i]))) for i in range(len(working[item]))]))
	
	print()
	print(('{0:' + str(longestLength) + '}').format('TOTALS:') + ''.join([('{0:' + str(longestComp) + '}').format(str(i)) for i in totals]))
	print(('{0:' + str(longestLength) + '}').format('OUT OF') + ''.join([('{0:' + str(longestComp) + '}').format(str(i)) for i in bigTotals]))
	print(('{0:' + str(longestLength) + '}').format(' ') + ''.join([('{0:' + str(longestComp) + '}').format('%.1f' % (100.*totals[i]/bigTotals[i]) + '%') for i in range(len(bigTotals))]))


def interval_size(undirected = False, chromatic = False, printIt = True):
	global output
	output = {}
	if chromatic:
		myDict = chromaticMotions
	else:
		myDict = totalMotions
	for item in myDict:
		if item.count(' ') == 4:
			itemSplit = item.split()
			i1 = int(itemSplit[-2])
			i2 = int(itemSplit[-1])
			if undirected:
				i1 = abs(i1)
				i2 = abs(i2)
			output[i1] = output.setdefault(i1, 0) + myDict[item]
			output[i2] = output.setdefault(i2, 0) + myDict[item]
	if printIt:
		for item in reversed(sorted(output.keys(), key = lambda x: output[x])):
			print(item, output[item])

def get_linear_motions(voice = [1, 2], pct = True, printIt = True):	
	global motions
	if type(voice) is int:
		voice = [voice]
	get(4, printIt = False)
	motions = {}
	for item in working:
		theSplit = item.split(' ')
		for i in voice:
			motion = theSplit[-i]
			motions[motion] = motions.setdefault(motion, 0) + working[item]
	del motions['1']
	if printIt:
		DT.print_dict(motions, pct = pct)

def vertical_interval(n1, n2):
	if (not n1) or (not n2):
		return False
	return abs(interval.Interval(n1, n2).generic.undirected)
	
def reduce_octaves():
	global totalMotions, newDict
	newDict = {}
	for item in totalMotions:
		item = item.replace('  ', ' *')
		itemSplit = item.split()
		l = len(itemSplit)
		if l == 1:
			newItem = fix_sym(itemSplit[0])
		elif l == 2:
			newItem = ' '.join([fix_sym(s) for s in itemSplit])
		else:
			newItem = ' '.join([fix_sym(s) for s in itemSplit[:2]] + itemSplit[2:])
			newItem = newItem.replace('*', ' ')
			item = item.replace('*', ' ')
		newDict[newItem] = newDict.setdefault(newItem, 0) + totalMotions[item]
	return newDict

def fix_sym(s):
	i = ((abs(int(s)) - 1) % 7) + 1
	#if i == 6: i = 3
	return str(i)

def diatonic_pc_interval(n1, n2):
	if (not n1) or (not n2):
		return False
	i = ((abs(interval.Interval(n1, n2).generic.semiSimpleDirected) - 1) % 7) + 1
	if i in [7, 6, 5]:
		return 9 - i
	return i

def horizontal_interval(n1, n2):		# fix using notation once I get some 
	if (not n1) or (not n2):
		return False
	i = interval.Interval(n1, n2).generic.directed
	return i

def find_root(myNotes):
	newNotes = sorted([x for x in myNotes if x])
	midiInt = (newNotes[1].pitch.midi - newNotes[0].pitch.midi) % 12
	if midiInt in [0, 3, 4, 7]:
		return newNotes[0]
	else:
		return newNotes[1]
	
def noteName(n):
	if n:
		return n.nameWithOctave
	else:
		return n		
		
def normal_numbering(i):
	return int(i/abs(i)) * ((abs(i) - 1) % 7)
	
def check_motions(matchStr = '', skipContrary = True):			# does the top voice behave differently
	global results
	results = {'biggerS': 0, 'biggerB': 0}
	for item in totalMotions:
		if item.count(' ') != 4 or (matchStr and not re.match(matchStr, item)):
			continue
		itemSplit = [int(x) for x in item.split()]
		itemCount = totalMotions[item]
		for i, theInt in enumerate(itemSplit[2:]):
			theInt = abs(theInt)
			if theInt not in results:
				results[theInt] = [0, 0]
			results[theInt][i] = results[theInt][i] + itemCount
		if skipContrary and (itemSplit[2] * itemSplit[3] < 0):
			continue									# avoid contrary motion
		if abs(itemSplit[3]) > abs(itemSplit[2]):
			results['biggerS'] += totalMotions[item]
		elif abs(itemSplit[3]) < abs(itemSplit[2]):
			results['biggerB'] += totalMotions[item]
	#DT.print_dict(results)	

def compare(c1 = 'JOSQUINCOMPLETE', c2 = 'PALRAW', theSize = 4):			# how are two composers different?
	global results, working
	working = {}
	results = {}
	for c in [c1, c2]:
		firstComp = copy.deepcopy(working)
		load(c)
		get(theSize, printIt = False)
		totals = sum(working.values())
		for item in working:
			working[item] = 100. * working[item] / totals
	totalKeys = list(set(firstComp.keys() + working.keys()))
	for item in totalKeys:
		results[item] = working.get(item, 0) - firstComp.get(item, 0)
	DT.print_dict(results)
		
		
	
	
"""def check_motions(matchStr = ''):			# does the top voice behave differently
	global results
	results = {'biggerS': 0, 'biggerB': 0}
	for item in totalMotions:
		if item.count(' ') != 4 or (matchStr and not re.match(matchStr, item)):
			continue
		itemSplit = [int(x) for x in item.split()]
		itemSplit = [(i/abs(i)) * (abs(i) - 1) for i in itemSplit]
		sopranoFirst = itemSplit[0] + itemSplit[2] - itemSplit[3]				
		sopranoSecond = itemSplit[0] - itemSplit[2] + itemSplit[3]
		if sopranoFirst != itemSplit[1] and sopranoSecond != itemSplit[1]: 
			continue
		if sopranoSecond == itemSplit[1]:
			bass, sop = itemSplit[2:]
		else:
			sop, bass = itemSplit[2:]
		if abs(sop) > abs(bass):
			results['biggerS'] += totalMotions[item]
		elif abs(sop) < abs(bass):
			results['biggerB'] += totalMotions[item]
	DT.print_dict(results)	"""

def reorder(item):
	itemSplit = [int(x.replace('.0', '')) for x in item.split()]
	itemSplit = [int((i/abs(i))) * (abs(i) - 1) for i in itemSplit]
	newItem = order_VL(itemSplit)
	for i, path in enumerate(newItem):
		if path >= 0:
			newItem[i] = path + 1
		else:
			newItem[i] = path - 1
	newItem = [str(x) for x in newItem]
	newItem[2] = ' ' + newItem[2]
	newItem = ' '.join(newItem)
	return newItem

def order_VL(theList):
	v1 = theList[2]	
	v2 = theList[0] + theList[3]
	if theList[0] == 0:
		return theList[:2] + sorted(theList[2:])
	elif abs(v1 - v2) == theList[1]:
		return theList
	return theList[:2] + theList[-1:-3:-1]

def order_voices():											# call before reduce_octaves
	global totalMotions
	newDict = {}
	for item in totalMotions:
		if item.count(' ') != 4:
			newDict[item] = totalMotions[item]
			continue
		newItem = reorder(item)
		newDict[newItem] = newDict.setdefault(newItem, 0) + totalMotions[item]
	totalMotions = newDict
 
def checkall(matchStr = ''):			# does the top voice behave differently
	for c in twoPartComposers:
		print(c)
		load(c)
		check_motions(matchStr)

def find_crossings():
	global crossings, noCrossings
	myKeys = [k for k in totalMotions if k.count('  ') > 0]
	crossings = {}
	noCrossings = {}
	for k in myKeys:
		theMotions = [fix_int(int(x)) for x in k.split()]
		newDist = theMotions[0] + theMotions[3] - theMotions[2]
		if newDist < 0:
			crossings[k] = totalMotions[k]
		else:
			noCrossings[k] = totalMotions[k]
	t1 = sum(crossings.values())
	t2 = sum(noCrossings.values())
	print("TOTAL CROSSING PERCENTAGE", round(100.*t1/(t1+t2), 1))
			
def fix_int(i):
	if i > 0:
		return i - 1
	else:
		return i + 1		

def find_small_motions(size = 2, thresh = 4):
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	myInts = allInts
	for i, s in myInts.iteritems():
		if not s:
			continue
		phrases = s.split(' * ')
		for phraseNumber, p in enumerate(phrases):
			theInts = p.split(' | ')
			outLists = [[], []]
			for vl in theInts:
				if (not vl): continue
				vl = [int(x) for x in vl.split(' ') if x]
				outLists[0].append(vl[:2])
				outLists[1].append(vl[2:])
			if not outLists[1]: continue
			theSum = [sum(x) for x in outLists[1]]
			s = ''.join([str(abs(x)) if abs(x) <= size else 'z' for x in theSum])
			#print outLists, theSum, s
			longest = max([len(x) for x in s.split('z')])
			if longest >= thresh:
				print(i, filenames[i], s, "phrase", phraseNumber, "of", len(phrases))
			"""if matchStr and re.match(matchStr, outStrs[0]):
				print i, outStrs
			elif matchStr2 and re.match(matchStr2, outStrs[1]):
				print i, outStrs"""

def change_of_center(removeParallels = False):
	global output, details
	myKeys = [k for k in totalMotions if k.count('  ') > 0]
	output = {}
	details = {}
	for k in myKeys:
		theMotions = [fix_int(int(x)) for x in k.split()]
		theInts = theMotions[2:]
		if removeParallels and all([x == theInts[0] for x in theInts]):
			continue
		delta = sum(theInts)
		output[delta] = output.get(delta, 0) + totalMotions[k]
		details.setdefault(delta, {})[k] = totalMotions[k]
	DT.print_dict(output, pct = True)		

def drift(c):							# what does this do?  CHECK ON A PHRASE LEVEL
	global theMotions
	theMotions = {}
	avg = 0
	load(c)
	for s in totalMotions:
		if s.count(' ') != 4:
			continue
		for t in s.split()[-2:]:
			i = int(t)
			theMotions[i] = theMotions.setdefault(i, 0) + totalMotions[s]
	for i in theMotions:
		avg += i * theMotions[i]
	print(avg, 1.0 * avg/sum(theMotions.values()))

def nhts():
	for i in sorted(nhtLocations.keys(), key = lambda x: -len(nhtLocations[x])):
		print(i, len(nhtLocations[i]))

def n():
	DT.print_dict(nhtDict)