from __future__ import print_function
import pickle, os, time, re, collections, copy, random

from music21 import *
import matplotlib.pyplot as plt

import DT

_THISFILE = 'CPT3'
totalMotions = {}

#threePartComposers = ['Josquincomplete', 'LassusDuets', "BachInventions"]

dates = {'14thC': 1350, 'OCKEGHEM': 1410, 'JOSQUINCOMPLETE': 1450, 'PALRAW': 1525, 'VICTORIARAW': 1545, 'LASSUSDUETS': 1525, 'WTC': 1680, 'WTCF': 1680, 'BACHINVENTIONS': 1680, 'BACH': 1680}

threePartComposers = [x for x in os.listdir(DT._DATAPATH) if x.count(_THISFILE) > 0]
threePartComposers = sorted(list(set([x.replace(_THISFILE,'').replace('no6', '').replace('.pkl', '') for x in threePartComposers])), key = lambda x: dates.get(x, 0))

globalChromatic = False

"""
Look at the internal logic of three-part counterpoint; based on cpt.py (for two-part music)

Also a bit of a mess ...

TODO:
	eliminate the 5-6 sonority, which is counted as a consonance according to my rules
	can I make the NHtone detector work?

"""

def gather_stats(source = 'BachInventions'):
	global thepiece, pieceNumber, theNotes, theInts, theMotions, totalMotions, globalSource, locations, globalSixths, bigString, allInts
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic, filenames, timeSigs
	globalSixths = True
	source = source.upper()
	globalSource = source
	suffix = ''
	if source not in DT._PATHS:
		print("SOURCE NOT FOUND!")
		return
	analysisPath = DT._PATHS[source][0]
	scorePath = DT._PATHS[source][1]
	files = sorted([x for x in os.listdir(scorePath) if (x.count('.xml') + x.count('.krn')) > 0])
	startT = time.time()
	totalMotions = {}
	chromaticMotions = {}
	locations = collections.defaultdict(lambda: [])
	chromaticLocations = collections.defaultdict(lambda: [])
	filenames = {}
	allInts = {}
	chromaticInts = {}
	pieceNumber = 0
	for i, myFile in enumerate(files):
		if myFile[-3:] in ['xml', 'krn']:
			"""if not myFile.startswith('Ock1012a-'):
				continue"""
			tempName = scorePath + myFile
			try:
				thepiece = converter.parse(tempName)
			except Exception as e:
				print("Error reading this file", tempName)
				continue
			if globalSource == 'BACH':
				newpiece = stream.Score()
				newpiece.insert(thepiece.parts[0])
				newpiece.insert(thepiece.parts[-1])
				thepiece = newpiece
			if 'Opus' in thepiece.classes:
				continue
			pieceNumber += 1
			timeSigs = thepiece.flat.getTimeSignatures()
			if len(thepiece.parts) != 3:
				print('File',i, 'of ', len(files), myFile)
				check_for_three_part_passages()
				"""try:
					check_for_three_part_passages()
				except Exception, e:
					print "ERROR ERROR ERROR", str(e)"""
			else:
				print('File',i, 'of ', len(files), myFile, " three-part piece")
				dofile()
				"""try:
					dofile()
				except Exception, e:
					print "ERROR ERROR ERROR", str(e)"""
			filenames[pieceNumber] = myFile
	outputFile = open(DT._DATAPATH + _THISFILE + globalSource + suffix +'.pkl', 'wb')
	pickle.dump(totalMotions, outputFile)
	pickle.dump(dict(locations), outputFile)
	pickle.dump(filenames, outputFile)
	pickle.dump(allInts, outputFile)
	pickle.dump(chromaticMotions, outputFile)
	pickle.dump(dict(chromaticLocations), outputFile)
	pickle.dump(chromaticInts, outputFile)
	outputFile.close()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	print("\a")
	return

def get_beat_strength(myOffset):
	global thepiece, timeSigs
	measure = thepiece.parts[0].getElementsByOffset(myOffset + .01, myOffset + .01, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure']).stream()
	if not measure:
		return 0
	mStart = measure[0].offset
	mOffset = myOffset - mStart
	timeSig = timeSigs.getElementAtOrBefore(myOffset)
	return timeSig.getAccentWeight(mOffset, forcePositionMatch = True)

def check_for_three_part_passages():
	global thepiece, flatparts, theNotes, bigString, allInts, chromaticString
	global chromaticMotions, chromaticLocations, chromaticInts
	offsets = []
	bigString = ''
	chromaticString = ''
	theNotes = {}
	flatparts = []
	totalPassages = 0
	totalNotes = 0
	for p in thepiece.parts:
		flatparts.append(p.flat.getElementsByClass(['Note', 'Rest']).stream())
		offsets += [x.offset for x in flatparts[-1]]
	offsets = sorted(list(set(offsets)))
	for o in offsets:
		temp = []
		for p in flatparts:
			#print p
			tempNote = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False, classList = ['Note', 'Rest'])
			if (not tempNote) or ('Rest' in tempNote[0].classes):
				temp.append(False)
			else:
				temp.append(tempNote[0])
		theNotes[o] = temp
	i = 0
	while i < len(offsets):
		o = offsets[i]
		if note_count(theNotes[o]) == 3:
			myParts = parts_with_notes(theNotes[o])
			j = i + 1
			while j < len(theNotes):
				if (note_count(theNotes[offsets[j]]) != 3) or parts_with_notes(theNotes[offsets[j]]) != myParts:
					break
				j += 1
			if j - i > 5:
				totalPassages += 1
				totalNotes += j - i
				check_section(offsets[i:j], myParts)
				i = j
		i += 1
	if totalPassages != 0: print(" Found ", totalPassages, "passages with", totalNotes, "notes")
	allInts[pieceNumber] = bigString
	chromaticInts[pieceNumber] = chromaticString

def dofile():
	global thepiece, pieceNumber, flatparts, theNotes, bigString, chromaticString
	global chromaticMotions, chromaticLocations, chromaticInts
	offsets = []
	theNotes = {}
	flatparts = []
	bigString = ''
	chromaticString = ''
	for p in reversed(thepiece.parts):
		flatparts.append(p.flat.getElementsByClass(['Note', 'Rest']).stream())
		offsets += [x.offset for x in flatparts[-1]]
	offsets = sorted(list(set(offsets)))
	for o in offsets:
		temp = []
		for p in flatparts:
			tempNote = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False, classList = ['Note', 'Rest'])
			if (not tempNote) or ('Rest' in tempNote[0].classes):
				temp.append(False)
			else:
				temp.append(tempNote[0])
		theNotes[o] = temp
	check_section(offsets, [0, 1, 2])
	allInts[pieceNumber] = bigString
	chromaticInts[pieceNumber] = chromaticString

"""The Main Routine, needs lots of updating"""

def check_section(offsets, myParts):
	global thepiece, flatparts, totalMotions, locations, pieceNumber, theNotes, bigString
	global chromaticMotions, chromaticLocations, chromaticInts, chromaticString
	foundVLCount = 0
	theInts = {}
	theMotions = {}
	cMotions = {}
	consonantOffsets = []
	chromaticConsonantOffsets = {}
	orderDict = {}
	#print "here ", offsets, myParts
	for o in offsets:
		if False in [theNotes[o][x] for x in myParts]:
			continue
		orderDict[o] = sorted(myParts, key = lambda x: theNotes[o][x].pitch.midi)
		theInts[o] = [vertical_interval(theNotes[o][orderDict[o][0]], theNotes[o][orderDict[o][1]]), vertical_interval(theNotes[o][orderDict[o][0]], theNotes[o][orderDict[o][2]])]
		thirdInt = vertical_interval(theNotes[o][orderDict[o][1]], theNotes[o][orderDict[o][2]])
		if theInts[o]:
			midiSize = [abs(theNotes[o][orderDict[o][0]].pitch.midi - theNotes[o][orderDict[o][1]].pitch.midi), abs(theNotes[o][orderDict[o][0]].pitch.midi - theNotes[o][orderDict[o][2]].pitch.midi)]
			reducedInt = [normal_numbering(theInts[o][0]), normal_numbering(theInts[o][1])] 
			thirdReducedInt = normal_numbering(thirdInt)
		else:
			midiSize = False
			reducedInt = False
		if (((reducedInt[0] in [0, 2, 5, False]) or (reducedInt[0] == 4 and ((midiSize[0] % 12) == 7))) and ((
			reducedInt[1] in [0, 2, 5, False]) or (reducedInt[1] == 4 and ((midiSize[1] % 12) == 7))) and (
			thirdReducedInt in [0, 2, 3, 4, 5, False])) or ((reducedInt[0] == 3 and reducedInt[1] == 5) or (reducedInt[1] == 3 and reducedInt[0] == 5)):
			consonantOffsets.append(o)
			chromaticConsonantOffsets[o] = midiSize
	
			
	for i in range(len(consonantOffsets[:-1])):
		h = []
		hChromatic = []
		for j in orderDict[consonantOffsets[i]]:
			h.append(horizontal_interval(theNotes[consonantOffsets[i]][j], theNotes[consonantOffsets[i+1]][j]))
			if theNotes[consonantOffsets[i]][j] and theNotes[consonantOffsets[i+1]][j]:
				hChromatic.append(abs(theNotes[consonantOffsets[i]][j].pitch.midi - theNotes[consonantOffsets[i+1]][j].pitch.midi))
			else:
				hChromatic.append(False)
		theMotions[consonantOffsets[i]] = h
		cMotions[consonantOffsets[i]] = hChromatic
	if theMotions:
		theMotions[consonantOffsets[-1]] = [False, False]
		cMotions[consonantOffsets[-1]] = [False, False]
	for i in range(len(consonantOffsets[:-1])):
		o = consonantOffsets[i]
		#print " ", [theNotes[consonantOffsets[i]][x] for x in myParts]
		if False in [theNotes[consonantOffsets[i]][x] for x in myParts]:
			bigString += ' * '
			chromaticString += ' * '
			continue
		theSym = str(theInts[consonantOffsets[i]])
		cSym = str(chromaticConsonantOffsets[consonantOffsets[i]])
		totalMotions[theSym] = totalMotions.setdefault(theSym, 0) + 1
		chromaticMotions[cSym] = chromaticMotions.setdefault(cSym, 0) + 1
		twoSyms = theSym + ' ' + str(theInts[consonantOffsets[i+1]])
		twoCSyms = cSym + ' ' + str(chromaticConsonantOffsets[consonantOffsets[i+1]])
		if theMotions[consonantOffsets[i]][0] == 1 and theMotions[consonantOffsets[i]][1] == 1 and theMotions[consonantOffsets[i]][2] == 1:
			continue
		if False in [theNotes[consonantOffsets[i + 1]][x] for x in myParts]:
			bigString += ' * '
			chromaticString += ' * '
			continue
		locations[theSym].append([pieceNumber, consonantOffsets[i]])
		chromaticLocations[cSym].append([pieceNumber, consonantOffsets[i]])
		totalMotions[twoSyms] = totalMotions.setdefault(twoSyms, 0) + 1
		chromaticMotions[twoCSyms] = chromaticMotions.setdefault(twoCSyms, 0) + 1
		locations[twoSyms].append([pieceNumber, consonantOffsets[i]])
		chromaticLocations[twoCSyms].append([pieceNumber, consonantOffsets[i]])
		theVL = (str(interval.Interval(theNotes[consonantOffsets[i]][orderDict[o][0]], theNotes[consonantOffsets[i + 1]][orderDict[o][0]]).generic.semiSimpleDirected) + ' ' + 
		         str(interval.Interval(theNotes[consonantOffsets[i]][orderDict[o][1]], theNotes[consonantOffsets[i + 1]][orderDict[o][1]]).generic.semiSimpleDirected) + ' ' + 
				 str(interval.Interval(theNotes[consonantOffsets[i]][orderDict[o][2]], theNotes[consonantOffsets[i + 1]][orderDict[o][2]]).generic.semiSimpleDirected))
		cVL = (str(interval.Interval(theNotes[consonantOffsets[i]][orderDict[o][0]], theNotes[consonantOffsets[i + 1]][orderDict[o][0]]).semitones) + ' ' + 
			   str(interval.Interval(theNotes[consonantOffsets[i]][orderDict[o][1]], theNotes[consonantOffsets[i + 1]][orderDict[o][1]]).semitones) + ' ' + 
			   str(interval.Interval(theNotes[consonantOffsets[i]][orderDict[o][2]], theNotes[consonantOffsets[i + 1]][orderDict[o][2]]).semitones))
		vlLabel = twoSyms + '  ' + theVL
		cLabel = twoCSyms + '  ' + cVL
		bigString += ' | ' + vlLabel
		chromaticString += ' | ' + cLabel
		
		totalMotions[vlLabel] = totalMotions.setdefault(vlLabel, 0) + 1
		chromaticMotions[cLabel] = chromaticMotions.setdefault(cLabel, 0) + 1
		locations[vlLabel].append([pieceNumber, consonantOffsets[i]])
		chromaticLocations[cLabel].append([pieceNumber, consonantOffsets[i]])
		foundVLCount += 1
	#print "FOUND", foundVLCount, "VLs"
	bigString += ' * '
	chromaticString += ' * '

def parts_with_notes(theList):
	return [i for i in range(len(theList)) if theList[i]]

def note_count(theList):
	return len([x for x in theList if x])
	
def newdata():
	for c in threePartComposers:
		gather_stats(c)

def get_comp(inString):
	inString = inString.upper()
	if inString in threePartComposers:
		return inString
	l = len(inString)
	myComps = [x for x in threePartComposers if x[:l] == inString]
	if len(myComps) == 1: 
		return myComps[0]
	return ''

def interval_sequence(matchStr = '', matchStr2 = '', chromatic = -1):
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	if chromatic == -1:
		chromatic = globalChromatic
	else:
		globalChromatic = chromatic
	if chromatic:
		myInts = chromaticInts
	else:
		myInts = allInts
	for i, s in myInts.iteritems():
		if not s:
			continue
		phrases = s.split(' * ')
		for p in phrases:
			theInts = p.split(' | ')
			outStrs = ['', '']
			for vl in theInts:
				if (not vl): continue
				vl = vl.split(' ')
				if not outStrs[0]:
					outStrs[0] = ' '.join(vl[:2])
					outStrs[1] = ' '.join(vl[3:])
				else:
					outStrs[0] += ' ' + vl[1]
					outStrs[1] += ' ' + ' '.join(vl[3:])
			#print outStrs
			if matchStr and re.match(matchStr, outStrs[0]):
				print(i, outStrs)
			elif matchStr2 and re.match(matchStr2, outStrs[1]):
				print(i, outStrs)

def small_big():
	global output
	output = {}
	for i, s in allInts.iteritems():
		if not s:
			continue
		phrases = s.split(' * ')
		for p in phrases:
			theInts = p.split(' | ')
			outStrs = ['', '']
			for vl in theInts:
				if (not vl): continue
				vl = vl.split(' ')
				if not outStrs[0]:
					outStrs[0] = ' '.join(vl[:2])
					outStrs[1] = ' '.join(vl[3:])
				else:
					outStrs[0] += ' ' + vl[1]
					outStrs[1] += ' ' + ' '.join(vl[3:])
			#print outStrs
			v1 = [int(x) for x in outStrs[1].split()[0::2]]
			v2 = [int(x) for x in outStrs[1].split()[1::2]]
			for v in [v1, v2]:
				for i in range(len(v) - 1):
					i1 = v[i]
					i2 = v[i+1]
					if (i1 > 2 and i2 > 2) or (i1 < -2 and i2 < -2):
						seq = str(i1) + ' ' + str(i2)
						output[seq] = output.setdefault(seq, 0) + 1
						if seq == '-5 -5': print(i, p)
	DT.print_dict(output)
							
def load(source = 'BachInventions'):
	global globalSource, totalMotions, locations, filenames, allInts
	global chromaticMotions, chromaticLocations, chromaticInts
	source = get_comp(source)
	suffix = ''
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource + suffix +'.pkl', 'rb') as myFile:
			totalMotions = pickle.load(myFile)
			locations = pickle.load(myFile)
			filenames = pickle.load(myFile)
			allInts = pickle.load(myFile)
			chromaticMotions = pickle.load(myFile)
			chromaticLocations = pickle.load(myFile)
			chromaticInts = pickle.load(myFile)
			return True
	except:
		return False

def show(theStr, maxCount = 0, chromatic = -1):								# fix this up
	global output, locations, globalSource
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	if chromatic == -1:
		chromatic = globalChromatic
	else:
		globalChromatic = chromatic
	if chromatic:
		myLocs = chromaticLocations
	else:
		myLocs = locations
	if type(theStr) is int:
		chorale = converter.parse(DT._PATHS[globalSource][1] + '/' + filenames[theStr])
		chorale.show()
		return
	output = stream.Score()
	tempOutput = stream.Score()
	for i in range(0, 2):
		tempOutput.append(stream.Part())
	try:
		locList = myLocs[theStr]							# check for key error
	except KeyError:
		print('Not found')
		return
	count = 0
	for instance in locList:
		count += 1
		if maxCount and count > maxCount: 
			break
		header = str(filenames[instance[0]])
		print('getting', header)
		try:
			chorale = converter.parse(DT._PATHS[globalSource][1] + '/' + header)
		except:
			print("CAN'T READ FILE")
			return
		measureList = DT.get_measure_number_and_beat_from_offset(chorale, instance[1])
		tempChorale = copy.deepcopy(DT.get_measures(chorale, max(measureList[0] - 1, 0), measureList[0] + 1))
		DT.insert_text(tempChorale.parts[0][0], header + ' ' + str(max(measureList[0] - 1, 0)), 55, 0)
		output = DT.append(tempChorale, output)
	output.show()
	return output

def fig357(c):
	if c.upper() != globalSource:
		load(c)
	get(1, reduceOctaves = True, printIt = False)
	finalDict = {}
	finalDict[(3, 5)] = working.get('[3, 5]', 0)
	finalDict[(3,)] = working.get('[3, 3]', 0) + working.get('[1, 3]', 0)
	finalDict[(3, 6)] = working.get('[3, 6]', 0)
	finalDict[(5,)] = working.get('[5, 5]', 0) + working.get('[1, 5]', 0)
	finalDict[(6,)] = working.get('[6, 6]', 0) + working.get('[1, 6]', 0)
	finalDict[(4, 6)] = working.get('[4, 6]', 0)
	finalDict[(1,)] = working.get('[1, 1]', 0)
	s = sum(finalDict.values())
	vs = sorted(finalDict.items(), key = lambda x: -x[1])
	for i in vs:
		print(i[0], round(i[1]*100./s, 1))

def get(size = 2, matchStr = '', pct = True, printIt = True, chromatic = False, reduceOctaves = False):
	global working
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	if chromatic == -1:
		chromatic = globalChromatic
	else:
		globalChromatic = chromatic
	if chromatic:
		myDict = chromaticMotions
	else:
		myDict = totalMotions
	if reduceOctaves:
		myDict = reduce_octaves(myDict)
	working = {}
	for item in myDict:
		if item.count('[') == size:
			if matchStr and not re.match(matchStr, item):
				continue
			working[item] = myDict[item]
	if printIt:
		DT.print_dict(working, pct = pct)

def parallel_thirds(countCommonTones = False, pct = True, reduceOctaves = False):
	global out1, out2, out3, out4
	get(2, '.*  ', printIt = False)
	if reduceOctaves:
		myDict = reduce_octaves(working)
	else:
		myDict = working
	out1 = {}
	out2 = {}
	out3 = {}
	out4 = {}
	imperfectInts = [3, 6, 10, 13, 17, 20]
	for s in myDict:
		shapes, VL = s.split('  ')
		VL = [int(x.replace('.0', '')) for x in VL.split(' ')]
		shapes = shapes.split('] [')
		for i in range(len(shapes)):
			shapes[i] = [int(x) for x in shapes[i].replace(']', '').replace('[', '').split(', ')]
		foundIt = False
		goodIresult = []
		for i in [0, 1, 2]:
			l = [0, 1, 2]
			l.remove(i)
			if l == [0, 1]:
				targetInt = shapes[0][0]
				resultInt = shapes[1][0]
			elif l == [0, 2]:
				targetInt = shapes[0][1]
				resultInt = shapes[1][1]
			elif l == [1, 2]:
				targetInt = shapes[0][1] - shapes[0][0] + 1
				resultInt = shapes[1][1] - shapes[1][0] + 1
			if (countCommonTones or VL[l[0]] != 1) and (VL[l[0]] == VL[l[1]]) and targetInt in imperfectInts:
				foundIt = True
				break
			elif resultInt in imperfectInts and targetInt in imperfectInts and abs(VL[l[0]]) != abs(VL[l[1]]):
				#print(s, VL[l[0]], VL[l[1]], myDict[s])
				goodIresult.append(i)
		if foundIt:
			out1[s] = myDict[s]
		elif goodIresult:
			out4[s] = myDict[s]
		else:
			if VL.count(1) <= 1:
				out2[s] = myDict[s]
			else:
				out3[s] = myDict[s]
	totalSum = sum(myDict.values())
	DT.print_dict(out2, pct = pct)
	print('PERCENT OF VOICE LEADINGS WITH PARALLEL THIRDS: %.2f' % (100. * sum(out1.values())/totalSum))
	print('PERCENT OF VOICE LEADINGS WITH TWO COMMON TONES: %.2f' % (100. * sum(out3.values())/totalSum))
	print('PERCENT OF VOICE LEADINGS WITH THIRDS MOVING IN CONTRARY MOTION TO SIXTHS: %.2f' % (100. * sum(out4.values())/totalSum))
	
def calculate_position(noteList):
	orderPos = sorted(range(len(noteList)), key = lambda x: noteList[x].pitch.midi)
	print(noteList, orderPos, interval.notesToGeneric(noteList[orderPos[0]], noteList[orderPos[1]]).directed)
	theInts = [interval.notesToGeneric(noteList[orderPos[0]], noteList[orderPos[1]]).directed, interval.notesToGeneric(noteList[orderPos[0]], noteList[orderPos[2]]).directed]
	return theInts, orderPos

def random_listmember(myList):					
	theSum = sum([x[-1] for x in myList])
	if theSum == 0: return 0
	myNum = random.randrange(theSum) + 1
	counter = 0
	for item in myList:
		counter += item[-1]
		if counter >= myNum:
			return item
	return item

def markov_phrase(length = 10, outputRanges = [[48, 67], [55, 74], [60, 79]], startPos = ['G3', 'D4', 'G4']):
	global markov_dict, output
	
	currentPos = [note.Note(x, quarterLength = 2) for x in startPos]
	outputRanges = [range(x[0], x[1]) for x in outputRanges]
	
	theParts = [stream.Part(), stream.Part(), stream.Part()]
	for p in theParts:
		p.insert(0, stream.Measure())
	output = stream.Score()
	for p in reversed(theParts):
		output.insert(0, p)
	
	"""MAKE THE MARKOV DICTIONARY"""
	markov_dict = {}
	get(2, '.*  ', printIt = False)
	for s in working:
		shapes, VL = s.split('  ')
		VL = [int(x) for x in VL.split(' ')]
		shapes = shapes.split('] [')
		for i in range(len(shapes)):
			shapes[i] = [int(x) for x in shapes[i].replace(']', '').replace('[', '').split(', ')]
		if tuple(shapes[0]) not in markov_dict:
			markov_dict[tuple(shapes[0])] = []
		markov_dict[tuple(shapes[0])].append([shapes[1], VL, working[s]])
	
	for i in range(length):
		for j in range(len(currentPos)):
			theParts[j][0].append(currentPos[j])
		startPos, orderPos = calculate_position(currentPos)
		
		for j in range(10):
			newPos = random_listmember(markov_dict[tuple(startPos)])
			VL = newPos[1]
			testPos = [0, 0, 0]
			inRange = True
			for j, n in enumerate(currentPos):
				testPos[j] = currentPos[j].transpose(interval.GenericInterval(VL[orderPos[j]]))
				if testPos[j].pitch.midi not in outputRanges[j]:
					inRange = False
					break
			if inRange: break
		if not inRange:
			print("COULDN'T STAY IN RANGE")
			break
		currentPos = testPos
	
	output.show(format = 'oldmusicxml')
		
			
	
	
		

sonorityFunc = {'\\[3, 5]|\\[5, 10]|\\[10, 12]|\\[3, 12]|\\[12, 17]': 'ROOT POS', '\\[3, 6]|\\[6, 10]|\\[10, 13]': 'FIRST INV', 
'\\[8, 10]|\\[3, 3]|\\[3, 8]|\\[3, 10]|\\[1, 3]|\\[10, 15]|\\[1, 10]|\\[10, 10]': 'DOUBLED THIRD', '\\[1, 5]|\\[5, 5]|\\[5, 8]|\\[8, 12]|\\[5, 12]': 'DOUBLED FIFTH', '\\[6, 6]|\\[6, 8]|\\[8, 13]|\\[1, 6]': 'DOUBLED SIXTH'}

sonorityList = sonorityFunc.keys()

"""Once again, the main analysis routine; see TAOM.py for an example of how to use"""
def getall(size = 0, matchStr = '', categoryList = [], returnGroup = [], returnFunc = lambda x: x, reduceOctaves = False, filterRare = True, pct = True, printIt = True, chromatic = False, noBach = True):
	global working
	global chromaticMotions, chromaticLocations, chromaticInts, globalChromatic
	if type(returnGroup) is int:
		returnGroup = [returnGroup]
	tempComps = threePartComposers[:]
	if noBach:
		tempComps = tempComps[:-1]
	longestComp = max([len(x) for x in tempComps]) + 5
	working = {}
	numberOfComps = len(tempComps)
	totals = [0] * numberOfComps
	bigTotals = totals[:]
	for i, c in enumerate(tempComps):
		load(c)
		if chromatic == -1:
			chromatic = globalChromatic
		else:
			globalChromatic = chromatic
		if chromatic:
			myDict = chromaticMotions
		else:
			myDict = totalMotions
		if reduceOctaves:
			myDict = reduce_octaves(totalMotions)
		for item in myDict:
			theCount = myDict[item]
			if item.count('[') != size:
				continue
			bigTotals[i] += myDict[item]
			m = False
			if matchStr:
				m = re.match(matchStr, item)
				if not m: continue
			if categoryList:
				for thing in categoryList:
					if re.match(thing, item):
						item = thing
						break
			itemList = [item]
			if m and returnGroup:
				itemList = [m.group(x) for x in returnGroup]
			itemList = returnFunc(itemList)
			for item in itemList:
				if item not in working:
					working[item] = [0] * numberOfComps
				working[item][i] += theCount
				totals[i] += theCount
	for item in working:
		if len(working[item]) != len(totals):
			#print item, len(working[item]), len(totals)
			working[item] += [0] * (len(totals) - len(working[item]))
	#myKeys = sorted(working.keys(), key = lambda x: -sum(working[x]))
	myKeys = sorted(working.keys(), key = lambda x: -working[x][-1])
	truetotals = totals[:]
	longestLength = max([len(x) for x in categoryList] + [len(x) for x in tempComps]) + 5
	if working.keys():
		tempLength = max([len(x) for x in working.keys()])
	else:
		tempLength = 1
	if tempLength >= longestLength:
		longestLength = tempLength + 5
	print()
	if pct:
		tempComps.append('AVERAGE')
	print(('{0:' + str(longestLength) + '}').format(' ') + ''.join([('{0:' + str(longestComp) + '}').format(x) for x in tempComps]))
	for i in range(len(totals)):
		if totals[i] == 0:
			totals[i] = 1
	if pct:
		for item in myKeys:
			working[item] = [(100.*working[item][i]/totals[i]) for i in range(len(working[item]))]
			working[item].append(sum(working[item])/len(working[item]))	
		if filterRare:
			myKeys = [x for x in myKeys if sum(working[x]) > len(working[x])]
		for item in myKeys:
			print(('{0:' + str(longestLength) + '}').format(item) + ''.join([('{0:' + str(longestComp) + '}').format(('%.2f' % working[item][i])) for i in range(len(working[item]))]))
	else:
		for item in myKeys:
			print(('{0:' + str(longestLength) + '}').format(item) + ''.join([('{0:' + str(longestComp) + '}').format((str(working[item][i]))) for i in range(len(working[item]))]))
	
	print()
	print(('{0:' + str(longestLength) + '}').format('TOTALS:') + ''.join([('{0:' + str(longestComp) + '}').format(str(i)) for i in totals]))
	print(('{0:' + str(longestLength) + '}').format('OUT OF') + ''.join([('{0:' + str(longestComp) + '}').format(str(i)) for i in bigTotals]))
	print(('{0:' + str(longestLength) + '}').format(' ') + ''.join([('{0:' + str(longestComp) + '}').format('%.1f' % (100.*totals[i]/bigTotals[i]) + '%') for i in range(len(bigTotals))]))

def interval_size(undirected = False, chromatic = False, printIt = True):
	global output
	output = {}
	if chromatic:
		myDict = chromaticMotions
	else:
		myDict = totalMotions
	for item in myDict:
		if item.count(' ') == 4:
			itemSplit = item.split()
			i1 = int(itemSplit[-2])
			i2 = int(itemSplit[-1])
			if undirected:
				i1 = abs(i1)
				i2 = abs(i2)
			output[i1] = output.setdefault(i1, 0) + myDict[item]
			output[i2] = output.setdefault(i2, 0) + myDict[item]
	if printIt:
		for item in reversed(sorted(output.keys(), key = lambda x: output[x])):
			print(item, output[item])

def get_linear_motions(voice = [1, 2], pct = True, printIt = True):	
	global motions
	if type(voice) is int:
		voice = [voice]
	get(4, printIt = False)
	motions = {}
	for item in working:
		theSplit = item.split(' ')
		for i in voice:
			motion = theSplit[-i]
			motions[motion] = motions.setdefault(motion, 0) + working[item]
	del motions['1']
	if printIt:
		DT.print_dict(motions, pct = pct)

def vertical_interval(n1, n2):
	if (not n1) or (not n2):
		return False
	return abs(interval.Interval(n1, n2).generic.undirected)
	
def reduce_octaves(myDict = totalMotions):
	newDict = {}
	for item in myDict:
		newItem = reorder(item, reduceOctaves = True)
		newDict[newItem] = newDict.setdefault(newItem, 0) + myDict[item]
	return newDict

def fix_sym(s):
	prefix = ''
	suffix = ''
	if not s[0].isdigit():
		prefix = s[0]
		s = s[1:]
	if not s[-1].isdigit():
		suffix = s[-1]
		s = s[:-1]
	i = ((abs(int(s)) - 1) % 7) + 1
	#if i == 6: i = 3
	return prefix + str(i) + suffix

def diatonic_pc_interval(n1, n2):
	if (not n1) or (not n2):
		return False
	i = ((abs(interval.Interval(n1, n2).generic.semiSimpleDirected) - 1) % 7) + 1
	if i in [7, 6, 5]:
		return 9 - i
	return i

def horizontal_interval(n1, n2):		# fix using notation once I get some 
	if (not n1) or (not n2):
		return False
	i = interval.Interval(n1, n2).generic.directed
	return i

def find_root(myNotes):
	newNotes = sorted([x for x in myNotes if x])
	midiInt = (newNotes[1].pitch.midi - newNotes[0].pitch.midi) % 12
	if midiInt in [0, 3, 4, 7]:
		return newNotes[0]
	else:
		return newNotes[1]
	
def noteName(n):
	if n:
		return n.nameWithOctave
	else:
		return n		
		
def normal_numbering(i):
	return int(i/abs(i)) * ((abs(i) - 1) % 7)
	
def check_motions(matchStr = '', skipContrary = True):			# does the top voice behave differently
	global results
	results = {'biggerS': 0, 'biggerB': 0}
	for item in totalMotions:
		if item.count(' ') != 4 or (matchStr and not re.match(matchStr, item)):
			continue
		itemSplit = [int(x) for x in item.split()]
		itemCount = totalMotions[item]
		for i, theInt in enumerate(itemSplit[2:]):
			theInt = abs(theInt)
			if theInt not in results:
				results[theInt] = [0, 0]
			results[theInt][i] = results[theInt][i] + itemCount
		if skipContrary and (itemSplit[2] * itemSplit[3] < 0):
			continue									# avoid contrary motion
		if abs(itemSplit[3]) > abs(itemSplit[2]):
			results['biggerS'] += totalMotions[item]
		elif abs(itemSplit[3]) < abs(itemSplit[2]):
			results['biggerB'] += totalMotions[item]
	#DT.print_dict(results)	

def compare(c1 = 'JOSQUINCOMPLETE', c2 = 'PALRAW', theSize = 4):			# how are two composers different?
	global results, working
	working = {}
	results = {}
	for c in [c1, c2]:
		firstComp = copy.deepcopy(working)
		load(c)
		get(theSize, printIt = False)
		totals = sum(working.values())
		for item in working:
			working[item] = 100. * working[item] / totals
	totalKeys = list(set(firstComp.keys() + working.keys()))
	for item in totalKeys:
		results[item] = working.get(item, 0) - firstComp.get(item, 0)
	DT.print_dict(results)

def check_for_descending_VL(thresh = 5, **kwargs):
	for pieceNumber, intSeq in allInts.items():
		sections = intSeq.split(' * ')
		for secNo, sec in enumerate(sections):
			if not sec: continue
			theChords = sec.split(' | ')
			descentLength = 0
			for i, c in enumerate(theChords):
				if not c: continue
				chords, theInt = c.split('  ')
				if check_int_sequence(theInt, **kwargs):
					descentLength += 1
				else:
					if descentLength >= thresh:
						print("PIECE", pieceNumber, filenames[pieceNumber], "SECTION", secNo, "OUT OF", len(sections))
						for j in range(i-descentLength, i):
							print(theChords[j])
					descentLength = 0
				
		
			
def check_int_sequence(s, intSequence = [1, -2]):
	theInts = [int(x) for x in s.split()]
	return all([x in intSequence for x in theInts])		
	
	
"""def check_motions(matchStr = ''):			# does the top voice behave differently
	global results
	results = {'biggerS': 0, 'biggerB': 0}
	for item in totalMotions:
		if item.count(' ') != 4 or (matchStr and not re.match(matchStr, item)):
			continue
		itemSplit = [int(x) for x in item.split()]
		itemSplit = [(i/abs(i)) * (abs(i) - 1) for i in itemSplit]
		sopranoFirst = itemSplit[0] + itemSplit[2] - itemSplit[3]				
		sopranoSecond = itemSplit[0] - itemSplit[2] + itemSplit[3]
		if sopranoFirst != itemSplit[1] and sopranoSecond != itemSplit[1]: 
			continue
		if sopranoSecond == itemSplit[1]:
			bass, sop = itemSplit[2:]
		else:
			sop, bass = itemSplit[2:]
		if abs(sop) > abs(bass):
			results['biggerS'] += totalMotions[item]
		elif abs(sop) < abs(bass):
			results['biggerB'] += totalMotions[item]
	DT.print_dict(results)	"""


def order_VL(theList):
	v1 = theList[2]	
	v2 = theList[0] + theList[3]
	if theList[0] == 0:
		return theList[:2] + sorted(theList[2:])
	elif abs(v1 - v2) == theList[1]:
		return theList
	return theList[:2] + theList[-1:-3:-1]

def reorder(item, reduceOctaves = False):											# call before reduce_octaves
	theItems = [[], [], []]
	theSplit = item.split('  ')
	if len(theSplit) > 1:
		theItems[2] = [int(x) for x in theSplit[1].split()]
		bassMotion = theItems[2][0]
		theItems[2] = theItems[2][1:]
	theSplit = theSplit[0].split('] [')
	for i, thing in enumerate(theSplit):
		#thing = str(filter(lambda s: (s.isdigit() or s in ' -'), thing))
		thing = ''.join([s for s in thing if s.isdigit() or s in ' r'])
		if reduceOctaves:
			theItems[i] = [((abs(int(x)) - 1) % 7) + 1 for x in thing.split()]
		else:
			theItems[i] = [int(x) for x in thing.split()]
	indices = sorted(range(len(theItems[0])), key = lambda x: theItems[0][x])
	for i in range(len(theItems)):
		if theItems[i]:
			theItems[i] = [str(theItems[i][x]) for x in indices]
	if theItems[2]:
		return '[' + ', '.join(theItems[0]) + ']' + ' [' + ', '.join(theItems[1]) + ']' + '  ' + str(bassMotion) + ' ' + ' '.join(theItems[2])
	elif theItems[1]:
		return '[' + ', '.join(theItems[0]) + ']' + ' [' + ', '.join(theItems[1]) + ']'
	else:
		return '[' + ', '.join(theItems[0]) + ']'


def diatonic_to_steps(i):
	if i > 0:
		return i - 1
	return i + 1

def steps_to_diatonic(i):
	if i >= 0:
		return i+1
	return i-1

def find_crossings():
	global crossings, noCrossings
	myKeys = [k for k in totalMotions if k.count('  ') > 0]
	crossings = {}
	noCrossings = {}
	for k in myKeys:
		if has_crossing(k):
			crossings[k] = totalMotions[k]
		else:
			noCrossings[k] = totalMotions[k]
	t1 = sum(crossings.values())
	t2 = sum(noCrossings.values())
	print("TOTAL CROSSING PERCENTAGE", round(100.*t1/(t1+t2), 1))	

def has_crossing(k):
	shapes, VL = parse_figure(k)
	startDists = [diatonic_to_steps(x) for x in shapes[0]]
	VL = [diatonic_to_steps(x) for x in VL]
	if individual_crossing(startDists[0], VL[0], VL[1]) or individual_crossing(startDists[1], VL[0], VL[2]) or individual_crossing(startDists[1] - startDists[0], VL[1], VL[2]):
		return True
	else:
		return False


def individual_crossing(startDist, bassMotion, topMotion):
	newDist = startDist + topMotion - bassMotion
	if newDist < 0 and startDist > 0:
		return True
	return False

def parse_figure(s):
	VL = False
	if s.count('  ') > 0:
		shapes, VL = s.split('  ')
		VL = [int(x) for x in VL.split(' ')]
	else:
		shapes = s
	shapes = shapes.split('] [')
	for i in range(len(shapes)):
		shapes[i] = [int(x) for x in shapes[i].replace(']', '').replace('[', '').split(', ')]
	if VL:
		return shapes, VL
	else:
		return shapes

def first_order_second_order(filterList = ['False']):
	global firstOrderProbs, changeDict
	firstOrderDict = {}
	changeDict = {}
	ouch(printOut = False)
	for p, count in ouchProgs.items():
		firstChord = p.split()[0]
		if firstChord in filterList: continue
		firstOrderDict[firstChord] = firstOrderDict.get(firstChord, 0) + count
	firstOrderProbs = {x[0]:x[1]*100./sum(firstOrderDict.values()) for x in firstOrderDict.items()}
	for firstChord in firstOrderProbs.keys():
		limitedDict = {x[0]:x[1] for x in ouchProgs.items() if x[0].startswith(firstChord + ' ')}
		limitedDict = {x[0]:x[1]*100./sum(limitedDict.values()) for x in limitedDict.items()}
		for secondChord in firstOrderProbs.keys():
			endsWithProgs = sum([x[1] for x in limitedDict.items() if x[0].endswith(' ' + secondChord)])
			difference = endsWithProgs - firstOrderProbs.get(secondChord, 0)
			changeDict[firstChord + ' -> ' + secondChord] = difference
	DT.print_dict(changeDict)


def graph(comps = ['josquincomplete', 'PALRAW', 'victoriaraw', 'wtc']):
	global output, ax
	
	labelSubs = {'Udi': 'DI', 'Uoo': 'OO'}
	nameSubs = {'PALRAW': 'Palestrina', 'JOSQUINCOMPLETE': 'Josquin', 'VICTORIARAW': 'Victoria', 'WTC': "W.T.C."}
					
	xyBoundaries = [[0, 16], [0, 16]]
	fig, ax = plt.subplots()
	#fig.set_size_inches(6, 6)
	ax.axes.get_xaxis().set_visible(False)
	ax.axes.get_yaxis().set_visible(False)
	ax.set_xlim(xyBoundaries[0][0], xyBoundaries[0][1])
	ax.set_ylim(xyBoundaries[1][0], xyBoundaries[1][1])
	ax.set_title('configuration size is proportional to occurrence frequency')
	
	orderedPositions = ['DI', 'C', 'H', 'O', 'OO', '?']
		
	#DT.print_dict(output, pct = True)
	
	for j, comp in enumerate(comps):
		
		load(comp)
		ouch(printOut = False)
		compName = nameSubs.get(comp.upper(), comp[0] + comp[1:].lower())
		
		totals = float(sum(ouchStats.values()))
		
		
		output = {labelSubs.get(x[0], x[0]):x[1]*100./totals for x in ouchStats.items()}
		
		newProgs = {}
		totalLabels = []
		
		for sym, count in ouchProgs.items():
			if sym.count('?') > 0:
				continue
			newSym = sym
			for s1, s2 in labelSubs.items():
				newSym = newSym.replace(s1, s2)
			newProgs[newSym] = count
		
		print('\n', compName, "transition frequencies\n")
		totalSum = sum(newProgs.values())
		
		for symType, mySym in enumerate([' -> ', ' -C-> ', ' -DV-> ']):
	
			graphTotal = 0
	
			for i in range(len(orderedPositions) - 1):
				for s in [orderedPositions[i] + mySym + orderedPositions[i+1], orderedPositions[i + 1] + mySym + orderedPositions[i]]:
					totalLabels.append(s)
					graphTotal += newProgs.get(s, 0)
	
			for s in ['C' + mySym + 'C', 'O' + mySym + 'O']: # possible to add: 'C' + mySym + 'O', 'O' + mySym + 'C'
				graphTotal += newProgs.get(s, 0)
				totalLabels.append(s)
			
			print(["   NOCROSSING", "   CROSSING", "   DIFFERENT VOICES"][symType], "GRAMMATICAL TOTAL", graphTotal * 100./totalSum, '\n')
		
		print("   MISSING ITEMS")
		missedGuys = [x for x in newProgs.keys() if x not in totalLabels]
		missedGuys = sorted(missedGuys, key = lambda x: -newProgs[x])
		
		for x in missedGuys[:10]:
			print("   ", x, newProgs[x] * 100./totalSum)
		
		
		DT.print_dict(newProgs, pct = True)
		

		for i, s in enumerate(orderedPositions):
			x = i * (11. / 5) + 1
			y = 3.5 * j + 2
			ax.text(x, y, s, horizontalalignment = 'center', verticalalignment = 'center', fontsize = 2. * output[s], fontname = 'Times')
			ax.text(13.5, y, compName, horizontalalignment = 'left', verticalalignment = 'center', fontsize = 16, fontname = 'Times')
		
		
	fig.canvas.draw()
	plt.ion()
	plt.show()

def ouch(printOut = True):
	global ouchStats, ouchShapes, ouchProgs, ouchProgShapes, ouchFail
	ouchStats = {}
	ouchProgs = {}
	ouchShapes = {}
	ouchProgShapes = {}
	ouchFail = {}
	myKeys = [k for k in totalMotions if (k.count('  ') == 1 and k.count('] [') > 0)]
	for bigKey in myKeys:
		theCount = totalMotions[bigKey]
		k, theMotions = bigKey.split('  ')
		shapes = parse_figure(k)
		shapeKey = ouch_parse(shapes[0])
		if shapeKey == '?':
			badGuy = tuple(shapes[0])
			ouchFail[badGuy] = ouchFail.get(badGuy, 0) + 1
		ouchStats[shapeKey] = ouchStats.get(shapeKey, 0) + theCount
		ouchShapes.setdefault(shapeKey, {})
		ouchShapes[shapeKey][tuple(shapes[0])] = ouchShapes[shapeKey].get(tuple(shapes[0]), 0) + theCount
		myArrow = [' -> ', ' -C-> '][int(has_crossing(bigKey))]
		shapeKey += myArrow + ouch_parse(shapes[1])
		ouchProgs[shapeKey] = ouchProgs.get(shapeKey, 0) + theCount
		ouchProgShapes.setdefault(shapeKey, {})
		shapeTuple = tuple(tuple(x) for x in shapes)
		ouchProgShapes[shapeKey][shapeTuple] = ouchProgShapes[shapeKey].get(shapeTuple, 0) + theCount
	if printOut:
		DT.print_dict(ouchStats, pct = True)
		print("\n" * 3)
		DT.print_dict(ouchProgs, pct = True)
		print("\n" * 3)
		print("FAILURES")
		DT.print_dict(ouchFail, pct = True)
		

def ouch_prog(startChord = 'C'):
	global ouchProgs
	myKeys = [x for x in ouchProgs.keys() if x.startswith(startChord)]
	outDict = {k:ouchProgs[k] for k in myKeys}
	DT.print_dict(outDict, pct = True)

def categorize(startChord = 'H'):
	ouch(printOut = False)
	for s in ['Udi', 'H', 'Uoo']:
		print(s)
		DT.print_dict(ouchShapes[s], pct = True)

def root_progressions(stepSizes = [3, 4], cutoff = 10):
	global output
	if type(stepSizes) is int:
		stepSizes = [stepSizes, 7 - stepSizes]
	output = {}
	myKeys = [k for k in totalMotions if (k.count('  ') > 0)]
	for k in myKeys:
		shapes, VL = parse_figure(k)
		dists = [[(diatonic_to_steps(x) % 7) for x in y] for y in shapes]
		newVL = [(diatonic_to_steps(x) % 7) for x in VL]
		if all([x in [0, 2, 4] for x in dists[0]]) and all([x in [0, 2, 4] for x in dists[1]]) and newVL[0] in stepSizes:
			output[k] = totalMotions[k]
	DT.print_dict(output, pct = True, cutoff = cutoff)

def thirds():
	global output, output2, output3
	output = {}
	output2 = {}
	output3 = {}
	myKeys = [k for k in totalMotions if (k.count('  ') > 0)]
	for k in myKeys:
		shapes, VL = parse_figure(k)
		dists = [[(diatonic_to_steps(x) % 7) for x in y] for y in shapes]
		newVL = [(diatonic_to_steps(x) % 7) for x in VL]
		if check_pair(dists[0][0], newVL[0], newVL[1]) or check_pair(dists[0][1], newVL[0], newVL[2]) or check_pair(dists[0][1] - dists[0][0], newVL[1], newVL[2]):
			output[k] = totalMotions[k]
		else:
			if (2 in dists[0] or 5 in dists[0] or (dists[0][1] - dists[0][0]) in [2, 5]) and (2 in dists[1] or 5 in dists[1] or (dists[1][1] - dists[1][0]) in [2, 5]):
				output3[k] = totalMotions[k]
			else:
				output2[k] = totalMotions[k]
	print("Percent with imperfect pair", round(100. * sum(output.values()) / (sum(output.values()) + sum(output2.values()) + sum(output3.values())), 1))


def all_possible_consonances():
	global totalMotions
	a = [0, 2, 4, 5]
	consonances = [[x,y] for x in a for y in a]
	consonances.remove([4, 5])
	consonances.remove([5, 4])
	totalMotions = {}
	for c1 in consonances:
		for c2 in consonances:
			int1 = steps_to_diatonic((c2[0] - c1[0]) % 7)
			int2 = steps_to_diatonic((c2[1] - c1[1]) % 7)
			totalInts = [1, int1, int2]
			newKey = str([steps_to_diatonic(x) for x in c1]) + ' ' + str([steps_to_diatonic(x) for x in c2]) + '  ' + str(totalInts).replace('[', '').replace(']', '').replace(',', '')
			totalMotions[newKey] = 1

def check_pair(startDist, bassMotion, topMotion):
	newDist = startDist + topMotion - bassMotion
	if (newDist % 7) in [2, 5] and startDist % 7 in [2, 5]:
		return True
	return False

def remove_shape(theShape):
	for ouchPosition, d in ouchShapes.items():
		if theShape in d:
			count = d[theShape]
			del d[theShape]
			return count
	return 0
	
def ouch_parse(fig):
	shapeKey = '?'
	if fig in [[3, 5], [3, 6], [4, 6]]:
		shapeKey = 'C'
	elif [fig[1] - 7, fig[0]] in [[3, 5], [3, 6], [4, 6]]:
		shapeKey = 'O'
	elif fig[1] == 8 and fig[0] > 1 and fig[0] < 8:
		shapeKey = 'H'
	elif (fig[0] == 1 and fig[1] < 8) or (fig[0] == fig[1] and fig[0] < 8):
		shapeKey = 'Udi'
	elif (fig[0] == 8 and fig[1] < 15) or (fig[0] + 7 == fig[1] and fig[0] < 8):
		shapeKey = 'Uoo'
	return shapeKey	
			
		

def average_motions():
	get(2, '.*  ', printIt = False)
	totalVLmotions = [{}, {}, {}]
	for item in working:
		vlMotions = [int(x) for x in item.split('  ')[1].split()]
		vlMotions = [abs(diatonic_to_steps(x)) for x in vlMotions]
		for i, vl in enumerate(vlMotions):
			totalVLmotions[i][vl] = totalVLmotions[i].setdefault(vl, 0) + working[item]
	for i, myDict in enumerate(totalVLmotions):
		motions = 0
		distance = 0
		for j in myDict:
			motions += myDict[j]
			distance += myDict[j] * j
		print("VOICE", i, 1.*distance/motions)


def individually_t_related():
	global totalMotions
	newDict = {}
	for item in totalMotions:
		if item.count('  ') > 0:
			itemSplit = item.split('  ')
			itemSplit[1] = ' '.join([str(y) for y in fix_paths([int(x) for x in itemSplit[1].split()])])
			newItem = '  '.join(itemSplit)
			newDict[newItem] = newDict.setdefault(newItem, 0) + totalMotions[item]
		else:
			newDict[item] = totalMotions[item]
	totalMotions = newDict

def fix_paths(inList):
	myList = [diatonic_to_steps(x) for x in inList]
	while sum(myList) > (len(myList) - 1):
		myList = [x - 1 for x in myList]
	while sum(myList) < 0:
		myList = [x + 1 for x in myList]
	return [steps_to_diatonic(x) for x in myList]
 
def checkall(matchStr = ''):			# does the top voice behave differently
	for c in threePartComposers:
		print(c)
		load(c)
		check_motions(matchStr)
		
def drift(c):
	global theMotions
	theMotions = {}
	avg = 0
	load(c)
	for s in totalMotions:
		if s.count(' ') != 4:
			continue
		for t in s.split()[-2:]:
			i = int(t)
			theMotions[i] = theMotions.setdefault(i, 0) + totalMotions[s]
	for i in theMotions:
		avg += i * theMotions[i]
	print(avg, 1.0 * avg/sum(theMotions.values()))

load('PALRAW')