from __future__ import print_function
import pickle
import os

from music21 import *
import DT

_THISFILE = 'DOUBLINGS'
internalReducedFlag = False
capture = False
progs = [{}, {}]
output = stream.Score()
pTreb = stream.Part()
pBass = stream.Part()
lastChorale = 1
chorale = stream.Score()
analyzed_file = stream.Score()

"""
Find doublings in hand-analyzed music, record them by Roman Numeral.

TODO: 
	1. keep track of the doubling locations?

"""

def gather_stats(source = 'MONTEVERDI', fileName = '', transposeBass = False, printChordalSkips = False):
	global progs, output, pTreb, pBass, lastChorale, chorale, analyzed_file, capture, internalReducedFlag, globalSource, globalTransposeBass, missingChords, upperVoicePositions, spacingChords, spacingDict
	global globalPrintChordalSkips
	globalSource = source.upper()
	globalTransposeBass = transposeBass
	globalPrintChordalSkips = printChordalSkips
	missingChords = {}
	spacingDict = {}
	spacingChords = {}
	progs = [{}, {}]
	upperVoicePositions = [{}, {}]
	if globalSource in DT._PATHS:
		analysisPath = DT._PATHS[globalSource][0]
		scorePath = DT._PATHS[globalSource][1]
		fileStr = globalSource
		files = sorted(os.listdir(analysisPath))
		recordResults = True
		counter = 0
		for myFile in files:
			if myFile[-3:] == 'txt':
				if fileName and fileName != myFile[:-4]:
					continue
				print('Analyzing', myFile)
				analyzed_file = converter.parse(analysisPath + myFile, format='romantext')
				try:
					chorale = converter.parse(scorePath + myFile.replace('.txt', '.xml'))
				except:
					try:
						chorale = converter.parse(scorePath + myFile.replace('.txt', '.krn'))
					except:
						continue
				if transposeBass: DT.transpose_bass(chorale, -12)	
				dochorale(counter)
				"""try:
					dochorale(counter)
				except:
					pass
				counter += 1"""
		if not fileName:
			outputFile = open(DT._DATAPATH + _THISFILE + fileStr + '.pkl', 'wb')
			pickle.dump(progs, outputFile)
			pickle.dump(missingChords, outputFile)
			pickle.dump(upperVoicePositions, outputFile)
			pickle.dump(spacingDict, outputFile)
			pickle.dump(spacingChords, outputFile)
			outputFile.close()
	return

def load(source='Palestrina'):
	global globalSource, progs, missingChords, upperVoicePositions, spacingDict, spacingChords
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource + '.pkl', 'rb') as inputFile:
			progs = pickle.load(inputFile)
			missingChords = pickle.load(inputFile)
			upperVoicePositions = pickle.load(inputFile)
			spacingDict = pickle.load(inputFile)
			spacingChords = pickle.load(inputFile)
		return True
	except:
		return False

def dochorale(n):
	global progs, output, pTreb, pBass, chorale, analyzed_file, globalTransposeBass, missingChords, upperVoicePositions, anal, score, spacingDict, spacingChords, globalPrintChordalSkips
	anal = analyzed_file.flat.getElementsByClass('RomanNumeral')																		# currently very kludgey
	measures = chorale.parts[1].getElementsByClass('Measure')
	preScore = chorale.chordify(removeRedundantPitches = False)
	score = preScore.flat.getElementsByClass('Chord')
	for i in range(0, len(anal)):
		RN = anal[i]
		RNoffset = RN.offset																				# do this because offsets can change unpredictably
		RNpitchClasses = DT.fix_roman(RN)																	# repair the seventh too						
		RNbeat = RN.beat - 1
		chordLabel = RN.figure
		foundInstance = False
		foundBass = False
		chordalSkipError = False
		foundFourNoteChord = False
		isSeventh = False
		foundSeventh = False
		if RN.key.mode == 'minor':
			keyStr = str(RN.key.tonic.name).lower() + ': '
			modeIndex = 1
		else:
			keyStr = str(RN.key.tonic.name) + ': '
			modeIndex = 0
		if RN.seventh:
			isSeventh = True
			seventh = DT.seventh														
		RNuniquePCs = set(RNpitchClasses)
		verts = score.getElementsByOffset(RNoffset + .01, RNoffset + RN.duration.quarterLength - .01, includeEndBoundary = False, mustBeginInSpan = False)
		#print RN.figure, RNoffset, RNoffset + RN.duration.quarterLength
		for chord in verts:													# NOTE: was limiting to four voices, but that's bad for Palestrina
			#if len(chord.pitches) == len(chorale.parts):
			#print '  ', [x.midi for x in chord.pitches]
			sortedPitches = sorted(chord.pitches)
			foundFourNoteChord = True														
			if isSeventh and (seventh in chord.pitchClasses):
				foundSeventh = True
			if set(chord.pitchClasses) <= RNuniquePCs: 														#and len(chord.pitches) == len(chorale.parts):
				topNotes = [x.midi for x in sortedPitches[1:]]
				if len(topNotes) == 3 and not isSeventh:
					upperVoicePosition = categorize_voicing(topNotes)
				else:
					upperVoicePosition = '-'
				foundInstance = True
				nFormStr = str(DT.normal_form(chord.pitchClasses))
				
				midiNotes = [x.midi for x in sortedPitches]
				spacingStr = count_spacing(midiNotes)
				spacingDict[spacingStr] = spacingDict.setdefault(spacingStr, 0) + 1
				if spacingStr not in spacingChords:
					spacingChords[spacingStr] = {}
				spacingChords[spacingStr][chordLabel] = spacingChords[spacingStr].setdefault(chordLabel, 0) + 1
				
				if chordLabel in progs[modeIndex]:
					progs[modeIndex][chordLabel][nFormStr] = progs[modeIndex][chordLabel].setdefault(nFormStr, 0) + 1
					if not isSeventh:
						upperVoicePositions[modeIndex][chordLabel][upperVoicePosition] = upperVoicePositions[modeIndex][chordLabel].setdefault(upperVoicePosition, 0) + 1
				else:
					progs[modeIndex][chordLabel] = {}
					if not isSeventh:
						upperVoicePositions[modeIndex][chordLabel] = {}
						upperVoicePositions[modeIndex][chordLabel][upperVoicePosition] = 1
					progs[modeIndex][chordLabel][nFormStr] = 1
			bassNote = sortedPitches[0]
			if bassNote.pitchClass == RNpitchClasses[0]:
				foundBass = True
			elif bassNote.pitchClass in RNpitchClasses:
				#print RNoffset, [x.midi for x in sortedPitches], bassNote.pitchClass, RNpitchClasses[0]
				chordalSkipError = True
		try:
			mNumber = measures.getElementsByOffset(RNoffset, RNoffset, includeEndBoundary = True, mustBeginInSpan = False)[0].number
		except:
			mNumber = '-1'
		if isSeventh and not foundSeventh:
			print('MISSING SEVENTH: Chorale', n, 'Measure', mNumber, 'Roman Numeral', chordLabel)
			if capture: 
				message = [str(n) + 'm' + str(mNumber), keyStr + chordLabel + ' (7th)']
				display_problem_measures(chorale, RNoffset, RNbeat, message)
		if foundFourNoteChord and not foundInstance: 
			print('Chorale', n, 'Measure', mNumber, 'Roman Numeral', chordLabel)
			if i > 0 and i < len(anal) - 1:
				if globalSource == 'PALESTRINA':
					if anal[i].bass().name == anal[i+1].bass().name:
						missingChordString = chordLabel + ' -> ' + anal[i+1].figure
					elif anal[i].bass().name == anal[i-1].bass().name:
						missingChordString = anal[i-1].figure + ' -> ' + chordLabel
					else:
						missingChordString = anal[i-1].figure + ' -> ' + chordLabel + ' -> ' + anal[i+1].figure
					print('     ', missingChordString)
					missingChords[missingChordString] = missingChords.setdefault(missingChordString, 0) + 1
				elif globalSource == 'BACH':
					if anal[i].root().name == anal[i+1].root().name:
						missingChordString = chordLabel + ' -> ' + anal[i+1].figure
					elif anal[i].root().name == anal[i-1].root().name:
						missingChordString = anal[i-1].figure + ' -> ' + chordLabel
					else:
						missingChordString = anal[i-1].figure + ' -> ' + chordLabel + ' -> ' + anal[i+1].figure
					print('     ', missingChordString)
					missingChords[missingChordString] = missingChords.setdefault(missingChordString, 0) + 1
			if capture: 
				message = [str(n) + 'm' + str(mNumber), keyStr + chordLabel]
				display_problem_measures(chorale, RNoffset, RNbeat, message)
		if foundFourNoteChord and not foundBass:
			print('BASS ERROR: Chorale', n, 'Measure', mNumber, 'Roman Numeral', chordLabel, 'Expecting', RNpitchClasses[0])
			if capture:
				message = [str(n) + 'm' + str(mNumber), keyStr + chordLabel + ' (bass)']
				display_problem_measures(chorale, RNoffset, RNbeat, message)
		if chordalSkipError:
			if globalPrintChordalSkips: print('CHORDAL SKIP ERROR: Piece', n, 'Measure', mNumber, 'Roman Numeral', chordLabel, 'Expecting', RNpitchClasses[0])
			if capture:
				message = [str(n) + 'm' + str(mNumber), keyStr + chordLabel + ' (CS)']
				display_problem_measures(chorale, RNoffset, RNbeat, message)	
	return

def categorize_voicing(pitchList):
	if pitchList[0] < pitchList[1] < pitchList[2]:
		if pitchList[2] == pitchList[0] + 12: return 'H'
		elif pitchList[2] - pitchList[0] < 12: return 'C'
		elif pitchList[0] < pitchList[2] - 12 < pitchList[1] and (pitchList[1] - pitchList[0]) < 12: return 'O'
		elif (pitchList[0] + 12 == pitchList[1] or pitchList[1] + 12 == pitchList[2]) and (pitchList[2] - pitchList[0]) < 24: return 'OCT+'
	elif (pitchList[0] == pitchList[1] or pitchList[1] == pitchList[2]) and (pitchList[2] - pitchList[0]) < 12: return 'DI'
	return '?'
	
def count(mode = 'major'):															# augmented triads don't get analyzed right
	global progs																	# dochorale needs fixing
	percent = True
	if mode[0:3].upper() == 'MAJ':
		modeIndex = 0
		romanNumerals = ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']
	else:
		modeIndex = 1
		romanNumerals = ['i', 'iio', 'ii', 'III', 'iv', 'IV', 'v', 'V', 'VI', 'vio', 'VII', 'viio']
	print("\t", "\t".join(['root', 'third', 'fifth']))
	for rootStr in romanNumerals:
		for inversion in ['', '6', '6/4']:
			found = True
			progStr = rootStr + inversion
			try:
				myDict = progs[modeIndex][progStr]
			except:
				print(progStr, "not found")
				found = False
			root = 0
			third = 0
			fifth = 0
			for invStr in ['[0, 0, 4, 7]', '[0, 0, 3, 7]', '[0, 0, 3, 6]']:
				if invStr in myDict: root += myDict[invStr]
			for invStr in ['[0, 4, 4, 7]', '[0, 3, 3, 7]', '[0, 3, 3, 6]']:
				if invStr in myDict: third += myDict[invStr]
			for invStr in ['[0, 4, 7, 7]', '[0, 3, 7, 7]', '[0, 3, 6, 6]']:
				if invStr in myDict: fifth += myDict[invStr]
			total = root + third + fifth
			if found and total != 0: print(progStr, '\t', '\t'.join([str(round(x*100./total, 0)) for x in [root, third, fifth]]))
	return

def count2(mode = 'major'):															# augmented triads don't get analyzed right
	global progs																	# dochorale needs fixing
	percent = True
	if mode[0:3].upper() == 'MAJ':
		modeIndex = 0
		romanNumerals = ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']
	else:
		modeIndex = 1
		romanNumerals = ['i', 'iio', 'ii', 'III', 'iv', 'IV', 'v', 'V', 'VI', 'vio', 'VII', 'viio']
	print("\t", "\t".join(['bass', 'third/fourth', 'fifth/sixth']))
	for rootStr in romanNumerals:
		for inversion in ['', '6', '6/4']:
			found = True
			progStr = rootStr + inversion
			try:
				myDict = progs[modeIndex][progStr]
			except:
				print(progStr, "not found")
				found = False
			root = 0
			third = 0
			fifth = 0
			if not inversion:
				for invStr in ['[0, 0, 4, 7]', '[0, 0, 3, 7]', '[0, 0, 3, 6]']:
					if invStr in myDict: root += myDict[invStr]
				for invStr in ['[0, 4, 4, 7]', '[0, 3, 3, 7]', '[0, 3, 3, 6]']:
					if invStr in myDict: third += myDict[invStr]
				for invStr in ['[0, 4, 7, 7]', '[0, 3, 7, 7]', '[0, 3, 6, 6]']:
					if invStr in myDict: fifth += myDict[invStr]
			elif inversion == '6':
				for invStr in ['[0, 0, 4, 7]', '[0, 0, 3, 7]', '[0, 0, 3, 6]']:
					if invStr in myDict: fifth += myDict[invStr]
				for invStr in ['[0, 4, 4, 7]', '[0, 3, 3, 7]', '[0, 3, 3, 6]']:
					if invStr in myDict: root += myDict[invStr]
				for invStr in ['[0, 4, 7, 7]', '[0, 3, 7, 7]', '[0, 3, 6, 6]']:
					if invStr in myDict: third += myDict[invStr]
			elif inversion == '6/4':
				for invStr in ['[0, 0, 4, 7]', '[0, 0, 3, 7]', '[0, 0, 3, 6]']:
					if invStr in myDict: third += myDict[invStr]
				for invStr in ['[0, 4, 4, 7]', '[0, 3, 3, 7]', '[0, 3, 3, 6]']:
					if invStr in myDict: fifth += myDict[invStr]
				for invStr in ['[0, 4, 7, 7]', '[0, 3, 7, 7]', '[0, 3, 6, 6]']:
					if invStr in myDict: root += myDict[invStr]
			total = root + third + fifth
			if total == 0: total = 1
			if found: print(progStr, '\t', '\t'.join([str(round(x*100./total, 0)).replace('.0', '') for x in [root, third, fifth]]))
	return


def show(progStr, mode = 'major', size = 0, percent = False):
	global progs
	if mode[0:3].upper() == 'MAJ':
		modeIndex = 0
	else:
		modeIndex = 1
	try:
		myList = sorted(progs[modeIndex][progStr].iteritems(), key=lambda x:-x[1])
	except:
		myList = []
		print("Not found")
	if size:
		for thing in reversed(myList):
			#print thing, thing[0].count(',')
			if thing[0].count(',') + 1 != size:
				myList.remove(thing)
	if percent:
		total = 0
		for name, count in myList:
			total += count
		for i in range(len(myList)):
			myList[i] = (myList[i][0], 100.0 * myList[i][1]/total)
	for thing in myList:
		print(thing)
	return

def display_problem_measures(chorale, offset, RNbeat, message):  # call this with the offset of the desired measure
	global output, pTreb, pBass
	convert_measure(0, 1, chorale, pTreb, offset, RNbeat, message[0], topstaff = True)
	convert_measure(2, 3, chorale, pBass, offset, RNbeat, message[1])
	return

def convert_measure(i, j, source, dest, offset, RNbeat, message, topstaff = False):				# offsets here look strange!!!!
	v1 = stream.Voice()
	v2 = stream.Voice()
	m1 = source.parts[i].getElementsByOffset(offset, offset + .05, classList = ['Measure'], mustBeginInSpan = False)
	mInd = source.parts[i].index(m1[0])
	ts = source.parts[i][mInd].getContextByClass('TimeSignature')
	m2 = source.parts[j].getElementsByOffset(offset, offset + .05, classList = ['Measure'], mustBeginInSpan = False)
	newOffset = RNbeat	
	for e in m1[0].elements:
		if 'Note' in e.classes:
			e.stemDirection = 'up'
			if e.beams: e.beams.beamsList = []
			if i == 2 and e.expressions: e.expressions = []									# ask Myke how to set Fermata vertical position
		v1.insert(e.offset, e)
	for e in m2[0].elements:
		if 'Note' in e.classes:
			e.stemDirection = 'down'
			if e.beams: e.beams.beamsList = []
			if e.expressions: e.expressions = []
		v2.insert(e.offset, e)
	m = stream.Measure()
	m.timeSignature = ts
	m.insert(0, v1)
	m.insert(0, v2)
	m.makeBeams(inPlace = True)																# why does this not work?
	if topstaff:
		labelOne = expressions.TextExpression(message)
		labelOne.positionVertical = 55
		m.insert(0, labelOne)
	else:
		labelOne = expressions.TextExpression(message)
		labelOne.positionVertical = -85
		labelOne.justify = 'center'
		m.insert(newOffset, labelOne)
	dest.append(m)

def incomplete(progStr, mode = 'major', size = 0, percent = True):
	global progs
	if mode[0:3].upper() == 'MAJ':
		modeIndex = 0
	else:
		modeIndex = 1
	try:
		myList = sorted(progs[modeIndex][progStr].iteritems(), key=lambda x:-x[1])
	except:
		myList = []
		print(progStr, "not found")
	if size:
		for thing in reversed(myList):
			if thing[0].count(',') + 1 != size:
				myList.remove(thing)
	incCount = 0
	for thing in reversed(myList):
		intList = len(set([int(x) for x in thing[0][1:-1].split(', ')]))
		if intList < 3: incCount += thing[1]
	if percent:
		total = 0
		for name, count in myList:
			total += count
		if total != 0: 
			print(progStr, 100.0 * incCount/total)
	return

def inc(size = 0):
	for item in DT._DIATONICMAJOR:
		if DT.parse_figure(item)[1] in ['', '6', '6/4']:
			incomplete(item, 'major', size)
			
def OUCH_report(mode = 'major'):
	modeInd = int(mode.upper()[:2] == 'MI')
	for RN in ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']:
		for inv in ['', '6', '6/4']:
			myLabel = RN + inv
			if myLabel not in upperVoicePositions[modeInd]: continue
			complete = 0
			incomplete = 0
			for pos in ['C', 'O']:
				if pos in upperVoicePositions[modeInd][myLabel]:
					complete += upperVoicePositions[modeInd][myLabel][pos]
			for pos in ['DI', 'H', 'OCT+']:
				if pos in upperVoicePositions[modeInd][myLabel]:
					incomplete += upperVoicePositions[modeInd][myLabel][pos]
			if complete + incomplete > 0:
				print(myLabel, 100.*complete/(complete+incomplete), 100.*incomplete/(complete+incomplete))

def positions(mode = 'major'):
	modeInd = int(mode.upper()[:2] == 'MI')
	tempStr = '\t'
	for label in ['DI', 'C', 'H', 'O', 'OCT+']:
		tempStr += '\t' + label
	print(tempStr)
	for RN in ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']:
		for inv in ['', '6', '6/4']:
			myLabel = RN + inv
			if myLabel not in upperVoicePositions[modeInd]: continue
			tempList = []
			tempStr = myLabel + '\t'
			total = 0
			for pos in ['DI', 'C', 'H', 'O', 'OCT+']:
				if pos in upperVoicePositions[modeInd][myLabel]:
					total += upperVoicePositions[modeInd][myLabel][pos]
					tempList.append(upperVoicePositions[modeInd][myLabel][pos])
				else:
					tempList.append(0)
			tempList.append(tempList[1] + tempList[3])
			tempList.append(tempList[0] + tempList[2] + tempList[4])
			if total != 0:
				tempList = [100.*x/total for x in tempList]
			for result in tempList:
				tempStr += ' \t' + ('%.1f' % result)
			print(tempStr)

def filter_length(l = 4, printIt = True):
	global working, spacingDict, spacingChords
	working = {}
	for i in spacingDict:
		if len(i.split()) == (l + 1):
			working[i] = spacingDict[i]
	if printIt:
		DT.print_dict(working)

def count_spacing(theChord):
	theChord = sorted(theChord)
	chordContent = sorted(list(set([x % 12 for x in theChord])))
	chordContent = make_stack_of_thirds(chordContent)
	#print(theChord, chordContent)
	if not chordContent:
		return '?'
	cardinality = len(chordContent)
	bottomNote = min(theChord)
	bottomPC = bottomNote % 12
	currentOctave = int(bottomNote / 12)
	rotations = 0
	while chordContent[0] % 12 != bottomPC:
		chordContent = rotate_and_transpose(chordContent)
		rotations += 1
	if chordContent[0] >= 12:
		chordContent = [x - 12 for x in chordContent]
	tempList = [x + (12 * currentOctave) for x in chordContent]
	currentOctave += 1
	topNote = max(theChord)
	while tempList[-1] < topNote:
		tempList += [x + (12 * currentOctave) for x in chordContent]
		currentOctave += 1
	while tempList[-1] > topNote:
		tempList = tempList[:-1]
	if rotations > 3: rotations = 4
	indexes = [tempList.index(x) for x in theChord]
	distances = [indexes[i] - indexes[i-1] for i in range(1, len(indexes))]
	strDistances = ' '.join([str(x) for x in distances])
	return str(cardinality) + ' ' + ['R', 'T', 'F', 'S', '?'][rotations] + ' ' + strDistances

def is_stack_of_thirds(theChord):
	distances = set([theChord[i] - theChord[i-1] for i in range(1, len(theChord))])
	if distances <= set([3, 4]):
		return True

def make_stack_of_thirds(theChord):
	for i in range(len(theChord)):
		if is_stack_of_thirds(theChord):
			return theChord
		else:
			theChord = rotate_and_transpose(theChord)
	return False
			
def rotate_and_transpose(theChord):
	return theChord[1:] + [theChord[0] + 12]