from __future__ import print_function
import os
import copy
import time
import pickle

from music21 import *

import DT

counts = [{}, {}]
locations = [{}, {}]
output = stream.Score()
lastChorale = 0
chorale = stream.Score()
analyzed_file = stream.Score()
targetMeter = '4/4'

try:
	storedFile = open (DT._DATAPATH + 'EIGHTHNOTE4TS44BACH CHORALES RAWMACHINE.pkl', 'rb')
	currentSize = 4
	counts = pickle.load(storedFile)
	storedFile.close()
except:
	print("Can't find stored file; run gather stats")


"""
Catalogue RN progressions in chorales by rhythmic position; looking at 4/4 chorales.  Used for automatic analysis, not meant for humans.
	
	1. Remove duplications in the stats generation?
	
TO DO:
	1. Gather all lengths from 1 to 5 at once
	2. Make it possible to gather all the files in a directory 
	
	>>> import os
	>>> os.listdir(DT._HUMANBEETHOVEN)
	['.DS_Store', 'sonata01-1.txt', 'sonata16-1.txt', 'sonata16-2.txt', 'sonata16-3.txt']
	>>> import glob
	>>> glob.glob(DT._HUMANBEETHOVEN + '/*.txt')
	
"""

def gather_stats(source='Bach Chorales RAWMACHINE', n = 1, m = 372, size = 4, meter='4/4', step = .5):
	global counts, locations, currentSize, analyzed_file, targetMeter
	global quarterChords, eighthEighthChords
	quarterChords = 0
	targetMeter = meter
	recordResults = False
	eighthEighthChords = 0
	counts = [{}, {}]
	locations = [{}, {}]
	currentSize = size
	startT = time.time()
	fileStr = source.upper()
	filePath = DT._PATHS[source][0]
	files = sorted(os.listdir(filePath))
	recordResults = True
	for myFile in files:
		if myFile[-3:] == 'txt':
			print('Analyzing', myFile)
			analyzed_file = converter.parse(filePath + myFile, format='romantext')
			dochorale(size, step)
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and', int((minutes - int(minutes)) * 60), 'seconds')
	if recordResults:
		if source == 'BachMechanical': fileStr += 'MACHINE'
		if source == 'BachClean': fileStr += 'CLEANMACHINE'
		outputFile = open (DT._DATAPATH + 'EIGHTHNOTE' + str(size)+'TS' + targetMeter.replace('/', '') + fileStr + '.pkl', 'wb')
		pickle.dump(counts, outputFile)
		outputFile.close()
	return

def dochorale(size, step):							# size is the length of the progstring, step is the length of the step
	global counts, locations, output, pTreb, pBass, chorale, analyzed_file, targetMeter
	global quarterChords, eighthEighthChords
	anal = analyzed_file.flat.getElementsByClass(['RomanNumeral', 'RomanTextUnprocessedToken'])
	#anal.show('text')
	pointer = 0
	endOffset = int(anal.highestTime - (step * size))
	for pointer in range(endOffset):
		pointer += .5
		
		RN = []
		stopRecording = False
		ourKey = []
		progStr = ''
		
		"""Main loop: breaks the recording process if we have a phrase marker ||, an unknown key sign ?: or a key change"""
		
		for j in range(0, size):
			nextPointer = pointer + (j * step)
			RN = anal.getElementAtOrBefore(nextPointer)
			if not RN:
				stopRecording = True
				break
			if 'RomanTextUnprocessedToken' in RN.classes: 
				stopRecording = True
				break
			elif RN.figure == '?:':
				stopRecording = True
				break
			elif RN.getContextByClass('TimeSignature').ratioString != targetMeter:
				stopRecording = True
				break
			else:
				if not ourKey:
					ourKey = [RN.key.tonic, RN.key.mode]
					progStr = RN.figure
				else:
					if [RN.key.tonic, RN.key.mode] == ourKey:
						progStr = progStr + ' -> ' + RN.figure
						endOffset = RN.offset
					else:
						stopRecording = True
						break
		if stopRecording: 
			continue
		
		#print pointer, progStr
		
		dictIndex = int(ourKey[1] == 'minor')			# 0 for major, 1 for minor
		if progStr in counts[dictIndex]:
			counts[dictIndex][progStr] += 1
		else:
			counts[dictIndex][progStr] = 1
	return

def load(comp = 'Bach', size = 4, meter = '4/4'):
	global counts, locations, currentSize
	fileStr = comp.upper()
	currentSize = size
	with open (DT._DATAPATH + 'EIGHTHNOTE' + str(size) + 'TS' + meter.replace('/', '') + fileStr + '.pkl', 'rb') as storedFile:
		counts = pickle.load(storedFile)

def starts(firstChord, mode = 'major'):
	results = []										
	modeInd = (mode[0:3].lower == 'min')
	for prog in counts[modeInd]:
		if prog[0:len(firstChord)] == firstChord:
			results.append([prog, counts[modeInd][prog]])
	tempList = sorted(results, key=lambda x:-x[1])
	tempCounts = 0
	for pair in tempList: 
		print(pair)
		tempCounts += pair[1]
	print(tempCounts)
	return

def ends(lastChord, mode = 'major'):
	results = []
	modeInd = (mode[0:3].lower == 'min')
	for prog in counts[modeInd]:
		if prog[-len(lastChord):] == lastChord:
			results.append([prog, counts[modeInd][prog]])
	tempList = sorted(results, key=lambda x:-x[1])
	tempCounts = 0
	for pair in tempList: 
		print(pair)
		tempCounts += pair[1]
	print(tempCounts)
	return


