from __future__ import print_function
import os
import pickle
import copy
import collections
import re

import matplotlib.pyplot as plt
from music21 import *
from scipy.stats.stats import pearsonr
import numpy


import DT
import rn
import vl
import sd
import nht
import doublings as db
import modulations
import reversals as rev

"""deal with these"""

try:
	import survey
except:
	pass

try:
	import raw_sonority
except:
	pass
	
try:
	import firstordertest
except:
	pass

"""

A giant mess of a program that graphs and consolidates output of other programs, to look at large-scale historical trends.  
	
Basically only used for displaying data.
	
The name comes from the fact that I used it to plot the historical frequency of root progressions, scale-degree progressions, etc.

"""

comps = DT.comps[:]

def sort_by_grammar(mode = 'major'):
	global comps
	modeInd = int(mode.upper()[:3] == 'MIN')
	comps = sorted(comps, key = lambda x: grammarValues[modeInd][x], reverse = True)

def sort_by_year():
	comps = sorted(comps, key = lambda x: DT.birthyears[x])

def trim(composers = ['Willaert2', 'Tallis2'], before = 0, after = 0, useSurvey = False):
	global comps
	if useSurvey:
		comps = survey.myComps
	if type(composers) is str:
		composers = [composers]
	if before:
		if type(before) is str:
			before = DT.birthyears[before]
		for item in DT.comps:
			if DT.birthyears[item] >= before and item in comps:
				comps.remove(item)
	if after:
		if type(after) is str:
			after = DT.birthyears[after]
		for item in DT.comps:
			if DT.birthyears[item] <= after and item in comps:
				comps.remove(item)
	for item in composers:
		newItem = get_comp(item)
		if newItem in comps:
			comps.remove(newItem)

def untrim():
	global comps
	comps = DT.comps[:]

def grammar_correlation(mode = 'major'):
	global comps
	modeInd = int(mode.upper()[:3] == 'MIN')
	gVal = []
	sVal = []
	for c in survey.averages:
		gVal.append(grammarValues[modeInd][c])
		sVal.append(survey.averages[c])
	print("correlation between grammar and survey", pearsonr(sVal, gVal))

"""def load(size = 1, dataType = 'CHORD'):
	dataType = dataType.upper()
	mySize = size
	if dataType[0] == 'C':
		dataType = 'CHORD'
		f = make_chord
	elif dataType[0] == 'B':
		dataType = 'BASS'
	try:
		inputFile = open (DT._DATAPATH + 'bigDegreeDict' + dataType + str(mySize) +'.pkl', 'rb')
		degreeDict = pickle.load(inputFile)
		sizes = pickle.load(inputFile)
		inputFile.close()
	except:
		print("Initializing, please wait!")
		f(mySize)

def make_bass(size = 2):
	for c in comps:
		rn.load(c)
		rn.get_size(size)
		rn.make_bass_dict()
		rn.rank_bass_motions(True, False)
		littleDict = copy.deepcopy(rn.rankedDict[0])
		degreeDict[c] = littleDict
		sizes[c] = DT.sum_dict(degreeDict[c])
	outputFile = open (DT._DATAPATH + 'bigDegreeDictBASS' + str(size) + '.pkl', 'wb')
	pickle.dump(degreeDict, outputFile)
	pickle.dump(sizes, outputFile)
	outputFile.close()

def make_chord(size = 2):
	for c in comps:
		rn.load(c)
		rn.get_size(size)
		rn.remove_inversions(printIt=False)
		littleDict = copy.deepcopy(rn.reduced)
		degreeDict[c] = littleDict
		sizes[c] = DT.sum_dict(degreeDict[c])
	outputFile = open (DT._DATAPATH + 'bigDegreeDictCHORD' + str(size) + '.pkl', 'wb')
	pickle.dump(degreeDict, outputFile)
	pickle.dump(sizes, outputFile)
	outputFile.close()

def make_raw(size = 2):
	for c in comps:
		rn.load(c)
		rn.get_size(size)
		littleDict = copy.deepcopy(rn.working[0])
		degreeDict[c] = littleDict
		sizes[c] = DT.sum_dict(degreeDict[c])

try:
	inputFile = open (DT._DATAPATH + 'bigDegreeDictCHORD1.pkl', 'rb')
	degreeDict = pickle.load(inputFile)
	sizes = pickle.load(inputFile)
	inputFile.close()
except:
	print("Initializing, please wait!")
	make_chord(1)"""

def set_font_size(i):
	if i > 30: return 16
	elif i > 25: return 18

def grammar(mode = 'BOTH', three = True, graph = True, filterDiatonic = True, filterSecondaries = True, includeMixolydian = False, version = ['Dmitri'], save = False, dates = False):
	global comps, grammarValues
	styleStrings = ['-', '--', '-','--']
	widths = [1, 2, 1, 2]
	markers = ['', '', '', '']
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	if save:
		saveFile = open(DT._DATAPATH + 'historyGrammar.pkl', 'wb')
		grammarSave = [{}, {}]
		mode = 'BOTH'
	if type(version) is str:
		if version.upper() == 'ALL':
			version = ['Dmitri', 'K&P', 'Riemann', 'Rameau']
		else:
			version = [version]
	if graph:
		plt.xlabel('Composers', fontsize = 18)
		plt.ylabel('Percent of progressions that conform to the grammar', fontsize = 18)
		plt.title('Harmonic Grammar', fontsize = 24)
	globalThree = three
	legendList = []
	if mode.upper() == 'BOTH':
		globalMode = ['major', 'minor']
	else:
		globalMode = [mode]
	bigCounter = 0
	for modeInd, myMode in enumerate(globalMode):
		values = {}
		for gram in version:
			values[gram] = []
		for c in comps:
			rn.load(c)
			for gram in version:
				tempValue = rn.grammar(myMode, three = globalThree, filterDiatonic = filterDiatonic, includeMixolydian = includeMixolydian, filterSecondaries = filterSecondaries, printOut = False, version = gram)
				values[gram].append(tempValue)
				if save and gram == 'Dmitri':
					grammarSave[modeInd][c] = tempValue
				"""if c == 'Cavalieri':
					values[gram].append(rn.grammar(myMode, globalThree, filterDiatonic = False, seven = False, printOut = False, version = gram))
				else:
					values[gram].append(rn.grammar(myMode, globalThree, filterDiatonic = False, filterSecondaries = True, printOut = False, version = gram))"""
				#print c, myMode, values[gram][-1]
		if graph: 
			for gram in version:
				if len(globalMode) > 1:
					plt.plot(values[gram], label = gram + ' ' + myMode, linestyle=styleStrings[bigCounter], marker = markers[bigCounter], linewidth = widths[bigCounter])
					legendList.append(gram + ' ' + myMode)
				else:
					plt.plot(values[gram], label = gram, linestyle=styleStrings[bigCounter], marker = markers[bigCounter], linewidth = widths[bigCounter])
					legendList.append(gram)					
				bigCounter += 1
	if save:
		pickle.dump(grammarSave, saveFile)
		saveFile.close()
	if graph: 
		#plt.ylim(0, 100)
		ticks = range(len(comps))
		grammarValues = values
		myLeg = plt.legend(tuple(legendList), loc=0, fontsize = 18)
		for i in range(len(myLeg.legendHandles)):
			myLeg.legendHandles[i].set_linewidth(2.0)
		plt.xticks(rotation=90)
		plt.xticks(ticks, comps)
		#plt.xticks(ticks, addbreaks(comps, addBirthYears = dates), fontsize = 12)
		ax.set_xlim(0, len(comps) -1)
		#plt.plot((0, len(comps) - 1), (71.1, 71.1), 'blackf')								# for palestrina graph
		#plt.plot((0, len(comps) - 1), (79.75, 79.75), 'black')								# ditto!
		plt.show()

def ratio(inList):
	global comps
	plt.xlabel('Composers')
	plt.ylabel('First chord / Total (as a percentage)')
	plt.title('Harmony Through History')
	if type(inList) is str:
		inList = [inList]
	for degStr in inList:
		values = []
		for c in comps:
			rn.load(c)
			total = 0
			numerator = 0
			numeratorDenominator = degStr.split(' / ')
			if len(numeratorDenominator) != 2: return
			rn.get_size(numeratorDenominator[0].count(' ->') + 1)
			rn.remove_inversions(printIt=False)
			myDict = rn.reduced
			for i in [0, 1]:
				progs = numeratorDenominator[i].split(' + ')
				for prog in progs:
					if prog in myDict:
						total += myDict[prog]
						if i == 0:
							numerator += myDict[prog]
			if total != 0:
				values.append(100.0*numerator/total)
			else:
				values.append(0)
		plt.plot(values, label = degStr)
	myLeg = plt.legend(tuple(inList), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(comps))
	labels= comps
	plt.xticks(ticks, labels)
	plt.show()

def meeus(requireInversions = []):		# Meeus type ratio of strong to weak forms of the root motion
	global comps, composerCounts
	plt.xlabel('Composers')
	plt.ylabel('Ratio')
	plt.title('Harmony Through History')
	composerCounts = {}
	values = []
	inList = []
	for c in comps:
		total = 0
		numerator = 0
		rn.load(c)
		rn.get_size(2, fauxbourdon = True)
		rn.make_root_dict(requireInversions = requireInversions)
		composerCounts[c] = copy.deepcopy(rn.cleanCounts[0])
	for i in [1, -2, 3]:
		values = []
		degStr = '0 -> ' + str((i) % 7)
		degStr1 = '0 -> ' + str((i * -1) % 7)
		inList.append(degStr)
		for c in comps:
			values.append(100.*composerCounts[c][degStr]/(composerCounts[c][degStr] + composerCounts[c][degStr1]))
		plt.plot(values, label = inList[-1])
	myLeg = plt.legend(tuple(inList), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(comps))
	labels= comps
	plt.xticks(ticks, addbreaks(labels))
	plt.show()

def root_ratios(size = 2, intervals = [1, 2, 3, 4, 5, 6], requireInversions = False, customLabels = ['ascending step', 'ascending third', 'descending fifth', 'ascending fifth', 'descending third', 'descending step']):			# progressions by root motion
	global comps, composerCounts
	if type(intervals) is int:
		intervals = [intervals]
	plt.xlabel('Composers')
	plt.ylabel('Percent of Total Progressions')
	if not requireInversions:
		plt.title('Root Progression Types (0 -> 1 is ascending step, 0 -> 6 is decending step, etc.)')
	else:
		newInversions = []
		for item in requireInversions:
			if not item:
				newInversions.append('5/3')
			else:
				if item == '*':
					newInversions.append('anything')
				else:
					newInversions.append(item)
		plt.title('Root Progression Types (considering only progressions where the first chord is a ' + newInversions[0] + ' and the second chord is a ' + newInversions[1]+')')
	composerCounts = {}
	values = []
	inList = []
	for c in comps:
		total = 0
		numerator = 0
		rn.load(c)
		rn.get_size(2, fauxbourdon = True)
		rn.make_root_dict(requireInversions = requireInversions)
		composerCounts[c] = copy.deepcopy(rn.cleanCounts[0])
	for i in intervals:
		values = []
		degStr = '0 -> ' + str((i) % 7)
		inList.append(degStr)
		for c in comps:
			x = DT.sum_dict(composerCounts[c])
			if x > 0:
				values.append(100.*composerCounts[c][degStr]/x)
			else:
				values.append(0)
		plt.plot(values, label = inList[-1])
	if not customLabels:
		myLeg = plt.legend(tuple(inList), loc=0)
	else:
		myLeg = plt.legend(tuple(customLabels), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(comps))
	labels= comps
	plt.xticks(ticks, addbreaks(labels))
	plt.show()

def undirected_root_ratios(size = 2):			# progressions by root motion
	global comps, composerCounts
	plt.xlabel('Composers')
	plt.ylabel('Percent of Total Progressions')
	plt.title('Root Progression Types (0 -> 1 is step, 0 -> 2 is third, 0 -> 3 is fourth/fifth)')
	composerCounts = {}
	values = []
	inList = []
	for c in comps:
		print(c)
		total = 0
		numerator = 0
		rn.load(c)
		rn.get_size(2, fauxbourdon = True)
		rn.make_root_dict()
		composerCounts[c] = copy.deepcopy(rn.cleanCounts[0])
	for i in [1, 2, 3]:
		values = []
		degStr = '0 -> ' + str((i) % 7)
		degStr2 = '0 -> ' + str((-i) % 7)
		inList.append(degStr)
		for c in comps:
			values.append(100.*(composerCounts[c][degStr] + composerCounts[c][degStr2])/(DT.sum_dict(composerCounts[c])))
		plt.plot(values, label = inList[-1])
	myLeg = plt.legend(tuple(inList), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(comps))
	labels= comps
	plt.xticks(ticks, addbreaks(labels))
	plt.show()

def regex(myStrings = ["^[^-]*7[^-]*$", "^[^-]*6/5[^-]*$", "^[^-]*4/3[^-]*$", "^[^-]*2[^-]*$"], size = 1, mode = 'MAJOR', customLabels = ['7', '6/5', '4/3', '2']):			# find progressions containing a substring
	global comps, composerCounts
	plt.xlabel('Composers')
	plt.ylabel('Percent of Total Progressions')
	plt.title('Progressions containing ' + ', '.join(myStrings))
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	modeInd = int(mode[:3].upper() == 'MIN')
	values = [[] for x in myStrings]
	for c in comps:
		print(c)
		rn.load(c)
		rn.get_size(size)
		total = sum(rn.working[modeInd].values()) / 100.0
		for i, s in enumerate(myStrings):
			temp = 0
			for p in rn.working[modeInd]:
				if re.match(s, p):
					temp += rn.working[modeInd][p]
			values[i].append(temp/total)
	if not customLabels:
		customLabels = myStrings
	for i in range(len(values)):
		plt.plot(values[i], label = customLabels[i])
	myLeg = plt.legend(tuple(customLabels), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(comps))
	labels= comps
	plt.xticks(ticks, addbreaks(labels))
	ax.set_xlim(0, len(comps) -1)
	plt.show()

"""Find voice leadings containing a substring"""

p1a = [['IV -> I', 'IV -> I6', 'IV6 -> I', 'IV6 -> I6'], ['IV -> I', 'IV -> I6', 'IV6 -> I', 'IV6 -> I6']]
p1b = ['[9, 3', '[9, 3P']
p1labels = ['with or without p.t.', 'with p.t.']

p2a = [['viio6 -> I', 'viio6 -> I6'], ['V -> I', 'V7 -> I']]
p2b = ['[11, 1', '[11, 1']

p3a = [['viio6 -> I', 'viio6 -> I6'], ['V -> I', 'V7 -> I','V -> I6', 'V7 -> I6','V -> I6/4', 'V7 -> I6/4', 'V6 -> I', 'V6/5 -> I', 'V6/4 -> I', 'V6/4 -> I6', 'V4/3 -> I', 'V4/3 -> I6', 'V2 -> I6']]
p3b = ['[11, 1', '[11, 1']

def voice_leading(progLists = [], vlLists = [], mode = 'MAJOR', noS = False, printIt = False, labels = [], cutoff = 25, combinePairs = []):
	"""
	search for specific voice motions in the voice leadings used to express a particular progression
	cutoff is the minimum number of progressions
	"""
	global comps, composerCounts, lastValues, values, showValues
	if type(progLists) is str:
		progLists = [progLists]
	if type(vlLists) is str:
		vlLists = [vlLists]
	if len(progLists) == 1 and len(vlLists) > 1:
		progLists = progLists * len(vlLists)
	elif len(progLists) > 1 and len(vlLists) == 1:
		vlLists = vlLists * len(progLists)
	tempComps = []
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers')
	plt.ylabel('Percent of all progressions')
	plt.title('Progressions containing the voice-leading motion')
	modeInd = int(mode[:3].upper() == 'MIN')
	
	numLines = len(progLists)
	values = [[] for x in range(numLines)]
	totals = [[] for x in range(numLines)]
	
	results = {}
	
	for c in comps:
		try:
			vl.load(c)
		except:
			continue
		results[c] = [[], []] # found, total
		for i in range(numLines):
			if progLists[i].count('*') == 0 and progLists[i].count('(') == 0:
				found, total = vl.contains(progLists[i], vlLists[i], ['major', 'minor'][modeInd], printIt, noS = noS)
			else:
				found = 0
				total = 0
				for p in vl.progs[modeInd]:
					if are_equal(p, progLists[i]):
						tempFound, tempTotal = vl.contains([p], vlLists[i], ['major', 'minor'][modeInd], printIt,  noS = noS)
						#print p, vlLists[i], tempFound, tempTotal
						found += tempFound
						total += tempTotal
			if total == 0: total += 1
			results[c][0].append(found)
			results[c][1].append(total)
	
	for pair in combinePairs:
		p1 = pair[0]
		p2 = pair[1]
		if p1 in results and p2 in results:
			for i in range(numLines):
				line1 = results[p1][i]
				line2 = results[p2][i]
				results[p1][i] = [line1[j] + line2[j] for j in range(len(line1))]
			del results[p2]
		elif p2 in results:
			results[p1] = results[p2]
			del results[p2]
			
	for c in comps:
		if c not in results: continue
		foundProgs = sum(results[c][1])
		duplicates = len(progLists) - len(set([tuple(x) for x in progLists])) + 1
		if foundProgs/duplicates >= cutoff:
			tempComps.append(c)
			outList = []
			totalProgs = 0
			for i in range(numLines):
				foundPct = results[c][0][i]*100./results[c][1][i]
				totalProgs += results[c][1][i]
				values[i].append(foundPct)
				outList.append(str(round(foundPct, 1)))
			print(c, ' '.join(outList), 'total progs', totalProgs/duplicates)
			
	comps = tempComps
	lastValues = values[-1][:]
	if labels:
		myStrings = labels
	else:
		myStrings = [str(progLists[x]) + ' ' + str(vlLists[x]) for x in range(numLines)]
	showValues = [myStrings, values]
	for i in range(len(values)):
		plt.plot(values[i], label = myStrings[i])
	myLeg = plt.legend(tuple(myStrings), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(tempComps))
	labels = tempComps
	plt.xticks(rotation=90)
	plt.xticks(ticks, labels)
	#plt.xticks(ticks, addbreaks(labels))
	ax.set_xlim(0, len(tempComps) -1)
	plt.show()

def doublings(theChord = 'I6', doublingLists = ['[0, 0, 4, 7]', '[0, 4, 4, 7]', '[0, 4, 7, 7]'], mode = 'MAJOR', printIt = False, labels = ['root', 'third', 'fifth']):
	global comps, composerCounts
	if type(doublingLists) is str:
		doublingLists = [doublingLists]
	tempComps = []
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers')
	plt.ylabel('Percent')
	plt.title('Doublings for ' + theChord)
	modeInd = int(mode[:3].upper() == 'MIN')
	numLines = len(doublingLists)
	values = [[] for x in range(numLines)]
	totals = []
	for c in comps:
		try:
			db.load(c)
		except:
			continue
		if theChord not in db.progs[modeInd]: continue
		print(c)
		totals.append(0)
		for i in range(numLines):
			tempFound = db.progs[modeInd][theChord].setdefault(doublingLists[i], 0)
			totals[-1] += tempFound
			values[i].append(tempFound)
		tempComps.append(c)
	"""STOPPED HERE"""
	if labels:
		myStrings = [labels[x] for x in range(numLines)]
	else:
		myStrings = [theChord + ' ' + doublingLists[x] for x in range(numLines)]
	for i in range(len(totals)):
		if totals[i] == 0: totals[i] = 1
	for myList in values:
		for i in range(len(myList)):
			myList[i] = 100. * myList[i]/totals[i]
	for i in range(len(values)):
		plt.plot(values[i], label = myStrings[i])
	myLeg = plt.legend(tuple(myStrings), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(tempComps))
	labels = tempComps
	plt.xticks(ticks, addbreaks(labels))
	ax.set_xlim(0, len(tempComps) -1)
	plt.show()

def are_equal(prog1, prog2):
	prog1 = prog1.split(' -> ')
	prog2 = prog2.split(' -> ')
	for i in range(len(prog1)):
		if prog1[i] != prog2[i] and prog1[i] != '*' and prog2[i] != '*':
			if prog1[i].split('(')[0] == prog2[i].split('(')[0] or prog1[i].replace('(','').replace(')', '') == prog2[i].replace('(','').replace(')', ''):
				continue
			else:
				return False
	return True
	
def parse_equation(myString):
	return myString.split('+')

def sd_tendencies(sds, mode = 'MAJOR'):
	global comps
	if type(sds) is str: sds = [sds]
	myStrings = sds[:]
	tempComps = []
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers')
	plt.ylabel('Percent of Total Progressions')
	plt.title('Scale degree tendencies')
	modeInd = int(mode[:3].upper() == 'MIN')
	numLines = len(sds)
	sds = [s.split(' +' ) for s in sds]
	values = [[] for x in range(numLines)]
	for c in comps:
		try:
			sd.load(c)
		except:
			continue
		print(c)
		total = 0
		size = sds[0][0].count('>')
		for s in sd.progs[modeInd]:
			if s.count('>') == size:
				total += sum(sd.progs[modeInd][s].values())
		print(total)
		if total == 0: total += 1
		for i, l in enumerate(sds):
			found = 0
			for s in l:
				if s in sd.progs[modeInd]:
					found += sum(sd.progs[modeInd][s].values())
			values[i].append(100.*found/total)
		print(100.*found/total)
			#print c, 100.*found/total
		tempComps.append(c)
	for i in range(len(values)):
		plt.plot(values[i], label = myStrings[i])
	myLeg = plt.legend(tuple(myStrings), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(tempComps))
	labels = tempComps
	plt.xticks(ticks, addbreaks(labels))
	ax.set_xlim(0, len(tempComps) -1)
	plt.show()

globalGrammar = []

def correlate(useShowValues = False, useSurveyData = False):
	global globalGrammar, comps, lastValues, showValues
	tempDict = {}
	if useSurveyData:
		globalGrammar = []
		for c in comps:
			globalGrammar.append(survey.averages[c])
	elif len(globalGrammar) != len(comps):
		globalGrammar = []
		for c in comps:
			rn.load(c)
			globalGrammar.append(rn.grammar(printOut = False))
	if useShowValues:
		labels, values = showValues
		tValues = []
		for i in range(len(labels)):
			#print values[i], len(values[i])
			#print globalGrammar, len(globalGrammar)
			t = pearsonr(values[i], globalGrammar)
			if numpy.isnan(t[0]):
				continue
			#print labels[i], "CORRELATION", t
			tValues.append(t)
			tempDict[labels[i]] = t
		return tempDict
	elif lastValues:
		#print lastValues, globalGrammar
		t = pearsonr(lastValues, globalGrammar)
		print("CORRELATION", t)
		return t
	else:
		print("NO LAST VALUES!")
		return (0, 0)
	

def best_diagnostic(size = 1, removeInversions = True, mode = 'major', useSurveyData = False, maxTries = 50):
	global diagnostics
	diagnostics = {}
	if useSurveyData:
		load_survey_data()
	modeInd = int(mode[0:2].upper() == 'MI')
	rn.load("Bach")
	if removeInversions:
		rn.remove_inversions(separateSixFour = False)
	rn.get_size(size, useReduced = removeInversions)
	myColl = copy.deepcopy(rn.working[modeInd])
	theKeys = sorted(myColl.keys(), key = lambda x: -myColl[x])[:maxTries]
	show(theKeys, mode = mode, removeInversions = removeInversions, showGraph = False)
	diagnostics = correlate(useShowValues = True, useSurveyData = useSurveyData)
	sortedKeys = sorted(diagnostics.keys(), key = lambda x: -abs(diagnostics[x][0]))
	for c in sortedKeys:
		print(c, diagnostics[c])

def ratio3(chordList = ["I", "V", 'viio'], sValues = False):
	global comps, composerCounts, grammarValues
	grammar(graph = False)
	plt.xlabel('Composers')
	plt.ylabel('Ratio')
	plt.title('Strong progressions vs. weak progressions')
	label1 = 'strong'
	label2 = 'grammatical'
	label3 = 'fifths'
	label4 = ' + '.join(chordList)
	values = []
	values3 = []
	chordCounts = []
	inList = []
	grammarValues = []
	for c in comps:
		total = 0
		numerator = 0
		rn.load(c)
		rn.get_size(2, fauxbourdon=True)
		rn.make_root_dict()
		for i in [1, -2, 3]:
			degStr = '0 -> ' + str((i) % 7)
			if degStr in rn.cleanCounts[0]:
				total += rn.cleanCounts[0][degStr]
		values.append(100.*total/DT.sum_dict(rn.cleanCounts[0]))
		values3.append(100.*(rn.cleanCounts[0]['0 -> 3'] + rn.cleanCounts[0]['0 -> 4'])/DT.sum_dict(rn.cleanCounts[0]))
		grammarValues.append(rn.grammar(printOut = False))
		rn.remove_inversions(separateSixFour=False)
		rn.get_size(1, useReduced=True)
		temp = 0
		totals = sum(rn.working[0].values())
		for myChord in chordList:
			if myChord in rn.working[0]:
				temp += rn.working[0][myChord]
		chordCounts.append(100.0 * temp/totals)
	print('strong vs. grammar', pearsonr(values, grammarValues))
	print('strong vs. fifths', pearsonr(values, values3))
	print('fifths vs. grammar', pearsonr(grammarValues, values3))
	print('chord counts vs. grammar', pearsonr(chordCounts, grammarValues))
	print('chord counts vs. fifths', pearsonr(chordCounts, values3))
	print('chord counts vs. strong', pearsonr(chordCounts, values))
	if sValues:
		print('strong vs. survey', pearsonr(values, surveyValues))
		print('fifths vs. survey', pearsonr(values3, surveyValues))
		print('chord counts vs. survey', pearsonr(chordCounts, surveyValues))
		print('grammar vs. survey', pearsonr(grammarValues, surveyValues))
	plt.plot(values, label = label1)
	plt.plot(grammarValues, label = label2)
	plt.plot(values3, label = label3)
	plt.plot(chordCounts, label=label4)
	myLeg = plt.legend((label1, label2, label3, label4), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(comps))
	labels= comps
	plt.xticks(ticks, addbreaks(labels))
	plt.show()

def load_survey_data():
	global comps, surveyValues
	surveyValues = survey.averages.values()
	comps = survey.averages.keys()

def survey_correlation():
	load_survey_data()
	ratio3(sValues = True)
	untrim()

def all_inversions(size = 2):
	global comps, composerCounts
	plt.xlabel('Composers')
	plt.ylabel('Ratio')
	plt.title('Harmony Through History')
	composerCounts = {}
	values = []
	inList = []
	for c in comps:
		total = 0
		numerator = 0
		rn.load(c)
		rn.get_size(1)
		for s in ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']:
			degStr = s
			degStr1 = s + '6'
			rootVal = 0
			invVal = 0
			if degStr in rn.working[0]:
				rootVal = rn.working[0][degStr]
			if degStr1 in rn.working[0]:
				invVal = rn.working[0][degStr1]
			numerator += invVal
			total += rootVal + invVal
		values.append(100.*numerator/total)
		plt.plot(values)
	ticks = range(len(comps))
	labels= comps
	plt.xticks(ticks, labels)
	plt.show()

"""NEW SHOW ROUTINE THAT WORKS BETTER"""

diaTriads = ['I', 'V', 'IV', 'ii', 'iii', 'vi', 'viio', 'I6/4']

def find_prog(progStr, myDict, basicFunction = False):
	global findProgMinimumCount
	addNumeratorAndDenominator = False
	if progStr.count(' // ') > 0:					# double slash keeps ratios between 0 and 100
		progStr = progStr.replace(' // ', ' / ')
		addNumeratorAndDenominator = True
	#print progStr
	if progStr.count(' / ') > 0:					# need spaces because the slash could be part of the RN
		progSplit = progStr.split(' / ')			# "V -> I / V -> *" is code for "what porportion of V chords go to I, as opposed to the other diatonic triads?"
		for item in progSplit:
			item = item.rstrip(' ').lstrip(' ')
		numerator = find_prog(progSplit[0], myDict)
		denominator = find_prog(progSplit[1], myDict)
		findProgMinimumCount = min(numerator, denominator)
		if addNumeratorAndDenominator:
			if (numerator + denominator) == 0:
				return 0.0
			else:
				return 100.0 * numerator/(numerator + denominator)
		else:	
			if denominator == 0:
				return 0.0
			else:
				return 100.0 * numerator/denominator
	if progStr.count('max(') > 0 and progStr[:4] == 'max(':				# cheap: max(a , b)
		progSplit = progStr[4:-1].split(' , ')
		return max(find_prog(progSplit[0], myDict), find_prog(progSplit[1], myDict))
	if progStr.count(' + ') > 0:
		progSplit = progStr.split(' + ')
		values = []
		for item in progSplit:
			item = item.rstrip(' ').lstrip(' ')
			values.append(find_prog(item, myDict))
		return sum(values)
	if not basicFunction:
		if progStr[-5:] == ' -> *':
			return rn.starts(progStr[:-1], source=myDict, printIt = False)
		elif progStr[:5] == '* -> ':
			forbiddenChords = []
			myStr = progStr[1:]
			if progStr.count("{") > 0:
				tempStr = progStr.split('{')
				myStr = tempStr[0][1:]
				forbiddenChords = [x + myStr for x in tempStr[1][:-1].split(', ')]
			mySum = rn.ends(myStr, source=myDict, printIt = False)
			if forbiddenChords:
				mySum = 0
				for item in rn.tempList:
					if item[0] in forbiddenChords:
						continue
					mySum += item[1]
			return mySum
		findProgMinimumCount = myDict.get(progStr, 0)
		return findProgMinimumCount
	else:
		tempTotal = 0
		for tempChord in myDict:
			parsed = DT.parse_figure(tempChord)
			parsed = parsed[0] + parsed[1]
			if parsed == progStr:
				tempTotal += myDict[tempChord]
		return tempTotal

def show(chords = ['I', 'V', 'IV', 'ii', 'vi', 'iii', 'viio'], mode='major', removeInversions = True, fauxbourdon = True, minimumCount = 0, duplicates = True, filterDiatonic = True, separateSixFour = False, labels = False, showGraph = True):			# add mode Index for major or minor
	global comps, composerCounts, lastValues, showValues
	styleStrings = ['-', '--', '-','--', '-', '--', '-','--', '-', '--', '-','--']
	widths = [1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2]
	markers = ['', '', '', '', '', '', '', '', '', '', '', '']
	modeInd = int(mode[0:2].upper() == 'MI')
	manualLabels = True
	fauxVar = fauxbourdon
	dupVar = duplicates
	minimumFailures = []
	if type(chords) is str:
		chords = [chords]
	values = []
	for i in range(len(chords)):
		values.append([])
	composerCounts = {}
	if not labels:
		labels = []
		manualLabels = False
	if chords[0].count(' // ') == 0:
		mySize = chords[0].split(' / ')[0].split(' + ')[0]
	else:
		mySize = chords[0].split(' // ')[0].split(' + ')[0]
	if mySize.count('max(') > 0:
		mySize = mySize.count(' -> ')
	else:
		mySize = mySize.count(' -> ') + 1
	for c in comps:
		#print c
		rn.load(c)
		if removeInversions: 
			rn.remove_inversions(printIt = False, filterDiatonic = filterDiatonic, separateSixFour = separateSixFour)
			rn.get_size(mySize, fauxbourdon = fauxVar, useReduced = True)
		else:
			rn.get_size(mySize, fauxbourdon = fauxVar, useReduced = False)
			if filterDiatonic:
				rn.strict_filter(modeInd)
		total = DT.sum_dict(rn.working[modeInd])
		for i in range(len(chords)):
			rootVal = 0
			rootVal = find_prog(chords[i], rn.working[modeInd])		
			if minimumCount and findProgMinimumCount < minimumCount:
				print(" FAILED TO MEET MINIMUM COUNT", c, findProgMinimumCount)	
				minimumFailures.append(c)
			if chords[i].count(' / ') == 0 and chords[i].count(' // ') == 0:
				values[i].append(100.0*rootVal/total)
			else:
				values[i].append(rootVal) 
	lastValues = values[-1][:]
	for i in range(len(chords)):
		if not manualLabels:
			myLabel = chords[i]
			labels.append(myLabel)
	showValues = [labels, values]
	if not showGraph:
		return
	fig = plt.figure()
	if minimumFailures:
		print(minimumFailures)
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers', fontsize = 18)
	plt.ylabel('Percentage', fontsize = 18)
	plt.title('Harmony Through History', fontsize = 24)
	for i in range(len(chords)):
		plt.plot(values[i], label = labels[i], linestyle=styleStrings[i], marker = markers[i], linewidth = widths[i])
	ticks = range(len(comps))
	myLeg = plt.legend(tuple(labels), loc=0)	# 0 = best; 4 = lower right, 7 = center right
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.xticks(rotation=90)
	plt.xticks(ticks, comps)
	ax.set_xlim(0, len(comps) -1)
	plt.show()

def inversions(size = 2, sameBass = True, fauxbourdon = False, mode = 'major'):
	global comps, composerCounts
	modeInd = int(mode[:3].upper() == 'MIN')
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers')
	plt.ylabel('Ratio')
	if sameBass:
		plt.title('Proportion of first-inversion chords, by bass scale degree')
	else:
		plt.title('Proportion of first-inversion chords, by root')
	composerCounts = {}
	values = [[],[],[],[],[],[],[]]
	inList = []
	chords = ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']
	if modeInd == 1:
		chords = ['i', 'iio', 'III', 'iv', 'V', 'VI', 'VII']
	labels = []
	for c in comps:
		#print c
		rn.load(c)
		rn.get_size(1, fauxbourdon)
		for i in range(len(chords)):
			degStr = chords[i]
			if sameBass:
				degStr1 = chords[i - 2] + '6'
			else:
				degStr1 = degStr + '6'
			print(degStr, degStr1)
			rootVal = 0
			invVal = 0
			if degStr in rn.working[modeInd]:
				rootVal = rn.working[modeInd][degStr]
			if degStr1 in rn.working[modeInd]:
				invVal = rn.working[modeInd][degStr1]
			if (invVal + rootVal) != 0:
				values[i].append(100.*invVal/(invVal + rootVal))
				print(100.*invVal/(invVal + rootVal))
			else:
				values[i].append(0)
				print(0)
	for i in range(len(chords)):
		if sameBass:
			myLabel = str(i + 1)
		else:
			myLabel = chords[i]
		if len(labels) < 7:
			labels.append(myLabel)
		plt.plot(values[i], label = myLabel)
	ticks = range(len(comps))
	myLeg = plt.legend(tuple(labels), loc=4)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.xticks(rotation=90)
	plt.xticks(ticks, comps)
	ax.set_xlim(0, len(comps) -1)
	plt.show()
	
def chord_proportions(size = 2):			# percentage of all proportions
	global comps, composerCounts
	plt.xlabel('Number of progressions')
	plt.ylabel('Percent of Total Progressions')
	plt.title('How many progressions are used')
	values = []
	for c in comps:
		rn.load(c)
		rn.get_size(size)
		rn.remove_inversions(printIt=False)
		total = DT.sum_dict(rn.working[0])
		myList = []
		for item in rn.working[0]:
			myList.append(rn.working[0][item])
		myList = reversed(sorted(myList))
		newList = []
		tempTotal = 0
		for item in myList:
			tempTotal += item
			newList.append(100.0 * tempTotal/total)	
		plt.plot(newList, label = c)
	myLeg = plt.legend(tuple(comps), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.ylim(50, 100)
	labels= comps
	plt.show()
	
def triads_and_sevenths(mode = 'major', triads = True):
	global comps, composerCounts
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers')
	plt.ylabel('Ratio')
	plt.title('Proportion of inversions')
	composerCounts = {}
	rootTriads = []
	firstInversion = []
	secondInversion = []
	other = []
	seventh = []
	labels = []
	for c in comps:
		rn.load(c)
		rn.get_size(1)
		tempRoot = 0
		tempFirst = 0
		tempSecond = 0
		tempSeventh = 0
		tempOther = 0
		tempTotal = 0
		for myChord in rn.working[0]:
			parsedChord = DT.parse_figure(myChord)
			tempTotal += rn.working[0][myChord]
			if triads:
				if not parsedChord[1]:				# include sevenths in root-pos figures?
					tempRoot += rn.working[0][myChord]
				elif parsedChord[1] == '6':
					tempFirst += rn.working[0][myChord]
				elif parsedChord[1] == '6/4':
					tempSecond += rn.working[0][myChord]
			if parsedChord[1] in ['7', '6/5', '4/3', '4/2', '2']:   #['7', '6/5', '4/3', '4/2', '2']:
				tempSeventh += rn.working[0][myChord]
			elif triads:
				tempOther += rn.working[0][myChord]
		rootTriads.append(100.*tempRoot/tempTotal)
		firstInversion.append(100.*tempFirst/tempTotal)
		secondInversion.append(100.*tempSecond/tempTotal)
		seventh.append(100.*tempSeventh/tempTotal)
		other.append(100.*tempOther/tempTotal)
	plt.plot(rootTriads, label = 'root')
	labels.append('root')
	plt.plot(firstInversion, label = 'first inv.')
	labels.append('first inv.')
	plt.plot(secondInversion, label = 'second inv.')
	labels.append('second inv.')
	plt.plot(seventh, label = 'sevenths')
	labels.append('sevenths')
	plt.plot(other, label = 'other')
	labels.append('other')
	ticks = range(len(comps))
	myLeg = plt.legend(tuple(labels), loc=6)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.xticks(ticks, addbreaks(comps))
	ax.set_xlim(0, len(comps) -1)
	plt.show()

def fakemod7(i):
	return ((i-1) % 7) + 1

"""see http://matplotlib.org/examples/pylab_examples/bar_stacked.html"""
"""def scaleDegrees(c = 'Bach', mode = 'major'):
	global comps, composerCounts
	modeInd = int(mode == 'minor')
	sd.load(c)
	ascents = []
	descents = []
	ind = [1, 2, 3, 4, 5, 6, 7]
	for i in range(7):
		descents.append(sd.counts[modeInd][str(i+1) + ' -> ' + str(fakemod7(i))])
		ascents.append(sd.counts[modeInd][str(i+1) + ' -> ' + str(fakemod7(i+2))])
	p1 = plt.bar(ind, descents, .35, color='r')
	p2 = plt.bar(ind, ascents, .35, color='y', bottom=descents)
	plt.ylabel('Counts')
	plt.title('Scale degree tendencies')
	#plt.xticks(ind+.35/2., ('1', '2', '3', '4', '5', '6', '7') )
	#plt.yticks(np.arange(0,81,10)
	plt.legend( (p1[0], p2[0]), ('descending', 'ascending') )
	plt.show()"""
	
colors = ['r', 'g', 'b', 'y', 'r', 'g', 'b', 'y']

def normalize(myList):
	mySum = sum(myList) * 1.0
	return [x/mySum for x in myList]

def scale_degrees(composers = ['Bach'], size = 1, mode = 'major'):
	global comps, composerCounts
	modeInd = int(mode == 'minor')
	for i in range(len(composers)):
		c = composers[i]
		sd.load(c)
		ascents = []
		descents = []
		ind = [1, 2, 3, 4, 5, 6, 7]
		for j in range(7):
			descents.append(sd.counts[modeInd].setdefault(str(j+1) + ' -> ' + str(fakemod7(j + 1 - size)), 0))
			ascents.append(sd.counts[modeInd].setdefault(str(j+1) + ' -> ' + str(fakemod7(j+1 + size)), 0))
		#p1 = plt.bar(ind, descents, .35, color='r')
		mySum = (sum(ascents) + sum(descents)) * 1.0
		descents = [x/mySum for x in descents]
		ascents = [x/mySum for x in ascents]
		p1 = plt.bar([x + (i *.35) for x in ind], descents, .35, color=colors[i+2])
		p2 = plt.bar([x + (i *.35) for x in ind], ascents, .35, bottom=descents, color=colors[i])
	plt.ylabel('Counts')
	plt.title('Scale degree tendencies')
	#plt.xticks(ind+.35/2., ('1', '2', '3', '4', '5', '6', '7') )
	#plt.yticks(np.arange(0,81,10)
	#plt.legend( (p1[0], p2[0]), ('descending', 'ascending') )
	plt.show()


def scaleDegrees2(composers = ['Bach'], mode = 'major'):
	global comps, composerCounts
	modeInd = int(mode == 'minor')
	for i in range(len(composers)):
		c = composers[i]
		sd.load(c)
		ascents = []
		ind = [1, 2, 3, 4, 5, 6, 7]
		for j in range(7):
			#descents.append(sd.counts[modeInd][str(i+1) + ' -> ' + str(fakemod7(i))])
			ascents.append(sd.counts[modeInd][str(j+1)])
		#p1 = plt.bar(ind, descents, .35, color='r')
		mySum = sum(ascents) * 1.0
		ascents = [x/mySum for x in ascents]
		p2 = plt.bar([x + (i *.35) for x in ind], ascents, .35, color=colors[i])
	plt.ylabel('Counts')
	plt.title('Scale degree tendencies')
	#plt.xticks(ind+.35/2., ('1', '2', '3', '4', '5', '6', '7') )
	#plt.yticks(np.arange(0,81,10)
	#plt.legend( (p1[0], p2[0]), ('descending', 'ascending') )
	plt.show()
	

def roto(size = 2, sameBass = True, fauxbourdon = True):
	global comps, composerCounts
	tempComps = comps[:]
	modeInd = 0
	plt.xlabel('Composers')
	plt.ylabel('Ratio')
	if sameBass:
		plt.title('Proportion of first-inversion chords, by bass scale degree')
	else:
		plt.title('Proportion of first-inversion chords, by root')
	composerCounts = {}
	values = [[],[],[],[],[],[],[]]
	inList = []
	chords = ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']
	labels = []
	for c in tempComps:
		rn.load(c)
		rn.get_size(1, fauxbourdon)
		for i in range(len(chords)):
			degStr = chords[i]
			if sameBass:
				degStr1 = chords[i - 2] + '6'
			else:
				degStr1 = degStr + '6'
			rootVal = 0
			invVal = 0
			if degStr in rn.working[modeInd]:
				rootVal = rn.working[modeInd][degStr]
			if degStr1 in rn.working[modeInd]:
				invVal = rn.working[modeInd][degStr1]
			if (invVal + rootVal) != 0:
				tempVal = 100.*invVal/(invVal + rootVal)
			else:
				tempVal = 0
			values[i].append(tempVal)
	average = []
	for i in range(len(values[1])):
		average.append((values[1][i] + values[3][i] + values[5][i])/3.)		# scale degrees 2, 4, 6 (not 7 because it is diminished)
	for i in range(len(chords)):
		if sameBass:
			myLabel = str(i + 1)
		else:
			myLabel = chords[i]
		if i in [0, 2, 4]:
			labels.append(myLabel)
			plt.plot(values[i], label = myLabel)
	labels.append('2, 4, 6')
	plt.plot(average, label = '2, 4, 6')
	ticks = range(len(tempComps))
	myLeg = plt.legend(tuple(labels), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.xticks(ticks, tempComps)
	plt.show()

def ascending_fifths(mode = 'major', steps=False):
	global comps, composerCounts
	modeInd = int(mode[0:2].upper()=='MI')
	print('MODE', modeInd)
	tempComps = comps
	plt.xlabel('Composers')
	plt.ylabel('Ratio')
	plt.title('Proportion of ascending-fifths progressions IV -> I, I -> V')
	composerCounts = {}
	values = []
	inList = []
	chords = ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']
	labels = []
	for c in comps:
		print(c)
		rn.load(c)
		myVal = rn.ascending_fifths(descendingSteps = steps)
		print(myVal)
		values.append(myVal[modeInd])
	plt.plot(values)
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	plt.show()

def size(mode = 'BOTH', combine = True):
	global comps, composerCounts
	untrim()
	trim(['Rock', 'Bill', 'Beethoven Quartets'])
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers')
	plt.ylabel('Number of chords')
	plt.title('Size of my corpus')
	values = []
	valuesMinor = []
	labels = []
	for c in comps:
		if combine and ('2' in c): continue
		labels.append(c)
		x = rn.count_chords(c, 'major')
		if combine and c+'2' in comps:
			x += rn.count_chords(c+'2', 'major')
		values.append(x)
		y = rn.count_chords(c, 'minor')
		if combine and c+'2' in comps:
			y += rn.count_chords(c+'2', 'major')
		valuesMinor.append(y)
		print(c, x + y)
	values = [x/1000. for x in values]
	valuesMinor = [x/1000. for x in valuesMinor]
	plt.plot(values, label='Major')
	plt.plot(valuesMinor, label='Minor')
	ticks = range(len(labels))
	plt.xticks(rotation=90)
	plt.xticks(ticks, labels)
	#plt.xticks(ticks, addbreaks(comps), fontsize = 10)
	myLeg = plt.legend(tuple(['Major', 'Minor']), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	print(sum(values) + sum(valuesMinor))
	ax.set_xlim(0, len(labels) -1)
	plt.show()

def chord_histo(composers = ['Bach', 'Monteverdi', 'Palestrina', 'Josquin'], mode = 'major', inversions = True, sameLabels = True, addChords = [], customLabels = []):
	global comps, composerCounts
	modeInd = int(mode[0:2].upper()=='MI')
	styleStrings = ['-', '--', '-','--', '-', '--', '-','--']
	widths = [1, 2, 1, 2, 1, 2, 1, 2]
	markers = ['', '', '', '', '', '', '', '', '', '']
	plt.title('Histogram of Chord Popularity', fontsize = 24)
	plt.ylabel('Percentage', fontsize = 18)
	tempComps = [get_comp(c) for c in composers]
	if sameLabels:
		if customLabels:
			plt.xlabel('Chords by popularity in ' + customLabels[0], fontsize = 18)
		else:
			plt.xlabel('Chords by popularity in ' + tempComps[0], fontsize = 18)
	else:
		plt.xlabel('Chords by popularity', fontsize = 18)
	composerCounts = {}
	values = []
	inList = []
	labels = []
	masterLabels = []
	for bigCounter, c in enumerate(tempComps):
		print(c)
		rn.load(c)
		newList = []
		newLabels = []
		tempList = rn.chord_histogram(mode, inversions, addChords)
		if sameLabels:
			if masterLabels:
				for mLabel in masterLabels:
					for item in tempList:
						if item[0] == mLabel:
							newList.append(item[1])
							break
			else:
				for item in tempList:
					newList.append(item[1])
					newLabels.append(item[0])
					masterLabels.append(item[0])
				labels.append(newLabels)
		else:
			for item in tempList:
				newList.append(item[1])
				newLabels.append(item[0])
			labels.append(newLabels)
		plt.plot(newList, label=c, linestyle=styleStrings[bigCounter], marker = markers[bigCounter], linewidth = widths[bigCounter])
	tickLabels = []
	if not sameLabels:
		for i in range(len(labels[0])):			# i = chordnumber or second index
			startChord = ''
			for j in range(len(labels)):		# j = composer index
				startChord = startChord + labels[j][i] + '\n'
			tickLabels.append(startChord)
	else:
		tickLabels = masterLabels
	ticks = range(len(newList))
	plt.xticks(rotation=90)
	plt.xticks(ticks, tickLabels)
	if customLabels:
		myLeg = plt.legend(tuple(customLabels), loc=0)
	else:
		myLeg = plt.legend(tuple(tempComps), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def first_inversion(composers = ['Palestrina', 'Josquin']):
	global comps, composerCounts
	modeInd = 0 								# int(mode[0:2].upper()=='MI')
	styleStrings = ['-', '--', '-','--', '-', '--', '-','--']
	widths = [1, 2, 1, 2, 1, 2, 1, 2]
	markers = ['', '', '', '', '', '', '', '', '', '']
	plt.title('First Inversion Moving Down', fontsize = 24)
	plt.ylabel('Percentage', fontsize = 18)
	tempComps = [get_comp(c) for c in composers]
	plt.xlabel('First Inversion Progressions', fontsize = 18)
	composerCounts = {}
	values = []
	inList = []
	labels = []
	diaLabels = ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']
	for i in range(7):
		labels.append(diaLabels[i-1] + '6 -> ' + diaLabels[i])
	masterLabels = []
	for bigCounter, c in enumerate(tempComps):
		print(c)
		rn.load(c)
		rn.get_size(2)
		newList = []
		totalCounts = sum(rn.working[modeInd].values())
		for prog in labels:
			if prog in rn.working[modeInd]:
				newList.append(100.*rn.working[modeInd][prog] / totalCounts)
			else:
				newList.append(0)
		plt.plot(newList, label=c, linestyle=styleStrings[bigCounter], marker = markers[bigCounter], linewidth = widths[bigCounter])
	ticks = range(len(labels))
	plt.xticks(ticks, labels, fontsize = 18)
	myLeg = plt.legend(tuple(tempComps), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def full_inversion_report(composers = ['Palestrina'], incoming = False):
	global comps, composerCounts
	modeInd = 0 								# int(mode[0:2].upper()=='MI')
	if type(composers) is str:
		composers = [composers]
	styleStrings = ['-', '--', '-','--', '-', '--', '-','--']
	widths = [1, 2, 1, 2, 1, 2, 1, 2]
	markers = ['', '', '', '', '', '', '', '', '', '']
	plt.title('Where do first inversion chords go?', fontsize = 24)
	plt.ylabel('Percentage of all progressions', fontsize = 18)
	tempComps = [get_comp(c) for c in composers]
	plt.xlabel('First Inversion Progressions', fontsize = 18)
	composerCounts = {}
	values = []
	inList = []
	diaLabels = ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']
	fullLabels = [[], [], [], [], [], [], []]
	c = composers[0]
	print(c)
	rn.load(c)
	rn.get_size(2)
	totalCounts = sum(rn.working[modeInd].values())
	for i in range(1, 7):
		labels = []
		rawCounts = []
		for j in range(7):
			if incoming:
				s = diaLabels[j] + ' -> ' + diaLabels[(j + i) % 7] + '6'
			else:
				s = diaLabels[j] + '6 -> ' + diaLabels[(j + i) % 7]
			labels.append(s)
			fullLabels[j].append(s)
		newList = []
		for prog in labels:
			if prog in rn.working[modeInd]:
				newList.append(100.*rn.working[modeInd][prog] / totalCounts)
				rawCounts.append(rn.working[modeInd][prog])
			else:
				newList.append(0)
		plt.plot(newList, label=str(i), linestyle=styleStrings[i], marker = markers[i], linewidth = widths[i])
		print(labels, rawCounts, sum(rawCounts))
	tickLabels = []
	for myList in fullLabels:		
		startChord = ''
		for curLabel in myList:
			startChord = startChord + curLabel + '\n'
		tickLabels.append(startChord)
	plt.xticks(range(len(tickLabels)), tickLabels, fontsize = 14)
	myLeg = plt.legend(tuple(range(1, 8)), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def five_six_five(mode = 'MAJ'):
	global comps, composerCounts
	eliminateComps = []
	modeInd = int(mode[0:2].upper()=='MI')
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers')
	plt.ylabel('Percent')
	plt.title('what proportion of 5-6-5 motions, with the first two chords over the same bass, move down by step rather than up by step')
	values = []
	values2 = []
	neighbors = []
	for c in comps:
		rn.load(c)
		rn.get_size(3)
		totalChords = sum(rn.working[modeInd].values())
		myVal = rn.five_six_five(printIt = False)
		values.append(100.*myVal[0]/totalChords)
		values2.append(100.*myVal[1]/totalChords)
		neighbors.append(100.*myVal[2]/totalChords)
	plt.plot(values)
	plt.plot(values2)
	plt.plot(neighbors)
	tempComps = copy.deepcopy(comps)
	for item in eliminateComps:
		tempComps.remove(item)
	ticks = range(len(tempComps))
	plt.xticks(ticks, addbreaks(tempComps), fontsize = 10)
	#print sum(values) + sum(valuesMinor)
	ax.set_xlim(0, len(tempComps) -1)
	plt.show()


def get_interval(item):
	bassNotes = item.split(' -> ')
	bassDegrees = []
	for thing in bassNotes:
		newString = ''
		for oneChar in thing:
			if oneChar.isdigit(): newString += oneChar
		bassDegrees.append(int(newString) - 1)
	bassInts = []
	for i in range(1, len(bassDegrees)):
		bassInts.append((bassDegrees[i] - bassDegrees[i-1]) % 7)
	return bassInts

def bass_motions(mode = 'major'):		# what kind of bass motions?
	global comps, composerCounts
	modeInd = int(mode[0:2].upper()=='MI')
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('Bass motion by Fifth')
	composerCounts = {}
	values = []
	inList = []
	for c in comps:
		total = 0
		numerator = 0
		tempValues = [0]*7
		rn.load(c)
		rn.get_size(2)
		rn.make_bass_dict()
		for item in rn.bassDictCounts[modeInd]:
			total += rn.bassDictCounts[modeInd][item]
			tempValues[get_interval(item)[0]] += rn.bassDictCounts[modeInd][item]
		for i in range(len(tempValues)):
			tempValues[i] = 100.*tempValues[i]/total
		values.append(tempValues)
	for i in range(len(values[0])):
		tempList = []
		for j in range(len(values)):
			tempList.append(values[j][i])
		plt.plot(tempList, label=str(i))
		inList.append(str(i))
	#plt.plot(values)
	myLeg = plt.legend(tuple(inList), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	plt.show()

def bass_fifths(mode = 'major', direction = 'BOTH'):		# what kind of bass motions?
	global comps
	modeInd = int(mode[0:2].upper()=='MI')
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('Bass motion by fifth vs. root motion by fifth')
	if direction[:2].upper() == 'UP':
		intList = [4]
	else:
		intList = [3]
	composerCounts = {}
	values = []
	grammarValues = []
	inList = []
	for c in comps:
		total = 0
		numerator = 0
		tempValues = [0]*7
		rn.load(c)
		rn.get_size(2)
		rn.make_bass_dict()
		for item in rn.bassDictCounts[modeInd]:
			total += rn.bassDictCounts[modeInd][item]
			tempValues[get_interval(item)[0]] += rn.bassDictCounts[modeInd][item]
		for i in range(len(tempValues)):
			if total != 0:
				tempValues[i] = 100.*tempValues[i]/total
			else:
				tempValues[i] = 0
		values.append(tempValues)
		rn.make_root_dict()
		composerCounts[c] = copy.deepcopy(rn.cleanCounts[0])
		grammarValues.append(rn.grammar(printOut=False))
	for i in intList:
		tempList = []
		for j in range(len(values)):
			if direction != 'BOTH':
				tempList.append(values[j][i])
			else:
				tempList.append(values[j][i] + values[j][7-i])
		plt.plot(tempList, label=str(i))
		inList.append('bass motion')
	for i in intList:
		values = []
		degStr = '0 -> ' + str((i) % 7)
		degStr2 = '0 -> ' + str((-i) % 7)
		inList.append('root motion')
		for c in comps:
			if direction != 'BOTH':
				values.append(100.*composerCounts[c][degStr]/DT.sum_dict(composerCounts[c]))
			else:
				values.append(100.*(composerCounts[c][degStr] + composerCounts[c][degStr2])/(DT.sum_dict(composerCounts[c])))
		plt.plot(values, label = inList[-1])
	print("grammaticality vs. bass-fifths",  r(grammarValues, tempList))
	print("grammaticality vs. root-fifths", pearsonr(grammarValues, values))
	print("bass-fifths vs. root-fifths", pearsonr(values, tempList))
	plt.plot(grammarValues, label = 'grammaticality')
	inList.append('grammaticality')
	myLeg = plt.legend(tuple(inList), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	plt.show()

def tonic_pct():		# what portion of the piece is in the tonic?
	global comps
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('Proportion of chords in the tonic key')
	values = []
	for c in comps:
		myValue = 0
		try:
			modulations.load(c)
			myValue = modulations.tonic_pct()
		except:
			pass
		values.append(myValue)
	plt.plot(values)
	"""myLeg = plt.legend(tuple(inList), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)"""
	ticks = range(len(comps))
	labels= comps
	plt.xticks(ticks, labels)
	plt.show()

def get_comp(c):
	if c in DT.comps: return c
	x = len(c)
	for comp in DT.comps:
		if comp[:x].upper() == c.upper(): return comp
	return c

def cadences(mode = 'major', inversions = False, composers = ['Josquin', 'Palestrina', 'Mozart'], cadenceProg = '-> V -> I', remove = []):		# histogram: what precedes V-I ?
	global comps, composerCounts
	modeInd = int(mode[0:2].upper()=='MI')
	tempProg = cadenceProg.replace(' ', '')
	if tempProg[-1] == '>':
		myFunc = rn.starts
		myIndex = -1
	else:
		myFunc = rn.ends
		myIndex = 0
	plt.xlabel('Chords by popularity')
	plt.ylabel('Percentage')
	plt.title('What ' + ['precedes ', 'follows '][myIndex] + cadenceProg.lstrip(' ->').rstrip(' ->') + '?')
	composerCounts = {}
	values = []
	inList = []
	chords = ['I', 'ii', 'iii', 'IV', 'V', 'vi', 'viio']
	plotValues = []
	labels = []
	tempComps = [get_comp(c) for c in composers]
	for c in tempComps:
		print(c)
		newList = []
		newLabels = []
		rn.load(c)
		if not inversions:
			rn.remove_inversions(separateSixFour=True, filterDiatonic = False)
		rn.get_size(cadenceProg.count('->')  + 1, useReduced=(not inversions))
		myFunc(cadenceProg, printIt=False)
		for i in reversed(range(len(rn.tempList))):
			preChord = rn.tempList[i][0].split(' -> ')[myIndex]
			if preChord in remove:
				del rn.tempList[i]
		for item in rn.tempList:
			preChord = item[0].split(' -> ')[myIndex]
			count = item[1]
			newList.append(count)
			newLabels.append(preChord)
		newList = [100.*x/sum(newList) for x in newList]
		newList = [x for x in newList if x > 1]
		print(sum(newList[:3]))
		plotValues.append(newList)
		labels.append(newLabels[:len(newList)])
		#print newLabels, len(newList)
	#print values
	tickLabels = []
	maxLen = max([len(x) for x in plotValues])
	#print "MAXLEN", maxLen
	tupleComps = tuple(tempComps)
	#print maxLen
	for i in range(len(plotValues)):
		labels[i] = labels[i] + ['-']*(maxLen - len(labels[i]))
	for i in range(len(plotValues)):
		plotValues[i] = plotValues[i] + [0]*(maxLen - len(plotValues[i])) 
	for i in range(len(tempComps)):
		plt.plot(plotValues[i], label=tupleComps[i])
	#print labels, plotValues
	for i in range(maxLen):			# i = chordnumber or second index
		startChord = ''
		for j in range(len(labels)):		# j = composer index
			startChord = startChord + labels[j][i] + '\n'
		tickLabels.append(startChord)
	"""for i in range(len(labels[0])):			# i = chordnumber or second index
		startChord = labels[0][i]
		for j in range(len(labels)):		# j = composer index
			if labels[j][i] != startChord:
				startChord = ''
		tickLabels.append(startChord)"""
	ticks = range(maxLen)
	plt.xticks(ticks, tickLabels)
	myLeg = plt.legend(tupleComps, loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def addbreaks(s, addBirthYears = False, extraLine = False):
	if extraLine:
		mySym = '\n'
	else:
		mySym = ''
	t = []
	for i in range(len(s)):
		if addBirthYears and s[i] in DT.birthyears:
			birthyear = '\n ' + str(DT.birthyears[s[i]])
		else:
			birthyear = ''
		if i % 2 == 1:
			t.append('\n' + mySym + s[i] + birthyear)
		else:
			t.append(s[i] + birthyear)
	return t

def entropy(mode = 'MAJOR', removeInversions = True, filterDiatonic = True, separateSixFour = True):						
	global comps
	modeInd = int(mode[:2].upper() == 'MI')
	plt.xlabel('Composers')
	plt.ylabel('Entropy')
	plt.title('Entropy of zeroth-order chord distributions')
	values = []
	values2 = []
	for c in comps:
		rn.load(c)
		if removeInversions:
			rn.remove_inversions(separateSixFour=separateSixFour, filterDiatonic=filterDiatonic)
		rn.get_size(1, useReduced=removeInversions)
		if filterDiatonic and not removeInversions:
			for mySym in rn.working[modeInd].keys():
				if DT.parse_figure(mySym)[2]:
					del rn.working[modeInd][mySym]
		values.append(DT.dict_entropy(rn.working[modeInd]))
		values2.append(rn.grammar(mode, printOut = False))
	ordering = sorted(range(len(comps)), key = lambda z: values2[z])
	ordering.reverse()
	newvalues = []
	newvalues2 = []
	newcomps = []
	for i in ordering:
		newvalues.append(values[i])
		newvalues2.append(values2[i])
		newcomps.append(comps[i])
	plt.plot(newvalues, label='Entropy')
	ticks = range(len(newcomps))
	plt.xticks(ticks, addbreaks(newcomps))
	myLeg = plt.legend(tuple(['Entropy']), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	print("CORRELATION BETWEEN ENTROPY AND GRAMMATICALITY", pearsonr(values, values2))
	plt.show()

def mutual_information(mode = 'MAJOR', removeInversions = True, filterDiatonic = True, separateSixFour = True):	
	global comps
	modeInd = int(mode[:2].upper() == 'MI')
	plt.xlabel('Composers')
	plt.ylabel('Mutual information')
	plt.title('Mutual information in first-order chord distributions')
	values = []
	values2 = []
	for c in comps:
		rn.load(c)
		if removeInversions:
			rn.remove_inversions(separateSixFour=separateSixFour, filterDiatonic=filterDiatonic)
		rn.get_size(2, useReduced=removeInversions)
		if filterDiatonic and not removeInversions:
			for myPr in rn.working[modeInd].keys():
				for mySym in myPr.split(' -> '):
					if DT.parse_figure(mySym)[2]:
						del rn.working[modeInd][myPr]
						break
		values.append(DT.mutual_information(rn.working[modeInd]))
		values2.append(rn.grammar(mode, printOut = False))
	#ordering = sorted(range(len(comps)), key = lambda z: values2[z])
	#ordering.reverse()
	plt.plot(values, label='Mutual Information')
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	myLeg = plt.legend(tuple(['Mutual Information']), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	print("CORRELATION BETWEEN MUTUAL INFORMATION AND GRAMMATICALITY", pearsonr(values, values2))
	plt.show()

def asymmetry(mode = 'MAJOR'):						# 5-6 vs 6-5 motions
	global comps
	modeInd = int(mode[:2].upper() == 'MI')
	plt.xlabel('Composers')
	plt.ylabel('56 percentage (out of 56 plus 65 motions)')
	plt.title('Strong 56 motions as a percentage of 56/65 motions')
	values = []
	values2 = []
	for c in comps:
		print(c)
		rn.load(c)
		myVal = rn.asymmetry()
		values.append(100*myVal[0]/sum(myVal))
	#labels = ('diatonic chords, no inversions', 'all chords and inversions')
	plt.plot(values)
	#plt.plot(values2, label=labels[1])
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	#myLeg = plt.legend(labels, loc=0)
	#for i in range(len(myLeg.legendHandles)):
	#	myLeg.legendHandles[i].set_linewidth(2.0)
	#print "CORRELATION BETWEEN THE TWO ENTROPIES", pearsonr(values, values2)
	plt.show()

def suspended_fifths(mode = 'MAJOR'):						# 
	global comps
	modeInd = int(mode[:2].upper() == 'MI')
	plt.xlabel('Composers')
	plt.ylabel('percentage of all suspensions')
	plt.title('Suspended fifths (including 6-5 progressions) as a percentage of all suspensions')
	values = []
	for c in comps:
		if c in ['Rock', 'Bill']: continue
		print(c)
		rn.load(c)
		myVal = rn.asymmetry()
		numerator = myVal[0]
		denominator = myVal[0]
		nht.load(c)
		myVal = nht.fifth_suspensions()
		numerator += myVal[0]
		denominator += myVal[1]
		values.append(100*numerator/denominator)
	#labels = ('diatonic chords, no inversions', 'all chords and inversions')
	plt.plot(values)
	#plt.plot(values2, label=labels[1])
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	#myLeg = plt.legend(labels, loc=0)
	#for i in range(len(myLeg.legendHandles)):
	#	myLeg.legendHandles[i].set_linewidth(2.0)
	#print "CORRELATION BETWEEN THE TWO ENTROPIES", pearsonr(values, values2)
	plt.show()

def suspended_fifths(mode = 'MAJOR'):						# 
	global comps
	modeInd = int(mode[:2].upper() == 'MI')
	plt.xlabel('Composers')
	plt.ylabel('percentage of all suspensions')
	plt.title('Suspended fifths (including 6-5 progressions) as a percentage of all suspensions')
	values = []
	for c in comps:
		if c in ['Rock', 'Bill']: continue
		print(c)
		rn.load(c)
		myVal = rn.asymmetry()
		numerator = myVal[0]
		denominator = myVal[0]
		nht.load(c)
		myVal = nht.fifth_suspensions()
		numerator += myVal[0]
		denominator += myVal[1]
		values.append(100*numerator/denominator)
	#labels = ('diatonic chords, no inversions', 'all chords and inversions')
	plt.plot(values)
	#plt.plot(values2, label=labels[1])
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	#myLeg = plt.legend(labels, loc=0)
	#for i in range(len(myLeg.legendHandles)):
	#	myLeg.legendHandles[i].set_linewidth(2.0)
	#print "CORRELATION BETWEEN THE TWO ENTROPIES", pearsonr(values, values2)
	plt.show()


def nhts(theNhts = ['APd']):						# what proportion of the grammaticality is caused by the first order statistics
	global comps
	plt.xlabel('Composers')
	plt.ylabel('Counts')
	plt.title('NHT counts')
	values = []
	totals = []
	tempComps = comps[:]
	for c in comps:
		#try:
		outList = []
		try:
			nht.load(c)
		except:
			tempComps.remove(c)
			continue
		for s in theNhts:
			tempSum = 0
			for t in nht.counts:
				#print c, t, s, re.match(s, t)
				if re.match(s, t):
					tempSum += nht.counts[t]
			outList.append(tempSum)
		totals.append(sum(nht.counts.values()))
		values.append(outList)
		#print outList, totals[-1]
		"""except:
			totals.append(1)
			values.append([0] * len(theNhts))"""
	labels = tuple(theNhts)
	print(values)
	for i, s in enumerate(theNhts):
		outVal = [x[i] for x in values]
		outVal = [100. * outVal[i] / totals[i] for i in range(len(outVal))]
		print(outVal)
		plt.plot(outVal, label=labels[0])
	ticks = range(len(tempComps))
	plt.xticks(ticks, addbreaks(tempComps))
	myLeg = plt.legend(labels, loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()


def chordal_entropy(myChords = ['I -> ', 'V -> '], mode = 'MAJOR', removeInversions = True, filterDiatonic = True, separateSixFour = True, removeSixFour = False):						# what proportion of the grammaticality is caused by the first order statistics
	global comps
	modeInd = int(mode[:2].upper() == 'MI')
	plt.xlabel('Composers')
	plt.ylabel('Entropy')
	plt.title('Entropy following or preceeding particular chords')
	newChords = []
	for progStr in myChords:
		if progStr[-1] == '>': 
			progStr += ' '
		elif progStr[0] == '-':
			progStr = ' ' + progStr
		newChords.append(progStr)
	myChords = newChords
	myValues = {}
	for item in myChords:
		myValues[item] = []
	for c in comps:
		rn.load(c)
		if removeInversions:
			rn.remove_inversions(separateSixFour=separateSixFour, filterDiatonic=filterDiatonic)
		rn.get_size(2, useReduced=removeInversions)
		for item in myChords:
			if item[-2:] == '> ':
				rn.starts(item, printIt = False, pct=False)
			else:
				rn.ends(item, printIt = False, pct=False)
			if removeSixFour:
				tempDict = {}
				for item2 in rn.tempList:
					if item2[0].count('6/4') == 0:
						tempDict[item2[0]] = item2[1]
				myValues[item].append(DT.dict_entropy(tempDict))
			else:
				myValues[item].append(DT.dict_entropy(rn.tempList))
	for values in myValues:
		plt.plot(myValues[values], label=values)
	#plt.plot(values2, label='Second order')
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	myLeg = plt.legend(tuple(myValues.keys()), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def harmonic_cycles(mode = 'MAJOR', index = 0, grammarThresh = 0, **kwargs):						# average length of harmonic cycles
	global comps, composerCounts
	modeStr = int(mode[:2].upper() == 'MI')
	legendLabel = ['Length of Harmonic Cycle', 'Pct. of Cycles Ending with a Dominant', 'Pct. of Cycles ending with a Subdominant'][index]
	plt.xlabel('Composers')
	plt.ylabel(legendLabel)
	plt.title('Harmonic cycles')
	values = []
	actualComps = []
	for c in comps:
		print(c)
		rn.load(c)
		if grammarThresh:
			tempValue = rn.grammar(mode, printOut = False, **kwargs)
			if tempValue < grammarThresh:
				continue
		values.append(rn.harmonic_cycles(mode, printOut = False)[index])
		actualComps.append(c)
	plt.plot(values, label='Length of cycles')
	ticks = range(len(actualComps))
	plt.xticks(ticks, addbreaks(actualComps))
	myLeg = plt.legend(tuple([legendLabel]), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def first_order(mode = 'MAJOR'):						# what proportion of the grammaticality is caused by the first order statistics
	global comps, composerCounts
	modeStr = int(mode[:2].upper() == 'MI')
	plt.xlabel('Composers')
	plt.ylabel('Grammaticality')
	plt.title('Grammaticality - Simulated First-Order Grammaticality')
	values = []
	values2 = []
	for c in comps:
		print(c)
		pcts = firstordertest.grammaticality(c)
		values.append(pcts[1] - pcts[0])
		print(pcts[1] - pcts[0])
		#values2.append(pcts[1])
	plt.plot(values, label='Second order - first order')
	#plt.plot(values2, label='Second order')
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	myLeg = plt.legend(tuple(['Second order - first order']), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def tendency(progs = ["IV -> I"], mode = 'MAJOR', start = True, removeInversions = True, separateSixFour = True):						# how common is the progression over and above the first order probability?
	global comps, composerCounts
	modeStr = int(mode[:2].upper() == 'MI')
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('First order probability - simulated first-order probability')
	if type(progs) is str:
		progs = [progs]
	for progStr in progs:
		theChords = progStr.split(' -> ')
		if start:															# this is when we want to test whether, in A->B, B is unusually likely to follow A
			newProgStr = ' -> '.join(theChords[:-1]) + ' -> '
			myChord = theChords[-1]
		else:
			newProgStr = ' -> ' + ' -> '.join(theChords[1:])				# this is when we want to test whether, in A->B, A is unusually likely to precede B
			myChord = theChords[0]
		values = []
		for c in comps:
			print(c)
			rn.load(c)
			rn.propensity(newProgStr, mode, removeInversions = removeInversions, separateSixFour = separateSixFour)
			if myChord.count(' or ') == 0:
				x = 0
				if myChord in rn.deltas:
					x = rn.deltas[myChord]
			else:
				x = 0
				for s in myChord.split(' or '):
					if s in rn.deltas: x += rn.deltas[s]
			values.append(x)
			print(x)
			#values2.append(pcts[1])
		plt.plot(values, label=progStr)
		#plt.plot(values2, label='Second order')
	ticks = range(len(comps))
	plt.xticks(rotation=90)
	plt.xticks(ticks, comps)
	ax.set_xlim(0, len(comps) -1)
	myLeg = plt.legend(tuple(progs), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def tendency_histo(*args, **kwargs):
	print('tendency_histo has moved to rn.py')
	rn.tendency_histo(*args, **kwargs)


def OUCH_tendency(composer = 'Bach', progStr = 'C -> ', sortByTendency = True, cutDifference = 0):
	global comps, composerCounts, mng
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	composer = get_comp(composer)
	if progStr[-1] == '>': 
		progStr += ' '
	elif progStr[0] == '-':
		progStr = ' ' + progStr
	if progStr[-4:] == ' -> ':
		endVar = True
	elif progStr[:4] == ' -> ':
		endVar = False
	else:
		print("INVALID PROGSTR, SHOULD START OR END WITH ' -> '")
		return
	plt.title('OUCH Tendencies in ' + composer, fontsize = 24)
	plt.ylabel('Percentage', fontsize = 18)
	if endVar:
		plt.xlabel('Positions that follow ' + progStr.rstrip(' ->'), fontsize = 18)
	else:
		plt.xlabel('Positions that precede ' + progStr.lstrip(' ->'), fontsize = 18)
	values = []
	labels = []
	values2 = []
	tempDict = {}
	myLen = len(progStr)
	vl.load(composer)
	vl.OUCHstats()
	if endVar:
		for item in vl.OUCHprogs:
			if item[:myLen] == progStr:
				tempDict[item] = vl.OUCHprogs[item]
	else:
		for item in vl.OUCHprogs:
			if item[-myLen:] == progStr:
				tempDict[item] = vl.OUCHprogs[item]
	totalCounts = sum(tempDict.values())
	for item in tempDict:
		newItem = item.replace(progStr, '')
		values.append(100. * tempDict[item]/totalCounts)
		labels.append(newItem)
		values2.append(vl.OUCHdict[newItem])
	values2 = [100. * x / sum(values2) for x in values2]
	if sortByTendency:
		indexes = range(len(values))
		indexes = sorted(indexes, key = lambda x: values2[x] - values[x])
		newValues = []
		newValues2 = []
		newLabels = []
		for i in indexes:
			if abs(values2[i] - values[i]) < cutDifference:						# for readability skip things whose tendency is essentially 0
				continue
			newValues.append(values[i])
			newValues2.append(values2[i])
			newLabels.append(labels[i])
		values = newValues
		values2 = newValues2
		labels = newLabels
	plt.plot(values, label=progStr)
	plt.plot(values2, label='raw probs.')
	ticks = range(len(labels))
	mySize = 18
	if len(labels) > 25:
		mySize = 14
	plt.xticks(ticks, addbreaks(labels), fontsize = mySize)
	myLeg = plt.legend((progStr, 'raw probs.'), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	ax.set_xlim(0, len(labels) -1)
	plt.show()

	
def count_files():
	totalCount = 0
	for c in DT.comps:
		print(c)
		c = c.upper()
		counter = 0
		if c in DT._PATHS:
			files = os.listdir(DT._PATHS[c][0])
			for myFile in files:
				if myFile[-3:] == 'txt':
					counter += 1
		totalCount += counter
		print(counter, totalCount)

kk_profiles = [[6.35, 2.23, 3.48, 2.33, 4.38, 4.09, 2.52, 5.19, 2.39, 3.66, 2.29, 2.88], [6.33, 2.68, 3.52, 5.38, 2.60, 3.53, 2.54, 4.75, 3.98, 2.69, 3.34, 3.17]] # major, minor

def kk(mode = 'MAJOR', filterDiatonic = True, doubleBass = True):
	modeInd = int(mode[0:2].upper()=='MI')
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('Correlation between pitches and KK-profile')
	values = []
	grammarValues = []
	for c in comps:
		rn.load(c)
		rn.remove_inversions(filterDiatonic = filterDiatonic, separateSixFour = False)
		rn.get_size(1)
		rn.make_pc_dict()
		pcDist = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
		for myChord in rn.working[modeInd]:
			myCount = rn.working[modeInd][myChord]
			for thePC in rn.pcDict[modeInd][myChord]:
				pcDist[thePC] += myCount
			if doubleBass:
				pcDist[rn.pcDict[modeInd][myChord][0]] += myCount
		corr = pearsonr(pcDist, kk_profiles[modeInd])
		print(c, corr)
		values.append(100.0 * corr[0])
		grammarValues.append(rn.grammar(mode, printOut = False))
	plt.plot(values, label='KK correlation')
	plt.plot(grammarValues, label='Grammaticality')
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	myLeg = plt.legend(('KK correlation', 'Grammaticality'), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	print("CORRELATION BETWEEN KK CORRELATION AND GRAMMATICALITY", pearsonr(values, grammarValues))
	plt.show()

"""A very useful routine to plot the location of bass motions by a particular interval -- so you can look at where ascending fourths motions tend to be in the bass,
or descending seconds, or what have you.  The routine also graphs what proportion of these motions are harmonized by root position triads.



"""

def bass_graph(composer = 'Bach', bassInterval = 4, mode = 'MAJOR'):
	composer = get_comp(composer)
	modeInd = int(mode[0:2].upper()=='MI')
	plt.xlabel('Bass progression')
	plt.ylabel('Percentage')
	plt.title('Bass motions in ' + composer + ' by scalar interval ' + str(bassInterval))
	values = []
	values2 = []
	labels = []
	rn.load(composer)
	rn.get_size(2)
	rn.make_bass_dict()
	fifthsList = [(((x - 1) * bassInterval) % 7) + 1 for x in range(7)]  # write out the interval cycle by bassInterval
	for i in range(len(fifthsList)):
		scaleStr = str(fifthsList[i]) + ' -> ' + str(fifthsList[(i + 1) % 7])
		labels.append(scaleStr)
		if scaleStr not in rn.bassDictCounts[modeInd]:
			values.append(0)
			values2.append(0)
			continue
		values.append(rn.bassDictCounts[modeInd][scaleStr])	
		tempCount = 0
		for item in rn.bassDict[modeInd][scaleStr]:
			rootPos = True
			progStr = item.split(' -> ')
			for myChord in progStr:
				parsedFig = DT.parse_figure(myChord)
				if parsedFig[1] and parsedFig[1] != '7':
					rootPos = False
					break
			if rootPos:
				tempCount += rn.bassDict[modeInd][scaleStr][item]
		values2.append(tempCount)
	totalCount = sum(values)
	values = [100.*x/totalCount for x in values]
	values2 = [100.*x/totalCount for x in values2]
	plt.plot(values, label='Bass Motion')
	plt.plot(values2, label='Root Pos. Triads')
	ticks = range(len(fifthsList))
	plt.xticks(ticks, labels)
	myLeg = plt.legend(('Bass Motion', 'Root Pos. Triads'), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()


def graph_rock_roots(composer = 'Rock', mode = 'MAJOR'):
	composer = get_comp(composer)
	modeInd = int(mode[0:2].upper()=='MI')
	plt.xlabel('Root interval (+/- semitones)')
	plt.ylabel('Percentage')
	plt.title('Root motions in Rock')
	rn.load(composer)
	rn.rock_roots()
	c = DT.sort_dict(rn.finalCounts[modeInd])
	labels = [x[0] for x in c]
	values = [x[1] for x in c]
	values = [100.*x/sum(values) for x in values]
	plt.plot(values)
	ticks = range(len(labels))
	plt.xticks(ticks, labels)
	plt.show()
	
def raw(progStr = ['[0, 5, 9] -> [0, 4, 7]'], normStr = []):		#
	global lastValues, comps
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('Raw Sonorities')
	values = []
	tempComps = []
	for i in range(len(progStr)):
		values.append([])
	for c in comps:
		try:
			rs.load(c)
		except:
			print("problem with composer", c)
			continue
		tempComps.append(c)
		if not normStr:
			totals = max(1, sum(rs.counts.values()))
			for i in range(len(progStr)):
				thePct = 100. * rs.counts.get(progStr[i], 0) / totals
				values[i].append(thePct)
		else:
			for i in range(len(progStr)):
				if len(normStr) > i:
					target = normStr[i]
				else:
					target = normStr[-1]
				totals = 0
				found = 0
				for s in rs.counts:
					if re.match(target, s):
						totals += rs.counts[s]
						#print 'totals', s, rs.counts[s]
					if re.match(progStr[i], s):
						#print 'found', s, rs.counts[s]
						found += rs.counts[s]
				totals = max(totals, 1)
				thePct = 100. * found / totals
				values[i].append(thePct)
	lastValues = values[-1][:]
	comps = tempComps
	for i, s in enumerate(progStr):
		plt.plot(values[i], label=progStr[i])
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	myLeg = plt.legend(tuple(progStr), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def graph_by_chord(composers = ['Bach', 'Mozart'], chords = ['V7', 'V6/5', 'V4/3', 'V2'], mode = 'MAJOR'):		#
	modeInd = int(mode[0:2].upper()=='MI')
	theSize = chords[0].count('->') + 1
	plt.xlabel('Chords')
	plt.ylabel('Percentage')
	plt.title('Chord popularity')
	for composer in composers:
		theList = []
		rn.load(composer)
		rn.get_size(theSize)
		theTotals = max(1, sum(rn.working[modeInd].values()))
		for x in chords:
			if x in rn.working[modeInd]:
				theList.append(100.*rn.working[modeInd][x]/theTotals)
			else:
				theList.append(0)
		plt.plot(theList,label=composer)
	ticks = range(len(chords))
	plt.xticks(ticks, chords)
	myLeg = plt.legend(tuple(composers), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def reversals(voice = -1, thresh = 4, direction = 0, outThresh = 100000):						# what proportion of the grammaticality is caused by the first order statistics
	global comps
	tempComps = copy.deepcopy(comps)
	if 'Josquin' in tempComps:
		tempComps.remove('Josquin')
	tempComps += ["JosquinCOMPLETE", "Essen", "AirdsAirs", "RyansMammoth", "Oneills1850"]
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('Mean reversal percentage')
	values = []
	finalComps = []
	for c in tempComps:
		try:
			rev.load(c)
			values.append(rev.summarize(voice, thresh, direction, outThresh))
			finalComps.append(c)
		except:
			pass
	plt.plot(values)
	ticks = range(len(finalComps))
	plt.xticks(ticks, addbreaks(finalComps))
	plt.show()

def major_and_minor(mode = 'MAJOR', sortByFunction = True):						
	global comps
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('Percentage of triads that are major')
	values = []
	for c in comps:
		rn.load(c)
		values.append(rn.major_and_minor(mode))
	plt.plot(values)
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	plt.show()

def root_function(chordList = ['IV'], mode = 'MAJOR'):						
	global comps
	plt.xlabel('Composers')
	plt.ylabel('Percentage')
	plt.title('Percentage of triads that are major')
	values = []
	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)
	for i in range(len(chordList)):
		values.append([])
	for c in comps:
		rn.load(c)
		for i, p in enumerate(chordList):
			values[i].append(rn.root_function(p, mode = mode)[0])
	for i, v in enumerate(values):
		plt.plot(v, label = chordList[i])
	ticks = range(len(comps))
	plt.xticks(ticks, addbreaks(comps))
	ax.set_ylim(-1, 1)
	myLeg = plt.legend(tuple(chordList), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()


def spacing(composers = ['Bach', 'Goudimel','Monteverdi', 'Palestrina', 'Josquin'], endString = '1 1', startString = '3 R', targetLength = 4):
	global comps, composerCounts
	plt.title('Space from Bass to Close-Position Upper Voices', fontsize = 24)
	plt.ylabel('Percentage', fontsize = 18)
	tempComps = [get_comp(c) for c in composers]
	plt.xlabel('Spacing Distance (in triad steps)', fontsize = 18)
	composerCounts = {}
	values = []
	inList = []
	labels = []
	masterLabels = []
	for bigCounter, c in enumerate(tempComps):
		print(c)
		db.load(c)
		s = db.spacingDict
		newList = [0] * 8
		for item in s:
			if len(item) <= (len(endString) + len(startString)): continue
			if len(item.split()) != (targetLength + 1): continue
			if item[:len(startString)] == startString and item[-(len(endString)):] == endString:
				i = int(item[len(startString):-len(endString)].replace(' ', ''))
				newList[i] = s[item]
		if sum(newList) > 0:
			newList = [100.*x/sum(newList) for x in newList]
		plt.plot(newList, label=c)
	ticks = range(6)
	plt.xticks(ticks, fontsize = 18)
	myLeg = plt.legend(tuple(tempComps), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()
