from __future__ import print_function
import os
import copy
import time
import pickle
import matplotlib.pyplot as plt
from music21 import *
import DT

_THISFILE = 'MODULATIONS'	# used to be 'modulations'; stored with secondaryString

counts = {}
locations = {}
symbolDict = [{}, {}]
#inputFile = open (DT._DATAPATH + 'modulations.pkl', 'rb')
#symbolDict = pickle.load(inputFile)
#inputFile.close()
output = stream.Score()
lastChorale = 0
chorale = stream.Score()
analyzed_file = stream.Score()

"""
Catalogue time spent in another key, used for human-analyzed files.

	
TO DO:
	1. RECORD A KEY THAT ASSOCIATES COUNTER WITH FILENAMES; STORE THESE FILENAMES IN THE PICKLED FILE
	1. Gather all lengths from 1 to 5 at once
	2. Make it possible to gather all the files in a directory 
	
	
"""

def gather_stats(source='BACH', n = 1, m = 372, secondaryDominants = True):
	global counts, locations, currentSize, analyzed_file, locNames
	global quarterChords, eighthEighthChords, globalSource, keyList, tempKeys, changesDict, globalSecondaries
	quarterChords = 0
	recordResults = False
	eighthEighthChords = 0
	globalSecondaries = secondaryDominants
	keyList = {}
	counts = {}
	locations = {}
	locNames = {}
	changesDict = {}
	tonicPctList = []
	startT = time.time()
	fileStr = ''
	secondaryString = ['NOSECONDARIES', ''][int(globalSecondaries)]
	globalSource = source.upper()
	if globalSource in DT._PATHS:
		filePath = DT._PATHS[globalSource][0]
		fileStr = globalSource
		files = os.listdir(filePath)
		recordResults = True
		counter = 1
		for myFile in files:
			if myFile[-3:] == 'txt':
				locNames[counter] = myFile[:-4]
				print('Analyzing', myFile, end = '')
				analyzed_file = converter.parse(filePath + myFile, format='romantext')
				dochorale(counter)
				tonicPct = 100.*max(tempKeys.values())/sum(tempKeys.values())
				tonicPctList.append(tonicPct)
				print(", tonic percentage", DT.float_string(tonicPct, 0))
				keyList[myFile[:-4]] = [tonicPct, copy.deepcopy(tempKeys)]
				counter += 1
	print("Average tonic pct", DT.float_string(1.*sum(tonicPctList)/len(tonicPctList), 0))
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and', int((minutes - int(minutes)) * 60), 'seconds')
	if recordResults:
		outputFile = open (DT._DATAPATH + _THISFILE + secondaryString + fileStr + '.pkl', 'wb')
		pickle.dump(keyList, outputFile)
		pickle.dump(changesDict, outputFile)
		outputFile.close()
	return

def mod7(i):
	if i > 0: i = i - 1
	if i < 0: i = i + 1
	return (i % 7)

def fix_key(k):
	if k.mode == 'minor':
		return k.tonic.name.lower()
	else:
		return k.tonic.name.upper()
		
def dochorale(n):															# record the keys of each chord
	global counts, locations, output, pTreb, pBass, chorale, analyzed_file, anal, changesDict, globalSecondaries
	global tempKeys
	global startOffset, endOffset, ourKey, lastMeasure
	anal = analyzed_file.flat.getElementsByClass(['RomanNumeral'])
	#if globalSource[:4] == 'ROCK': fix_temperley()
	tempKeys = {}
	lastFigure = ''
	lastKey = ''
	lastRawKey = False
	for myChord in anal:
		if globalSecondaries and myChord.secondaryRomanNumeralKey:
			myFigure = myChord.primaryFigure
			rawKey = myChord.secondaryRomanNumeralKey
		else:
			myFigure = myChord.figure
			rawKey = myChord.key
		myKey = fix_key(rawKey)
		if (myFigure == lastFigure) and (myKey == lastKey): 
			continue
		if rawKey != lastRawKey and lastRawKey:
			i = str(steps_to_diatonic(diatonic_to_steps(interval.notesToGeneric(lastRawKey.tonic, rawKey.tonic).value) % 7))
			modes = (lastRawKey.mode + ' ' + rawKey.mode).replace('major', 'M').replace('minor', 'm')
			modulationString = i + ' ' + modes
			changesDict[modulationString] = changesDict.setdefault(modulationString, 0) + 1
			#print lastKey, myKey, modulationString
		tempKeys[myKey] = tempKeys.setdefault(myKey, 0) + 1
		lastRawKey = rawKey
		lastKey = myKey
		lastFigure = myFigure
		"""FIX SECONDARY DOMINANTS HERE"""
	return

def diatonic_to_steps(i):
	if i > 0:
		return i - 1
	if i < 0:
		return i + 1

def steps_to_diatonic(i):
	if i >= 0:
		return i+1
	if i < 0:
		return i-1

def load(source = 'Mozart', secondaryDominants = True):
	global counts, locations, globalSource, locNames, currentSize, working, keyList, changesDict, globalSecondaries
	globalSecondaries = secondaryDominants
	secondaryString = ['NOSECONDARIES', ''][int(globalSecondaries)]
	try:
		working = [{}, {}]
		changesDict = {}
		currentSize = 5
		globalSource = source.upper()
		with open(DT._DATAPATH + _THISFILE + secondaryString + globalSource + '.pkl', 'rb') as storedFile:
			keyList = pickle.load(storedFile)
			changesDict = pickle.load(storedFile)
		return True
	except:
		return False

def remove_modes(pct = True):
	global working
	modulationNames = {'4': 'A4', '5': 'D4', '3':'A3', '6':'D3', '2':'A2', '7':'D2', '1': 'P'}
	working = {}
	for s in changesDict:
		i = modulationNames[s.split()[0]]
		working[i] = working.setdefault(i, 0) + changesDict[s]
	DT.print_dict(working, pct = pct)

def transpose_letter(letterName, transp):
	if letterName == '?': return False
	return pitch.Pitch(letterName).transpose(transp).name

def normalize(printOut = True):
	global keyList, fixedList
	referencePitch = pitch.Pitch('C')
	fixedList = {}
	for item in keyList:
		normalizedKeys = {}
		myKeys = keyList[item][1]
		workingTonic = pitch.Pitch(max(myKeys, key=myKeys.get))
		myInterval = '-'+interval.Interval(noteStart=referencePitch, noteEnd=workingTonic).name
		#print "BIGGEST KEY, INTERVAL TO TRANSPOSE", workingTonic.name, myInterval
		for theKey in myKeys:
			newName = transpose_letter(theKey, myInterval)
			if theKey[0].islower(): newName = newName.lower()
			#print "old, interval, new", theKey, myInterval, newName
			normalizedKeys[newName] = myKeys[theKey]
		outList = [keyList[item][0], copy.deepcopy(normalizedKeys)]
		fixedList[item] = outList
	if not printOut: return
	for item in fixedList:
		print(item, fixedList[item][0])
		DT.print_dict(fixedList[item][1])
	return

def tonic_pct():
	global keyList
	tonicPctList = [x[0] for x in keyList.values()]
	#print tonicPctList
	return 1.*sum(tonicPctList)/len(tonicPctList)

def average():
	global keyList, fixedList, averages
	normalize(printOut=False)
	averages = [{}, {}]
	counts = [0, 0]
	for item in fixedList:
		myDict = fixedList[item][1]
		workingTonic = max(myDict, key=myDict.get)
		totalKeys = sum(myDict.values())
		modeInd = int(workingTonic=='c')
		counts[modeInd] += 1
		for eachKey in myDict:
			if eachKey in averages[modeInd]:
				averages[modeInd][eachKey].append(100.*myDict[eachKey]/totalKeys)
			else:
				averages[modeInd][eachKey] = [100.*myDict[eachKey]/totalKeys]
	for modeInd in [0, 1]:
		print(['MAJOR', 'MINOR'][modeInd])
		for eachKey in averages[modeInd]:
			averages[modeInd][eachKey] = sum(averages[modeInd][eachKey])/counts[modeInd]
		DT.print_dict(averages[modeInd])
		
	
