from __future__ import print_function
import os
import copy
import time
import pickle
from collections import OrderedDict
import subprocess

from music21 import *

import DT																				

_THISFILE = 'NHT'

counts = {}
locations = {}
progDict = []
suspensionDoublings = {}
locNames = {}
chorale = stream.Score()
output = stream.Score()
pTreb = stream.Part()
pBass = stream.Part()
analyzed_file = stream.Score()
lastChorale = 1
checkUntil = 372

suspensionCounts = [0, 0, 0]

"""

A program to classify, count, and remove nonharmonic tones.

	format:

		nht.rewrite("op127m1", "Beethoven")

			will look in the Beethoven directory in _MAINPATH/Music for (a) a romantext file op127m1.txt and (b) a score op127m1.xml (or krn, mxl, etc) and
			combine the two and show them.  Setup _MAINPATH in DT.py.

		This works pretty well 90% of the time or so, but there are lots of advanced nonharmonic tones I can't catch.
		No making fun of my code, I was learning python when I wrote this!

TO DO:

	Strip out voice 2 as part of chord processing
	Filter out grace notes

	CONSOLIDATE REPETITIONS THAT OCCUR WITHIN A SINGLE CHORD
	remove bass-note sevenths that are not part of the analysis
	think more about the two-sixteenths anticipation pattern.  Should it be removed when the N is harmonic?  Can I decide when it should and shouldn't happen?
		YES, FIX THIS -- at least in the limited case where a seventh resolves down
		#231 measure 5 is a great test case for this
	perhaps title the output file
	remove extraneous time signatures put into place by make_beams
	look for quarter-note nonharmonic tones [tricky because some get broken -- include tied eighth notes both of which are suspensions]
	compare ascending and descending PTs and NTs and ANTs
	check for pairs of passing tones.  do 3 ever appear in a row?
	could write a routine that checked that every chord was what it was supposed to be
	Someday: look at every example of the eighth-two-sixteenths-pattern
	MAKE SCORES WITH ALL NHTONES LABELED
	
	currently, we remove both notes of the DA figure in the case where the previous note is a seventh; in chorale 132, m27, this creates parallel octaves
	we could correct for this

"""

def gather_stats(source = 'rockcomposite', rewrite = False, saveDir = ''):
	global counts, locations, progDict, output, pTreb, pBass, lastChorale, checkUntil, chorale, globalSource, suspensionDoublings, analyzed_file, globalSource,suspensionCounts, locNames, suspensionCounts, suspensionLocations
	source = source.upper()
	globalSource = source
	if source not in DT._PATHS:
		print("SOURCE NOT FOUND!")
		return
	analysisPath = DT._PATHS[source][0]
	scorePath = DT._PATHS[source][1]
	files = sorted(os.listdir(analysisPath))
	recordResults = False
	startT = time.time()
	globalSource = source
	output = stream.Score()
	pTreb = stream.Part()
	pBass = stream.Part()
	counts = {}
	suspensionDoublings = {}
	suspensionLocations = {}
	locations = {}
	locNames = {}
	suspensionCounts = [0, 0, 0]
	counter = 1
	for myFile in files:
		if myFile[-3:] == 'txt':
			print('File', myFile)
			analyzed_file = converter.parse(analysisPath + myFile, format='romantext')
			tempName = scorePath + myFile.replace('.txt', '.xml')
			chorale = ''
			try:
				chorale = converter.parse(tempName)
			except:
				tempName = scorePath + myFile.replace('.txt', '.krn')
				chorale = converter.parse(tempName)
			if not chorale:
				print("ERROR: ", tempName)
				continue
			for myPart in chorale.parts:								# needed for some figured basses
				for myStream in myPart.recurse(streamsOnly=True):
					myStream.removeByClass('ChordSymbol')
					#myStream.removeByClass('TextExpression')
			dochorale(counter)
			locNames[counter] = myFile[:-4]
			counter += 1
			if rewrite:
				remove_nonharmonic_tones()
				add_roman_numerals(myFile[:-4], twoLevel = True)
				if saveDir: chorale.write('musicxml', scorePath + saveDir + myFile.replace('.txt', '.xml'))
	outputFile = open(DT._DATAPATH + _THISFILE + source +'.pkl', 'wb')
	print(suspensionCounts)
	pickle.dump(counts, outputFile)
	pickle.dump(locations,outputFile)
	pickle.dump(suspensionDoublings, outputFile)
	pickle.dump(locNames, outputFile)
	pickle.dump(suspensionCounts, outputFile)
	pickle.dump(suspensionLocations, outputFile)
	outputFile.close()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	return

def list_them(nhtype = ['APd']):
	global counts, locations, suspensionDoublings, locNames
	for myType in nhtype:
		if myType not in counts:
			print(myType, ' not found')
			continue
		print(myType, counts[myType])
		if myType not in locations:
			print(myType, ' ERROR, not in locations!')
			continue
		for pair in locations[myType]:
			print('     ', locNames[pair[0]], 'measure', pair[1])

def load(source = 'PALESTRINA'):
	global counts, locations, suspensionDoublings, locNames, globalSource, suspensionCounts, suspensionLocations
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + source + '.pkl', 'rb') as storedFile:
			counts = pickle.load(storedFile)
			locations = pickle.load(storedFile)
			suspensionDoublings = pickle.load(storedFile)
			locNames = pickle.load(storedFile)
			suspensionCounts = pickle.load(storedFile)
			suspensionLocations = pickle.load(storedFile)
		return True
	except:
		return False
	
def rewrite(i, source = 'rockcomposite', remove = False, saveDir = ''):
	global chorale, analyzed_file
	source = source.upper()
	if source not in DT._PATHS: 
		print("NO SOURCE!")
		return
	analysisPath = DT._PATHS[source][0]
	scorePath = DT._PATHS[source][1]
	globalSource = source
	fileStr = source
	files = os.listdir(analysisPath)
	recordResults = True
	counter = 1
	go = False
	for myFile in files:
		if myFile[-3:] == 'txt':
			if type(i) == str:
				if i == myFile[:-4]:
					go = True
					break
			elif type(i) == int:
				if counter == i:
					go = True
					break
			counter += 1
	if go:
		subprocess.call(('open', analysisPath + myFile))
		analyzed_file = converter.parse(analysisPath + myFile, format='romantext')
		try:
			chorale = converter.parse(scorePath + myFile.replace('.txt', '.xml'))
		except:
			chorale = converter.parse(scorePath + myFile.replace('.txt', '.mxl'))
		for myPart in chorale.parts:								# needed for some figured basses
			for myStream in myPart.recurse(streamsOnly=True):
				myStream.removeByClass('ChordSymbol')
				#myStream.removeByClass('TextExpression')
		print('File', i)
		result = dochorale(i)
		if remove:
			remove_nonharmonic_tones()
		add_roman_numerals(i, twoLevel = True)
		chorale.show()
		#annotate()				# add this later
		if saveDir:
			chorale.write('musicxml', scorePath + saveDir + myFile.replace('.txt', '.xml'))
	else:
		print('FILE NOT FOUND!')
	return

def resume():
	global lastChorale, checkUntil
	gather_stats(lastChorale + 1, checkUntil, reset = False)
	
def show_chorale(i, source = 'PALESTRINA', remove = False):
	global chorale, analyzed_file
	source = source.upper()
	if source not in DT._PATHS: 
		print("NO SOURCE!")
		return
	analysisPath = DT._PATHS[source][0]
	scorePath = DT._PATHS[source][1]
	globalSource = source
	fileStr = source
	files = os.listdir(analysisPath)
	recordResults = True
	counter = 1
	go = False
	for myFile in files:
		if myFile[-3:] == 'txt':
			if type(i) == str:
				if i == myFile[:-4]:
					go = True
					break
			elif type(i) == int:
				if counter == i:
					go = True
					break
			counter += 1
	if go:
		analyzed_file = converter.parse(analysisPath + myFile, format='romantext')
		chorale = converter.parse(scorePath + myFile.replace('.txt', '.xml'))
		#DT.anal(analysisPath + myFile)
		print('File', i)
		result = dochorale(i)
		if not result:
			if remove: 
				remove_nonharmonic_tones()
				add_roman_numerals(i, twoLevel = True)
			else:
				add_roman_numerals(i)
			chorale.show()
	else:
		print('FILE NOT FOUND!')
	return

def show(nhType, RNs = True, rangeList = []):
	global output, locNames, globalSource, suspensionLocations
	output = stream.Score()
	tempOutput = stream.Score()
	lastChoraleNo = False
	for i in range(0, 4):
		tempOutput.append(stream.Part())
	try:
		locList = locations[nhType]							# check for key error
		if rangeList:
			locList = locList[rangeList[0]: rangeList[1]]
	except KeyError:
		if nhType in suspensionLocations:
			locList = suspensionLocations[nhType]
		else:
			print('Not found')
			return
	for instance in locList:
		measureList = instance[1:]
		if len(measureList) == 1:
			measureList = [measureList[0], measureList[0] + 1]
		else:
			measureList[-1] = measureList[-1] + 1
		print('getting', locNames[instance[0]], measureList, DT._PATHS[globalSource][0]) # + choraleNo + '.xml'
		header = str(locNames[instance[0]])+'m'+str(measureList[0])
		if locNames[instance[0]] != lastChoraleNo:
			choraleNo = locNames[instance[0]]
			lastChoraleNo = choraleNo
			try:
				chorale = converter.parse(DT._PATHS[globalSource][1] + choraleNo + '.xml')
			except:
				chorale = converter.parse(DT._PATHS[globalSource][1] + choraleNo + '.krn')
		if RNs: 
			analyzed_file = converter.parse(DT._PATHS[globalSource][0] + choraleNo + '.txt', format='romantext')
			DT.add_roman(chorale, analyzed_file, measureList[0], measureList[1])
		tempChorale = copy.deepcopy(DT.get_measures(chorale, measureList[0], measureList[1]))
		DT.insert_text(tempChorale.parts[0][0], header, 55, 0)
		output = DT.append(tempChorale, output)
	output = DT.fix_roman_placement(output)
	for p in output.parts:
		for item in p.flat:
			if 'Note' in item.classes: item.lyrics = []
	output.show()
	return output

def preceded_by_rest(myNote):
	global literalScore
	#print myNote.offset
	#prevItem = literalScore.getElementBeforeOffset(myNote.offset)
	try:
		prevItem = literalScore.getElementBeforeOffset(myNote.offset)
	except:
		return True
	if not prevItem: return False
	if 'Rest' in prevItem.classes: return True
	return False

def note_extends_to_RN_boundary(myNote):
	global analyzed_file
	myRN = roman_numerals.getElementAtOrBefore(myNote.offset)
	if not myRN: return False
	if myNote.offset + myNote.duration.quarterLength == myRN.offset + myRN.duration.quarterLength: return True
	return False


def dochorale(n, xml = False):
	global counts, locations, chorale, output, timeSigs, progDict, output, pTreb, pBass, analyzed_file, roman_numerals, suspensionOffsets, globalSource, myNote, mNumber, literalScore
	global outputMessages
	outputMessages = {}
	#chorale.show()
	#chorale.show('text')
	#chorale.parts[0].measure(105).show('text')
	suspensionOffsets = []
	timeSigs = analyzed_file.flat.getTimeSignatures()
	roman_numerals = analyzed_file.flat.getElementsByClass(['RomanNumeral', 'Rest']).stream()
	RNOffsets = []
	stopSearching = False
	for i in range(1, len(roman_numerals)):	
		if ('Rest' in roman_numerals[i].classes) or not (roman_numerals[i].figure == 'V7' and roman_numerals[i -1].figure == 'V' and 		
		roman_numerals[i].key.tonic == roman_numerals[i-1].key.tonic and roman_numerals[i].key.mode == roman_numerals[i-1].key.mode):
			RNOffsets.append(roman_numerals[i].offset)
	#for i in RNOffsets:
	#	print i
	for voice in range (0, len(chorale.parts)):
		firstMeasure = chorale.parts[voice].getElementsByOffset(0, 0, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure'])[0]
		firstMeasure.removeByClass('MetronomeMark')
		chorale.parts[voice].sliceAtOffsets(RNOffsets, inPlace = True)
		score = chorale.parts[voice].flat.getElementsByClass('Note').stream()
		literalScore = chorale.parts[voice].flat.getElementsByClass(['Note', 'Rest'])
		for noteNum, myNote in enumerate(score):
			myNote.lyrics = []
			if 'Chord' in myNote.classes:
				score[noteNum] = max(myNote)
		#print(len(score))
		voiceName = str(voice)
		
		"""NEED THE FOLLOWING FOR NH TONES THAT ARE REPEATED!"""
		
		for RN in roman_numerals:
			start = RN.offset
			end = RN.offset + RN.duration.quarterLength
			testNotes = score.getElementsByOffset(start, end, includeEndBoundary=False, mustBeginInSpan=True).stream()
			if not testNotes: continue
			for j in reversed(range(1, len(testNotes))):
				if testNotes[j].pitch.midi == testNotes[j-1].pitch.midi: score.remove(testNotes[j])
				
			"""midiNotes = [x.midi for x in testNotes]
			if len(set(midiNotes)) == 1 and len(midiNotes) != 1:
				#testNotes[0].duration.quarterLength = end - testNotes[0].offset
				for j in range(1, len(testNotes)):
					score.remove(testNotes[j])"""
		
		#print(len(score))
		for i in range(len(score)):
			myNote = score[i]
			#had a try: loop here
			RN = roman_numerals.getElementsByOffset(myNote.offset + .001, myNote.offset + .002, includeEndBoundary = True, mustBeginInSpan = False)
			if not RN:
				continue
			RN = RN[0]
			if 'Rest' in RN.classes: 
				myNote.lyric = ' '
				continue
			RNbeat = RN.beat - 1
			if RN.key.mode == 'minor':
				keyStr = str(RN.key.tonic.name).lower() + ': '
			else:
				keyStr = str(RN.key.tonic.name) + ': '
			chordLabel = RN.figure
			RNPitchClasses = DT.fix_roman(RN)														# always use RNPitchClasses, not RN.pitchClasses
			if RN.seventh:
				seventh = DT.seventh
			else:
				seventh = -9999
			if myNote.pitch.pitchClass in RNPitchClasses or (chordLabel == '?:'):													
				myNote.lyric = ' '				
				#print(i, chordLabel, RNPitchClasses, myNote, myNote.offset)					
			
			""" the following seems to impose separate treatment of sevenths in the bass; appropriate for chorales maybe?
				causing problems in Beethoven
			"""
			
			"""if voice == (len(chorale.parts) - 1) and RN.seventh and globalSource != 'PALESTRINA':
				if RN.inversion() == 0 and myNote.pitch.pitchClass == seventh: 
					myNote.lyrics = []															# bass sevenths in V7 and
				if RN.inversion() == 3 and myNote.pitch.pitchClass == RN.getChordStep(1).pitchClass: 
					myNote.lyrics = [] 	"""														# bass roots in V2 (see 91, 111)
			"""except:
				print('oops1')
				myNote.lyric = ' '
				#print myNote.offset"""
		
		for i in range(1, len(score) - 1):																		# 162 and 164 have good examples
			notesLeft = len(score) - 1 - i 
			myNote = score[i]
			myMeasure = analyzed_file[1].getElementsByOffset(myNote.offset + .01, myNote.offset + .02, includeEndBoundary = True, mustBeginInSpan = False)
			if not myMeasure: 
				#print('oops')
				continue
			myMeasure = myMeasure[0]
			mNumber = myMeasure.number
			locList = [n, mNumber]
			#print(locList)
			myBeat = myNote.offset - myMeasure.offset
			myBeatStrength = get_beat_strength(myNote.offset, voice)
			if myNote.lyric != ' ':
				prevNote = score[i-1]
				prevBeatStrength = get_beat_strength(prevNote.offset, voice)
				nextNote = score[i+1]
				nextBeatStrength = get_beat_strength(nextNote.offset, voice)
				twoOnBeatStrength = threeOnBeatStrength = twoOnInt = threeOnInt = 0
				if notesLeft > 1:
					twoOn = score[i+2]
					twoOnBeatStrength = get_beat_strength(twoOn.offset, voice)
					twoOnInt = interval.notesToGeneric(myNote, twoOn).value
				if notesLeft > 2:
					threeOn = score[i+3]
					threeOnBeatStrength = get_beat_strength(threeOn.offset, voice)
					threeOnInt = interval.notesToGeneric(myNote, threeOn).value
				firstInt = interval.notesToGeneric(prevNote, myNote).value
				secondInt = interval.notesToGeneric(myNote, nextNote).value
				#print voice, myMeasure, myBeat
				pRest = preceded_by_rest(myNote)
#			NOTE: ORDER MATTERS IN THE FOLLOWING!  OTHERWISE SOME NHTs GET PARSED INCORRECTLY
				if not myNote.lyric: 
					myNote.lyric = '?'   																				# capture the weirdos
				if myNote.lyric != '?':																					# this should all be embedded
					pass																								# in the previous if consequence
				elif (notesLeft > 2 and firstInt == 1 and threeOnInt == -2 and secondInt == -2 and twoOnInt == -3 and nextNote.duration.quarterLength == .25 and twoOn.duration.quarterLength == .25 and threeOnBeatStrength and threeOnBeatStrength <= myBeatStrength and prevNote.lyric == ' ' and threeOn.lyric == ' '):
					myNote.lyric = 'DS2'
					nextNote.lyric = 'A'
					if myNote.pitch.pitchClass == seventh: twoOn.lyric = 'Nd'  # THIS MIGHT BE CONFUSED!!!!  OR MAYBE NOT?
					get_notes(myNote.offset, locList)												# to record suspension doublings
					# are much more delicate -- 231 m 5 is a test case
				elif (notesLeft > 1 and twoOnInt == 1 and abs(secondInt) == 2 and twoOn.lyric == ' ' and myNote.duration.quarterLength == .25 and
				nextNote.duration.quarterLength == .25):			# originally had and nextNote.lyric != ' ' but that seems wrong
					myNote.lyric = 'DA'								# decorated anticipation
					if prevNote.pitch.pitchClass == seventh: 
						if secondInt == -2: 
							nextNote.lyric = 'Nd'			# more cases where this is needed?  look systematically!!!!
						else:
							nextNote.lyric = 'Na'
				elif (firstInt == 2 and secondInt == 2) or (firstInt == -2 and secondInt == -2): 
					if nextBeatStrength >= myBeatStrength:
						if firstInt == 2: 
							myNote.lyric = 'Pa' 
						else:
							myNote.lyric = 'Pd'
					else:
						if firstInt == 2:
							myNote.lyric = 'APa'
						else:
							myNote.lyric = 'APd'
				elif not pRest and ((firstInt == 2 and secondInt == -2) or (firstInt == -2 and secondInt == 2)): 
					if nextBeatStrength >= myBeatStrength: 
						if firstInt == 2:
							tempSuffix = 'a'
						else:
							tempSuffix = 'd'
						if prevNote.duration.quarterLength == nextNote.duration.quarterLength and myNote.duration.quarterLength == 2 * prevNote.duration.quarterLength:
							myNote.lyric = 'EQEn' + tempSuffix
						else:
							myNote.lyric = 'N' + tempSuffix
					else:
						if firstInt == 2:
							myNote.lyric = 'ANa' 
						else:
							myNote.lyric = 'ANd'
					if not (prevNote.lyric == ' ' and nextNote.lyric == ' '):
						myNote.lyric = 'Nnh'
				elif (firstInt == 1 and secondInt == -2 and nextBeatStrength <= myBeatStrength and (prevNote.lyric == ' ' or prevNote.lyric == 'S') and nextNote.lyric == ' '): 
					myNote.lyric = 'S'																									# standard suspension
					get_notes(myNote.offset, locList)
				elif (notesLeft > 1 and firstInt == 1 and secondInt == 1 and twoOnInt == -2 and twoOnBeatStrength <= myBeatStrength and (prevNote.lyric == ' ' or prevNote.lyric == 'S') and twoOn.lyric == ' '): 
					myNote.lyric = 'S'																									# suspension w/chordchange
					get_notes(myNote.offset, locList)
					if not nextNote.lyric == ' ': nextNote.lyric = 'S'
				elif (notesLeft > 1 and firstInt == 1 and twoOnInt == -2 and twoOnBeatStrength <= myBeatStrength and (secondInt == -3 or secondInt == -5 or secondInt == 2) and prevNote.lyric == ' ' and twoOn.lyric == ' '): 
					myNote.lyric = 'DS'																									# decorated suspension
					get_notes(myNote.offset, locList)
					if secondInt == 2: nextNote.lyric = 'E'
				elif ((abs(firstInt) == 2 or pRest) and secondInt == 1 and nextBeatStrength >= myBeatStrength and nextNote.lyric == ' '): 
					myNote.lyric = 'A'																									# standard anticipation
				elif (abs(firstInt) > 2 and secondInt == 1 and nextBeatStrength >= myBeatStrength and nextNote.lyric == ' '): 
					myNote.lyric = 'LA'																								# leapt-to anticipation
				elif (firstInt == -2 and secondInt == -3 and twoOnInt == -2 and nextNote.lyric == ' '): 
					myNote.lyric = 'Camb'																							# cambiata?
					#print_everything('Cambiata: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (abs(firstInt) == 2 and abs(secondInt) > 2 and nextBeatStrength >= myBeatStrength and prevNote.lyric == ' ' and not pRest): 
					myNote.lyric = 'E'																									# echappee
				elif ((pRest or abs(firstInt) > 2) and abs(secondInt) == 2 and nextBeatStrength >= myBeatStrength and nextNote.lyric == ' '): 
					myNote.lyric = 'I'																									# incomplete neighbor
				elif ((pRest or abs(firstInt) > 2) and abs(secondInt) == 2 and nextBeatStrength < myBeatStrength and nextNote.lyric == ' '): 
					myNote.lyric = 'APP'																									# incomplete neighbor
				elif (firstInt == 1 and (nextNote.pitch.midi - myNote.pitch.midi == 1) and nextBeatStrength <= myBeatStrength and prevNote.lyric == ' ' and nextNote.lyric == ' ' and not pRest): 
					myNote.lyric = 'R'																									# retardation
					print_everything ('RETARDATION: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (nextNote.pitch.midi - myNote.pitch.midi == 1 and myNote.pitch.midi - prevNote.pitch.midi == 1):											# chromatic passing tone
					myNote.lyric = 'CPT'					#	"""put in stuff requiring that the EQE nhtones are connected to nonharmonic tones???"""
				elif (notesLeft > 1 and (firstInt == 2 and secondInt == 1 and twoOnInt == -2) and prevNote.duration.quarterLength == myNote.duration.quarterLength == nextNote.duration.quarterLength == twoOn.duration.quarterLength): 				# EQE neighbor should I add rhythm?
					myNote.lyric = 'EQEn'
					nextNote.lyric = 'EQEn'
					print_everything ('EQE neighbor: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (notesLeft > 1 and (firstInt == 2 and secondInt == 1 and twoOnInt == -2)): 				# needed for Monteverdi
					myNote.lyric = 'Weqen'
					nextNote.lyric = 'Weqen'
					print_everything ('WEIRD EQE neighbor: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (notesLeft > 1 and (firstInt == -2 and secondInt == 1 and twoOnInt == -2) and prevNote.duration.quarterLength == myNote.duration.quarterLength == nextNote.duration.quarterLength == twoOn.duration.quarterLength): 				# add rhythm?
					myNote.lyric = 'EQEp'
					nextNote.lyric = 'EQEp'
					print_everything ('EQE passing: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (notesLeft > 1 and (firstInt == -2 and secondInt == 1 and twoOnInt == -2)): 				# for monteverdi again
					myNote.lyric = 'Weqep'
					nextNote.lyric = 'Weqep'
					print_everything ('WEIRD EQE passing: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (notesLeft > 1 and (firstInt == 2 and secondInt == -3 and twoOnInt == -2) and prevNote.lyric == twoOn.lyric == ' ' and	not nextNote.lyric == ' '):	# double neighbor 
					myNote.lyric = 'DN'
					nextNote.lyric = 'DN'
				elif (firstInt == 1 and secondInt == -2 and nextBeatStrength > myBeatStrength and (prevNote.lyric == ' ' or prevNote.lyric == 'S') and nextNote.lyric == ' ' and not pRest): 
#					print myNote.pitch, myNote.lyric, myNote.offset, RN.figure, RNPitchClasses
					myNote.lyric = 'WBsus'																								# weak beat suspension!
					print_everything ('WEAK BEAT suspension: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (notesLeft > 1 and firstInt == 1 and twoOnInt == -2 and twoOnBeatStrength > myBeatStrength and (secondInt == -3 or secondInt == -5) and prevNote.lyric == ' ' and twoOn.lyric == ' '): 
					myNote.lyric = 'WBsus'																								# decorated WB sus!
					print_everything ('WEAK BEAT suspension: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (notesLeft > 2 and firstInt == 1 and threeOnInt == -2 and threeOnBeatStrength <= myBeatStrength and secondInt == 2 and
				twoOnInt == 1 and prevNote.lyric == ' ' and threeOn.lyric == ' ' and nextNote.duration.quarterLength == .25 and 
				twoOn.duration.quarterLength == .25):
					myNote.lyric = 'DS3'
					get_notes(myNote.offset, locList)
					nextNote.lyric = 'Na'
					twoOn.lyric = 'S'
				elif (firstInt == 1 and prevNote.lyric == ' ' and note_extends_to_RN_boundary(myNote) and not pRest): 
#					print notesLeft, firstInt, threeOnInt, threeOnBeatStrength, myBeatStrength, secondInt, twoOnInt, prevNote.lyric == ' ', threeOn.lyric == ' ', nextNote.duration.quarterLength, twoOn.duration.quarterLength
					myNote.lyric = 'PED'																								# pedal	
					print_everything ('PEDAL: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice')
				elif (firstInt == -7 and secondInt == 2) or (firstInt == 7 and secondInt == -2): 								# PT plus octave switch
					if nextBeatStrength >= myBeatStrength: 
						if firstInt == 7:
							myNote.lyric = 'Pd'
						else:
							myNote.lyric = 'Pa'
					else:
						if firstInt == 7:
							myNote.lyric = 'APd'
						else:
							myNote.lyric = 'APa'
				#print("HERE", myNote.nameWithOctave, myNote.lyric)
				if myNote.lyric == '?':
					message = [str(n) + 'm' + str(mNumber), 'Voice ' + str(voice) + ' ' + myNote.lyric]
					tempMsg = ['UNIDENTIFIED TONE: Chorale', n, 'Measure', mNumber, 'Beat', myBeat, voiceName, 'voice \n']
					tempMsg += ['ints [first, second, twoOn, threeOn]: ', firstInt, secondInt, twoOnInt, threeOnInt, ' \n']
					tempMsg += ['strengths ', prevBeatStrength, myBeatStrength, nextBeatStrength, twoOnBeatStrength, threeOnBeatStrength]
					print_everything(tempMsg)
					#print myNote.nameWithOctave, RN.key.pitchAndMode, RN.figure, RNPitchClasses
#					stopSearching = True
#					display_problem_measures(chorale, RN.offset, RNbeat, message)
				dummyList = []
				counts[myNote.lyric] = counts.setdefault(myNote.lyric, 0) + 1
				if myNote.lyric in locations:
					locations[myNote.lyric].append(locList)
				else:
					locations[myNote.lyric] = [locList]
	if stopSearching:
		add_roman_numerals(n)
		chorale.show()
		final_print()
		return True
	final_print()
	return False

def final_print():
	global outputMessages
	myKeys = sorted(list(outputMessages))
	if not myKeys:
		return
	for item in myKeys:
		for theStr in outputMessages[item]:
			print('Measure', item, theStr)

def print_everything(*s):
	global outputMessages
	skip = -1
	if len(s) == 1:
		s = s[0]
	newS = []
	for i, item in enumerate(s):
		if item == 'Measure':
			mNumber = s[i+1]
			skip = i
		newS.append(str(item))
	if skip == -1: 
		return
	myMeas = s[skip + 1]
	myList = ' '.join(newS[:skip] + newS[skip+2:])
	if myMeas in outputMessages:
		outputMessages[myMeas].append(myList)
	else:
		outputMessages[myMeas] = [myList]

def get_notes(offset, locList):
	global chorale, roman_numerals, suspensionDoublings, suspensionOffsets, myNote, mNumber, suspensionCounts, suspensionLocations
	if offset in suspensionOffsets: return
	suspensionOffsets.append(offset)
	theNotes = []
	RN = roman_numerals.getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['RomanNumeral'])
	if not RN or not myNote: return
	RN = RN[0]
	RNpcs = [x.midi % 12 for x in RN.pitches]
	transp = RNpcs[0]
	RNpcs = [(x - transp) % 12 for x in RNpcs]
	for part in chorale.parts:
		note = part.flat.getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Note'])
		if not note: continue
		theNotes.append(note[0].pitch.pitchClass)
	theNotes.reverse()
	theNotes = [(x - transp) % 12 for x in theNotes]
	theNotes[1:] = sorted(theNotes[1:])
	noDuplicateNotes = remove_duplicates(sorted(theNotes))
	suspensionString = str(RNpcs) + ' ' + str(theNotes)
	nf_suspensionString = str(RNpcs) + ' ' + str(noDuplicateNotes)
	if (myNote.pitch.pitchClass == (RN.fifth.pitchClass + 1) % 12) or (myNote.pitch.pitchClass == (RN.fifth.pitchClass + 2) % 12):
		suspensionCounts[2] += 1
		#print "    SUSPENDED FIFTH HERE", mNumber
		#print suspensionString
	elif (myNote.pitch.pitchClass == (RN.third.pitchClass + 1) % 12) or (myNote.pitch.pitchClass == (RN.third.pitchClass + 2) % 12):
		suspensionCounts[1] += 1
	elif (myNote.pitch.pitchClass == (RN.root().pitchClass + 1) % 12) or (myNote.pitch.pitchClass == (RN.root().pitchClass + 2) % 12):
		suspensionCounts[0] += 1
	
	#print suspensionString, nf_suspensionString
	if suspensionString in suspensionDoublings:
		suspensionDoublings[suspensionString] += 1
		suspensionLocations[suspensionString].append(locList)
	else:
		suspensionDoublings[suspensionString] = 1
		suspensionLocations[suspensionString] = [locList]

def remove_duplicates(sortedList):
	lastNum = None
	outList = []
	for item in sortedList:
		if item != lastNum:
			outList.append(item)
			lastNum = item
	return outList

"""
this does not deal with NH tones preceded by rests; could be a problem
DA sixteenths still aren't perfect.  Build a list of which should be removed:
	1) prev note is seventh, descent is to fifth
"""
def remove_nonharmonic_tones():												# must be called after "dochorale"
	global chorale, analyzed_file
	for voice in range (0, len(chorale.parts)):
		offsetList = []
		preScore = chorale.parts[voice].flat.getElementsByClass('Note')
		offsetList = [note.offset for note in preScore]						# iterate over these offsets, not the notes themselves (since we will modify those)
		badOffsets = []														# for notes that will be removed
		for i in range(1, len(offsetList) - 1):
			if offsetList[i] in badOffsets:
				continue
			myMeasure = analyzed_file[1].getElementsByOffset(offsetList[i], offsetList[i], includeEndBoundary = True, mustBeginInSpan = False)
			if not myMeasure: continue
			myMeasure = myMeasure[0]
			scoreMeasure = chorale.parts[voice].getElementsByOffset(offsetList[i], offsetList[i], includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure'])[0]
			measureIndex = chorale.parts[voice].index(scoreMeasure)
			myNote = chorale.parts[voice].flat.getElementsByOffset(offsetList[i], offsetList[i] + .01, includeEndBoundary = False, mustBeginInSpan = True, classList = ['Note'])
			if not myNote:
				continue
			myNote = myNote[0]
			myNote.tie = None
			nextNote = chorale.parts[voice].flat.getElementAfterElement(myNote, classList = ['Note'])
			prevNote = chorale.parts[voice].flat.getElementBeforeOffset(myNote.offset, classList = ['Note'])
			twoOn = chorale.parts[voice].flat.getElementAfterElement(nextNote, classList = ['Note'])
			prevBeatStrength = get_beat_strength(prevNote.offset, voice)
			try:
				RN = roman_numerals.getElementsByOffset(myNote.offset, myNote.offset, includeEndBoundary = True, mustBeginInSpan = False)[0]
			except:
				print('NO ROMAN NUMERAL HERE', myMeasure.number, myNote.offset)
			RNPitchClasses = DT.fix_roman(RN)
			if RN.seventh:
				seventh = DT.seventh
			else:
				seventh = -9999						
#			myBeat = myNote.offset - myMeasure.offset
			myBeatStrength = get_beat_strength(myNote.offset, voice)
			"""REMOVING THE DAs still needs more work"""
			if not myNote.lyric:
				continue
			elif myNote.duration.quarterLength == .25 and myNote.lyric == 'DA' and nextNote.duration.quarterLength == .25:								
				if nextNote.lyric != ' ': 
					scoreMeasure.remove(myNote)
					scoreMeasure.remove(nextNote)
					badOffsets.append(offsetList[i+1])
					prevNote.duration.quarterLength += .5
				else:
#					prevNote.duration.quarterLength += .25
#					scoreMeasure.remove(myNote)
					nRN = roman_numerals.getElementsByOffset(nextNote.offset, nextNote.offset, includeEndBoundary = True, mustBeginInSpan = False)[0]
					if nRN.seventh:
						nSeventh = nRN.seventh.pitchClass
					else:
						nSeventh = -99999
					if prevNote.pitch.pitchClass == nSeventh:         # this should work
						scoreMeasure.remove(myNote)
						scoreMeasure.remove(nextNote)
						badOffsets.append(offsetList[i+1])
						prevNote.duration.quarterLength += .5
					else:
						myNote.pitch = nextNote.pitch												
						myNote.lyrics = []															
						myNote.duration.quarterLength += nextNote.duration.quarterLength
						chorale.parts[voice][measureIndex].remove(nextNote)
						badOffsets.append(offsetList[i+1])
			elif myNote.lyric == 'S' or myNote.lyric == 'DS' or myNote.lyric == 'DS2' or myNote.lyric == 'DS3' or myNote.lyric == 'WBsus':		# do suspensions this way; because the note
				myNote.transpose('-m2', inPlace = True)																	# can change (see no. 12)
				myNote.lyric = ' '
				if myNote.pitch.pitchClass not in RNPitchClasses:
					myNote.transpose('-A1', inPlace = True)
					if myNote.pitch.pitchClass not in RNPitchClasses:
						myNote.transpose('-A1', inPlace = True)
						if myNote.pitch.pitchClass not in RNPitchClasses:
							myNote.transpose('-A1', inPlace = True)
							if myNote.pitch.pitchClass not in RNPitchClasses: print('Error ', voice, myNote.pitch, RN.figuremy, myMeasure.number)
			elif myNote.lyric and (myNote.duration.quarterLength == .5 and myNote.lyric != ' ' and myNote.lyric != 'PED'):	# remove eighth-note nonharmonic tones
				if (myNote.offset - int(myNote.offset) > .05):											# start with weak eighth note
					if prevNote.pitch.pitchClass in RNPitchClasses: 
						prevNote.duration.quarterLength += .5
						scoreMeasure.remove(myNote)
					elif nextNote.pitch.pitchClass in RNPitchClasses:
						myNote.pitch = nextNote.pitch													# don't know why it is safer to do it this way
						myNote.lyric = ' '																# mysteries of music21
						myNote.duration.quarterLength += nextNote.duration.quarterLength
						chorale.parts[voice][measureIndex].remove(nextNote)
						badOffsets.append(offsetList[i+1])
				else:
					if nextNote.pitch.pitchClass in RNPitchClasses:
						myNote.pitch = nextNote.pitch													# don't know why it is safer to do it this way
						myNote.lyric = ' ' 																# change the earlier note and delete the later
						myNote.duration.quarterLength += nextNote.duration.quarterLength				# keeps all offsets in place
						chorale.parts[voice][measureIndex].remove(nextNote)
						badOffsets.append(offsetList[i+1])
					elif prevNote.pitch.pitchClass in RNPitchClasses:
						myNote.pitch = prevNote.pitch													# do it this way; could be in a different measure
						myNote.lyrics = []
			elif myNote.lyric and myNote.duration.quarterLength == .25 and myNote.lyric != ' ' and myNote.lyric != 'PED':	# repeat this whole chunk for sixteenths; ugly
				if (myNote.offset - int(myNote.offset) > .05) and myNote.lyric[0:2] != 'AP':	   					# start with weak eighth sixteenth
					if prevNote.pitch.pitchClass in RNPitchClasses: 
						prevNote.duration.quarterLength += .25
						scoreMeasure.remove(myNote)
					elif nextNote.pitch.pitchClass in RNPitchClasses:
						myNote.pitch = nextNote.pitch													# don't know why it is safer to do it this way
						myNote.lyric = ' ' 															# mysteries of music21
						myNote.duration.quarterLength += nextNote.duration.quarterLength
						chorale.parts[voice][measureIndex].remove(nextNote)
						badOffsets.append(offsetList[i+1])
				else:
					if nextNote.pitch.pitchClass in RNPitchClasses:
						myNote.pitch = nextNote.pitch														# don't know why it is safer to do it this way
						myNote.lyric = ' '															# mysteries of music21
						myNote.duration.quarterLength += nextNote.duration.quarterLength
						chorale.parts[voice][measureIndex].remove(nextNote)
						badOffsets.append(offsetList[i+1])
					elif prevNote.pitch.pitchClass in RNPitchClasses:
						myNote.pitch = prevNote.pitch													# do it this way; could be in a different measure
						myNote.lyrics = []
			elif myNote.lyric and (myNote.duration.quarterLength == .5 and myNote.lyric == ' ' and (myNote.offset - int(myNote.offset) > .05) and	
			prevNote.duration.quarterLength == .5 and prevNote.pitch == myNote.pitch): 						# remove tied harmonic eighths
				prevNote.duration.quarterLength += .5
				scoreMeasure.remove(myNote)
			elif myNote.lyric and (myNote.duration.quarterLength == .25 and myNote.lyric == ' ' and (myNote.offset - int(myNote.offset) > .05) and
			prevNote.duration.quarterLength == .25 and prevNote.pitch == myNote.pitch): 						# remove tied harmonic eighths
				prevNote.duration.quarterLength += .25
				scoreMeasure.remove(myNote)
			elif myNote.duration.quarterLength == 1.0 and (myNote.lyric[0:2] == 'AN' or myNote.lyric[0:2] == 'AP'):
				myNote.pitch = nextNote.pitch
				myNote.lyric = ' '
			elif myNote.duration.quarterLength == 1.0 and (myNote.lyric == 'E' or myNote.lyric == 'A'):
				myNote.pitch = prevNote.pitch
				myNote.lyric = ' '	
			elif myNote.duration.quarterLength == 1.0 and (myNote.lyric[0] == 'P' 
			or myNote.lyric[0] == 'N') and prevNote.duration.quarterLength == 1.0 and prevBeatStrength > myBeatStrength: 
				myNote.pitch = prevNote.pitch
				myNote.lyric = ' '
			if myNote.lyric == ' ': myNote.lyrics = []
#	make_beams()
	return

"""would like to remove all the TimeSigs I insert into the piece"""

def make_beams():
	global chorale, analyzed_file
	for voice in range (0, len(chorale.parts)):
		measures = chorale.parts[voice].getElementsByClass('Measure')
		for myMeas in measures:
			if myMeas.timeSignature: 
				lastSig = myMeas.timeSignature
			else:
				myMeas.timeSignature = lastSig
			myMeas.makeBeams(inPlace = True)
#			myMeas.remove(lastSig) 					# CAN WE REMOVE ALL THESE TIME SIGS???

"""add 7DI and 7O+, which do occasionally appear"""
def annotate():
	global chorale, analyzed_file
	upperVoices = chorale.flat.getElementsByClass(['Note'])
	offsetList = sorted(list(set([note.offset for note in upperVoices])))
	for offset in offsetList:
		try:
			sop = chorale.parts[0].flat.getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Note', 'Rest'])[0]
			alt = chorale.parts[1].flat.getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Note', 'Rest'])[0]
			ten = chorale.parts[2].flat.getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Note', 'Rest'])[0]
			bass = chorale.parts[3].flat.getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Note', 'Rest'])[0]
		except:
			print('Prroblem with offset: ', offset)
			continue
		if 'Rest' in sop.classes + alt.classes + ten.classes + bass.classes: continue
		position = ' '
		if sop.midi < alt.midi or alt.midi < ten.midi: position = '-'
		myChord = chord.Chord([sop, alt, ten, bass])
		pcs = set([sop.pitchClass, alt.pitchClass, ten.pitchClass, bass.pitchClass])
		iTenAlt = interval.notesToGeneric(ten, alt).value
		iAltSop = interval.notesToGeneric(alt, sop).value
		iTenSop = interval.notesToGeneric(ten, sop).value
		if myChord.seventh:
			position = '7?'
			if (iTenAlt == 2 and iAltSop == 3) or (iTenAlt == 3 and iAltSop == 3) or (iTenAlt == 3 and iAltSop == 2): position = '7C'
			elif (iTenAlt == 5 and iAltSop == 3) or (iTenAlt == 5 and iAltSop == 2) or (iTenAlt == 4 and iAltSop == 3): position = '7SC'
			elif (iTenAlt == 2 and iAltSop == 5) or (iTenAlt == 3 and iAltSop == 5) or (iTenAlt == 3 and iAltSop == 4): position = '7SC'
			elif iTenSop == 8 and (iTenAlt > 1 and iAltSop > 1): position = '7H'
			elif (iTenAlt == 5 and iAltSop == 6) or (iTenAlt == 4 and iAltSop == 7) or (iTenAlt == 4 and iAltSop == 6): position = '7SO'
			elif (iTenAlt == 6 and iAltSop == 4) or (iTenAlt == 7 and iAltSop == 4) or (iTenAlt == 6 and iAltSop == 5): position = '7SO'  
			elif (iTenAlt == 6 and iAltSop == 6) or (iTenAlt == 7 and iAltSop == 6) or (iTenAlt == 6 and iAltSop == 7): position = '7O'
		else:
			if iTenSop < 8 and (iTenAlt == 1 or iAltSop == 1): position = 'DI'
			elif (iTenSop == 5 or iTenSop == 6) and (iTenAlt == 3 or iTenAlt == 4) and (iAltSop == 3 or iAltSop == 4): position = 'C'
			elif iTenSop == 8 and (iTenAlt > 1 and iAltSop > 1): position = 'H'
			elif (iTenSop == 10 or iTenSop == 11) and (iTenAlt == 5 or iTenAlt == 6) and (iAltSop == 5 or iAltSop == 6): position = 'O'
			elif iTenSop > 8 and (iTenAlt == 8 or iAltSop == 8): position = 'O+'	
		if not position == ' ':
			myLabel = expressions.TextExpression(position)
			myLabel.positionVertical = 35			
			myLabel.justify = 'center'
			measure = chorale.parts[0].getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure'])
			mStart = measure[0].offset
			mOffset = offset - mStart
			measure[0].insert(mOffset, myLabel)	

def get_beat_strength(offset, tempDummy):
	global analyzed_file, timeSigs
	measure = analyzed_file[1].getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure'])
	if not measure:
		return 0
#	print offset, measure[0].number
	mStart = measure[0].offset
	mOffset = offset - mStart
	timeSig = timeSigs.getElementAtOrBefore(offset)
	return timeSig.getAccentWeight(mOffset, forcePositionMatch = True)

def add_roman_numerals(i, twoLevel = False):
	global chorale, analyzed_file, roman_numerals
	oldKey = '?'
	label = 'Piece ' + str(i)
	labelOne = expressions.TextExpression(label)
	labelOne.positionVertical = 65
	measure = chorale.parts[0].getElementsByOffset(1, 1, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure'])
	measure[0].insert(0, labelOne)
	counter = 0
	for RN in roman_numerals: 
		if 'Rest' in RN.classes: continue
		counter += 1
		label = RN.figure
		if RN.key.mode == 'minor':
			keyStr = str(RN.key.tonic.name).lower() + ': '
		else:
			keyStr = str(RN.key.tonic.name) + ': '
		if keyStr != oldKey:
			oldKey = keyStr
			label = keyStr + label
		labelOne = expressions.TextExpression(label)
		labelOne.positionVertical = -105
		if twoLevel and (counter % 2) == 0:
			labelOne.positionVertical = -125			
		labelOne.justify = 'center'
		partNo = len(chorale.parts) - 1
		measure = chorale.parts[partNo].getElementsByOffset(RN.offset, RN.offset, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure'])
		if not measure: continue
		mStart = measure[0].offset
		mOffset = RN.offset - mStart
		measure[0].insert(mOffset, labelOne)
			
			
def getstats(percent = False):
	global counts, progDict
	list = sorted(counts.iteritems(), key=lambda x:x[1], reverse = True)
	if percent:
		total = 0
		for name, count in list:
			total += count
		for i in range(len(list)):
			list[i] = (list[i][0], round(100.0 * list[i][1]/total, 2))
	return list

def suspensions_in_normal_form(printIt = False):
	global suspensionDoublings, nf_suspensionDoublings
	nf_suspensionDoublings = {}
	for item in suspensionDoublings:
		count = suspensionDoublings[item]
		if printIt: print(item, count)
		pair = item.split('] [')
		harmony = pair[0] + ']'
		doublingSonority = pair[1]
		ints = []
		doublingSonority = doublingSonority.replace(']','')
		element = doublingSonority.split(',')
		#print 'string', element
		for myNum in element:
			myNum = myNum.replace(' ', '')
			ints.append(int(myNum))
		#print 'ints', ints, set(ints)
		ints = sorted(ints)
		newInts = []
		lastNum = -8
		for i in ints:
			if i != lastNum:
				newInts.append(i)
				lastNum = i
		newKey = harmony + str(newInts)
		if printIt: print(item, harmony, newInts, count)
		nf_suspensionDoublings[newKey] = nf_suspensionDoublings.setdefault(newKey, 0) + count

fifthSuspensions = ['[0, 4, 7][0, 4, 9]', '[0, 3, 7][0, 3, 9]', '[0, 3, 6][0, 3, 8]', '[0, 4, 8][0, 4, 10]', '[0, 3, 8][0, 5, 8]', '[0, 4, 9][0, 5, 9]', '[0, 3, 9][0, 5, 9]']

def fifth_suspensions():
	suspensions_in_normal_form()
	totalCounts = 0
	for item in nf_suspensionDoublings:
		if item in fifthSuspensions:
			totalCounts += nf_suspensionDoublings[item]
			print(item, nf_suspensionDoublings[item])
	return [totalCounts, sum(nf_suspensionDoublings.values())]
							
	