from __future__ import print_function
import os
import pickle
import time
import re

from music21 import *
import matplotlib.pyplot as plt
from scipy.stats.stats import pearsonr

import DT

"""
Look at upper-voice configurations to see how often the 'OUCH' positions appear.

"""

_THISFILE = 'OUCH'

pieceCounts = {}


def gather_stats(source = 'BACH', fileName = '', printOut = False):
	global progs, piece, globalSource, locNames, globalPrint, centersOfGravity, completeProgs
	startT = time.time()
	globalPrint = printOut
	globalSource = source.upper()
	progs = {}
	centersOfGravity = {}
	completeProgs = {}
	locNames = {}
	if globalSource not in DT._PATHS:
		print("Source not found!")
		return
	scorePath = DT._PATHS[globalSource][1]
	files = sorted(os.listdir(scorePath))
	counter = 1
	for myFile in files:
		if myFile[-3:] in ['xml', 'krn', 'mxl']:
			if fileName and myFile != fileName: continue
			print('Analyzing', myFile)
			piece = converter.parse(scorePath + myFile)
			if 'Opus' in piece.classes:
				bigOpus = piece
				for piece in bigOpus.scores:
					dopiece(counter)
			else:
				dopiece(counter)
			locNames[counter] = myFile[:-4]
			counter += 1
			if counter % 50 == 0:
				save()
	if not fileName:
		save()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))

def save():
	with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'wb') as outputFile:
		pickle.dump(progs, outputFile)
		pickle.dump(locNames, outputFile)
		pickle.dump(centersOfGravity, outputFile)
		pickle.dump(completeProgs, outputFile)

def load(source = 'PALRAW'):
	global masterProgs, globalSource, locNames, centersOfGravity, completeProgs
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'rb') as inputFile:
			masterProgs = pickle.load(inputFile)
			locNames = pickle.load(inputFile)
			centersOfGravity = pickle.load(inputFile)
			completeProgs = pickle.load(inputFile)
		get_data()
		return True
	except:
		return False
		
	
def get_data(pieceList = [], skipMax = 2, filterList = ['-DV->']):
	"""use filterList to eliminate some voice leadings, like those with different voices"""
	global progs, masterProgs, globalSource, locNames, rootProgs, centersOfGravity, rootProgs2
	if not pieceList:
		if globalSource not in pieceCounts:
			pieceList = masterProgs.keys()
		else:
			pieceList = pieceCounts[globalSource]
	progs = {}
	for i in pieceList:
		for j in masterProgs[i]:
			if j <= skipMax:
				combine_dicts(progs, masterProgs[i][j])
	for k in list(progs.keys()):
		for f in filterList:
			if k.count(f) > 0:
				del progs[k]
				break
	
def average_motions(targetVL = False, skipMax = 2, filterZero = False):
	global progs, masterProgs, globalSource, locNames, rootProgs, centersOfGravity, rootProgs2
	theForms = ['Udi', 'C', 'H', 'O', 'Uoo']
	if not targetVL:
		targetVL = [x + ' -> ' + y for x in theForms for y in theForms]
	if type(targetVL) is str:
		targetVL = [targetVL]
	for t in targetVL:
		motions = []
		for i in centersOfGravity.keys():
			for j in centersOfGravity[i]:
				if j <= skipMax:
					motions += centersOfGravity[i][j].get(t, [])
		motions = [abs(x) for x in motions]
		if filterZero:
			motions = [x for x in motions if x]
		print(t, round(sum(motions)/float(len(motions)), 2))
	
def combine_dicts(outputDict, *otherDicts):
	for newDict in otherDicts:
		for k, v in newDict.items():
			outputDict[k] = outputDict.setdefault(k, 0) + v

"""def combine_dicts(*args):
	if len(args) == 1 and type(args[0]) is list:
		args = args[0]
	outputDict = {}
	for newDict in args:
		for k, v in newDict.items():
			outputDict[k] = outputDict.setdefault(k, 0) + v
	return outputDict"""

def root_motion(s):
	tempDict = {'R': 0, 'T': 2, 'F': 4}
	bassMotion = int(s[4:])
	i = (bassMotion + tempDict[s[0]] - tempDict[s[1]]) % 7
	if i > 3: i = i - 7
	return i		

def dopiece(pieceNumber):	
	global progs, centersOfGravity, piece, consonantO, globalPrint
	score = [piece.parts[i].flat.getElementsByClass(['Note', 'Chord']).stream() for i in range(len(piece.parts))]
	numParts = len(score)
	if numParts < 4: return
	newO = sorted(list(set([n.offset for p in score for n in p])))
	consonantO = {}
	forms = {}
	originalNotes = {}
	progs[pieceNumber] = {}
	centersOfGravity[pieceNumber] = {}
	completeProgs[pieceNumber] = {}
	for o in newO:
		myMidi = [None] * numParts 
		myNotes = [None] * numParts
		foundChord = False
		for i, p in enumerate(score):
			tempNotes = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False)
			if tempNotes:
				for n in tempNotes:
					if 'Note' in n.classes:
						myNotes[i] = n
					elif 'Chord' in n.classes:					# redundant code, for later
						foundChord = True
						myNotes[i] = []
						for n1 in n.pitches:
							myNotes[i].append(note.Note(n1))
		if foundChord: continue
		theNotes = [x.pitch.midi if x else None for x in myNotes]
		rawNotes = list(set([x % 12 for x in theNotes if x]))
		if rawNotes and DT.normal_form(rawNotes) in [[0, 4, 7], [0, 3, 7], [0, 3, 6], [0, 4, 8], [0, 5], [0, 4]] and len(theNotes) - theNotes.count(None) == 4:
				consonantO[o] = theNotes
				forms[o] = ouch_catalog(theNotes)
				originalNotes[o] = myNotes
				if globalPrint: 
					chordName = [x.pitch.nameWithOctave for x in myNotes if x][::-1]
					print(DT.get_measure_number_and_beat_from_offset(piece, o), chordName, forms[o])
				
	oList = sorted(list(consonantO.keys()))
	for i in range(len(oList) - 1):
		o = oList[i]
		nextO = oList[i+1]
		if consonantO[o] == consonantO[nextO]: continue
		skippedChords = newO.index(nextO) - newO.index(o) - 1
		if  skippedChords > 2: continue
		#print(o, nextO, consonantO[o], consonantO[nextO])
		if same_voices(consonantO[o], consonantO[nextO]):
			if no_crossing(consonantO[o], consonantO[nextO]) and forms[o] and forms[nextO]:
				theSymbol = str(forms[o]) + ' -> ' + str(forms[nextO])
			else:
				theSymbol = str(forms[o]) + ' -C-> ' + str(forms[nextO])			# crossing
			rawVL = raw_vl(originalNotes[o], originalNotes[nextO])
		else:
			theSymbol = str(forms[o]) + ' -DV-> ' + str(forms[nextO])				# different voices
			rawVL = False
		if consonantO[o] == consonantO[nextO]:
			continue
		else:
			#print originalNotes[nextO], originalNotes[o]
			deltaO = center_of_gravity(originalNotes[nextO]) - center_of_gravity(originalNotes[o])
			if skippedChords not in progs[pieceNumber]:
				progs[pieceNumber][skippedChords] = {}
				centersOfGravity[pieceNumber][skippedChords] = {}
				completeProgs[pieceNumber][skippedChords] = {}
			progs[pieceNumber][skippedChords][theSymbol] = progs[pieceNumber][skippedChords].setdefault(theSymbol, 0) + 1
			centersOfGravity[pieceNumber][skippedChords].setdefault(theSymbol, []).append(deltaO)
			if rawVL: 
				completeProgs[pieceNumber][skippedChords][rawVL] = completeProgs[pieceNumber][skippedChords].get(rawVL, 0) + 1
				#print [x.nameWithOctave for x in originalNotes[o] if x], [x.nameWithOctave for x in originalNotes[nextO] if x], rawVL
			if globalPrint: 
				print(consonantO[o], consonantO[nextO], theSymbol, deltaO)

def raw_vl(chord1, chord2):
	dn1 = [DT.note_to_scalarMIDIdegree(x) for x in reversed(chord1) if x]
	dn2 = [DT.note_to_scalarMIDIdegree(x) for x in reversed(chord2) if x]
	bassNote = min(dn1)
	dn1 = [x - bassNote for x in dn1]
	dn2 = [x - bassNote for x in dn2]
	return tuple([tuple(dn1), tuple(dn2)])

def ouch_catalog(noteList):
	myNotes = sorted([x for x in noteList if x])
	b, t, a, s = myNotes
	if (a - t in [3, 4, 5, 6] and s - a in [3, 4, 5, 6]):
		return 'C'
	elif t + 12 == s and s > a > t:
		return 'H'
	elif (a - t in [6, 7, 8, 9] and s - a in [6, 7, 8, 9]):
		return 'O'
	elif t == a or a == s and s - t < 12:
		return 'Udi'
	elif t + 12 == a and s - a < 12:
		return 'Uoo'
	return False

def center_of_gravity(theNotes):
	scaleDegrees = [DT.note_to_scalarMIDIdegree(x) for x in theNotes[1:] if x]
	return sum(sorted(scaleDegrees)[1:])/3.

def same_voices(n1, n2):
	ol1 = set(order_list(n1))
	ol2 = set(order_list(n2))
	if ol1 == ol2:
		return True
	return False

def no_crossing(n1, n2):
	ol1 = order_list(n1)
	ol2 = order_list(n2)
	if ol1 == ol2:
		return True
	for i, index in enumerate(ol1[:-1]):
		index2 = ol1[i+1]
		if n1[index] == n1[index2]: continue
		if n2[index] > n2[index2]: 
			return False
	for i, index in enumerate(ol2[:-1]):
		index2 = ol2[i+1]
		if n2[index] == n2[index2]: continue
		if n1[index] > n1[index2]: 
			return False	
	return True

def order_list(n1):
	l = sorted(list(range(len(n1))), key = lambda x: sortkey(n1[x]))
	return [x for x in l if n1[x]]

def sortkey(i):
	if type(i) is int:
		return i
	return 0
		
def fix_int(i):
	if i > 0: return i -1
	return i + 1

def count_formations(printOut = True):
	global counts
	counts = {}
	for p, tempCount in progs.items():
		firstChord = p.split(' -')[0]
		counts[firstChord] = counts.get(firstChord, 0) + tempCount
	if printOut: 
		DT.print_dict(counts, pct = True)

allProgs = {}

"""A BUNCH OF ROUTINES FOLLOW FOR JUDGING THE INDEPENDENCE OF THE FOUR VOICES USING THE TOTALMOTIONS VARIABLE"""

"""assemble list from individual files"""
def get_total_motions(skipLength = 2):
	global totalMotions
	totalMotions = {}
	for pieceNumber in completeProgs:
		for i in completeProgs[pieceNumber]:
			if i > skipLength: continue
			for vl, count in completeProgs[pieceNumber][i].items():
				totalMotions[vl] = totalMotions.get(vl, 0) + 1

"""calculate average distance between voices"""
def average_spacing(sortChords = True):
	get_total_motions()
	voiceSpacing = [0, 0, 0]
	totalCount = 0
	for vl, count in totalMotions.items():
		firstChord, secondChord = vl
		if sortChords:
			firstChord = sorted(firstChord)
		distances = [abs(firstChord[i] - firstChord[i-1]) for i in range(1, len(firstChord))]
		totalCount += count
		for i in range(len(voiceSpacing)):
			voiceSpacing[i] += distances[i]*count
	averages = [float(x)/totalCount for x in voiceSpacing]
	print("AVERAGE DISTANCES BETWEEN VOICE PAIRS, BOTTOM TO TOP [B-T, T-A, A-S]", [round(x, 2) for x in averages])

"""average motion of each voice"""
def average_voice_motions():
	get_total_motions()
	voiceSpacing = [0, 0, 0, 0]
	totalCount = 0
	for vl, count in totalMotions.items():
		firstChord, secondChord = vl
		distances = [(secondChord[i] - firstChord[i]) for i in range(len(firstChord))]
		totalCount += count
		for i in range(len(voiceSpacing)):
			voiceSpacing[i] += abs(distances[i])*count
	averages = [float(x)/totalCount for x in voiceSpacing]
	print("AVERAGE MOTION IN EACH VOICE (B, T, A, S)", [round(x, 2) for x in averages])

"""inverval histogram, voice by voice"""
def voice_histogram(voices = [0, 1, 2, 3], absoluteValue = False):
	if type(voices) is int:
		voices = [voices]
	get_total_motions()
	voiceSpacing = [{}, {}, {}, {}]
	totalCount = 0
	for vl, count in totalMotions.items():
		firstChord, secondChord = vl
		distances = [(secondChord[i] - firstChord[i]) for i in range(len(firstChord))]
		if absoluteValue:
			distances = [abs(x) for x in distances]
		totalCount += count
		for i in range(len(voiceSpacing)):
			voiceSpacing[i][distances[i]] = voiceSpacing[i].get(distances[i], 0) + count
	for i in voices:
		print("\n", ['BASS', 'TENOR', 'ALTO', 'SOPRANO'][i], "\n")
		DT.print_dict(voiceSpacing[i], pct=True)


"""correlation of each voice"""
def voice_correlations():
	global voiceSpacing
	voiceNames = ['BASS', 'TENOR', 'ALTO', 'SOPRANO']
	get_total_motions()
	voiceSpacing = [[], [], [], []]
	for vl, count in totalMotions.items():
		firstChord, secondChord = vl
		distances = [(secondChord[i] - firstChord[i]) for i in range(len(firstChord))]
		for i in range(len(voiceSpacing)):
			voiceSpacing[i].append(float(distances[i]))
	for i in range(3):
		for j in range(i+1, 4):
			print("CORRELATION BETWEEN", voiceNames[i], "AND", voiceNames[j], pearsonr(voiceSpacing[i], voiceSpacing[j]))

"""voices in contrary motion"""	
def percent_contrary(voicePair = [0, 3]):
	get_total_motions()
	contrary = 0
	totalCount = 0
	for vl, count in totalMotions.items():
		firstChord, secondChord = vl
		distances = [(secondChord[i] - firstChord[i]) for i in range(len(firstChord))]
		if distances[voicePair[0]] * distances[voicePair[1]] < 0:
			contrary += count
		totalCount += count
	return contrary * 100./totalCount

"""which voices have the most doublings"""	
def doubling_locations():
	get_total_motions()
	doublings = [0, 0, 0, 0]
	totalCount = 0
	for vl, count in totalMotions.items():
		firstChord, secondChord = vl
		firstChord = [x % 7 for x in firstChord]
		doublingLocations = [i for i in range(len(firstChord)) if firstChord.count(firstChord[i]) > 1]
		for i in doublingLocations:
			doublings[i] += count
	return [x*100./(sum(doublings)) for x in doublings]

"""which voice is most likely to have which elements"""
def chord_roots(myVoice = 0, sortChords = True):
	get_total_motions()
	chordElements = [0, 0, 0]
	totalCount = 0
	for vl, count in totalMotions.items():
		firstChord, secondChord = vl
		if sortChords:
			firstChord = sorted(firstChord)
		firstChord = [(x - min(firstChord)) % 7 for x in firstChord]
		setClass = list(set(firstChord))
		if setClass == [0, 2, 4]:
			r, t, f = [0, 2, 4]
		elif setClass == [0, 2, 5]:
			t, f, r = [0, 2, 5]
		elif setClass == [0, 3, 5]:
			f, r, t = [0, 3 ,5]
		else:
			continue
		chordIndex = [r, t, f].index(firstChord[myVoice])
		chordElements[chordIndex] += count
	print("RAW COUNTS", chordElements)
	print("% root, third, and fifth for voice", myVoice, [round(x*100./sum(chordElements), 2) for x in chordElements])

def regex(patternStrings, printIt = True, pct = True, filterRare = 2, useSum = False):
	global results, progs, rootProgs, centersOfGravity, rootProgs2, allProgs, bigSum
	source = progs
	results = {}
	theSums = {}
	if type(patternStrings) is str:
		patternStrings = [patternStrings] 
	bigSum = sum(source.values())
	for prog in source:
		for patternString in patternStrings:
			if patternString not in results:
				results[patternString] = []
			if re.match(patternString, prog):
				results[patternString].append([prog, source[prog]])
	for patternString in patternStrings:
		tempList = sorted(results[patternString], key=lambda x: x[1], reverse = True)
		mySum = sum([x[1] for x in tempList])
		print(patternString, mySum, "%.1f of all progressions" % (100. * mySum/bigSum))
		if useSum:
			tempList = [[patternString, mySum * mySum/bigSum]]
		if printIt:
			for pair in tempList: 
				if pct:
					p = (100. * pair[1]/mySum)
					#print pair[0], allProgs.setdefault(pair[0], [])
					allProgs[pair[0]] = allProgs.setdefault(pair[0], [])
					allProgs[pair[0]].append(p)
					if not filterRare or p > filterRare:
						print(pair[0], "%.1f" % p)
				else:
					print(pair[0], pair[1])

def doall(*args, **kwargs):
	global allProgs
	allProgs = {}
	for c in ['OCKEGHEM', 'JOSQUINCOMPLETE', 'PALRAW']:
		print('Composer ', c)
		load(c)
		regex(*args, **kwargs)
	print() 
	theKeys = [x for x in allProgs.keys() if len(allProgs[x]) > 2]
	k = sorted(theKeys, key = lambda x: -abs(allProgs[x][0] - allProgs[x][-1]))
	anomalies = []
	for theKey in k:
		print(theKey, ' '.join(["%.1f" % x for x in allProgs[theKey]]))
		data = allProgs[theKey]
		if (data[1] > data[0] and data[1] > data[2]) or (data[1] < data[0] and data[1] < data[2]):
			anomalies.append(theKey)
	print()
	print()
	print()
	print("ANOMALIES")
	for theKey in anomalies:
		print(theKey, ' '.join(["%.1f" % x for x in allProgs[theKey]]))

"""
		
What tendencies are there among OUCH motion?  Does following a C formation make another formation more or less likely?
changeDict gives a list of the changes in probability.
		
Filterlist eliminates chords from consideration like 'False' (used for non-OUCH chords)
		
"""
		
def first_order_second_order(filterList = ['False']):
	global firstOrderProbs, changeDict
	firstOrderDict = {}
	changeDict = {}
	for p, count in progs.items():
		firstChord = p.split()[0]
		if firstChord in filterList: continue
		firstOrderDict[firstChord] = firstOrderDict.get(firstChord, 0) + count
	firstOrderProbs = {x[0]:x[1]*100./sum(firstOrderDict.values()) for x in firstOrderDict.items()}
	for firstChord in firstOrderProbs.keys():
		limitedDict = {x[0]:x[1] for x in progs.items() if x[0].startswith(firstChord + ' ')}
		limitedDict = {x[0]:x[1]*100./sum(limitedDict.values()) for x in limitedDict.items()}
		for secondChord in firstOrderProbs.keys():
			endsWithProgs = sum([x[1] for x in limitedDict.items() if x[0].endswith(' ' + secondChord)])
			difference = endsWithProgs - firstOrderProbs.get(secondChord, 0)
			changeDict[firstChord + ' -> ' + secondChord] = difference
	DT.print_dict(changeDict)

"""
Make a nice graph of the OUCH formations for different composers; also summarizes various properties of their usage
"""

def graph(comps = ['PALRAW', 'BACH']):
	global output, ax
	
	labelSubs = {'Udi': 'DI', 'Uoo': 'OO', 'False': '?'}
	nameSubs = {'PALRAW': 'Palestrina', 'JOSQUINCOMPLETE': 'Josquin', 'VICTORIARAW': "Victoria"}
					
	xyBoundaries = [[0, 16], [0, 12]]
	fig, ax = plt.subplots()
	#fig.set_size_inches(6, 6)
	ax.axes.get_xaxis().set_visible(False)
	ax.axes.get_yaxis().set_visible(False)
	ax.set_xlim(xyBoundaries[0][0], xyBoundaries[0][1])
	ax.set_ylim(xyBoundaries[1][0], xyBoundaries[1][1])
	ax.set_title('configuration size is proportional to occurrence frequency')
	
	orderedPositions = ['DI', 'C', 'H', 'O', 'OO']
		
	#DT.print_dict(output, pct = True)
	
	for j, comp in enumerate(comps):
		
		load(comp)
		count_formations(printOut = False)
		compName = nameSubs.get(comp.upper(), comp)
		compName = compName[0] + compName[1:].lower()
		totals = float(sum(counts.values()))
		output = {labelSubs.get(s, s):counts[s] * 100./ totals for s in counts}
		
		newProgs = {}
		totalLabels = []
		
		for sym, count in progs.items():
			if sym.count('False') > 0:
				continue
			newSym = sym
			for s1, s2 in labelSubs.items():
				newSym = newSym.replace(s1, s2)
			newProgs[newSym] = count
		
		#print newProgs
		#newProgs = {labelSubs.get(x[0], x[0]):x[1] for x in progs.items() if x[0].count('False') == 0}
		
		print('\n', compName, "transition frequencies\n")
		totalSum = sum(newProgs.values())
		
		for symType, mySym in enumerate([' -> ', ' -C-> ', ' -DV-> ']):
	
			graphTotal = 0
	
			for i in range(len(orderedPositions) - 1):
				for s in [orderedPositions[i] + mySym + orderedPositions[i+1], orderedPositions[i + 1] + mySym + orderedPositions[i]]:
					totalLabels.append(s)
					graphTotal += newProgs.get(s, 0)
	
			for s in ['C' + mySym + 'C', 'O' + mySym + 'O', 'C' + mySym + 'O', 'O' + mySym + 'C']:
				graphTotal += newProgs.get(s, 0)
				totalLabels.append(s)
			
			print(["   NOCROSSING", "   CROSSING", "   DIFFERENT VOICES"][symType], "GRAMMATICAL TOTAL", graphTotal * 100./totalSum, '\n')
		
		print("   MISSING ITEMS")
		missedGuys = [x for x in newProgs.keys() if x not in totalLabels]
		missedGuys = sorted(missedGuys, key = lambda x: -newProgs[x])
		
		for x in missedGuys[:10]:
			print("   ", x, newProgs[x] * 100./totalSum)
		
		
		DT.print_dict(newProgs, pct = True)
		

		for i, s in enumerate(orderedPositions + ['?']):
			x = i * (11. / 5) + 1
			y = 3.5 * j + 2
			ax.text(x, y, s, horizontalalignment = 'center', verticalalignment = 'center', fontsize = 2. * output[s], fontname = 'Times')
			ax.text(13, y, compName, horizontalalignment = 'left', verticalalignment = 'center', fontsize = 16, fontname = 'Times')
		
		
	fig.canvas.draw()
	plt.ion()
	plt.show()
			
