from __future__ import print_function
import os, time
from music21 import *
import DT
import pickle
import copy

"""
Analyzes raw scores looking for direct fifths and octaves (where the soprano leaps), parallel fifths and octaves, voice overlap, antiparallel fifths, etc.

Particularly useful when applied to REDUCED files as then it can be used to reveal parallels that are hidden by nonharmonic tones.  (You have to do this by hand, comparing
the lists of parallels in unreduced and reduced files, but that is not too much of a problem.)

The easiest way to use it is just to run gather_stats and look at the output; however the output is saved in case you want it.

NB: for overlaps, I have filtered out cases where the bass moves by octave

"""

_THISFILE = 'OVERLAP'
directFifthsOctavesLocations = []
overlapLocations = []
crossingsLocations = []
parallelsLocations = []
overlapVoices = {}
overlapCount = 0
crossingsCount = 0
parallelsCount = 0
antiParallelsCount = 0
directFifthsOctavesCount = 0
lastPiece = 1
output = stream.Score()
bigOpus = False

continuuoPieces = {'MONTEVERDI': 'madrigal.5'}

def gather_stats(source = 'MORLEYRED', fileName = ''):
	global piece, overlapCount, directFifthsOctavesCount, directFifthsOctavesLocations, globalSource, tritoneLocs, voiceCounter, genericOverlap, genericCrossings, bachDirect, genericParallels
	global overlapLocations, crossingsLocations, crossingsCount, parallelsCount, parallelsLocations, antiParallelsLocations, antiParallelsCount, genericDirect, genericAntiparallels
	global locNames, filePath, totalChords, totalConsonances, fileStr, melodyComments, myFile, consonances, bigOpus
	startT = time.time()
	overlapCount = 0
	directFifthsOctavesCount = 0
	crossingsCount = 0
	parallelsCount = 0
	antiParallelsCount = 0
	directFifthsOctavesLocations = []
	parallelsLocations = []
	overlapLocations = []
	antiParallelsLocations = []
	crossingsLocations = []
	genericDirect = {}
	genericOverlap = {}
	genericParallels = {}
	genericCrossings = {}
	genericAntiparallels = {}
	bachDirect = {}
	tritoneLocs = {}
	voiceCounter = {}
	locNames = {}
	totalChords = {}
	totalConsonances = {}
	melodyComments = {}
	globalSource = source.upper()	
	if globalSource in DT._PATHS:
		analysisPath = DT._PATHS[globalSource][0]
		scorePath = DT._PATHS[globalSource][1]
		fileStr = globalSource
		files = sorted(os.listdir(scorePath))
		recordResults = True
		counter = 1
		for myFile in files:
			if myFile[-3:] in ['xml', 'krn', 'mxl']:
				if fileName and myFile != fileName: continue
				print('Analyzing', myFile)
				piece = converter.parse(scorePath + myFile)
				if globalSource in continuuoPieces:
					for s in continuuoPieces[globalSource]:
						if myFile.startswith(s):
							#print "HERE!"
							piece.remove(piece.parts[-1])
							break
				bigOpus = False
				if 'Opus' in piece.classes:
					bigOpus = piece
					for subpieceNo, piece in enumerate(bigOpus.scores):
						print(" subpiece", subpieceNo)
						dopiece([counter, subpieceNo])
				else:
					dopiece(counter)
				locNames[counter] = myFile
				counter += 1
				if counter % 50 == 0:
					save()
		if not fileName:
			save()
		print("Crossings", crossingsCount, "Overlaps", overlapCount, "Direct Fifths and Octaves", directFifthsOctavesCount, "Tritones to Perfect", len(tritoneLocs))
		filePath = analysisPath
		minutes = (time.time() - startT)/60.
		print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	return

def save():
	with open(DT._DATAPATH + _THISFILE + globalSource + '.pkl', 'wb') as outputFile:
		pickle.dump(genericDirect, outputFile)
		pickle.dump(bachDirect, outputFile)
		pickle.dump(genericOverlap, outputFile)
		pickle.dump(genericCrossings, outputFile)
		pickle.dump(genericParallels, outputFile)
		pickle.dump(genericAntiparallels, outputFile)
		pickle.dump(tritoneLocs, outputFile)
		pickle.dump(voiceCounter, outputFile)
		pickle.dump(locNames, outputFile)
		pickle.dump(melodyComments, outputFile)
		pickle.dump(totalChords, outputFile)
		pickle.dump(totalConsonances, outputFile)

def dopiece(pieceNumber, xml = False):
	global piece, overlapCount, directFifthsOctavesCount, directFifthsOctavesLocations, voiceCounter, genericOverlap, genericCrossings, bachDirect, genericAntiparallels
	global overlapLocations, crossingsCount, crossingsLocationsm, parallelsCount, antiParallelsCount, antiParallelsLocations, overlapVoices, genericDirect, tritoneLocs, genericParallels
	piece_notes = []
	offset_list = []
	interval_list = []
	fix_humdrum_file()
	#measures = piece.parts[0].getElementsByClass('Measure').stream()
	for i in range(0, len(piece.parts)):
		piece_notes.append(piece.parts[i].flat.getElementsByClass(['Note', 'Rest']).stream())
		interval_list.append(make_intlist(piece_notes[i]))
		for myNote in piece_notes[i]:
			offset_list.append(myNote.offset)
	for i in range(len(interval_list)):
		for j in range(len(interval_list[i]) - 1):
			myInt = interval_list[i][j]
			nextInt = interval_list[i][j + 1]
			if not myInt or not nextInt:
				continue
			#print "HERE", myInt
			#m = DT.get_measure_number_and_beat_from_offset(piece, myInt[2], printErrors = True)[0]
			try:
				m = piece.beatAndMeasureFromOffset(myInt[2])[1].number
			except:
				continue
			#m = measures.getElementAtOrBefore(myInt[2]).number
			if myInt[1].count("A") > 0 and myInt.count("A1") == 0:
				print_comment("  AUGMENTED INTERVAL! MEASURE " + str(m)+ " VOICE " + str(i) + ' ' + str(myInt) + ' ' + str(nextInt))
			elif myInt[0] in [10, 11] or myInt[0] > 12:
				print_comment("  LARGE INTERVAL MEASURE ", m, "VOICE ", i, myInt, nextInt)
			elif myInt[0] > 7 and (nextInt[0] != 0) and (myInt[0] / nextInt[0]) > 0:
				print_comment("  LARGE INTERVAL WITHOUT REVERSAL MEASURE ", m, "VOICE ", i, myInt, nextInt)
			elif myInt[1].count("d") > 0 and myInt[1].count("d1") == 0 and (nextInt[0] != 0) and (myInt[0] / nextInt[0]) > 0:
				print_comment("  DIMINISHED INTERVAL WITHOUT REVERSAL MEASURE ", m, "VOICE ", i, myInt, nextInt)
			elif abs(myInt[0]) > 2 and abs(nextInt[0]) > 2 and (myInt[0] / nextInt[0]) > 0:
				intPattern = sorted([abs(myInt[0]), abs(nextInt[0])])
				if intPattern in [[3, 5], [3, 4], [4, 5], [3, 3], [5, 7], [3, 6]]: continue
				print_comment("  CONSECUTIVE LEAPS ", m, "VOICE ", i, myInt, nextInt)
				
	offsets = sorted(list(set(offset_list)))
	for i in range(0, len(offsets) - 1):
		firstOffset = offsets[i]
		secondOffset = offsets[i+1]
		p1 = []
		p2 = []
		
		for j in range(0, len(piece_notes)):
			testNote = piece_notes[j].getElementsByOffset(firstOffset + .01, firstOffset + .02, includeEndBoundary = False, mustBeginInSpan = False, classList = ['Note'])
			if testNote:
				p1.append(testNote[0])
			else:
				p1.append(note.Note('C0'))
		for j in range(0, len(piece_notes)):
			testNote = piece_notes[j].getElementsByOffset(secondOffset + .01, secondOffset + .02, includeEndBoundary = False, mustBeginInSpan = False, classList = ['Note'])
			if testNote:
				p2.append(testNote[0])
			else:
				p2.append(note.Note('C0'))
		if p1[0].expressions:
			continue
		
		firstChord = [x.pitch.midi for x in p1]
		secondChord = [x.pitch.midi for x in p2]
		if max(firstChord) == 12 or max(secondChord) == 12:
			continue
			
		topNote1 = max(firstChord)
		topNote2 = max(secondChord)
		bottomNote1 = min([y for y in firstChord if y != 12])
		bottomNote2 = min([y for y in secondChord if y != 12])
		numVoices = get_number_of_voices(p1, p2)
		if is_consonance(p1):
			totalConsonances[numVoices] = totalConsonances.get(numVoices, 0) + 1
		totalChords[numVoices] = totalChords.get(numVoices, 0) + 1
		for j in range(0, len(piece_notes) - 1):					# j = first voice
			if p1[j].pitch.midi == 12 or p2[j].pitch.midi == 12: continue
			for k in range(j + 1, len(piece_notes)):				# k = second voice
				if p1[k].pitch.midi == 12 or p2[k].pitch.midi == 12: continue
				voiceCounter[numVoices] = voiceCounter.setdefault(numVoices, 0) + 1
				if (p1[k].pitch.midi < p1[j].pitch.midi and p2[k].pitch.midi > p1[j].pitch.midi and p2[k].pitch.midi <= p2[j].pitch.midi) or (
					p1[j].pitch.midi < p1[k].pitch.midi and p2[j].pitch.midi > p1[k].pitch.midi and p2[j].pitch.midi <= p2[k].pitch.midi):
					m = DT.get_measure_number_and_beat_from_offset(piece, firstOffset)[0]
					overlapStr = str(j)+str(k)
					overlapVoices[overlapStr] = overlapVoices.get(overlapStr, 0) + 1
					overlapCount += 1
					record(numVoices, genericOverlap, [pieceNumber, m])
					print("Voice Overlap! Piece", pieceNumber, "Measure", m, 'Notes (', p1[j].pitch, ',', p1[k].pitch, ') -> (', p2[j].pitch,',', p2[k].pitch,')')
				crossing = (p1[k].pitch.midi < p1[j].pitch.midi and p2[k].pitch.midi > p2[j].pitch.midi) or (p1[k].pitch.midi > p1[j].pitch.midi and p2[k].pitch.midi < p2[j].pitch.midi)
				if crossing:
					m = DT.get_measure_number_and_beat_from_offset(piece, firstOffset)[0]
					record(numVoices, genericCrossings, [pieceNumber, m])
					crossingsCount += 1
					#print "Crossing! Piece", n, "Measure", m, 'Notes (', p1[j].pitch, ',', p1[k].pitch, ') -> (', p2[j].pitch,',', p2[k].pitch,')'
				firstInt = abs(p1[j].pitch.midi - p1[k].pitch.midi) % 12
				secondInt = abs(p2[j].pitch.midi - p2[k].pitch.midi) % 12
				kDirection = (p2[k].pitch.midi - p1[k].pitch.midi)
				if kDirection != 0: 
					kDirection = int(kDirection/abs(kDirection))
				jDirection = (p2[j].pitch.midi - p1[j].pitch.midi)
				if jDirection != 0:
					jDirection = int(jDirection/abs(jDirection))
				if jDirection != 0:
					if (secondInt == 0 or secondInt == 7) and (jDirection == kDirection):
						m = DT.get_measure_number_and_beat_from_offset(piece, firstOffset)
						if not m: continue
						m = m[0]
						record(numVoices, genericDirect, [pieceNumber, m])
						if ((p1[j].pitch.midi == topNote1 and p2[j].pitch.midi == topNote2 and abs(p2[j].pitch.midi - p1[j].pitch.midi) > 2) or (
							p1[k].pitch.midi == topNote1 and p2[k].pitch.midi == topNote2 and abs(p2[k].pitch.midi - p1[k].pitch.midi) > 2)) and (
							(p1[k].pitch.midi == bottomNote1 and p2[k].pitch.midi == bottomNote2) or (p1[j].pitch.midi == bottomNote1 and p2[j].pitch.midi == bottomNote2)):
							directFifthsOctavesCount += 1
							record(numVoices, bachDirect, [pieceNumber, m])
							print("Direct Fifths/Octaves! Piece", pieceNumber, "Measure", m, 'Notes (', p1[j].pitch, ',', p1[k].pitch, ') -> (', p2[j].pitch,',', p2[k].pitch,')')
					if ((firstInt == secondInt == 0) or (firstInt == secondInt == 7)) and (not crossing) and kDirection != 0:
						m = DT.get_measure_number_and_beat_from_offset(piece, firstOffset)[0]
						if (p1[j].pitch.midi - p1[k].pitch.midi) != (p2[j].pitch.midi - p2[k].pitch.midi): 
							antiParallelsCount += 1
							record(numVoices, genericAntiparallels, [pieceNumber, m])
							if firstInt == 7:
								print("Antiparallel fifths! Piece", pieceNumber, "Measure", m, 'Notes (', p1[j].pitch, ',', p1[k].pitch, ') -> (', p2[j].pitch,',', p2[k].pitch,')')
							else:
								print("Antiparallel octaves! Piece", pieceNumber, "Measure", m, 'Notes (', p1[j].pitch, ',', p1[k].pitch, ') -> (', p2[j].pitch,',', p2[k].pitch,')')
						else: 
							parallelsCount += 1
							record(numVoices, genericParallels, [pieceNumber, m])
							print("True parallels! Piece", pieceNumber, "Measure", m, 'Notes (', p1[j].pitch, ',', p1[k].pitch, ') -> (', p2[j].pitch,',', p2[k].pitch,')')
					if ((firstInt == 6 and secondInt == 7)) and (jDirection == kDirection):
						m = DT.get_measure_number_and_beat_from_offset(piece, firstOffset)[0]
						record(numVoices, tritoneLocs, [pieceNumber, m])
						print("Tritone to perfect fifth! Piece", pieceNumber, "Measure", m, 'Notes (', p1[j].pitch, ',', p1[k].pitch, ') -> (', p2[j].pitch,',', p2[k].pitch,')')
	return

def get_number_of_voices(p1, p2):
	voiceCount1 = [i for i in range(len(p1)) if p1[i].pitch.midi != 12]
	voiceCount2 = [i for i in range(len(p2)) if p2[i].pitch.midi != 12]
	if voiceCount1 == voiceCount2:
		return len(voiceCount1)
	else:
		return tuple((len(voiceCount1), len(voiceCount2)))

def remove(myDict, thingsToRemove):
	for numVoices, removeThing in thingsToRemove:
		if removeThing in myDict[numVoices]:
			myDict[numVoices].remove(removeThing)

def record(numVoices, myDict, newList):
	myDict.setdefault(numVoices, []).append(newList)

def return_intervals(n1, n2):
	if 'Rest' in n1.classes or 'Rest' in n2.classes or n1.expressions:
		return False
	else:
		midi = n2.pitch.midi - n1.pitch.midi
		if midi < 0: 
			sign = '-'
		else:
			sign = ''
		return [midi, sign + interval.Interval(n1, n2).name, n1.offset]

def make_intlist(myList):
	out = []
	for i in range(len(myList) - 1):
		out.append(return_intervals(myList[i], myList[i+1]))
	return out
	
def is_consonance(myList):
	sortedNotes = sorted([x.pitch.midi for x in myList if x.pitch.midi != 12])
	figuredBass = sorted(list(set([(x - sortedNotes[0]) % 12 for x in sortedNotes])))
	if figuredBass in [[0, 3], [0, 4], [0, 7], [0, 8], [0, 9], [0, 3, 7], [0, 4, 7], [0, 3, 8], [0, 3, 9], [0, 4, 9]]:
		return True
	else:
		return False
	
def print_comment(*args):
	myStr = ' '.join([str(x) for x in args])
	melodyComments.setdefault(myFile, []).append(myStr)
	print(myStr)
	
def show(locList, RNs = False):
	global output, locNames, globalSource
	output = stream.Score()
	tempOutput = stream.Score()
	lastPieceNo = False
	if not locList:
		print('Not found')
		return
	for i in range(0, 4):
		tempOutput.append(stream.Part())
	for instance in locList:
		measureList = instance[1:]
		if len(measureList) == 1:
			measureList = [measureList[0], measureList[0] + 1]
		else:
			measureList[-1] = measureList[-1] + 1
		header = str(locNames[instance[0]])+'m'+str(measureList[0])
		print('getting', locNames[instance[0]], measureList)
		if type(instance[0]) is list:
			pieceNumber = instance[0][0]
			subpieceNumber = instance[0][1]
		else:
			pieceNumber = instance[0]
			subpieceNumber = False
		if locNames[pieceNumber] != lastPieceNo:
			pieceNo = locNames[instance[0]]
			lastPieceNo = pieceNo
			piece = converter.parse(DT._PATHS[globalSource][1] + pieceNo)
			if 'Opus' in piece.classes: 
				if subpieceNumber is not None:
					piece = piece.scores[subpieceNumber]
				else:
					continue
		tempPiece = copy.deepcopy(DT.get_measures(piece, measureList[0], measureList[1]))
		DT.insert_text(tempPiece.parts[0][0], header, 55, 0)
		output = DT.append(tempPiece, output)
	for p in output.parts:
		for item in p.flat:
			if 'Note' in item.classes or 'Chord' in item.classes: item.lyrics = []
	output.show()
	return output

def load(source = 'PALRAW'):
	global genericOverlap, genericCrossings, genericAntiparallels, genericParallels, genericDirect, bachDirect, tritoneLocs, voiceCounter
	global directFifthsOctavesLocations, overlapLocations, parallelsLocations, antiParallelsLocations, locNames, melodyComments, globalSource
	global totalChords, totalConsonances
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource + '.pkl', 'rb') as inputFile:
			genericDirect = pickle.load(inputFile)
			bachDirect = pickle.load(inputFile)
			genericOverlap = pickle.load(inputFile)
			genericCrossings = pickle.load(inputFile)
			genericParallels = pickle.load(inputFile)
			genericAntiparallels = pickle.load(inputFile)
			tritoneLocs = pickle.load(inputFile)
			voiceCounter = pickle.load(inputFile)
			locNames = pickle.load(inputFile)
			melodyComments = pickle.load(inputFile)
			totalChords = pickle.load(inputFile)
			totalConsonances = pickle.load(inputFile)
		return True
	except:
		return False

def choose_2(n):
	return int(n*(n-1)/2)

def print_out(myDict, voiceCount = True):
	global voiceCounter
	theKeys = sorted([x for x in myDict.keys() if type(x) is int])
	if not(myDict.values()):
		return
	print("  per piece %.2f" % (1.0*sum([len(x) for x in myDict.values()])/len(locNames)) ,'raw count', sum([len(x) for x in myDict.values()]), "  chords per event %.2f" % (sum(totalChords.values())/sum([len(x) for x in myDict.values()])))
	for voiceNumber in theKeys:
		myChords = voiceCounter[voiceNumber]/choose_2(voiceNumber)		# per chord values"""
		totalAdjacencies = voiceCounter[voiceNumber]/(voiceNumber - 1)
		totalTransitions = voiceCounter[voiceNumber]
		perChordNumber = 1.0* len(myDict[voiceNumber])/myChords
		print("  voices", voiceNumber, "per adjacent pair %.2f" % (100.*len(myDict[voiceNumber])/totalAdjacencies), 'per transition %.2f' % (100.*len(myDict[voiceNumber])/totalTransitions), 'per chord %.2f' % (100.*perChordNumber),'per piece %.2f' % (perChordNumber*totalChords[voiceNumber]/len(locNames)) ,'raw count', len(myDict[voiceNumber]))
		
def report(source = "", **kwargs):
	global genericOverlap, genericCrossings, genericAntiparallels, genericParallels, genericDirect, bachDirect, tritoneLocs, voiceCounter
	if source: load(source)
	chordsPerPiece = round(1.*sum(totalChords.values())/len(locNames), 1)
	chordsPerEvent = round(1.*sum(totalChords.values())/len(locNames), 1)
	print("TOTALPIECES", len(locNames), "CONSONANCES PER PIECE",round(1.*sum(totalConsonances.values())/len(locNames), 1), "CHORDS PER PIECE", chordsPerPiece)
	theKeys = sorted([x for x in genericCrossings.keys() if type(x) is int])
	for i in theKeys:
		print("voices", i, "total chords", totalChords[i], "total consonances", totalConsonances.get(i, 0))
	for i, myDict in enumerate([genericOverlap, genericCrossings, genericAntiparallels, genericParallels, genericDirect, bachDirect, tritoneLocs]):
		print(["Overlap", "Crossings", "Antiparallels", "Parallels", "Direct fifths and octaves", "Bach-forbidden direct fifths and octaves", "Tritone to perfect fifths"][i])
		print_out(myDict, **kwargs)

def show_parallels(i = 0, j = None, voices = 4):
	if j is None:
		j = len(genericParallels[voices])
	locList = genericParallels[voices][i:j]
	show(locList)

def consecutive_leaps():
	global leapCount
	leapCount = {}
	for piece, comments in melodyComments.items():
		for c in comments:
			if c.count("CONSECUTIVE LEAPS") > 0:
				cSplit = c.split("'")
				ints = cSplit[1] + ' ' + cSplit[3]
				leapCount[ints] = leapCount.get(ints, 0) + 1
				if c.count("P4") > 1:
					print(piece, c)
	DT.print_dict(leapCount, pct = True)

def showpiece(subpieceno = 0):
	if bigOpus:
		bigOpus.scores[subpieceno].show()
	else:
		piece.show()
		

def fix_humdrum_file():
	theMeasures = [p.getElementsByClass('Measure') for p in piece.parts]
	measureLengths = [len(x) for x in theMeasures]
	maxLength = max(measureLengths)
	longestPart = measureLengths.index(maxLength)
	for i, p in enumerate(piece.parts):
		if i == longestPart: continue
		for j in range(measureLengths[i], maxLength):
			m = theMeasures[longestPart][j]
			mNumber = m.number
			ts = m.getElementsByClass("TimeSignature")
			newTS = False
			newM = stream.Measure()
			newM.number = mNumber
			if ts:
				newTS = meter.TimeSignature(ts[0].ratioString)
				newM.insert(ts[0].offset, newTS)
			newM.insert(note.Rest(quarterLength = m.highestTime))	
			p.insert(m.offset, newM)				
