from __future__ import print_function

import os
import operator
import copy
import time
import pickle
																	
from music21 import *
from collections import OrderedDict

import DT			

_THISFILE = 'PEDALS'

"""
Look for pedal tones, particularly those that end as nonharmonic notes.

"""

def gather_stats(source = 'Mozart'):
	global counts, rawPedals, analyzed_file, myFile
	globalSource = source.upper()
	if globalSource not in DT._PATHS:
		print ("SOURCE NOT FOUND!")
		return
	analysisPath = DT._PATHS[globalSource][0]
	scorePath = DT._PATHS[globalSource][1]
	files = sorted(os.listdir(analysisPath))
	startT = time.time()
	counts = {}
	locNames = {}
	counter = 1
	for myFile in files:
		if myFile[-3:] == 'txt':
			rawPedals = []
			with open(analysisPath + myFile) as rawFile:
				for line in rawFile:
					line = line.upper()
					lineSplit = line.split()
					if len(lineSplit) > 2 and lineSplit[0].startswith('PED'):
						rawPedals.append(lineSplit)
			if rawPedals:
				analyzed_file = converter.parse(analysisPath + myFile, format='romantext')
				dochorale(counter)
			locNames[counter] = myFile[:-4]
			counter += 1
	minutes = (time.time() - startT)/60.
	print ('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	return
	
def parse_pedals():
	global parsedPedals, rawPedals
	parsedPedals = []
	for lSplit in rawPedals:
		if lSplit[0] != 'PEDAL:' or len(lSplit) < 3 or lSplit[2][0] != 'M':
			print("  FORMAT ERROR?", lSplit)
			continue
		if len(lSplit[1]) > 1 and lSplit[1][-1] == 'B':
			lSplit[1] = lSplit[1][:-1] + '-'
		currentPedal = [lSplit[1]]
		newList = lSplit[2:]
		pedal1, index = get_pedal_location(newList)
		currentPedal.append(pedal1)
		pedal2, junk = get_pedal_location(newList[index:])
		currentPedal.append(pedal2)
		parsedPedals.append(currentPedal)

def get_pedal_location(lSplit):
	firstM = int(lSplit[0][1:])
	if len(lSplit) > 1 and lSplit[1][0] == 'B':
		firstB = lSplit[1][1:]
		if firstB.count('.') == 0:
			firstB = int(firstB)
		else:
			firstB = float(firstB)
		return [firstM, firstB], 2
	else:
		return [firstM, None], 1
	
def dochorale(pieceNumber):
	parse_pedals()
	theNumerals = analyzed_file.flat.getElementsByClass("RomanNumeral").stream()
	for thePedal in parsedPedals:
		noteName, startPoint, endPoint = thePedal
		thePitch = pitch.Pitch(noteName)
		firstOffset = get_offset(startPoint)
		if not firstOffset:
			print('  ', myFile, thePedal)
			continue
		firstOffset = sum(get_offset(startPoint)[:-1])
		secondOffset = get_offset(endPoint)
		if not secondOffset:
			print('  ', myFile, thePedal)
			continue
		if endPoint[1] is None:
			secondOffset = secondOffset[0] + secondOffset[-1] - .001
		else:
			secondOffset = sum(secondOffset[:-1])
		firstRN = theNumerals.getElementsByOffset(firstOffset, mustBeginInSpan = False)
		if not firstRN:
			print("CAN'T GET FIRST RN")
			continue
		firstRN = firstRN[0]
		pedalDegree = firstRN.key.getScaleDegreeFromPitch(thePitch)
		secondRN = theNumerals.getElementsByOffset(secondOffset, mustBeginInSpan = False)
		if not secondRN:
			print("CAN'T GET SECOND RN")
			continue
		secondRN = secondRN[0]
		inChord = thePitch.pitchClass in secondRN.pitchClasses
		print(myFile, "(" + format_time(startPoint) + "-"+ format_time(endPoint)+")", noteName, "Degree:", pedalDegree, "Ends as Harmonic:", inChord)

def format_time(l):
	if l[1] == None:
		return 'm' + str(l[0])
	return 'm' + str(l[0]) + 'b' + str(l[1])
		
def get_offset(startPoint):
	measureNumber, beatNumber = startPoint
	myMeas = analyzed_file.parts[0].measure(measureNumber)
	if not myMeas:
		print("CAN'T FIND MEASURE", startPoint)
		return False
	ts = myMeas.getContextByClass("TimeSignature")
	if not ts:
		myMeas = analyzed_file.parts[0].measure(measureNumber + 1)
		if myMeas:
			ts = myMeas.getContextByClass("TimeSignature")
			if not ts:
				print('CANNOT FIND TIME SIGNATURE', myFile, startPoint)
				return False
		else:
			print('CANNOT FIND TIME SIGNATURE', myFile, startPoint)
			return False
	beatDur = ts.beatDuration.quarterLength
	barDur = ts.barDuration.quarterLength
	if beatNumber is not None:
		return [myMeas.offset, (beatNumber - 1) * beatDur, barDur]
	else: 
		return [myMeas.offset, 0, barDur]
	