from __future__ import print_function
import os
import operator
import pickle
import copy
import time
import re
from music21 import *
import DT
import matplotlib.pyplot as plt

_THISFILE = 'PHRASE_AND_MELODY'
lastChorale = 1
chorale = stream.Score()
analysis = stream.Score()
progs = [{}, {}]
progs2 = [{}, {}]
rawFiles = [{}, {}, {}]	
histoDict = {}
globalSource = ''

internalSizeVariable = 2

def load(source = 'BACH'):
	global internalSizeVariable, counts, progs, progs2, rawFiles, histoDict, globalSource
	globalSource = source.upper()
	histoDict = {}
	try:
		with open(DT._DATAPATH + _THISFILE + source + '.pkl', 'rb') as storedFile:
			progs = pickle.load(storedFile)
			progs2 = pickle.load(storedFile)
			rawFiles = pickle.load(storedFile)
		return True
	except:
		return False
	

"""

(1)Strip down the chorales to basic melody + bass patterns for use in machine learning.
	currently there are three versions of this, contained in the "rawFiles" document:
		rawFiles[0] -- RN + melody degree
		rawFiles[1] -- bass degree melody degree
		rawFiles[2] -- rhythmicized version of the same (with barlines, quantized to the eighth note)
		
(2)Performs basic statistics on these melody + bass patterns

(3)Also can graph chord frequencies by rhythmic position
		
Its output is used by the vlgen automatic-harmonizer program

TODO: identify cadences in other repertoires -- perhaps link to the "cadences" module if that is possible?

"""

def gather_stats(source = 'BACH', fileName = '', bassDegrees = True):
	global counts, progs, progs2, locations, locNames, chorale, analysis, internalSizeVariable, RNprogs, rawFiles, globalSource
	globalSource = source.upper()
	rawFiles = [{}, {}, {}]										# RNs + melody degrees, melody + bass degrees, rhythmicized RNs + melody degrees
	progs = [{}, {}]											# RNs + melody degrees
	progs2 = [{}, {}]											# melody + bass degrees
	RNprogs = [{}, {}]
	locations = [{}, {}]
	locNames = {}
	startT = time.time()
	if globalSource in DT._PATHS:
		analysisPath = DT._PATHS[globalSource][0]
		scorePath = DT._PATHS[globalSource][1]
		fileStr = globalSource
		files = os.listdir(analysisPath)
		recordResults = True
		counter = 1
		for myFile in files:
			if myFile[-3:] == 'txt':
				if fileName and fileName != myFile[:-4]:
					continue
				#print 'Analyzing', myFile
				locNames[counter] = myFile[:-4]
				foundIt = False
				for suffix in ['.xml', '.krn', '.mxl']:
					try:
						chorale = converter.parse(scorePath + myFile.replace('.txt', suffix))
						foundIt = True
						break
					except:
						pass
				if not foundIt: 
					print("CAN'T FIND SCORE!!!!")
					continue
				analysis = converter.parse(analysisPath + myFile, format='romantext')
				if counter > 1: print()
				print("Piece ", counter)
				dochorale(counter, bassDegrees)
				counter += 1
		if not fileName:
			outputFile = open(DT._DATAPATH + _THISFILE + fileStr + '.pkl', 'wb')
			pickle.dump(progs, outputFile)
			pickle.dump(progs2, outputFile)
			pickle.dump(rawFiles, outputFile)
			outputFile.close()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	return
	
def get_top_note(offset):
	global flatScore
	testNote = flatScore.getElementsByOffset(offset + .01, offset + .02, includeEndBoundary = False, mustBeginInSpan = False)
	if not testNote: return False
	return max(testNote, key=lambda n: n.pitch.midi)
	
def replace_chord_with_top_note(myStream):
	insertList = []
	for i in reversed(range(0, len(myStream))):										# convert chordal tones to single notes
		if 'Chord' in myStream[i].classes:
			myOffset = myStream[i].offset
			myNote = note.Note(max(myStream[i].pitches).nameWithOctave)
			myNote.duration.quarterLength = myStream[i].duration.quarterLength
			myStream.remove(myStream[i])
			insertList.append(myOffset)
			insertList.append(myNote)
	myStream.insert(insertList)
	return myStream

def dochorale(n, bassDegrees = True):
	global chorale, analysis, internalSizeVariable, counts, progs, progs2, flatScore, startOffset, endOffset, ourKey, lastMeasure, RNprogs, rawFiles, rawRhythms
	flatScore = chorale.flat.getElementsByClass(['Note', 'Chord'])
	rawFiles[0][n] = []
	rawFiles[1][n] = []
	rawFiles[2][n] = []
	expressionOffsets = []
	topNotes = {}
	topSDs = {}
	for item in flatScore:
		if item.expressions:
			expressionOffsets.append(item.offset)
	flatScore = replace_chord_with_top_note(flatScore)
	noteOffsets = sorted(list(set([x.offset for x in flatScore])))
	anal = analysis.flat.getElementsByClass('RomanNumeral')
	lastSym = ''
	getNewKey = True
	lastOffset = noteOffsets[0]
	suspensionFlag = False
	suspensionOffset = 0
	for myOffset in noteOffsets:
		topNotes[myOffset] = get_top_note(myOffset)
		RN = anal.getElementAtOrBefore(myOffset + .01)
		pitchNames = [x.name for x in RN.pitches]
		if getNewKey:
			getNewKey = False
			lastOffset = myOffset
			homeKey = [RN.key.tonic, RN.key.mode]
			intervalToC = interval.Interval(homeKey[0], pitch.Pitch('C'))					# check bigStr to make this work right!
			if homeKey[1] == 'major': 
				bigStr = 'M '
			else: 
				bigStr = 'm '
			bigStr2 = bigStr
			bigStr3 = bigStr
		if RN:
			keyStr = ''
			tempPitchAndMode = [RN.key.tonic, RN.key.mode]
			if tempPitchAndMode != homeKey:
				homeKey = tempPitchAndMode
				newKey = RN.key.transpose(intervalToC)
				keyStr = convert_keystring(newKey) + ' '
			if topNotes[myOffset].name in pitchNames:
				topSDs[myOffset] = accidental_string(RN.key.getScaleDegreeAndAccidentalFromPitch(topNotes[myOffset].pitch))
				mySym = accidental_string(RN.key.getScaleDegreeAndAccidentalFromPitch(RN.bass())) + '-' + topSDs[myOffset] + ' '
				mySym2 = RN.figure + '-' + topSDs[myOffset] + ' '
				if not suspensionFlag:
					numberOfDots = int(((myOffset - lastOffset) * 2) + .5) - 1
					lastOffset = myOffset
				else:
					numberOfDots = int(((myOffset - suspensionOffset) * 2) + .5) - 1
					lastOffset = suspensionOffset
					suspensionFlag = False
				bigStr3 += '. ' * numberOfDots
				measureBeat = DT.get_measure_number_and_beat_from_offset(chorale, myOffset)
				if measureBeat[1] == 1.0: bigStr3 += '| '
				if mySym2 != lastSym:
					bigStr += keyStr + mySym
					bigStr2 += keyStr + mySym2
					bigStr3 += keyStr + mySym2
					lastSym = mySym2
					keyStr = ''
				else:
					bigStr3 += '. '
			else:
				if RN.offset == myOffset:
					numberOfDots = int(((myOffset - lastOffset) * 2) + .5) - 1
					bigStr3 += '. ' * numberOfDots
					measureBeat = DT.get_measure_number_and_beat_from_offset(chorale, myOffset)
					if measureBeat[1] == 1.0: bigStr3 += '| '
					suspensionFlag = True
					suspensionOffset = myOffset
		if myOffset in expressionOffsets:
			if bigStr.count(' ') > 2:
				rawFiles[0][n].append(bigStr2)
				rawFiles[1][n].append(bigStr)
				rawFiles[2][n].append(bigStr3)
				print(bigStr3)
				parse(bigStr, progs)
				parse(bigStr2, progs2)
			getNewKey = True
			lastSym = ''
	if bigStr.count(' ') > 2:
		rawFiles[0][n].append(bigStr2)
		rawFiles[1][n].append(bigStr)
		rawFiles[2][n].append(bigStr3)
		print(bigStr3)
		parse(bigStr, progs)
		parse(bigStr2, progs2)
	getNewKey = True
	lastSym = ''

def parse(myStr, myDict = progs):
	modeInd = int(myStr[0] == 'm')
	chords = myStr[2:].split()
	for i in range(0, len(chords)):
		c = chords[i]
		myDict[modeInd][c] = myDict[modeInd].setdefault(c, 0) + 1
		if c.count(':') > 0:
			modeInd = int(c[0].islower())
			continue
		for j in range(min(i + 1, len(chords)), min(i + 6, len(chords))):
			c += " -> " + chords[j]
			if c.count(":") > 0: break
			myDict[modeInd][c] = myDict[modeInd].setdefault(c, 0) + 1
				
def convert_keystring(x):
	outStr = x.tonic.name.replace('-', 'b') + ':'
	if x.mode == 'minor': outStr = outStr.lower()
	return outStr

def starts(firstChord):
	global progs, results
	if firstChord.count('v') + firstChord.count('V') + firstChord.count('i') + firstChord.count('I') > 0: 
		myProgs = working2
	else:
		myProgs = working
	results = []										
	for prog in myProgs:
		if prog[0:len(firstChord)] == firstChord:
			results.append([prog, myProgs[prog]])
	results = sorted(results, key = lambda x: -x[1])
	for item in results:
		print(item[0], item[1])
	return

def ends(firstChord):
	global progs, results
	if firstChord.count('v') + firstChord.count('V') + firstChord.count('i') + firstChord.count('I') > 0: 
		myProgs = working2
	else:
		myProgs = working
	results = []										
	for prog in myProgs:
		if prog[-len(firstChord):] == firstChord:
			results.append([prog, myProgs[prog]])
	results = sorted(results, key = lambda x: -x[1])
	for item in results:
		print(item[0], item[1])
	return

def accidental_string(s):
  if s[1] is None:
     degreeString = str(s[0])
  else:
     degreeString = str(s[1].modifier) + str(s[0])
  return degreeString.replace('-', 'b')

def get_size(size, mode = 'major'):
	global progs, progs2, working, working2
	modeInd = int(mode[:2].upper() == 'MI')
	size = size - 1	
	myProgs = progs[modeInd]
	working = {}
	working2 = {}							
	for prog in myProgs:
		if prog.count(">") == size:
			working[prog] = myProgs[prog]
	myProgs = progs2[modeInd]
	for prog in myProgs:
		if prog.count(">") == size:
			working2[prog] = myProgs[prog]
	#for item in sorted(working.keys(), key = lambda x: -working[x]):
	#	print item, working[item]
	return

def voice(degStr, voice = 1, useRNs = False, mode = 'major', totalOnly = False):
	global progs, working, working2, results
	results = {}
	modeInd = int(mode[:2].upper() == 'MI')
	if useRNs:
		voice = 1
		myDict = working2
	else:
		myDict = working
	mySize = degStr.count('>')
	get_size(mySize + 1)
	degStr = degStr.replace(' ->', '')	# degrees in the form A B C
	for item in myDict:
		newItem = ''
		for thing in item.split(' -> '):
			newItem += thing.split('-')[voice] + ' '
		newItem = newItem.rstrip(' ')
		if newItem == degStr:
			results[item] = myDict[item]
	if not totalOnly:
		for item in sorted(results.keys(), key = lambda x: -results[x]):
			print(item, results[item])
	print("TOTAL", sum(results.values()))

def get_mode(mode = 'major'):
	if type(mode) == str:
		if mode == 'minor' or mode == 'm' or mode == 'min': 
			return 1
		else: 
			return 0
	elif type(mode) == int:
		return mode

def survey(myInt, totalOnly = True, useRNs = False, mode = 'major'):
	for i in range(1, 8):
		myStr = str(i) + ' -> ' + str(mod7(i + myInt))
		print(myStr)
		voice(myStr, 1, useRNs, mode, totalOnly)

def dyads():
	global results
	results = {}
	modeInd = 0
	for i in range(1, 8):
		for j in range(1, 8):
			for k in range(1, 8):
				for l in range(1, 8):
					myStr1 = str(i)+'-'+str(j)
					myStr2 = str(k)+'-'+str(l)
					if myStr1 not in progs[modeInd] or myStr2 not in progs[modeInd]: continue
					myStr = myStr1 +' -> ' + myStr2
					if myStr in progs[modeInd]:
						results[myStr] = progs[modeInd][myStr]
					else:
						pass
						#results[myStr] = 0
	for s in sorted(results.keys(), key = lambda x: -results[x]):
		print(s, results[s])
		
def expectation(progLen = 2, modeInd = 0, orderParam = 0):												#compares simulated probabiities to actual
	global results
	results = {}
	rawString = []
	for i in range(progLen):
		rawString = addstring2(rawString, modeInd)
	get_size(orderParam + 1)
	bigLength = sum(working.values())											# how many of the first-order lengths are there?
	factor = (bigLength)**(progLen - orderParam - 1)
	print(bigLength, factor)
	#get_size(orderParam + 1)
	#print sum(working.values()), (progLen - orderParam - 1), (sum(working.values()))**(progLen - orderParam - 1)
	for s in rawString:
		expected = calcstring(s, modeInd, orderParam) / factor
		if s in progs[modeInd]:
			actual = progs[modeInd][s]
		else:
			actual = 0
		if abs(actual - expected) > 2 and norepeats(s):
			results[s] = actual - expected
	resultsKeys = sorted(results.keys(), key = lambda x: -results[x])
	resultsLen = min(len(resultsKeys) / 3, 10)
	for s in resultsKeys[:resultsLen] + resultsKeys[-resultsLen:]:
		print(s, results[s])

def norepeats(inStr):
	s = inStr.split(' -> ')
	lastSym = s[0]
	for item in s[1:]:
		if item == lastSym:
			return False
		lastSym = item
	return True

def calcstring(inStr, modeInd = 0, orderParam = 0):
	s = inStr.split(' -> ')
	result = 1
	for i in range(0, len(s) - orderParam):
		t = ' -> '.join(s[i:i+1+orderParam])
		if t not in progs[modeInd]:
			return 0
		#print t, progs[modeInd][t]
		result *= progs[modeInd][t]
	return result

def addstring(inList = [], modeInd = 0):
	extras = []
	if modeInd: extras = ['#6', "#7"]
	if not inList:
		return [str(x) for x in range(1, 8) + extras]
	else:
		return [x + ' -> ' + str(y) for x in inList for y in range(1, 8) + extras]

def addstring2(inList = [], modeInd = 0):
	extras = []
	if modeInd: extras = ['#6', "#7"]
	if not inList:
		return [str(x) + '-' + str(y) for x in range(1, 8) + extras for y in range(1, 8) + extras]
	else:
		return [x + ' -> ' + str(y) + '-' + str(z) for x in inList for y in range(1, 8)+ extras for z in range(1, 8) + extras]

def mod7(i):
	return ((i - 1) % 7) + 1

def compare(c1 = 'Monteverdi', c2 = 'Bach', size = 2):
	global results
	results = {}
	load(c1)
	get_size(size)
	oldworking = copy.deepcopy(working)
	oldtotal = sum(oldworking.values())/100.
	load(c2)
	get_size(size)
	total = sum(working.values())/100.
	print("MEASURED IN TENTHS OF A PERCENT, SINCE THERE ARE SO MANY POSSIBLE DYADIC PROGRESSIONS")
	for item in oldworking:
		if item in working:
			results[item] = working[item]/total - oldworking[item]/oldtotal
		else:
			results[item] = -oldworking[item]/oldtotal
	for item in working:
		if item not in oldworking:
			results[item] = working[item]/oldtotal
	resultsKeys = sorted(results.keys(), key = lambda x: -results[x])
	resultsLen = min(len(resultsKeys) / 3, 10)
	for s in resultsKeys[:resultsLen] + resultsKeys[-resultsLen:]:
		print(s, "%.2f" % results[s])

def make_histo_dict(mode = 'major', avoidModulations = True, includeSoprano = False, removeInversions = False):				# fix the mode problem!
	global histoDict, histoMode, startChords
	histoDict = {}
	startChords = {}
	modeInd = int(mode[:2].upper() == 'MI')
	histoMode = modeInd
	for n in rawFiles[2]:
		for line in rawFiles[2][n]:
			if ['m', 'M'][modeInd] == line[0]: continue
			if avoidModulations and line.count(":") > 0: continue
			line = line[2:]
			newLine = []
			for item in line.split(' '):
				if item.count(":") == 0 and item != '|':
					if removeInversions and item and item != '.':
						newItem = item.split('-')
						newChord = DT.parse_figure(newItem[0])
						if newChord[2]:
							newChord = newChord[0] + '/' + newChord[2]
						else:
							newChord = newChord[0]
						#print newItem
						item = newChord + '-' + newItem[1]
					if includeSoprano:
						newLine.append(item)
					else:
						newLine.append(item.split('-')[0])
						"""for item in item.split('-'):									
							newLine.append(item)"""
				elif item.count(":") > 0:
					if (item[0].islower() and modeInd == 0) or (item[0].isupper() and modeInd == 1):
						break
			for i in reversed(range(len(newLine))):
				if newLine[i] not in ['.', '']:
					break
			newLine = newLine[:i+1]
			total = len(newLine) - 1
			#if total == 0: print newLine, line
			foundStart = False
			for i, item in enumerate(newLine):
				if item in ['.', '']: continue
				if not foundStart:
					startChords[item] = startChords.setdefault(item, 0) + 1
					foundStart = True
				timePoint = int(((100. * i) / max(total, 1)) + .5)									# expressed as % of the phrase
				histoDict.setdefault(item, []).append(timePoint)


def harmonic_cycles(mode = 'major', avoidModulations = True, includeSoprano = False):				# fix the mode problem!
	global histoDict, histoMode, startChords
	histoDict = {}
	startChords = {}
	modeInd = int(mode[:2].upper() == 'MI')
	histoMode = modeInd
	for n in rawFiles[2]:
		for line in rawFiles[2][n]:
			if ['m', 'M'][modeInd] == line[0]: continue
			if avoidModulations and line.count(":") > 0: continue
			line = line[2:]
			newLine = []
			for item in line.split(' '):
				if item.count(":") == 0 and item != '|':
					if includeSoprano:
						newLine.append(item)
					else:
						for item in item.split('-'):									
							newLine.append(item)
				elif item.count(":") > 0:
					if (item[0].islower() and modeInd == 0) or (item[0].isupper() and modeInd == 1):
						break
			for i in reversed(range(len(newLine))):
				if newLine[i] not in ['.', '']:
					break
			newLine = newLine[:i+1]
			total = len(newLine) - 1
			foundStart = False
			for i, item in enumerate(newLine):
				if item in ['.', '']: continue
				if not foundStart:
					startChords[item] = startChords.setdefault(item, 0) + 1
					foundStart = True
				timePoint = int(((100. * i) / total) + .5)									# expressed as % of the phrase
				histoDict.setdefault(item, []).append(timePoint)




def make_markov(mode = 'major', avoidModulations = True):				# fix the mode problem!
	global markovDict, histoMode
	markovDict = {}
	modeInd = int(mode[:2].upper() == 'MI')
	histoMode = modeInd
	for n in rawFiles[0]:
		for line in rawFiles[0][n]:
			if ['m', 'M'][modeInd] == line[0]: continue
			if avoidModulations and line.count(":") > 0: continue
			line = line[2:].split(' ')
			while '' in line:
				line.remove('')
			for i, item in enumerate(line[:-1]):
				if item not in markovDict:
					markovDict[item] = {}
				markovDict[item][line[i+1]] = markovDict[item].setdefault(line[i+1], 0) + 1
	with open(DT._DATAPATH + _THISFILE + 'MARKOV' + globalSource + '.pkl', 'wb') as outputFile:
		pickle.dump(markovDict, outputFile)		

def clean_histo_dict(mode = 'major', avoidModulations = True, ticks = 16):				# fix the mode problem!
	global cleanChordDict
	cleanChordDict = {}
	make_histo_dict(mode, avoidModulations)
	binSize = 100. / ticks
	for c in histoDict: 
		values = [0] * ticks
		for j in histoDict[c]:
			binLoc = int(j/binSize)
			if binLoc >= ticks: 
				binLoc = ticks - 1
			values[binLoc] += 1
		cleanChordDict[c] = values

def save_histo_dict(mode = 'major', avoidModulations = True, includeSoprano = False, ticks = 16):
	global normalizeColl, globalSource
	normalizeColl = {}
	make_histo_dict(mode, avoidModulations, includeSoprano)
	if includeSoprano:
		theName = 'OUTERVOICESBYRHYTHMICPOSITION'
		with open(DT._DATAPATH + _THISFILE + theName + globalSource + '.pkl', 'wb') as outputFile:
			pickle.dump(histoDict, outputFile)
	else:
		theName = 'CHORDSBYRHYTHMICPOSITION'
		with open(DT._DATAPATH + _THISFILE + theName + globalSource + '.pkl', 'wb') as outputFile:
			for mode in ['major', 'minor']:
				clean_histo_dict(mode = mode, avoidModulations = avoidModulations, ticks = ticks)
				pickle.dump(cleanChordDict, outputFile)
				pickle.dump(startChords, outputFile)

Vequiv = ['V + V6 + V6/4 + V7 + V6/5 + V4/3 + V2 + viio + viio6 + viio6/4 + viio7 + viio6/5 + viio4/3 + viio2 + vii/o7 + vii/o6/5 + vii/o4/3 + vii/o2']
IVequiv = ['ii + ii7 + ii6 + ii6/5 + ii4/3 + ii6/4 + ii2 + IV + IVmaj7 + IV6 + IVmaj6/5 + IV6/4 + IVmaj4/3 + IVmaj2']
Iequiv = ['I + I6 + I6/4 + Imaj7 + Imaj6/5 + Imaj4/3 + Imaj2']

def rhythm_histo(chordList = Vequiv, mode = 'major', ticks = 8, norm = False, specialLabels = [], **kwargs):
	if type(chordList) is str:
		chordList = [chordList]
	modeInd = int(mode[:2].upper() == 'MI')
	make_histo_dict(mode, **kwargs)
	plt.xlabel('Percentage through the phrase')
	plt.ylabel('Percentage')
	plt.title('Chord frequency by phrasal position')
	binSize = 100. / ticks
	values = [[]] * len(chordList)	
	for i, c in enumerate(chordList):
		values[i] = [0] * ticks
		chords = c.split(' + ')
		for c in chords:
			if c not in histoDict: continue
			for j in histoDict[c]:
				binLoc = int(j/binSize)
				if binLoc >= ticks: 
					binLoc = ticks - 1
				values[i][binLoc] += 1
	if norm:
		total = sum([sum(x) for x in values])
	if specialLabels:
		chordList = specialLabels
	else:
		chordList = [s.replace('+', 'or') for s in chordList]
	for i, v in enumerate(values):
		if not norm:
			total = sum(v)
		v = [100. * x/max(total, .001) for x in v]
		plt.plot(v, label = chordList[i])
	ticks = range(ticks)
	plt.xticks(ticks, [str(int((binSize * x) + .5)) for x in ticks])
	myLeg = plt.legend(tuple(chordList), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()

def decorative_secondary_dominants(mode = 'major'):			
	global cycleDict
	cycleDict = {}
	cycleStrengths = []
	totalCycles = 0
	modeInd = int(mode[:2].upper() == 'MI')
	for n in rawFiles[2]:
		for line in rawFiles[2][n]:
			if ['m', 'M'][modeInd] == line[0]: continue
			if line.count(":") > 0: continue
			line = line[2:].replace('/o', 'H')
			if re.match('.*/[^0123456789]', line): 
				newLine = line.replace('.', '').replace('|', '').replace('H', '/o')
				while newLine.count('  ') > 0:
					newLine = newLine.replace('  ', ' ')
				newLine = ' '.join([strip_scale_degree(x) for x in newLine.split()])
				print(n, newLine.lstrip())
	#print cycleStrengths

def strip_scale_degree(s):
	if s.count('-'):
		return s[:s.index('-')]
	return s

def harmonic_cycle_length(mode = 'major', avoidModulations = True, phraseMustStartWithTonic = True):			
	global cycleDict, cycleStrengths, totalCycles
	cycleDict = {}
	cycleStrengths = []
	totalCycles = 0
	modeInd = int(mode[:2].upper() == 'MI')
	for n in rawFiles[2]:
		for line in rawFiles[2][n]:
			if ['m', 'M'][modeInd] == line[0]: continue
			if avoidModulations and line.count(":") > 0: continue
			line = line[2:]
			newLine = []
			for item in line.split(' '):
				if item.count(":") == 0 and item != '|':
					newLine.append(item)								
					#newLine.append(item.split('-')[0])
				elif item.count(":") > 0:
					if (item[0].islower() and modeInd == 0) or (item[0].isupper() and modeInd == 1):
						break
			for i in reversed(range(len(newLine))):
				if newLine[i] not in ['.', '']:
					break
			newLine = newLine[:i+1]
			total = len(newLine) - 1
			cycleStart = 0
			foundNonTonic = False
			if phraseMustStartWithTonic and (not newLine[0].upper().startswith('I-')): continue
			for i, item in enumerate(newLine):
				#print i, item
				if item in ['.', '']: continue
				newFigure = DT.parse_figure(item.split('-')[0])
				if newFigure[0].upper() == 'I' and newFigure[1] != '6/4':
					if foundNonTonic:
						if cycleStart is not None:
							timePoint = int(((100. * i) / total) + .5)									# expressed as % of the phrase
							cycleDict.setdefault(timePoint, []).append(i - cycleStart)
							cycleStrength = evaluate_cycle(newLine, cycleStart, i, total)
							if cycleStrength is not None:
								cycleStrengths.append([timePoint, cycleStrength])
							#print "  CYCLE", cycleStart, i, i - cycleStart, timePoint
						cycleStart = i
						foundNonTonic = False
				elif i == total:
					cycleStrength = evaluate_cycle(newLine, cycleStart, i, total)
					if cycleStrength is not None:
						cycleStrengths.append([100, cycleStrength])
					#print "  RESIDUE", cycleStart, i, i - cycleStart, timePoint
				else:
					foundNonTonic = True
	print(totalCycles, "cycles found")
	#print cycleStrengths

def evaluate_cycle(newLine, cycleStart, i, totalLength):
	global totalCycles
	temp = [x for x in newLine[cycleStart:i+1] if x != '.']
	strength = cycle_strength(temp)
	if strength is not None:
		totalCycles += 1
	return strength
	
def cycle_strength(l):
	finalLine = []
	for item in l:
		chord, sd = item.split('-')
		finalLine.append([DT.parse_figure(chord), sd])
		
	totalStrength = 0
	lastChord, lastMelody = finalLine[-1]
	
	if lastChord[0].upper() == 'I' and not lastChord[2]:
		if not lastChord[1]: totalStrength += 1
		if lastMelody == '1': totalStrength += 1
	else:
		return None
	
	if len(finalLine) == 1: return totalStrength
	
	penultimateChord, penultimateMelody = finalLine[-2]
	
	if penultimateChord[0] in ['viio', 'vii/o'] and not penultimateChord[2]:
		totalStrength += .25
	elif penultimateChord[0] == 'V' and not penultimateChord[2]:
		if not penultimateChord[1]: totalStrength += .5
		if penultimateChord[1] == '7': totalStrength += .5
		if penultimateMelody == '2': totalStrength += .5
	
		if len(finalLine) == 2: return totalStrength
	
		prepChord, prepMelody = finalLine[-3]
		
		if prepChord[0] == 'V' and not prepChord[2] and len(finalLine) > 3:
			prepChord, prepMelody = finalLine[-4]
	
		if prepChord[0].upper() in ['IV', 'II', 'IIO', 'II/O']: totalStrength += .5
	
	return totalStrength
	
def cycle_histo(mode = 'major', ticks = 8):
	global values
	modeInd = int(mode[:2].upper() == 'MI')
	harmonic_cycle_length(mode)
	plt.xlabel('Percentage through the phrase')
	plt.ylabel('Average length of cycles')
	plt.title('Harmonic cycle length by phrasal position')
	binSize = 100. / ticks
	values = []
	for i in range(ticks):
		values.append([])
	for j in cycleDict:
		binLoc = int(j/binSize)
		if binLoc >= ticks: 
			binLoc = ticks - 1
		values[binLoc] += cycleDict[j]
	v = [sum(x)*1./max(len(x), .01) for x in values]
	"""print values
	if norm:
		total = sum([sum(x) for x in values])
	if not norm:
		total = sum(values)
	v = [100. * x/total for x in values]"""
	plt.plot(v)
	ticks = range(ticks)
	plt.xticks(ticks, [str(int((binSize * x) + .5)) for x in ticks])
	plt.show()	

def strength_histo(mode = 'major', ticks = 8):
	global values
	modeInd = int(mode[:2].upper() == 'MI')
	harmonic_cycle_length(mode)
	plt.xlabel('Percentage through the phrase')
	plt.ylabel('Strength of cycles')
	plt.title('Harmonic cycle strength by phrasal position')
	binSize = 100. / ticks
	values = []
	for i in range(ticks):
		values.append([])
	for j, strength in cycleStrengths:
		binLoc = int(j/binSize)
		if binLoc >= ticks: 
			binLoc = ticks - 1
		values[binLoc].append(strength)
	#print values
	v = [sum(x)/max(float(len(x)), .001) for x in values]
	#print('HERE', v)
	plt.plot(v)
	ticks = range(ticks)
	plt.xticks(ticks, [str(int((binSize * x) + .5)) for x in ticks])
	plt.show()	


def save_data():
	for i in rawFiles[2]:
		print("")
		print("CHORALE ", i)
		print("")
		for j in rawFiles[2][i]:
			print(j)
	
def count_chords(mode = 'major', avoidModulations = False, norm = True, graphNo = 40, labels = True):
	global chordDict, chordMode
	modeInd = int(mode[:2].upper() == 'MI')
	chordMode = modeInd
	chordDict = {}
	for n in rawFiles[1]:
		for line in rawFiles[2][n]:
			if ['m', 'M'][modeInd] == line[0]: continue
			if avoidModulations and line.count(":") > 0: continue
			line = line[2:]
			newLine = []
			for item in line.split(' '):
				if item in [' ', '.', '|', '']: continue
				if item.count(":") == 0 and item != '|':
					chordDict[item] = chordDict.setdefault(item, 0) + 1
				elif item.count(":") > 0:
					if (item[0].islower() and modeInd == 0) or (item[0].isupper() and modeInd == 1):
						break
	plt.xlabel('Chord + melodynote')
	plt.ylabel('Percentage')
	plt.title('Chord/melody by percentage of all possibilities')
	k = sorted(chordDict.keys(), key = lambda x: -chordDict[x])
	labels = k[:min(graphNo, len(k))]
	values = [chordDict[x] for x in labels]
	if norm:
		total = sum(values)
		values = [x*100./total for x in values]
	plt.plot(values)
	ticks = range(len(labels))
	plt.xticks(ticks, addbreaks(labels), fontsize = 4)
	plt.show()	

def addbreaks(s, extraLine = False):
	if extraLine:
		mySym = '\n'
	else:
		mySym = ''
	t = []
	for i in range(len(s)):
		if i % 2 == 1:
			t.append('\n' + mySym + s[i])
		else:
			t.append(s[i])
	return t

