from __future__ import print_function
import os
import operator
import pickle
from music21 import *
import DT

"""
This is an internal program used by analyzer.py.

It takes a raw analysis in which every chord is considered harmonic and "prunes" away those surface chords that are likely to be nonharmonic.

It uses the raw statistcs as 'priors' to do thiis; see the preface to chapter 7 of TAOM.

The result is a substantial improvement in analytical accuracy.
"""


lastChorale = 1
chorale = stream.Score()
analysis = stream.Score()

DISCOUNT_THRESHOLD = 4					# play with this!

counts = [{}, {}]
#storedFile = open(DT._DATAPATH + '4_TIMESLICEprogressions44CLEANMACHINE.pkl', 'rb')
try:
	with open(DT._DATAPATH + 'EIGHTHNOTE4TS44BACH CHORALES RAWMACHINE.pkl', 'rb') as storedFile:
		counts = pickle.load(storedFile)
except:
	print('To run prune_progressions you need a raw analysis source; run')
	print('   > import eighthnote_harmonies') #
	print('   > eighthnote_harmonies.gather_stats("Bach Chorales Rawmachine")')

two_counts = [{}, {}]

try:
	with open (DT._DATAPATH + 'RNBACH CHORALES RAWMACHINE.pkl', 'rb') as storedFile:
		all_counts = pickle.load(storedFile)
	for modeInd in [0, 1]:
		for item in all_counts[modeInd]:
			if item.count(' -> ') == 1:
				two_counts[modeInd][item] = all_counts[modeInd][item]
except:
	print('To run prune_progressions you need a raw analysis source; run')
	print('   > import rn') #
	print('   > rn.gather_stats("Bach Chorales Rawmachine")')

def init():
	global counts, endCounts, startCounts, strongCounts, weakCounts, strongWeakRatio
	endCounts = [{}, {}]
	startCounts = [{}, {}]
	strongCounts = [{}, {}]
	weakCounts = [{}, {}]
	strongWeakRatio = [{}, {}]
	for modeInd in [0, 1]:
		for item in counts[modeInd]:
			splits = item.split(' -> ')
			strongEighth = splits[1]
			weakEighth = splits[2]
			endItem = ' -> '.join(splits[1:])
			startItem = ' -> '.join(splits[:-1])
			endCounts[modeInd][endItem] = endCounts[modeInd].setdefault(endItem, 0) + counts[modeInd][item]
			startCounts[modeInd][startItem] = startCounts[modeInd].setdefault(startItem, 0) + counts[modeInd][item]
			strongCounts[modeInd][strongEighth] = strongCounts[modeInd].setdefault(strongEighth, 0) + counts[modeInd][item]
			weakCounts[modeInd][weakEighth] = weakCounts[modeInd].setdefault(weakEighth, 0) + counts[modeInd][item]
	for modeInd in [0, 1]:
		for item in strongCounts[modeInd]:
			if item in weakCounts[modeInd]:
				strongWeakRatio[modeInd][item] = [strongCounts[modeInd][item], weakCounts[modeInd][item]]
			else:
				strongWeakRatio[modeInd][item] = [strongCounts[modeInd][item], 0]
	for modeInd in [0, 1]:
		for item in weakCounts[modeInd]:
			if item in strongCounts[modeInd]: continue
			strongWeakRatio[modeInd][item] = [0, weakCounts[modeInd][item]]
				

init()

"""

HERE WE FIX A MACHINE-MADE BACH CHORALE ANALYSIS BY "PRUNING" EIGHTH-NOTE PROGRESSIONS THAT ARE REALLY THE RESULT OF PASSING OR NEIGHBORING TONES


	ALLOW FOR THE CASE WHERE THE BASS CHANGES AND THE CHORD GETS DISGUISED BY PASSING TONES (like I6-"vi6" which is really I with 5-6 motion in the soprano)
	
	Interesting to think about 3/4 chorales, which come in two flavors: 
		3 chords per measure (where the eighth-note chords can be dealt with normally) 
		2 chords per measure (where beat 2 should be treated like an eighth note)
	
	!!!! this will reduce i6->i, which isn't right.  Chorale 5 !!!
	
	IN GENERAL, THE METHOD DOES NOT WORK WITH SUPER RARE PROGRESSIONS:
	
		- Chorale 2, measure 3.  vi2 is a super rare chord, but our method doesn't eliminate it, because all the relevant progressions are rare.
			(NOT RIGHT -- actually, IV6 -> ii6 always has something in between, either I6/4 or vi2.  Very deep issues here)
		- maybe if the counts are all small (< 2) we can just go with chord frequency?
		- chorale 3 has two chords to be eliminated in a row: e: i b1.5 III6/4 b2 iio6/4 b2.5 iv6
			This isn't conceptually hard, but getting the details right would be complicated.
		- chorale 4, there are not enough IV->ii->viio6->I6 progressions.  This is FIXED by going to pairs of stats
		
	This method also won't deal with complicated combinations of harmonic and nonharmonic tones; 
		e.g. Chorale 2, measure 9, beat 2.5, V7->V2->I6 with the lt passing down 7-6-5 as the bass goes 5-4-3.
		It only works if one of the two chords is the real one.
	 	this could be fixed for weak-beat chords, probably

	For key changes, maybe don't require the final chord?  (Chorale 5 has a problem here)
	
	KILL OFF MAJOR KEY AREAS WITH NO V OR V7 (see chorale 3)
	
	Problems for the underlying MARTIANS analysis module: 
		favor key change at the beginning of a phrase if possible [DONE]; ends of phrases (picardy) are a similar problem
		chorale 3, m2, b1.5 V-vii should maybe be removed? (though it is nice that this gets done automatically!)
		incomplete triads should be done in a more intelligent way, using prior probabilities
		idiom: first inversion chord, bass leaps down to an incomplete seventh; need to put that in
		suspension resolution isn't great: chorale 5, m 10 -- it's using the next note, instead of the one below
		chorale 5, penultimate measure, last beat: doesn't get the ii7

"""

def dochorale(inputChorale, inputAnalysis, strict = False):
	global chorale, analysis, anal, score, counts, two_counts, figures_to_remove, endCounts, startCounts, seventhList
	chorale = inputChorale
	analysis = inputAnalysis
	score = [i.flat.getElementsByClass(['Note', 'Rest']).stream() for i in chorale.parts]							#add rest here?
	"""score = []
	for i in range(0, len(chorale.parts)):
		score.append([chorale.parts[i].flat.getElementsByClass('Note')]"""
	anal = analysis.flat.getElementsByClass(['RomanNumeral'])
	offsets = [i.offset for i in anal]
	offsets = sorted(list(set(offsets)))
	quarters_to_check = []
	cadentialSchemaQuarters = []
	weakEighthsToCheck = []
	figures_to_remove = {}
	seventhList = {}
	offsets_to_remove = []
	
	for i in range(len(offsets) - 2):
		if (offsets[i+2] - 1 == offsets[i] and offsets[i+1] - .5 == offsets[i]) and (offsets[i] - int(offsets[i]) < .001): quarters_to_check.append(offsets[i])
		elif (offsets[i+2] - .75 == offsets[i+1] - .5 == offsets[i]) and (offsets[i] - int(offsets[i]) < .001): cadentialSchemaQuarters.append(offsets[i])
		elif (offsets[i] - int(offsets[i]) == .5) and (offsets[i+1] - .5 == offsets[i]): weakEighthsToCheck.append(offsets[i])
	
	#print weakEighthsToCheck
	
	for i in weakEighthsToCheck:																			# manually get rid of weak eighth sevenths
		harmonyA = anal.getElementsByOffset(i, i, includeEndBoundary = True, mustBeginInSpan = False)
		if not harmonyA: continue
		harmonyA = harmonyA[0]
		parsedA = DT.parse_figure(harmonyA.figure)
		if parsedA[1] in ['7', '6/5', '4/3', '2'] and parsedA[0] != 'V': #and could_be_nonharmonic(i, strongBeat = False):  #FIX THIS!!!
			offsets_to_remove.append(i)
			measure, beat = DT.get_measure_number_and_beat_from_offset(chorale, i)
			if measure in figures_to_remove:
				figures_to_remove[measure].append(beat)
			else:
				figures_to_remove[measure] = [beat]
	
	if strict: cadentialSchemaQuarters = []
	
	for i in cadentialSchemaQuarters:
		
		harmonyA = anal.getElementsByOffset(i, i, includeEndBoundary = True, mustBeginInSpan = False)
		harmonyB = anal.getElementsByOffset(i + .5, i + .5, includeEndBoundary = True, mustBeginInSpan = False)
		nextHarmony = anal.getElementsByOffset(i + .75, i + .75, includeEndBoundary = True, mustBeginInSpan = False)
		if (not nextHarmony) or (not harmonyA) or (not harmonyB): continue
		
		nextHarmony = nextHarmony[0]
		harmonyA = harmonyA[0]
		harmonyB = harmonyB[0]
		
		if (harmonyA.key != harmonyB.key) or (harmonyA.key != nextHarmony.key): continue
		
		parsedHarmonyA = DT.parse_figure(harmonyA.figure)
		parsedHarmonyB = DT.parse_figure(harmonyB.figure)
		parsedNextHarmony = DT.parse_figure(nextHarmony.figure)
		
		if parsedHarmonyA[2] or parsedHarmonyB[2] or parsedNextHarmony[2]: continue
		
		if parsedHarmonyA[0] == 'ii' and (parsedHarmonyB[0] == 'viio' or parsedHarmonyB[0] == 'vii/o') and parsedNextHarmony[0] == 'ii': 
		
			remove_offset = i + .5
		
			offsets_to_remove.append(remove_offset)
			measure, beat = DT.get_measure_number_and_beat_from_offset(chorale, remove_offset)
			if measure in figures_to_remove:
				figures_to_remove[measure].append(beat)
			else:
				figures_to_remove[measure] = [beat]
		
			print ('FOUND SCHEMA A', i)

		elif parsedHarmonyA[0] == 'V' and parsedHarmonyB[0] == 'iii' and parsedNextHarmony[0] == 'V': 

			remove_offset = i + .5

			offsets_to_remove.append(remove_offset)
			measure, beat = DT.get_measure_number_and_beat_from_offset(chorale, remove_offset)
			if measure in figures_to_remove:
				figures_to_remove[measure].append(beat)
			else:
				figures_to_remove[measure] = [beat]

			print ('FOUND SCHEMA B', i)
	
	if strict: quarters_to_check = []
	
	for i in quarters_to_check:
		
		prevHarmony = anal.getElementsByOffset(i - .01, i - .01, includeEndBoundary = True, mustBeginInSpan = False)
		harmonyA = anal.getElementsByOffset(i, i, includeEndBoundary = True, mustBeginInSpan = False)
		harmonyB = anal.getElementsByOffset(i + .5, i + .5, includeEndBoundary = True, mustBeginInSpan = False)
		nextHarmony = anal.getElementsByOffset(i + 1, i + 1, includeEndBoundary = True, mustBeginInSpan = False)
		if (not prevHarmony) or (not nextHarmony) or (not harmonyA) or (not harmonyB): continue
		
		prevHarmony = prevHarmony[0]
		nextHarmony = nextHarmony[0]
		harmonyA = harmonyA[0]
		harmonyB = harmonyB[0]
		
		parsedA = DT.parse_figure(harmonyA.figure)
		parsedB = DT.parse_figure(harmonyB.figure)
		
		if parsedA[2] or parsedB[2]: continue			# don't prune secondaries!
		
		modulation = 0					# 0 for no modulation; 1 for before our pair; 2 for after
		currentCounts = counts
		
		if prevHarmony.offset in offsets_to_remove:
			prevHarmony = anal.getElementBeforeOffset(prevHarmony.offset, ['RomanNumeral'])
			if not prevHarmony: continue
				
		if (harmonyA.key != harmonyB.key): continue
		
		if (prevHarmony.key != harmonyA.key) and (harmonyB.key != nextHarmony.key): continue
		
		harmonyC = ''
		
		if could_be_seventh(i + .5):
			measure, beat = DT.get_measure_number_and_beat_from_offset(chorale, i + .5)
			harmonyCpitchList = []
			for j in range(len(score)):
				newNote = score[j].getElementsByOffset(i, i, classList = ['Note'], includeEndBoundary = True, mustBeginInSpan = False)
				if not newNote: continue
				newNote = newNote[0].pitch
				harmonyCpitchList.append(pitch.Pitch(str(newNote)))
			newPitch = pitch.Pitch(str(could_be_seventh.seventh))
			harmonyCpitchList.append(newPitch)
			harmonyCpitchList = sorted(harmonyCpitchList)
			harmonyCpitchList.reverse()
			harmonyCpitches = chord.Chord(harmonyCpitchList)
			harmonyC = DT.figure_from_chord(harmonyCpitches, harmonyA.key)
			if harmonyC == harmonyB.figure: harmonyC = ''
		
		#harmonyC = ''
		#alternativeFigure = ''
		
		if (prevHarmony.key != nextHarmony.key):
			if prevHarmony.key == harmonyA.key:
				modulation = 2
				currentCounts = startCounts
			else:
				modulation = 1
				currentCounts = endCounts
		
		if DT.parse_figure(prevHarmony.figure)[2]:
			modulation = 1													# no stats for applied chords
			currentCounts = endCounts
		elif DT.parse_figure(nextHarmony.figure)[2]:
			modulation = 2
			currentCounts = startCounts
		
		modeInd = int(prevHarmony.key.mode == 'minor')			# 0 for major, 1 for minor
		
		if modulation == 0:
			fullFigure = ' -> '.join([prevHarmony.figure, harmonyA.figure, harmonyB.figure, nextHarmony.figure])
			if harmonyC: alternativeFigure = ' -> '.join([prevHarmony.figure, harmonyA.figure, harmonyC, nextHarmony.figure])
		elif modulation == 1:
			fullFigure = ' -> '.join([harmonyA.figure, harmonyB.figure, nextHarmony.figure])
			if harmonyC: alternativeFigure = ' -> '.join([harmonyA.figure, harmonyC, nextHarmony.figure])
		else:
			fullFigure = ' -> '.join([prevHarmony.figure, harmonyA.figure, harmonyB.figure])
			if harmonyC: alternativeFigure = ' -> '.join([prevHarmony.figure, harmonyA.figure, harmonyC])
			
		fullFigure_count = 0
		alternativeFigureCount = 0
		if fullFigure in currentCounts[modeInd]: 
			fullFigure_count = currentCounts[modeInd][fullFigure] - 1				# -1 to account for the fact that the raw analysis includes the current progression!
		
		best_so_far = [fullFigure, fullFigure_count]
		seventhFlag = False
		
		if harmonyC and alternativeFigure in currentCounts[modeInd]: 		
			alternativeFigureCount = currentCounts[modeInd][alternativeFigure]		
			if alternativeFigureCount >= fullFigure_count:						# experimental; used to be >
				best_so_far = [alternativeFigure, alternativeFigureCount]
				print ("USING SEVENTH INTERPRETATION, replacing", fullFigure, "with", alternativeFigure)
				seventhFlag = True
		
		nonharmonicA = False
		nonharmonicB = False
		noHarmonyA_count = 0
		noHarmonyB_count = 0
		remove_offset = -1
		
		if could_be_nonharmonic(i + .5, strongBeat = False) and (not parsedB[2]) and parsedB[0] not in ['V', 'viio', 'vii/o']:
			nonharmonicB = True
			
			if modulation == 0:
				noHarmonyB = ' -> '.join([prevHarmony.figure, harmonyA.figure, harmonyA.figure, nextHarmony.figure])
			elif modulation == 1:
				noHarmonyB = ' -> '.join([harmonyA.figure, harmonyA.figure, nextHarmony.figure])
			else:
				noHarmonyB = ' -> '.join([prevHarmony.figure, harmonyA.figure, harmonyA.figure])
			
			if noHarmonyB in currentCounts[modeInd]: noHarmonyB_count = currentCounts[modeInd][noHarmonyB]
			if noHarmonyB_count > best_so_far[1]:							
				best_so_far = [noHarmonyB, noHarmonyB_count]
				remove_offset = i + .5
				seventhFlag = False
		
		if could_be_nonharmonic(i) and (not parsedA[2]) and parsedA[0] not in ['V', 'viio', 'vii/o']:
			nonharmonicA = True
			
			if modulation == 0:
				noHarmonyA = ' -> '.join([prevHarmony.figure, harmonyB.figure, harmonyB.figure, nextHarmony.figure])
			elif modulation == 1:
				noHarmonyA = ' -> '.join([harmonyB.figure, harmonyB.figure, nextHarmony.figure])
			else:
				noHarmonyA = ' -> '.join([prevHarmony.figure, harmonyB.figure, harmonyB.figure])
			
			if noHarmonyA in currentCounts[modeInd]: noHarmonyA_count = currentCounts[modeInd][noHarmonyA]
			if noHarmonyA_count > best_so_far[1]:
				if could_be_nonharmonic.foundANHT:								# Accented NTs and PTs need to meet a higher crossbar
					if noHarmonyA_count > 2 * best_so_far[1]:					# this multiplier should be made more principled!
						best_so_far = [noHarmonyA, noHarmonyA_count]
						remove_offset = i
						seventhFlag = False
				else:
					best_so_far = [noHarmonyA, noHarmonyA_count]
					remove_offset = i
					seventhFlag = False
		
		"""Now deal with the case where our stats are insufficient"""
		
		if fullFigure_count < DISCOUNT_THRESHOLD and alternativeFigureCount < DISCOUNT_THRESHOLD and noHarmonyA_count < DISCOUNT_THRESHOLD and noHarmonyB_count < DISCOUNT_THRESHOLD and modulation == 0:
			remove_offset = -1
			firstProg = ' -> '.join([prevHarmony.figure, harmonyA.figure])
			secondProg = ' -> '.join([harmonyA.figure, harmonyB.figure])
			thirdProg = ' -> '.join([harmonyB.figure, nextHarmony.figure])
			firstAlternative = ' -> '.join([prevHarmony.figure, harmonyB.figure])
			secondAlternative = ' -> '.join([harmonyA.figure, nextHarmony.figure])
			firstProgCount = 0 
			secondProgCount = 0
			thirdProgCount = 0
			firstAlternativeCount = 0
			secondAlternativeCount = 0
			if firstProg in two_counts[modeInd]: firstProgCount = two_counts[modeInd][firstProg]
			if secondProg in two_counts[modeInd]: secondProgCount = two_counts[modeInd][secondProg]
			if thirdProg in two_counts[modeInd]: thirdProgCount = two_counts[modeInd][thirdProg]
			if firstAlternative in two_counts[modeInd]: 
				try:
					myRatio = strongWeakRatio[modeInd][harmonyB.figure]										# don't move a weak-beat chord to the strong beat
				except:
					myRatio = [0, 1]
				if 1.0 * myRatio[0] / sum(myRatio) > .2: 
					firstAlternativeCount = two_counts[modeInd][firstAlternative]
			if secondAlternative in two_counts[modeInd]: secondAlternativeCount = two_counts[modeInd][secondAlternative]
			best_so_far = max(firstProgCount * secondProgCount, secondProgCount * thirdProgCount)		# need to do this better
			alternativeA = firstAlternativeCount * thirdProgCount
			alternativeB = firstProgCount * secondAlternativeCount
			if nonharmonicB and alternativeB > best_so_far:												# should penalize APTS more
				best_so_far = alternativeB
				remove_offset = i + .5
				seventhFlag = False
			if nonharmonicA and alternativeA > 2. * best_so_far:
				best_so_far = alternativeA
				remove_offset = i
				seventhFlag = False	
		
		if seventhFlag: remove_offset = i + .5
		
		if remove_offset != -1:
			offsets_to_remove.append(remove_offset)
			measure, beat = DT.get_measure_number_and_beat_from_offset(chorale, remove_offset)
			if seventhFlag:
				strBeat = 'b' + str(beat)
				if strBeat[-2:] == '.0': strBeat = strBeat[:-2]
				if measure in seventhList:
					seventhList[measure][strBeat] = harmonyC
				else:
					seventhList[measure] = {}
					seventhList[measure][strBeat] = harmonyC
			if measure in figures_to_remove:
				figures_to_remove[measure].append(beat)
			else:
				figures_to_remove[measure] = [beat]
	if figures_to_remove:
		print ('Measure, Beats to remove:', figures_to_remove)
	return offsets_to_remove, seventhList

def could_be_nonharmonic(offset, strongBeat = True, chord_duration = .5):
	
	global chorale, analysis, anal, score
	could_be_nonharmonic.foundANHT = False																		# need this to disfavor accented NTs and PTs
	myHarmony = anal.getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = True)
	if not myHarmony: return False
	myHarmony = myHarmony[0]
	newOffset = offset + .5
	if not strongBeat: newOffset -= 1.0
	contenderHarmony = anal.getElementsByOffset(newOffset, newOffset, includeEndBoundary = True, mustBeginInSpan = False)
	if not contenderHarmony: return False
	contenderHarmony = contenderHarmony[0]
	
	if strongBeat and myHarmony.romanNumeral == 'V':
		fig1 = DT.parse_figure(myHarmony.figure)
		fig2 = DT.parse_figure(contenderHarmony.figure)
		if fig1[0] == fig2[0] and fig1[2] == fig2[2] and fig1[3] == fig2[3]:
			if fig1[1] == '' and fig2[1] == '7': return False
			if fig1[1] == '6' and fig2[1] == '6/5': return False
			if fig1[1] == '6/4' and fig2[1] == '4/3': return False
			
	if (not strongBeat) and myHarmony.romanNumeral == 'V':
		fig1 = DT.parse_figure(contenderHarmony.figure)				# now contender is first because we're looking back from strong to weak
		fig2 = DT.parse_figure(myHarmony.figure)
		if fig1[0] == fig2[0] and fig1[2] == fig2[2] and fig1[3] == fig2[3]:
			if fig1[1] == '' and fig2[1] == '7': return False
			if fig1[1] == '6' and fig2[1] == '6/5': return False
			if fig1[1] == '6/4' and fig2[1] == '4/3': return False
	
	found_problem = False
	for i in range(len(chorale.parts)):
		myNote = score[i].getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False)				# this was once "true"
		if not myNote: continue
		myNote = myNote[0]
		if 'Rest' in myNote.classes: continue
		
		prevNote = score[i].getElementsByOffset(offset - .01, offset - .01, includeEndBoundary = True, mustBeginInSpan = False)
		prevHarmony = anal.getElementsByOffset(offset - .01, offset - .01, includeEndBoundary = True, mustBeginInSpan = False)
		if (not prevNote) or (not prevHarmony): 
			found_problem = True
			break
		prevNote = prevNote[0]
		prevHarmony = prevHarmony[0]
		
		"""Right now I am being strict; assuming that the other notes must be harmonic"""
		
		if ('Rest' in prevNote.classes) or (prevNote.pitch.pitchClass not in prevHarmony.pitchClasses):
			found_problem = True
			break
		
		nextNote = score[i].getElementsByOffset(offset + chord_duration, offset + chord_duration, includeEndBoundary = True, mustBeginInSpan = False)
		nextHarmony = anal.getElementsByOffset(offset + chord_duration, offset + chord_duration, includeEndBoundary = True, mustBeginInSpan = False)
		if (not nextNote) or (not nextHarmony): 
			found_problem = True
			break
		nextNote = nextNote[0]
		nextHarmony = nextHarmony[0]
		
		if ('Rest' in nextNote.classes) or nextNote.pitch.pitchClass not in nextHarmony.pitchClasses:
			found_problem = True
			break
		
		if strongBeat and myNote.pitch == nextNote.pitch: continue
		elif not strongBeat and myNote.pitch == prevNote.pitch: continue
		
		prevInt = interval.notesToGeneric(prevNote, myNote).value
		nextInt = interval.notesToGeneric(myNote, nextNote).value
		
		if (prevInt == 2 and nextInt == -2) or (prevInt == -2 and nextInt == 2): 					# diatonic neighbor
			if strongBeat: could_be_nonharmonic.foundANHT = True
			continue																				
		if (prevInt == 2 and nextInt == 2) or (prevInt == -2 and nextInt == -2): 					# diatonic passing
			if strongBeat: could_be_nonharmonic.foundANHT = True
			continue
		if prevInt == 1 and nextInt == -2 and strongBeat: continue									# suspension
		
		if (myNote.pitch.midi - prevNote.pitch.midi) == 1 and (nextNote.pitch.midi - myNote.pitch.midi) == 1: 				# ascending chromatic passing 
			if strongBeat: could_be_nonharmonic.foundANHT = True
			continue
		if (myNote.pitch.midi - prevNote.pitch.midi) == -1 and (nextNote.pitch.midi - myNote.pitch.midi) == -1: 			# descending chromatic passing 
			if strongBeat: could_be_nonharmonic.foundANHT = True
			continue
		
		found_problem = True
	
	if found_problem: return False
	return True

def could_be_seventh(offset, chord_duration = .5):
	strongBeat = False
	global chorale, analysis, anal, score
	could_be_seventh.foundANHT = False																		# need this to disfavor accented NTs and PTs
	could_be_seventh.seventh = False
	myHarmony = anal.getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = True)
	if not myHarmony: return False
	myHarmony = myHarmony[0]
	newOffset = offset - .5
	contenderHarmony = anal.getElementsByOffset(newOffset, newOffset, includeEndBoundary = True, mustBeginInSpan = True)
	if not contenderHarmony: return False
	contenderHarmony = contenderHarmony[0]
	if not contenderHarmony.isTriad(): return False
	prevRoot = contenderHarmony.root().pitchClass
	found_problem = False
	for i in range(len(chorale.parts)):
		myNote = score[i].getElementsByOffset(offset, offset, includeEndBoundary = True, mustBeginInSpan = False)
		if not myNote: continue
		myNote = myNote[0]
		if 'Rest' in myNote.classes: continue
		prevNote = score[i].getElementsByOffset(offset - .01, offset - .01, includeEndBoundary = True, mustBeginInSpan = False)
		prevHarmony = anal.getElementsByOffset(offset - .01, offset - .01, includeEndBoundary = True, mustBeginInSpan = False)
		if (not prevNote) or (not prevHarmony): 
			found_problem = True
			break
		prevNote = prevNote[0]
		prevHarmony = prevHarmony[0]
		"""Right now I am being strict; assuming that the other notes must be harmonic"""
		if ('Rest' in prevNote.classes) or (prevNote.pitch.pitchClass not in prevHarmony.pitchClasses):
			found_problem = True
			break	
		nextNote = score[i].getElementsByOffset(offset + chord_duration, offset + chord_duration, includeEndBoundary = True, mustBeginInSpan = False)
		nextHarmony = anal.getElementsByOffset(offset + chord_duration, offset + chord_duration, includeEndBoundary = True, mustBeginInSpan = False)
		if (not nextNote) or (not nextHarmony): 
			found_problem = True
			break
		nextNote = nextNote[0]
		nextHarmony = nextHarmony[0]
		if ('Rest' in nextNote.classes) or nextNote.pitch.pitchClass not in nextHarmony.pitchClasses:
			found_problem = True
			break
		if myNote.pitch == prevNote.pitch: continue
		prevInt = interval.notesToGeneric(prevNote, myNote).value
		nextInt = interval.notesToGeneric(myNote, nextNote).value
		if prevNote.pitch.pitchClass == prevRoot:
			if prevInt == -2 and nextInt == -2: 
				could_be_seventh.seventh = myNote.pitch
				continue								# only passing allowed
		else:
			if (prevInt == 2 and nextInt == -2) or (prevInt == -2 and nextInt == 2): 					# diatonic neighbor
				if strongBeat: could_be_nonharmonic.foundANHT = True
				continue																				
			if (prevInt == 2 and nextInt == 2) or (prevInt == -2 and nextInt == -2): 					# diatonic passing
				if strongBeat: could_be_nonharmonic.foundANHT = True
				continue
			if prevInt == 1 and nextInt == -2 and strongBeat: continue									# suspension
			if (myNote.pitch.midi - prevNote.pitch.midi) == 1 and (nextNote.pitch.midi - myNote.pitch.midi) == 1: 				# ascending chromatic passing 
				if strongBeat: could_be_nonharmonic.foundANHT = True
				continue
			if (myNote.pitch.midi - prevNote.pitch.midi) == -1 and (nextNote.pitch.midi - myNote.pitch.midi) == -1: 			# descending chromatic passing 
				if strongBeat: could_be_nonharmonic.foundANHT = True
				continue	
		found_problem = True
	if found_problem or (not could_be_seventh.seventh): return False
	return True

def rewrite_analysis(choraleNo):				# remove duplicates
	global figures_to_remove, seventhList
	myFile = open(DT._MACHINEANALYSES + 'riemenschneider' + str(choraleNo).zfill(3) + '.txt', 'rb')
	outFile = open(DT._CLEANMACHINE + 'riemenschneider' + str(choraleNo).zfill(3) + '.txt', 'w')
	for line in myFile:
		if line[0] != 'm': 
			outFile.write(line)
			continue
		symbols = line.split()
		measure = int(symbols[0][1:])
		if measure not in figures_to_remove:
			outFile.write(line)
			continue
		outSymbols = []
		moveBack = False
		beatsToRewrite = []
		if measure in seventhList:
			beatsToRewrite = seventhList[measure]
		beatsToKill = ['b' + str(x) for x in figures_to_remove[measure]]
		for i in range(len(beatsToKill)):
			if beatsToKill[i][-2:] == '.0': beatsToKill[i] = beatsToKill[i][:-2]
		recordThem = True
		nextSeventh = False
		nextSym = ''
		for symbol in symbols:
			if symbol in beatsToKill:
				if symbol in beatsToRewrite:
					nextSym = beatsToRewrite[symbol]
				else:
					recordThem = False
			elif symbol[0] == 'b' and recordThem == False:
				recordThem = True
				if symbol[-2:] == '.5': 				# killed .0 beat
					symbol = symbol[:-2]				# change this???
				if symbol[-2:] == '75':					# killed .5 beat, followed by .75
					symbol = symbol[:-3] + '.5'
					nextSeventh = True
			elif nextSym:
				symbol = nextSym
				nextSym = ''
			if recordThem: 
				if nextSeventh and symbol[0] != 'b':
					if symbol[-1] == '6': symbol += '/5'
					elif (not symbol[-1].isdigit()):
						symbol += '7'
					nextSeventh = False
				outSymbols.append(symbol)
		newLine = ' '.join(outSymbols) + '\n'
		outFile.write(newLine)
	outFile.close()
	myFile.close()
	