from __future__ import print_function
import os
import pickle
import time
import copy
import sys
import random
import functools
import itertools
import traceback

from music21 import *
import matplotlib.pyplot as plt

import DT

_THISFILE = 'RANGES'

palestrinaData = DT.palestrina_data()

"""
Find the range of each part
	
"""

def gather_stats(source = 'PALRAW', fileName = '', fileNumber = None, debug = False, subpiece = None, showReduction = False):
	global progs, piece, globalSource, locNames, ranges
	startT = time.time()
	globalSource = source.upper()
	ranges = {}
	locNames = {}
	globalPiece = 1
	subpieceNumber = -1
	if globalSource not in DT._PATHS:
		print("Source not found!")
		return
	scorePath = DT._PATHS[globalSource][1]
	files = sorted(os.listdir(scorePath))
	
	forcedQuit = False
	files = [x for x in files if x[-4:].lower() in ['.xml', '.krn', '.mxl', '.mid', 'midi']]
	if fileNumber == -1:
		fileName = random.choice(files)
		fileNumber = None
	for myFile in files:
		if fileName and myFile != fileName: continue
		if (fileNumber is not None) and fileNumber != globalPiece: continue
		if myFile in palestrinaData:
			print('Analyzing', myFile, palestrinaData[myFile])
		else:
			print('Analyzing', myFile)
		piece = converter.parse(scorePath + myFile)
		try:
			if 'Opus' in piece.classes:
				bigOpus = piece
				theScores = bigOpus.scores
				if fileName or (fileNumber is not None):
					if subpiece is not None:
						subpieceNumber = subpiece
					else:
						subpieceNumber = random.choice(range(len(theScores)))
					print(("Subpiece", subpieceNumber))
					piece = theScores[subpieceNumber]
					locNames[globalPiece] = myFile[:-4]
					dopiece(globalPiece, debug = debug)
					globalPiece += 1
				else:
					for subpieceNumber, piece in enumerate(theScores):
						print(("Subpiece", subpieceNumber))
						dopiece(globalPiece, debug = debug)
						locNames[globalPiece] = myFile[:-4]
						dopiece(globalPiece, debug = debug)
						globalPiece += 1
			else:
				locNames[globalPiece] = myFile[:-4]
				dopiece(globalPiece, debug = debug)
				globalPiece += 1
		except KeyboardInterrupt:
			print("ENDING EARLY")
			forcedQuit = True
			break
		except:
			traceback.print_exc()
			print(("ERROR!"), sys.exc_info())
			forcedQuit = True
			#pass
		if (not forcedQuit) and (globalPiece % 50 == 0) and (not fileName) and (fileNumber is None):
			save()
	if fileName or fileNumber:
		if not forcedQuit:
			piece.show()
		if showReduction:
			make_list_of_consonances()
	else:
		if not forcedQuit:
			save()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))

def load(source = 'PALRAW', printOut = False):
	global ranges, locNames
	try:
		globalSource = source.upper()
		with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'rb') as inputFile:
			ranges = pickle.load(inputFile)
			locNames = pickle.load(inputFile)
		return True
	except:
		return False

def save():
	with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'wb') as outputFile:
		pickle.dump(ranges, outputFile)
		pickle.dump(locNames, outputFile)


def record_note(myDict, myNote):
	if (not myNote.tie) or (myNote.tie.type == 'start'):
		n = DT.note_to_scalarMIDIdegree(myNote)
		myDict[n] = myDict.get(n, 0) + 1
		
def record_chord(myDict, myChord):
	for myNote in myChord.pitches:
		if (not myChord.getTie(myNote)) or (myChord.getTie(myNote) == 'start'):
			n = myNote.diatonicNoteNum + 31
			myDict[n] = myDict.get(n, 0) + 1

def dopiece(pieceNumber, printOut = False, debug = False):	
	global ranges, piece, score, consonantO, timeSigs, dictOfConsonances, nhtDict, nextO, halfNoteAnticipations, _MEASUREBEAT, nhtLocations
	global resolutionTones, suspensionStarts, newO, consonantOList
	#score = [piece.parts[i].flat.getElementsByClass(['Note', 'Chord']).stream() for i in range(len(piece.parts))]
	
	ranges[pieceNumber] = []
	
	for p in piece.parts:
		ranges[pieceNumber].append({})
		d = ranges[pieceNumber][-1]
		for n in p.flat.getElementsByClass(['Note', 'Chord']):
			if 'Note' in n.classes:
				record_note(d, n)
			elif 'Chord' in n.classes:					# redundant code, for later
				record_chord(d, n)

"""
standard clefs have a range an eleventh; max note is 10 greater than min note
					
"""

clefDict = {'G2': 61, 'C1':  59, 'C2': 57, 'C3': 55, 'C4': 53, 'F3': 51, 'F4': 49, 'F5': 47}

cleffings = {'G2': ['G2', 'C2', 'C3', 'F3'], 'G2*': ['G2', 'C2', 'C3', 'C4'], 'C1': ['C1', 'C3', 'C4', 'F4'], 'C2': ['C2', 'C4', 'F3', 'F5']}


def best_fit(pieceNumber):
	partsList = []
	theRanges = ranges[pieceNumber]
	for myDict in theRanges:
		result = fits_clef(myDict)
		if result: partsList.append(result)
	clefs = [set(x[0] for x in partsList)]
	for name, theClefs in sorted(cleffings.items()):
		clefSet = set(theClefs)
		overlap = [x & clefSet for x in clefs]
		if all(overlap):
			return name, sum([x[1] for x in partsList])/float(len(partsList))
	return False

def fits_clef(myDict):
	fitPct = {}
	totalNotes = sum(myDict.values())
	if totalNotes == 0: return False
	for c in clefDict:
		minNote = clefDict[c]
		maxNote = clefDict[c] + 10
		outOfRangeNotes = 0
		for k, count in myDict.items():
			if k < minNote or k > maxNote:
				outOfRangeNotes += count
		fitPct[c] = outOfRangeNotes * 100./totalNotes
	return min(fitPct.items(), key = lambda x: x[1])
	
def get_clef(partRange):
	return [s[0] for s in clefDict.items() if s[1] <= partRange[0] and (s[1] + 10) >= partRange[1]]

def overall_range(pieceNumber, partList = []):
	d = ranges[pieceNumber]
	if partList:
		d = [d[x] for x in partList]
	if not d:
		return False
	#print d, [part_range(x) for x in d]
	pr = [part_range(x) for x in d if x]
	clefs = [set(get_clef(x)) for x in pr]
	for name, theClefs in sorted(cleffings.items()):
		clefSet = set(theClefs)
		overlap = [x & clefSet for x in clefs]
		if all(overlap):
			return name
	return False

def note_translator(i):
	octaveNumber = ((i + 3) / 7) - 5
	noteName = 'CDEFGAB'[(i+3)% 7]
	return noteName+str(octaveNumber)

def part_range(myDict):
	k = sorted(myDict.keys())
	if myDict:
		return min(k), max(k)

def get_range(pieceNumber):
	d = ranges[pieceNumber]
	print(len(d), "PARTS")
	for i in range(len(d)):
		pr = part_range(d[i])
		print("  part", i, [note_translator(x) for x in pr], get_clef(pr))
	

