from __future__ import print_function

import os
import time
import pickle
import copy

from music21 import *
import DT

_THISFILE = "RAWSEQUENCES"

"""  
LOOKS FOR SEQUENCES IN RAW SCORES, based on note repetition.

Once these are found, I use the catalog() routine to analyze them by hand, storing the result in RAWSEQUENCESCATALOGUE.

TODO: 

	Eliminate tied notes -- take only notes that begin on the offset and are not tied to a preceding note
	Look for half-beat sequences if we have a beat that is an integer

"""

globalSource = 'WTC'
sequences = {}
sequenceCatalog = {}

def gather_stats(source = 'WTC', fileName = ''):
	global piece, locNames, globalSource, filePath, totalChords, totalConsonances, fileStr, melodyComments, myFile, consonances, bigOpus, sequences
	startT = time.time()
	sequences = {}
	locNames = {}
	globalSource = source.upper()	
	if globalSource in DT._PATHS:
		analysisPath = DT._PATHS[globalSource][0]
		scorePath = DT._PATHS[globalSource][1]
		fileStr = globalSource
		files = sorted(os.listdir(scorePath))
		sequences = {}
		recordResults = True
		counter = 1
		for myFile in files:
			if myFile[-3:] in ['xml', 'krn', 'mxl']:
				if fileName and myFile != fileName: continue
				print('Analyzing', myFile)
				piece = converter.parse(scorePath + myFile)
				bigOpus = False
				if 'Opus' in piece.classes:
					bigOpus = piece
					for subpieceNo, piece in enumerate(bigOpus.scores):
						print(" subpiece", subpieceNo)
						dopiece([counter, subpieceNo])
				else:
					dopiece(counter)
				locNames[counter] = myFile
				counter += 1
				if counter % 50 == 0:
					save()
		if not fileName:
			save()
		filePath = analysisPath
		minutes = (time.time() - startT)/60.
		print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	return

def load(source = 'WTC'):
	global sequences, globalSource, locNames, sequenceCatalog
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource + '.pkl', 'rb') as inputFile:
			sequences = pickle.load(inputFile)
			locNames = pickle.load(inputFile)
		myName = _THISFILE + "CATALOGUE" + globalSource + '.pkl'
		with open(DT._DATAPATH + myName, 'rb') as theFile:
			sequenceCatalog = pickle.load(theFile)
		return True
	except:
		return False

def get_file(fileName):
	global piece
	scorePath = DT._PATHS[globalSource][1]
	piece = converter.parse(scorePath + fileName)
	piece.show()

def save():
	with open(DT._DATAPATH + _THISFILE + globalSource + '.pkl', 'wb') as outputFile:
		pickle.dump(sequences, outputFile)
		pickle.dump(locNames, outputFile)

def dopiece(pieceNumber):
	global progs, piece, consonantO, globalPrint, theSigs, lastChords, weightedProgs, newO, sortedPitchesByOffset
	score = [piece.parts[i].flat.getElementsByClass(['Note', 'Chord']).stream() for i in range(len(piece.parts))]
	newO = sorted(list(set([n.offset for p in score for n in p])))
	ts = piece.flat.getElementsByClass('TimeSignature')
	PCsByOffset = {}
	sortedPCsByOffset = {}
	sortedPitchesByOffset = {}
	
	_MAXDIST = 16
	
	if ts:
		_BEATLENGTH = ts[0].beatDuration.quarterLength
	else:
		_BEATLENGTH = 1.
		
	_MINLENGTH = 2
	
	beatsToCheck = int(round(_MAXDIST/_BEATLENGTH, 0)) + 1
	
	#print "checkpoint 1"
	
	for o in newO:
		myMidi = []
		myNotes = []
		for p in score:
			tempNotes = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False)
			if tempNotes:
				for n in tempNotes:
					if 'Note' in n.classes:
						myNotes.append(n)
					elif 'Chord' in n.classes:
						for n1 in n.pitches:
							myNotes.append(note.Note(n1))
		theNotes = [DT.note_to_scalarMIDIdegree(x) for x in myNotes]
		sortedPitchesByOffset[o] = sorted(theNotes)
	
	#print "checkpoint 2"
	lastEndOfSequence = None
	for o in newO:
		if (lastEndOfSequence is not None) and o < lastEndOfSequence:
			continue
		for j in range(1, beatsToCheck):
			#print ("CHECKING", o, "to", o + j)
			if is_sequence(o, o + (j * _BEATLENGTH)):
				sequenceInterval = lastSequentialIntFound
				#print "  found one unit"
				lastEndOfSequence, totalUnits = find_sequence_end(o, o + (j * _BEATLENGTH))
				if totalUnits >= _MINLENGTH:
					if (j > 1) or totalUnits > _MINLENGTH:
						print("FOUND SEQUENCE", o, "total units", totalUnits, "beat distance", j, DT.get_measure_number_and_beat_from_offset(piece, o), sequenceInterval)
						sequences.setdefault(tuple(sequenceInterval), []).append([pieceNumber, o, j, totalUnits])
						break
	
def compare_offsets(startOffset, secondStart, dist):
	global firstGroupOfOffsets, secondGroupOfOffsets
	distBetweenOffsets = secondStart - startOffset
	firstGroupOfOffsets = [x for x in newO if (x >= startOffset and x < (startOffset + dist))]
	secondGroupOfOffsets = [x - distBetweenOffsets for x in newO if (x >= secondStart and x < (secondStart + dist))]
	firstTest = (firstGroupOfOffsets == secondGroupOfOffsets)
	if firstTest:
		return len(firstGroupOfOffsets), 1
	if len(firstGroupOfOffsets) > 12 and len(firstGroupOfOffsets) > 12:					# NEED TO MAKE THIS A CONSTANT
		return almost_equal(firstGroupOfOffsets, secondGroupOfOffsets)
	else:
		return False

def almost_equal(list1, list2):
	minLen = min(len(list1), len(list2))
	maxLen = max(len(list1), len(list2))
	equalMembers = 0
	i = 0
	for i in range(minLen):
		if list1[i] == list2[i]:
			equalMembers += 1
		else:
			break
	return i, equalMembers*1.0/maxLen

def compare_sites(offset1, offset2):
	if len(sortedPitchesByOffset[offset2]) != len(sortedPitchesByOffset[offset1]):
		return False
	theIntervals = [sortedPitchesByOffset[offset2][x] - sortedPitchesByOffset[offset1][x] for x in range(len(sortedPitchesByOffset[offset1]))]
	lastInt = None
	finalInts = []
	for i in theIntervals:
		if i != lastInt:
			finalInts.append(i)
			lastInt = i
	if (len(finalInts) == 1 and finalInts[0] % 7 != 0) or (len(finalInts) == 2 and abs(finalInts[1] - finalInts[0]) == 7 and finalInts[0] % 7 != 0):
		return finalInts
	return False

def is_sequence(startOffset, secondStart):
	global lastSequentialIntFound, pcSequence
	pcSequence = False
	dist = secondStart - startOffset
	c = compare_offsets(startOffset, secondStart, dist)
	if not c:
		return False
	i, pct = c
	if pct < .9:
		return False
	lastSequentialInt = None
	foundHarmony = False
	for o in firstGroupOfOffsets[:i]:
		newInt = compare_sites(o, o + dist)
		if len(sortedPitchesByOffset[o]) > 1:
			foundHarmony = True
		if not newInt:
			return False
		if (lastSequentialInt != None and lastSequentialInt != newInt):
			if lastSequentialInt[0] % 7 == newInt[0] % 7:
				newInt = longer_list(lastSequentialInt, newInt)
			else:
				return False
		lastSequentialInt = newInt
	if not foundHarmony: 
		return False
	lastSequentialIntFound = newInt
	return lastSequentialIntFound

def longer_list(list1, list2):
	if len(list2) > len(list1):
		return list2
	return list1
	
def find_sequence_end(startOffset, secondStart):
	dist = secondStart - startOffset
	lastStart = secondStart
	totalUnits = 2
	goalInt = lastSequentialIntFound
	keepGoing = True
	while keepGoing:
		newInt = is_sequence(lastStart, lastStart + dist)
		if newInt != goalInt:
			keepGoing = False
		else:
			lastStart = lastStart + dist
			totalUnits += 1
			if totalUnits > 10:
				print("LOOP PROBLEM HERE")
				break
	return lastStart + dist, totalUnits

def report(theDict = sequences, digits = 0):
	ks = sorted(theDict.keys(), key = lambda x: -(len(theDict[x])))
	total = sum([len(x) for x in theDict.values()])
	for k in ks:
		print(k, len(theDict[k]), round(100.*len(theDict[k])/total, digits))

def basic_report():
	global cleanSequences
	cleanSequences = clean_sequences()
	print("SEQUENCES BY GRAND-PERIOD DIATONIC INTERVAL OF REPETITION")
	report(cleanSequences)

def specific_report(k = -1):
	global cleanSequences
	cleanSequences = clean_sequences()
	print("\n  SEQUENCES MOVING BY DIATONIC INTERVAL", k)
	report2(keys = [k], compress = False, theDict = cleanSequences)

def full_report():
	global cleanSequences, globalSource
	basic_report()
	for k in sorted(cleanSequences.keys(), key = lambda x: -len(cleanSequences[x])):
		print("\n  SEQUENCES MOVING BY DIATONIC INTERVAL", k)
		report2(keys = [k], compress = False, theDict = cleanSequences)

def clean_sequences():
	cleanSequences = {}
	for k in sequenceCatalog:
		newKey = clean_key(k)
		for item, seqType in sequenceCatalog[k].items():
			if not seqType.startswith('X'):
				cleanSequences.setdefault(newKey, {})[item] = seqType
	return cleanSequences
	
def clean_key(s, modulus = 7):
	newKey = s[0] % modulus
	if newKey > modulus / 2.:
		newKey -= modulus
	return newKey

def show(sequenceType, specificType = False, rangeList = None, minUnits = 0, minLength = 0, instance = False):
	global output, locNames, globalSource, sequences, locList
	output = stream.Score()
	tempOutput = stream.Score()
	lastChoraleNo = False
	if specificType and sequenceType:
		locList = [x[0] for x in sequenceCatalog[sequenceType].items() if x[1].startswith(specificType)]
	elif sequenceType: 
		locList = sequences[sequenceType]
	else:
		if instance:
			locList = [instance]
		else:
			locList = []
			for sequenceType in sequences:
				locList += sequences[sequenceType]
	if not locList:
		print('Not found')
		return
	locList = sorted(locList)
	for i in range(0, 4):
		tempOutput.append(stream.Part())
	#print locList
	newLocList = []
	for instance in locList:
		if (minUnits and (instance[-1] < minUnits)) or (minLength and (instance[-2] < minLength)):
			continue
		newLocList.append(instance)
	locList = newLocList
	if rangeList:
		locList = locList[rangeList[0]: rangeList[1]]
	lastPieceNo = False
	for instance in locList:
		if type(instance[0]) is list:
			pieceNumber = instance[0][0]
			subpieceNumber = instance[0][1]
		else:
			pieceNumber = instance[0]
			subpieceNumber = False
		if subpieceNumber or (locNames[pieceNumber] != lastPieceNo):
			pieceNo = locNames[pieceNumber]
			lastPieceNo = pieceNo
			piece = converter.parse(DT._PATHS[globalSource][1] + pieceNo)
			if 'Opus' in piece.classes: 
				if subpieceNumber is not None:
					piece = piece.scores[subpieceNumber]
				else:
					continue
		o = instance[1]
		measureList = DT.get_measure_number_and_beat_from_offset(piece, o)
		print('getting', locNames[pieceNumber], "measure/beat", measureList, "beat distance", instance[2], "total units", instance[3])
		measureList = [measureList[0] - 1, measureList[0] + 3]
		header = str(locNames[pieceNumber])+'m'+str(measureList[0])
		tempChorale = copy.deepcopy(DT.get_measures(piece, measureList[0], measureList[1]))
		try:
			DT.insert_text(tempChorale.parts[0][0], header, 55, 0)
			output = DT.append(tempChorale, output)
		except:
			pass
			
	for p in output.parts:
		for item in p.flat:
			if 'Note' in item.classes or 'Chord' in item.classes: item.lyrics = []
	if output:
		output.show()
	return output
	
def catalog(k = (-1,), n = 5, replace = False):
	global sequenceCatalog
	myName = _THISFILE + "CATALOGUE" + globalSource + '.pkl'
	if replace:
		sequenceCatalog = {}
	else:
		with open(DT._DATAPATH + myName, 'rb') as theFile:
			sequenceCatalog = pickle.load(theFile)
	completedNumbers = sequenceCatalog.setdefault(k, {}).keys()
	l = len(completedNumbers)
	try:
		show(k, [l, l+n])
	except:
		print("ERROR SHOWING THIS FILE?!?!")
	
	masterKeys = sequences[k]
	
	for i in range(l, l + n):
		if len(masterKeys) > i:
			location = masterKeys[i]
		else:
			break
		s = raw_input("sequence #" + str(i)+' (' + locNames[location[0]]+')? ')
		sequenceCatalog[k][tuple(location)] = s
		
	save_catalog()
		
"""def fix_catalog():
	global sequenceCatalog
	myName = _THISFILE + "CATALOGUE" + globalSource + '.pkl'
	with open(DT._DATAPATH + myName, 'rb') as theFile:
		sequenceCatalog = pickle.load(theFile)
		
	newSequenceCatalog = {}
	
	for i, s in enumerate(sequenceCatalog[(-1,)]):
		location = sequences[(-1,)][i]
		newSequenceCatalog.setdefault((-1,), {})[tuple(location)] = s[-1]
		
	sequenceCatalog = newSequenceCatalog"""

def save_catalog():
	myName = _THISFILE + "CATALOGUE" + globalSource + '.pkl'
	with open(DT._DATAPATH + myName, 'wb') as theFile:
		pickle.dump(sequenceCatalog, theFile)

def filter_catalog(k = (-2,), replaceList = [['U', 'A']], save = True):
	#for k in sorted(sequenceCatalog.keys(), key = lambda x: -len(sequenceCatalog[x])):
	print("SEQUENCE TYPE", k)
	for t, v in sequenceCatalog[k].items():
		#sequenceCatalog[k][t] = v.replace('*', ' *').replace('U', 'A').replace('/', ' /').replace('?', ' ?').replace('  ', ' ')
		for pair in replaceList:
			v = v.replace(pair[0], pair[1])
		sequenceCatalog[k][t] = v
		print("  ", t, "\t", sequenceCatalog[k][t])
	if save:
		save_catalog()

def filter_all(**kwargs):
	for k in sequenceCatalog.keys():
		filter_catalog(k, **kwargs)

def report2(keys = None, compress = True, pct = True, theDict = sequenceCatalog):
	global output
	output = {}
	if keys:
		if type(keys) is tuple:
			keys = [keys]
	else:
		keys = theDict.keys()
	for k in theDict:
		if (compress and k[0] % 7 in keys) or (k in keys):
			for k2 in theDict[k]:
				sType = theDict[k][k2]
				s = sType.split()[0]
				if s != 'X':
					output[s] = output.setdefault(s, 0) + 1
	DT.print_dict(output, pct = pct, digits = 0)
			
	
		
