from __future__ import print_function
import os
import pickle
import time
import copy
import sys
import random
import functools
import itertools
import traceback
from music21 import *


note.Note.__hash__ = lambda x: id(x)	# needed so we can use notes as dict keys

import DT
import progressions
import cadences
import matplotlib.pyplot as plt

_THISFILE = 'RENAISSANCE_ANALYZER'
_MAXANTICIPATIONLENGTH = 1

globalConsonanceList = [[0], [0, 3], [0, 4], [0, 7], [0, 8], [0, 9], [0, 3, 7], [0, 4, 7], [0, 4, 9], [0, 3, 8], [0, 3, 9]]
upperVoiceConsonanceList = [[0, 5], [0, 6], [0, 5, 9], [0, 5, 8]]
fbDict = {(0, 2, 4): 'R', (0, 2, 5): 'T', (0, 3, 5): 'F', (0, 2): '3', (0, 4): '5', (0, 5): '6', (0,): '1'}
nfDict = {(0, 4, 7): 'M', (0, 3, 7): 'm', (0, 4, 8): 'A', (0, 3, 6): 'd', (0, 5): 'P', (0, 4): 'M', (0, 3): 'm', (0, 8): 'm', (0, 9): 'M', (0,): 'P'}

with open(DT._DATAPATH + 'PALESTRINACORPUSDATA.pkl', 'rb') as myFile:
	palestrinaData = pickle.load(myFile)

"""
Identify nonharmonic tones in Renaissance scores with any number of voices.  Will fail with scores having a very large number of parts.
	
The random_piece() method is particularly useful I find.
	
TODO:
	
	Could code this better: 
		at each offset, get a list of possible readings of each note
		I am reanalizing for every combination of voices, which is dumb
		anticipations should be to a chord tone, I think

	Double dissonance with different metrical levels can't be analyzed yet; this is kind of tricky.
	You need to start with the longest tone and rectify it; then proceed to shorter tones.
	Q: is it a general rule that the longest note at any point is a consonance? 
		Agnus_1.krn, m. 8 
		Agnus_4.krn, m. 32
	
		Santus_78, m4 -- why is this lower neighbor D not caught?

	add location tracking code?
	
	add exotics:
		incomplete neighbors
		suspensions decorated by leap
		leaping anticipations
		rogue notes
	
	think about suspension decorations: HQQ should maybe not be a decoration?  Tricky issue.
	
	track exotics?  Or have a dict of trackable events
	
	Anticipations are not reliable
		m15 Kyrie_31.krn
		m7  Agnus_II_57.krn
	Should the rule be that the anticipaton has to anticipate a harmonic tone?
	
	
	check four quarters on a beat with a dissonant 3rd quarter; Jeppessen has a big thing about this
	
	rhythmic position of eighth note pairs -- 2 and 4 only?  Where are they in triple meters?
	
"""

def random_piece(source = 'PALRAW'):
	gather_stats(source, fileNumber = -1)

def gather_stats(source = 'PALRAW', fileName = '', fileNumber = None, debug = False, subpiece = None, showReduction = False, replace = False):
	global progs, piece, globalSource, locNames, globalConsonanceList, nhtCounts, subpieceNumber, globalPiece, dissonantAttacks, suspensionFormations, nhtLocations
	startT = time.time()
	globalSource = source.upper()
	if replace or (fileName) or (fileNumber is not None):
		progs = {}
		locNames = {}
		nhtCounts = {}
		nhtLocations = {}
		dissonantAttacks = {}
		suspensionFormations = {}
	else:
		i = load(globalSource)
		if i:
			completedPieces = max(locNames.keys())
		else:
			progs = {}
			locNames = {}
			nhtCounts = {}
			nhtLocations = {}
			dissonantAttacks = {}
			suspensionFormations = {}
	globalPiece = 1
	subpieceNumber = -1
	if globalSource not in DT._PATHS:
		print("Source not found!")
		return
	scorePath = DT._PATHS[globalSource][1]
	files = os.listdir(scorePath)
	
	forcedQuit = False
	files = sorted([x for x in files if x[-4:].lower() in ['.xml', '.krn', '.mxl', '.mid', 'midi']])
	if fileNumber == -1:
		fileName = random.choice(files)
		fileNumber = None
	for myFile in files:
		if fileName and (myFile != fileName): 
			#print myFile, fileName
			continue
		if (fileNumber is not None) and fileNumber != globalPiece: 
			globalPiece += 1
			continue
		if (not replace) and (not fileName) and (fileNumber is None) and myFile[:-4] in locNames.values():
			globalPiece = max([x for x in locNames if locNames[x] == myFile[:-4]]) + 1
			continue
		if globalSource == 'JOSQUINCOMPLETE' and myFile.count('Jos1808'): 		# this one is just too long to analyze, takes like 2-3 days or something!
			continue
		if myFile in palestrinaData:
			print('Analyzing', myFile, palestrinaData[myFile])
		else:
			print('Analyzing', myFile)
		piece = converter.parse(scorePath + myFile)
		try:
			if 'Opus' in piece.classes:
				bigOpus = piece
				theScores = bigOpus.scores
				if fileName or (fileNumber is not None):
					if subpiece is not None:
						subpieceNumber = subpiece
					else:
						subpieceNumber = random.choice(range(len(theScores)))
					print(("Subpiece", subpieceNumber))
					piece = theScores[subpieceNumber]
					locNames[globalPiece] = myFile[:-4]
					dopiece(globalPiece, debug = debug)
					globalPiece += 1
				else:
					for subpieceNumber, piece in enumerate(theScores):
						print(("Subpiece", subpieceNumber))
						dopiece(globalPiece, debug = debug)
						locNames[globalPiece] = myFile[:-4]
						dopiece(globalPiece, debug = debug)
						globalPiece += 1
			else:
				locNames[globalPiece] = myFile[:-4]
				dopiece(globalPiece, debug = debug)
				globalPiece += 1
		except KeyboardInterrupt:
			print("ENDING EARLY")
			forcedQuit = True
			break
		except:
			traceback.print_exc()
			print(("ERROR!"), sys.exc_info())
			forcedQuit = True
			#pass
		if (not forcedQuit) and (globalPiece % 10 == 0) and (not fileName) and (fileNumber is None):
			save()
	if fileName or fileNumber:
		if not forcedQuit:
			piece.show()
		if showReduction:
			make_list_of_consonances()
	else:
		if not forcedQuit:
			save()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))

def load(source = 'PALRAW', printOut = False, loadProgressions = True):
	global nhtCounts, locNames, dissonantAttacks, suspensionFormations, globalSource, nhtLocations, aptCensus
	globalSource = source.upper()
	if loadProgressions: progressions.load(globalSource)
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'rb') as inputFile:
			nhtCounts = pickle.load(inputFile)
			locNames = pickle.load(inputFile)
			dissonantAttacks = pickle.load(inputFile)
			suspensionFormations = pickle.load(inputFile)
			nhtLocations = pickle.load(inputFile)
			return True
	except:
		return False

def load_apt():
	global aptCensus
	with open(DT._DATAPATH + _THISFILE + globalSource +'APTCENSUS.pkl', 'rb') as inputFile:
		aptCensus = pickle.load(inputFile)

def save():
	with open(DT._DATAPATH + _THISFILE + globalSource +'.pkl', 'wb') as outputFile:
		pickle.dump(nhtCounts, outputFile)
		pickle.dump(locNames, outputFile)
		pickle.dump(dissonantAttacks, outputFile)
		pickle.dump(suspensionFormations, outputFile)
		pickle.dump(nhtLocations, outputFile)

def dissonant_attacks():
	global catalog
	catalog = {}
	for piece in dissonantAttacks:
		for l in dissonantAttacks[piece]:
			s = l[-1]
			if l[2] == (0, 3): continue
			if s.count("Pd+") > 0:
				catalog["Pd+"] = catalog.get("Pd+", 0) + 1
			elif s.count("S") > 0:
				catalog["S"] = catalog.get("S", 0) + 1
			elif s == '':
				catalog["Harmonic"] = catalog.get("Harmonic", 0) + 1
			else:
				catalog[s] = catalog.get(s, 0) + 1
	DT.print_dict(catalog)

def get_da(target = 'Pd'):
	global daList
	daList = []
	for piece in dissonantAttacks:
		for l in dissonantAttacks[piece]:
			if l[2] != (0, 3) and l[-1] == target:
				daList.append([piece, l])
	return daList

def dopiece(pieceNumber, printOut = False, debug = False):	
	global progs, piece, score, consonantO, timeSigs, dictOfConsonances, nhtDict, nextO, halfNoteAnticipations, _MEASUREBEAT, nhtLocations
	global resolutionTones, suspensionStarts, newO, consonantOList, chordWarning
	_MEASUREBEAT = functools.partial(DT.get_measure_number_and_beat_from_offset, piece)
	chordWarning = False
	suspensionFormations[pieceNumber] = []
	smallDebug = False
	fix_humdrum_file()
	if globalSource == 'BYRDRAW':
		fix_byrd_file()
	timeSigs = piece.flat.getTimeSignatures()
	score = [piece.parts[i].flat.getElementsByClass(['Note', 'Chord']).stream() for i in range(len(piece.parts))]
	numParts = len(score)
	partsList = range(numParts)
	masterOffsetList = [n.offset for p in score for n in p]
	newO = sorted(list(set(masterOffsetList)))
	maximumO = newO[-1]
	consonantO = {}
	resolutionTones = {}
	unanalyzedO = []
	noConsonanceO = []
	suspensionStarts = {}
	halfNoteAnticipations = {}
	nhtLocations[pieceNumber] = []
	if pieceNumber not in suspensionFormations:
		suspensionFormations[pieceNumber] = []
	
	for o in newO:
		myMidi = []
		myNotes = get_notes(o)
		for n in myNotes:
			n.lyric = ''
			"""if n.lyric:
				n.lyric = ''"""
		if is_consonance(myNotes):
			consonantO[o] = myNotes
	consonantOList = sorted(consonantO.keys())
	
	timePointO = sorted(list(set(newO + list(range(0, int(max(newO)))))))
	dictOfConsonances = {}
	lastHarmony = False
	
	for o in timePointO:
		if o not in newO and lastHarmony:
			dictOfConsonances[o] = lastHarmony
		elif o in consonantO:
			lastHarmony = consonantO[o]
			dictOfConsonances[o] = lastHarmony
		else:
			lastHarmony = []
			dictOfConsonances[o] = lastHarmony
	
	"""INITIAL MAIN LOOP"""
			
	for i in range(1, len(newO) - 1):
		o = newO[i]
		if o in consonantOList:
			continue
		if smallDebug: print(('round 1', o, DT.get_measure_number_and_beat_from_offset(piece, o)))
		if debug: print(('testing ', o, DT.get_measure_number_and_beat_from_offset(piece, o)))
		"""if (nextO not in consonantOList):
			mNum = DT.get_measure_number_and_beat_from_offset(piece, o)
			print("TWO DISSONANCES IN A ROW", mNum)"""
		
		success = False
		possibleInterpretation = False
		completeSuccess = False
		for tryParts in range(1, 4):
			removalParts = itertools.combinations(partsList, tryParts)
			for testParts in removalParts:
				success = find_chord(o, testParts, partsList, debug = debug)
				if success:
					finalChord = othernotes + [x[1][1] for x in nhtDict.items()]
					if debug: print(finalChord)
					if is_consonance(finalChord):
						completeSuccess = True
						break
					elif not possibleInterpretation:
						possibleInterpretation = nhtDict
			if completeSuccess:
				break
		
		if completeSuccess:
			label_nonharmonic_tones(debug = debug)
		elif possibleInterpretation:
			nhtDict = possibleInterpretation
			label_nonharmonic_tones(debug = debug)
			noConsonanceO.append(o)			
		else:
			unanalyzedO.append(o)
	
	for p in score:
		for i in range(1, len(p) - 1):
			if p[i].lyric and p[i].lyric.startswith('P') and p[i+1].lyric and p[i+1].lyric.startswith('P'):
				resolutionTones[p[i]] = p[i-1]
				resolutionTones[p[i+1]] = p[i-1]
	
	dissonantO = []
	
	for o in sorted(list(set(unanalyzedO + dissonantO))):
		if smallDebug: print(('round 2', o, DT.get_measure_number_and_beat_from_offset(piece, o)))
		mNum = DT.get_measure_number_and_beat_from_offset(piece, o)
		#print('trying to analyze', mNum)
		success = False
		nhtParts = []
		for i in partsList:
			theNotes = get_notes(o, [i])
			if not theNotes:
				continue
			theNote = theNotes[0]
			if theNote.lyric:
				if (theNote in suspensionStarts) and o < suspensionStarts[theNote]:
					pass
				else:
					nhtParts.append(i)
		if not nhtParts:
			noteList = get_notes(o, partsList)
			for n in noteList:
				if n.offset == o and (not n.lyric):
					#n.lyric = '?'
					pass
			#print("COULDN'T FIND CONSONANCE HERE", mNum, "VERTICAL SONORITY", figuredBassSetClass(noteList))
		else:
			extraNotes = get_notes(o, nhtParts)
			resTones = [resolutionTones[x] for x in extraNotes]
			partsSubset = [x for x in partsList if x not in nhtParts]
			success = False
			#print(resTones)
			for tryParts in range(1, 4):
				removalParts = itertools.combinations(partsSubset, tryParts)
				for testParts in removalParts:
					#print(" trying", o, testParts, partsSubset, resTones)
					success = find_chord(o, testParts, partsSubset, resTones)
					#print(success)
					if success:
						break
				if success:
					break
			if success:
				label_nonharmonic_tones()
				unanalyzedO.remove(o)
				#print("SECOND ORDER SUCCESS AT", mNum)
	
	"""FINAL PASS WITH A MORE LIBERAL PASSING AND NEIGHBORING CRITERION"""
	
	if debug: print((unanalyzedO, [_MEASUREBEAT(x) for x in unanalyzedO]))
	
	for i, o in enumerate(unanalyzedO):
		if smallDebug: print(('round 3', o, DT.get_measure_number_and_beat_from_offset(piece, o)))
		index = newO.index(o)
		if (index < len(unanalyzedO) + 1 and newO[index+1] in unanalyzedO) or (index > 0 and newO[index-1] in unanalyzedO):
			success = False
			possibleInterpretation = False
			completeSuccess = False
			for tryParts in range(1, 4):
				removalParts = itertools.combinations(partsList, tryParts)
				for testParts in removalParts:
					success = find_chord(o, testParts, partsList, useNextOffset = False, debug = False)
					if success:
						finalChord = othernotes + [x[1][1] for x in nhtDict.items()]
						#print(finalChord)
						if is_consonance(finalChord):
							completeSuccess = True
							break
						elif not possibleInterpretation:
							possibleInterpretation = nhtDict
				if completeSuccess:
					break
			if completeSuccess:
				label_nonharmonic_tones(debug = False)
			elif possibleInterpretation:
				nhtDict = possibleInterpretation
				label_nonharmonic_tones()
			
	dissonantO = [x for x in newO if x not in consonantOList]
	
	finalO = copy.deepcopy(consonantO)
	
	for o in dissonantO:
		finalNotes = []
		literalNotes = get_notes(o, partsList)
		for n in literalNotes:
			if n.lyric and n.lyric not in ['?', '7', '64', 'o7']:
				if n.lyric == 'S' and suspensionStarts[n] > o:
					finalNotes.append(n)
				else:
					if n in resolutionTones:
						finalNotes.append(resolutionTones[n])
					else:
						print(("NOT IN RESOLUTIONTONES, fix this", _MEASUREBEAT(o), n.pitch.nameWithOctave))
			else:
				finalNotes.append(n)
		if is_consonance(finalNotes):
			finalO[o] = finalNotes
		else:
			fbSonority = figuredBassSetClass(finalNotes)
			literalSonority = figuredBassSetClass(literalNotes)
			mySym = '?'
			if DT.normal_form(literalSonority) in [[0, 1, 5, 8], [0, 3, 5, 8], [0, 3, 6, 8], [0, 3, 6, 9], [0, 2, 5, 8], [0, 2, 5], [0, 2, 6], [0, 1, 5], [0, 4, 5], [0, 3, 5], [0, 4, 6]]:
				mySym = '7'
			elif literalSonority in [[0, 5, 9], [0, 5, 8]]:
				mySym = '64'
			elif fbSonority == [0, 3, 6] or literalSonority == [0, 3, 6]:
				mySym = 'o7'
			elif fbSonority == [0, 4, 8] or literalSonority == [0, 4, 8]:
				mySym = '+'
			else:
				if printOut: print(("STRANGE SONORITY AT", DT.get_measure_number_and_beat_from_offset(piece, o), "VERTICAL SONORITY", fbSonority))
			noteList = get_notes(o, partsList)
			for n in noteList:
				if n.offset == o and not n.lyric:
					n.lyric = mySym
				pass
			
	suspension_decorations(partsList)
	
	tempKeys = sorted(finalO.keys())
	simplifiedHarmony = {}
	lastHarmony = False
	
	for k in tempKeys:
		fbHarmony = figuredBassMidi(finalO[k])
		if  fbHarmony != lastHarmony:
			simplifiedHarmony[k] = fbHarmony
			lastHarmony = fbHarmony
	
	"""RECORD DISSONANT SIMULTANEOUS ATTACKS"""
	
	dissonantAttacks[pieceNumber] = []
	for o in newO:
		myMidi = []
		myNotes = []
		for p in score:
			tempNotes = p.getElementsByOffset(o)
			if tempNotes:
				for n in tempNotes:
					if 'Note' in n.classes:
						if (not n.tie) or (n.tie.type == 'start'): myNotes.append(n)
					elif 'Chord' in n.classes:
						for n1 in n.notes:
							if (not n1.tie) or (n1.tie.type == 'start'): myNotes.append(n1)
		if not myNotes: continue
		diatonicNotes = sorted([DT.note_to_scalarMIDIdegree(x) for x in myNotes])
		fbSetClass = tuple(sorted(list(set([(x - diatonicNotes[0]) % 7 for x in diatonicNotes]))))
		nForm = tuple(DT.normal_form([x.pitch.midi for x in myNotes], removeDuplicates = True))
		if not diatonicNotes or len(diatonicNotes) == 1: continue
		if fbSetClass not in fbDict:
			myLyrics = ' '.join([n.lyric for n in myNotes if n.lyric])
			mNum = DT.get_measure_number_and_beat_from_offset(piece, o)
			dissonantAttacks[pieceNumber].append([mNum, o, fbSetClass, nForm, myLyrics])
			if (fbSetClass != (0, 3)):
				if debug: print(" dissonant attack", mNum, o, fbSetClass, nForm, myLyrics)
	
	for o in sorted(suspensionEnds.keys()):
		mNum = DT.get_measure_number_and_beat_from_offset(piece, o)
		startNotes = [x.pitch.midi for x in get_notes(o)]
		endNotes = sorted([x.pitch.midi for x in get_notes(suspensionEnds[o])])
		endNoteOffset = endNotes[0]
		endNotes = sorted(list(set([(x - endNoteOffset) % 12 for x in endNotes])))
		bassNote = min(startNotes)
		startNotes = sorted(list(set([(x - bassNote) % 12 for x in startNotes])))
		endNotes = [(x - bassNote + endNoteOffset) % 12 for x in endNotes]
		theDuration = suspensionEnds[o] - o
		suspensionFormations[pieceNumber].append([o, theDuration, startNotes, endNotes])
		if debug: print(" ", mNum, "suspension duration", theDuration, startNotes, endNotes)
		
	"""for i, c in sorted(simplifiedHarmony.items()):
		print DT.get_measure_number_and_beat_from_offset(piece, i), c"""
	
	for partNumber, p in enumerate(score):
		for n in p:
			if n.lyric:
				l = n.lyric
				if l == 'S' and n in suspensionStarts:
					o = suspensionStarts[n]
				else:
					o = n.offset
				nhtLocations[pieceNumber].append([partNumber, l, o])

def replace_list_member(l, i, j):
	return [j if x == i else x for x in l]

def convert_chromatic_to_diatonic(myList):
	replaceList = [[3, 4], [8, 9]]
	for i, j in replaceList:
		myList = replace_list_member(myList, i, j)
	return myList

def make_list_of_consonances():
	global simplifiedOutput
	simplifiedOutput = stream.Score()
	scoreLen = len(score)
	figuredBassClasses = {}
	for i in range(scoreLen):
		simplifiedOutput.insert(0, stream.Part())
	lastOutput = False
	for o in consonantOList:
		myNotes = get_notes(o)
		partsList = whereTheNotesCameFrom
		theNotes = zip(partsList, myNotes)
		noteDict = {x[0]:x[1] for x in theNotes}
		tempOutput = [0] * scoreLen
		for i in range(scoreLen):
			if i not in noteDict:
				tempOutput[i] = note.Rest(quarterLength = 1)
			else:
				tempOutput[i] = note.Note(noteDict[i].pitch, quarterLength = 1)
		if tempOutput != lastOutput:
			for i in range(scoreLen):
				simplifiedOutput[i].append(tempOutput[i])
			lastOutput = tempOutput
			if len(myNotes) == 3:
				fb = figuredBassSetClass(myNotes)
				fb = tuple(convert_chromatic_to_diatonic(fb))
				figuredBassClasses[fb] = figuredBassClasses.get(fb, 0) + 1
	for f, c in sorted(figuredBassClasses.items(), key = lambda x: -x[1]):
		print(f, c)
	simplifiedOutput.show()

def find_chord(o, testParts, partsList, extraNotes = [], debug = False, useNextOffset = True):
	global nhtDict, newO, nextO, prevO, othernotes
	i = newO.index(o)
	prevO = newO[i-1]
	nextO = newO[i+1]
	otherparts = [x for x in partsList if x not in testParts]
	othernotes = get_notes(o, otherparts) + extraNotes
	if debug:
		print((' midi', [x.pitch.midi for x in othernotes], is_consonance(othernotes)))
	if not(is_consonance(othernotes)):
		if is_consonance(othernotes, upperVoiceConsonanceList):
			myNotes = get_notes(o, testParts)
			myMIDI = [x.pitch.midi for x in myNotes]
			otherMIDI = [x.pitch.midi for x in othernotes]
			if debug: print((' midi2', myMIDI, otherMIDI))
			if myMIDI and otherMIDI and min(myMIDI) >= min(otherMIDI):
				return False
		else:
			return False
	success = True
	nhtDict = {}
	if debug: print((" TESTPARTS", testParts))
	for j in testParts:
		success = success and check_for_nht(o, j, useNextOffset)
		if debug: print(('  part', j, success))
		if not success:
			return False
	if debug: print(nhtDict)
	return success

def label_nonharmonic_tones(debug = False):
	global nhtCounts
	for n, l in nhtDict.items():
		n.lyric = l[0]
		nhtCounts[l[0]] = nhtCounts.setdefault(l[0], 0) + 1
		if debug:
			print(n, n.lyric, l[1])
		resolutionTones[n] = l[1]

def check_for_nht(o, i, useNextOffset = True):
	global nhtDict
	passingAndNeighboringFunc = functools.partial(check_for_passing_and_neighboring, useNextOffset = useNextOffset)
	"""EXPLORE SWITCHING SUSPENSION AND PASSINGANDNEIGHBORING HERE?
	in the earliest, passing and neighboring came first; for useNextOffset = False, this might not be the way to go
	"""
	for f in [check_for_suspension_and_anticipation, passingAndNeighboringFunc, check_for_cambiata]:
		foundNHT = f(o, i)
		if foundNHT:
			nhtDict[foundNHT[0]] = foundNHT[1:]
			return True
	return False

def check_for_suspension_and_anticipation(o, i):			
	global score, halfNoteAnticipations, piece
	
	curnote = get_notes(o, [i])
	if not curnote:
		return False
	curnote = curnote[0]
	nextnote = get_notes(curnote.offset + curnote.duration.quarterLength, [i])
	if not nextnote:
		return False
	nextnote = nextnote[0]
	prevnote = get_notes(o - .05, [i])
	if not prevnote:
		return False
	prevnote = prevnote[0]
	
	curBeatStrength = get_beat_strength(o)
	nextBeatStrength = get_beat_strength(curnote.offset + curnote.duration.quarterLength)
	nextInt = curnote.pitch.midi - nextnote.pitch.midi
	
	prevInt = curnote.pitch.midi - prevnote.pitch.midi
	
	if curBeatStrength < nextBeatStrength:		# is there a more principled way to do this?
		if nextInt == 0 and prevInt in [-1, -2]:
			if curnote.duration.quarterLength <= _MAXANTICIPATIONLENGTH:
				return [curnote, "A", prevnote]
			elif curnote not in halfNoteAnticipations:
				halfNoteAnticipations[curnote] = [prevnote, nextnote]
				#print("offfset", o)
				#print(("possible half note anticipation here", DT.get_measure_number_and_beat_from_offset(piece, o)))
				return False
		else:
			return False
			
	if nextInt not in [1, 2]:
		return False
	if (o - prevnote.offset) < (curnote.offset + curnote.duration.quarterLength - o) or prevInt != 0:
		return False
	suspensionStarts[curnote] = o
	return [curnote, "S", nextnote]

def suspension_decorations(partsList = []):
	global suspensionStarts, whereTheNotesCameFrom, nhtDict, suspensionEnds
	
	nhtDict = {}
	suspensionEnds = {}
	suspensionOffsets = list(set(suspensionStarts.values()))
	
	for o in suspensionOffsets:
		theNotes = get_notes(o, partsList)
		noteLocations = whereTheNotesCameFrom[:]
		for index, curnote in enumerate(theNotes):
			if (not curnote.lyric) or (curnote.lyric != 'S'):
				continue
			i = noteLocations[index]
			nextNotes = [curnote]
			suspensionEnds[o] = curnote.offset + curnote.duration.quarterLength
			for j in range(3):
				n = get_notes(nextNotes[-1].offset + nextNotes[-1].duration.quarterLength, [i])
				if not n:
					break
				nextNotes.append(n[0])
			if len(nextNotes) <= 2:
				continue
			nextNotes = nextNotes[1:]
			nextBeatStrengths = [get_beat_strength(n.offset) for n in nextNotes]
			curBeatStrength = get_beat_strength(o)
			if nextNotes[1].pitch.midi == nextNotes[0].pitch.midi and (nextBeatStrengths[0] < nextBeatStrengths[1]) and (nextBeatStrengths[1] <= curBeatStrength
				) and nextNotes[0].duration.quarterLength == (nextNotes[0].offset - o):
				nhtDict[nextNotes[0]] = ['As1', nextNotes[1]]
				suspensionEnds[o] = nextNotes[1].offset
				continue
			if len(nextNotes) <= 2:
				continue
			"""in this next check, we need to account for decorations like A->(G, F#)->G#"""
			if ((nextNotes[2].pitch.midi == nextNotes[0].pitch.midi and ((nextNotes[0].pitch.midi - nextNotes[1].pitch.midi) in [1, 2])) or
			    (nextNotes[2].pitch.midi == (nextNotes[0].pitch.midi + 1) and ((nextNotes[0].pitch.midi - nextNotes[1].pitch.midi) == 1))
				) and (nextNotes[0].duration.quarterLength == nextNotes[1].duration.quarterLength) and ((2*nextNotes[0].duration.quarterLength) == (nextNotes[0].offset - o)
				) and (nextBeatStrengths[2] <= curBeatStrength) and nextBeatStrengths[1] < nextBeatStrengths[0] and nextBeatStrengths[0] < nextBeatStrengths[2]:
				nhtDict[nextNotes[0]] = ['As2', nextNotes[2]]
				nhtDict[nextNotes[1]] = ['Nd', nextNotes[0]]
				suspensionEnds[o] = nextNotes[2].offset
				continue
	
	label_nonharmonic_tones()
	
				
def check_for_passing_and_neighboring(o, i, useNextOffset = True):			
	"""
	
	In looking for passing and neighboring tones, there are two possibilities:
	
			1. determine the previous and next note by the offset at which any voice in the piece attacks; 
			2. or consider the previous and next attacks of the current voice we are looking at.
	
	Option #1 is much less susceptible to false positives, so it is worth using in a first pass analysis; however it will not pick up certain nested passing tone
	formulations; option #2 will get those but it picks up more junk.
	
	I am currently using option #1 on the first pass, option #2 on the second.
	
	"""
	curnote = get_notes(o, [i])
	if not curnote:
		return False
	curnote = curnote[0]
	if useNextOffset:
		nextnote = get_notes(nextO, [i])
	else:
		nextnote = get_notes(curnote.offset + curnote.duration.quarterLength, [i])
	if not nextnote:
		return False
	nextnote = nextnote[0]
	secondInt = nextnote.pitch.midi - curnote.pitch.midi
	if abs(secondInt) not in [1, 2]:
		return False
	if useNextOffset:
		prevnote = get_notes(o - .05, [i])
	else:
		prevnote = get_notes(curnote.offset - .05, [i])
	if not prevnote:
		return False
	prevnote = prevnote[0]
	firstInt = curnote.pitch.midi - prevnote.pitch.midi
	if abs(firstInt) not in [1, 2]:
		return False
	index = int(round((o % (2 * curnote.duration.quarterLength)/curnote.duration.quarterLength), 1) * 10)
	addString = ''
	if index == 0:
		ptResolution = nextnote
		addString = '+'
	else:
		ptResolution = prevnote
	if (float(firstInt)/secondInt) > 0:
		if firstInt > 0:
			return [curnote, "Pa" + addString, ptResolution]
		else:
			return [curnote, "Pd" + addString, ptResolution]
	else:
		if firstInt < 0:
			return [curnote, "Nd" + addString, nextnote]
		else:
			return [curnote, "Na" + addString, nextnote]

def check_for_cambiata(o, i):
	"""next offset here?"""
	curnote = get_notes(o, [i])
	if not curnote:
		return False
	curnote = curnote[0]
	nextnote = get_notes(nextO, [i])
	if not nextnote:
		return False
	nextnote = nextnote[0]
	secondInt = nextnote.pitch.midi - curnote.pitch.midi
	if secondInt not in [-3, -4]:
		return False
	prevnote = get_notes(o - .05, [i])
	if not prevnote:
		return False
	prevnote = prevnote[0]
	firstInt = curnote.pitch.midi - prevnote.pitch.midi
	if firstInt not in [-1, -2]:
		return False
	thirdNote = get_notes(nextnote.offset + nextnote.duration.quarterLength, [i])
	if not thirdNote:
		return False
	thirdNote = thirdNote[0]
	thirdInt = thirdNote.pitch.midi - nextnote.pitch.midi
	if thirdInt not in [1, 2]:
		return False
	return [curnote, "Camb", prevnote]
	
def get_beat_strength(offset):
	global piece, timeSigs
	measure = piece.parts[0].getElementsByOffset(offset + .01, offset + .02, includeEndBoundary = True, mustBeginInSpan = False, classList = ['Measure'])
	if not measure:
		return False
	mStart = measure[0].offset
	mOffset = offset - mStart
	timeSig = timeSigs.getElementAtOrBefore(offset)
	return timeSig.getAccentWeight(mOffset % timeSig.barDuration.quarterLength, forcePositionMatch = True)

def get_notes(o, partsList = []):
	global score, whereTheNotesCameFrom, chordWarning
	myNotes = []
	whereTheNotesCameFrom = []
	if not partsList:
		partsList = range(len(score))
	for i in partsList:
		p = score[i]
		tempNotes = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False)
		if tempNotes:
			for n in tempNotes:
				if 'Note' in n.classes:
					myNotes.append(n)
					whereTheNotesCameFrom.append(i)
				elif 'Chord' in n.classes:
					if not chordWarning:
						print("     CONSIDER REMOVING THIS FILE!  IT CONTAINS CHORDS!")
						chordWarning = True
					for n1 in n.pitches:
						myNotes.append(note.Note(n1, quarterLength = n.quarterLength))
						whereTheNotesCameFrom.append(i)
	return myNotes

#cadencesList = [['[7, 11, 2] *[0, 0, 4, 7]', 1, 8, 7, 56.0], ['[11, 2, 7] [0, 0, 3, 7]', 1, 11, 2, 82.0], ['[7, 11, 2] [9, 0, 0, 5]', 1, 12, 2, 92.0], ['[7, 11, 2] [9, 0, 0]', 1, 21, 2, 160.0], ['[7, 11, 2] [0, 0, 0, 3, 7]', 1, 27, 2, 208.0], ['[7, 11, 2] *[0, 0, 4, 7]', 1, 34, 2, 264.0], ['[11, 2, 7] *[0, 4, 7]', 1, 36, 0, 284.0], ['[7, 11, 2] [0, 0, 4, 7]', 1, 44, 7, 344.0]]


def cadence_distances(separateSuspensions = False, minLength = 50):
	global nhtDistances
	nhtDistances = {}
	for i in locNames:
		newL = cadences.locNames[i].split('***}')[-1]
		if locNames[i] != newL:
			print("LOCNAME MISMATCH", i, locNames[i], cadences.locNames[i])
		else:
			tempDistances = distance_to_cadence(i, separateSuspensions = separateSuspensions)
			if tempDistances is None:
				print(i, tempDistances, distance_to_cadence(i))
			for nType, nList in tempDistances.items():
				nhtDistances.setdefault(nType, [0, 0])[0] += sum(nList)
				nhtDistances[nType][1] += len(nList)
	print("TOTAL PIECES", i)
	for nType, nCounts in nhtDistances.items():
		if nCounts[1] < minLength:
			del nhtDistances[nType]
		else:
			nhtDistances[nType] = nCounts[0]/float(nCounts[1])
	DT.print_dict(nhtDistances)

def distance_to_cadence(pieceNumber = 1, separateSuspensions = False):
	nhtDistances = {}
	if cadences.globalSource != globalSource:
		cadences.load(globalSource)
	cadencesList = cadences.get_cadences(pieceNumber)
	cadenceOffsets = sorted([x[-1] for x in cadencesList])
	if not cadenceOffsets:
		return {}
	if separateSuspensions:
		nhtList = suspensionFormations[pieceNumber]
		for n in nhtList:
			nOffset, nLength, nStart, nEnd  = n
			nType = tuple([tuple(nStart), tuple(nEnd)])
			for o in cadenceOffsets:
				if o > nOffset: break
			distToCadence = o - nOffset
			if distToCadence >= 0:
				#print nOffset, o, distToCadence
				nhtDistances.setdefault(nType, []).append(distToCadence)
		return nhtDistances
	
	else:	
		nhtList = nhtLocations[pieceNumber]
		for n in nhtList:
			nPart, nType, nOffset = n
			for o in cadenceOffsets:
				if o >= nOffset: break
			distToCadence = o - nOffset
			#print nOffset, o, distToCadence
			if distToCadence >= 0:
				nhtDistances.setdefault(nType, []).append(distToCadence)
		return nhtDistances

susTargets = [((0, 5, 7), (0, 4, 7)), ((0, 6, 9), (0, 4, 7)), ((0, 2, 4, 7), (0, 4, 7)), ((0, 3, 10), (0, 3, 8))]
def nht_graph(targets = ['P', 'S', 'N'], ticks = 16, ticksize = 4, norm = True):
	"""graphs either NHTs or suspensions as well"""
	global nhtDistances
	cadences.load(globalSource)
	
	if targets and type(targets[0]) is tuple:
		separateSuspensions = True
		titleWord = 'Suspensions'
	else:
		separateSuspensions = False
		titleWord = 'Nonharmonic tones'
	nhtDistances = {}
	
	for i in locNames:
		newL = cadences.locNames[i].split('***}')[-1].replace('.krn', '').replace('.xml', '').replace('.mxl', '')
		if locNames[i] != newL:
			print("LOCNAME MISMATCH", i, locNames[i], cadences.locNames[i])
		else:
			tempDistances = distance_to_cadence(i, separateSuspensions = separateSuspensions)
			for nType, nList in tempDistances.items():
				for n in nList:
					nhtDistances.setdefault(nType, [0] * (ticks))[min(int(n / ticksize), int(ticks - 1))] += 1
	
	fig = plt.figure()
	ax = fig.add_subplot()
	plt.xlabel('Distance from cadence (in whole notes)')
	plt.ylabel('% of ' + titleWord.lower() + ' of that type')
	plt.title(titleWord + ' by phrasal position')
	labels = []
	if not targets:
		targets = nhtDistances.keys()
	
	for nType in targets:
		nList = nhtDistances.get(nType, [])
		if not nList:
			nList = [0] * ticks
			tempList = [x for x in nhtDistances.keys() if x.startswith(nType)]
			for i in tempList: nList = [sum(x) for x in zip(nList, nhtDistances[i])]
		if (not nList) or sum(nList) == 0: continue
		labels.append(str(nType))
		if norm:
			nList = [x * 100./sum(nList) for x in nList]
		plt.plot(nList[::-1], label = labels[-1])
	ticks = range(ticks)
	plt.xticks(ticks, [str(x)+'-'+str(x+1) for x in ticks[::-1]])
	myLeg = plt.legend(tuple(labels), loc=0)	# 0 = best; 4 = lower right, 7 = center right
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.show()	

def show(suspension, rangeList = [-1, -1]):
	global output, locNames, globalSource, tempChorale
	output = stream.Score()
	if type(rangeList) is int:
		rangeList = [rangeList, -1]
	elif len(rangeList) == 1:
		rangeList.append(-1)
	tempOutput = stream.Score()
	lastChoraleNo = False
	if type(suspension) is str:
		locList = get_nht_locations(suspension)
	else:
		locList = suspensionLocations.get(suspension, [])
	locList = [x[0:2] for x in locList]
	if not locList: return
	for i in range(0, 6):
		tempOutput.append(stream.Part())
	for chordNo, instance in enumerate(locList):
		if chordNo < rangeList[0] or (rangeList[1] != -1 and chordNo > rangeList[1]): continue
		if locNames[instance[0]] != lastChoraleNo:
			choraleNo = locNames[instance[0]]
			lastChoraleNo = choraleNo
			try:
				chorale = converter.parse(DT._PATHS[globalSource][1] + choraleNo + '.xml')
			except:
				chorale = converter.parse(DT._PATHS[globalSource][1] + choraleNo + '.krn')
		offset = instance[1]
		targetMeasure = DT.get_measure_number_and_beat_from_offset(chorale, instance[1])
		if not targetMeasure: continue
		targetMeasure = targetMeasure[0]
		measureList = [targetMeasure - 1, targetMeasure + 1]
		header = str(locNames[instance[0]])+'m'+str(measureList[0])
		print('getting', instance[0], locNames[instance[0]], measureList)
		tempChorale = copy.deepcopy(DT.get_measures(chorale, int(measureList[0]), int(measureList[1])))
		if not tempChorale or len(tempChorale.parts) == 0 or len(tempChorale.parts[0]) == 0: continue
		DT.insert_text(tempChorale.parts[0][0], header, 55, 0)
		output = DT.append(tempChorale, output)
	for p in output.parts:
		for item in p.flat:
			if 'Note' in item.classes or 'Chord' in item.classes: item.lyrics = []
	output.show()
	return output

def get_nht_locations(s):
	outlist = []
	for i, p in nhtLocations.items():
		for l in p:
			if l[1] == s:
				outlist.append([i, l[-1]])
	return outlist
	

def is_consonance(myNotes, theConsonances = globalConsonanceList):
	if not myNotes:
		return False
	return figuredBassSetClass(myNotes) in theConsonances

def figuredBassSetClass(myNotes):
	sortedNotes = sorted([x.pitch.midi for x in myNotes])
	return sorted(list(set([(x - sortedNotes[0]) % 12 for x in sortedNotes])))
	
def figuredBassMidi(myNotes):
	sortedNotes = sorted([x.pitch.midi for x in myNotes])
	tempList = sorted(list(set([(x - sortedNotes[0]) % 12 for x in sortedNotes])))
	return [(x + sortedNotes[0]) % 12 for x in tempList]

def fix_humdrum_file():
	theMeasures = [p.getElementsByClass('Measure') for p in piece.parts]
	measureLengths = [len(x) for x in theMeasures]
	maxLength = max(measureLengths)
	longestPart = measureLengths.index(maxLength)
	for i, p in enumerate(piece.parts):
		if i == longestPart: continue
		for j in range(measureLengths[i], maxLength):
			m = theMeasures[longestPart][j]
			ts = m.getElementsByClass("TimeSignature")
			newTS = False
			newM = stream.Measure()
			if ts:
				newTS = meter.TimeSignature(ts[0].ratioString)
				newM.insert(ts[0].offset, newTS)
			newM.insert(note.Rest(quarterLength = m.highestTime))	
			p.insert(m.offset, newM)
	
def fix_byrd_file():
	for i, p in enumerate(piece.parts):
		theTS = []
		for r in p.getElementsByOffset(0, classList = ['Rest']):
			if r.quarterLength == 1:
				p.remove(r)
			else:
				r.quarterLength = r.quarterLength - 1
		for n in p:
			if n.offset >= 1:
				n.offset = n.offset - 1
			if 'TimeSignature' in n.classes:
				theTS.append(n)
		for t in theTS:
			p.remove(t)

def print_nht():
	global nhts
	nhts = {}
	for n, count in nhtCounts.items():
		if n == 'A':
			nhts['A*'] = count
		else:
			nhts[n[0]] = nhts.setdefault(n[0], 0) + count
	DT.print_dict(nhts, pct = True)

seventhChordSuspensions = 	[((0, 7, 9), (2, 6, 9)), ((0, 4, 7, 9), (2, 6, 9)), ((0, 7, 9), (2, 5, 9)), ((0, 4, 7, 9), (2, 5, 9)), 
							((0, 3, 7, 9), (2, 6, 9)), ((0, 7, 8), (2, 5, 8)), ((0, 3, 7, 9), (2, 5, 9))]

def catalog_suspensions(printOut = True, top = 25):
	global suspensions, suspensionLocations, exceptions
	suspensions = {}
	suspensionLocations = {}
	sameBass = [0, 0]
	bassSus = 0
	seventhChords = 0
	exceptions = {}
	for i in suspensionFormations:
		for l in suspensionFormations[i]:
			t = tuple([tuple(x) for x in l[2:]])
			sameBass[1] += 1
			if t[0][0] == t[1][0]: 
				sameBass[0] += 1
			elif t[0][1] in [1, 2] and t[1][0] in [10, 11]:
				bassSus +=1
			elif t in seventhChordSuspensions:
				seventhChords += 1
			else: 
				exceptions[t] = exceptions.get(t, 0) + 1
			suspensions[t] = suspensions.get(t, 0) + 1
			suspensionLocations.setdefault(t, []).append([i] + l)
	if printOut:
		s = sum(suspensions.values())
		vals = sorted(suspensions.items(), key = lambda x: -x[1])
		for i in vals[:top]:
			print(i[0], round(i[1]*100./s, 1))
		#DT.print_dict(suspensions, pct = True)
		print("SUSPENSIONS OVER A FIXED BASS", round(sameBass[0] * 100./sameBass[1], 0))
		print("BASS VOICE SUSPENSIONS", round(bassSus * 100./sameBass[1], 0))
		print("SEVENTH CHORD IDIOMS", round(seventhChords * 100./sameBass[1], 0))

chromaticToDiatonic = {0:1, 1:2, 2:2, 3:3, 4:3, 5:4, 6:4.5, 7:5, 8:6, 9:6, 10:7, 11:7}

def diatonic_suspensions(pct = True):
	global diatonicSuspensions
	diatonicSuspensions = {}
	catalog_suspensions(False)
	for i, count in suspensions.items():
		newaddress = [tuple([chromaticToDiatonic[y] for y in x]) for x in i]
		newaddress = tuple([x if x != (2, 4.5, 6) else (2, 4, 6) for x in newaddress])
		#print i, newaddress, count
		diatonicSuspensions[newaddress] = diatonicSuspensions.get(newaddress, 0) + count
	DT.print_dict(diatonicSuspensions, pct = pct)

def semitones():
	global output
	output = {}
	catalog_suspensions(False)
	for s, count in suspensions.items():
		for i in range(0, 10):
			chord1 = s[0]
			if i in chord1 and i+1 in chord1:
				output[s] = count
	DT.print_dict(output)

notePCs = {'A': 9, 'B-': 10, 'B': 11, 'C': 0, 'C#': 1, 'D': 2, 'E-': 3, 'E': 4, 'F': 5, 'F#': 6, 'G': 7, 'G#': 8}
qualitiesToSets = {	'RM': (0, 4, 7), 'Rm': (0, 3, 7), 'Rd': (0, 3, 6), 'RA': (0, 4, 8), 'TM': (0, 3, 8), 'Tm': (0, 4, 9), 'Td': (0, 3, 9), 'TA': (0, 4, 8), 
					'FM': (0, 5, 9), 'Fm': (0, 5, 8), 'Fd': (0, 6, 9),'FA': (0, 4, 8), '1P': (0,), '3M': (0, 4), '3m': (0, 3), '5P': (0, 7), '6m': (0, 8), '6M': (0, 9)}

def get_next_sonority(target = [1, 52.0, 2.0, [0, 5, 7], [0, 4, 7]]):
	"""BUILD A FILTER HERE TO GET RID OF REPETITIONS OF THE RESOLUTION SONORITY"""
	pieceNumber, startOffset, duration, startSonority, endSonority = target
	endSonority = tuple(endSonority)
	endOffset = startOffset + duration
	#pieceNumber = locnumber_translator(pieceNumber)				# should be unnecessary soon!
	if not pieceNumber:
		return False
	rawSonorities = progressions.fullProgs[pieceNumber]
	foundResolution = 0
	nextChord = False
	for i, l in enumerate(rawSonorities):
		if l[0] == endOffset:
			foundResolution = i
			suspensionResolution = l[1]
			suspensionBassPC = notePCs.get(suspensionResolution[:-2], None)
			if suspensionBassPC is None:
				if suspensionResolution != '?':
					print("Can't find suspension bass pc", suspensionResolution, suspensionResolution[:-2])
				return False
		else:
			if foundResolution:
				if l[1] != '?':
					nextChord = l[1]
					bassAdjustment = suspensionBassPC - endSonority[0]
					resolutionBassPC = notePCs.get(nextChord[:-2], None)
					if resolutionBassPC is None: 
						print("Can't find resolution bass pc", nextChord, nextChord[:-2])
						return False
					resolutionBassPC = (resolutionBassPC - bassAdjustment) % 12
					nextChord = qualitiesToSets.get(nextChord[-2:], nextChord[-2:])
					"""if type(nextChord) is not str:
						nextChord = tuple([(x + bassAdjustment) % 12 for x in nextChord])"""
					if not ((resolutionBassPC == endSonority[0]) and (set(nextChord) <= set(endSonority))):
						break
					else:
						nextChord = False
	if (not foundResolution) or (not nextChord): 
		return False
	return [resolutionBassPC, nextChord]

def get_previous_sonority(target = [1, 52.0, 2.0, [0, 5, 7], [0, 4, 7]]):
	"""BUILD A FILTER HERE TO GET RID OF REPETITIONS OF THE RESOLUTION SONORITY"""
	pieceNumber, startOffset, duration, startSonority, endSonority = target
	endSonority = tuple(endSonority)
	endOffset = startOffset + duration
	#pieceNumber = locnumber_translator(pieceNumber)				# should be unnecessary soon!
	if not pieceNumber:
		return False
	rawSonorities = progressions.fullProgs[pieceNumber]
	foundResolution = 0
	nextChord = False
	lastChord = False
	for i, l in enumerate(rawSonorities):
		if l[0] != startOffset:
			lastChord = l[1]
		else:
			suspensionResolution = l[1]
			suspensionBassPC = notePCs.get(suspensionResolution[:-2], None)
			if suspensionBassPC is None:
				if suspensionResolution != '?':
					print("Can't find suspension bass pc", suspensionResolution, suspensionResolution[:-2])
				return False
			else:
				bassAdjustment = suspensionBassPC - startSonority[0]
				resolutionBassPC = notePCs.get(lastChord[:-2], None)
				if resolutionBassPC is None: 
					print("Can't find resolution bass pc", lastChord, lastChord[:-2])
					return False
				resolutionBassPC = (resolutionBassPC - bassAdjustment) % 12
				nextChord = qualitiesToSets.get(lastChord[-2:], lastChord[-2:])
				"""if type(nextChord) is not str:
					nextChord = tuple([(x + bassAdjustment) % 12 for x in nextChord])"""
				return [resolutionBassPC, nextChord]


consolidateDict = {	(0, 3, 7): (0, 2, 4), (0, 4, 7): (0, 2, 4), (0, 7): (0, 2, 4), (0, 4): (0, 2, 4), (0, 3): (0, 2, 4), (0,): (0, 2, 4),
					(0, 3, 9): (0, 2, 5), (0, 3, 8): (0, 2, 5), (0, 4, 9): (0, 2, 5), (0, 8): (0, 2, 5), (0, 9): (0, 2, 5),
					(0, 5, 9): (0, 3, 5), (0, 5, 8): (0, 3, 5), (0, 5): (0, 3, 5)}	

def after_resolutions(theSus, pct = True, consolidate = True):
	global successorDict
	successorDict = {}	
	if type(theSus[0]) is int:
		susTargets = [x for x in suspensionLocations if x[0] == theSus]
	else:
		susTargets = [theSus]
	for mySus in susTargets:
		print(mySus, "total count", suspensions[mySus])
		for l in suspensionLocations[mySus]:
			resolution = get_next_sonority(l)
			if not resolution: continue
			if consolidate:
				newFig = consolidateDict.get(resolution[1], resolution[1])
				resolution[1] = newFig
			resolution = tuple(resolution)
			successorDict[resolution] = successorDict.get(resolution, 0) + 1
	DT.print_dict(successorDict, pct = pct)

def before_suspension(theSus, pct = True, consolidate = True):
	global successorDict
	successorDict = {}	
	if type(theSus[0]) is int:
		susTargets = [x for x in suspensionLocations if x[0] == theSus]
	else:
		susTargets = [theSus]
	for mySus in susTargets:
		print(mySus, "total count", suspensions[mySus])
		for l in suspensionLocations[mySus]:
			resolution = get_previous_sonority(l)
			if not resolution: continue
			if consolidate:
				newFig = consolidateDict.get(resolution[1], resolution[1])
				resolution[1] = newFig
			resolution = tuple(resolution)
			successorDict[resolution] = successorDict.get(resolution, 0) + 1
	DT.print_dict(successorDict, pct = pct)


duplicateNames = []
numberTranslator = {}

def locnumber_translator(n):
	global duplicateNames, numberTranslator
	
	if not duplicateNames:
		duplicateNames = [x for x in progressions.locNames.values() if progressions.locNames.values().count(x) > 1]
		inverseDict = {x[1]:x[0] for x in progressions.locNames.items() if x[1] not in duplicateNames}
		
		for n, name in locNames.items():
			if name not in duplicateNames:
				numberTranslator[n] = inverseDict[name]
			
	return numberTranslator.get(n, False)
	

def reconstruct(pieceNumber):
	global piece, score
	
	scorePath = DT._PATHS[globalSource][1]
	files = os.listdir(scorePath)
	targetFile = locNames[pieceNumber]
	possibleNames = [targetFile + x for x in ['.krn', '.xml', '.mxl', '.mid', '.midi']]
	possibleFiles = [f for f in files if f in possibleNames]
	
	if len(possibleFiles) > 1:
		print("PIECE NUMBER", pieceNumber, "AMBIGUOUS NAME", possibleFiles)
	elif len(possibleFiles) == 0:
		print("PIECE NUMBER", pieceNumber, "CAN'T FIND FILE")
		
	piece = converter.parse(scorePath + possibleFiles[0])
	
	if 'Opus' in piece.classes:
		possibleIndices = [i for i in locNames if locNames[i] == targetFile]
		subpieceNumber = pieceNumber - min(possibleIndices)
		piece = piece.scores[subpieceNumber]
	
	score = [piece.parts[i].flat.getElementsByClass(['Note', 'Chord']).stream() for i in range(len(piece.parts))]
	nhtList = nhtLocations[pieceNumber]
	
	for p, l, o in nhtList:
		n = score[p].getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False)
		n[0].lyric = l
		
def accented_passing_tones():
	
	global aptCensus 
	
	aptCensus = {}
	
	for pieceNumber in locNames:
		
		print(locNames[pieceNumber])
		
		reconstruct(pieceNumber)
		nhtList = nhtLocations[pieceNumber]
	
		for partNumber, l, o in nhtList:
			if l != 'Pd+': continue
			p = score[partNumber]
			n = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False)[0]
			i = p.index(n)
			nNumber = DT.note_to_scalarMIDIdegree(n)
			outList = []
			for j in range(max(0, i - 2), min(i + 3, len(p))):
				outList.append(tuple([DT.note_to_scalarMIDIdegree(p[j]) - nNumber, p[j].duration.quarterLength]))
			if outList[1][1] != 1.0:
				outList = outList[1:]
			outList = tuple(outList)
			#aptCensus[outList] = aptCensus.get(outList, 0) + 1
			aptCensus.setdefault(outList, []).append([pieceNumber, partNumber, o])
			

def save_apt():
	global aptCensus
	with open(DT._DATAPATH + _THISFILE + globalSource +'APTCENSUS.pkl', 'wb') as outputFile:
		pickle.dump(aptCensus, outputFile)

def clean_apts(aptCensus):
	global cleanDict
	cleanDict = {}
	
	for k, count in aptCensus.items():
		count = len(count)
		if k[1] == (0, 1.0) and k[0][0] == 1 and k[0][1] > 1.0 and k[2] == (-1, 1.0) and k[3][0] == 0:
			cleanDict[((1,), (0, 1.0), (-1, 1.0), (0,))] = cleanDict.get(((1,), (0, 1.0), (-1, 1.0), (0,)), 0) + count
		elif k[1] == (0, 1.0) and k[0][0] == 1 and k[0][1] > 1.0 and k[2] == (-1, 1.0) and k[3][0] == -2:
			cleanDict[((1,), (0, 1.0), (-1, 1.0), (-2,))] = cleanDict.get(((1,), (0, 1.0), (-1, 1.0), (-2,)), 0) + count
		elif k[1] == (0, 0.5) and k[0][0] == 1 and k[0][1] >= 1.0 and k[2] == (-1, 0.5) and k[3][0] == -2:
			cleanDict[((1,), (0, 0.5), (-1, 0.5), (-2,))] = cleanDict.get(((1,), (0, 0.5), (-1, 0.5), (-2,)), 0) + count
		elif k[2] == (0, 0.5) and k[1][0] == 1 and k[1][1] >= 1.0 and k[3] == (-1, 0.5) and (len(k) > 4 and k[4][0]) == -2:
			cleanDict[((1,), (0, 0.5), (-1, 0.5), (-2,))] = cleanDict.get(((1,), (0, 0.5), (-1, 0.5), (-2,)), 0) + count
		elif k[2] == (0, 0.5) and k[1][0] == 1 and k[1][1] >= 1.0 and k[3] == (-1, 0.5) and (len(k) > 4 and k[4][0]) == 0:
			cleanDict[((1,), (0, 0.5), (-1, 0.5), (0,))] = cleanDict.get(((1,), (0, 0.5), (-1, 0.5), (0,)), 0) + count
		elif k[1] == (0, 0.5) and k[0][0] == 1 and k[0][1] >= 1.0 and k[2] == (-1, 0.5) and k[3][0] == 0:
			cleanDict[((1,), (0, 0.5), (-1, 0.5), (0,))] = cleanDict.get(((1,), (0, 0.5), (-1, 0.5), (0,)), 0) + count
		elif k[0][0] == 2 and k[0][0] >= 1.0 and k[1] == (1, 1.0) and k[2] == (0, 1.0) and k[3] == (-1, 1.0) and (len(k) > 4 and k[4][0] == 0):
			cleanDict[((1, 1.0), (1, 1.0), (0, 1.0), (-1, 1.0), (0,))] = cleanDict.get(((1,), (1, 1.0), (0, 1.0), (-1, 1.0), (0,)), 0) + count
		else:
			cleanDict[k] = cleanDict.get(k, 0) + count
	
	DT.print_dict(cleanDict, pct = True)

def clean_apts2(aptCensus):
	global cleanDict
	cleanDict = {}
	
	for k, theList in aptCensus.items():
		count = len(theList)
		if k[0][0] == 2 and k.count((0, .5)) == 0 and k.count((0, 2.0)) == 0 :
			newK = tuple(list(k[:-1]) + [k[-1][0]])
			cleanDict[newK] = cleanDict.get(newK, 0) + count
		
	DT.print_dict(cleanDict, pct = True)

def six_four_resolutions():
	global output
	rootResolutions = 0
	output = {}
	for s, count in suspensions.items():
		if s[0] in ((0, 5, 9), (0, 5, 8)):
			if s[1] in ((0, 3, 7), (0, 4, 7)):
				output[s] = count
				rootResolutions += count
			elif s[1] in ((0, 3, 9), (0, 4, 9), (0, 3, 8)):
				output[s] = count
	print(rootResolutions * 100./sum(output.values()))

def get_piece(locNumber):
	scorePath = DT._PATHS[globalSource][1]
	if type(locNumber) is str:
		piece = converter.parse(scorePath + locNumber)
		piece.show()
		return
	locName = locNames[locNumber]
	files = os.listdir(scorePath)
	targetFile = [f for f in files if f.startswith(locName) and f[-3:] in ['xml', 'krn', 'mxl', 'mid']]
	if targetFile:
		print(targetFile)
		piece = converter.parse(scorePath + targetFile[0])
		piece.show()
				
				
		
	
	
