from __future__ import print_function
from music21 import *
import DT

try:
	import xerox
except:
	def xerox():
		pass
	setattr(xerox, 'copy', lambda x: x)
	setattr(xerox, 'paste', lambda: '')

"""
Add a constant to all measure #s in a romantext file

	This is useful for hand analyzing -- typically I use xerox and copy the problematic part of the music; the program then renumbers/rekeys the text on the clipboard.


"""

def go(fileName = '', offset = 0, transposition = '', rekey = ''):
	global globaloffset, globalTransposition, globalKey, outputString
	globaloffset = offset
	globalTransposition = transposition
	globalKey = False
	outputString = ''
	if rekey: globalKey = key.Key(fix_key(rekey))
	if fileName:
		myFile = open(fileName, 'rb')
		outName = fileName.split('.')
		outName = outName[0] + 'NEW' + '.' + outName[1]
		outFile = open(outName, 'w+')
	else:
		myFile = xerox.paste().split('\n')
	mLines = 0
	for line in myFile:
		if line == '':
			continue
		if line == '\n':
			if fileName: outFile.write(line)
			print_function(line)
			continue
		elif line[-1] == '\n': 
			line = line[:-1]
		newLine = fixline(line)
		if newLine[0] == 'm' and rekey and mLines == 0:
			mLines = 1
			resplit = newLine.split()
			newLine = resplit[0] + ' ' + rekey + ': ' + ' '.join(resplit[1:])
		print_function(newLine)
		if fileName: outFile.write(newLine + '\n')
	if fileName: 
		myFile.close()
		outFile.close()
	xerox.copy(outputString)
	return

lastKey = 'C'

def print_function(theline):
	global outputString
	print(theline)
	outputString += theline + '\n'
	

def fix_key(keyName):
	keyName = keyName.rstrip(':')
	if len(keyName) > 1:
		keyName = keyName[0] + keyName[1:].replace('b', '-')
	return keyName

def fixline(line):
	global globalTransposition, globalKey, lastKey
	if not (line[0] == 'm' or line[0:5] == 'Pedal'): return line
	symbols = line.split()
	if globalTransposition or globalKey:
		if globalTransposition and symbols[0][0:5] == 'Pedal':
			symbols[1] = DT.transpose_letter(symbols[1], globalTransposition)
		else:
			for i in range(len(symbols)):
				if symbols[i][-1] == ':':
					lastKey = fix_key(symbols[i])
					if globalTransposition:
						newSymbol = symbols[i][0] + symbols[i][1:-1].replace('b', '-')
						newSymbol = DT.transpose_letter(newSymbol, globalTransposition)
						symbols[i] = newSymbol.replace('-', 'b') + ':'
				elif globalKey:
					if symbols[i][0] in ['m', 'b'] or symbols[i][-1] == '|':
						continue
					rn = roman.RomanNumeral(symbols[i], lastKey)
					symbols[i] = DT.figure_from_chord(chord.Chord(rn.pitches), globalKey)
	result = ''
	for item in symbols:
		if item[-1] == ':' and globalKey:
			continue
		result += fixsymbol(item) + ' '
	result = remove_duplicates(result.rstrip(' '))
	return result

def remove_duplicates(line):
	tempLine = line.split()
	deleteList = []
	lastSymbol = ''
	for i in reversed(range(1, len(tempLine) - 1)):
		if tempLine[i + 1] == tempLine[i-1] and tempLine[i][0] == 'b':
			deleteList.append(i)
			deleteList.append(i + 1)
	deleteList = sorted(list(set(deleteList)))
	deleteList.reverse()
	for i in deleteList:	
		del tempLine[i]
	return ' '.join(tempLine)

def fixsymbol(symbol):
	if symbol[0] != 'm': return symbol
	if symbol.count('-') == 0: return fixunit(symbol)
	first, second = symbol.split('-')
	return fixunit(first) + '-' + fixunit(second)

def fixunit(unit):
	global globaloffset
	finalThing = ''
	if unit.count('var') > 0:
		newUnit = unit.split('var')
		unit = newUnit[0]
		finalThing = 'var' + newUnit[1]
	prefix = ''
	myNum = ''
	postfix = ''
	foundit = False
	for i in range(len(unit)):
		if unit[i].isdigit():
			myNum += unit[i]
			foundit = True
		else:
			if foundit:
				postfix += unit[i]
			else:
				prefix += unit[i]
	return prefix + str(int(myNum) + globaloffset) + postfix + finalThing
	
