from __future__ import print_function
import os
import pickle
from music21 import *
import DT

"""
Find sequences of repeated voice-leadings; kind of cool to check out.

Todo: make this work for all consonant harmonies (e.g. 3 voice examples?)

"""

def gather_stats(source = 'BACH', fileName = '', includeBass = True, minLength = 2):
	global progs, piece, globalSource, locNames, globalMinLength
	globalSource = source.upper()
	globalMinLength = minLength
	progs = {}
	locNames = {}
	if globalSource not in DT._PATHS:
		print("Source not found!")
		return
	scorePath = DT._PATHS[globalSource][1]
	files = os.listdir(scorePath)
	counter = 1
	for myFile in files:
		if myFile[-3:] in ['xml', 'krn', 'mxl']:
			if fileName and myFile != fileName: continue
			print('Analyzing', myFile)
			try:
				piece = converter.parse(scorePath + myFile)
				dopiece(counter)
			except:
				pass
			locNames[counter] = myFile[:-4]
			counter += 1
	if not fileName:
		outputFile = open(DT._DATAPATH + 'repeatedVL' + globalSource +'.pkl', 'w+')
		pickle.dump(progs, outputFile)
		pickle.dump(locNames, outputFile)
		outputFile.close()
	return

def load(source = 'BACH'):
	global progs, globalSource, locNames, OUCHReport
	globalSource = source.upper()
	with open(DT._DATAPATH + 'repeatedVL' + globalSource +'.pkl', 'rb') as inputFile:
		progs = pickle.load(inputFile)
		locNames = pickle.load(inputFile)

load('Bach')

def get_file(i):
	scorePath = DT._PATHS[globalSource][1]
	try:
		piece = converter.parse(scorePath + locNames[i] + '.xml')
	except:
		piece = converter.parse(scorePath + locNames[i] + '.krn')
	piece.show()

def dopiece(pieceNumber):	
	global progs, piece, consonantO, globalMinLength
	score = [piece.parts[i].flat.getElementsByClass('Note') for i in range(len(piece.parts))]
	offsets = [[n.offset for n in p] for p in score]
	newO = []
	for p in offsets: newO += p
	newO = sorted(list(set(newO)))
	consonantO = []
	for o in newO:
		myNotes = []
		for p in score:
			tempNotes = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False)
			if tempNotes:
				for n in tempNotes:
					myNotes.append(n.midi % 12)
		nf = DT.normal_form(list(set(myNotes)))
		if nf in [[0, 4, 7], [0, 3, 7], [0, 3, 6]]:
			consonantO.append(o)
	theVLs = {}
	for i in range(len(consonantO) - 1):
		o = consonantO[i]
		nextO = consonantO[i+1]
		keepGoing = True
		theVL = []
		for p in score:
			firstNote = p.getElementsByOffset(o + .01, o + .02, mustBeginInSpan = False)
			secondNote = p.getElementsByOffset(nextO + .01, nextO + .02, mustBeginInSpan = False)
			if (firstNote and not secondNote) or (secondNote and not firstNote):
				keepGoing = False
				break
			if (not firstNote) and (not secondNote):		# OK if one part is resting
				continue
			firstNote = firstNote[0]
			secondNote = secondNote[0]
			myInt = interval.notesToInterval(firstNote, secondNote)
			myInt = fix_int(myInt.generic.value)
			theVL.append([firstNote.midi % 12, myInt])
		if not keepGoing: 
			theVLs[o] = False
			continue
		theVL = DT.vl_normal_form(theVL)
		for l in theVL:						# convert to abstract "r, t, f" form, 0 = root, 4 = third, 7 = fifth
			if l[0] == 3:
				l[0] = 4
			if l[0] == 6:
				l[0] = 7
		theVLs[o] = theVL
	numbersToSkip = []
	vlOffsets = sorted(theVLs.keys())
	for i, o in enumerate(vlOffsets):
		if i in numbersToSkip: continue
		j = i+1
		while j < len(vlOffsets) and theVLs[vlOffsets[j]] == theVLs[o]:
			numbersToSkip.append(j)
			j += 1
		if j >= (i+globalMinLength):
			paths = [abs(x[1]) for x in theVLs[o]]
			if max(paths) != 0:
				mNum = DT.get_measure_number_and_beat_from_offset(piece, o)
				print((" FOUND ONE! measure", mNum, "total VLs", j - i, theVLs[o]))
				s = str(theVLs[o])
				if s not in progs:
					progs[s] = []
				progs[s].append([pieceNumber, mNum, j - i])
		
def fix_int(i):
	if i > 0: return i -1
	return i + 1

			
			
				
			
