from __future__ import print_function
import os, time
import random
import matplotlib.pyplot as plt
from music21 import *
import DT
import pickle
import copy

comps = ['13thc', 'OCKEGHEM', 'JOSQUINCOMPLETE', 'PALRAW', 'MONTEVERDI', 'BACH', 'ESSEN', 'SACRED HARP', 'ROCK']
comps2 = ['CORELLI', 'WTCF', 'MOZART[0]', 'BEETHOVENQUARTETS', "CHOPIN[0]"]

_THISFILE = 'REVERSALS'

"""Do large intervals get reversed?"""


useReduced = True
capture = True
directFifthsOctavesLocations = []
overlapLocations = []
crossingsLocations = []
parallelsLocations = []
overlapVoices = {}
overlapCount = 0
crossingsCount = 0
parallelsCount = 0
antiParallelsCount = 0
directFifthsOctavesCount = 0
offsetHisto = {}
lastChorale = 1
chorale = stream.Score()
output = stream.Score()


def gather_stats(source = '', corpusSource = '', partLimit = [], rangeByVoice = False, ranges = [[60, 79], [52, 74], [48, 70], [41, 62]], filterRests = False):
	global chorale, voiceData, overlapCount, directFifthsOctavesCount, directFifthsOctavesLocations, globalSource, tritoneLocs, voiceCounter, genericOverlap, genericCrossings, bachDirect, genericParallels
	global overlapLocations, crossingsLocations, crossingsCount, parallelsCount, parallelsLocations, antiParallelsLocations, antiParallelsCount, genericDirect, genericAntiparallels
	global locNames, filePath, globalRanges, singleInts, globalFilterRests, globalPartLimit
	globalPartLimit = partLimit
	startT = time.time()
	voiceData = {-1: {}}
	singleInts = {-1: {}}
	locNames = {}
	globalFilterRests = filterRests
	if not rangeByVoice:
		globalRanges = ranges
	else:
		globalRanges = False
	globalSource = source.upper()	
	if (globalSource in DT._PATHS) or corpusSource:
		if globalSource:
			analysisPath = DT._PATHS[globalSource][0]
			scorePath = DT._PATHS[globalSource][1]
		else:
			scorePath = corpus.getComposerDir(corpusSource) + '/'
			globalSource = corpusSource.upper()	
		fileStr = globalSource
		files = sorted(os.listdir(scorePath))
		#print scorePath, files
		recordResults = True
		counter = 1
		for myFile in files:
			if myFile[-3:] in ['xml', 'krn', 'abc', 'mxl']:
				locNames[counter] = myFile[:-4]
				print(myFile)
				try:
					chorale = converter.parse(scorePath + myFile)
				except:
					continue
				if rangeByVoice and len(chorale.parts) != 4: continue
				if 'Opus' in chorale.classes:
					bigOpus = chorale
					for chorale in bigOpus:
						try:
							print("  ", chorale.metadata.title)
						except: 
							pass
						try:
							dochorale(counter)
						except: 
							pass
						counter+=1
				else:
					dochorale(counter)
					counter += 1
		if -2 in voiceData:
			del voiceData[-2]
		s = ''
		if globalPartLimit:
			s = str(globalPartLimit).replace(" ", '')
		if filterRests: 
			s += "FR"
		outputFile = open(DT._DATAPATH + _THISFILE + fileStr + s + '.pkl', 'wb')
		pickle.dump(voiceData, outputFile)
		pickle.dump(singleInts, outputFile)
		outputFile.close()
		minutes = (time.time() - startT)/60.
		print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	return

def dochorale(n, xml = False):
	global chorale, voiceData, globalRanges, singleInts
	for i in range(0, len(chorale.parts)):
		if globalPartLimit and (i not in globalPartLimit):
			continue
		s = chorale.parts[i].flat.getElementsByClass(['Note', 'Rest', 'Chord'])
		s = s.getElementsNotOfClass('ChordSymbol').stream()
		myInts = make_intlist(s)
		process_intlist(i, myInts)

def process_intlist(i, myInts):
	global chorale, voiceData, globalRanges, singleInts
	myVoice = -2
	midiNotes = [x[2] for x in myInts if x]
	if not midiNotes: 
		return
	lowNote = min(midiNotes)
	highNote = max(midiNotes)
	meanNote = int((highNote - lowNote + .5) / 2) + lowNote
	if globalRanges: 
		for j, testRange in enumerate(globalRanges):
			if lowNote >= testRange[0] and highNote <= testRange[1]:
				myVoice = j
				break
	else:
		myVoice = i
	if myVoice not in voiceData:
		voiceData[myVoice] = {}
		singleInts[myVoice] = {}
	myInts.append(False)
	for j in range(1, len(myInts) - 1):
		if not myInts[j]: continue
		nextInt = myInts[j][0]
		myNote = myInts[j][2]
		normedNote = myNote - meanNote
		if myNote not in singleInts[myVoice]:
			singleInts[myVoice][myNote] = {}
		if normedNote not in singleInts[-1]:
			singleInts[-1][normedNote] = {}
		singleInts[myVoice][myNote][str(nextInt)] = singleInts[myVoice][myNote].setdefault(str(nextInt), 0) + 1
		singleInts[-1][normedNote][str(nextInt)] = singleInts[-1][normedNote].setdefault(str(nextInt), 0) + 1
		if not myInts[j-1]: continue
		prevInt = myInts[j-1][0]
		if myNote not in voiceData[myVoice]:
			voiceData[myVoice][myNote] = {}
		if normedNote not in voiceData[-1]:
			voiceData[-1][normedNote] = {}
		intStr = str(prevInt) + ' ' + str(nextInt)
		voiceData[myVoice][myNote][intStr] = voiceData[myVoice][myNote].setdefault(intStr, 0) + 1
		voiceData[-1][normedNote][intStr] = voiceData[-1][normedNote].setdefault(intStr, 0) + 1

rawCounts = {}

def make_diatonic(myDict):
	outDict = {}
	diaInts = {1: 1, 2:1, 3:2, 4:2, 5:3, 6:3, 7:4, 8: 5, 9: 5, 10:6, 11: 6, 12: 7}
	for i in diaInts.keys():
		diaInts[i+12] = diaInts[i] + 7
		diaInts[i+24] = diaInts[i] + 14
	for i in diaInts.keys():
		diaInts[-i] = -diaInts[i]
	#print diaInts
	for i in myDict:
		outDict[diaInts[i]] = outDict.setdefault(diaInts[i], 0) + myDict[i]
	return outDict

def get_pitches(myStream):
	outPitches = []
	for i in myStream:
		if 'Note' in i.classes:
			outPitches.append(i.nameWithOctave)
		elif 'Chord' in i.classes:
			outPitches.append(''.join([x.nameWithOctave for x in c.pitches]))
	return outPitches

def chord_to_topnote(c):
	return max(c)
	
def filter_ties(inStream):
	#x = get_pitches(inStream)
	#print len(x), ' '.join(x)
	for i in reversed(range(len(inStream))):
		n = inStream[i]
		if type(n) is int: continue
		m = 0
		if n.tie and n.tie.type == 'stop':
			inStream.remove(n)
	#x = get_pitches(inStream)
	#print len(x), ' '.join(x)
	return inStream


def filter_rests_and_ties(inStream):
	outStream = stream.Stream()
	lastNote = -500
	for n in inStream:
		if ('Rest' in n.classes) or n.pitch.midi == lastNote: 
			continue
		lastNote = n.pitch.midi
		outStream.append(note.Note(n.pitch))
	return outStream

def count_single_intervals(voice = -1, makeDiatonic = True, pct = True):
	global intReport
	intReport = {}
	for startNote in singleInts[voice]:
		for theIntStr in singleInts[voice][startNote]:
			theInt = int(theIntStr)
			intReport[theInt] = intReport.setdefault(theInt, 0) + singleInts[voice][startNote][theIntStr]
	del intReport[0]
	if makeDiatonic: 
		intReport = make_diatonic(intReport)
	print('\t', 'Down', '\t', 'Up')
	for i in range(1, 24):
		if pct:
			theTotal = intReport.get(-i, 0) + intReport.get(i, 0)
			theTotal = max(theTotal, 1)
			print(i, '\t', '%.1f' % (100. * intReport.get(-i, 0) / theTotal), '\t', '%.1f' % (100. * intReport.get(i, 0) / theTotal))
		else:
			print(i, '\t', intReport.get(-i, 0), '\t', intReport.get(i, 0))

def make_markov(myVoice = -1):
	global singleInts, rawCounts
	rawCounts = {}
	for theNote in singleInts[myVoice]:
		rawCounts[theNote] = 0
		for theInt in singleInts[myVoice][theNote]:
			if theInt == '0': continue
			rawCounts[theNote] += singleInts[myVoice][theNote][theInt]

def random_dictmember(myDict = rawCounts):					# key, count pairs; returns a random member of the list weighted by count
	theSum = sum([x for x in myDict.values()])
	if theSum == 0: return 0
	myNum = random.randrange(theSum) + 1
	counter = 0
	for item in myDict:
		counter += myDict[item]
		if counter >= myNum:
			return item
	return item

def make_phrase(length = 10, myVoice = -1, myRange = [-15, 15]):
	global globalSource, voiceData, singleInts, oldSingleInts, oldVoiceData
	myRange = range(myRange[0], myRange[1])
	median = myRange[0] + sum(myRange) / 2 
	noteList = []
	offset = 0
	lastNote = random_dictmember(rawCounts)
	if lastNote not in myRange: 
		offset = median
	noteList.append(lastNote + offset)
	for i in range(length):
		try:
			newInt = int(random_dictmember(oldSingleInts[myVoice][lastNote]))
		except:
			break
		lastNote = lastNote + newInt
		noteList.append(lastNote + offset)
	process_intlist(myVoice, make_intlist(noteList))

lastGlobalSource = ''

def make_second_order_phrase(length = 10, myVoice = -1):
	global globalSource, voiceData, singleInts, oldSingleInts, oldVoiceData
	noteList = []
	offset = 0
	lastNote = random_dictmember(rawCounts)
	noteList.append(lastNote + offset)
	newInt = int(random_dictmember(oldSingleInts[myVoice][lastNote]))
	lastNote = lastNote + newInt
	noteList.append(lastNote + offset)
	for i in range(length):
		lastInt = noteList[-1] - noteList[-2]
		lastNote = noteList[-1] - offset
		try:
			newInt = random_dictmember(secondOrderDict[lastNote][lastInt])
		except:
			break
		lastNote = lastNote + newInt
		noteList.append(lastNote + offset)
	process_intlist(myVoice, make_intlist(noteList))

def create_second_order_dict(myVoice = -1):
	global secondOrderDict, voiceData
	secondOrderDict = {}
	for myNote in voiceData[myVoice]:
		secondOrderDict[myNote] = {}
		for intStr in voiceData[myVoice][myNote]:
			intList = [int(x) for x in intStr.split()]
			firstInt = intList[0]
			secondInt = intList[1]
			if firstInt not in secondOrderDict[myNote]:
				secondOrderDict[myNote][firstInt] = {}
			secondOrderDict[myNote][firstInt][secondInt] = secondOrderDict[myNote][firstInt].setdefault(secondInt, 0) + voiceData[myVoice][myNote][intStr]	

def make_piece(phrases = 50, order = 1 , myVoice = -1, show = True):
	global globalSource, voiceData, singleInts, oldSingleInts, oldVoiceData, lastGlobalSource, secondOrderDict, compositeInts
	oldSingleInts = copy.deepcopy(singleInts)
	oldVoiceData = copy.deepcopy(voiceData)
	compositeInts = {}												# for a registrally neutral version
	for theNote in oldSingleInts[myVoice]:
		for theInt in oldSingleInts[myVoice][theNote]:
			compositeInts[theInt] = compositeInts.setdefault(theInt, 0) + oldSingleInts[myVoice][theNote][theInt]
	if order == 2:
		create_second_order_dict()
	voiceData = {-1: {}}
	singleInts = {-1: {}}
	if globalSource == "TEMP":
		if lastGlobalSource:
			load(lastGlobalSource)
			make_piece(phrases, myVoice, show)
			return
		else:
			print("Choose a real source!") 
			return
	lastGlobalSource = globalSource
	globalSource = "TEMP"
	make_markov()
	for i in range(phrases):
		print("phrase ", i)
		phraselength = random.randrange(10, 30)
		if order == 1:
			make_phrase(phraselength, myVoice)	
		else:
			make_second_order_phrase(phraselength, myVoice)	

def return_intervals(n1, n2):
	if type(n1) is int and type(n2) is int:
		return [n2 - n1, False, n1]
	if 'Rest' in n1.classes or 'Rest' in n2.classes or n1.expressions:
		return False
	else:
		midi = n2.pitch.midi - n1.pitch.midi
		if midi < 0: 
			sign = '-'
		else:
			sign = ''
		return [midi, sign + interval.Interval(n1, n2).name, n1.pitch.midi]

def filter_chords(myList):
	for i, n in enumerate(myList):
		if (type(n) is not int) and 'Chord' in n.classes:
			myOffset = n.offset
			myList[i] = chord_to_topnote(n)
			myList[i].offset = myOffset
	return myList

def make_intlist(myList):
	global globalFilterRests
	myList = filter_chords(myList)
	myList = filter_ties(myList)
	if globalFilterRests:
		myList = filter_rests_and_ties(myList)
	out = []
	for i in range(len(myList) - 1):
		out.append(return_intervals(myList[i], myList[i+1]))
	return out
			
def report(myNote = 60, voice = 0, thresh = 7, direction = 0, outThresh = 100000):
	global voiceData
	leapR = 0
	leapS = 0
	if myNote not in voiceData[voice]:
		print(myNote, "not found")
		return
	for item in voiceData[voice][myNote]:
		ints = [int(x) for x in item.split()]
		if ints[0] == 0 or ints[1] == 0: 
			#print "  ignoring", ints
			continue
		if abs(ints[0]) > thresh and abs(ints[1]) < outThresh:
			if direction and (1.0 * ints[0] / direction) < 0: continue
			if 1.0 * ints[0]/ints[1] > 0: 
				leapS +=  voiceData[voice][myNote][item]
				continue
			elif 1.0 * ints[0]/ints[1] < 0:
				leapR +=  voiceData[voice][myNote][item]
				continue
			print("  ignoring", ints)
	return [leapR, leapS]

def report_less(myNote = 60, voice = 0, thresh = 3, direction = 0, outThresh = 100000):
	global voiceData
	leapR = 0
	leapS = 0
	if myNote not in voiceData[voice]:
		print(myNote, "not found")
		return
	for item in voiceData[voice][myNote]:
		ints = [int(x) for x in item.split()]
		#print ints
		if ints[0] == 0 or ints[1] == 0: continue
		if abs(ints[0]) < thresh and abs(ints[1]) < outThresh:
			if direction and (1.0 * ints[0] / direction) < 0: continue
			if 1.0 * ints[0]/ints[1] > 0: 
				#print ' same', voiceData[voice][myNote][item]
				leapS +=  voiceData[voice][myNote][item]
			elif 1.0 * ints[0]/ints[1] < 0:
				#print ' reverse', voiceData[voice][myNote][item]
				leapR +=  voiceData[voice][myNote][item]
	return [leapR, leapS]

def exact_size(myNote = 60, voice = 0, intSize = 5):
	global voiceData
	leapR = 0
	leapS = 0
	if myNote not in voiceData[voice]:
		print(myNote, "not found")
		return
	for item in voiceData[voice][myNote]:
		ints = [int(x) for x in item.split()]
		if ints[0] == 0 or ints[1] == 0: continue
		if ints[0] == intSize:
			if 1.0 * ints[0]/ints[1] > 0: 
				#print ' same', voiceData[voice][myNote][item]
				leapS +=  voiceData[voice][myNote][item]
			elif 1.0 * ints[0]/ints[1] < 0:
				#print ' reverse', voiceData[voice][myNote][item]
				leapR +=  voiceData[voice][myNote][item]
	return [leapR, leapS]

def moves_to_the_mean(voice = 0, dummy1 = 0, dummy2 = 0):
	global voiceData
	bottomNote = min(voiceData[voice].keys())
	topNote = max(voiceData[voice].keys())
	mean = (topNote - bottomNote)/2.	
	closer = 0
	farther = 0
	for myNote in range(bottomNote, topNote + 1):
		curDist = abs(myNote - mean)
		for item in voiceData[voice][myNote]:
			ints = [int(x) for x in item.split()]
			newDist = abs((myNote + ints[1] - mean)/2.)
			if newDist > curDist:
				farther +=  voiceData[voice][myNote][item]
			elif newDist < curDist:
				closer +=  voiceData[voice][myNote][item]
	return [closer, farther]


def load(source = 'PALRAW'):
	global genericOverlap, genericCrossings, voiceData, genericAntiparallels, genericParallels, genericDirect, bachDirect, tritoneLocs, voiceCounter, globalRanges
	global directFifthsOctavesLocations, overlapLocations, parallelsLocations, antiParallelsLocations, locNames, filePath, globalSource, singleInts, globalFilterRests
	globalFilterRests = False
	globalRanges = False
	globalSource = source.upper()
	try:
		with open(DT._DATAPATH + _THISFILE + globalSource + '.pkl', 'rb') as inputFile:
			voiceData = pickle.load(inputFile) 
			singleInts = pickle.load(inputFile)
		return True
	except:
		return False


def test(voice = 3):
	global voiceData
	startNote = min(voiceData[voice].keys())
	endNote = max(voiceData[voice].keys())
	for i in range(startNote, endNote + 1):
		print(i)
		x = report(i, voice)
		print("REVERSED", x[0], "KEEP GOING", x[1])

def interval_histo():
	global output
	for myVoice in voiceData:
		print("VOICE", myVoice)
		output = {}
		for myNote in voiceData[myVoice]:
			for myInt in voiceData[myVoice][myNote]:
				ints = [int(x) for x in myInt.split()]
				output[ints[1]] = output.setdefault(ints[1], 0) + voiceData[myVoice][myNote][myInt]
		newKeys = sorted(output.keys(), key = lambda x: -output[x])
		for i in newKeys: 
			print(i, output[i])

def big_skip_little_skip():
	global output
	for myVoice in voiceData:
		bigFirst = 0
		littleFirst = 0
		for myNote in voiceData[myVoice]:
			for myInt in voiceData[myVoice][myNote]:
				ints = [int(x) for x in myInt.split()]
				if ints[0] > 2 and ints[1] > 2:
					if ints[0] > ints[1]:
						bigFirst += voiceData[myVoice][myNote][myInt]
					elif ints[1] > ints[0]:
						littleFirst += voiceData[myVoice][myNote][myInt]
				elif ints[0] < -2 and ints[1] < -2:
					if ints[0] > ints[1]:
						bigFirst += voiceData[myVoice][myNote][myInt]
					elif ints[1] > ints[0]:
						littleFirst += voiceData[myVoice][myNote][myInt]
		print("VOICE", myVoice, bigFirst, littleFirst)

def title_caps(myStr):
	return ' '.join([x[0].upper() + x[1:].lower() for x in myStr.split()])
	
def graph(voice = -1, thresh = 2, direction = 0, diatonic = False, title = True, labelVoice = True, stepMax = 3, outThresh = 10000, ignoreThresh = 20, xRange = [], titlesize = 12):			# string flag to average ascending and descending
	global voiceData, globalSource, lastGlobalSource
	fig = plt.figure()
	ax = fig.gca()
	if type(direction) is str:
		direction = 0
		directionFlag = True
	tempSource = fix_composers(globalSource)
	tempSource = title_caps(tempSource)
	if globalSource.upper() == 'TEMP' and lastGlobalSource:
		tempSource = "artificial " + lastGlobalSource.replace("COMPLETE", "").replace('PALRAW', 'Palestrina').replace("WTCF", "Bach")
	if voice != -1:
		plt.xlabel('MIDI note')
	else:
		plt.xlabel('MIDI note (semitones from mean)')
	plt.ylabel('Percentage of melodic reversals')
	if not direction:
		if title:
			if labelVoice:
				plt.title('Melodic reversals in ' + tempSource + ', ' + ['soprano', 'alto', 'tenor', 'bass', 'every'][voice] + ' voice, after steps and leaps of more than ' + str(thresh) + ' semitones', fontsize = titlesize)
			else:
				plt.title('Melodic reversals in ' + tempSource + ', ' 'after steps and leaps of more than ' + str(thresh) + ' semitones', fontsize = titlesize)
	else:
		direction = direction / abs(direction)
		strDirection = ['', 'ascending', 'descending'][direction]
		if title:
			plt.title('Melodic reversals in ' + tempSource + ', ' + ['soprano', 'alto', 'tenor', 'bass', 'every'][voice] + ' voice, after ' + strDirection +' steps and ' + strDirection + ' leaps of more than ' + str(thresh) + ' semitones', fontsize = titlesize)
	values = []
	values2 = []
	leapRaw = []
	stepRaw = []
	startNote = min(voiceData[voice].keys())
	endNote = max(voiceData[voice].keys())
	print(startNote, endNote)
	for i in range(startNote, endNote + 1):
		if diatonic and (i % 12) in [1, 3, 6, 8]:
			values.append(-1)
			values2.append(-1)
			continue
		if direction:
			x = report(i, voice, thresh, direction, outThresh)
			y = report_less(i, voice, stepMax, direction, outThresh)
			if x:
				leapRaw.append(sum(x))
			if y:
				stepRaw.append(sum(y))
			if not x or sum(x) == 0 or sum(x) < ignoreThresh:
				values.append(-1)
			else:
				values.append(100.*x[0]/sum(x))
			if not y or sum(y) == 0 or sum(y) < ignoreThresh:
				values2.append(-1)
			else:
				values2.append(100.*y[0]/sum(y))
		else:
			"""HERE WE TAKE AN AVERAGE OF THE TWO TENDENCIES-TO-REVERSE, ASCENDING AND DESCENDING.  DOUBLE CHECK THE LOGIC."""
			theAverages = [-1, -1, -1, -1, -1, -1]		# descending leap, descending step, ascending leap, ascending step, leaps, steps
			x = report(i, voice, thresh, -1, outThresh)
			y = report_less(i, voice, stepMax, -1, outThresh)
			z = report(i, voice, thresh, 1, outThresh)
			w = report_less(i, voice, stepMax, 1, outThresh)
			if x and y: 
				leapRaw.append(sum(x) + sum(z))
			if z and w:
				stepRaw.append(sum(y) + sum(w))
			a = 0
			b = 0
			if x:
				a = [x[j] + z[j] for j in range(len(x))]
			if y:
				b = [y[j] + w[j] for j in range(len(y))]
			for j, item in enumerate([x, y, z, w, a, b]):
				if item and (sum(item) != 0) and sum(item) >= ignoreThresh:
					theAverages[j] = 100.*item[0]/sum(item)
			if i == 0:
				print('NOTE', i)
				print('\t', [x, y, z, w, a, b])
				print('\t',theAverages)
			myLeapVal = theAverages[-2] #average_values(theAverages[0], theAverages[2])
			myStepVal = theAverages[-1] #average_values(theAverages[1], theAverages[3])
			values.append(myLeapVal)
			values2.append(myStepVal)
	myRange = range(startNote, endNote + 1)
	while values[0] == -1:
		values = values[1:]
		values2 = values2[1:]
		myRange = myRange[1:]
	while values[-1] == -1:
		values = values[:-1]
		values2 = values2[:-1]
		myRange = myRange[:-1]
	while values2[0] == -1:
		values = values[1:]
		values2 = values2[1:]
		myRange = myRange[1:]
	while values2[-1] == -1:
		values = values[:-1]
		values2 = values2[:-1]
		myRange = myRange[:-1]
	lastVal = values[0]
	for i in range(len(values)):
		if values[i] == -1:
			values[i] = lastVal
		else:
			lastVal = values[i]
	lastVal = values2[0]
	for i in range(len(values2)):
		if values2[i] == -1:
			values2[i] = lastVal
		else:
			lastVal = values2[i]
	#print values, values2
	print("RAW COUNTS (step)", sum(stepRaw), stepRaw)
	print("RAW COUNTS (leap)", sum(leapRaw), leapRaw)
	ax.plot(myRange, values, color = 'black', label = 'after leap')
	ax.plot(myRange, values2, linestyle = '--', color = 'black', label = 'after step')
	myLeg = plt.legend(('after leap', 'after step'), loc=0, fontsize = 18)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.ylim([0, 100])
	if xRange:
		plt.xlim(xRange[0], xRange[-1])
	else:
		plt.xlim(myRange[0], myRange[-1])
	plt.show()

def add_vectors(i, j):
	return [i[x] + j[x] for x in range(len(i))]

def compare_directions(voice = -1, in_interval = 5, diatonic = False, sum_intervals = False, title = True, outThresh = 10000, ignoreThresh = 20, xRange = [], titlesize = 12):			# string flag to average ascending and descending
	global voiceData, globalSource, lastGlobalSource
	if type(in_interval) is int:
		inStr = str(in_interval) + ' semitones'
		in_interval = [in_interval]
	else:
		inStr = str(in_interval) + ' semitones'
	if set(in_interval) == set([1, 2]):
		inStr = 'a step'
	elif set(in_interval) == set([3, 4]):
		inStr = 'a third'
	elif set(in_interval) == set([5, 6, 7]):
		inStr = 'a fourth or fifth'
	fig = plt.figure()
	ax = fig.gca()
	if globalSource.upper() == 'TEMP' and lastGlobalSource:
		tempSource = "Synthetic " + lastGlobalSource.replace("COMPLETE", "")
	else:
		tempSource = fix_composers(globalSource)
	if voice != -1:
		plt.xlabel('MIDI note')
	else:
		plt.xlabel('Normed MIDI note (semitones from mean)')
	plt.ylabel('Percentage of melodic reversals')
	direction = False
	if not direction:
		if title:
			plt.title('Melodic reversals in ' + tempSource + ', ' + ['soprano', 'alto', 'tenor', 'bass', 'every'][voice] + ' voice, after motions of exactly ' + inStr, fontsize = titlesize)
	values = []
	values2 = []
	leapRaw = []
	stepRaw = []
	startNote = min(voiceData[voice].keys())
	endNote = max(voiceData[voice].keys())
	for i in range(startNote, endNote + 1):
		if diatonic and (i % 12) in [1, 3, 6, 8]:
			values.append(-1)
			values2.append(-1)
			continue
		x = [0, 0]
		y = [0, 0]
		theAverages = [-1, -1, -1, -1, -1, -1]		# descending leap, descending step, ascending leap, ascending step, leaps, steps
		for j in in_interval:
			x = add_vectors(x, exact_size(i, voice, j))
			y = add_vectors(y, exact_size(i, voice, -1 * j))
		for j, item in enumerate([x, y]):
			if item and (sum(item) != 0) and sum(item) >= ignoreThresh:
				theAverages[j] = 100.*item[0]/sum(item)
		if sum_intervals:
			for j, item in enumerate([x, y]):
				if item and (sum(item) != 0) and sum(item) >= ignoreThresh:
					theAverages[j] = sum(item)
		myLeapVal = theAverages[0]
		myStepVal = theAverages[1]
		values.append(myLeapVal)
		values2.append(myStepVal)
	print("RAW COUNTS (step)", sum(values), values)
	print("RAW COUNTS (leap)", sum(values2), values2)
	megavalues = add_vectors(values, values2)
	print(megavalues)
	if sum_intervals:
		values = [100. * values[i] / megavalues[i] for i in range(len(values))]
		values2 = [100. * values2[i] / megavalues[i] for i in range(len(values2))]
	print(values)
	print(values2)
	myRange = range(startNote, endNote + 1)
	while values[0] < 0:
		values = values[1:]
		myRange = myRange[1:]
	while values[-1] < 0:
		values = values[:-1]
		myRange = myRange[:-1]
	lastVal = values[0]
	for i in range(len(values)):
		if values[i] < 0:
			values[i] = lastVal
		else:
			lastVal = values[i]
	ax.plot(myRange, values, color = 'black', label = 'ascending')
	
	myRange = range(startNote, endNote + 1)
	while values2[0] < 0:
		values2 = values2[1:]
		myRange = myRange[1:]
	while values2[-1] < 0:
		values2 = values2[:-1]
		myRange = myRange[:-1]
	lastVal = values2[0]
	for i in range(len(values2)):
		if values2[i] < 0:
			values2[i] = lastVal
		else:
			lastVal = values2[i]
	#print values, values2
	
	ax.plot(myRange, values2, linestyle = '--', color = 'black', label = 'descending')
	myLeg = plt.legend(('ascending', 'descending'), loc=0, fontsize = 18)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.ylim([0, 100])
	if xRange:
		plt.xlim(xRange[0], xRange[-1])
	else:
		plt.xlim(myRange[0], myRange[-1])
	plt.show()

def count_ascending_and_descending(voice = -1, in_interval = 5, diatonic = False, title = True, outThresh = 10000, ignoreThresh = 20, xRange = [], titlesize = 12):			# string flag to average ascending and descending
	global voiceData, globalSource, lastGlobalSource
	sum_intervals = True, 
	if type(in_interval) is int:
		inStr = str(in_interval) + ' semitones'
		in_interval = [in_interval]
	else:
		inStr = str(in_interval) + ' semitones'
	if set(in_interval) == set([1, 2]):
		inStr = 'a step'
	elif set(in_interval) == set([3, 4]):
		inStr = 'a third'
	elif set(in_interval) == set([5, 6, 7]):
		inStr = 'a fourth or fifth'
	fig = plt.figure()
	ax = fig.gca()
	if globalSource.upper() == 'TEMP' and lastGlobalSource:
		tempSource = "Synthetic " + lastGlobalSource.replace("COMPLETE", "")
	else:
		tempSource = fix_composers(globalSource)
	if voice != -1:
		plt.xlabel('MIDI note')
	else:
		plt.xlabel('Normed MIDI note (semitones from mean)')
	plt.ylabel('Percentage of melodic reversals')
	direction = False
	if not direction:
		if title:
			plt.title('Melodic reversals in ' + tempSource + ', ' + ['soprano', 'alto', 'tenor', 'bass', 'every'][voice] + ' voice, after motions of exactly ' + inStr, fontsize = titlesize)
	valuesDict = {}
	values2Dict = {}
	values = []
	values2 = []
	leapRaw = []
	stepRaw = []
	startNote = min(voiceData[voice].keys())
	endNote = max(voiceData[voice].keys())
	for i in range(startNote, endNote + 1):
		if diatonic and (i % 12) in [1, 3, 6, 8]:
			continue
		x = [0, 0]
		y = [0, 0]
		theAverages = [-1, -1, -1, -1, -1, -1]		# descending leap, descending step, ascending leap, ascending step, leaps, steps
		for j in in_interval:
			x = add_vectors(x, exact_size(i, voice, j))
			y = add_vectors(y, exact_size(i, voice, -1 * j))
			valuesDict[i - j] = valuesDict.setdefault(i-j, 0) + sum(x)
			values2Dict[i + j] = values2Dict.setdefault(i-j, 0) + sum(y)
	for i in range(startNote, endNote + 1):
		for c in [valuesDict, values2Dict]:
			if i not in c:
				c[i] = -1
		values.append(valuesDict[i])
		values2.append(values2Dict[i])
	print("RAW COUNTS (step)", sum(values), values)
	print("RAW COUNTS (leap)", sum(values2), values2)
	megavalues = add_vectors(values, values2)
	print(megavalues)
	if sum_intervals:
		values = [100. * values[i] / megavalues[i] for i in range(len(values))]
		values2 = [100. * values2[i] / megavalues[i] for i in range(len(values2))]
	print(values)
	print(values2)
	myRange = range(startNote, endNote + 1)
	while values[0] < 0:
		values = values[1:]
		myRange = myRange[1:]
	while values[-1] < 0:
		values = values[:-1]
		myRange = myRange[:-1]
	lastVal = values[0]
	for i in range(len(values)):
		if values[i] < 0:
			values[i] = lastVal
		else:
			lastVal = values[i]
	ax.plot(myRange, values, color = 'black', label = 'ascending')
	
	myRange = range(startNote, endNote + 1)
	while values2[0] < 0:
		values2 = values2[1:]
		myRange = myRange[1:]
	while values2[-1] < 0:
		values2 = values2[:-1]
		myRange = myRange[:-1]
	lastVal = values2[0]
	for i in range(len(values2)):
		if values2[i] < 0:
			values2[i] = lastVal
		else:
			lastVal = values2[i]
	#print values, values2
	
	ax.plot(myRange, values2, linestyle = '--', color = 'black', label = 'descending')
	myLeg = plt.legend(('ascending', 'descending'), loc=0, fontsize = 18)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.ylim([0, 100])
	if xRange:
		plt.xlim(xRange[0], xRange[-1])
	else:
		plt.xlim(myRange[0], myRange[-1])
	plt.show()


def average_values(a, b):
	if (a != -1 and b != -1) or (a == -1 and b == -1): return (a + b) / 2.
	if a != -1: return a
	return b

def summarize(voice = 0, thresh = 4, direction = 0, outThresh = 10000):
	global voiceData
	values = []
	values2 = []
	startNote = min(voiceData[voice].keys())
	endNote = max(voiceData[voice].keys())
	offset = (endNote - startNote) / 4
	#print startNote, endNote, offset
	for i in range(startNote + offset, endNote + 1 - offset):
		x = report(i, voice, thresh, direction, outThresh)
		y = report_less(i, voice, 3, direction, outThresh)
		#print i
		if not x or sum(x) == 0:
			continue
		else:
			leapR = 100.*x[0]/sum(x)
		if not y or sum(y) == 0:
			continue
		else:
			stepR = 100.*y[0]/sum(y)
		values.append(leapR - stepR)
	#print values
	if len(values) != 0:
		return sum(values)/len(values)
	return 0

def up_and_down(voice = -1, thresh = 2, direction = 0, theRange = range(-4, 3), diatonic = False, title = True, outThresh = 10000, ignoreThresh = 10, titlesize = 12, printIt = True):			# string flag to average ascending and descending
	global voiceData, globalSource, lastGlobalSource
	directionFlag = False
	if type(direction) is str:
		direction = 0
		directionFlag = True
	tempSource = globalSource.replace("COMPLETE", "")
	if globalSource.upper() == 'TEMP' and lastGlobalSource:
		tempSource = "Synthetic " + lastGlobalSource.replace("COMPLETE", "")
	tempSource = title_caps(tempSource)
	if direction:
		direction = direction / abs(direction)
		strDirection = ['', 'ascending', 'descending'][direction]
	ascendingLeaps = []
	descendingLeaps = []
	ascendingSteps = []
	descendingSteps = []
	masterList = [[], [], [], []]
	leapRaw = []
	stepRaw = []
	startNote = min(voiceData[voice].keys())
	endNote = max(voiceData[voice].keys())
	for i in range(startNote, endNote + 1):
		if theRange and i not in theRange:
			continue
		if diatonic and (i % 12) in [1, 3, 6, 8]:
			values.append(-1)
			values2.append(-1)
			continue
		theAverages = [-1, -1, -1, -1]		# descending leap, descending step, ascending leap, ascending step
		x = report(i, voice, thresh, -1, outThresh)
		y = report_less(i, voice, 3, -1, outThresh)
		z = report(i, voice, thresh, 1, outThresh)
		w = report_less(i, voice, 3, 1, outThresh)
		for j, item in enumerate([x, y, z, w]):
			if item and (sum(item) != 0) and sum(item) >= ignoreThresh:
				tempVal = 100.*item[0]/sum(item)
				masterList[j].append(item)
	y1 = (100.0 * sum([x[0] for x in masterList[0] + masterList[2]]) / max(1, sum(sum(x) for x in masterList[0] + masterList[2])))
	y2 = (100.0 * sum([x[0] for x in masterList[1] + masterList[3]]) / max(1, sum(sum(x) for x in masterList[1] + masterList[3])))
	
	if printIt:
		print("DESCENDING LEAP REVERSALS", 100.0 * sum([x[0] for x in masterList[0]]) / max(1, sum(sum(x) for x in masterList[0])))
		print("ASCENDING LEAP REVERSALS", 100.0 * sum([x[0] for x in masterList[2]]) / max(1, sum(sum(x) for x in masterList[2])))
		print("DESCENDING STEP REVERSALS", 100.0 * sum([x[0] for x in masterList[1]]) / max(1, sum(sum(x) for x in masterList[1])))
		print("ASCENDING STEP REVERSALS", 100.0 * sum([x[0] for x in masterList[3]]) / max(1, sum(sum(x) for x in masterList[3])))
		print()
		print("*" * 40)
		print()

		print("LEAP REVERSALS %.0f" % y1, "total count", sum(sum(x) for x in masterList[0] + masterList[2]))
		print("STEP REVERSALS %.0f" % y2, "total count", sum(sum(x) for x in masterList[1] + masterList[3]))
		print()
		print()
	return [y1, y2]

def approach_size(composers = [], leapRange = [3, 12]):
	myStyles = ['solid', 'dashed', 'solid', 'dotted', 'dashed'] # | (offset, on-off-dash-seq) | '-' | '--' | '-.' | ':' | 'None' | ' ' | '']
	myColors = ['black', 'black', 'lightgrey', 'black', 'lightgrey']
	if not composers:
		composers = [globalSource]
	fig = plt.figure()
	ax = fig.gca()
	leapRange = range(leapRange[0], leapRange[1])
	plt.xlabel('Minimum leap size')
	plt.ylabel('Percentage of melodic reversals')
	composerNames = []
	for i in range(len(composers)):
		load(composers[i])
		composerNames.append(fix_composers(composers[i]))
		tempValues = []
		for j in leapRange:
			print("Leap size:", j)
			l, s = up_and_down(thresh = j)
			tempValues.append(l)
		if composers[i].upper() in ['13THC', 'SACRED HARP', 'ROCK']:
			ax.plot(leapRange[:6], tempValues[:6], color = myColors[i % len(myColors)], linestyle = myStyles[i % len(myStyles)], label = composerNames[-1])
		elif composers[i].upper() in ['XMONTEVERDI']:
			ax.plot(leapRange[:5], tempValues[:5], color = myColors[i % len(myColors)], linestyle = myStyles[i % len(myStyles)], label = composerNames[-1])
		elif composers[i].upper() in ['XOCKEGHEM']:
			ax.plot(leapRange[:4], tempValues[:4], color = myColors[i % len(myColors)], linestyle = myStyles[i % len(myStyles)], label = composerNames[-1])
		else:
			ax.plot(leapRange, tempValues, color = myColors[i % len(myColors)], linestyle = myStyles[i % len(myStyles)], label = composerNames[-1])
		#ax.plot(leapRange, tempValues, color = myColors[i % len(myColors)], linestyle = myStyles[i % len(myStyles)], label = composerNames[-1])
	myLeg = plt.legend(tuple(composerNames), loc=0, fontsize = 12)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.ylim([20, 100])
	plt.xlim(leapRange[0], leapRange[-1])
	plt.show()

def fix_composers(s):
	return (s[0].upper() + s[1:].lower()).replace("complete", '').replace('alraw', 'alestrina').replace('[0]', ' (r.h. only)').replace('raw', '').replace('13thc', 'Medieval').replace("Wtcf", 'Bach')
	
def summarize_steps_and_leaps(composers = [], voice = -1, stepThresh = 2):
	if not composers: composers = comps + comps2
	avg = []
	for c in composers:
		load(c)
		temp = steps_and_leaps(voice, stepThresh)
		print(c, temp)
		avg.append(temp[2])
	print("AVERAGE", 1.0 * sum(avg)/len(avg))
			
def steps_and_leaps(voice = -1, stepThresh = 2):
	stepCount = 0
	leapCount = 0
	for myNote in voiceData[voice]:
		for myInt in voiceData[voice][myNote]:
			ints = [int(x) for x in myInt.split()]
			if abs(ints[0]) > stepThresh:
				leapCount += voiceData[voice][myNote][myInt]
			elif abs(ints[0]) > 0:
				stepCount += voiceData[voice][myNote][myInt]
	return stepCount, leapCount, int(.5 + 100.*stepCount/(stepCount + leapCount))
	
		