from __future__ import print_function
import os
import pickle
import time
import copy
from music21 import *
import DT
import matplotlib.pyplot as plt

_THISFILE = 'SD'
lastChorale = 1
chorale = stream.Score()
analysis = stream.Score()
counts = [{}, {}]
progs = [{}, {}]

internalSizeVariable = 2

"""

FOR EACH SCALE-DEGREE PROGRESSION a -> b -> c- >... we compile a dictionary of [prog: number] pairs realizing that progression.

	This is for the melody only
	Note that accidentals come after the number, so '6# -> 7# -> 1'

The arrow, -++>, -+>, ->, -->, --->, gives the size of the interval.  

	-> is the smallest path between two scale degrees: 1->2 and 7->1 are ascending steps, while 4->3 and 1->7 descend by step.
	-+> takes the second note of the smallest path and transposes it up by octaves: 1-+>7 is an ascending seventh, while 1-+>5 is an ascending fifth.
	-++> goes up two octaves.  --> is down an octave (so that 1-->4 is a descending fifth), while ---> is down 2 octaves.

NB: Use 2MOZART instead of MOZART to get scale degree progressions based on the highest notes in a chord.

sd2.py differs from sd.py from sd in an important way:

	sd looks for progressions from the last chord tone in the first chord to the first chord tone in the second chord; 
		this is good for chorale textures, butcauses problems in keyboard textures.

	sd2 looks for progressions from the highest chord tone in the first chord to the highest chord tone in the second chord;
		this is generally better for keyboard textures.


"""

def load(source = 'ENCINA'):
	global internalSizeVariable, counts, progs, reducedCounts, reducedProgs, globalSource
	source = source.upper()
	globalSource = source
	try:
		with open(DT._DATAPATH + _THISFILE + source + '.pkl', 'rb') as storedFile:
			counts = pickle.load(storedFile)
			progs = pickle.load(storedFile)
		return True
	except:
		return False

def gather_stats(source = 'MOZART', fileName = '', size = 6):
	global counts, progs, locations, locNames, chorale, analysis, internalSizeVariable, rockFlag
	globalSource = source.upper()
	#if globalSource == 'ROCK': rockFlag = True
	internalSizeVariable = size
	counts = [{}, {}]
	progs = [{}, {}]
	locations = [{}, {}]
	locNames = {}
	startT = time.time()
	if globalSource in DT._PATHS:
		analysisPath = DT._PATHS[globalSource][0]
		scorePath = DT._PATHS[globalSource][1]
		fileStr = globalSource
		files = sorted(os.listdir(analysisPath))
		recordResults = True
		counter = 1
		for myFile in files:
			if myFile[-3:] == 'txt':
				if fileName and fileName != myFile[:-4]:
					continue
				print('Analyzing', myFile)
				locNames[counter] = myFile[:-4]
				foundIt = False
				for suffix in ['.xml', '.krn', '.mxl']:
					try:
						chorale = converter.parse(scorePath + myFile.replace('.txt', suffix))
						foundIt = True
						break
					except:
						pass
				if not foundIt: 
					print("CAN'T FIND SCORE!!!!")
					continue
				analysis = converter.parse(analysisPath + myFile, format='romantext')
				dochorale(counter)
				counter += 1
		if not fileName:
			outputFile = open(DT._DATAPATH + _THISFILE + fileStr + '.pkl', 'wb')
			pickle.dump(counts, outputFile)
			pickle.dump(progs, outputFile)
			pickle.dump(locNames, outputFile)
			#pickle.dump(filePath, outputFile)			# ditto!
			outputFile.close()
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and %.0f seconds' % ((minutes - int(minutes)) * 60))
	return
	
def clear():
	global counts, progs, reducedCounts, reducedProgs
	counts = [[{}, {}],[{}, {}],[{}, {}],[{}, {}]]
	progs = [[{}, {}],[{}, {}],[{}, {}],[{}, {}]]
	reducedCounts = [[{}, {}],[{}, {}],[{}, {}],[{}, {}]]
	reducedProgs = [[{}, {}],[{}, {}],[{}, {}],[{}, {}]]

def get_top_note(offset):
	global flatScore
	testNote = flatScore.getElementsByOffset(offset + .01, offset + .02, includeEndBoundary = False, mustBeginInSpan = False)
	if not testNote: return False
	return max(testNote, key=lambda n: n.pitch.midi)
	"""bestMidi = 0
	bestNote = ''
	for item in testNote:
		if item.midi > bestMidi: 
			bestMidi = item.midi
			bestNote = item
	return bestNote"""
	
def replace_chord_with_top_note(myStream):
	insertList = []
	for i in reversed(range(0, len(myStream))):										# convert chordal tones to single notes
		if 'Chord' in myStream[i].classes:
			myOffset = myStream[i].offset
			myNote = note.Note(max(myStream[i].pitches).nameWithOctave)
			myNote.duration.quarterLength = myStream[i].duration.quarterLength
			myStream.remove(myStream[i])
			insertList.append(myOffset)
			insertList.append(myNote)
	myStream.insert(insertList)
	return myStream

def dochorale(n):
	global chorale, analysis, internalSizeVariable, counts, progs, flatScore, startOffset, endOffset, ourKey, lastMeasure
	flatScore = chorale.flat.getElementsByClass(['Note', 'Chord']).stream()
	flatScore = replace_chord_with_top_note(flatScore)
	noteOffsets = sorted(list(set([x.offset for x in flatScore])))
	anal = analysis.flat.getElementsByClass('RomanNumeral')
	topNotes = {}
	topSDs = {}
	for myOffset in noteOffsets:
		topNotes[myOffset] = get_top_note(myOffset)
		#print(topNotes[myOffset])
		RN = anal.getElementAtOrBefore(myOffset)
		if RN:
			topSDs[myOffset] = accidental_string(RN.key.getScaleDegreeAndAccidentalFromPitch(topNotes[myOffset].pitch))
		else:
			topSDs[myOffset] = '?'
	lastMeasure = 0
	#preChords = chorale.chordify()
	#chords = preChords.flat.getElementsByClass('Chord')
	lastStartNote = False
	lastRN = False
	for i in range(len(noteOffsets)):
		worthContinuing = True
		myNoteOffset = noteOffsets[i]
		startOffset = noteOffsets[i]
		myNote = topNotes[myNoteOffset]
		if (not myNote): 
			continue
		RN = anal.getElementAtOrBefore(myNoteOffset)
		extraPCs = []
		#if rockFlag and RN.figure in ['V', 'V6', 'V6/5']:
		#	extraPCs.append((RN.root().pitchClass - 2) % 12)
		if not RN:
			continue
		if myNote == lastStartNote and RN == lastRN:
			continue
		if myNote.pitch.pitchClass not in RN.pitchClasses and myNote.pitch.pitchClass not in extraPCs: 
			continue
		lastStartNote = myNote
		lastRN = RN
		ourKey = RN.key
		#s = accidental_string(RN.key.getScaleDegreeAndAccidentalFromPitch(myNote))
		s = topSDs[myNoteOffset]
		progString = RN.figure
		endOffset = noteOffsets[i]
		modeIndex = int(RN.key.mode == 'minor')
		record(n, s, progString)
		j = i
		while j < len(noteOffsets) - 1:
			j += 1
			nextNoteOffset = noteOffsets[j]
			endOffset = noteOffsets[j]
			nextNote = topNotes[nextNoteOffset]
			if (not nextNote):
				worthContinuing = False
				break
			nRN = anal.getElementAtOrBefore(nextNoteOffset)
			nextraPCs = []
			#if rockFlag and nRN.figure in ['V', 'V6', 'V6/5']:
			#	nextraPCs.append((nRN.root().pitchClass - 2) % 12)
			if not nRN:
				continue
			if (myNote == nextNote and nRN == RN) or (nextNote.pitch.pitchClass not in nRN.pitchClasses and nextNote.pitch.pitchClass not in nextraPCs):
				continue
			if RN.key != nRN.key: 
				worthContinuing = False
				break
			"""CODE: -++>, -+>, ->, -->, ---> gives the size of the interval.  -> is the smallest path, the others add and subtract octaves"""
			tempString = topSDs[nextNoteOffset]
			if interval.notesToGeneric(myNote, nextNote).value > 11:
				s += ' -++> ' + tempString
			elif interval.notesToGeneric(myNote, nextNote).value > 4:
				s += ' -+> ' + tempString
			elif interval.notesToGeneric(myNote, nextNote).value < -11:
				s += ' ---> ' + tempString
			elif interval.notesToGeneric(myNote, nextNote).value < -4:
				s += ' --> ' + tempString
			else:
				s += ' -> ' + tempString	
			progString += ' -> ' + nRN.figure
			record(n, s, progString)
			myNote = nextNote
			if progString.count('->') >= (internalSizeVariable - 1):
				break
	return	

lastMeasure = 0

def record(n, s, progString):
	global counts, progs, locations, output, chorale, analysis
	global startOffset, endOffset, ourKey, lastMeasure
	#print s, progString
	myMeasures = analysis.parts[0].getElementsByOffset(startOffset, startOffset, includeEndBoundary = True, mustBeginInSpan = False, classList=['Measure'])
	if myMeasures:
		mNumber = myMeasures[0].number
	else:
		mNumber = lastMeasure
	if mNumber == 0: 
		mNumber = lastMeasure
	lastMeasure = mNumber
	nextMeasures = analysis.parts[0].getElementsByOffset(endOffset, endOffset, includeEndBoundary = True, mustBeginInSpan = False, classList=['Measure'])
	if nextMeasures:
		nmNumber = nextMeasures[0].number
	else:
		nmNumber = mNumber
	if nmNumber == 0: nmNumber = mNumber
	mList = [n, mNumber]
	if not mNumber == nmNumber: mList.append(nmNumber)
	
	modeIndex = int(ourKey.mode == 'minor')			# 0 for major, 1 for minor
	
	"""if progStr in counts[dictIndex]:
		counts[dictIndex][progStr] += 1
		locations[dictIndex][progStr].append(mList)
	else:
		counts[dictIndex][progStr] = 1
		locations[dictIndex][progStr] = [mList]"""
		
	if s in counts[modeIndex]:
		counts[modeIndex][s] += 1
		if progString in progs[modeIndex][s]:
			progs[modeIndex][s][progString] += 1
			locations[modeIndex][s][progString].append(mList)
		else:
			progs[modeIndex][s][progString] = 1
			locations[modeIndex][s][progString] = [mList]
	else:
		counts[modeIndex][s] = 1
		progs[modeIndex][s] = {}
		locations[modeIndex][s] = {}
		progs[modeIndex][s][progString] = 1
		locations[modeIndex][s][progString] = [mList]

def showall(**kwargs):
	for i in range(7, 1, -1):
		s = str(i) + ' -> ' + str(i-1)
		print(s)
		show(s, **kwargs)
	s = '1 -> 7'
	print(s)
	show(s, **kwargs)

def show(intString, reduce = True, mode = 'major', pcts = True, printOut = True, secondaryDominants = True, **kwargs):
	global progs, output
	output = {}
	if mode == 'min' or mode == 'm' or mode == 'minor' or mode == 0:
		modeIndex = 1
	else:
		modeIndex = 0
	if reduce:
		for s, count in progs[modeIndex][intString].items():
			newS = reduce_prog(s, **kwargs)
			if secondaryDominants or (newS.count('/') - newS.count('6/4') == 0):
				output[newS] = output.get(newS, 0) + count
	else:
		output = {x[0]:x[1] for x in progs[modeIndex][intString].items()}
	if printOut:
		DT.print_dict(output, pct = pcts)
	else:
		return output

def normalize_dict(d):
	s = sum(d.values())/100.
	return {x[0]:x[1]/s for x in d.items()}

def SD_comparison(intStrings = ['6 -> 5', '5 -> 4', '4 -> 3', '3 -> 2', '2 -> 1', '1 -> 7'], comps = ['Josquin', 'Goudimel', 'Morley'], cutOff = 19, **kwargs):
	global bigOutput
	if type(intStrings) is str:
		intStrings = [intStrings]
	bigOutput = {}
	for c in comps:
		load(c)
		for curInt in intStrings:
			outDict = bigOutput.setdefault(curInt, {})
			results = normalize_dict(show(curInt, printOut = False, **kwargs))
			for prog, pct in results.items():
				#if pct < cutOff: continue
				progDict = outDict.setdefault(prog, {})
				progDict[c] = round(pct, 1)
	print(' '.ljust(30) + '\t'.join(comps))
	for curInt in intStrings:
		progDict = bigOutput[curInt]
		theProgs = sorted(progDict.keys(), key = lambda x: -sum(progDict[x].values()))
		for p in theProgs:
			outList = [progDict[p].get(x, 0) for x in comps]
			if max(outList) < cutOff: continue
			print(curInt, ' ', p.ljust(20), '\t'.join([str(x) for x in outList]))
		
			
		
	
	
	

def reduce_prog(progStr, secondaries = True, sixFour = True):
	reduced_prog_list = []
	for item in progStr.split(' -> '):
		parsed_figure = DT.parse_figure(item)
		reduced_item = parsed_figure[0].replace('maj','').replace('/o','o')
		if sixFour and parsed_figure[1] == '6/4': reduced_item += '6/4'
		if secondaries and parsed_figure[2]: reduced_item += '/' + parsed_figure[2]
		reduced_prog_list.append(reduced_item)
	return ' -> '.join(reduced_prog_list)


grammarDictLoose = {'1 -> 2': ['I -> V', 'IV -> V'], '2 -> 3': ['V -> I', 'viio -> I'], '3 -> 4': ['I -> IV', 'I -> viio'], 
'4 -> 5': ['IV -> I', 'viio -> I', 'IV -> V', 'ii -> V'], '5 -> 6': ['I -> IV'], '6 -> 7': ['IV -> V', 'IV -> viio'], '7 -> 1': ['V -> I', 'viio -> I', 'V -> vi'],
'1 -> 7': ['I -> V', 'I -> viio', 'IV -> V', 'IV -> viio'], '2 -> 1': ['V -> I', 'viio -> I', 'V -> vi'], '3 -> 2': ['I -> V', 'I -> viio'], '4 -> 3': ['IV -> I', 'viio -> I'],
'5 -> 4': ['I -> IV', 'I -> viio'], '6 -> 5': ['IV -> I', 'IV -> V'], '7 - 6': []}

grammarDictInvs = {'7 -> 1': ['V -> IV6'], '5 -> 4': ['V -> IV6'], '2 -> 1': ['V -> IV6']}

""" PURE I-IV-V version of this"""

grammarDictPure = {'1 -> 2': ['I -> V', 'IV -> V'], '2 -> 3': ['V -> I', 'viio -> I'], '3 -> 4': ['I -> IV', 'I -> viio'], 
'4 -> 5': ['IV -> I', 'viio -> I', 'IV -> V'], '5 -> 6': ['I -> IV'], '6 -> 7': ['IV -> V', 'IV -> viio'], '7 -> 1': ['V -> I', 'viio -> I'],
'1 -> 7': ['I -> V', 'I -> viio', 'IV -> V', 'IV -> viio'], '2 -> 1': ['V -> I', 'viio -> I'], '3 -> 2': ['I -> V', 'I -> viio'], '4 -> 3': ['IV -> I', 'viio -> I'],
'5 -> 4': ['I -> IV', 'I -> viio'], '6 -> 5': ['IV -> I', 'IV -> V'], '7 - 6': []}


def grammar(mode = 'major', printIt = True, strict = False):
	global progs
	if strict:
		grammarDict = grammarDictPure
	else:
		grammarDict = grammarDictLoose
	if mode == 'min' or mode == 'm' or mode == 'minor' or mode == 0:
		modeIndex = 1
	else:
		modeIndex = 0
	approved = 0
	total = 0
	for sdProg in grammarDict:
		subtotal = 0
		subapproved = 0
		if printIt: print(sdProg)
		if sdProg not in progs[modeIndex]:
			continue
		for chordProg in progs[modeIndex][sdProg]:
			#print chordProg, progs[modeIndex][sdProg][chordProg]
			chords = chordProg.split(' -> ')
			for i in range(len(chords)):
				parsedFig = DT.parse_figure(chords[i])
				if not parsedFig[2]:
					chords[i] = parsedFig[0]
			strippedProg = ' -> '.join(chords)
			subtotal += progs[modeIndex][sdProg][chordProg]
			if (strippedProg in grammarDict[sdProg]) or (sdProg in grammarDictInvs and chordProg in grammarDictInvs[sdProg]):
				subapproved += progs[modeIndex][sdProg][chordProg]
			else:
				if printIt: print('    VIOLATION', chordProg, progs[modeIndex][sdProg][chordProg]) 
		if printIt: print('%.2f' % (100.*subapproved/subtotal), 'are in the list')
		total += subtotal
		approved += subapproved
	print('%.2f' % (100.*approved/total), 'are in the list')
	return 100.*approved/total

def get_intervals(size, voice, mode = 'major', useReduced = True, printOut = True, sizeString = ''):
	total = 0
	if mode == 'min' or mode == 'm' or mode == 'minor' or mode == 0:
		modeIndex = 1
	else:
		modeIndex = 0
	for i in range(1, 8):
		if sizeString:
			s = str(i)+' -' + sizeString +'> '+str(sd(i+size))
		else:
			s = str(i)+' -> '+str(sd(i+size))
		try:
			if printOut: print(s, counts[modeIndex][s])
			total += counts[modeIndex][s]
		except KeyError:
			if printOut: print(0)
	if printOut: print('TOTAL:', total)
	return total
	
def summarize(useReduced = False, percent = False, ascending = True):
	#load(2)
	if useReduced: 
		print("USING REDUCED CHORALES")
	else:
		print("USING REAL CHORALES")
	if ascending:
		print("ASCENDING INTERVALS (UNISON, SECOND, THIRD, FOURTH, FIFTH, SIXTH, SEVENTH, OCTAVE)")
	else:
		print("DESCENDING INTERVALS (UNISON, SECOND, THIRD, FOURTH, FIFTH, SIXTH, SEVENTH, OCTAVE)")
	if ascending:
		for i in range(0, 1):
			totals = []
			for j in range(0, 8):
				if j<=3:
					totals.append(get_intervals(j, i, useReduced = useReduced, printOut=False))
				else:
					totals.append(get_intervals(j, i, useReduced = useReduced, printOut=False, sizeString='+'))
			if percent:
				sum = 0
				for k in totals: sum += k
				percents = [100 * x / sum for x in totals]
				print('voice', i, 'intervals', percents)
			else:
				print('voice', i, 'intervals', totals)
	else:
		for i in range(0, 4):
			totals = []
			for j in range(0, 8):
				j = j * -1
				if j>=-3:
					totals.append(get_intervals(j, i, useReduced = useReduced, printOut=False))
				else:
					totals.append(get_intervals(j, i, useReduced = useReduced, printOut=False, sizeString='-'))
			if percent:
				sum = 0
				for k in totals: sum += k
				percents = [100 * x / sum for x in totals]
				print('voice', i, 'intervals', percents)
			else:
				print('voice', i, 'intervals', totals)
	
def intervals(size, voice, mode = 'major'):
	print("NOT REDUCED")
	get_intervals(size, voice, mode, False)
	get_intervals(-1*size, voice, mode, False)
	print("REDUCED")
	get_intervals(size, voice, mode, True)
	get_intervals(-1*size, voice, mode, True)

def starts(firstChord, voice = 3, mode = 'major', useReduced = False):
	global progs, reducedProgs
	modeInd = get_mode(mode)
	if useReduced: 
		myProgs = reducedProgs[voice][modeInd]
	else:
		myProgs = progs[voice][modeInd]
	results = []										
	for prog in myProgs:
		if prog[0:len(firstChord)] == firstChord:
			results.append([prog, myProgs[prog]])
	return results

def get_mode(mode = 'major'):
	if type(mode) == str:
		if mode == 'minor' or mode == 'm' or mode == 'min': 
			return 1
		else: 
			return 0
	elif type(mode) == int:
		return mode

def sd(i):
	return ((i - 1) % 7) + 1

def accidental_string(s):
  if s[1] is None:
     degreeString = str(s[0])
  else:
     degreeString = str(s[0]) + str(s[1].modifier)
  return degreeString

def graph(myInts = -1, modeInd = 0):
	global counts
	if myInts == -1:
		myInts = [1, 2, 3]
	else:
		myInts = [myInts]
	intLabels = ['unison', 'second', 'third', 'fourth']
	plt.xlabel('Starting Degree')
	plt.ylabel('Intervals of step size X')
	plt.title('Proportion of melodic degrees')
	labels = []
	for myInt in myInts:
		for j in [-1, 1]:
			values = []
			if j == -1:
				myLabel = 'descending ' + intLabels[myInt]
			else:
				myLabel = 'ascending ' + intLabels[myInt]
			labels.append(myLabel)
			for i in range(1, 8):
				myKey = str(i) + ' -> ' + str((((i - 1) + (j * myInt)) % 7) + 1)
				if myKey in counts[modeInd]:
					values.append(counts[modeInd][myKey])
				else:
					values.append(0)
			plt.plot(values, label = myLabel)
	ticks = range(0, 7)
	myLeg = plt.legend(tuple(labels), loc=0)
	for i in range(len(myLeg.legendHandles)):
		myLeg.legendHandles[i].set_linewidth(2.0)
	plt.xticks(ticks, [x + 1 for x in ticks])
	plt.show()