from __future__ import print_function
import os
import copy
import time
import pickle
import matplotlib.pyplot as plt

from music21 import *
import DT

_BADCHORALES = []
counts = {}
locations = {}
symbolDict = [{}, {}]
inputFile = open (DT._DATAPATH + 'bass_dict.pkl', 'rb')
symbolDict = pickle.load(inputFile)
inputFile.close()
output = stream.Score()
lastChorale = 0
chorale = stream.Score()
analyzed_file = stream.Score()

"""
Catalogue harmonic sequences based on human RN analyses

	
TO DO:
	1. RECORD A KEY THAT ASSOCIATES COUNTER WITH FILENAMES; STORE THESE FILENAMES IN THE PICKLED FILE
	1. Gather all lengths from 1 to 5 at once
	2. Make it possible to gather all the files in a directory 
	
	>>> import os
	>>> os.listdir(DT._HUMANBEETHOVEN)
	['.DS_Store', 'sonata01-1.txt', 'sonata16-1.txt', 'sonata16-2.txt', 'sonata16-3.txt']
	>>> import glob
	>>> glob.glob(DT._HUMANBEETHOVEN + '/*.txt')
	
"""

def gather_stats(source='BACH', n = 1, m = 372, size = 5):
	global counts, locations, currentSize, analyzed_file, locNames
	global quarterChords, eighthEighthChords, globalSource
	quarterChords = 0
	recordResults = False
	eighthEighthChords = 0
	counts = {}
	locations = {}
	locNames = {}
	currentSize = size
	startT = time.time()
	fileStr = ''
	globalSource = source.upper()
	if globalSource in DT._PATHS:
		filePath = DT._PATHS[globalSource][0]
		fileStr = globalSource
		files = os.listdir(filePath)
		recordResults = True
		counter = 1
		for myFile in files:
			if myFile[-3:] == 'txt':
				locNames[counter] = myFile[:-4]
				print('Analyzing', myFile)
				if globalSource[:4] == "ROCK":
					mySong = romanText.clercqTemperley.CTSong(filePath + myFile)
					analyzed_file = mySong.toScore()
				else:
					analyzed_file = converter.parse(filePath + myFile, format='romantext')
				dochorale(counter, size)
				counter += 1
	elif source in ['Bach', 'BachMechanical', 'BachClean']:
		for i in range(n,m):
			if m == 372: recordResults = True
			lastChorale = i
			if i in _BADCHORALES:
				print('chorale', i, 'is bad')
				continue
			print('chorale', i)
			if source == 'BachClean':
				analyzed_file = converter.parse(DT._CLEANMACHINE + 'riemenschneider' + str(i).zfill(3) + '.txt', format='romantext')
			elif source == 'BachMechanical':
				analyzed_file = converter.parse(DT._MACHINEANALYSES + 'riemenschneider' + str(i).zfill(3) + '.txt', format='romantext')
			else:
				analyzed_file = converter.parse(DT._ANALYSISPATH + 'riemenschneider' + str(i).zfill(3) + '.txt', format='romantext')
			dochorale(i, size)
	#print "Quarter-note chords", quarterChords, "(" + str(100*quarterChords/(quarterChords + eighthEighthChords)) + '%)', "Eighth-note + eighth-note pairs", eighthEighthChords
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and', int((minutes - int(minutes)) * 60), 'seconds')
	if recordResults:
		if source == 'BachMechanical': fileStr += 'MACHINE'
		if source == 'BachClean': fileStr += 'CLEANMACHINE'
		outputFile = open (DT._DATAPATH + 'sequences' + fileStr + '.pkl', 'wb')
		pickle.dump(counts, outputFile)
		pickle.dump(locations, outputFile)
		pickle.dump(locNames, outputFile)			# need to make use of this
		pickle.dump(filePath, outputFile)			# ditto!
		outputFile.close()
	return

def do_file(name='', source='BACH'):
	global counts, locations, currentSize, analyzed_file
	global quarterChords, eighthEighthChords, globalSource
	quarterChords = 0
	recordResults = False
	eighthEighthChords = 0
	counts = {}
	locations = {}
	locNames = []
	currentSize = 5
	startT = time.time()
	fileStr = ''
	globalSource = source.upper()
	if globalSource in DT._PATHS:
		filePath = DT._PATHS[globalSource][0]
		fileStr = globalSource
		files = os.listdir(filePath)
		counter = 0
		for myFile in files:
			if myFile[-3:] == 'txt' and myFile[:-4] == name:
				print('Analyzing', myFile)
				analyzed_file = converter.parse(filePath + myFile, format='romantext')
				dochorale(counter, currentSize)
				counter += 1
	else:
		print("SOURCE NOT FOUND")
	print("Quarter-note chords", quarterChords, "(" + str(100*quarterChords/(quarterChords + eighthEighthChords)) + '%)', "Eighth-note + eighth-note pairs", eighthEighthChords)
	minutes = (time.time() - startT)/60.
	print('Total analysis took', int(minutes), 'minutes and', int((minutes - int(minutes)) * 60), 'seconds')
	return

def fix_temperley():
	global anal
	lastKeyString = ['', '']
	majorTonicCount = 0
	minorTonicCount = 0
	startChord = 0
	changeToMinor = []
	for i in range(len(anal)):
		item = anal[i]
		if 'RomanNumeral' in item.classes:
			item.figure = item.figure.replace('64', '6/4')			# slashes missing in slash figures
			item.figure = item.figure.replace('65', '6/5')
			item.figure = item.figure.replace('43', '4/3')
			item.figure = item.figure.replace('42', '2')
			item.figure = item.figure.replace('a', '+')				# a for augmented
			item.figure = item.figure.replace('d', '')				# d for dominant?
			parsedFig = DT.parse_figure(item.figure)
			if parsedFig[0] == 'i' and (not parsedFig[2]):
				minorTonicCount += 1
			elif parsedFig[0] == 'I' and (not parsedFig[2]):
				majorTonicCount += 1
			if item.key.pitchAndMode != lastKeyString:
				if minorTonicCount > majorTonicCount:
					changeToMinor.append([startChord, i])
				majorTonicCount = 0
				minorTonicCount = 0
				startChord = i
				lastKeyString = item.key.pitchAndMode
	if minorTonicCount > majorTonicCount:
		changeToMinor.append([startChord, i + 1])
	for minorRegion in changeToMinor:
		keyStr = anal[minorRegion[0]].key.pitchAndMode[0].name.lower()
		newKey = key.Key(keyStr)
		for j in range(minorRegion[0], minorRegion[1]):
			#print anal[j], DT.figure_from_chord(anal[j], newKey), newKey, newKey.classes
			anal[j] = roman.RomanNumeral(DT.figure_from_chord(anal[j], newKey), newKey)

def mod7(i):
	if i > 0: i = i - 1
	if i < 0: i = i + 1
	return (i % 7)
		
def dochorale(n, size):															# record sequences of minimal length n
	global counts, locations, output, pTreb, pBass, chorale, analyzed_file, anal
	global quarterChords, eighthEighthChords, globalSource, rootInts, rootOffsets
	global startOffset, endOffset, ourKey, lastMeasure
	anal = analyzed_file.flat.getElementsByClass(['RomanNumeral'])
	if globalSource[:4] == 'ROCK': fix_temperley()
	roots = [x.root() for x in anal]
	rootOffsets = {}
	rootInts = []
	for i in range(1, len(roots)):
		theInt = interval.notesToGeneric(roots[i-1], roots[i]).value
		theInt = mod7(theInt)
		if theInt != 0:
			rootInts.append(theInt)
			rootOffsets[len(rootInts) - 1] = anal[i-1].offset	
	lastMeasure = 0
	startOffset = 0
	endOffset = 0
	i = -1
	while i < (len(rootInts) - size):
		i += 1
		startOffset = rootOffsets[i]
		outSeq = [rootInts[i], rootInts[i+1]]
		#print i, outSeq
		if (sum(outSeq) % 7) == 0:
			continue
		stopRecording = False
		for j in range(i + 2, len(rootInts)):
			if rootInts[j] != outSeq[j % 2]:
				if (j - i) < size:
					break
				else:
					endOffset = rootOffsets[j - 1]
					progStr = str(sorted(outSeq)) + ' L ' + str(j - i)
					record(n, progStr)
					i = j - 1
					break
		if (j - i) >= size: 
			endOffset = rootOffsets[j - 1]
			progStr = str(outSeq) + ' L ' + str(j - i)
			record(n, progStr)
			i = j - 1
	return

def record(n, progStr):
	global counts, locations, output, pTreb, pBass, chorale, analyzed_file
	global startOffset, endOffset, ourKey, lastMeasure, globalSource
	
	if globalSource[:4] == 'ROCK':
		mNumber = startOffset				# TIME_SIGNATURE.ratioString
	else: 
		mNumber = analyzed_file[1].getElementsByOffset(startOffset, startOffset, includeEndBoundary = True, mustBeginInSpan = False, classList=['Measure'])[0].number
		if mNumber == 0: mNumber = lastMeasure
	lastMeasure = mNumber
	
	if globalSource[:4] == 'ROCK':
		nmNumber = endOffset
	else:
		nmNumber = analyzed_file[1].getElementsByOffset(endOffset, endOffset, includeEndBoundary = True, mustBeginInSpan = False, classList=['Measure'])[0].number
		if nmNumber == 0: nmNumber = mNumber
	mList = [n, mNumber]
	if not mNumber == nmNumber: mList.append(nmNumber)
	
	print(progStr, mList)
	
	if progStr in counts:
		counts[progStr] += 1
		locations[progStr].append(mList)
	else:
		counts[progStr] = 1
		locations[progStr] = [mList]

def load(source = 'BACH'):
	global counts, locations, globalSource, locNames, currentSize, working
	working = [{}, {}]
	currentSize = 5
	globalSource = source.upper()
	storedFile = open(DT._DATAPATH + 'sequences' + globalSource + '.pkl', 'rb')
	if not storedFile: 
		print("error")
		return
	counts = pickle.load(storedFile)
	locations = pickle.load(storedFile)
	locNames = pickle.load(storedFile)
	storedFile.close()

def list(progStr, modeStr = 'major'):
	global counts, locations, globalSource, locNames, currentSize, working
	locList = []
	myLen = len(progStr)
	for item in locations:
		if item[:myLen] == progStr:
			for eachLoc in locations[item]:
				locList.append(eachLoc)
	for item in locList:
		print('    ', locNames[item[0]], 'measure', item[1])

def consolidate(minLen = 0):
	global cleanCounts
	cleanCounts = {}
	for item in counts:
		myLen = int(item.split(' L ')[1])
		if myLen > minLen:
			cleanCounts[item[:6]] = cleanCounts.setdefault(item[:6], 0) + counts[item]
	DT.print_dict(cleanCounts)
	return

def remove_duplicates():
	global counts, locations
	newCounts = [{}, {}]
	newLocations = [{}, {}]
	for modeInd in [0,1]:
		for prog in counts:
			newProg = rem_dups(prog)
			if newProg == 'I -> bIII -> IV': print(prog, counts[prog])
			newCounts[newProg] = newCounts.setdefault(newProg, 0) + counts[prog]
			if newProg in newLocations:
				for oldLoc in locations[prog]:
					newLocations[newProg].append(oldLoc)
			else:
				newLocations[newProg] = locations[prog]
	counts = newCounts
	locations = newLocations

def show(rnProg, RNs = True):
	global output, locNames, globalSource
	output = stream.Score()
	tempOutput = stream.Score()
	lastChoraleNo = False
	locList = []
	myLen = len(rnProg)
	for item in locations:
		if item[:myLen] == rnProg:
			for eachLoc in locations[item]:
				locList.append(eachLoc)
	if not locList:
		print('Not found')
		return
	locList = sorted(locList)
	if globalSource == 'ROCK':
		for instance in locList:
			if locNames[instance[0]] != lastChoraleNo:
				choraleNo = locNames[instance[0]]
				print("")
				print("GETTING ",choraleNo)
				lastChoraleNo = choraleNo
				myFile = open(DT._PATHS[globalSource][0] + choraleNo + '.txt', 'rb')
				for line in myFile:
					print("     ", line.rstrip('\n'))
				myFile.close()
		return
	for i in range(0, 4):
		tempOutput.append(stream.Part())
	for instance in locList:
		measureList = instance[1:]
		if len(measureList) == 1:
			measureList = [measureList[0], measureList[0] + 1]
		else:
			measureList[-1] = measureList[-1] + 1
		header = str(locNames[instance[0]])+'m'+str(measureList[0])
		print('getting', locNames[instance[0]], measureList)
		if locNames[instance[0]] != lastChoraleNo:
			choraleNo = locNames[instance[0]]
			lastChoraleNo = choraleNo
			try:
				chorale = converter.parse(DT._PATHS[globalSource][1] + choraleNo + '.xml')
			except:
				chorale = converter.parse(DT._PATHS[globalSource][1] + choraleNo + '.krn')
			if RNs: 
				analyzed_file = converter.parse(DT._PATHS[globalSource][0] + choraleNo + '.txt', format='romantext')
			if globalSource == 'MOZART':
				DT.remove_repeats(chorale)
				DT.remove_repeats(analyzed_file)
		if RNs:
			DT.add_roman(chorale, analyzed_file, measureList[0], measureList[1])
		tempChorale = copy.deepcopy(DT.get_measures(chorale, measureList[0], measureList[1]))
		DT.insert_text(tempChorale.parts[0][0], header, 55, 0)
		output = DT.append(tempChorale, output)
	output = DT.fix_roman_placement(output)
	for p in output.parts:
		for item in p.flat:
			if 'Note' in item.classes or 'Chord' in item.classes: item.lyrics = []
	output.show()
	return output

def get_mode(mode = 'major'):
	if type(mode) == str:
		if mode == 'minor' or mode == 'm' or mode == 'min': 
			return 1
		else: 
			return 0
	elif type(mode) == int:
		return mode

def starts(firstChord, mode = 'major', source=''):
	global working
	if not source:
		source = working
	results = []										
	modeInd = get_mode(mode)
	for prog in source:
		if prog[0:len(firstChord)] == firstChord:
			results.append([prog, source[prog]])
	tempList = sorted(results, key=lambda x:x[1], reverse = True)
	tempCounts = 0
	for pair in tempList: 
		print(pair)
		tempCounts += pair[1]
	print(tempCounts)
	return

def ends(lastChord, mode = 'major'):
	results = []
	modeInd = get_mode(mode)
	for prog in working:
		if prog[-len(lastChord):] == lastChord:
			results.append([prog, working[prog]])
	tempList = sorted(results, key=lambda x:x[1], reverse = True)
	tempCounts = 0
	for pair in tempList: 
		print(pair)
		tempCounts += pair[1]
	print(tempCounts)
	return

def contains(lastChord, mode = 'major'):
	results = []
	modeInd = get_mode(mode)
	for prog in working:
		if prog.count(lastChord) > 0:
			results.append([prog, working[prog]])
	tempList = sorted(results, key=lambda x:x[1], reverse = True)
	tempCounts = 0
	for pair in tempList: 
		print(pair)
		tempCounts += pair[1]
	print(tempCounts)
	return

"""TODO: update this for arbitrary lengths"""

def bass_degrees(degStr, mode = 'major'):										# works but is very slow!
	progSize = degStr.count('>') + 1
	load(progSize)
	results = []
	degStr = remove_arrows(degStr)
	modeInd = get_mode(mode)
	if modeInd == 0:
		myKey = key.Key('C')
		keyStr = 'C'
	else:
		myKey = key.Key('c')
		keyStr = 'c'
	firstDeg = degStr[0]
	for prog in counts:
		foundMismatch = False
		symbols = remove_arrows(prog)
		for i in range(0, len(symbols)):
			RN = roman.RomanNumeral(symbols[i], keyStr)
			if RN.pitches[0].pitchClass != DT.pc_from_scale_degree(degStr[i], myKey):
				foundMismatch = True
				break
		if not foundMismatch:
			results.append([prog, counts[prog]])			 
	return sorted(results, key=lambda x:x[1], reverse = True)

def remove_arrows(s):
	t = s.split()
	for i in range(0, (len(t)-1)/2):
		t.remove('->')
	return t
	
def getstats(progStr, percent = False):
	global counts, progDict, progDictKey
	myList = sorted(progDict[progDictKey[progStr]].iteritems(), key=lambda x:x[1], reverse = True)
	if percent:
		total = 0
		for name, count in myList:
			total += count
		for i in range(len(myList)):
			myList[i] = (myList[i][0], round(100.0 * myList[i][1]/total, 2))
	return myList
	
def fix_half_diminished(progStr):
	progStr = progStr.replace('o7', '/o7')
	progStr = progStr.replace('o6/5', '/o6/5')
	progStr = progStr.replace('o4/3', '/o4/3')
	progStr = progStr.replace('o2', '/o2')
	return progStr

def fix_major(progStr):
	progStr = progStr.replace('o7', '/o7')
	progStr = progStr.replace('o6/5', '/o6/5')
	progStr = progStr.replace('o4/3', '/o4/3')
	progStr = progStr.replace('o2', '/o2')
	return progStr

def reduce(modeStr = 'major', tonicizations = True):
	global reduced_counts
	reduced_counts = {}
	if modeStr[0:3].upper() == 'MAJ': 
		modeIndex = 0
	else:
		modeIndex = 1
	for prog in counts[modeIndex]:
		newprog = reduce_prog(prog, tonicizations)
		reduced_counts[newprog] = reduced_counts.setdefault(newprog, 0) + counts[modeIndex][prog]
		
def all_inversions(progStr, modeInd = 0):
	progList = progStr.split(' -> ')
	totalCounts = 0
	for firstInv in ['', '6', '6/4', '7', '6/5', '4/3', '2']:
		if progList[0] in ['I', 'IV'] and firstInv not in ['', '6', '6/4']:
			firstInv = 'maj' + firstInv 
		for secondInv in ['', '6', '6/4', '7', '6/5', '4/3', '2']:
			if progList[1].isupper() and secondInv not in ['', '6', '6/4']:
				secondInv = 'maj' + secondInv
			tryProg = progList[0] + firstInv + ' -> ' + progList[1] + secondInv
			print(tryProg)
			if tryProg in counts[0]:
				totalCounts += counts[0][tryProg]
	print(totalCounts)

def remove_inversions(matchProg = '', modeInd = 0, three = True, printIt = True, filterDiatonic = True, separateSixFour = True):
	global working, reduced
	reduced = {}
	for item in working:
		prog = item.split(' -> ')
		for i in range(len(prog)):
			if prog[i][-1] == '/': prog[i] = prog[i][:-1]
			if not three:
				prog[i] = prog[i].replace('iii6', 'V').replace('iii', 'I6')
			newFig = DT.parse_figure(prog[i])
			if separateSixFour:
				if newFig[1] != '6/4': newFig[1] = ''
			else:
				newFig[1] = ''
			if newFig[2]: newFig[2] = '/' + newFig[2]
			newFig[0] = newFig[0].replace('maj', '').replace('/o','o')
			prog[i] = ''.join(newFig)
		if len(prog) == 2 and prog[0] == prog[1]: continue
		newProg = ' -> '.join(prog)
		if matchProg:
			if newProg == matchProg:
				reduced[item] = working[item]
		else:
			reduced[newProg] = reduced.setdefault(newProg, 0) + working[item]
	if filterDiatonic:
		if modeInd == 0: 
			mySource = DT._DIATONICMAJOR
		else:
			mySource = DT._DIATONICMINOR
		for item in reduced.keys():
			for prog in item.split(' -> '):
				if prog not in mySource: 
					del reduced[item]
					break
	if printIt: DT.print_dict(reduced, pct = True)
	
	
def reduce_prog(progStr, secondaries = True):
	reduced_prog_list = []
	for item in progStr.split(' -> '):
		parsed_figure = DT.parse_figure(item)
		reduced_item = parsed_figure[0].replace('maj','').replace('/o','o')
		if secondaries and parsed_figure[2]: reduced_item += '/' + parsed_figure[2]
		reduced_prog_list.append(reduced_item)
	return ' -> '.join(reduced_prog_list)

working = [{}, {}]

def get_size(size = 3, fauxbourdon = True, duplicates = False):
	global counts, working
	working = [{}, {}]
	if fauxbourdon or (size != 1):
		size = size - 1
		for modeInd in [0, 1]:
			for item in counts:
				if item.count(' -> ') == size:
					working[item] = counts[item]
		return
	"ELSE:"
	for modeInd in [0, 1]:
		for item in counts:
			if item.count(' -> ') == 4:
				chords = item.split(' -> ')
				analyses = [DT.parse_figure(x) for x in chords]
				if ((analyses[0][1] == '6' and analyses[1][1] == '6' and analyses[2][1] == '6')
				or (analyses[1][1] == '6' and analyses[2][1] == '6' and analyses[3][1] == '6')
				or (analyses[2][1] == '6' and analyses[3][1] == '6' and analyses[4][1] == '6')):
					continue
				if duplicates:
					working[chords[2]] = working.setdefault(chords[2], 0) + counts[item]
				else:
					if analyses[1][0] == analyses[2][0]: continue
					working[chords[2]] = working.setdefault(chords[2], 0) + counts[item]

"""def get_size_noduplicates():
	global counts, working
	for modeInd in [0, 1]:
		for item in counts:
			if item.count(' -> ') == 1:
				chords = item.split(' -> ')
				analyses = [DT.parse_figure(x) for x in chords]
				if analyses[0][0] != analyses[1][0]:
					working[chords[1]] = working.setdefault(chords[1], 0) + counts[item]"""
