import sys
import os
from music21 import corpus

#_MAINPATH = '/Users/dmitri/Source Code/Python/'
#_PALCORPUS = '/Library/Frameworks/Python.framework/Versions/2.7/lib/python2.7/site-packages/music21/corpus/palestrina/'
#_ENGLISHWORDS = '/usr/share/dict/words'			# optional, not needed

"""
================================================================
				Setup routines here	
================================================================
"""

instructionString = """Change directory to the location of the DT.py file. 
 
From the command line type
	py setup.py . 
	
to use that directory as your main directory.

Or py setup.py <directory> to use a different directory (enclose in quotes if the path has spaces)."""

def get_palestrina_directory():
	directories = {}
	a = corpus.getComposer('palestrina')
	for x in a:
		sSplit = str(x).split('/')[:-1]
		s = '/'.join(sSplit)
		directories[s] = directories.get(s, 0) + 1
	if len(directories) > 1:
		print("Multiple Palestrina directories available:")
		sDir = [x for x in sorted(directories.items(), key = lambda x:-x[1])]
		for item, count in sDir:
			print('\t', item, str(count), 'files')
		print('Choosing the most popular', sDir[0][0])
		return sDir[0][0]
	return list(directories.keys())[0]

def setup(newMainDir = ''):
	workingDir = os.getcwd()
	workingFiles = os.listdir('.')
	if 'DT.py' not in workingFiles:
		print('Change directory to the location of DT.py!')
		return
	if not newMainDir:
		print(instructionString)
		return
	try:
		os.listdir(newMainDir)
	except:
		pass
		print('"'+newMainDir+'"', "seems not to be a working directory; exiting.")
		return
	if newMainDir == '.':
		newMainDir = workingDir + '/'
	if newMainDir[-1] != '/': 
		newMainDir += '/'
	palDir = get_palestrina_directory()+'/'
	os.rename('DT.py', 'DT.py.bak')
	with open('DT.py.bak', 'r') as inFile:
		with open('DT.py', 'w+') as outFile:
			for line in inFile:
				if line.startswith('_MAINPATH ='):
					outFile.write("_MAINPATH = '" + newMainDir+"'\n")
				elif line.startswith('_PALCORPUS ='):
					outFile.write("_PALCORPUS = '" + palDir+"'\n")
				else:
					outFile.write(line)
	currentDirectories = list(os.walk(newMainDir))[0][1]
	if 'Music' not in currentDirectories:
		os.mkdir(newMainDir+'Music')
	if 'Music Data' not in currentDirectories:
		os.mkdir(newMainDir+'Music Data')
	print('Success!')
	
if __name__ == "__main__":
	args = sys.argv
	if len(args) < 2:
		setup('.')
	else:
		setup(args[1])
	