from __future__ import print_function
import random
import os
import pickle
import copy
from music21 import *
import DT
import DTmidi

"""

Harmonize Bach chorales using canned voice leadings and a "phrase timer" approach that alters the probability of chords according to their position in a phrase.

Dretske: 'if you can't build one, you don't know how it works.'

Fix passing tones in minor!!!!  Second pass, looking for aug seconds
Some minor accidentals should be sharps?

TO DO: 

	USING CANNED VOICE LEADINGS
		-add spacing check
		-add RN notations under score
		-rhythm?
		
	USING OUCH VOICE LEADINGS
		Seventh chords
		Non maximally efficient voice leadings
		check range, parallels, etc.
"""

outTable = {'H': [['H -> O', 3], ['H -> H', 2], ['H -> C', 7], ['H -> OCT+', 1]], 'C': [['C -> C', 11], ['C -> H', 4
], ['C -> O', 1], ['C -> DI', 1]], 'OCT+': [['OCT+ -> C', 1], ['OCT+ -> H', 2], ['OCT+ -> O', 5], ['OCT+ -> OCT+', 1]
], 'O': [['O -> OCT+', 1], ['O -> O', 5], ['O -> C', 1], ['O -> H', 1]], 'DI': [['DI -> H', 2], ['DI -> C', 6], ['DI -> DI', 1]]}

def linear_map(value, firstRange, secondRange):
	pct = 1.0 * (value - firstRange[0])/(firstRange[1] - firstRange[0])
	output = secondRange[0] + pct*(secondRange[1] - secondRange[0])
	return output

progs = [{}, {}]
locations = [{}, {}]
chorale = stream.Score()
totalPossibilities = {}
chordProgs = {}

currentState = [key.Key('C'), 'I', [72, 67, 64, 48]]		# key, start chord, startnotes
ranges = [[60, 79], [53, 72], [48, 67], [41, 64]]			# SATB

def top_note_dict(d, modeInd = 0):							
	tempChords = {}
	if modeInd == 0: 
		theKey = key.Key('C')
	else:
		theKey = key.Key('c')
	for c in d:
		try:
			rn = roman.RomanNumeral(c, theKey)
			for p in rn.pitches:
				p = p.midi % 12
				if p not in tempChords:
					tempChords[p] = []
				tempChords[p].append([c, d[c]])
		except:
			pass
	return tempChords

"""ticks should be set to 16 when running save_histo_dict in phrase_and_melody (includeSoprano = False) """

f = open(DT._DATAPATH + 'PHRASE_AND_MELODYCHORDSBYRHYTHMICPOSITIONBACH.pkl', 'rb')	# made by phrase_and_melody
chordPositionDictMajor = pickle.load(f)
startChordsMajor = top_note_dict(pickle.load(f), 0)
chordPositionDictMinor = pickle.load(f)
startChordsMinor = top_note_dict(pickle.load(f), 1)
f.close()
chordPositionDict = chordPositionDictMajor
startChords = startChordsMajor

globalWeightFactor = 1

outScore = stream.Score()
for i in range(4):
	outScore.insert(0, stream.Part())

def harmonize(melodyNotes = [72, 72, 71, 72, 74, 76, 77, 76, 74, 72], myKey = 'C', weightByPosition = 2, maxTries = 100):		#
	global globalWeightFactor, startChords, chordPositionDict, globalKey, chordProgs, RNprogs, totalPossibilities
	modeInd = int(myKey[0].islower())
	globalKey = key.Key('Cc'[modeInd])
	finalTransposition = interval.notesToInterval(globalKey.tonic, key.Key(myKey).tonic)
	RNprogs = [RNprogsMajor, RNprogsMinor][modeInd]
	chordProgs = [chordProgsMajor, chordProgsMinor][modeInd]
	totalPossibilities = [totalPossibilitiesMajor, totalPossibilitiesMinor][modeInd]
	chordPositionDict = [chordPositionDictMajor, chordPositionDictMinor][modeInd]
	startChords = [startChordsMajor, startChordsMinor][modeInd]
	globalWeightFactor = weightByPosition
	if type(melodyNotes) is str:
		melodyNotes = pitch_names_to_midi(melodyNotes.replace('b', '-'))
	melodyNotes = [x - finalTransposition.semitones for x in melodyNotes]
	print(melodyNotes)
	for i in range(maxTries):
		print("TRY", i)
		if try_to_harmonize(melodyNotes):
			outScore.transpose(finalTransposition, inPlace = True)
			outScore.show()
			break

def try_to_harmonize(melodyNotes = [72, 72, 71, 72, 74, 76, 77, 76]):
	global currentState, outScore, phrasePosition
	outScore = stream.Score()
	for i in range(4):
		outScore.insert(0, stream.Part())
	currentState = make_start_chord(melodyNotes[0])			
	insert_chord()
	totalTries = 0
	for i, target in enumerate(melodyNotes):
		if i == 0: continue
		phrasePosition = int(linear_map(i, [0, len(melodyNotes) - 1], [0, 15]))
		subtractor = currentState[2][-1] % 12
		melodyOptions = find_melody(target)
		if not melodyOptions:
			return 0
		nextThing = random_listmember(melodyOptions)
		print(currentState, nextThing)
		newVL = nextThing[1]
		subtractor = currentState[2][-1] % 12
		newNotes = apply_VL(newVL, subtractor)
		currentState[1] = nextThing[0]
		currentState[2] = newNotes
		insert_chord(permutate(nextThing[2]))
	for p in outScore:
		fix_passing(p)
	return 1

def pitch_names_to_midi(myString):
	myList = myString.split()
	lastOctave = '5'
	for i, s in enumerate(myList):
		#print s, s[-1].isdigit(), lastOctave
		if s[-1].isdigit():
			lastOctave = s[-1]
		else:
			myList[i] = s + lastOctave
	return [pitch.Pitch(i).midi for i in myList]

def make_start_chord(topNote):										# add a percentage chance for open position
	chordSymbol = random_listmember(startChords[topNote % 12])[0]
	k = globalKey
	c = roman.RomanNumeral(chordSymbol, k)
	thePCs = [x.pitchClass for x in c.pitches]
	bassPC = c.bass().pitchClass
	bassNote = bassPC + 48
	if bassNote >= 55 and random.randrange(0, 2) >= 1:
		bassNote -= 12
	melodyPC = topNote % 12
	remainingPCs = [x for x in thePCs if x not in [bassPC, melodyPC]]
	if len(remainingPCs) == 1:
		remainingPCs.append(bassPC)
	theOctave = int(topNote / 12) * 12
	remainingNotes = [x + theOctave for x in remainingPCs]
	for i, p in enumerate(remainingNotes):
		if p > topNote:
			remainingNotes[i] = p - 12
	remainingNotes = sorted(remainingNotes)
	remainingNotes.reverse()
	return [k, chordSymbol, [topNote, remainingNotes[0], remainingNotes[1], bassNote]]


def fix_pitchlist(myPitches):
	while min(myPitches) > 12:
		myPitches = [x - 12 for x in myPitches]
	return myPitches

def find_next_RN(targetNote):
	global currentState
	possibilities = []
	startChord = currentState[1]
	for prog in RNprogs:
		firstChord, rn = prog.split(' -> ')
		if str(firstChord) != startChord:
			continue
		if str(rn) in rnPitchDict:
			myPitches = rnPitchDict[rn]
		else:
			myPitches = fix_pitchlist([x.midi for x in roman.RomanNumeral(rn, myKey).pitches])
			rnPitchDict[str(rn)] = myPitches
		myPCs = [x % 12 for x in myPitches]
		if (targetNote % 12) in myPCs:
			possibilities.append([rn, RNprogs[prog]])
	return possibilities


rnPitchDict = {}
myKey = key.Key('C')

def permutate(list_of_four):
	global thePermutation
	if thePermutation:
		return [list_of_four[i] for i in [0, 2, 1, 3]]
	return list_of_four


def insert_chord(pts = [0, 0, 0, 0]):
	global currentSate, outScore
	thePitches = [x.pitchClass for x in globalKey.getScale().pitches][:-1]
	for i in range(4):
		if pts[i] == 0:
			newNote = note.Note(currentState[2][i], quarterLength = 1)
			if globalKey.mode == 'minor' and newNote.accidental.alter > 0 and newNote.pitchClass != 6:
				newNote.transpose('D2', inPlace = True)
			outScore.parts[i].append(newNote)
		else:
			outScore.parts[i][-1].duration.quarterLength = .5
			mySign = (currentState[2][i] - outScore.parts[i][-1].pitch.midi) / abs(currentState[2][i] - outScore.parts[i][-1].pitch.midi) # sign of change
			if (currentState[2][i] - (mySign * 1)) % 12 in thePitches:
				newNote = note.Note(currentState[2][i] - (mySign * 1), quarterLength = .5)
			else:
				newNote = note.Note(currentState[2][i] - (mySign * 2), quarterLength = .5)
			if globalKey.mode == 'minor' and newNote.accidental.alter > 0 and newNote.pitchClass != 6:
				newNote.transpose('D2', inPlace = True)
			outScore.parts[i].insert(outScore.parts[i][-1].offset + .5, newNote)
			newNote = note.Note(currentState[2][i], quarterLength = 1)
			if globalKey.mode == 'minor' and newNote.accidental.alter > 0 and newNote.pitchClass != 6:
				newNote.transpose('D2', inPlace = True)
			outScore.parts[i].append(newNote)


def find_melody(targetNote):
	possibilities = []
	curChord = currentState[1]
	subtractor = currentState[2][-1] % 12
	topVoice = (currentState[2][0] - subtractor) % 12
	if topVoice not in chordProgs[curChord]:
		return False
	for secondChord in chordProgs[curChord][topVoice]:
		tryProg = curChord + ' -> ' + secondChord[0]
		newPossibilities = check_prog(tryProg, target = targetNote)
		if not newPossibilities:
			continue
		else:
			for item in newPossibilities:
				if globalWeightFactor and secondChord[0] in chordPositionDict:
					numWeights = 1
					totalWeightFactor = chordPositionDict[secondChord[0]][phrasePosition]
					if phrasePosition > 0:
						totalWeightFactor = chordPositionDict[secondChord[0]][phrasePosition - 1]
						numWeights += 1
					if phrasePosition < 15:
						totalWeightFactor = chordPositionDict[secondChord[0]][phrasePosition + 1]
						numWeights += 1
					totalWeightFactor = 0.5 + totalWeightFactor / numWeights
					newItem = item[:]
					newItem[-1] = int(.5 + (item[-1] * (totalWeightFactor*2))) + 1	
					possibilities.append([secondChord[0]] + newItem)
				else:
					possibilities.append([secondChord[0]] + item)
	return possibilities

def fix_passing(myStream):
	for i, n in enumerate(myStream):
		fracPart = n.offset - int(n.offset)
		if fracPart < .1 or fracPart > .9: continue
		if i == 0 or i == len(myStream): continue
		lastInt = n.pitch.midi - myStream[i-1].pitch.midi
		nextInt = myStream[i+1].pitch.midi - n.pitch.midi
		if (lastInt == -3 and nextInt == -1) or (lastInt == 1 and nextInt == 3):
			n.transpose('m2', inPlace = True)
		elif (lastInt == 3 and nextInt == 1) or (lastInt == -1 and nextInt == -3):
			n.transpose('-m2', inPlace = True)
		

def convert_VLstr(t):			# convert to ints
	outPS = []
	sVoice = 0
	s = t.split('],')
	for i, item in enumerate(s):
		outFlag = 0
		if item.count('P') > 0:
			outFlag = 1
		if item.count('S') > 0:
			sVoice = i
		outPS.append(outFlag)
	t = ''.join(filter(lambda x: x in '-] ' or x.isdigit(), t))
	t = t.replace(']]','').split('] ')
	output = []
	for u in t:
		u = u.split()
		l = []
		for v in u: l.append(int(v))
		output.append(l)
	if output[-1][0] != 0:
		subtractor = output[-1][0]
		for item in output:
			item[0] = (item[0] - subtractor) % 12
	output = [output[sVoice]] + output[0:sVoice] + output[sVoice + 1:]
	outPS = [outPS[sVoice]] + outPS[0:sVoice] + outPS[sVoice + 1:]
	return output, outPS

def load(source = 'BACH'):
	global progs, globalSource, locations, locNames, totalPossibilitiesMajor, totalPossibilitiesMinor, chordProgsMajor, chordProgsMinor, RNprogsMajor, RNprogsMinor
	globalSource = source.upper()
	storedFile = open(DT._DATAPATH + 'RN' + globalSource + '.pkl', 'rb')
	if not storedFile: 
		print("error")
		return
	counts = pickle.load(storedFile)
	storedFile.close()
	inputFile = open(DT._DATAPATH + 'VL' + globalSource +'.pkl', 'rb')
	progs = pickle.load(inputFile)
	try:
		locations = pickle.load(inputFile)
		locNames = pickle.load(inputFile)
	except:
		pass
	inputFile.close()
	RNprogsMajor = {}
	chordProgsMajor = {}
	totalPossibilitiesMajor = {}
	RNprogsMinor = {}
	chordProgsMinor = {}
	totalPossibilitiesMinor = {}
	for modeInd in [0, 1]:
		RNprogs = [RNprogsMajor, RNprogsMinor][modeInd]
		chordProgs = [chordProgsMajor, chordProgsMinor][modeInd]
		totalPossibilities = [totalPossibilitiesMajor, totalPossibilitiesMinor][modeInd]
		for item in counts[modeInd]:
			if item.count(' -> ') == 1:
				RNprogs[item] = counts[modeInd][item]
		chordProgs = [chordProgsMajor, chordProgsMinor][modeInd]
		for item in progs[modeInd]:
			firstChord, secondChord = item.split(" -> ")
			myDict = clean_prog(item, modeInd)
			totalPossibilities[item] = myDict
			if firstChord not in chordProgs:
				chordProgs[firstChord] = {}
			for item in myDict:
				if item not in chordProgs[firstChord]:
					chordProgs[firstChord][item] = [[secondChord, sum([x[-1] for x in myDict[item]])]]
				else:
					chordProgs[firstChord][item].append([secondChord, sum([x[-1] for x in myDict[item]])])
		
def clean_prog(myProg = 'I -> I6', modeInd = 0):
	possibilities = {}
	PCcounts = [count_PCs(x) for x in myProg.split(' -> ')]
	for item in progs[modeInd][myProg]:
		output, outPS = convert_VLstr(item)
		if len(output) != 4: continue
		firstPitches = set([x[0] for x in output])
		if len(firstPitches) != PCcounts[0]: continue
		secondPitches = set([sum(x) % 12 for x in output])
		if len(secondPitches) != PCcounts[1]: continue
		if output[0][0] in possibilities:
			possibilities[output[0][0]].append([output, outPS, progs[modeInd][myProg][item]])
		else:
			possibilities[output[0][0]] = [[output, outPS, progs[modeInd][myProg][item]]]
	for item in possibilities:
		possibilities[item] = sorted(possibilities[item], key = lambda x: -x[-1])
	return possibilities

def count_PCs(rnString):
	if rnString.count('7') > 0 or rnString.count('6/5') > 0 or rnString.count('4/3') > 0 or rnString.count('2') > 0:
		return 4
	else:
		return 3
	
def check_prog(testProg, printIt = False, doublingOnly = False, target = 0):			# FIX DOUBLING ONLY TO SOMETHING ELSE
	global currentState, realOptions
	realOptions = []
	myNotes = currentState[2][:]
	subtractor = myNotes[-1] % 12
	melodyNote = (myNotes[0] - subtractor) % 12
	if melodyNote not in totalPossibilities[testProg]:
		print("ERROR!  CAN'T FIND MELODYNOTE!")
		return False
	testOptions = totalPossibilities[testProg][melodyNote]
	for fullItem in testOptions:
		item = fullItem[0]
		if (right_doublings(item, subtractor) and no_parallels(item, subtractor) and good_ranges(item, subtractor) and (
		target == 0 or (target != 0 and ((myNotes[0] + item[0][1]) == target)))) or (doublingOnly and target == 0 and right_doublings(item, subtractor)):
			realOptions.append(fullItem)
		else:
			if item[1][0] == item[2][0]:
				item = [item[0], item[2], item[1], item[3]]
				if (right_doublings(item, subtractor) and no_parallels(item, subtractor) and good_ranges(item, subtractor) and (
				target == 0 or (target != 0 and ((myNotes[0] + item[0][1]) == target)))) or (doublingOnly and target == 0 and right_doublings(item, subtractor)):
					realOptions.append([item, [fullItem[1][0], fullItem[1][2], fullItem[1][1], fullItem[1][3]], fullItem[2]])
					continue
			if printIt:
				if not right_doublings(item, subtractor):
					print("REJECTING WRONG DOUBLINGS:", item)
				elif not no_parallels(item, subtractor):
					print("REJECTING PARALLELS:", item)
				elif not good_ranges(item, subtractor):
					print("REJECTING RANGES:", item)
	if not realOptions:
		#print "ERROR!  NO OPTIONS"
		return False
	return realOptions

def right_doublings(myVL, subtractor = currentState[2][-1] % 12):
	global currentState
	innerVoices1 = [(x - subtractor) % 12 for x in currentState[2][1:3]]
	innerVoices2 = [x[0] % 12 for x in myVL[1:3]]
	if (innerVoices1[0] == innerVoices2[0] and innerVoices1[1] == innerVoices2[1]) or (innerVoices1[1] == innerVoices2[0] and innerVoices1[0] == innerVoices2[1]):
		return True
	return False

def apply_VL(myVL, subtractor = currentState[2][-1] % 12):
	global thePermutation
	curNotes = currentState[2][:]
	for i in [0, 3]:
		curNotes[i] += myVL[i][1]
	if (curNotes[1] - subtractor) % 12 == myVL[1][0]:
		curNotes[1] += myVL[1][1]
		curNotes[2] += myVL[2][1]
		thePermutation = 0
	else:
		curNotes[1] += myVL[2][1]
		curNotes[2] += myVL[1][1]
		thePermutation = 1
	return curNotes

def no_parallels(myVL, subtractor = currentState[2][-1] % 12):
	curNotes = currentState[2][:]
	nextNotes = apply_VL(myVL, subtractor)
	for i in range(0, 3):
		for j in range(i + 1, 4):
			firstInt = (curNotes[i] - curNotes[j]) % 12
			nextInt = (nextNotes[i] - nextNotes[j]) % 12
			if firstInt == 7 and nextInt == 7:
				return False
	return True
	
def good_ranges(myVL, subtractor = currentState[2][-1] % 12):
	curNotes = apply_VL(myVL, subtractor)
	if curNotes[3] <= curNotes[2] <= curNotes[1] <= curNotes[0]:			# no crossings
		for i in range(0, 4):
			if curNotes[i] < ranges[i][0] or curNotes[i] > ranges[i][1]:	# out of range
				return False
		return True
	return False

def print_options(myProg = 'I -> ii', mySop = 0):
	for item in totalPossibilities[myProg][mySop]:
		print(item)

def random_listmember(myList):					# key, count pairs; returns a random member of the list weighted by count
	theSum = sum([x[-1] for x in myList])
	if theSum == 0: return 0
	myNum = random.randrange(theSum) + 1
	counter = 0
	for item in myList:
		counter += item[-1]
		if counter >= myNum:
			return item
	return item


load('Bach')
#make_top_note_dict()

